/* accounting_control.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include <gnome.h>
#include <glade/glade.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <glib.h>
#include <time.h>

#include "a-shortcut.h"
#include "support.h"
#include "expense.h"
#include "expense_model.h"
#include "issued_book_model.h"
#include "received_book_model.h"
#include "incoming_book_model.h"

#include "util.h"
#include "parameter_manager.h"
#include "control_util.h"
#include "accounting_control.h"
#include "accounting_period.h"

#include "enterprise_model.h"
#include "accounting_period_model.h"

#include "../interface/printing/listing-print-wrapper.h"

#define C_ISSUED_INVOICES  0
#define C_RECEIVED_INVOICES 1
#define C_INCOMINGS           2
#define C_EXPENSES             3
#define C_COLUMNS            8

static GtkWidget *accounting=NULL;
static GladeXML *xml = NULL;
static gchar widget_name[]="vbox-libro-iva-contabilidad";
static AccountingPeriod *accounting_period=NULL;
static gchar url[]="/src/interface/glade/accounting_view.glade";
static Listing * listing=NULL;

static void on_accounting_book_vat_search_clicked(GtkButton *button, gpointer data);
static void on_accounting_book_vat_clean_clicked(GtkButton *button, gpointer data);
static void on_accounting_book_vat_print_clicked(GtkButton *button, gpointer data);
static void on_close_period_button_clicked(GtkButton *button, gpointer data);
static void on_save_period_button_clicked(GtkButton *button, gpointer data);
static void on_accounting_book_vat_filter_enterprise_combo_entry_cged(void);
static void on_recover_period_button_click(GtkButton *button, gpointer data);
static void confirm_close(gint reply, gpointer data);
static void on_group_toggled(GtkToggleButton *togglebutton,
			     gpointer user_data);
static gint get_selected_operation(void);
static void fill_account_list(GtkCList * list, gint size);
static ConditionAccountingPeriod *  generate_condition_from_screen(void);
static void fill_enterprise_combo(void);
static void clean_combos(void);
static void clean_screen(void);
static void inicialize_account_list(void);
static ReceivedBook ** control_find_received_book(gulong tuples,gulong rows);

GtkWidget *c_accounting_open_accounting (GtkWidget *app1, GtkWidget *vbox) {
    gchar * directory = g_strconcat(AA_DATA_DIR,url,NULL);

    if (accounting==NULL) {
	GtkWidget *dialog;
	gchar *e_name = NULL;

	xml = glade_xml_new (directory, widget_name);
	dialog = glade_xml_get_widget (xml, widget_name);

	gtk_box_pack_start (GTK_BOX (vbox), dialog, TRUE, TRUE, 0);
	glade_xml_signal_connect (xml, "on_Buscar-libro-iva-contabilidad_clicked",
				  GTK_SIGNAL_FUNC (on_accounting_book_vat_search_clicked));
	glade_xml_signal_connect (xml, "on_Limpiar-libro-iva-contabilidad_clicked",
				  GTK_SIGNAL_FUNC (on_accounting_book_vat_clean_clicked));
	glade_xml_signal_connect (xml, "on_Imprimir-libro-iva-contabilidad_clicked",
				  GTK_SIGNAL_FUNC (on_accounting_book_vat_print_clicked));
	glade_xml_signal_connect (xml, "on_combo-entry-empresa-filtrado-libro-iva-contabilidad_changed",
				  GTK_SIGNAL_FUNC (on_accounting_book_vat_filter_enterprise_combo_entry_cged));
	glade_xml_signal_connect (xml, "on_button_cerrar_periodo_clicked",
				  GTK_SIGNAL_FUNC (on_close_period_button_clicked));
	glade_xml_signal_connect (xml, "on_button_guardar_periodo_clicked",
				  GTK_SIGNAL_FUNC (on_save_period_button_clicked));
	glade_xml_signal_connect (xml, "on_group_toggled",
				  GTK_SIGNAL_FUNC (on_group_toggled));
	glade_xml_signal_connect (xml, "on_button_recuperar_periodo_clicked",
				  GTK_SIGNAL_FUNC (on_recover_period_button_click));



	accounting=dialog;
	e_name = u_parameter_manager_get_parameter ("empresa");
	if (e_name!=NULL) {
	    gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget (xml, "combo-entry-empresa-filtrado-libro-iva-contabilidad")), e_name);
	}
	g_free(e_name);

    }

    clean_screen();
    clean_combos();
    fill_enterprise_combo();
    if (listing!=NULL)
	p_listing_clear (listing);
    listing =NULL;

    g_free(directory);
    return accounting;

}

static gint get_selected_operation() {
    GtkWidget       *widget = glade_xml_get_widget(xml, "radiobutton-facturasemitidas-libro-iva-contabilidad");
    GtkToggleButton *button = GTK_TOGGLE_BUTTON(widget);

    if (gtk_toggle_button_get_active(button)) {
        return C_ISSUED_INVOICES;
    }

    widget = glade_xml_get_widget(xml, "radiobutton-facturasrecibidas-libro-iva-contabilidad");
    button = GTK_TOGGLE_BUTTON(widget);
    if (gtk_toggle_button_get_active(button)) {
        return C_RECEIVED_INVOICES;
    }

    widget = glade_xml_get_widget(xml, "radiobutton-ingresos-libro-iva-contabilidad");
    button = GTK_TOGGLE_BUTTON(widget);
    if (gtk_toggle_button_get_active(button)) {
        return C_INCOMINGS;
    }

    widget = glade_xml_get_widget(xml, "radiobutton-gastos-libro-iva-contabilidad");
    button = GTK_TOGGLE_BUTTON(widget);
    if (gtk_toggle_button_get_active(button)) {
        return C_EXPENSES;
    }

    return -1;
}

static void on_accounting_book_vat_search_clicked(GtkButton *button, gpointer data) {
    GtkWidget       *dateedit_fecha_inicio = NULL;
    GtkWidget       *dateedit_fecha_fin = NULL;
    GtkCList        *list = GTK_CLIST(glade_xml_get_widget(xml,"clist-libro-iva-contabilidad"));
    gulong           code = ULONG_MAX;
    gint             selected = get_selected_operation();
    gulong           tuples = ULONG_MAX;
    gulong             rows = 0;
    GtkWidget * combo_empresa = glade_xml_get_widget (xml, "combo-empresa-filtrado-libro-iva-contabilidad");
    GtkWidget * window;
    GdkCursor * cursor;

    window = lookup_widget(GTK_WIDGET(list),"app1");
    cursor = gdk_cursor_new(GDK_WATCH);
    gdk_window_set_cursor(GTK_WIDGET(window)->window, cursor);
    gdk_cursor_destroy(cursor);


    dateedit_fecha_inicio = glade_xml_get_widget(xml,"dateedit-fechainicio-libro-iva-contabilidad");
    dateedit_fecha_fin = glade_xml_get_widget(xml,"dateedit-fechafin-libro-iva-contabilidad");
    code = c_util_get_code_from_combo (combo_empresa);

    inicialize_account_list();
    /*    if (listing!=NULL)
     p_listing_clear (listing);
     listing=NULL;
     */
    listing = p_listing_init ();
    while (gtk_events_pending())
	gtk_main_iteration();

    switch (selected) {
    case C_ISSUED_INVOICES:
	{
	    ConditionAccountingPeriod * condition = generate_condition_from_screen();
	    IssuedBook ** issued_books = NULL;
            IssuedBook ** resumen=NULL;
            IssuedBook ** resumen2=NULL;
            IssuedBook ** agrupadas=NULL;
	    IssuedBook ** ib=NULL;
	    IssuedBook ** ib2=NULL;
            IssuedBook ** ib_total=NULL;
	    issued_books   = m_issued_books_find_by_condition (condition, &tuples);
	    resumen = d_issued_book_calculate_total_from_list (issued_books);
            d_issued_book_group_by_vat (issued_books);
	    agrupadas = m_issued_books_group_by_vat (condition,&tuples);
	    resumen2 = d_issued_book_calculate_total_from_list (agrupadas);

	    ib = d_issued_book_concat (issued_books,resumen);
	    ib2 = d_issued_book_concat (agrupadas,resumen2);

            ib_total = d_issued_book_concat (ib,ib2);
	    if (issued_books!=NULL) {
		listing->content_title = d_issued_book_get_content_title ();
		listing->listing_title = d_issued_book_get_listing_title ();
		listing->content       = d_issued_book_array_to_content (ib_total, &rows);
		listing->width_of_cols = d_issued_book_get_listing_columns_width ();
		listing->alignment_of_cols = d_issued_book_get_listing_columns_alignment ();
		listing->n_rows        = rows;
	    listing->n_columns     = d_issued_book_get_content_size ();
	    listing->orientation   = 4;
	    listing->type_of_paper = 2;
	    }
	    m_issued_books_clear_array  (issued_books);
	    m_issued_books_clear_array  (resumen);
	    m_issued_books_clear_array  (agrupadas);
	    m_issued_books_clear_array  (resumen2);
	    m_issued_books_clear_array  (ib);
	    m_issued_books_clear_array  (ib2);
	    m_issued_books_clear_array  (ib_total);
	    g_free(condition->ending_date);
	    g_free(condition->initial_date);
	    g_free(condition);
	}
	break;
    case C_RECEIVED_INVOICES:
	{
            ReceivedBook ** received_books = NULL;
	    received_books = control_find_received_book(tuples, rows);
	    m_received_books_clear_array (received_books);
            rows = listing->n_rows;
	}
        break;
    case C_INCOMINGS:
	{
	    IncomingBook ** incoming_books = NULL;
            IncomingBook ** resumen=NULL;
            IncomingBook ** ib=NULL;

	    ConditionAccountingPeriod * condition = generate_condition_from_screen();


	    incoming_books   = m_incoming_book_find_by_condition (condition, &tuples);
	    resumen = d_incoming_book_calculate_total_from_list (incoming_books);
	    ib = d_incoming_book_concat (incoming_books,resumen);
	    if (incoming_books!=NULL) {
		listing->content_title = d_incoming_book_get_content_title ();
		listing->listing_title = d_incoming_book_get_listing_title ();
		listing->content       = d_incoming_book_array_to_content (ib, &rows);
		listing->width_of_cols = d_incoming_book_get_listing_columns_width ();
		listing->alignment_of_cols = d_incoming_book_get_listing_columns_alignment ();
		listing->n_rows        = rows;
	    listing->n_columns     = d_incoming_book_get_content_size ();
	    listing->orientation   = 3;
	    listing->type_of_paper = 2;
	    }
	    m_incoming_book_clear_array  (incoming_books);
	    m_incoming_book_clear_array  (resumen);
	    m_incoming_book_clear_array  (ib);
	    g_free(condition->ending_date);
	    g_free(condition->initial_date);
	    g_free(condition);
	}
        break;
    case C_EXPENSES:
	{
	    ConditionExpense condition;
	    Expense ** expenses = NULL;
            Expense ** resumen=NULL;
	    GtkWidget       *check = glade_xml_get_widget(xml,"checkbutton_proveedores_500000");
            Expense ** ex=NULL;

	    condition.enterprise_code = code;
	    condition.initial_date = gnome_date_edit_get_date(GNOME_DATE_EDIT(dateedit_fecha_inicio));
	    condition.final_date = gnome_date_edit_get_date(GNOME_DATE_EDIT(dateedit_fecha_fin));

            condition.greater_than=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check));
	    expenses   = m_expense_find_by_condition (&condition, &tuples);
	    resumen = d_expense_calculate_total_data (expenses);
	    ex = d_expense_concat (expenses,resumen);
	    if (expenses!=NULL) {
		listing->content_title = d_expense_get_content_title ();
		listing->listing_title = d_expense_get_listing_title ();
		listing->content       = d_expense_array_to_content (ex, &rows);
		listing->width_of_cols = d_expense_get_listing_columns_width ();
		listing->alignment_of_cols = d_expense_get_listing_columns_alignment ();
	    listing->n_rows        = rows;
	    listing->n_columns     = d_expense_get_content_size ();
	    listing->orientation   = 4;
	    listing->type_of_paper = 2;
	    }
            m_expense_clear_array (expenses);
	    m_expense_clear_array (resumen);
	    m_expense_clear_array (ex);
	}
	break;
    }

    fill_account_list(list, rows);
    cursor = gdk_cursor_new(GDK_LEFT_PTR);
    gdk_window_set_cursor(GTK_WIDGET(window)->window, cursor);
    gdk_cursor_destroy(cursor);

}

static void fill_account_list(GtkCList * list, gint size) {
    gint i;
    gint column = 0;
    gchar * temp;
    gchar **data;

    gchar *empty_row[]={"","","","","","","",""};


    if (listing!=NULL) {
	if (listing->content_title!=NULL) {
	    for(column=0; listing->content_title[column]!=NULL; column++){
		temp = g_strdup_printf("%s", listing->content_title[column]);
		gtk_clist_set_column_title((GtkCList *)list, column, temp);
		gtk_clist_set_column_width ((GtkCList *)list,column, listing->width_of_cols[column]*7);
		g_free(temp);
	    }
	}

	for(i=column;i<C_COLUMNS;i++) {
	    gtk_clist_set_column_visibility((GtkCList *)list, i, FALSE);
	}

	if (listing->content!=NULL) {
	    for (i=0;i<size;i++) {
		if (i==30) gtk_clist_freeze ((GtkCList *) list);

		column = 0;
		data = listing->content[i];
		gtk_clist_append ((GtkCList *) list,empty_row);

		for (column=0; data[column]!=NULL; column++) {
		    temp = g_strdup(data[column]);
		    gtk_clist_set_text((GtkCList *) list, i, column, temp);
		    g_free(temp);
		}
	    }
	    gtk_clist_thaw ((GtkCList *) list);

	}
    }
}

static void on_accounting_book_vat_clean_clicked(GtkButton *button, gpointer data) {
    clean_screen();
    clean_combos();
    fill_enterprise_combo();
}

static void on_accounting_book_vat_print_clicked(GtkButton *button, gpointer data) {
    if (listing !=NULL) {
	    p_listing_wrap_print (listing);
    } else {
	c_util_open_info_window (_("You have to do a previous search before printing"));
    }

}

static void inicialize_account_list() {
    GtkWidget *list;
    gint       column;

    list = glade_xml_get_widget(xml,"clist-libro-iva-contabilidad");

    gtk_clist_clear(GTK_CLIST(list));

    for(column=0;column<C_COLUMNS;column++) {
	gtk_clist_set_column_title((GtkCList *)list, column, "");
	gtk_clist_set_column_visibility((GtkCList *)list, column, TRUE);
    }

}
static void clean_screen() {

    GtkWidget *vbox;

    vbox = GTK_WIDGET(glade_xml_get_widget(xml, widget_name));
    gtk_container_foreach(GTK_CONTAINER(vbox),&c_util_clean_widget,NULL);

    if (listing!=NULL)
	p_listing_clear (listing);
    listing=NULL;
    inicialize_account_list();
    if (accounting_period!=NULL) {
	d_accounting_period_clear (accounting_period);
    }
    accounting_period=NULL;

}


static void on_accounting_book_vat_filter_enterprise_combo_entry_cged() {
    clean_screen();

}

static void fill_enterprise_combo() {
    GtkWidget * combo;
    Enterprise ** enterprises;

    combo = glade_xml_get_widget (xml, "combo-empresa-filtrado-libro-iva-contabilidad");
    enterprises = m_enterprise_find_all ();

    c_util_fill_combo_with_enterprises (combo,enterprises);
}

static void clean_enterprise_combo() {
    GtkWidget * combo;
    GtkWidget * entry;

    entry = glade_xml_get_widget (xml, "combo-entry-empresa-filtrado-libro-iva-contabilidad");
    gtk_entry_set_text(GTK_ENTRY(entry),"");

    combo = glade_xml_get_widget (xml, "combo-empresa-filtrado-libro-iva-contabilidad");
    c_util_clean_code_and_name_combo (combo);

}

static void clean_combos() {
    gchar *e_name = NULL;

    clean_enterprise_combo();
    e_name = u_parameter_manager_get_parameter ("empresa");
    if (e_name!=NULL) {
	gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget (xml, "combo-entry-empresa-filtrado-libro-iva-contabilidad")), e_name);
    }

    g_free(e_name);
}

static void confirm_close(gint reply, gpointer data){
    if (!reply) {
	if (accounting_period!=NULL) {
	    GtkCList        *list = GTK_CLIST(glade_xml_get_widget(xml,"clist-libro-iva-contabilidad"));
	    ReceivedBook ** received_books;
	    gulong rows=0;
	    gulong tuples=ULONG_MAX;
            gboolean done;
	    received_books = control_find_received_book(tuples,rows);
	    fill_account_list(list,rows);
	    done = m_received_books_set_receipt (received_books,accounting_period);
	    if (done == FALSE) c_util_open_error_window (_("Accounting period closing failed"));
            else c_util_open_info_window (_("Accounting period has been closed correctly"));
	} else {
        c_util_open_info_window (_("You have not choosen any existing accounting period. \n If you don't want any of them, save the current accounting period."));
    }

    }

}

static void on_close_period_button_clicked(GtkButton *button, gpointer data) {

    c_util_open_question_dialog(_("Are you really sure you want to close accounting of this period?"),confirm_close);

}
static void on_save_period_button_clicked(GtkButton *button, gpointer data) {
    GtkWidget       *dateedit_fecha_inicio = NULL;
    GtkWidget       *dateedit_fecha_fin = NULL;
    GtkWidget * combo_empresa = glade_xml_get_widget (xml, "combo-empresa-filtrado-libro-iva-contabilidad");
    
    gulong code=ULONG_MAX;
    gulong accounting_code=ULONG_MAX;
    time_t initial_date;
    time_t ending_date;

    if (accounting_period!=NULL) {
        d_accounting_period_clear (accounting_period);
    }
    accounting_period=NULL;
    accounting_period = d_accounting_period_create_empty ();
    dateedit_fecha_inicio = glade_xml_get_widget(xml,"dateedit-fechainicio-libro-iva-contabilidad");
    dateedit_fecha_fin = glade_xml_get_widget(xml,"dateedit-fechafin-libro-iva-contabilidad");
    code = c_util_get_code_from_combo (combo_empresa);
    initial_date = gnome_date_edit_get_date(GNOME_DATE_EDIT(dateedit_fecha_inicio));
    ending_date = gnome_date_edit_get_date(GNOME_DATE_EDIT(dateedit_fecha_fin));


    d_accounting_set_initial_date (accounting_period,initial_date);
    d_accounting_set_ending_date (accounting_period,ending_date);
    d_accounting_set_enterprise_code (accounting_period,code);
    d_accounting_set_closed (accounting_period,FALSE);
    accounting_code =m_accounting_period_insert (accounting_period);
    if (accounting_code==ULONG_MAX) {
        c_util_open_error_window (_("Accounting period has not been correctly saved."));
    } else if (accounting_code==(ULONG_MAX-1)) {
        c_util_open_error_window (_("Accounting period already exists in the database."));
    } else {
        c_util_open_info_window (_("Accounting period has been saved correctly."));
    }

}
static void on_group_toggled(GtkToggleButton *togglebutton,
			     gpointer user_data) {
    GtkWidget * button_cerrar=glade_xml_get_widget(xml,"button_cerrar_periodo");
    gint selected = get_selected_operation();
    switch (selected) {
    case C_RECEIVED_INVOICES:
        gtk_widget_show(button_cerrar);
	break;
    default:
        gtk_widget_hide(button_cerrar);
        break;
    }

}
ConditionAccountingPeriod *  generate_condition_from_screen() {
    GtkWidget       *dateedit_init_date = glade_xml_get_widget(xml,"dateedit-fechainicio-libro-iva-contabilidad");
    GtkWidget       *dateedit_ending_date = glade_xml_get_widget(xml,"dateedit-fechafin-libro-iva-contabilidad");
    GtkWidget * combo_empresa = glade_xml_get_widget (xml, "combo-empresa-filtrado-libro-iva-contabilidad");
    gulong code=ULONG_MAX;
    GtkWidget       *check = glade_xml_get_widget(xml,"checkbutton_proveedores_500000");
    

    ConditionAccountingPeriod *  condition = (ConditionAccountingPeriod *) g_malloc0(sizeof(ConditionAccountingPeriod));


    time_t initial_date = gnome_date_edit_get_date(GNOME_DATE_EDIT(dateedit_init_date));
    time_t ending_date = gnome_date_edit_get_date(GNOME_DATE_EDIT(dateedit_ending_date));

    gnome_date_edit_set_time(GNOME_DATE_EDIT(dateedit_ending_date), ending_date);

    code = c_util_get_code_from_combo (combo_empresa);
    condition->greater_than = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check));

    condition->enterprise_code = code;
    condition->initial_date = g_strdup(ctime(&initial_date));
    u_delete_intros (condition->initial_date);

    c_util_add_days(&ending_date, 1);    
    condition->ending_date = g_strdup(ctime(&ending_date));

    u_delete_intros (condition->ending_date);

    return condition;

}
static void on_recover_period_button_click(GtkButton *button, gpointer data) {

    ConditionAccountingPeriod * condition = generate_condition_from_screen();
    if (accounting_period!=NULL) {
        d_accounting_period_clear (accounting_period);
    }
    accounting_period=NULL;
    accounting_period = m_accounting_period_find_by_condition (condition);
    if (accounting_period!=NULL) {
        c_util_open_info_window (_("Accounting period retrieved successfully"));
    } else {
        c_util_open_error_window (_("The accounting period has not been retrieved"));
    }
    if (condition->initial_date!=NULL)
	g_free(condition->initial_date);
    if (condition->ending_date!=NULL)
	g_free(condition->ending_date);
    g_free(condition);

}

static ReceivedBook ** control_find_received_book(gulong tuples,gulong rows) {
    ConditionAccountingPeriod * condition = generate_condition_from_screen();

    ReceivedBook ** received_books = NULL;
    ReceivedBook ** resumen = NULL;
    ReceivedBook ** rb = NULL;
    ReceivedBook ** resumen2=NULL;
    ReceivedBook ** agrupadas=NULL;
    ReceivedBook ** rb2=NULL;
    ReceivedBook ** rb_total=NULL;


    received_books   = m_received_books_find_by_condition (condition, &tuples);
    resumen = d_received_book_calculate_total_from_list (received_books);
    rb = d_received_book_concat (received_books,resumen);
    agrupadas = m_received_books_group_by_vat (condition,&tuples);
    resumen2 = d_received_book_calculate_total_from_list (agrupadas);
    rb2 =d_received_book_concat (agrupadas,resumen2);
    rb_total =d_received_book_concat (rb,rb2);
    if (received_books!=NULL) {
	listing->content_title = d_received_book_get_content_title ();
	listing->listing_title = d_received_book_get_listing_title ();
	listing->content       = d_received_book_array_to_content (rb_total, &rows);
	listing->width_of_cols = d_received_book_get_listing_columns_width ();
	listing->alignment_of_cols = d_received_book_get_listing_columns_alignment ();
	listing->n_rows        = rows;
	listing->n_columns     = d_received_book_get_content_size ();
	listing->orientation   = 4;
	listing->type_of_paper = 2;
    }

    m_received_books_clear_array (resumen);
    m_received_books_clear_array (resumen2);
    m_received_books_clear_array (received_books);
    m_received_books_clear_array (agrupadas);
    m_received_books_clear_array (rb);
    m_received_books_clear_array (rb2);
    g_free(condition->ending_date);
    g_free(condition->initial_date);
    g_free(condition);

    return rb_total;

}

