/* charge_invoice_control.h
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */


#ifndef _CHARGE_INVOICE_CONTROL_
#define _CHARGE_INVOICE_CONTROL_

#include "charge_invoice_model.h"
#include "order_line.h"

/**
 * open_charge_invoice:
 * @app1: #Widget que contiene la aplicacin.
 * @vbox: Contenedor que est dentro de la ventana de contabilidad.
 *
 * Inicializa la ventana de insercin de facturas de cargo. Esta ventana permite insertar
 * facturas de pago a proveedores, asociandole un tipo de pago a la factura. Tambin permite
 * asociar lneas de pedido a las lneas de una factura.
 *
 * Returns:
 * La ventana de insercin de facturas inicializada.
 */
GtkWidget *c_charge_invoice_open (GtkWidget *app1, GtkWidget *vbox);
/**
 * inverse_state_pago_toggle_button:
 *
 * Invierte el estado del botn que indica que existe un pago asociado a una factura.
 *
 */
//void inverse_state_pago_toggle_button(void);
/**
 * change_state_pago_toggle_button:
 * @new_state: Nuevo estado para el botn.
 *
 * Cambia el estado del botn que indica que existe un pago asociado a una factura.
 *
 */
void c_charge_invoice_type_payment_toggle_button_change_state (gboolean new_state);
/**
 * change_state_pagada_toggle_button:
 * @new_state: Nuevo estado para el botn.
 *
 * Cambia el estado del botn de pagada al estado solicitado.
 *
 */
void c_charge_invoice_is_paid_toggle_button_change_state (gboolean new_state);
/**
 * load_charge_invoice:
 * @chargeInvoiceId: Identificador de la factura de pago, compuesto por el cdigo de la
 * empresa a la que se le emiti, el cdigo del proveedor y el nmero de factura.
 * @iva: Indica si la factura es una factura con iva o bien una nota.
 *
 * Carga en la ventana de insercin de facturas una factura de pago. Una factura de cargo
 * se puede modificar.
 *
 */
void load_charge_invoice(ChargeInvoiceId * chargeInvoiceId,gboolean iva);
/**
 * add_charge_line:
 * @charge_invoice_line: Lnea de factura a aadir.
 *
 * Aade una lnea de factura a una factura de pago.
 *
 */
void c_charge_invoice_add_charge_line (ChargeInvoiceLine *charge_invoice_line);
/**
 * load_charge_lines_clist:
 *
 * Recarga la lista de lneas de factura de cargo de una factura en la lista
 * que se visualiza en pantalla, es decir, repinta las lneas en pantalla.
 * 
 */
void c_charge_invoice_load_charge_lines_from_list (void);
/**
 * load_order_lines:
 * @order_lines: Lista enlazada que contiene una lista de lneas de pedido.
 *
 * Carga una lsta de lneas de pedido en la pantalla, transformandolas antes a lneas de factura.
 *
 * 
 */
void c_charge_invoice_load_order_lines (GList *order_lines);

#endif

