

/* documents.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include <gnome.h>
#include <glade/glade.h>
#include <glib.h>

#include "documents_control.h"
#include "repairs_control.h"
#include "invoice_control.h"
#include "document_model.h"
#include "control_util.h"
#include "document.h"
#include "participant.h"

static GtkWidget *documents_control=NULL;
static GladeXML *xml = NULL;
static Company *company = NULL;
static Document **documents = NULL;
static gboolean repair = TRUE;

static gchar url[]="/src/interface/glade/repairs_view.glade";
static gchar widget_name[]="documentos";

static void on_documentos_destroy(GtkWidget *widget,gpointer user_data);
static void load_documents_list(void);

GtkWidget*  c_documents_open(GtkWidget *app1, gpointer data)
{
   gchar * directory = g_strconcat(AA_DATA_DIR,url,NULL);



   if (documents_control==NULL)
     {
	GtkWidget *window;

	xml = glade_xml_new (directory, widget_name);
	window = glade_xml_get_widget (xml, widget_name);
	
	glade_xml_signal_connect (xml, "on_documentos_destroy",
				  GTK_SIGNAL_FUNC (on_documentos_destroy));


	repair = *(gboolean *)data;
	load_documents_list();
	documents_control = window;
	
     }

   g_free(directory);

   return documents_control;
   
}


static void load_documents_list() {

  GtkWidget *text_widget =glade_xml_get_widget (xml, "documents_text");

  if (company != NULL) {
    gchar *text = g_strdup(_("Company required documents: \n\n"));
    guint len = gtk_text_get_length (GTK_TEXT(text_widget));
    gint *position = (gint *) g_malloc0(sizeof(gint));
    gint len_text = 0;
	
    *position = 0;
	
    gtk_text_backward_delete (GTK_TEXT(text_widget), len);
    
    documents = m_document_find_by_company (d_participant_get_code (d_company_get_participant (company)), repair);

    
    if (documents!=NULL) {
      gint index = 0;
      while (documents[index]!=NULL) {
	  gchar *text_aux = NULL;
	  gchar * desc_aux = NULL;
	  desc_aux = d_document_get_description (documents[index]);
	  text_aux = g_strconcat(text," * ",desc_aux, " \n",NULL);
	  g_free(text);
	  g_free(desc_aux);
	  text = text_aux;
	  text_aux = NULL;
	  index++;
      }
    }
    
    len_text = strlen(text);

    gtk_editable_insert_text (GTK_EDITABLE(text_widget), text, len_text, position);

    g_free(position);
    g_free(text);
  }
  else {
    guint len = gtk_text_get_length (GTK_TEXT(text_widget));

    gtk_text_backward_delete (GTK_TEXT(text_widget), len);
  }
}


static void on_documentos_destroy(GtkWidget *widget,gpointer user_data) {

  if (documents!=NULL) {
    gint index = 0;
    while (documents[index]!=NULL) {
      d_document_clear (documents[index]);
      index++;
    }
    
    g_free(documents);   
      
    documents = NULL;
  }

/*   d_company_clear (company); */
  documents_control = NULL;
/*   company = NULL; */
  xml = NULL;

  if (repair)
    c_repair_document_destroyed();
  else
     c_invoice_document_destroyed();
}

void c_documents_load_company(Company *local_company) {

  d_company_clear (company);

  company = local_company;

  load_documents_list();

}



