/* edit_clients_control.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include <gnome.h>
#include <glade/glade.h>
#include <glib.h>

#include "edit_clients_control.h"
#include "client_model.h"
#include "clients_control.h"
#include "control_util.h"
#include "consortium_model.h"
#include "company_model.h"
#include "client.h"
#include "account_model.h"
#include "insert_account_control.h"
#include "participant.h"
#include "enterprise.h"
#include "supplier.h"
#include "consortium.h"
#include "company.h"
#include "particular_object.h"
#include "garage.h"
#include "util.h"
#include "support.h"
#include "post_address_control.h"
#include "post_address_model.h"
#include <../interface/printing/invoice-print-wrapper.h>

#define C_ADDRES_NONE_CODE ULONG_MAX
static GtkWidget *client_edition=NULL;
static GladeXML *xml;
static Client *general_client = NULL;
static gpointer *person_data = NULL;
static gint general_type = ULONG_MAX;
static GList * accounts;
static PostAddress ** post_addresses=NULL;
static gchar url[]="/src/interface/glade/clients_view.glade";
static gchar widget_name[]="edicion_persona";
static gboolean event_changed = TRUE;

static void show_particular(void);
static void show_garage(void);
static void show_enterprise_general(void);
static void show_enterprise(void);
static void show_company(void);
static void on_person_combo_entry_changed(GtkEntry *entry,gpointer user_data);
static void on_company_combo_entry_changed(GtkEntry *entry,gpointer user_data);
static void fill_person_combo(void);
static void fill_template_combo(void);
static void on_window_destroyed(GtkWidget *widget,gpointer user_data);
static void close_edition_window(GtkWidget *widget,gpointer user_data);
static void fill_consortium_combo(void);
static void clean_screen(void);
static PostAddress * get_address_to_post(void);
static void delete_person(void);
static void modify_person(void);
static void on_insert_checkbutton_toggled(GtkToggleButton *togglebutton,
							  gpointer user_data);
static void set_account_checkbutton_state(gboolean new_state);
static void open_post_edit(GtkWidget *widget,gpointer user_data);
static void on_edit_envelope_button_clicked(GtkWidget *widget,gpointer user_data);
static void repaint_accounts(gpointer value);
static void show_post_addresses(gpointer value);
static void set_modal_true();
static void set_modal_true(void);
static void on_modify_checkbutton_toggled(GtkToggleButton *togglebutton,
					    gpointer user_data);
static void reload_accounts(gpointer value);


GtkWidget *c_clients_open_edition_window(GtkWidget *app1, gpointer data)
{
   gchar * directory = g_strconcat(AA_DATA_DIR,url,NULL);
   
   if (client_edition==NULL)
     {
	GtkWidget *window;
	
	xml = glade_xml_new (directory, widget_name);
	window = glade_xml_get_widget (xml, widget_name);
	
	glade_xml_signal_connect (xml, "on_close_button_client_edition_clicked",
				  GTK_SIGNAL_FUNC (close_edition_window));

	glade_xml_signal_connect (xml, "on_edicion_persona_destroy",
				  GTK_SIGNAL_FUNC (on_window_destroyed));

	glade_xml_signal_connect (xml, "on_person_entry_edit_person_changed",
				  GTK_SIGNAL_FUNC (on_person_combo_entry_changed));

	glade_xml_signal_connect (xml, "on_entry_combo_compania_changed",
				  GTK_SIGNAL_FUNC (on_company_combo_entry_changed));

	glade_xml_signal_connect (xml, "on_limpiar_button_clicked",
				  GTK_SIGNAL_FUNC (clean_screen));

	glade_xml_signal_connect (xml, "on_button_direccion_edita_clicked",
				  GTK_SIGNAL_FUNC (open_post_edit));
	glade_xml_signal_connect (xml, "on_button_sobre_edita_clicked",
				  GTK_SIGNAL_FUNC (on_edit_envelope_button_clicked));

	glade_xml_signal_connect (xml, "on_borrar_button_clicked",
				  GTK_SIGNAL_FUNC (delete_person));

	glade_xml_signal_connect (xml, "on_guardar_button_clicked",
				  GTK_SIGNAL_FUNC (modify_person));

	glade_xml_signal_connect (xml, "on_checkbutton_inserta_edita_clientes_toggled",
				  GTK_SIGNAL_FUNC (on_insert_checkbutton_toggled));

	glade_xml_signal_connect (xml, "on_checkbutton_modifica_toggled",
				  GTK_SIGNAL_FUNC (on_modify_checkbutton_toggled));

	fill_consortium_combo();
	
	fill_person_combo();

	client_edition=window;
	
     }

   g_free(directory);
   return client_edition;
   
}

void c_edit_clients_load_person(Participant *participant){

  gchar *enterprise_legal_id = d_participant_get_legal_id (participant);
  gchar *street = d_participant_get_street (participant);
  gchar *number = d_participant_get_number (participant);
  gchar *floor = d_participant_get_floor (participant);
  gulong postal_code_gulong = d_participant_get_postal_code (participant);
  gchar *postal_code = NULL;
  gchar *town = d_participant_get_town (participant);
  gchar *province = d_participant_get_province (participant);
  gchar *phone1 = d_participant_get_phone (participant);
  gchar *phone2 = d_participant_get_phone2 (participant);
  gchar *fax = d_participant_get_fax (participant);
  gchar *email = d_participant_get_email (participant);
  gchar *name = d_participant_get_name (participant);
  gchar *short_name = d_participant_get_short_name (participant);
  
  gulong codigo_gulong = d_participant_get_code (participant);
  gint type = m_client_get_type (codigo_gulong);
  Client *client = m_client_get_by_code (codigo_gulong);
  gdouble discount_gint = -1;
  gchar *discount = NULL;
  gboolean agent = FALSE;
  GtkWidget *agent_check =glade_xml_get_widget (xml, "agente_checkbutton");

  if (client != NULL){
    discount_gint = d_client_get_discount (client);
    agent = d_client_get_agent (client);
  }

  if (discount_gint!=-1)
    discount = g_strdup_printf("%.2f",discount_gint);

  if (client!=NULL)
    d_client_set_participant (client, participant);
  
  if (postal_code_gulong!=ULONG_MAX)
    postal_code = g_strdup_printf("%lu",postal_code_gulong);
  
  person_data = m_client_get_data (type,codigo_gulong);

  gtk_toggle_button_set_active((GtkToggleButton *) agent_check, agent);

  if (street != NULL) {
    GtkWidget *street_entry =glade_xml_get_widget (xml, "calle_entry");
    gtk_entry_set_text (GTK_ENTRY(street_entry),street);
    g_free(street);
  }

  if (number != NULL) {
    GtkWidget *number_entry =glade_xml_get_widget (xml, "numero_entry");
    gtk_entry_set_text (GTK_ENTRY(number_entry),number);
    g_free(number);
  }

  if (floor != NULL) {
    GtkWidget *floor_entry =glade_xml_get_widget (xml, "piso_entry");
    gtk_entry_set_text (GTK_ENTRY(floor_entry),floor);
    g_free(floor);
  }

  if (postal_code != NULL) {
    GtkWidget *postal_code_entry =glade_xml_get_widget (xml, "cp_entry");
    gtk_entry_set_text (GTK_ENTRY(postal_code_entry),postal_code);
    g_free(postal_code);
  }

  if (discount != NULL) {
    GtkWidget *discount_entry =glade_xml_get_widget (xml, "dto_entry");
    gtk_entry_set_text (GTK_ENTRY(discount_entry),discount);
    g_free(discount);
  }

  if (town != NULL) {
    GtkWidget *town_entry =glade_xml_get_widget (xml, "municipio_entry");
    gtk_entry_set_text (GTK_ENTRY(town_entry),town);
    g_free(town);
  }

  if (province != NULL) {
    GtkWidget *province_entry =glade_xml_get_widget (xml, "provincia_entry");
    gtk_entry_set_text (GTK_ENTRY(province_entry),province);
    g_free(province);
  }

  if (phone1 != NULL) {
    GtkWidget *phone1_entry =glade_xml_get_widget (xml, "telefono1_entry");
    gtk_entry_set_text (GTK_ENTRY(phone1_entry),phone1);
    g_free(phone1);
  }

  if (phone2 != NULL) {
    GtkWidget *phone2_entry =glade_xml_get_widget (xml, "telefono2_entry");
    gtk_entry_set_text (GTK_ENTRY(phone2_entry),phone2);
    g_free(phone2);
  }

  if (fax != NULL) {
    GtkWidget *fax_entry =glade_xml_get_widget (xml, "fax_entry");
    gtk_entry_set_text (GTK_ENTRY(fax_entry),fax);
    g_free(fax);
  }

  if (email != NULL) {
    GtkWidget *email_entry =glade_xml_get_widget (xml, "email_entry");
    gtk_entry_set_text (GTK_ENTRY(email_entry),email);
    g_free(email);
  }

  if (short_name != NULL) {
      GtkWidget *name_entry =glade_xml_get_widget (xml, "nombre_corto_edit_entry");
      gtk_entry_set_text (GTK_ENTRY(name_entry),short_name);
      g_free(short_name);
  }

  switch (type)
    {
    case M_CLIENT_PERSON_CODE:{      

      GtkWidget *combo_entry =glade_xml_get_widget (xml, "combo_entry_edit");
      GtkWidget *combo_person =glade_xml_get_widget (xml, "person_combo_edition_person");

      gtk_entry_set_text(GTK_ENTRY(combo_entry),_(C_PERSON_MESSAGE));
      gtk_widget_set_sensitive (combo_person, FALSE);


      if (person_data!=NULL){
	gchar *surname1 = d_particular_get_first_surname ((ParticularObject *) person_data);
	gchar *surname2 = d_particular_get_second_surname ((ParticularObject *) person_data);
	
	if (surname1 != NULL) {
	  GtkWidget *surname1_entry =glade_xml_get_widget (xml, "apellido1_entry");
	  gtk_entry_set_text (GTK_ENTRY(surname1_entry),surname1);
	  g_free(surname1);
	}

	if (surname2 != NULL) {
	  GtkWidget *surname2_entry =glade_xml_get_widget (xml, "apellido2_entry");
	  gtk_entry_set_text (GTK_ENTRY(surname2_entry),surname2);
	  g_free(surname2);
	}
	
	d_particular_set_participant ((ParticularObject *)person_data, participant);
      }

      if (enterprise_legal_id != NULL) {
	GtkWidget *enterprise_legal_id_entry =glade_xml_get_widget (xml, "dni_entry");
	gtk_entry_set_text (GTK_ENTRY(enterprise_legal_id_entry),enterprise_legal_id);
	g_free(enterprise_legal_id);
      }

      if (name != NULL) {
	GtkWidget *name_entry =glade_xml_get_widget (xml, "nombre_entry");
	gtk_entry_set_text (GTK_ENTRY(name_entry),name);
	g_free(name);
      }

      
      break;}
      
    case M_CLIENT_ENTERPRISE_CODE:{

      GtkWidget *combo_entry =glade_xml_get_widget (xml, "combo_entry_edit");
      GtkWidget *combo_person =glade_xml_get_widget (xml, "person_combo_edition_person");

      gtk_entry_set_text(GTK_ENTRY(combo_entry),_(C_ENTERPRISE_MESSAGE));
      gtk_widget_set_sensitive (combo_person, FALSE);

      if (person_data!=NULL){
	GtkWidget *parent_company =glade_xml_get_widget (xml, "parent_company_checkbutton");  
	gboolean belong_to_parent_company = d_enterprise_get_belong_to_principal ((Enterprise *) person_data);
	d_enterprise_set_participant ((Enterprise *)person_data, participant);

	gtk_toggle_button_set_active((GtkToggleButton *) parent_company, belong_to_parent_company);

      }

      if (enterprise_legal_id != NULL) {
	GtkWidget *enterprise_legal_id_entry =glade_xml_get_widget (xml, "cif_entry");
	gtk_entry_set_text (GTK_ENTRY(enterprise_legal_id_entry),enterprise_legal_id);
	g_free(enterprise_legal_id);
      }

      if (name != NULL) {
	GtkWidget *name_entry =glade_xml_get_widget (xml, "razonsocial_entry");
	gtk_entry_set_text (GTK_ENTRY(name_entry),name);
	g_free(name);
      }
      
      

    }break;
    case M_CLIENT_SUPPLIER_CODE:{
      
      GtkWidget *combo_entry =glade_xml_get_widget (xml, "combo_entry_edit");
      GtkWidget *combo_person =glade_xml_get_widget (xml, "person_combo_edition_person");
      gtk_entry_set_text(GTK_ENTRY(combo_entry),_(C_SUPPLIER_MESSAGE));
      gtk_widget_set_sensitive (combo_person, FALSE);

      if (person_data!=NULL)
	d_supplier_set_participant ((Supplier *)person_data, participant);
      
      if (enterprise_legal_id != NULL) {
	GtkWidget *enterprise_legal_id_entry =glade_xml_get_widget (xml, "cif_entry");
	gtk_entry_set_text (GTK_ENTRY(enterprise_legal_id_entry),enterprise_legal_id);
	g_free(enterprise_legal_id);
      }

      if (name != NULL) {
	GtkWidget *name_entry =glade_xml_get_widget (xml, "razonsocial_entry");
	gtk_entry_set_text (GTK_ENTRY(name_entry),name);
	g_free(name);
      }
      

    }break;
    case M_CLIENT_COMPANY_CODE:{

      GtkWidget *combo_entry =glade_xml_get_widget (xml, "combo_entry_edit");
      GtkWidget *combo_person =glade_xml_get_widget (xml, "person_combo_edition_person");      
      gtk_entry_set_text(GTK_ENTRY(combo_entry),_(C_COMPANY_MESSAGE));
      gtk_widget_set_sensitive (combo_person, FALSE);

      if (person_data!=NULL){
	gulong consortium_code = d_company_get_consortium_code ((Company *) person_data);
	gdouble discount = d_company_get_discount ((Company *) person_data);
	gint template = d_company_get_template ((Company *) person_data);
	gchar *ck_code = d_company_get_ck_code ((Company *) person_data);	
	
	GtkWidget *combo_consortium = glade_xml_get_widget (xml, "compania_combo_edit");
	gchar *consortium_code_gchar=NULL;
	gchar *consortium_name=NULL;

	if (consortium_code !=ULONG_MAX) {
	    consortium_code_gchar = g_strdup_printf("%lu", consortium_code);
	} else {
            consortium_code_gchar = g_strdup_printf("%d", D_CONSORTIUM_NONE);
	}
        consortium_name = g_strdup(c_util_get_name_of_codechar_from_combo (combo_consortium, consortium_code_gchar));
	if (consortium_name !=NULL){
	  GtkWidget *entry_combo_consortium = glade_xml_get_widget (xml, "entry_combo_consorcio");
	  gtk_entry_set_text(GTK_ENTRY(entry_combo_consortium), consortium_name);
	  g_free(consortium_name);
	}

	if (discount != -1) {
	  GtkWidget *discount_company = glade_xml_get_widget (xml, "dto_company_entry");
	  gchar *discount_gchar = g_strdup_printf("%.2f",discount);
	  gtk_entry_set_text(GTK_ENTRY(discount_company),discount_gchar);
	  g_free(discount_gchar);
	}

	if (template != -1) {
	  GtkWidget *template_entry = glade_xml_get_widget (xml, "template_entry");
	  gchar *template_gchar = d_invoice_get_template_name (template);
	  gtk_entry_set_text(GTK_ENTRY(template_entry), template_gchar);
	  g_free(template_gchar);
	}
	
	if (ck_code != NULL) {
		GtkWidget *ck_code_entry = glade_xml_get_widget (xml, "ck_code_entry");
		gtk_entry_set_text(GTK_ENTRY(ck_code_entry),ck_code);
		g_free(ck_code);
	}

	g_free(consortium_code_gchar);
	
	d_company_set_participant ((Company *)person_data, participant);	
      }

      if (enterprise_legal_id != NULL) {
	GtkWidget *enterprise_legal_id_entry =glade_xml_get_widget (xml, "cif_entry");
	gtk_entry_set_text (GTK_ENTRY(enterprise_legal_id_entry),enterprise_legal_id);
	g_free(enterprise_legal_id);
      }

      if (name != NULL) {
	GtkWidget *name_entry =glade_xml_get_widget (xml, "razonsocial_entry");
	gtk_entry_set_text (GTK_ENTRY(name_entry),name);
	g_free(name);
      }

      
    } break;
    
    case M_CLIENT_CONSORTIUM_CODE:{

      GtkWidget *combo_entry =glade_xml_get_widget (xml, "combo_entry_edit");
      GtkWidget *combo_person =glade_xml_get_widget (xml, "person_combo_edition_person");
      gtk_entry_set_text(GTK_ENTRY(combo_entry),_(C_CONSORTIUM_MESSAGE));
      gtk_widget_set_sensitive (combo_person, FALSE);

      if (person_data!=NULL)
	d_consortium_set_participant ((Consortium *)person_data, participant);

      if (enterprise_legal_id != NULL) {
	GtkWidget *enterprise_legal_id_entry =glade_xml_get_widget (xml, "cif_entry");
	gtk_entry_set_text (GTK_ENTRY(enterprise_legal_id_entry),enterprise_legal_id);
	g_free(enterprise_legal_id);
      }

      if (name != NULL) {
	GtkWidget *name_entry =glade_xml_get_widget (xml, "razonsocial_entry");
	gtk_entry_set_text (GTK_ENTRY(name_entry),name);
	g_free(name);
      }
      

    } break;
    
    case M_CLIENT_GARAGE_CODE:{

      GtkWidget *combo_entry =glade_xml_get_widget (xml, "combo_entry_edit");
      GtkWidget *combo_person =glade_xml_get_widget (xml, "person_combo_edition_person");
      gtk_entry_set_text(GTK_ENTRY(combo_entry),_(C_GARAGE_MESSAGE));
      gtk_widget_set_sensitive (combo_person, FALSE);

      if (person_data!=NULL){
	gdouble comision = d_garage_get_commission ((Garage *) person_data);
	
	if (comision != -1) {
	  GtkWidget *comision_entry = glade_xml_get_widget (xml, "comision_entry");
	  gchar *comision_gchar = g_strdup_printf("%.2f",comision);

	  gtk_entry_set_text(GTK_ENTRY(comision_entry), comision_gchar);
	  g_free(comision_gchar);
	}
	
	d_garage_set_participant ((Garage *)person_data, participant);
      }

      if (enterprise_legal_id != NULL) {
	GtkWidget *enterprise_legal_id_entry =glade_xml_get_widget (xml, "cif_entry");
	gtk_entry_set_text (GTK_ENTRY(enterprise_legal_id_entry),enterprise_legal_id);
	g_free(enterprise_legal_id);
      }

      if (name != NULL) {
	GtkWidget *name_entry =glade_xml_get_widget (xml, "razonsocial_entry");
	gtk_entry_set_text (GTK_ENTRY(name_entry),name);
	g_free(name);
      }

      
    } break;
    
    default: break;
    
    }
  
  accounts = m_account_find_by_participant_code (codigo_gulong);
  repaint_accounts(NULL);
  general_client = client;
  show_post_addresses(NULL);
  general_type = type;

}


static void fill_template_combo() {
    GtkWidget * combo = glade_xml_get_widget (xml, "template_combo_update");
    	    	
    c_util_clean_code_and_name_combo (combo);
    		 
    c_util_set_code_and_name_on_combo (D_INVOICE_NORMAL_CODE,D_INVOICE_MESSAGE_NORMAL,GTK_COMBO(combo));
}


static void fill_consortium_combo() {
    GtkWidget * combo;
    Consortium ** consortiums;

    combo = glade_xml_get_widget (xml, "compania_combo_edit");

    c_util_set_code_and_name_on_combo (D_CONSORTIUM_NONE,g_strdup(_("<None>")),GTK_COMBO(combo));
    consortiums = m_consortium_find_by_code (ULONG_MAX);

    c_util_fill_combo_with_consortiums (combo,consortiums);    

}

static void fill_person_combo() {
    GtkWidget * combo;

    combo = glade_xml_get_widget (xml, "person_combo_edition_person");

    c_util_set_code_and_name_on_combo (M_CLIENT_PERSON_CODE,_(C_PERSON_MESSAGE),GTK_COMBO(combo));
    c_util_set_code_and_name_on_combo (M_CLIENT_ENTERPRISE_CODE,_(C_ENTERPRISE_MESSAGE),GTK_COMBO(combo));
#ifndef IS_SHOP
    c_util_set_code_and_name_on_combo (M_CLIENT_SUPPLIER_CODE,_(C_SUPPLIER_MESSAGE),GTK_COMBO(combo));
    c_util_set_code_and_name_on_combo (M_CLIENT_COMPANY_CODE,_(C_COMPANY_MESSAGE),GTK_COMBO(combo));
    c_util_set_code_and_name_on_combo (M_CLIENT_CONSORTIUM_CODE,_(C_CONSORTIUM_MESSAGE),GTK_COMBO(combo));
#endif
    c_util_set_code_and_name_on_combo (M_CLIENT_GARAGE_CODE,_(C_GARAGE_MESSAGE),GTK_COMBO(combo));

}

static void on_window_destroyed(GtkWidget *widget,gpointer user_data){

  if (person_data != NULL)
    {
      
      switch (general_type) {
      case M_CLIENT_PERSON_CODE: {
	d_particular_set_participant ((ParticularObject *) person_data, NULL);
	d_particular_clear ((ParticularObject *) person_data);
	break;}
      case M_CLIENT_ENTERPRISE_CODE: {
	d_enterprise_set_participant ((Enterprise *) person_data, NULL);
	d_enterprise_clear ((Enterprise *) person_data);
	break;}
      case M_CLIENT_SUPPLIER_CODE: {
	d_supplier_set_participant ((Supplier *) person_data, NULL);
	d_supplier_clear ((Supplier *) person_data);
	break;}
      case M_CLIENT_COMPANY_CODE: {
	d_company_set_participant ((Company *) person_data, NULL);
	d_company_clear ((Company *) person_data);
	break;}
      case M_CLIENT_CONSORTIUM_CODE: {
	d_consortium_set_participant ((Consortium *) person_data, NULL);
	d_consortium_clear ((Consortium *) person_data);
	break;}
      case M_CLIENT_GARAGE_CODE: {
	d_garage_set_participant ((Garage *) person_data, NULL);
	d_garage_clear ((Garage *) person_data);
	break;}
      default :	break;
      }
    }

  if (general_client!=NULL){
    d_client_set_participant (general_client, NULL);
    if (accounts!=NULL) {
	g_list_free(accounts);
    }
    if (post_addresses!=NULL) {
	m_post_address_clear_array (post_addresses);
    }
    post_addresses = NULL;
    accounts =NULL;
    d_client_clear (general_client);

    general_client = NULL;
  }

  client_edition = NULL;
  general_type= ULONG_MAX;

  c_clients_control_find_persons ();

}


static void close_edition_window(GtkWidget *widget,gpointer user_data){
    if (client_edition!=NULL)
	gtk_widget_destroy(client_edition);
    client_edition=NULL;

  /*   on_edicion_persona_destroy(widget, user_data); */
}

static void on_person_combo_entry_changed(GtkEntry *entry,gpointer user_data){
  GtkWidget * combo;
  gulong codeCombo;

  combo = glade_xml_get_widget (xml,"person_combo_edition_person");

  codeCombo = c_util_get_code_from_combo (combo);

  switch (codeCombo)
    {
    case M_CLIENT_PERSON_CODE: show_particular();break;
    case M_CLIENT_ENTERPRISE_CODE: show_enterprise();break;
    case M_CLIENT_SUPPLIER_CODE: show_enterprise_general();break;
    case M_CLIENT_COMPANY_CODE: show_company();break;
    case M_CLIENT_CONSORTIUM_CODE: show_enterprise_general();break;
    case M_CLIENT_GARAGE_CODE: show_garage();break;
    }
}


static void on_company_combo_entry_changed(GtkEntry *entry,gpointer user_data){
 
  GtkWidget *ck_label = glade_xml_get_widget(xml, "ck_label");
  GtkWidget *ck_entry = glade_xml_get_widget(xml, "ck_code_entry");
/*   GtkWidget * combo = glade_xml_get_widget (xml,"compania_combo_edit"); */
/*   gulong codeCombo = c_util_get_code_from_combo (combo); */

  
  gtk_widget_hide(ck_label);
  gtk_widget_hide(ck_entry);
  gtk_entry_set_text(GTK_ENTRY(ck_entry),"");

}


static void show_particular(){
  
  GtkWidget *name_label = glade_xml_get_widget(xml, "nombre_label");
  GtkWidget *surname1_label = glade_xml_get_widget(xml, "apellido1_label");
  GtkWidget *surname2_label = glade_xml_get_widget(xml, "apellido2_label");
  GtkWidget *name_entry = glade_xml_get_widget(xml, "nombre_entry");
  GtkWidget *surname1_entry = glade_xml_get_widget(xml, "apellido1_entry");
  GtkWidget *surname2_entry = glade_xml_get_widget(xml, "apellido2_entry");
  GtkWidget *personal_legal_id_label = glade_xml_get_widget(xml, "dni_label");
  GtkWidget *personal_legal_id_entry = glade_xml_get_widget(xml, "dni_entry");
  GtkWidget *trade_name_label = glade_xml_get_widget(xml, "razonsocial_label");
  GtkWidget *trade_name_entry = glade_xml_get_widget(xml, "razonsocial_entry");
//  GtkWidget *short_name_label = glade_xml_get_widget(xml, "nombre_corto_label");
//  GtkWidget *short_name_entry = glade_xml_get_widget(xml, "nombre_corto_edit_entry");  
  GtkWidget *enterprise_legal_id_label = glade_xml_get_widget(xml, "cif_label");
  GtkWidget *enterprise_legal_id_entry = glade_xml_get_widget(xml, "cif_entry");
  GtkWidget *parent_company_checkbutton = glade_xml_get_widget(xml, "parent_company_checkbutton");
  GtkWidget *company_combo_edit = glade_xml_get_widget(xml, "compania_combo_edit");
  GtkWidget *consortium_label = glade_xml_get_widget(xml, "consorcio_label");
  GtkWidget *comision_entry = glade_xml_get_widget(xml, "comision_entry");
  GtkWidget *comision_label = glade_xml_get_widget(xml, "comision_label");
  GtkWidget *discount_company_label = glade_xml_get_widget(xml, "dto_company_label");
  GtkWidget *discount_company_entry = glade_xml_get_widget(xml, "dto_company_entry");
  GtkWidget *ck_label = glade_xml_get_widget(xml, "ck_label");
  GtkWidget *ck_code_entry = glade_xml_get_widget(xml, "ck_code_entry");
  GtkWidget *template_combo_update = glade_xml_get_widget(xml, "template_combo_update");
  GtkWidget *template_label = glade_xml_get_widget(xml, "template_label");

 
  gtk_widget_show(name_label);
  gtk_widget_show(name_entry);

  gtk_widget_show(surname1_label);
  gtk_widget_show(surname1_entry);

  gtk_widget_show(surname2_label);
  gtk_widget_show(surname2_entry);

  gtk_widget_show(personal_legal_id_label);
  gtk_widget_show(personal_legal_id_entry);
  
  gtk_widget_hide(trade_name_label);
  gtk_widget_hide(trade_name_entry);
  
//  gtk_widget_hide(short_name_label);
//  gtk_widget_hide(short_name_entry);
  
  gtk_widget_hide(enterprise_legal_id_label);
  gtk_widget_hide(enterprise_legal_id_entry);

  gtk_widget_hide(parent_company_checkbutton);

  gtk_widget_hide(company_combo_edit);
  gtk_widget_hide(consortium_label);

  gtk_widget_hide(comision_entry);
  gtk_widget_hide(comision_label);

  gtk_widget_hide(discount_company_label);
  gtk_widget_hide(discount_company_entry);
  
  gtk_widget_hide(ck_label);
  gtk_widget_hide(ck_code_entry);   
  
  gtk_widget_hide(template_combo_update);
  gtk_widget_hide(template_label);   
}

static void show_enterprise(){
#ifndef IS_SHOP
  GtkWidget *parent_company_checkbutton = glade_xml_get_widget(xml, "parent_company_checkbutton");
#endif

  show_enterprise_general();
#ifndef IS_SHOP
  gtk_widget_show(parent_company_checkbutton);
#endif

}

static void show_company(){
  GtkWidget *company_combo_edit = glade_xml_get_widget(xml, "compania_combo_edit");
/*   gulong company_combo_code = c_util_get_code_from_combo (company_combo_edit); */
  GtkWidget *consortium_label = glade_xml_get_widget(xml, "consorcio_label");
  GtkWidget *discount_company_label = glade_xml_get_widget(xml, "dto_company_label");
  GtkWidget *discount_company_entry = glade_xml_get_widget(xml, "dto_company_entry");
  GtkWidget *template_combo_update = glade_xml_get_widget(xml, "template_combo_update");
  GtkWidget *template_label = glade_xml_get_widget(xml, "template_label");
  
  fill_template_combo();

  show_enterprise_general();
  gtk_widget_show(company_combo_edit);
  gtk_widget_show(consortium_label);
  gtk_widget_show(discount_company_label);
  gtk_widget_show(discount_company_entry);
  gtk_widget_show(template_combo_update);
  gtk_widget_show(template_label);  

}

static void show_garage(){
  GtkWidget *comision_entry = glade_xml_get_widget(xml, "comision_entry");
  GtkWidget *comision_label = glade_xml_get_widget(xml, "comision_label");

  show_enterprise_general();
  gtk_widget_show(comision_entry);
  gtk_widget_show(comision_label);
}

static void show_enterprise_general(){
 
  GtkWidget *name_label = glade_xml_get_widget(xml, "nombre_label");
  GtkWidget *surname1_label = glade_xml_get_widget(xml, "apellido1_label");
  GtkWidget *surname2_label = glade_xml_get_widget(xml, "apellido2_label");
  GtkWidget *name_entry = glade_xml_get_widget(xml, "nombre_entry");
  GtkWidget *surname1_entry = glade_xml_get_widget(xml, "apellido1_entry");
  GtkWidget *surname2_entry = glade_xml_get_widget(xml, "apellido2_entry");
  GtkWidget *personal_legal_id_label = glade_xml_get_widget(xml, "dni_label");
  GtkWidget *personal_legal_id_entry = glade_xml_get_widget(xml, "dni_entry");
  GtkWidget *trade_name_label = glade_xml_get_widget(xml, "razonsocial_label");
  GtkWidget *trade_name_entry = glade_xml_get_widget(xml, "razonsocial_entry");
  GtkWidget *short_name_label = glade_xml_get_widget(xml, "nombre_corto_label");
  GtkWidget *short_name_entry = glade_xml_get_widget(xml, "nombre_corto_edit_entry");
  GtkWidget *enterprise_legal_id_label = glade_xml_get_widget(xml, "cif_label");
  GtkWidget *enterprise_legal_id_entry = glade_xml_get_widget(xml, "cif_entry");
  GtkWidget *parent_company_checkbutton = glade_xml_get_widget(xml, "parent_company_checkbutton");
  GtkWidget *company_combo_edit = glade_xml_get_widget(xml, "compania_combo_edit");
  GtkWidget *consortium_label = glade_xml_get_widget(xml, "consorcio_label");
  GtkWidget *comision_entry = glade_xml_get_widget(xml, "comision_entry");
  GtkWidget *comision_label = glade_xml_get_widget(xml, "comision_label");
  GtkWidget *discount_company_label = glade_xml_get_widget(xml, "dto_company_label");
  GtkWidget *discount_company_entry = glade_xml_get_widget(xml, "dto_company_entry");
  GtkWidget *ck_label = glade_xml_get_widget(xml, "ck_label");
  GtkWidget *ck_code_entry = glade_xml_get_widget(xml, "ck_code_entry");
  GtkWidget *template_combo_update = glade_xml_get_widget(xml, "template_combo_update");
  GtkWidget *template_label = glade_xml_get_widget(xml, "template_label");

  gtk_widget_hide(name_label);
  gtk_widget_hide(name_entry);

  gtk_widget_hide(surname1_label);
  gtk_widget_hide(surname1_entry);

  gtk_widget_hide(surname2_label);
  gtk_widget_hide(surname2_entry);

  gtk_widget_hide(personal_legal_id_label);
  gtk_widget_hide(personal_legal_id_entry);
  
  gtk_widget_show(trade_name_label);
  gtk_widget_show(trade_name_entry);
  
  gtk_widget_show(short_name_label);
  gtk_widget_show(short_name_entry);
  
  gtk_widget_show(enterprise_legal_id_label);
  gtk_widget_show(enterprise_legal_id_entry);

  gtk_widget_hide(parent_company_checkbutton);
  gtk_widget_hide(company_combo_edit);

  gtk_widget_hide(consortium_label);
  gtk_widget_hide(comision_entry);
  gtk_widget_hide(comision_label);

  gtk_widget_hide(discount_company_label);
  gtk_widget_hide(discount_company_entry);

  gtk_widget_hide(template_combo_update);
  gtk_widget_hide(template_label);
  
  gtk_widget_hide(ck_label);
  gtk_widget_hide(ck_code_entry);  
}

static void clean_screen() {

  GtkWidget *container = GTK_WIDGET(glade_xml_get_widget(xml, widget_name));
  
  gtk_container_foreach(GTK_CONTAINER(container),&c_util_clean_widget,NULL);

}

static void delete_person() {

  if (general_client != NULL) {
    Participant *participant = d_client_get_participant (general_client);
    gboolean result = FALSE;
    GtkWidget *combo = glade_xml_get_widget (xml, "person_combo_edition_person");
    GtkWidget *combo_accounts = glade_xml_get_widget (xml, "combo_cuentas_edita_clientes");
    GtkWidget *combo_post = glade_xml_get_widget (xml, "combo_direccion_edita");
    gulong type = c_util_get_code_from_combo (combo);
    
    result = m_client_delete (participant, type);
        
    d_participant_clear (participant);
    c_util_clean_code_and_name_combo (combo);
    c_util_clean_code_and_name_combo (combo_accounts);
    c_util_clean_code_and_name_combo (combo_post);
    if (accounts!=NULL) {
	g_list_free(accounts);
        accounts = NULL;
    }
    if (post_addresses!=NULL) {
	m_post_address_clear_array (post_addresses);
	post_addresses = NULL;
    }


    if (result) {
	if (client_edition!=NULL)
	    gtk_widget_destroy(client_edition);
	client_edition=NULL;

    }
    else c_util_open_error_window (_("The client couldn't be removed, check the data"));
  }

}

static void modify_person() {

  Participant *participant = d_client_get_participant (general_client);
  GtkWidget *combo = glade_xml_get_widget (xml, "person_combo_edition_person");
  gulong type = c_util_get_code_from_combo (combo);
  gchar *street = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "calle_entry")),0,-1);
  gchar *number = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "numero_entry")),0,-1);
  gchar *floor = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "piso_entry")),0,-1);
  gchar *postal_code = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "cp_entry")),0,-1);
  gchar *town = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "municipio_entry")),0,-1);
  gchar *province = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "provincia_entry")),0,-1);
  gchar *phone1 = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "telefono1_entry")),0,-1);
  gchar *phone2 = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "telefono2_entry")),0,-1);
  gchar *fax = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "fax_entry")),0,-1);
  gchar *email = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "email_entry")),0,-1);
  gchar *discount = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "dto_entry")),0,-1);
  gchar *short_name = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "nombre_corto_edit_entry")),0,-1);      

  gboolean agent = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget (xml, "agente_checkbutton")));
  
  gboolean result = FALSE;
  gchar ** stop;

  d_client_set_participant (general_client, participant);
 
  stop = (gchar **) g_malloc0(sizeof(gchar *));
      
  if (g_strcasecmp(short_name,""))
    d_participant_set_short_name (participant, short_name);

  if (g_strcasecmp(email,""))
    d_participant_set_email (participant, email);
  else d_participant_set_email (participant, NULL);
  if (g_strcasecmp(phone1,""))
    d_participant_set_phone (participant, phone1);
  else d_participant_set_phone (participant, NULL);
  if (g_strcasecmp(phone2,""))
    d_participant_set_phone2 (participant,phone2);
  else d_participant_set_phone2 (participant,NULL);
  if (g_strcasecmp(fax,""))
    d_participant_set_fax (participant,fax);
  else d_participant_set_fax (participant,NULL);
  if (g_strcasecmp(street,""))
    d_participant_set_street (participant,street);
  else d_participant_set_street (participant,NULL);
  if (g_strcasecmp(number,""))
    d_participant_set_number (participant,number);
  else d_participant_set_number (participant,NULL);
  if (g_strcasecmp(floor,""))
    d_participant_set_floor (participant, floor);
  else d_participant_set_floor (participant, NULL);
  if (g_strcasecmp(postal_code,""))
    d_participant_set_postal_code (participant,strtoul(postal_code,stop,10));
  else d_participant_set_postal_code (participant, ULONG_MAX);
  if (g_strcasecmp(town,""))
    d_participant_set_town (participant, town);
  else d_participant_set_town (participant, NULL);
  if (g_strcasecmp(province,""))
    d_participant_set_province (participant, province);
  else d_participant_set_province (participant, NULL);
  if (g_strcasecmp(discount,"")) {
      u_points_to_commas (discount);
      d_client_set_discount (general_client, atof(discount));
  }
  else d_client_set_discount (general_client,-1);

  d_client_set_agent (general_client,agent);

  switch (type)
    {
    case M_CLIENT_PERSON_CODE:{
      
      ParticularObject *particular = (ParticularObject *) person_data;
      gchar *personal_legal_id = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "dni_entry")),0,-1);
      gchar *name = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "nombre_entry")),0,-1);
      gchar *surname1 = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "apellido1_entry")),0,-1);
      gchar *surname2 = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "apellido2_entry")),0,-1);      
      gchar *nick = g_strconcat(name," ",surname1," ",surname2);

      if (g_strcasecmp(personal_legal_id,""))
	d_participant_set_legal_id (participant, personal_legal_id);
      if (g_strcasecmp(nick,""))
	d_participant_set_nick (participant, nick);
      if (g_strcasecmp(name,""))
	d_participant_set_name (participant, name);


      if (g_strcasecmp(surname1,""))
	d_particular_set_first_surname (particular, surname1);
      if (g_strcasecmp(surname2,""))
	d_particular_set_second_surname (particular, surname2);

      d_particular_set_participant (particular, participant);
      
      result = m_client_update ((gpointer) particular, general_client, type,accounts);
      
      particular = NULL;
      g_free(personal_legal_id);
      g_free(name);
      g_free(surname1);
      g_free(surname2);
      g_free(nick);
      break;}
   
    case M_CLIENT_ENTERPRISE_CODE:{
      
      Enterprise *enterprise = (Enterprise *) person_data;
      gchar *enterprise_legal_id = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "cif_entry")),0,-1);
      gchar *trade_name = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "razonsocial_entry")),0,-1);
      gboolean parent_company_checkbutton = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget (xml, "parent_company_checkbutton")));

      if (g_strcasecmp(enterprise_legal_id,""))
	d_participant_set_legal_id (participant, enterprise_legal_id);
      
      if (g_strcasecmp(trade_name,""))
	d_participant_set_nick (participant, trade_name);
      
      if (g_strcasecmp(trade_name,""))
	d_participant_set_name (participant, trade_name);

      
      d_enterprise_set_belong_to_principal (enterprise, parent_company_checkbutton);	
      d_enterprise_set_participant (enterprise, participant);

      result = m_client_update ((gpointer) enterprise, general_client, type,accounts);

      enterprise = NULL;
      g_free(enterprise_legal_id);
      g_free(trade_name);
      break;}

    case M_CLIENT_SUPPLIER_CODE:{

      Supplier *supplier = (Supplier *) person_data;
      gchar *enterprise_legal_id = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "cif_entry")),0,-1);
      gchar *trade_name = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "razonsocial_entry")),0,-1);

      if (g_strcasecmp(enterprise_legal_id,""))
	d_participant_set_legal_id (participant, enterprise_legal_id);
      if (g_strcasecmp(trade_name,""))
	d_participant_set_nick (participant, trade_name);
      if (g_strcasecmp(trade_name,""))
	d_participant_set_name (participant, trade_name);

      d_supplier_set_participant (supplier, participant);

      result = m_client_update ((gpointer) supplier, general_client, type,accounts);

      supplier = NULL;
      g_free(enterprise_legal_id);
      g_free(trade_name);
      break;}
  
    case M_CLIENT_COMPANY_CODE:{
 
      Company *company = (Company *) person_data;
      gchar *enterprise_legal_id = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "cif_entry")),0,-1);
      gchar *trade_name = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "razonsocial_entry")),0,-1);
      GtkWidget *combo_consortium = glade_xml_get_widget (xml, "compania_combo_edit");
      GtkWidget *combo_plantilla = glade_xml_get_widget (xml, "template_combo_update");      
      gchar *ck_code = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "ck_code_entry")),0,-1);
      gchar *discount = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "dto_company_entry")),0,-1);
      gulong consortium = c_util_get_code_from_combo (combo_consortium);
      gint template = c_util_get_code_from_combo (combo_plantilla);

      if (g_strcasecmp(enterprise_legal_id,""))
	d_participant_set_legal_id (participant, enterprise_legal_id);

      if (g_strcasecmp(trade_name,""))
	d_participant_set_nick (participant, trade_name);

      if (g_strcasecmp(trade_name,""))
	d_participant_set_name (participant, trade_name);

 
      d_company_set_consortium_code (company, consortium);
      d_company_set_template (company, template);

      if (g_strcasecmp(discount,"")) {
	  u_points_to_commas (discount);
	  d_company_set_discount (company, atof(discount));
      }

      if (g_strcasecmp(ck_code,""))
	d_company_set_ck_code (company, ck_code);            
      
      d_company_set_participant (company, participant);

      result = m_client_update ((gpointer) company, general_client, type,accounts);

      company = NULL;
      g_free(enterprise_legal_id);
      g_free(trade_name);
      g_free(discount);
      break;}
      
    case M_CLIENT_CONSORTIUM_CODE:{
      
      Consortium *consortium = (Consortium *) person_data;
      gchar *enterprise_legal_id = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "cif_entry")),0,-1);
      gchar *trade_name = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "razonsocial_entry")),0,-1);

      if (g_strcasecmp(enterprise_legal_id,""))
	d_participant_set_legal_id (participant, enterprise_legal_id);
      if (g_strcasecmp(trade_name,""))
	d_participant_set_nick (participant, trade_name);
      if (g_strcasecmp(trade_name,""))
	d_participant_set_name (participant, trade_name);


      d_consortium_set_participant (consortium, participant);

      result = m_client_update ((gpointer) consortium, general_client, type,accounts);

      consortium = NULL;
      g_free(enterprise_legal_id);
      g_free(trade_name);
      break;}

    case M_CLIENT_GARAGE_CODE:{

      Garage *garage = (Garage *) person_data;
      gchar *enterprise_legal_id = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "cif_entry")),0,-1);
      gchar *trade_name = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "razonsocial_entry")),0,-1);
      gchar *comision = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "comision_entry")),0,-1);

      if (g_strcasecmp(enterprise_legal_id,""))
	d_participant_set_legal_id (participant, enterprise_legal_id);
      if (g_strcasecmp(trade_name,""))
	d_participant_set_nick (participant, trade_name);
      if (g_strcasecmp(trade_name,""))
	d_participant_set_name (participant, trade_name);

      if (g_strcasecmp(comision,"")) {
          u_points_to_commas (comision);
	  d_garage_set_commission (garage, atof(comision));
      }
      
      d_garage_set_participant (garage, participant);

      result = m_client_update ((gpointer) garage, general_client, type,accounts);

      garage = NULL;
      g_free(enterprise_legal_id);
      g_free(trade_name);
      g_free(comision);
      break;}
    }

  g_free(short_name);      
  g_free(stop);
  g_free(street);
  g_free(number);
  g_free(discount);
  g_free(email);
  g_free(fax);
  g_free(phone2);
  g_free(phone1);
  g_free(province);
  g_free(town);
  g_free(postal_code);
  g_free(floor);
  participant = NULL;

  if (result) c_util_open_info_window (_("The client has been modified correctly"));
  else c_util_open_error_window (_("The client couldn't be removed, check the data"));
  
}


static void repaint_accounts(gpointer value) {
    GtkWidget * combo;
    GList * actual_account = g_list_first(accounts);

    combo = glade_xml_get_widget (xml, "combo_cuentas_edita_clientes");
    c_util_clean_code_and_name_combo (combo);
    while (actual_account!=NULL) {
	gchar * complete_account_number=NULL;
	Account * account = (Account *) actual_account->data;

	complete_account_number=d_account_get_complete_number(account);
	c_util_set_code_and_name_on_combo (g_list_length(accounts),complete_account_number,GTK_COMBO(combo));
	g_free(complete_account_number);

	actual_account=g_list_next(actual_account);
    }
}

static void on_insert_checkbutton_toggled(GtkToggleButton *togglebutton,
							  gpointer user_data) {
    if (event_changed!=FALSE) {
	GtkWidget * open_insert = c_insert_account_open_window(repaint_accounts,C_ACCOUNT_FROM_EDIT_CLIENTS);
	GtkWidget *app1  = lookup_widget(GTK_WIDGET(open_insert), "app1");
	c_generic_repair_line_set_update(FALSE,ULONG_MAX,NULL);

	if (open_insert!=NULL) {
	    gtk_window_set_modal(GTK_WINDOW(open_insert),1);
	    gtk_window_set_position(GTK_WINDOW(open_insert),GTK_WIN_POS_CENTER);
	    gtk_window_set_transient_for(GTK_WINDOW(open_insert),
					 GTK_WINDOW(app1));
	    gtk_widget_show(GTK_WIDGET(open_insert));
	}
	set_account_checkbutton_state(FALSE);
    }
}

static void on_modify_checkbutton_toggled(GtkToggleButton *togglebutton,
							  gpointer user_data) {
    if (event_changed!=FALSE) {
	gchar * complete_account = g_strdup(gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(xml,"entry_combo_cuentas_edita_clientes"))));
	g_print("\nHOLA\n");
	if (complete_account!=NULL) {
	    if (strcmp(complete_account,"")) {
		GtkWidget * open_insert = c_insert_account_open_window(reload_accounts,C_ACCOUNT_FROM_EDIT_CLIENTS);
		GtkWidget *app1  = lookup_widget(GTK_WIDGET(open_insert), "app1");

		c_generic_repair_line_set_update(TRUE,d_participant_get_code (d_client_get_participant (general_client)),complete_account);

		if (open_insert!=NULL) {
		    gtk_window_set_modal(GTK_WINDOW(open_insert),1);
		    gtk_window_set_position(GTK_WINDOW(open_insert),GTK_WIN_POS_CENTER);
		    gtk_window_set_transient_for(GTK_WINDOW(open_insert),
						 GTK_WINDOW(app1));
		    gtk_widget_show(GTK_WIDGET(open_insert));
		}


	    } else {
		c_util_open_error_window (_("There is no account to modify"));
	    }
            g_free(complete_account);
	} else {
	  c_util_open_error_window (_("There is no account to modify"));
	}
    }
    event_changed = FALSE;
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(xml,"checkbutton_modifica")),FALSE);
    event_changed = TRUE;
}

static void set_account_checkbutton_state(gboolean new_state) {
    GtkWidget * account_checkbutton = glade_xml_get_widget(xml,"checkbutton_inserta_edita_clientes");
                                                               
    event_changed = FALSE;
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(account_checkbutton),new_state);

    event_changed = TRUE;
}

void c_edit_clients_add_account_client(Account * account) {
    accounts=g_list_append(accounts,account);
}

static void open_post_edit(GtkWidget *widget,gpointer user_data) {
    GtkWidget * open_edit = c_post_address_open_window(TRUE,show_post_addresses);
    GtkWidget *app1  = lookup_widget(GTK_WIDGET(open_edit), "app1");
    c_post_address_set_participant_code(d_participant_get_code (d_client_get_participant (general_client)));
    if (open_edit!=NULL) {
	gtk_window_set_modal(GTK_WINDOW(open_edit),1);
	gtk_window_set_position(GTK_WINDOW(open_edit),GTK_WIN_POS_CENTER);
	gtk_window_set_transient_for(GTK_WINDOW(open_edit),
				     GTK_WINDOW(app1));
	gtk_widget_show(GTK_WIDGET(open_edit));
    }

}

static void show_post_addresses(gpointer value) {
    GtkWidget * combo_post = glade_xml_get_widget(xml,"combo_direccion_edita");
    if (general_client!=NULL) {
	gulong party_code = d_participant_get_code (d_client_get_participant (general_client));
	gint index = 0;
	if (post_addresses!=NULL) {
	    m_post_address_clear_array (post_addresses);
	    post_addresses = NULL;
	}
	c_util_clean_code_and_name_combo (combo_post);
	post_addresses = m_post_address_find_by_participant_code (party_code);
	c_util_set_code_and_name_on_combo (C_ADDRES_NONE_CODE,_("<Address of the client>"),GTK_COMBO(combo_post));

	if (post_addresses!=NULL) {
	    while (post_addresses[index]!=NULL) {
		gchar * char_to_combo=m_post_address_show_on_one_line(post_addresses[index]);
	        c_util_set_code_and_name_on_combo (index+1,char_to_combo,GTK_COMBO(combo_post));
		index++;
	    }
	}
    }
}

static void on_edit_envelope_button_clicked(GtkWidget *widget,gpointer user_data) {
    PostAddress * post_address;
    gtk_window_set_modal(GTK_WINDOW(client_edition),0);
    post_address = get_address_to_post();
    p_envelope_wrap_print (post_address,set_modal_true,d_participant_get_nick (d_client_get_participant (general_client)));
}


static void set_modal_true() {
      gtk_window_set_modal(GTK_WINDOW(client_edition),1);
}

static PostAddress * get_address_to_post(void) {
    PostAddress * post_address=NULL;
    GtkWidget * combo = glade_xml_get_widget(xml,"combo_direccion_edita");
    gulong code_combo=ULONG_MAX;

    code_combo = c_util_get_code_from_combo (combo);
    if (code_combo!=ULONG_MAX) {
        post_address = d_post_address_duplicate (post_addresses[code_combo-1]);
    } else {
	
	gchar * street = d_participant_get_street (d_client_get_participant (general_client));
	gchar * floor = d_participant_get_floor (d_client_get_participant (general_client));
	gchar * number = d_participant_get_number (d_client_get_participant (general_client));
	gchar * town = d_participant_get_town (d_client_get_participant (general_client));
	gchar * province = d_participant_get_province (d_client_get_participant (general_client));
        post_address = d_post_address_create_empty ();
	d_post_address_set_postal_code (post_address,d_participant_get_postal_code (d_client_get_participant (general_client)));
        d_post_address_set_street (post_address,street);
        d_post_address_set_floor (post_address,floor);
        d_post_address_set_number (post_address,number);
        d_post_address_set_town (post_address,town);
        d_post_address_set_province (post_address,province);

        g_free(street);
        g_free(floor);
        g_free(province);
        g_free(number);
        g_free(town);
    }

    return post_address;
}

static void reload_accounts(gpointer value) {
    if (accounts!=NULL) {
	g_list_free(accounts);
        accounts=NULL;
    }
    accounts = m_account_find_by_participant_code (d_participant_get_code (d_client_get_participant (general_client)));
    repaint_accounts(NULL);
}

