
/* find_agent.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include <gnome.h>
#include <glade/glade.h>
#include <glib.h>

#include "find_agent.h"
#include "control_util.h"
#include "agent_model.h"
#include "repairs_control.h"
#include "participant.h"
#include "agent.h"

#define C_LEGAL_ID_COLUMN 0
#define C_NICK_COLUMN 1

static GtkWidget *find_agent=NULL;
static GladeXML *xml;
static Agent **agents = NULL;

static gchar url[]="/src/interface/glade/repairs_view.glade";
static gchar widget_name[]="seleccion_agente";

static void on_find_button_clicked(GtkButton *button,gpointer user_data);
static void on_accept_button_clicked(GtkButton *button,gpointer user_data);
static void on_cancel_button_clicked(GtkButton *button,gpointer user_data);
static void on_agent_selection_destroy(GtkWidget *widget,gpointer user_data);
static void on_name_entry_key_press_event(GtkWidget *widget, GdkEventKey *event, gpointer user_data);
static void load_agents_list(void);

GtkWidget *c_find_agent_open_window(GtkWidget *app1, gpointer data)
{
   gchar * directory = g_strconcat(AA_DATA_DIR,url,NULL);
   
   if (find_agent==NULL)
     {
	GtkWidget *window;
	GtkWidget *toolbar;

	xml = glade_xml_new (directory, widget_name);
	window = glade_xml_get_widget (xml, widget_name);
	
	toolbar = glade_xml_get_widget (xml, "find_toolbar");

	glade_xml_signal_connect (xml, "on_find_button_clicked",
				  GTK_SIGNAL_FUNC (on_find_button_clicked));
	glade_xml_signal_connect (xml, "on_accept_button_clicked",
				  GTK_SIGNAL_FUNC (on_accept_button_clicked));
	glade_xml_signal_connect (xml, "on_cancel_button_clicked",
				  GTK_SIGNAL_FUNC (on_cancel_button_clicked));
	glade_xml_signal_connect (xml, "on_seleccion_agente_destroy",
				  GTK_SIGNAL_FUNC (on_agent_selection_destroy));
	glade_xml_signal_connect (xml, "on_nombre_entry_key_press_event",
				  GTK_SIGNAL_FUNC (on_name_entry_key_press_event));

	gtk_toolbar_set_style(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_ICONS);
	find_agent = window;
	
     }

   g_free(directory);

   return find_agent;
   
}


static void load_agents_list(){

    gint index=0;
    gchar *empty_row[]={"",""};
    GtkWidget *list =glade_xml_get_widget (xml, "agents_clist");
    
    gtk_clist_clear(GTK_CLIST(list));

    if (agents!=NULL) {
        while (agents[index]!=NULL) {
	  Participant *participant = d_agent_get_participant (agents[index]);
	  gchar *enterprise_legal_id = d_participant_get_legal_id (participant);
	  gchar *nick = d_participant_get_nick (participant);	  
	  
            if (index==30) gtk_clist_freeze (GTK_CLIST(list));

            gtk_clist_append ((GtkCList *) list,empty_row);

            if (enterprise_legal_id!=NULL) {
                gtk_clist_set_text((GtkCList *) list, index, C_LEGAL_ID_COLUMN, enterprise_legal_id);
            }

            if (nick!=NULL) {
                gtk_clist_set_text((GtkCList *) list, index, C_NICK_COLUMN, nick);
            }


            index++;
	    g_free(enterprise_legal_id);
	    g_free(nick);	    

        }
    }
    gtk_clist_thaw ((GtkCList *) list);
}

static void on_find_button_clicked(GtkButton *button,gpointer user_data){

  gchar *name = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "nombre_entry")),0,-1);

  if (agents != NULL){   
    m_agent_clear_array (agents);
    agents = NULL;
  }

  if (!g_strcasecmp(name,""))
    agents = m_agent_find_by_name (NULL);
  else agents = m_agent_find_by_name (name);
  
  load_agents_list();

  g_free(name);

}

static void on_accept_button_clicked(GtkButton *button,gpointer user_data){

  GtkCList * list = GTK_CLIST(glade_xml_get_widget (xml, "agents_clist"));

  GArray *selected = c_util_get_selected_rows_from_clist (list);

  if (selected->len>0){
    gint row = g_array_index(selected,gint,0);

    c_repair_toggle_agent(TRUE);
    c_repair_insert(agents[row]);

  }
  else {
    c_repair_toggle_agent(FALSE);
    c_repair_insert(NULL);
  }

  g_array_free(selected, TRUE);    

  gtk_widget_destroy(find_agent);

}

static void on_cancel_button_clicked(GtkButton *button,gpointer user_data){

  c_repair_toggle_contrary_agent();
  
  gtk_widget_destroy(find_agent);
}

static void on_agent_selection_destroy(GtkWidget *widget,gpointer user_data){

  gint i=0;

  if (agents!=NULL)
    while (agents[i] != NULL){
      d_agent_clear (agents[i]);
      i++;
    }

  agents=NULL;
  find_agent = NULL;
  xml = NULL;
  
}

static void on_name_entry_key_press_event(GtkWidget *widget, GdkEventKey *event, gpointer user_data){

  GtkWidget *button =glade_xml_get_widget (xml, "find_button");

  if (event->keyval==GDK_Return)
    on_find_button_clicked(GTK_BUTTON(button), user_data);
}
