/* find_charge_invoice_control.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include <gnome.h>
#include <glade/glade.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <glib.h>
#include <stdlib.h>

#define C_INVOICE_NUMBER_COLUMN 0
#define C_TOTAL_COLUMN 1
#define C_STATE_COLUMN 2
#define C_PAY_WAY_COLUMN 3
#define C_ALL_SHOPS_CODE 0
#define C_ALL_SUPPLIERS 0
#define C_FIND_ALL_STATES 17171

#include "a-shortcut.h"
#include "support.h"
#include "control_util.h"
#include "parameter_manager.h"
#include <../interface/printing/listing-print-wrapper.h>
#include "find_charge_invoice_control.h"
#include "charge_invoice.h"
#include "charge_invoice_model.h"
#include "charge_invoice_control.h"
#include "shop_model.h"
#include "supplier_model.h"


static GtkWidget *find_charge_invoices=NULL;
static GladeXML *xml;
static gboolean first = FALSE;
static ChargeInvoice ** charge_invoices;
static gchar widget_name[]="vbox1_gestion_cargo";
static gchar url[]="/src/interface/glade/invoice_view.glade";

static void clean_screen(void);
static void fill_shop_combo(void);
static void fill_invoice_state_combo(void);
static void fill_invoice_type_combo(void);
static void fill_payments_combo(void);
static void fill_supplier_combo(void);
static void view_charge_invoice(void);
static void on_management_charge_invoice_press_key(GtkWidget *widget,
					   GdkEventKey *event,
					   gpointer user_data);
static void on_invoice_list_select_row (GtkCList        *list,
					  gint             row,
					  gint             column,
					  GdkEvent        *event,
					  gpointer         user_data);
//static gchar * get_code_from_shops(void);
static void on_delete_button_clicked(GtkButton *button, gpointer data);
static void on_accept_button_clicked(GtkButton *button, gpointer data);
static void on_print_button_clicked(GtkButton *button, gpointer data);
static void on_search_button_clicked(GtkButton *button, gpointer data);
static void on_charge_invoice_type_combo_entry_changed(GtkEntry *entry,gpointer user_data);
static void do_search(void);
static ChargeInvoiceId * obtain_charge_invoice_id_from_list(gint value,GtkCList * list);

GtkWidget *c_find_charge_invoice_open_window(GtkWidget *app1, GtkWidget *vbox)

{
    gchar * directory = g_strconcat(AA_DATA_DIR,url,NULL);
    first=TRUE;

    if (find_charge_invoices==NULL)
    {
	GtkWidget *dialog;
        gchar * s_name;
	xml = glade_xml_new (directory, widget_name);
	dialog = glade_xml_get_widget (xml, widget_name);

	gtk_box_pack_start (GTK_BOX (vbox), dialog, TRUE, TRUE, 0);

	glade_xml_signal_connect (xml, "on_button_borrar_gestion_cargo_clicked",
				  GTK_SIGNAL_FUNC (on_delete_button_clicked));
	glade_xml_signal_connect (xml, "on_button_aceptar_gestion_cargo_clicked",
				  GTK_SIGNAL_FUNC (on_accept_button_clicked));
	glade_xml_signal_connect (xml, "on_button_imprimir_gestion_cargo_clicked",
				  GTK_SIGNAL_FUNC (on_print_button_clicked));
	glade_xml_signal_connect (xml, "on_button_buscar_gestion_cargo_clicked",
				  GTK_SIGNAL_FUNC (on_search_button_clicked));
	glade_xml_signal_connect (xml, "on_lista_facturas_select_row",
				  GTK_SIGNAL_FUNC (on_invoice_list_select_row));
	glade_xml_signal_connect (xml, "on_press_key_gestion_cargo",
				  GTK_SIGNAL_FUNC (on_management_charge_invoice_press_key));
	glade_xml_signal_connect (xml, "on_combo_entry_tipo_factura_gestion_cargo_changed",
				  GTK_SIGNAL_FUNC (on_charge_invoice_type_combo_entry_changed));

	fill_invoice_type_combo();
	fill_invoice_state_combo();
	fill_shop_combo();
	fill_supplier_combo();
	fill_payments_combo();

	s_name = u_parameter_manager_get_parameter ("tienda");
	if (s_name!=NULL) {
	    gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget (xml, "combo_entry_tienda_gestion_cargo")), s_name);
	}
	g_free(s_name);

#ifdef IS_SHOP
	gtk_widget_set_sensitive(glade_xml_get_widget (xml, "combo_tienda_gestion_cargo"),FALSE);
#endif

	find_charge_invoices=dialog;

    } else {
	if (charge_invoices!=NULL) {
	    if (charge_invoices[0]!=NULL) {
		m_charge_invoice_clear_array (charge_invoices);
		charge_invoices=NULL;
		clean_screen();
	    }
	} 
    }

    first=FALSE;

    return find_charge_invoices;

}





static void fill_charge_invoice_list_on_clist(GtkCList * list,ChargeInvoice ** charge_invoice_array) {
    gchar * content=NULL;
    gint index=0;
    gchar *empty_row[]={"","","",""};

    if (charge_invoice_array!=NULL) {
	while (charge_invoice_array[index]!=NULL) {
	    Payment * payment;
	    gchar* temp_invoice=NULL;
	    if (index==30) gtk_clist_freeze ((GtkCList *) list);
	    gtk_clist_append ((GtkCList *) list,empty_row);
            temp_invoice = d_charge_invoice_get_invoice_number (charge_invoice_array[index]);
	    if ((temp_invoice)!=NULL) {
		gtk_clist_set_text((GtkCList *) list, index, C_INVOICE_NUMBER_COLUMN, temp_invoice);
		g_free(temp_invoice);
	    }
	    if ((d_charge_invoice_get_total (charge_invoice_array[index])) != G_MINDOUBLE) {
		content = g_strdup_printf("%.2f",d_charge_invoice_get_total (charge_invoice_array[index]));
		gtk_clist_set_text((GtkCList *) list, index, C_TOTAL_COLUMN, content);
		g_free(content);
	    }
	    payment = d_charge_invoice_get_payment (charge_invoice_array[index]);
	    if (payment!=NULL) {
		if (d_payment_get_is_transfer (payment)) {
		    content = g_strdup(D_PAYMENT_MESSAGE_TRANSFER);
		} else if (d_payment_get_is_credit_card (payment)) {
		    content = g_strdup(D_PAYMENT_MESSAGE_CREDIT_CARD);
		} else if (d_payment_get_is_cheque (payment)) {
		    content = g_strdup(D_PAYMENT_MESSAGE_CHEQUE);
		} else if (d_payment_get_is_direct_debit (payment)) {
		    content = g_strdup(D_PAYMENT_MESSAGE_DIRECT_DEBIT);
		} else {
		    content = g_strdup(D_PAYMENT_MESSAGE_IN_CASH);
		}
		gtk_clist_set_text((GtkCList *) list, index, C_PAY_WAY_COLUMN, content);
		g_free(content);
	    }
	    if (d_charge_invoice_get_vat (charge_invoice_array[index])!=FALSE) {
		if (d_charge_invoice_get_state (charge_invoice_array[index])>-1) {
		    if (d_charge_invoice_get_state (charge_invoice_array[index])==D_CHARGE_INVOICE_NOT_PAID) {
			content = g_strdup(D_CHARGE_INVOICE_MESSAGE_NOT_PAID);
		    } else if (d_charge_invoice_get_state (charge_invoice_array[index])==D_CHARGE_INVOICE_PAID) {
			content = g_strdup(D_CHARGE_INVOICE_MESSAGE_PAID);
		    }
		    gtk_clist_set_text((GtkCList *) list, index, C_STATE_COLUMN, content);
		    g_free(content);
		}
	    } else {
		content = g_strdup(D_CHARGE_INVOICE_MESSAGE_PAID);
		gtk_clist_set_text((GtkCList *) list, index, C_STATE_COLUMN, content);
		g_free(content);
	    }
	    index++;
	}
    }
    gtk_clist_thaw ((GtkCList *) list);

}


static void fill_invoice_type_combo() {
    GtkWidget * combo=glade_xml_get_widget(xml,"combo_tipo_factura_gestion_cargo");
    gchar * temp;

    temp = g_strdup(D_CHARGE_INVOICE_MESSAGE);
    c_util_set_code_and_name_on_combo (D_CHARGE_INVOICE_CODE,temp,GTK_COMBO(combo));
    g_free(temp);
    temp = g_strdup(D_CHARGE_INVOICE_NOTE_MESSAGE);
    c_util_set_code_and_name_on_combo (D_CHARGE_INVOICE_NOTE_CODE,temp,GTK_COMBO(combo));
    g_free(temp);
//    clean_screen();

}

static void fill_invoice_state_combo() {
    GtkWidget * combo=glade_xml_get_widget(xml,"combo_estado_factura_gestion_cargo");
    gchar * temp;

    temp = g_strdup(N_("All the states"));
    c_util_set_code_and_name_on_combo (C_FIND_ALL_STATES,_(temp),GTK_COMBO(combo));
    g_free(temp);
    temp = g_strdup(D_CHARGE_INVOICE_MESSAGE_NOT_PAID);
    c_util_set_code_and_name_on_combo (D_CHARGE_INVOICE_NOT_PAID,temp,GTK_COMBO(combo));
    g_free(temp);
    temp = g_strdup(D_CHARGE_INVOICE_MESSAGE_PAID);
    c_util_set_code_and_name_on_combo (D_CHARGE_INVOICE_PAID,temp,GTK_COMBO(combo));
    g_free(temp);
//    clean_screen();

}

static void on_charge_invoice_type_combo_entry_changed(GtkEntry *entry,
								 gpointer user_data) {
    GtkWidget * label = glade_xml_get_widget(xml,"label_numero_gestion_cargo");
    gchar * temp =g_strdup(gtk_entry_get_text(entry));
    GtkWidget * state_combo = glade_xml_get_widget(xml,"combo_estado_factura_gestion_cargo");
    GtkWidget * expiry_dateedit = glade_xml_get_widget(xml,"dateedit_vencimiento_gestion_cargo");
    if (!strcmp(temp,D_CHARGE_INVOICE_MESSAGE)) {
      gtk_label_set_text(GTK_LABEL(label),_("Invoice number"));
        if (!GTK_WIDGET_SENSITIVE(state_combo)) gtk_widget_set_sensitive(state_combo,TRUE);
        if (!GTK_WIDGET_SENSITIVE(expiry_dateedit)) gtk_widget_set_sensitive(expiry_dateedit,TRUE);
    } else {
        gtk_label_set_text(GTK_LABEL(label),_("Payment note"));
        if GTK_WIDGET_SENSITIVE(state_combo) gtk_widget_set_sensitive(state_combo,FALSE);
        if (!GTK_WIDGET_SENSITIVE(expiry_dateedit)) gtk_widget_set_sensitive(expiry_dateedit,TRUE);
    }
    do_search();

}

static void fill_shop_combo() {
    GtkWidget * combo;
    Shop ** shops;
    gchar * temp;

    combo = glade_xml_get_widget (xml, "combo_tienda_gestion_cargo");

    temp = g_strdup(N_("All the shops"));
    c_util_set_code_and_name_on_combo (C_ALL_SHOPS_CODE,_(temp),GTK_COMBO(combo));
    shops = m_shop_find_all ();
    g_free(temp);

    c_util_fill_combo_with_shops (combo,shops);

}
static void fill_supplier_combo() {
    GtkWidget * combo;
    Supplier ** suppliers;
    gchar * temp;

    combo = glade_xml_get_widget (xml, "combo_proveedor_gestion_cargo");

    temp = g_strdup(N_("All the suppliers"));
    c_util_set_code_and_name_on_combo (C_ALL_SUPPLIERS,_(temp),GTK_COMBO(combo));
    suppliers = m_supplier_find_all ();
    g_free(temp);

    c_util_fill_combo_with_suppliers (combo,suppliers);

}

static void on_delete_button_clicked(GtkButton *button, gpointer data) {
    clean_screen();
}

static void on_accept_button_clicked(GtkButton *button, gpointer data) {
    view_charge_invoice();
}

static void on_print_button_clicked(GtkButton *button, gpointer data) {
    if (charge_invoices != NULL) {
	Listing list;
        Listing * listing=(Listing *) g_malloc0(sizeof(Listing));
        gulong    rows = 0;
	gchar * temp = NULL;

	temp = g_strdup(gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(xml,"entry_proveedores_busqueda"))));
	list.content_title = d_charge_invoice_get_content_title ();
	list.listing_title = d_charge_invoice_get_listing_title (temp);
	list.content       = d_charge_d_invoice_array_to_content (charge_invoices, &rows);
	list.width_of_cols = d_charge_invoice_get_listing_columns_width ();
	list.alignment_of_cols = d_charge_invoice_get_listing_columns_alignment ();
	list.n_rows        = rows;
	list.n_columns     = d_charge_invoice_get_content_size ();
        g_free(temp);
        // Fixme!!!
        // Hardcoded -> get this from a combo
	list.orientation   = 4;
	list.type_of_paper = 2;
	*listing = list;
        p_listing_wrap_print (listing);
    } else {
	c_util_open_error_window (_("Search the invoices you want to print"));
    }
}

static void on_search_button_clicked(GtkButton *button, gpointer data) {
    do_search();
}


static void clean_screen() {

    GtkWidget *vbox;

    vbox = GTK_WIDGET(glade_xml_get_widget(xml, widget_name));

    gtk_container_foreach(GTK_CONTAINER(vbox),&c_util_clean_widget,NULL);


}

static void on_management_charge_invoice_press_key(GtkWidget *widget,
					   GdkEventKey *event,
					   gpointer user_data) {
    if (event->keyval==GDK_Return)
	do_search();
                                           
}

static void on_invoice_list_select_row (GtkCList        *list,
					  gint             row,
					  gint             column,
					  GdkEvent        *event,
					  gpointer         user_data) {

    if (event!=NULL) {
	if (event->type == GDK_2BUTTON_PRESS)
	{
	    view_charge_invoice();
	}
    }
}

static void do_search() {
    if (first != TRUE) {

	GtkWidget * wid =NULL;
	gchar * temp;
	gulong invoice_state;
	ConditionFindChargeInvoice * con;
	time_t initial_date;
	time_t expiry_date;
	gint no_error = 1;
	GtkWidget * clist = glade_xml_get_widget(xml,"lista_facturas_gestion_cargo");
	GtkWidget * window;
	GdkCursor * cursor;

	window = lookup_widget(GTK_WIDGET(clist),"app1");
	cursor = gdk_cursor_new(GDK_WATCH);
	gdk_window_set_cursor(GTK_WIDGET(window)->window, cursor);
	gdk_cursor_destroy(cursor);

	while (gtk_events_pending())
	    gtk_main_iteration();


	con = (ConditionFindChargeInvoice * ) g_malloc0(sizeof(ConditionFindChargeInvoice));

	wid = glade_xml_get_widget(xml,"combo_estado_factura_gestion_cargo");

	invoice_state = c_util_get_code_from_combo (wid);
	if (invoice_state!=C_FIND_ALL_STATES) {
	    con -> invoice_state = invoice_state;
	} else {
	    con -> invoice_state = -1;
	}
	wid = glade_xml_get_widget(xml,"dateedit_vencimiento_gestion_cargo");
	initial_date = gnome_date_edit_get_date(GNOME_DATE_EDIT(wid));

	if (initial_date>-1) {
	    con -> initial_expire_date = (time_t *) g_malloc0(sizeof(time_t));
	    *(con -> initial_expire_date) = initial_date;
	} else {
	    con->initial_expire_date=NULL;
	}

	wid = glade_xml_get_widget(xml,"dateedit_hasta_gestion_cargo");
	expiry_date = gnome_date_edit_get_date(GNOME_DATE_EDIT(wid));
	if (expiry_date>-1) {
	    con -> ending_expire_date = (time_t *) g_malloc0(sizeof(time_t));
	    *(con -> ending_expire_date) = expiry_date;
	} else {
	    con->ending_expire_date=NULL;
	}

	wid = glade_xml_get_widget(xml,"combo_tipo_factura_gestion_cargo");
	con->type= c_util_get_code_from_combo (wid);


	wid = glade_xml_get_widget(xml,"combo_tienda_gestion_cargo");
	con->shop_code= c_util_get_code_from_combo (wid);

	wid = glade_xml_get_widget(xml,"combo_pagos_gestion_cargo");
	con->payment_type= c_util_get_code_from_combo (wid);

	wid = glade_xml_get_widget(xml,"spinbutton_cantidad_gestion_cargo");
	gtk_spin_button_update(GTK_SPIN_BUTTON(wid));
	con->amount = gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(wid));


	wid = glade_xml_get_widget(xml,"combo_proveedor_gestion_cargo");
	con ->supplier_code = c_util_get_code_from_combo (wid);

	wid = glade_xml_get_widget(xml,"entry_numero_factura_gestion_cargo");
	temp = g_strdup(gtk_entry_get_text(GTK_ENTRY(wid)));
	con->invoice_number = g_strdup(temp);
	g_free(temp);

	if (charge_invoices!=NULL) {
	    if (charge_invoices[0]!=NULL) {
		m_charge_invoice_clear_array (charge_invoices);
		charge_invoices =NULL;
		gtk_clist_clear(GTK_CLIST(clist));
	    }
	}
	if (con->type == D_CHARGE_INVOICE_CODE) {
	    con->vat = TRUE;
	} else {
	    con->vat = FALSE;
	}

	if (no_error == 1) {
	    charge_invoices = m_charge_invoice_find_by_condition (con);
	    fill_charge_invoice_list_on_clist(GTK_CLIST(clist),charge_invoices);
	}

	cursor = gdk_cursor_new(GDK_LEFT_PTR);
	gdk_window_set_cursor(GTK_WIDGET(window)->window, cursor);
	gdk_cursor_destroy(cursor);

	g_free(con);
    }
}

static ChargeInvoiceId * obtain_charge_invoice_id_from_list(gint value,GtkCList * list) {
    ChargeInvoiceId * charge_invoiceId = (ChargeInvoiceId *) g_malloc0(sizeof(ChargeInvoiceId));

    
    
    charge_invoiceId -> invoice_number = g_strdup_printf("%lu",d_charge_invoice_get_code (charge_invoices[value]));
    charge_invoiceId -> supplier_code =  g_strdup_printf("%lu",
						 d_participant_get_code (d_supplier_get_participant (d_charge_invoice_get_supplier (charge_invoices[value])))
						);

    return charge_invoiceId;

}



static void view_charge_invoice() {
    GArray * selected;
    gint value;
    GtkCList * list = GTK_CLIST(glade_xml_get_widget (xml, "lista_facturas_gestion_cargo"));
    GtkWidget * combo_tipo_factura = glade_xml_get_widget (xml, "combo_tipo_factura_gestion_cargo");
    gulong codeCombo = c_util_get_code_from_combo (combo_tipo_factura);

    selected = c_util_get_selected_rows_from_clist (list);

    if (selected->len==1) {
	ChargeInvoiceId * charge_invoiceId = (ChargeInvoiceId *)g_malloc0(sizeof(ChargeInvoiceId));

	value = g_array_index(selected,gint,0);


	charge_invoiceId = obtain_charge_invoice_id_from_list(value,list);

	g_array_free(selected,TRUE);

	i_shortcut_change_dialog (GTK_WIDGET(list),1,3);

	if (codeCombo==D_CHARGE_INVOICE_CODE) {
	    load_charge_invoice(charge_invoiceId,TRUE);
	} else {
	    load_charge_invoice(charge_invoiceId,FALSE);
	}
    } else {
	c_util_open_error_window (_("Select an invoice"));
    }

}

static void fill_payments_combo(void) {
    GtkWidget * combo=glade_xml_get_widget(xml,"combo_pagos_gestion_cargo");
    gchar * temp;

    temp = g_strdup(N_("All the types of payments"));
    c_util_set_code_and_name_on_combo (ULONG_MAX,temp,GTK_COMBO(combo));
    g_free(temp);
    temp = g_strdup(D_PAYMENT_MESSAGE_TRANSFER);
    c_util_set_code_and_name_on_combo (D_PAYMENT_TRANSFER_CODE,temp,GTK_COMBO(combo));
    g_free(temp);
    temp = g_strdup(D_PAYMENT_MESSAGE_IN_CASH);
    c_util_set_code_and_name_on_combo (D_PAYMENT_IN_CASH,temp,GTK_COMBO(combo));
    g_free(temp);
    temp = g_strdup(D_PAYMENT_MESSAGE_CHEQUE);
    c_util_set_code_and_name_on_combo (D_PAYMENT_CHEQUE,temp,GTK_COMBO(combo));
    g_free(temp);
    temp = g_strdup(D_PAYMENT_MESSAGE_DIRECT_DEBIT);
    c_util_set_code_and_name_on_combo (D_PAYMENT_DIRECT_DEBIT_CODE,temp,GTK_COMBO(combo));
    g_free(temp);
    temp = g_strdup(D_PAYMENT_MESSAGE_CREDIT_CARD);
    c_util_set_code_and_name_on_combo (D_PAYMENT_CREDIT_CARD,temp,GTK_COMBO(combo));
    g_free(temp);


}
