/* find_delivery_payment_control.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include <gnome.h>
#include <glade/glade.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <glib.h>
#include <stdlib.h>

#define C_PARTICIPANT_CODE_COLUMN 0
#define C_CODE_COLUMN 1
#define C_NICK_COLUMN 2
#define C_ALL_SHOPS_OF_ENTERPRISE 0
#define C_ALL_ENTERPRISES 0
#define C_DELIVERY_PAYMENT_COLUMN 0
#define C_LICENSE_NUMBER_COLUMN 1
#define C_TOTAL_COLUMN 2
#define C_SHOP_COLUMN 3
#define C_ENTERPRISE_COLUMN 4
#define C_DATE_COLUMN 5

#include "a-shortcut.h"
#include "support.h"
#include "control_util.h"

#include "parameter_manager.h"

#include "shop_model.h"
#include "enterprise_model.h"
#include "client_model.h"

#include "delivery_payment_model.h"
//#include "invoice_model.h"
#include "enterprise.h"
#include "shop.h"
#include "find_delivery_payment_control.h"
#include "delivery_payment.h"
#include "client.h"
#include "invoice_control.h"
#include "../interface/printing/listing-print-wrapper.h"

static GtkWidget *find_delivery_payments=NULL;
static GladeXML *xml;
static DeliveryPayment ** delivery_payments;
static gboolean search_garage=FALSE;
static gchar widget_name[]="vbox_busca_albaran";
static gchar url[]="/src/interface/glade/delivery_payment_view.glade";
static gboolean do_listing;

static void clean_screen(void);
static void on_search_button_clicked(GtkButton *button, gpointer data);
static void on_invoice_button_clicked(GtkButton *button, gpointer data);
static void on_delete_button_clicked(GtkButton *button, gpointer data);
static void on_print_listing_button_clicked(GtkButton *button, gpointer data);

static void fill_shop_combo(void);
static void fill_enterprise_combo(void);
static void on_enterprise_entry_changed(GtkEntry *entry,
					      gpointer user_data);

static void fill_list_with_clients(GtkCList * list,Client ** clients);

static void fill_list_with_delivery_payments(GtkCList * list,DeliveryPayment ** deliveryPayments);
static void on_search_nat_pressed(GtkButton *button,gpointer user_data);
static void on_search_cif_pressed(GtkButton *button,gpointer user_data);
static void on_search_one_pressed(GtkButton *button,gpointer user_data);
static void on_search_with_insurance_pressed(GtkButton *button,gpointer user_data);
static void on_search_without_insurance_pressed(GtkButton *button,gpointer user_data);
static void on_search_all_pressed(GtkButton *button,gpointer user_data);
static void on_search_listing_button_pressed(GtkButton *button,gpointer user_data);
static void do_clients_search(void);
static void do_delivery_payment_search(GtkCList * list,gint row);
static void clean_combos(void);
static void clean_nat(void);
static void clean_cif(void);
static void do_print(void);
static void on_search_invoice_press_key(GtkWidget *widget,
                         GdkEventKey *event,
                         gpointer user_data);
static void do_listing_search(void);
static void on_clients_list_select_row (GtkCList        *list,
					  gint             row,
					  gint             column,
					  GdkEvent        *event,
					  gpointer         user_data);
static void clean_enterprise_combo(void);
static void clean_shop_combo(void);
static gchar * get_code_from_shops(void);
static gchar * get_code_from_enterprises(void);
static void do_invoice(void);
DeliveryPaymentId ** obtain_delivery_payments_ids(GArray * selected,DeliveryPayment ** delivery_payments);
GtkWidget *c_find_delivery_payment_open_window(GtkWidget *app1, GtkWidget *vbox)

{
   
    gchar * directory = g_strconcat(AA_DATA_DIR,url,NULL);
    time_t fecha_ven = time(NULL) + 86400;
    GtkWidget * dateedit_fecha= NULL;
    time_t desde = time(NULL) - 86400;
    GtkWidget * dateedit_initial_date= NULL;
   
    if (find_delivery_payments==NULL)
    {
	GtkWidget *dialog;
	GtkWidget * check1;
	GtkWidget * check2;
	GtkWidget * check3;
	GtkWidget * list;
	GtkWidget * print_button;
	GSList * group;
	gchar * e_name =NULL;
	gchar * s_name =NULL;

	xml = glade_xml_new (directory, widget_name);
	dialog = glade_xml_get_widget (xml, widget_name);

	gtk_box_pack_start (GTK_BOX (vbox), dialog, TRUE, TRUE, 0);

	glade_xml_signal_connect (xml, "on_buscar_busqueda_facturas_clicked",
				  GTK_SIGNAL_FUNC (on_search_button_clicked));
	glade_xml_signal_connect (xml, "on_facturar_busqueda_facturas_clicked",
				  GTK_SIGNAL_FUNC (on_invoice_button_clicked));
	glade_xml_signal_connect (xml, "on_borrar_busqueda_facturas_clicked",
				  GTK_SIGNAL_FUNC (on_delete_button_clicked));
	glade_xml_signal_connect (xml, "on_entry_empresa_busqueda_facturas_changed",
				  GTK_SIGNAL_FUNC (on_enterprise_entry_changed));
	glade_xml_signal_connect (xml, "on_DNI_busqueda_pressed",
				  GTK_SIGNAL_FUNC (on_search_nat_pressed));
	glade_xml_signal_connect (xml, "on_cif_busqueda_pressed",
				  GTK_SIGNAL_FUNC (on_search_cif_pressed));
	glade_xml_signal_connect (xml, "on_uno_busqueda_pressed",
				  GTK_SIGNAL_FUNC (on_search_one_pressed));
	glade_xml_signal_connect (xml, "on_todos_busqueda_pressed",
				  GTK_SIGNAL_FUNC (on_search_all_pressed));
	glade_xml_signal_connect (xml, "on_listado_busqueda_pressed",
				  GTK_SIGNAL_FUNC (on_search_listing_button_pressed));
	glade_xml_signal_connect (xml, "on_press_key_busqueda_facturas",
				  GTK_SIGNAL_FUNC (on_search_invoice_press_key));
	glade_xml_signal_connect (xml, "on_lista_clientes_select_row",
				  GTK_SIGNAL_FUNC (on_clients_list_select_row));
	glade_xml_signal_connect (xml, "on_sin_seguro_busqueda_pressed",
				  GTK_SIGNAL_FUNC (on_search_without_insurance_pressed));
	glade_xml_signal_connect (xml, "on_con_seguro_busqueda_pressed",
				  GTK_SIGNAL_FUNC (on_search_with_insurance_pressed));
	glade_xml_signal_connect (xml, "on_button_imprimir_listado_clicked",
				  GTK_SIGNAL_FUNC (on_print_listing_button_clicked));


	fill_enterprise_combo();
	e_name = u_parameter_manager_get_parameter ("empresa");
	if (e_name!=NULL) {
	    gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget (xml, "entry_empresa_busqueda_facturas")), e_name);
	}
	g_free(e_name);

	s_name = u_parameter_manager_get_parameter ("tienda");
	if (s_name!=NULL) {
	    gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget (xml, "combo_entry_filtro_tiendas")), s_name);
	}
	g_free(s_name);



#ifdef IS_SHOP
	gtk_widget_set_sensitive(glade_xml_get_widget (xml, "combo_filtro_tiendas_facturas"),FALSE);
	gtk_widget_set_sensitive(glade_xml_get_widget (xml, "combo_empresa_busqueda_facturas"),FALSE);
#endif
	check1 = glade_xml_get_widget(xml, "radiobutton_DNI_busqueda_facturas");
	check2 = glade_xml_get_widget(xml, "radiobutton_CIF_busqueda_facturas");

	group = gtk_radio_button_group(GTK_RADIO_BUTTON(check1));
	gtk_radio_button_set_group(GTK_RADIO_BUTTON(check2),group);

	check1 = glade_xml_get_widget(xml, "radiobutton_persona_busqueda_facturas");
	check2 = glade_xml_get_widget(xml, "radiobutton_empresa_busqueda_facturas");
	check3 = glade_xml_get_widget(xml, "radiobutton_ambos_busqueda_facturas");

	group = gtk_radio_button_group(GTK_RADIO_BUTTON(check2));
	gtk_radio_button_set_group(GTK_RADIO_BUTTON(check1),group);
	group = gtk_radio_button_group(GTK_RADIO_BUTTON(check1));
	gtk_radio_button_set_group(GTK_RADIO_BUTTON(check3),group);

	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check3),TRUE);


	check1 = NULL;
	check2 = NULL;
	check3 = NULL;

	list = glade_xml_get_widget(xml, "lista_clientes");
	gtk_clist_set_column_visibility(GTK_CLIST(list),C_PARTICIPANT_CODE_COLUMN,FALSE);

	find_delivery_payments=dialog;
	print_button = glade_xml_get_widget(xml, "button_imprimir_listado");
	if (GTK_WIDGET_VISIBLE(print_button)) {
	    gtk_widget_hide(print_button);
	}

    } else if (delivery_payments!=NULL) {
	if (delivery_payments[0]!=NULL) {
	    m_delivery_payment_clear_array (delivery_payments);
	    delivery_payments=NULL;
	    clean_screen();
	}

    }

    dateedit_fecha = glade_xml_get_widget (xml, "dateedit_hasta");
    gnome_date_edit_set_time(GNOME_DATE_EDIT(dateedit_fecha), fecha_ven);
    dateedit_initial_date = glade_xml_get_widget (xml, "dateedit_desde");
    gnome_date_edit_set_time(GNOME_DATE_EDIT(dateedit_initial_date), desde);

  return find_delivery_payments;
  
}

static void do_clients_search() {
    GtkWidget * list;
    GtkWidget * list2;
    GtkWidget * check1;
    GtkWidget * check2;
    GtkWidget * check_todos;
    GtkWidget * modal_window;
    ConditionFindClients * con;
    gchar * content;
    gchar * cifAux;
    gchar * numberOfCif;
    gchar ** stop;
    Client ** list_of_clients;
    gint correctCode = 0;
    gint isDNI = 0;
    gint isOne = 1;
    gint ret;
    GdkCursor * cursor;
    GtkWidget * window;
    GtkWidget * dateedit_desde = glade_xml_get_widget(xml,"dateedit_desde");
    GtkWidget * dateedit_hasta = glade_xml_get_widget(xml,"dateedit_hasta");
    time_t initial_date = gnome_date_edit_get_date(GNOME_DATE_EDIT(dateedit_desde));
    time_t ending_date = gnome_date_edit_get_date(GNOME_DATE_EDIT(dateedit_hasta));

    list = glade_xml_get_widget(xml,"lista_clientes");

    window = lookup_widget(GTK_WIDGET(list),"app1");
    cursor = gdk_cursor_new(GDK_WATCH);
    gdk_window_set_cursor(GTK_WIDGET(window)->window, cursor);
    gdk_cursor_destroy(cursor);

    while (gtk_events_pending())
	gtk_main_iteration();

    stop = (gchar **) g_malloc0(sizeof(gchar *));;
    list2 = glade_xml_get_widget(xml,"lista_albaranes");
    check2 = glade_xml_get_widget(xml, "radiobutton_uno_busqueda_facturas");
    check_todos = glade_xml_get_widget(xml, "radiobutton_todos_busqueda_facturas");
    check1 = glade_xml_get_widget(xml, "radiobutton_DNI_busqueda_facturas");

    con = (ConditionFindClients *) g_malloc0(sizeof(ConditionFindClients));

    con->enterprise_code=get_code_from_enterprises();
    con->shop_code=get_code_from_shops();

    con->initial_date = g_strdup(ctime(&initial_date));
    con->ending_date = g_strdup(ctime(&ending_date));

    if ((gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check2))==TRUE)) {
        GtkWidget * check_insurance=glade_xml_get_widget(xml, "radiobutton_con_seguro_busqueda_facturas");

	con->insurance = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_insurance));
	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check1))==TRUE) {
            /* IF WE SEARCH BY DNI*/
	    content=g_strdup(gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "entry_dni_busqueda_facturas")),0,-1));
	    isDNI=1;
	    if (!strcmp(content,"")) {
                correctCode=1;
		con->code=NULL;
	    } else {
		correctCode = u_person_legal_id_is_correct (content);
		if (correctCode==1) {
		    con->code   = g_strdup(content);

		}
	    }
            con->search_type=M_CLIENT_PERSON_SEARCH;
	    g_free(content);

	} else {
            /* IF WE SEARCH BY CIF*/
	    content=g_strdup(gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "entry_cif_busqueda_facturas")),0,-1));
	    isDNI=0;
	    if (!strcmp(content,"")) {
                correctCode=1;
		con->code=NULL;
	    } else {
		cifAux = (gchar *) g_malloc0(sizeof(gchar)*8);
		numberOfCif = (gchar *) g_malloc0(sizeof(gchar)*9);
		numberOfCif = g_strdup(&content[1]);
		cifAux = strncpy(cifAux,numberOfCif,8);
		correctCode = u_enterprise_legal_id_is_correct (strtoul(cifAux,stop,10));
		g_free(numberOfCif);
		g_free(cifAux);
		g_free(stop);
		if (correctCode==1) {
		    con->code = g_strdup(content);
		}
	    }
            con->search_type=M_CLIENT_ENTERPRISE_SEARCH;
	    g_free(content);
	}
	content=g_strdup(gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "entry_nombre_busqueda_facturas")),0,-1));
        con->complete_name=g_strdup(content);
	g_free(content);

        

	/* IF WE SEARCH SOME CLIENTS */
    } else if ((gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_todos))==TRUE)) {
        GtkWidget * check_personas=glade_xml_get_widget(xml, "radiobutton_persona_busqueda_facturas");
	GtkWidget * check_empresas=glade_xml_get_widget(xml, "radiobutton_empresa_busqueda_facturas");
        GtkWidget * check_insurance=glade_xml_get_widget(xml, "radiobutton_con_seguro_busqueda_facturas");
	gboolean personas = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_personas));
	gboolean empresas = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_empresas));

	
	if ((personas==TRUE) && (empresas==FALSE)) {
	    con->search_type = M_CLIENT_PERSON_SEARCH;
	} else if ((empresas==TRUE) && (personas==FALSE)){
            con->search_type = M_CLIENT_ENTERPRISE_SEARCH;
	} else  {
            con->search_type = C_NO_SPECIFIED_SEARCH;
	}
	con->insurance = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_insurance));
	con->code = NULL;
	con->complete_name = NULL;
        correctCode = 1;
        isOne = 0;
    } else {
	correctCode=0;
    }

    if (delivery_payments!=NULL) {
	if (delivery_payments[0]!=NULL) {
	    m_delivery_payment_clear_array (delivery_payments);
	    delivery_payments=NULL;
	}
    }
    gtk_clist_clear(GTK_CLIST(list));
    gtk_clist_clear(GTK_CLIST(list2));

    if (correctCode==1)  {

	list_of_clients = m_client_find_by_delivery_payment_condition (con);


        fill_list_with_clients(GTK_CLIST(list),list_of_clients);

	if (list_of_clients!=NULL) {
	    if (list_of_clients[1]==NULL) {
		do_delivery_payment_search(GTK_CLIST(list),0);
	    }
	}
	m_client_clear_array (list_of_clients);
        list_of_clients=NULL;



    } else {
        list_of_clients = NULL;
	if ((isDNI) && (isOne)) {
	    modal_window = glade_xml_get_widget (xml, "messageDNI");
	    modal_window = gnome_message_box_new (_("Not a valid legal id. \nInsert a valid legal id"),
						  GNOME_MESSAGE_BOX_ERROR,
						  _("Ok"), NULL);
	} else if (isOne) {
	    modal_window = glade_xml_get_widget (xml, "messageCIF");
	    modal_window = gnome_message_box_new (_("Not a valid tax code. \nInsert a valid tax code"),
						  GNOME_MESSAGE_BOX_ERROR,
						  _("Ok"), NULL);

	} else {
            modal_window = NULL;
	}

	if (modal_window !=NULL) {
	    gtk_window_set_modal(GTK_WINDOW(modal_window),1);
	    gtk_window_set_position(GTK_WINDOW(modal_window),GTK_WIN_POS_CENTER);
	    gtk_window_set_transient_for(GTK_WINDOW(modal_window),
					 GTK_WINDOW(lookup_widget((modal_window), "app1")));
	    ret = gnome_dialog_run_and_close(GNOME_DIALOG(modal_window));
	    switch (ret) {
	    default:
              break;
	    }

	}
    }
    if (con!=NULL) {
	if (con->code!=NULL) {
	    g_free(con->code);
            con->code=NULL;
	}
	if (con->complete_name!=NULL) {
	    g_free(con->complete_name);
            con->complete_name=NULL;
	}
	if (con->enterprise_code!=NULL) {
	    g_free(con->enterprise_code);
	    con->enterprise_code=NULL;
	}
	if (con->shop_code!=NULL) {
	    g_free(con->shop_code);
            con->shop_code=NULL;
	}
	if (con->initial_date!=NULL) {
	    g_free(con->initial_date);
            con->initial_date=NULL;
	}
	if (con->ending_date!=NULL) {
	    g_free(con->ending_date);
            con->ending_date=NULL;
	}	
	g_free(con);
        con=NULL;
    }


    cursor = gdk_cursor_new(GDK_LEFT_PTR);
    gdk_window_set_cursor(GTK_WIDGET(window)->window, cursor);
    gdk_cursor_destroy(cursor);

}



static void fill_list_with_clients(GtkCList * list,Client ** clients) {

    gchar * content;
    gint index=0;
    gchar *empty_row[]={"","",""};

    if (clients!=NULL) {
	while (clients[index]!=NULL) {

	    if (index==30) gtk_clist_freeze ((GtkCList *) list);
	    gtk_clist_append ((GtkCList *) list,empty_row);

	    if (d_participant_get_code (d_client_get_participant (clients[index]))!=ULONG_MAX) {
		content = g_strdup_printf("%lu",d_participant_get_code (d_client_get_participant (clients[index])));
		gtk_clist_set_text((GtkCList *) list, index, C_PARTICIPANT_CODE_COLUMN, content);
		g_free(content);
	    }

	    if (d_participant_get_legal_id (d_client_get_participant (clients[index]))!=NULL) {
		content = d_participant_get_legal_id (d_client_get_participant (clients[index]));
		gtk_clist_set_text((GtkCList *) list, index, C_CODE_COLUMN, content);
		g_free(content);
	    }

	    if (d_participant_get_nick (d_client_get_participant (clients[index]))!=NULL) {
		content = d_participant_get_nick (d_client_get_participant (clients[index]));
		gtk_clist_set_text((GtkCList *) list, index, C_NICK_COLUMN, content);
		g_free(content);
	    }
	    index++;
	}
    }
    gtk_clist_thaw ((GtkCList *) list);
}

static void fill_list_with_delivery_payments(GtkCList * list,DeliveryPayment ** deliveryPayments) {
    gchar * content;
    gint index=0;
    gchar *empty_row[]={"","","","","","",""};
    Repair * repair;
    time_t fechaEmision;
    
    if (deliveryPayments!=NULL) {
	while (deliveryPayments[index]!=NULL) {
	    size_t stored;
	    gchar *timer_char = (gchar *) g_malloc(sizeof(gchar)*100);


	    if (index==30) gtk_clist_freeze ((GtkCList *) list);
	    gtk_clist_append ((GtkCList *) list,empty_row);
	    if ((d_delivery_payment_get_number (deliveryPayments[index]))!=ULONG_MAX) {
		content = g_strdup_printf("%lu",d_delivery_payment_get_number (deliveryPayments[index]));
		gtk_clist_set_text((GtkCList *) list, index, C_DELIVERY_PAYMENT_COLUMN, content);
		g_free(content);
	    }
	    repair = d_delivery_payment_get_repair (deliveryPayments[index]);
	    if (d_vehicle_get_license_number (d_repair_get_vehicle (repair))!=NULL) {
		content = d_vehicle_get_license_number (d_repair_get_vehicle (repair));
		gtk_clist_set_text((GtkCList *) list, index, C_LICENSE_NUMBER_COLUMN,content);
		g_free(content);
	    }

	    if (d_repair_get_total (repair)!=G_MINDOUBLE) {
		content = g_strdup_printf("%10.2f",(d_repair_get_total (repair)));
		gtk_clist_set_text((GtkCList *) list, index, C_TOTAL_COLUMN, content);
		g_free(content);
	    }

	    if (d_participant_get_nick (d_shop_get_participant (d_repair_get_shop (repair)))!=NULL) {
		content = d_participant_get_nick (d_shop_get_participant (d_repair_get_shop (repair)));
		gtk_clist_set_text((GtkCList *) list, index, C_SHOP_COLUMN, content);
		g_free(content);
	    }

	    if (d_participant_get_nick (d_company_get_participant (d_repair_get_company (repair)))!=NULL) {
		content = d_participant_get_nick (d_company_get_participant (d_repair_get_company (repair)));
		gtk_clist_set_text((GtkCList *) list, index, C_ENTERPRISE_COLUMN, content);
		g_free(content);
	    }
	    
	    fechaEmision = d_delivery_payment_get_issue_date (delivery_payments[index]);

	    stored=strftime(timer_char,sizeof(gchar)*100,"%d-%m-%Y", localtime(&fechaEmision));

	    
	    gtk_clist_set_text((GtkCList *) list, index, C_DATE_COLUMN, timer_char);
            g_free(timer_char);
	    	    
	    index++;
	}
    }
    gtk_clist_thaw ((GtkCList *) list);

}


static void fill_shop_combo() {
    GtkWidget * combo;
    GtkWidget * combo2;
    Shop ** shops;
    gulong codeCombo;

    combo = glade_xml_get_widget (xml, "combo_empresa_busqueda_facturas");
    combo2 = glade_xml_get_widget (xml, "combo_filtro_tiendas_facturas");
    codeCombo = c_util_get_code_from_combo (combo);


    shops = m_shop_find_by_enterprise_code (codeCombo);

    g_print("%lu",codeCombo);
    clean_shop_combo();

    c_util_set_code_and_name_on_combo (C_ALL_SHOPS_OF_ENTERPRISE,g_strdup(_("All the shops")),GTK_COMBO(combo2));

    c_util_fill_combo_with_shops (combo2,shops);

    m_shop_clear_array (shops);
    shops=NULL;

}

static void fill_enterprise_combo() {
    GtkWidget * combo;
    Enterprise ** enterprises;

    combo = glade_xml_get_widget (xml, "combo_empresa_busqueda_facturas");

    c_util_set_code_and_name_on_combo (C_ALL_ENTERPRISES,g_strdup(_("All the enterprises")),GTK_COMBO(combo));
    enterprises = m_enterprise_find_all ();

    c_util_fill_combo_with_enterprises (combo,enterprises);

}

static void on_search_button_clicked(GtkButton *button, gpointer data) {

    if (do_listing==FALSE)
	do_clients_search();
    else
        do_listing_search();


}

static void do_listing_search() {
    ConditionDeliveryPaymentClients * con;
    GtkWidget * delivery_payments_list;
    GtkWidget * check_insurance=glade_xml_get_widget(xml, "radiobutton_con_seguro_busqueda_facturas");
    GtkWidget * list = glade_xml_get_widget(xml,"lista_clientes");
    GtkWidget * dateedit_desde = glade_xml_get_widget(xml,"dateedit_desde");
    GtkWidget * dateedit_hasta = glade_xml_get_widget(xml,"dateedit_hasta");
    time_t initial_date = gnome_date_edit_get_date(GNOME_DATE_EDIT(dateedit_desde));
    time_t ending_date = gnome_date_edit_get_date(GNOME_DATE_EDIT(dateedit_hasta));
    con = (ConditionDeliveryPaymentClients *) g_malloc0(sizeof(ConditionDeliveryPaymentClients));

    con->enterprise_code = get_code_from_enterprises();
    con->shop_code = get_code_from_shops();
    con->insurance = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_insurance));

    con -> initial_date = g_strdup(ctime(&initial_date));
    con -> ending_date = g_strdup(ctime(&ending_date));

    delivery_payments = m_delivery_payment_find_by_condition (con);
    delivery_payments_list = glade_xml_get_widget (xml, "lista_albaranes");
    gtk_clist_clear(GTK_CLIST(delivery_payments_list));
    gtk_clist_clear(GTK_CLIST(list));

    fill_list_with_delivery_payments(GTK_CLIST(delivery_payments_list),delivery_payments);
    if (con!=NULL) {
	if (con->enterprise_code!=NULL) {
	    g_free(con->enterprise_code);
            con->enterprise_code=NULL;
	}
	if (con->shop_code!=NULL) {
	    g_free(con->shop_code);
            con->shop_code=NULL;
	}
	if (con->nick!=NULL) {
	    g_free(con->nick);
            con->nick=NULL;
	}
	if (con->code!=NULL) {
	    g_free(con->code);
            con->code=NULL;
	}
	if (con->initial_date!=NULL) {
	    g_free(con->initial_date);
            con->initial_date=NULL;
	}
	if (con->ending_date!=NULL) {
	    g_free(con->ending_date);
            con->ending_date=NULL;
	}
	g_free(con);
        con=NULL;
    }


}

static void clean_shop_combo() {
    GtkWidget * combo;
    GtkWidget * entry;

    entry = glade_xml_get_widget (xml, "combo_entry_filtro_tiendas");
    gtk_entry_set_text(GTK_ENTRY(entry),"");

    combo = glade_xml_get_widget (xml, "combo_filtro_tiendas_facturas");
    c_util_clean_code_and_name_combo (combo);
}

static void clean_enterprise_combo() {
    GtkWidget * combo;
    GtkWidget * entry;

    entry = glade_xml_get_widget (xml, "entry_empresa_busqueda_facturas");
    gtk_entry_set_text(GTK_ENTRY(entry),"");

    combo = glade_xml_get_widget (xml, "combo_empresa_busqueda_facturas");
    c_util_clean_code_and_name_combo (combo);

}

static void clean_combos(){

    gchar * e_name =NULL;
    gchar * s_name =NULL;
    clean_enterprise_combo();
    clean_shop_combo();
    fill_enterprise_combo();

    e_name = u_parameter_manager_get_parameter ("empresa");
    if (e_name!=NULL) {
	gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget (xml, "entry_empresa_busqueda_facturas")), e_name);
    }
    g_free(e_name);

    s_name = u_parameter_manager_get_parameter ("tienda");
    if (s_name!=NULL) {
	gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget (xml, "combo_entry_filtro_tiendas")), s_name);
    }
    g_free(s_name);

}

static void clean_nat(void) {

    GtkWidget * entry;

    entry = glade_xml_get_widget (xml, "entry_dni_busqueda_facturas");
    gtk_entry_set_text(GTK_ENTRY(entry),"");
    entry = glade_xml_get_widget (xml, "entry_nombre_busqueda_facturas");
    gtk_entry_set_text(GTK_ENTRY(entry),"");

    entry =NULL;
}

static void clean_cif(void){

    GtkWidget * entry;

    entry = glade_xml_get_widget (xml, "entry_cif_busqueda_facturas");
    gtk_entry_set_text(GTK_ENTRY(entry),"");
    entry = glade_xml_get_widget (xml, "entry_nombre_busqueda_facturas");
    gtk_entry_set_text(GTK_ENTRY(entry),"");

    entry =NULL;

}



static void on_invoice_button_clicked(GtkButton *button, gpointer data) {
    if (do_listing==FALSE)
	do_invoice();
    else
        c_util_open_info_window (_("This option just let you print the data"));
}

static void on_delete_button_clicked(GtkButton *button, gpointer data) {

    clean_screen();
    clean_combos();
    clean_nat();
    clean_cif();

}


static void on_enterprise_entry_changed(GtkEntry *entry,
					      gpointer user_data) {
          fill_shop_combo();

}


static void clean_screen() {

    GtkWidget *vbox;

    vbox = GTK_WIDGET(glade_xml_get_widget(xml, widget_name));

    gtk_container_foreach(GTK_CONTAINER(vbox),&c_util_clean_widget,NULL);


}

static void on_search_nat_pressed(GtkButton *button,gpointer user_data) {
    GtkWidget * label1;
    GtkWidget * label2;
    GtkWidget * entry1;
    GtkWidget * entry2;
    GtkWidget * label3;
    GtkWidget * entry3;
    clean_cif();

    label1 = glade_xml_get_widget(xml, "label_dni_busqueda_facturas");
    label2 = glade_xml_get_widget(xml, "label_cif_busqueda_facturas");
    entry1 = glade_xml_get_widget(xml, "entry_dni_busqueda_facturas");
    entry2 = glade_xml_get_widget(xml, "entry_cif_busqueda_facturas");
    label3 = glade_xml_get_widget(xml, "label_nombre_busqueda_facturas");
    entry3 = glade_xml_get_widget(xml, "entry_nombre_busqueda_facturas");

    if (!GTK_WIDGET_VISIBLE(label1)) {
	gtk_widget_show(label1);
    }
    if (!GTK_WIDGET_VISIBLE(entry1)) {
	gtk_widget_show(entry1);
    }
    if (GTK_WIDGET_VISIBLE(label2)) {
        gtk_widget_hide(label2);
    }
    if (GTK_WIDGET_VISIBLE(entry2)) {
        gtk_widget_hide(entry2);
    }
    if (!GTK_WIDGET_VISIBLE(label3)) {
	gtk_widget_show(label3);
    }
    if (!GTK_WIDGET_VISIBLE(entry3)) {
	gtk_widget_show(entry3);
    }


}

static void on_search_cif_pressed(GtkButton *button,gpointer user_data) {
    GtkWidget * label1;
    GtkWidget * label2;
    GtkWidget * entry1;
    GtkWidget * entry2;
    GtkWidget * label3;
    GtkWidget * entry3;

    label1 = glade_xml_get_widget(xml, "label_dni_busqueda_facturas");
    label2 = glade_xml_get_widget(xml, "label_cif_busqueda_facturas");
    entry1 = glade_xml_get_widget(xml, "entry_dni_busqueda_facturas");
    entry2 = glade_xml_get_widget(xml, "entry_cif_busqueda_facturas");
    label3 = glade_xml_get_widget(xml, "label_nombre_busqueda_facturas");
    entry3 = glade_xml_get_widget(xml, "entry_nombre_busqueda_facturas");

    clean_nat();

    if (!GTK_WIDGET_VISIBLE(label2)) {
	gtk_widget_show(label2);
    }
    if (!GTK_WIDGET_VISIBLE(entry2)) {
	gtk_widget_show(entry2);
    }
    if (GTK_WIDGET_VISIBLE(label1)) {
        gtk_widget_hide(label1);
    }
    if (GTK_WIDGET_VISIBLE(entry1)) {
        gtk_widget_hide(entry1);
    }
    if (!GTK_WIDGET_VISIBLE(label3)) {
	gtk_widget_show(label3);
    }
    if (!GTK_WIDGET_VISIBLE(entry3)) {
	gtk_widget_show(entry3);
    }

}
static void on_search_one_pressed(GtkButton *button,gpointer user_data) {

    GtkWidget * label1;
    GtkWidget * label2;
    GtkWidget * entry1;
    GtkWidget * entry2;
    GtkWidget * button2;
    GtkWidget * frame1;
    GtkWidget * frame2;
    GtkWidget * frame3;
    GtkWidget * label3;
    GtkWidget * entry3;
    GtkWidget * print_button = glade_xml_get_widget(xml, "button_imprimir_listado");

    label1 = glade_xml_get_widget(xml, "label_dni_busqueda_facturas");
    label2 = glade_xml_get_widget(xml, "label_cif_busqueda_facturas");
    entry1 = glade_xml_get_widget(xml, "entry_dni_busqueda_facturas");
    entry2 = glade_xml_get_widget(xml, "entry_cif_busqueda_facturas");
    button2 = glade_xml_get_widget(xml,"radiobutton_DNI_busqueda_facturas");
    frame1 = glade_xml_get_widget(xml,"frame_checks_facturas");
    label3 = glade_xml_get_widget(xml, "label_nombre_busqueda_facturas");
    entry3 = glade_xml_get_widget(xml, "entry_nombre_busqueda_facturas");
    frame2 = glade_xml_get_widget(xml,"frame_checks_tipo_persona_facturas");
    frame3 = glade_xml_get_widget(xml,"frame_checks_seguro");

    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button2),TRUE);
    if (!GTK_WIDGET_VISIBLE(label1)) {
        gtk_widget_show(label1);
    }
    if (!GTK_WIDGET_VISIBLE(entry1)) {
        gtk_widget_show(entry1);
    }
    if (GTK_WIDGET_VISIBLE(label2)) {
	gtk_widget_hide(label2);
    }
    if (GTK_WIDGET_VISIBLE(entry2)) {
	gtk_widget_hide(entry2);
    }
    if (!GTK_WIDGET_VISIBLE(frame1)) {
        gtk_widget_show(frame1);
    }
    if (!GTK_WIDGET_VISIBLE(label3)) {
	gtk_widget_show(label3);
    }
    if (!GTK_WIDGET_VISIBLE(entry3)) {
	gtk_widget_show(entry3);
    }
    if (GTK_WIDGET_VISIBLE(frame2)) {
	gtk_widget_hide(frame2);
    }
    if (!GTK_WIDGET_VISIBLE(frame3)) {
	gtk_widget_show(frame3);
    }
    if (GTK_WIDGET_VISIBLE(print_button)) {
	gtk_widget_hide(print_button);
    }
    do_listing=FALSE;


}

static void on_search_with_insurance_pressed(GtkButton *button,gpointer user_data){
    GtkWidget * list = glade_xml_get_widget(xml,"lista_clientes");
    GtkWidget * list2 = glade_xml_get_widget(xml,"lista_albaranes");

    gtk_clist_clear(GTK_CLIST(list));
    gtk_clist_clear(GTK_CLIST(list2));
    gtk_clist_set_selection_mode(GTK_CLIST(list2),GTK_SELECTION_SINGLE);

}
static void on_search_without_insurance_pressed(GtkButton *button,gpointer user_data){
    GtkWidget * list = glade_xml_get_widget(xml,"lista_clientes");
    GtkWidget * list2 = glade_xml_get_widget(xml,"lista_albaranes");

    gtk_clist_clear(GTK_CLIST(list));
    gtk_clist_clear(GTK_CLIST(list2));
    gtk_clist_set_selection_mode(GTK_CLIST(list2),GTK_SELECTION_MULTIPLE);

}

static void on_search_all_pressed(GtkButton *button,gpointer user_data) {
    GtkWidget * label1;
    GtkWidget * label2;
    GtkWidget * entry1;
    GtkWidget * entry2;
    GtkWidget * frame1;
    GtkWidget * label3;
    GtkWidget * entry3;
    GtkWidget * frame2;
    GtkWidget * frame3;
    GtkWidget * print_button = glade_xml_get_widget(xml, "button_imprimir_listado");



    clean_cif();
    clean_nat();
    label1 = glade_xml_get_widget(xml, "label_dni_busqueda_facturas");
    label2 = glade_xml_get_widget(xml, "label_cif_busqueda_facturas");
    entry1 = glade_xml_get_widget(xml, "entry_dni_busqueda_facturas");
    entry2 = glade_xml_get_widget(xml, "entry_cif_busqueda_facturas");
    frame1 = glade_xml_get_widget(xml,"frame_checks_facturas");
    label3 = glade_xml_get_widget(xml, "label_nombre_busqueda_facturas");
    entry3 = glade_xml_get_widget(xml, "entry_nombre_busqueda_facturas");
    frame2 = glade_xml_get_widget(xml,"frame_checks_tipo_persona_facturas");
    frame3 = glade_xml_get_widget(xml,"frame_checks_seguro");
    //frame4 = glade_xml_get_widget(xml,"frame_fechas");

    if (GTK_WIDGET_VISIBLE(label1)) {
        gtk_widget_hide(label1);
    }
    if (GTK_WIDGET_VISIBLE(entry1)) {
        gtk_widget_hide(entry1);
    }
    if (GTK_WIDGET_VISIBLE(label2)) {
	gtk_widget_hide(label2);
    }
    if (GTK_WIDGET_VISIBLE(entry2)) {
	gtk_widget_hide(entry2);
    }
    if (GTK_WIDGET_VISIBLE(frame1)) {
        gtk_widget_hide(frame1);
    }
    if (GTK_WIDGET_VISIBLE(label3)) {
	gtk_widget_hide(label3);
    }
    if (GTK_WIDGET_VISIBLE(entry3)) {
	gtk_widget_hide(entry3);
    }
    if (!GTK_WIDGET_VISIBLE(frame2)) {
        gtk_widget_show(frame2);
    }
    if (!GTK_WIDGET_VISIBLE(frame3)) {
	gtk_widget_show(frame3);
    }
    if (GTK_WIDGET_VISIBLE(print_button)) {
	gtk_widget_hide(print_button);
    }
    do_listing=FALSE;

}

static void on_search_listing_button_pressed(GtkButton *button,gpointer user_data) {
    GtkWidget * label1;
    GtkWidget * label2;
    GtkWidget * entry1;
    GtkWidget * entry2;
    GtkWidget * frame1;
    GtkWidget * label3;
    GtkWidget * entry3;
    GtkWidget * frame2;
    GtkWidget * frame3;
    GtkWidget * print_button = glade_xml_get_widget(xml, "button_imprimir_listado");



    clean_cif();
    clean_nat();

    label1 = glade_xml_get_widget(xml, "label_dni_busqueda_facturas");
    label2 = glade_xml_get_widget(xml, "label_cif_busqueda_facturas");
    entry1 = glade_xml_get_widget(xml, "entry_dni_busqueda_facturas");
    entry2 = glade_xml_get_widget(xml, "entry_cif_busqueda_facturas");
    frame1 = glade_xml_get_widget(xml,"frame_checks_facturas");
    label3 = glade_xml_get_widget(xml, "label_nombre_busqueda_facturas");
    entry3 = glade_xml_get_widget(xml, "entry_nombre_busqueda_facturas");
    frame2 = glade_xml_get_widget(xml,"frame_checks_tipo_persona_facturas");
    frame3 = glade_xml_get_widget(xml,"frame_checks_seguro");

    if (GTK_WIDGET_VISIBLE(label1)) {
        gtk_widget_hide(label1);
    }
    if (GTK_WIDGET_VISIBLE(entry1)) {
        gtk_widget_hide(entry1);
    }
    if (GTK_WIDGET_VISIBLE(label2)) {
	gtk_widget_hide(label2);
    }
    if (GTK_WIDGET_VISIBLE(entry2)) {
	gtk_widget_hide(entry2);
    }
    if (GTK_WIDGET_VISIBLE(frame1)) {
        gtk_widget_hide(frame1);
    }
    if (GTK_WIDGET_VISIBLE(label3)) {
	gtk_widget_hide(label3);
    }
    if (GTK_WIDGET_VISIBLE(entry3)) {
	gtk_widget_hide(entry3);
    }
    if (GTK_WIDGET_VISIBLE(frame2)) {
        gtk_widget_hide(frame2);
    }
    if (!GTK_WIDGET_VISIBLE(frame3)) {
	gtk_widget_show(frame3);
    }
    if (!GTK_WIDGET_VISIBLE(print_button)) {
	gtk_widget_show(print_button);
    }
    do_listing = TRUE;

}


static void on_search_invoice_press_key(GtkWidget *widget,
					   GdkEventKey *event,
					   gpointer user_data) {
    if (event->keyval==GDK_Return) {
        if (do_listing==FALSE)
	    do_clients_search();
        else
            do_listing_search();
    }

                                           
}
static void on_clients_list_select_row (GtkCList        *list,
					  gint             row,
					  gint             column,
					  GdkEvent        *event,
					  gpointer         user_data) {

    if (event!=NULL) {
	if (event->type == GDK_2BUTTON_PRESS)
	{
            if (do_listing==FALSE)
		do_delivery_payment_search(list,row);
	}
    }
}

static void do_delivery_payment_search(GtkCList * list,gint row) {

    ConditionDeliveryPaymentClients * con;
    gchar ** content;
    gchar ** stop = (gchar **) g_malloc0(sizeof(gchar *));
    gulong participant_code = ULONG_MAX;
    GtkWidget * delivery_payments_list;
    GtkWidget * check_insurance=glade_xml_get_widget(xml, "radiobutton_con_seguro_busqueda_facturas");
    
    content = (gchar **) g_malloc0(sizeof(gchar *));
    con = (ConditionDeliveryPaymentClients *) g_malloc0(sizeof(ConditionDeliveryPaymentClients));

    gtk_clist_get_text(list,row,C_CODE_COLUMN,content);
    con->code = g_strdup(*content);
    gtk_clist_get_text(list,row,C_NICK_COLUMN,content);
    con->nick = g_strdup(*content);
    gtk_clist_get_text(list,row,C_PARTICIPANT_CODE_COLUMN,content);
    con->participant_code = g_strdup(*content);
    participant_code = strtoul(con->participant_code,stop,10);
    con->client_type=m_client_get_type (participant_code);
    search_garage = (con->client_type==M_CLIENT_GARAGE_CODE);
    g_free(stop);
    g_free(content);
    con->enterprise_code = get_code_from_enterprises();
    con->shop_code = get_code_from_shops();
    con->insurance = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_insurance));

    delivery_payments = m_delivery_payment_find_by_condition (con);
    delivery_payments_list = glade_xml_get_widget (xml, "lista_albaranes");
    gtk_clist_clear(GTK_CLIST(delivery_payments_list));
    fill_list_with_delivery_payments(GTK_CLIST(delivery_payments_list),delivery_payments);
    if (con!=NULL) {
	if (con->enterprise_code!=NULL) {
	    g_free(con->enterprise_code);
            con->enterprise_code=NULL;
	}
	if (con->shop_code!=NULL) {
	    g_free(con->shop_code);
            con->shop_code=NULL;
	}
	if (con->nick!=NULL) {
	    g_free(con->nick);
            con->nick=NULL;
	}
	if (con->code!=NULL) {
	    g_free(con->code);
            con->code=NULL;
	}
	g_free(con);
        con=NULL;
    }

}

static gchar * get_code_from_shops() {
    GtkWidget * combo;
    gulong code;

    combo = glade_xml_get_widget (xml, "combo_filtro_tiendas_facturas");
    code = c_util_get_code_from_combo (combo);
    if (code > 0) {
	return g_strdup_printf("%lu",code);
    } else {
        return NULL;
    }

}

static gchar * get_code_from_enterprises() {
    GtkWidget * combo;
    gulong code;

    combo = glade_xml_get_widget (xml, "combo_empresa_busqueda_facturas");
    code = c_util_get_code_from_combo (combo);
    if (code > 0) {
	return g_strdup_printf("%lu",code);
    } else {
	return NULL;
    }
}

DeliveryPaymentId **  obtain_delivery_payments_ids(GArray * selected,DeliveryPayment ** delivery_payments) {

    gint index=0;
    gint indexOfDelivery=0;
    DeliveryPaymentId ** selected_deliverys;

    selected_deliverys = (DeliveryPaymentId **) g_malloc0(sizeof(DeliveryPaymentId *)*(selected->len+1));
    while (index<selected->len) {
        selected_deliverys[index] = (DeliveryPaymentId *) g_malloc0(sizeof(DeliveryPaymentId));
	indexOfDelivery = g_array_index(selected,gint,index);
	selected_deliverys[index] -> delivery_payment_number = g_strdup_printf("%lu",d_delivery_payment_get_number (delivery_payments[indexOfDelivery]));
        selected_deliverys[index] -> enterprise_code = g_strdup_printf("%lu",
	   d_participant_get_code (d_enterprise_get_participant (d_repair_get_enterprise (d_delivery_payment_get_repair (delivery_payments[indexOfDelivery])))));
	selected_deliverys[index] -> shop_code =  g_strdup_printf("%lu",
								     d_participant_get_code (d_shop_get_participant (d_repair_get_shop (d_delivery_payment_get_repair (delivery_payments[indexOfDelivery])))));
	if (search_garage!=FALSE) {
	    gulong codigoTaller = ULONG_MAX;
	    gulong codigoCliente = ULONG_MAX;
	    if (d_repair_get_garage (d_delivery_payment_get_repair (
								     delivery_payments[indexOfDelivery]
								    ))!=NULL) {
		codigoTaller = d_participant_get_code (d_garage_get_participant (d_repair_get_garage (d_delivery_payment_get_repair (
																  delivery_payments[indexOfDelivery]
																 ))));

		codigoCliente = d_participant_get_code (d_client_get_participant (d_repair_get_client (d_delivery_payment_get_repair (
																   delivery_payments[indexOfDelivery]
																  ))));
	    }
	    //	    if (d_delivery_payment_get_garage_total (delivery_payments[indexOfDelivery])>0)
	    if (m_delivery_payment_client_is_not_from_garage (codigoTaller,
                                               codigoCliente,
					       d_delivery_payment_get_number (delivery_payments[indexOfDelivery]),
					       d_participant_get_code (d_shop_get_participant (d_repair_get_shop (d_delivery_payment_get_repair (delivery_payments[indexOfDelivery])))),
					       d_participant_get_code (d_enterprise_get_participant (d_repair_get_enterprise (d_delivery_payment_get_repair (delivery_payments[indexOfDelivery]))))
					      )!=FALSE)
		selected_deliverys[index]->from_garage=TRUE;
	    else
		selected_deliverys[index]->from_garage=FALSE;
	} else {
            selected_deliverys[index]->from_garage=FALSE;
	}
	index++;
    }
    selected_deliverys[index]=NULL;
    return selected_deliverys;
}

static void do_invoice() {
    GArray * selected;
    gint value;
    gint index = 0;
    GtkCList * list = GTK_CLIST(glade_xml_get_widget (xml, "lista_albaranes"));
    DeliveryPaymentId ** delivery_payment_clients;
    selected = c_util_get_selected_rows_from_clist (list);

    while (index<selected->len) {        
	value = g_array_index(selected,gint,index);
        index++;
    }

    delivery_payment_clients = obtain_delivery_payments_ids(selected,delivery_payments);

    g_array_free(selected,TRUE);


    if (selected->len>0) {

    i_shortcut_change_dialog (GTK_WIDGET(list),1,0);

    


    c_invoice_load_delivery_payments(delivery_payment_clients);



    } else {
	GtkWidget * modal_window;
	modal_window = gnome_message_box_new (_("Select a delivery payment"),
					      GNOME_MESSAGE_BOX_ERROR,
					      _("Ok"), NULL);
	gtk_window_set_modal(GTK_WINDOW(modal_window),1);
	gtk_window_set_position(GTK_WINDOW(modal_window),GTK_WIN_POS_CENTER);
	gtk_window_set_transient_for(GTK_WINDOW(modal_window),
				     GTK_WINDOW(lookup_widget((modal_window), "app1")));
	gnome_dialog_run_and_close(GNOME_DIALOG(modal_window));


    }


}

static void do_print() {
    if (delivery_payments != NULL) {
	Listing list;
        Listing * listing = (Listing *) g_malloc0(sizeof(Listing));
        gulong    rows = 0;
	gchar * shop_name = g_strdup(gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(xml,"combo_entry_filtro_tiendas"))));

	list.content_title = d_delivery_payment_get_content_title ();
	list.listing_title = d_delivery_payment_get_listing_title (shop_name);
	list.content       = d_delivery_payment_array_to_content (delivery_payments, &rows);
	list.width_of_cols = d_delivery_payment_get_listing_columns_width ();
	list.alignment_of_cols = d_delivery_payment_get_listing_columns_alignment ();
	list.n_rows        = rows;
	list.n_columns     = d_delivery_payment_get_content_size ();

        g_free(shop_name);
        // Fixme!!!
        // Hardcoded -> get this from a combo
	list.orientation   = 4;
	list.type_of_paper = 2;

	*listing = list;
        p_listing_wrap_print (listing);
    } else {
	c_util_open_error_window (_("You have to make the search of the delivery payments for printing"));
    }
}

static void on_print_listing_button_clicked(GtkButton *button, gpointer data) {
    do_print();
}

