
/* find_documents_control.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include <gnome.h>
#include <glade/glade.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <glib.h>
#include <time.h>

#include "a-shortcut.h"
#include "support.h"
#include "document_model.h"
#include "find_documents_control.h"
#include "associated_comp_documents_control.h"
#include "insert_document_control.h"
#include "util.h"
#include "parameter_manager.h"
#include "control_util.h"

#define C_NAME_COLUMN      0
#define C_DESCRIPTION_COLUMN 1
#define C_TYPE_COLUMN 2

static GtkWidget *find_documents=NULL;
static GladeXML *xml = NULL;
static gchar dialog_file[]="/src/interface/glade/documents_view.glade";
static gchar widget_name[]="vbox-listado-documentos";
static Document ** documents_array;

static void fill_list(GtkCList * list,gulong size);
static void on_delete_button_clicked(GtkButton *button,gpointer data);
static void on_clean_button_clicked(GtkButton *button, gpointer data);
static void on_search_button_clicked(GtkButton *button, gpointer data);
static void on_new_button_clicked(GtkButton *button, gpointer data);
static void on_modify_button_clicked(GtkButton *button, gpointer data);
static void on_company_button_clicked(GtkButton *button,  gpointer data);
static void on_description_press_key(GtkWidget *widget,
					   GdkEventKey *event,
					   gpointer user_data);
static void obtain_documents(void);
static void clean_screen(void);
static Document *get_selected_document(GtkWidget * list, GArray *selected);
static gint obtain_documents_ids(GArray *selected, Document **docs, gulong *ids);


GtkWidget *c_find_documents_open_window(GtkWidget *app1, GtkWidget *vbox)
{

    gchar *directory = g_strconcat(AA_DATA_DIR,dialog_file,NULL);

    if (find_documents==NULL) {
	GtkWidget *dialog;

	xml = glade_xml_new (directory, widget_name);
	dialog = glade_xml_get_widget (xml, widget_name);



	gtk_box_pack_start (GTK_BOX (vbox), dialog, TRUE, TRUE, 0);

	glade_xml_signal_connect (xml, "on_entry-nombre-filtrado-listadodocumentos_key_press_event",
				  GTK_SIGNAL_FUNC (on_description_press_key));
	glade_xml_signal_connect (xml, "on_entry-descripciondocumento-filtrado-listadodocumentos_key_press_event",
				  GTK_SIGNAL_FUNC (on_description_press_key));

	glade_xml_signal_connect (xml, "on_boton-eliminar-listadodocumentos_clicked",
				  GTK_SIGNAL_FUNC (on_delete_button_clicked));
	glade_xml_signal_connect (xml, "on_Limpiar-listadodocumentos_clicked",
				  GTK_SIGNAL_FUNC (on_clean_button_clicked));
	glade_xml_signal_connect (xml, "on_Buscar-listadodocumentos_clicked",
				  GTK_SIGNAL_FUNC (on_search_button_clicked));

	glade_xml_signal_connect (xml, "on_boton-nuevo-listadodocumentos_clicked",
				  GTK_SIGNAL_FUNC (on_new_button_clicked));
	glade_xml_signal_connect (xml, "on_boton-modificar-listadodocumentos_clicked",
				  GTK_SIGNAL_FUNC (on_modify_button_clicked));
                                        
	glade_xml_signal_connect (xml, "on_boton-compania-listadodocumentos_clicked",
  				  GTK_SIGNAL_FUNC (on_company_button_clicked));
        


	find_documents = dialog;
    }

    clean_screen();

    m_document_clear_array (documents_array);
    documents_array=NULL;

    g_free(directory);
    return find_documents;

}

static void fill_list(GtkCList * list,gulong size) {

    gint i;
    gchar * temp;

    gchar *empty_row[]={"","","","","","","","",""};


    for (i=0;i<size;i++) {
	if (i==30) gtk_clist_freeze ((GtkCList *) list);

	gtk_clist_append ((GtkCList *) list,empty_row);

	temp = d_document_get_name (documents_array[i]);
	gtk_clist_set_text((GtkCList *) list, i, C_NAME_COLUMN, temp);
	g_free(temp);

	temp = d_document_get_description (documents_array[i]);
	gtk_clist_set_text((GtkCList *) list, i, C_DESCRIPTION_COLUMN, temp);
	g_free(temp);

	if (d_document_get_from_repair (documents_array[i]) &&
	    d_document_get_from_invoice (documents_array[i])) {
	    temp = g_strdup(N_("R/I"));
	} else if (d_document_get_from_repair (documents_array[i])){
	    temp = g_strdup(N_("Repair"));
	} else if (d_document_get_from_invoice (documents_array[i])){
	    temp = g_strdup(N_("Invoicing"));
	} else {
	    temp = g_strdup("");
	}

	gtk_clist_set_text((GtkCList *) list, i, C_TYPE_COLUMN, _(temp));
	g_free(temp);
    }

    gtk_clist_thaw ((GtkCList *) list);

}

static gint obtain_document_ids(GArray *selected, Document **docs,
				gulong  *doc_ids) {

    gint index=0;
    gint doc_index=0;

    while (index<selected->len) {
	doc_index = g_array_index(selected,gint,index);

	doc_ids[index] =
	    d_document_get_code (docs[doc_index]);
	index++;
    }
    doc_ids[index]=ULONG_MAX;
    return index;
}


static void on_delete_button_clicked(GtkButton *button,
							gpointer data) {
    GtkWidget *list = glade_xml_get_widget (xml,"clist-listadodocumentos");
    GtkWidget *warning_dialog = NULL;
    GtkWidget *app1  = lookup_widget(GTK_WIDGET(GTK_CLIST(list)), "app1");
    GArray    *selected = c_util_get_selected_rows_from_clist (GTK_CLIST(list));
    gulong    *document_ids;
    gint       response = -1,i;
    gulong      number = 1;

    if ((selected->len)>0) {

        number = (gulong)selected->len;
	document_ids = (gulong *) g_malloc(sizeof(gulong)*(number+1));
	obtain_document_ids(selected, documents_array, document_ids);
	warning_dialog =
	    gnome_message_box_new (_("Are you sure you want to REMOVE the selected items from de database?"),
				   GNOME_MESSAGE_BOX_WARNING,
				   GNOME_STOCK_BUTTON_OK,
				   GNOME_STOCK_BUTTON_CANCEL,
				   NULL);
	if (warning_dialog !=NULL) {
	    gtk_window_set_modal(GTK_WINDOW(warning_dialog),1);
	    gtk_window_set_position(GTK_WINDOW(warning_dialog),GTK_WIN_POS_CENTER);
	    gtk_window_set_transient_for(GTK_WINDOW(warning_dialog),
					 GTK_WINDOW(app1));
	    response = gnome_dialog_run_and_close(GNOME_DIALOG (warning_dialog));
	}

	switch (response){
	case 0:
	    clean_screen();
            i = 0;
	    while(document_ids[i]!=ULONG_MAX) {
		m_document_delete_by_code (document_ids[i]);
		i++;
	    }
	    break;
	case 1:
	    break;
	default:
	    /* User closed dialog with window manager,
	     * assume case 1 (cancel)
	     */
	    break;
	}
	g_free(document_ids);
        document_ids = NULL;
    } else {
	c_util_open_error_window (_("Select at least one document to remove from the database"));
    }
    g_array_free(selected, FALSE);
}

static void on_clean_button_clicked(GtkButton *button, gpointer data) {
    clean_screen();
}

static void on_search_button_clicked(GtkButton *button, gpointer data) {
    gtk_clist_clear(GTK_CLIST(glade_xml_get_widget(xml,"clist-listadodocumentos")));

    m_document_clear_array (documents_array);
    documents_array=NULL;

    obtain_documents();

}

static void on_new_button_clicked(GtkButton *button, gpointer data) {
    GtkWidget * list = glade_xml_get_widget(xml,"clist-listadodocumentos");
    GtkWidget * app1 = lookup_widget(GTK_WIDGET(GTK_CLIST(list)), "app1");
    GtkWidget * vbox = lookup_widget(GTK_WIDGET(GTK_CLIST(list)), "main_container");

    clean_screen();

    c_document_open_insertion_window(app1, vbox);
    i_shortcut_change_dialog (GTK_WIDGET(list),5,0);

}

static void on_modify_button_clicked(GtkButton *button, gpointer data) {
    GtkWidget * list = glade_xml_get_widget(xml,"clist-listadodocumentos");
    GtkWidget * app1 = lookup_widget(GTK_WIDGET(GTK_CLIST(list)), "app1");
    GtkWidget * vbox = lookup_widget(GTK_WIDGET(GTK_CLIST(list)), "main_container");

    GArray    *selected  = c_util_get_selected_rows_from_clist (GTK_CLIST(list));
    Document  *document = NULL;
    gulong     number;

    number = (gulong)(selected->len);

    if (number>0) {
	if (number==1) {
	    document = get_selected_document(list, selected);

	    c_document_open_insertion_window(app1, vbox);
	    i_shortcut_change_dialog (GTK_WIDGET(list),5,0);
	    c_document_load(document);

	    clean_screen();
	} else {
            c_util_open_error_window (_("Select just one document"));
	}
    } else {
	c_util_open_error_window (_("Select at least one document"));
    }

}

static void obtain_documents() {
    GtkWidget * list;
    ConditionFindDocument condition;
    gchar * content;
    GdkCursor * cursor;
    GtkWidget * window;
    GtkWidget *check       = NULL;
    gulong documents_array_len = 0;
    gboolean active = FALSE;


    list = glade_xml_get_widget(xml,"clist-listadodocumentos");
    window = lookup_widget(GTK_WIDGET(list),"app1");
    cursor = gdk_cursor_new(GDK_WATCH);
    gdk_window_set_cursor(GTK_WIDGET(window)->window, cursor);
    gdk_cursor_destroy(cursor);

    while (gtk_events_pending())
	gtk_main_iteration();

    content=gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "entry-nombre-filtrado-listadodocumentos")),0,-1);
    if (!strcmp(content,"")) {
	condition.name=NULL;
    } else {
	condition.name   = (gchar *) g_malloc (strlen(content)+1);
	strcpy(condition.name, content);
    }
    g_free(content);

    content=gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "entry-descripciondocumento-filtrado-listadodocumentos")),0,-1);
    if (!strcmp(content,"")) {
	condition.description=NULL;
    } else {
	condition.description   = (gchar *) g_malloc (strlen(content)+1);
	strcpy(condition.description, content);
    }
    g_free(content);

    check = glade_xml_get_widget(xml, "checkbutton-facturacion-filtrado-listadodocumentos");
    active = (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check))==TRUE);
    condition.invoice = active;

    check = glade_xml_get_widget(xml, "checkbutton-reparacion-filtrado-listadodocumentos");
    active = (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check))==TRUE);
    condition.repair = active;

    gtk_clist_clear ((GtkCList *) list);
    m_document_clear_array (documents_array);
    documents_array=NULL;


    documents_array = m_document_find_by_condition (&condition,
							&documents_array_len);

    if ((documents_array!=NULL) && (documents_array_len>0)) {
	fill_list(GTK_CLIST(list), documents_array_len);
    } else {
	c_util_open_info_window (_("No document could be found"));
    }

    cursor = gdk_cursor_new(GDK_LEFT_PTR);
    gdk_window_set_cursor(GTK_WIDGET(window)->window, cursor);
    gdk_cursor_destroy(cursor);
}


static void clean_screen() {

    GtkWidget *vbox;
    GtkWidget *list;

    list = glade_xml_get_widget(xml,"clist-listadodocumentos");
    vbox = GTK_WIDGET(glade_xml_get_widget(xml, widget_name));
    gtk_container_foreach(GTK_CONTAINER(vbox),&c_util_clean_widget,NULL);

    if (documents_array!=NULL) {
	gtk_clist_clear ((GtkCList *) list);
	m_document_clear_array (documents_array);
	documents_array=NULL;
    }

}

static void on_description_press_key(GtkWidget *widget,
					   GdkEventKey *event,
					   gpointer user_data) {
    GtkWidget * wid;
    GtkContainer * container;
    GList * list=NULL;
    GList * last_list;
    GList * first_focusable_of_list;


    if (event->keyval==GDK_Return) {
	on_search_button_clicked(NULL,user_data);
    } else if (event->keyval==GDK_Tab) {

	container = (GtkContainer *) widget->parent;
	list = gtk_container_children(container);
	wid=GTK_WIDGET((list)->data);

	while (((GTK_WIDGET_CAN_FOCUS(wid))==0) ||
	       ((GTK_WIDGET_VISIBLE(wid))==0) ||
	       ((GTK_WIDGET_SENSITIVE(wid))==0)) {
	    list=g_list_next(list);
	    wid=GTK_WIDGET((list)->data);
	}

	first_focusable_of_list = list;

	last_list = (GList *) g_list_last(list);
	while (last_list!=list) {
	    wid=GTK_WIDGET((list)->data);
	    if (GTK_WIDGET_HAS_FOCUS(wid)==1) {
		break;
	    }
	    list=g_list_next(list);

	}
	list=g_list_next(list);
	wid=GTK_WIDGET((list)->data);
	while ((((GTK_WIDGET_CAN_FOCUS(wid))==0) ||
		((GTK_WIDGET_VISIBLE(wid))==0) ||
		((GTK_WIDGET_SENSITIVE(wid))==0)) && (last_list!=list)){
	    list=g_list_next(list);
	    wid=GTK_WIDGET((list)->data);
	}
	if (last_list==list) {
	    list=first_focusable_of_list;
	    wid=GTK_WIDGET((list)->data);
	}

	gtk_signal_emit_stop_by_name(GTK_OBJECT (widget),"key_press_event");

	gtk_widget_grab_focus(wid);

	/*  	g_print("Nombre activable: %s\n",wid->name);  */
    }

}

static gint obtain_documents_ids(GArray *selected, Document **docs, gulong *ids){

    gint index=0;
    gint document_index=0;

    while (index<selected->len) {
	document_index = g_array_index(selected,gint,index);

	ids[index] = d_document_get_code (docs[document_index]);
	index++;
    }
    ids[index]=ULONG_MAX;
    return index;
}

static Document *get_selected_document(GtkWidget * list, GArray *selected) {
    gulong    *doc_ids   = NULL;
    Document **doc_array = NULL;
    Document  *result    = NULL;
    gulong     number;

    number = (gulong)(selected->len);

    if (number==1) {
	doc_ids = (gulong *) g_malloc(sizeof(gulong)*(number+1));
	doc_array = (Document **) g_malloc(sizeof(Document *)*(number+1));
	obtain_documents_ids(selected, documents_array, doc_ids);

	m_document_find_by_code_array (doc_array, doc_ids);

	if (doc_array[0]!=NULL) {
	    result = d_document_duplicate (doc_array[0]);
	}

	g_free(doc_ids);
	doc_ids = NULL;
	m_document_clear_array (doc_array);
	doc_array = NULL;
    }

    g_array_free(selected, FALSE);

    return result;
}

static void on_company_button_clicked(GtkButton *button,  gpointer data) {
    GtkWidget *app1  = lookup_widget(GTK_WIDGET(button), "app1");
    GtkWidget *open_associated = NULL;
    GtkWidget * list = glade_xml_get_widget (xml, "clist-listadodocumentos");
    GArray    *selected  = c_util_get_selected_rows_from_clist (GTK_CLIST(list));
    Document *document = NULL;
    gulong     number;

    number = (gulong)(selected->len);

    if (number>0) {
	if (number==1) {
	    document = get_selected_document(list, selected);

	    open_associated = c_associated_companies_open (app1,NULL, document);

	    if (open_associated!=NULL) {
		gtk_window_set_modal(GTK_WINDOW(open_associated),1);
		gtk_window_set_position(GTK_WINDOW(open_associated),GTK_WIN_POS_CENTER);
		gtk_window_set_transient_for(GTK_WINDOW(open_associated),
					     GTK_WINDOW(app1));
		gtk_widget_show(GTK_WIDGET(open_associated));
	    }

	    d_document_clear (document);
	} else {
	    c_util_open_error_window (_("Select a document to link it to a company"));
	}
    } else {
	c_util_open_error_window (_("Select at least one document to link to a company"));
    }
}

