/* find_invoice_control.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */
#include "config.h"
#include <gnome.h>
#include <glade/glade.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <glib.h>
#include <stdlib.h>

#define C_INVOICE_NUMBER_COLUMN 0
#define C_GROUP_CODE_COLUMN 1
#define C_CLIENT_COLUMN 2
#define C_COMPANY_COLUMN 3
#define C_STATE_COLUMN 4
#define C_TOTAL_COLUMN 5
#define C_ALL_SHOPS_CODE 0
#define C_COMPANY_FILTER_NONE 0
#define C_FIND_ALL_STATES 17171

#include "a-shortcut.h"
#include "support.h"
#include "control_util.h"
#include <../interface/printing/listing-print-wrapper.h>
#include "find_invoice_control.h"
#include "invoice.h"
#include "invoice_model.h"
#include "invoice_control.h"
#include "payment_note_model.h"
#include "shop_model.h"
#include "company_model.h"
#include "direct_debit_model.h"
#include "parameter_manager.h"
#include "../interface/printing/listing-print-wrapper.h"

#include "cuaderno58.h"
//#include "cuaderno19.h"

static GtkWidget *find_invoices=NULL;
static gboolean first = FALSE;
static GladeXML *xml;
static Invoice ** invoices;
static PaymentNote ** payment_notes;
static gchar widget_name[]="vbox1_gestion_facturas";
//static gchar url[]="/src/interface/glade/invoice_view.glade";
static gchar url[]="/src/interface/glade/invoice_second_glade.glade";

static void fill_invoice_list(GtkCList * list,Invoice ** invoice_array);
static void clean_screen(void);
static void fill_shop_combo(void);
static void fill_invoice_state_combo(void);
static void fill_invoice_type_combo(void);
static void fill_payments_combo(void);
static void fill_company_combo(void);
static void view_invoice(void);
static void on_press_key(GtkWidget *widget,
					   GdkEventKey *event,
					   gpointer user_data);
static void on_invoice_list_select_row (GtkCList        *list,
					  gint             row,
					  gint             column,
					  GdkEvent        *event,
					  gpointer         user_data);
//static gchar * get_code_from_shops(void);
static void on_delete_button_clicked(GtkButton *button, gpointer data);
static void on_accept_button_clicked(GtkButton *button, gpointer data);
static void on_print_button_clicked(GtkButton *button, gpointer data);
static void on_by_totals_button_clicked(GtkButton *button, gpointer data);
static void on_search_button_clicked(GtkButton *button, gpointer data);
static void on_repayment_button_clicked(GtkButton *button, gpointer data);
static void on_generate_cuaderno58_button_clicked(GtkButton *button, gpointer data);

static gint on_invoice_list_button_press_event(GtkWidget *widget, GdkEventButton *event, gpointer user_data);
static void on_activate_paids_to_managed(GtkMenuItem *menuitem,gpointer user_data);
static void on_ok_paids_to_managed(gint reply, gpointer user_data);
static void on_activate_managed_to_paids(GtkMenuItem *menuitem, gpointer user_data);
static void on_ok_managed_to_paids(gint reply, gpointer user_data);
static void on_activate_selection(GtkMenuItem *menuitem, gpointer user_data);
static void on_activate_unselection(GtkMenuItem *menuitem, gpointer user_data);
static void on_ok_do_direct_debit(gint reply,gpointer user_data);
static void on_invoice_type_combo_entry_changed(GtkEntry *entry,gpointer user_data);
static void do_search(void);
static InvoiceId * obtain_invoice_id_from_list(gint value,GtkCList * list);
static PaymentNoteId * obtain_payment_note_id_from_list(gint value,GtkCList * list);
static void proccess_filename(GtkFileSelection *selector, gpointer user_data);
static void on_activate_buttons(GtkMenuItem *menuitem,
				   gpointer user_data);
static void on_hide_buttons(GtkMenuItem *menuitem,
				   gpointer user_data);
static gint on_toolbar_button_press_event(GtkWidget *widget,
					  GdkEventButton *event, gpointer user_data);


GtkWidget *c_find_invoice_open_window(GtkWidget *app1, GtkWidget *vbox)

{
   
    gchar * directory = g_strconcat(AA_DATA_DIR,url,NULL);
    time_t expiry_date = time(NULL) + 86400;
    GtkWidget * date_dateedit= NULL;
    time_t init_date = time(NULL) - 86400*30;
    GtkWidget * init_date_dateedit = NULL;
    
    first = TRUE;
   
  if (find_invoices==NULL)
    {
      GtkWidget *dialog;
      gchar * s_name = NULL;

      xml = glade_xml_new (directory, widget_name);
      dialog = glade_xml_get_widget (xml, widget_name);
      
      gtk_box_pack_start (GTK_BOX (vbox), dialog, TRUE, TRUE, 0);  

      glade_xml_signal_connect (xml, "on_button_borrar_gestion_facturas_clicked",
				GTK_SIGNAL_FUNC (on_delete_button_clicked));
      glade_xml_signal_connect (xml, "on_button_aceptar_gestion_facturas_clicked",
				GTK_SIGNAL_FUNC (on_accept_button_clicked));
      glade_xml_signal_connect (xml, "on_button_imprimir_gestion_facturas_clicked",
				GTK_SIGNAL_FUNC (on_print_button_clicked));
      glade_xml_signal_connect (xml, "on_button_por_totales_clicked",
				GTK_SIGNAL_FUNC (on_by_totals_button_clicked));
      glade_xml_signal_connect (xml, "on_button_buscar_gestion_facturas_clicked",
				GTK_SIGNAL_FUNC (on_search_button_clicked));
      glade_xml_signal_connect (xml, "on_lista_facturas_select_row",
				GTK_SIGNAL_FUNC (on_invoice_list_select_row));
      glade_xml_signal_connect (xml, "on_press_key_gestion_facturas",
				GTK_SIGNAL_FUNC (on_press_key));
      glade_xml_signal_connect (xml, "on_combo_entry_tipo_factura_gestion_facturas_changed",
				GTK_SIGNAL_FUNC (on_invoice_type_combo_entry_changed));

      glade_xml_signal_connect (xml, "on_button_generate_cuaderno58_clicked",
				GTK_SIGNAL_FUNC (on_generate_cuaderno58_button_clicked));

      glade_xml_signal_connect (xml, "on_lista_facturas_button_press_event",
				GTK_SIGNAL_FUNC (on_invoice_list_button_press_event));
      glade_xml_signal_connect (xml, "on_toolbar_button_press_event",
				GTK_SIGNAL_FUNC (on_toolbar_button_press_event));
      glade_xml_signal_connect (xml, "on_button_devolucion_clicked",
				GTK_SIGNAL_FUNC (on_repayment_button_clicked));


            
      fill_shop_combo();
      s_name = u_parameter_manager_get_parameter ("tienda");
      if (s_name!=NULL) {
	  gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget (xml, "combo_entry_tienda_gestion_facturas")), s_name);
      }
      g_free(s_name);

      date_dateedit = glade_xml_get_widget (xml, "dateedit_vencimiento_gestion_facturas");
      gnome_date_edit_set_time(GNOME_DATE_EDIT(date_dateedit), expiry_date);
      init_date_dateedit = glade_xml_get_widget (xml, "dateedit_desde_gestion_facturas");
      gnome_date_edit_set_time(GNOME_DATE_EDIT(init_date_dateedit), init_date);

      fill_invoice_type_combo();
      fill_invoice_state_combo();
      fill_company_combo();
      fill_payments_combo();
#ifdef IS_SHOP
      gtk_widget_set_sensitive(glade_xml_get_widget (xml, "combo_tienda_gestion_facturas"),FALSE);
      gtk_widget_hide(glade_xml_get_widget(xml,"checkbutton_iva_gestion_facturas"));
#endif
      gtk_widget_hide(glade_xml_get_widget(xml,"generate_cuaderno58"));
      gtk_widget_hide(glade_xml_get_widget(xml,"button_devolucion"));


      find_invoices=dialog;

    } else {
	date_dateedit = glade_xml_get_widget (xml, "dateedit_vencimiento_gestion_facturas");
	gnome_date_edit_set_time(GNOME_DATE_EDIT(date_dateedit), expiry_date);
	init_date_dateedit = glade_xml_get_widget (xml, "dateedit_desde_gestion_facturas");
	gnome_date_edit_set_time(GNOME_DATE_EDIT(init_date_dateedit), init_date);
	if (invoices!=NULL) {
	    if (invoices[0]!=NULL) {
		m_invoice_clear_array (invoices);
		invoices=NULL;
                payment_notes =NULL;
		clean_screen();
	    }
	} else if (payment_notes!=NULL) {
	    if (payment_notes[0]!=NULL) {
		m_payment_note_clear_array (payment_notes);
		invoices=NULL;
                payment_notes =NULL;
		clean_screen();
	    }
	}
    }

  first = FALSE;
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(xml,"checkbutton_iva_gestion_facturas")),TRUE);
  return find_invoices;
  
}

static gint on_invoice_list_button_press_event(GtkWidget *widget,
							 GdkEventButton *event, gpointer user_data) {


    if (invoices!=NULL) {
	GtkWidget * menu_item1 = gtk_menu_item_new_with_label(_("Select all the lines"));
	GtkWidget * menu_item2 = gtk_menu_item_new_with_label(_("Unselect all the lines"));
	GtkWidget * menu_item3 = gtk_menu_item_new_with_label(_("Pass invoices in management to paid"));
	GtkWidget * menu_item4 = gtk_menu_item_new_with_label(_("Pass paid invoices to not paid"));

	GtkWidget * menu = gtk_menu_new();
	GdkEventButton *event_button;


	gtk_menu_item_activate(GTK_MENU_ITEM(menu_item1));
	gtk_menu_item_activate(GTK_MENU_ITEM(menu_item2));
	gtk_menu_item_activate(GTK_MENU_ITEM(menu_item3));
	gtk_menu_item_activate(GTK_MENU_ITEM(menu_item4));

	gtk_menu_set_title(GTK_MENU(menu),_("Select the command"));
	gtk_menu_prepend(GTK_MENU(menu),menu_item1);
	gtk_menu_prepend(GTK_MENU(menu),menu_item2);
	gtk_menu_prepend(GTK_MENU(menu),menu_item3);
	gtk_menu_prepend(GTK_MENU(menu),menu_item4);

	gtk_widget_show(menu_item1);
	gtk_widget_show(menu_item2);
	gtk_widget_show(menu_item3);
	gtk_widget_show(menu_item4);

	gtk_signal_connect_object(GTK_OBJECT(menu_item3), "activate",
				  GTK_SIGNAL_FUNC (on_activate_managed_to_paids), NULL);
	gtk_signal_connect_object(GTK_OBJECT(menu_item4), "activate",
				  GTK_SIGNAL_FUNC (on_activate_paids_to_managed), NULL);
	gtk_signal_connect_object(GTK_OBJECT(menu_item1), "activate",
				  GTK_SIGNAL_FUNC (on_activate_selection), NULL);
	gtk_signal_connect_object(GTK_OBJECT(menu_item2), "activate",
				  GTK_SIGNAL_FUNC (on_activate_unselection), NULL);

	if (event->type == GDK_BUTTON_PRESS)
	{
	    event_button = (GdkEventButton *) event;
	    if (event_button->button == 3)
	    {

		gtk_menu_popup (GTK_MENU(menu), NULL, NULL, NULL, NULL,
				event_button->button, event_button->time);
		return TRUE;
	    }
	}
    }
    return FALSE;
}


	
static void on_activate_paids_to_managed(GtkMenuItem *menuitem,
				   gpointer user_data) {
  c_util_open_question_dialog(_("You are going to pass to not pait all the selected invoices"),
		       (*on_ok_paids_to_managed));
}

static void on_ok_paids_to_managed(gint reply,
				   gpointer user_data) {
  if (!reply) {
    GArray * selected;
    GtkCList * list;
    
    list = GTK_CLIST(glade_xml_get_widget (xml, "lista_facturas"));
    selected = c_util_get_selected_rows_from_clist (list);
  
    if (selected->len>=1) {
	m_invoice_set_state_selected  (invoices, selected, D_INVOICE_PAID, D_INVOICE_NOT_PAID);
	gtk_clist_clear(GTK_CLIST(list));
	fill_invoice_list(GTK_CLIST(list),invoices);
    }
    else {
      c_util_open_error_window (_("Select at least one invoice to process"));
    }
  }
}


static void on_activate_managed_to_paids(GtkMenuItem *menuitem,
				   gpointer user_data) {
  c_util_open_question_dialog(_("You are going to pass to paid all the invoices that have been selected and are in management"), 
		       (*on_ok_managed_to_paids));
}

static void on_ok_managed_to_paids(gint reply,
				   gpointer user_data) {
  if (!reply) {
    GArray * selected;
    GtkCList * list;
    
    list = GTK_CLIST(glade_xml_get_widget (xml, "lista_facturas"));
    selected = c_util_get_selected_rows_from_clist (list);
  
    if (selected->len>=1) {
	m_invoice_set_state_selected  (invoices, selected, D_INVOICE_MANAGING, D_INVOICE_PAID);
	gtk_clist_clear(GTK_CLIST(list));
	fill_invoice_list(GTK_CLIST(list),invoices);
    }
    else {
      c_util_open_error_window (_("Select at least one invoice to process"));
    }
  }
}

static void on_activate_selection(GtkMenuItem *menuitem,
				   gpointer user_data) {
    GtkWidget * clist = glade_xml_get_widget(xml,"lista_facturas");

    gtk_clist_select_all(GTK_CLIST(clist));
}

static void on_activate_unselection(GtkMenuItem *menuitem,
				   gpointer user_data) {
    GtkWidget * clist = glade_xml_get_widget(xml,"lista_facturas");

    gtk_clist_unselect_all(GTK_CLIST(clist));
}


static void fill_invoice_list(GtkCList * list,Invoice ** invoice_array) {
    gchar * content=NULL;
    gint index=0;
    gchar *empty_row[]={"","","","","",""};
    gdouble total_sum=0;

    if (invoice_array!=NULL) {
	while (invoice_array[index]!=NULL) {
            Repair * repair;
	    if (index==30) gtk_clist_freeze ((GtkCList *) list);
	    gtk_clist_append ((GtkCList *) list,empty_row);
	    if ((d_invoice_get_invoice_number (invoice_array[index]))!=ULONG_MAX) {
		content = g_strdup_printf("%lu",d_invoice_get_invoice_number (invoice_array[index]));
		gtk_clist_set_text((GtkCList *) list, index, C_INVOICE_NUMBER_COLUMN, content);
		g_free(content);
		content = NULL;
	    }
	    content = d_shop_get_group_code (d_invoice_get_shop (invoice_array[index]));
	    if (content!=NULL) {
		gtk_clist_set_text((GtkCList *) list, index, C_GROUP_CODE_COLUMN, content);
		g_free(content);
		content = NULL;
	    }
            content =d_participant_get_nick (d_invoice_get_participant (
							    invoice_array[index]));
	    if (content!=NULL) {
		gtk_clist_set_text((GtkCList *) list, index, C_CLIENT_COLUMN,content);
		g_free(content);
		content = NULL;

	    }
	    if (d_invoice_get_repair (invoice_array[index])==NULL) {
                repair = d_delivery_payment_get_repair (invoice_array[index]->delivery_payments[0]);
	    } else {
		repair = d_invoice_get_repair (invoice_array[index]);
	    }
            if (d_repair_get_company (repair)!=NULL)
		content = d_participant_get_name (d_company_get_participant (d_repair_get_company (
											repair)));

	    if (content!=NULL) {
		gtk_clist_set_text((GtkCList *) list, index, C_COMPANY_COLUMN,content);
		g_free(content);
		content = NULL;

	    }
	    if (d_invoice_get_state (invoice_array[index])>-1) {
	      if (d_invoice_get_state (invoice_array[index])==D_INVOICE_NOT_ISSUED) {
		content = g_strdup(D_INVOICE_MESSAGE_NOT_ISSUED);
	      } else if (d_invoice_get_state (invoice_array[index])==D_INVOICE_NOT_PAID) {
		content = g_strdup(D_INVOICE_MESSAGE_NOT_PAID);
	      } else if (d_invoice_get_state (invoice_array[index])==D_INVOICE_PAID) {
		content = g_strdup(D_INVOICE_MESSAGE_PAID);
	      } else if (d_invoice_get_state (invoice_array[index])==D_INVOICE_MANAGING) {
		content = g_strdup(D_INVOICE_MESSAGE_MANAGING);
	      }
	      gtk_clist_set_text((GtkCList *) list, index, C_STATE_COLUMN, content);
	      g_free(content);
	      content = NULL;
	      
	    }

	    if (d_invoice_get_total (invoice_array[index])!=G_MINDOUBLE) {
                total_sum = total_sum + d_invoice_get_total (invoice_array[index]);
		content =g_strdup_printf("%.2f",d_invoice_get_total (invoice_array[index]));
		if (content!=NULL) {
		    gtk_clist_set_text((GtkCList *) list, index, C_TOTAL_COLUMN,content);
		    g_free(content);
		    content = NULL;

		}
	    }

	    index++;
	}
    }
    gtk_clist_thaw ((GtkCList *) list);
    content = g_strdup_printf("%.2f",total_sum);
    gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(xml,"entry_total_lista")),content);
    g_free(content);

}

static void fill_payment_note_list_on_clist(GtkCList * list,PaymentNote ** payment_note_array) {
    gchar * content=NULL;
    gint index=0;
    gchar *empty_row[]={"","","","","",""};
    gdouble total_sum=0;

    if (payment_note_array!=NULL) {
	while (payment_note_array[index]!=NULL) {

	    if (index==30) gtk_clist_freeze ((GtkCList *) list);
	    gtk_clist_append ((GtkCList *) list,empty_row);
	    if ((d_payment_note_get_note_code (payment_note_array[index]))!=ULONG_MAX) {
		content = g_strdup_printf("%lu",d_payment_note_get_note_code (payment_note_array[index]));
		gtk_clist_set_text((GtkCList *) list, index, C_INVOICE_NUMBER_COLUMN, content);
		g_free(content);
	    }
	    if (d_shop_get_group_code (d_repair_get_shop (d_payment_note_get_repair_detail (payment_note_array[index])))!=NULL) {
		content = d_shop_get_group_code (d_repair_get_shop (d_payment_note_get_repair_detail (payment_note_array[index])));
		gtk_clist_set_text((GtkCList *) list, index, C_GROUP_CODE_COLUMN, content);
		g_free(content);
	    }
	    if (d_participant_get_nick (d_client_get_participant (d_repair_get_client (d_payment_note_get_repair_detail (
									      payment_note_array[index]
                                                                             ))))!=NULL) {
		content = d_participant_get_nick (d_client_get_participant (d_repair_get_client (d_payment_note_get_repair_detail (
									      payment_note_array[index]
									     ))));
		gtk_clist_set_text((GtkCList *) list, index, C_CLIENT_COLUMN,content);
		g_free(content);
	    }

	    if (d_participant_get_name (d_company_get_participant (
							      d_repair_get_company (d_payment_note_get_repair_detail (payment_note_array[index]))
							     ))!=NULL) {
		content = d_participant_get_name (d_company_get_participant (d_repair_get_company (
											    d_payment_note_get_repair_detail (payment_note_array[index])))
					      );
		gtk_clist_set_text((GtkCList *) list, index, C_COMPANY_COLUMN,content);
		g_free(content);
	    }

            content=g_strdup(D_INVOICE_MESSAGE_PAID);
	    gtk_clist_set_text((GtkCList *) list, index, C_STATE_COLUMN, content);
	    g_free(content);

	    if (d_payment_note_get_total (payment_note_array[index])!=G_MINDOUBLE) {
		total_sum = total_sum + d_payment_note_get_total (payment_note_array[index]);
		content =g_strdup_printf("%.2f",d_payment_note_get_total (payment_note_array[index]));
		if (content!=NULL) {
		    gtk_clist_set_text((GtkCList *) list, index, C_TOTAL_COLUMN,content);
		    g_free(content);
		    content = NULL;

		}
	    }

	    index++;
	}
    }
    gtk_clist_thaw ((GtkCList *) list);
    content = g_strdup_printf("%.2f",total_sum);
    gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(xml,"entry_total_lista")),content);
    g_free(content);

}



static void fill_invoice_type_combo() {
    GtkWidget * combo=glade_xml_get_widget(xml,"combo_tipo_factura_gestion_facturas");
    gchar * temp;
#ifndef IS_SHOP
    temp = g_strdup(D_INVOICE_MESSAGE);
    c_util_set_code_and_name_on_combo (D_INVOICE_CODE,temp,GTK_COMBO(combo));
    g_free(temp);
#endif
    temp = g_strdup(D_PAYMENT_NOTE);
    c_util_set_code_and_name_on_combo (D_PAYMENT_NOTE_CODE,temp,GTK_COMBO(combo));
    g_free(temp);
#ifndef IS_SHOP
    temp = g_strdup(D_INVOICE_SPLIT_INVOICE);
    c_util_set_code_and_name_on_combo (D_INVOICE_SPLIT_INVOICE_CODE,temp,GTK_COMBO(combo));
    g_free(temp);
#endif

}

static void fill_invoice_state_combo() {
    GtkWidget * combo=glade_xml_get_widget(xml,"combo_estado_factura_gestion_facturas");
    gchar * temp;

    temp = g_strdup(N_("All the states"));
    c_util_set_code_and_name_on_combo (C_FIND_ALL_STATES,_(temp),GTK_COMBO(combo));
    g_free(temp);
#ifndef IS_SHOP
    temp = g_strdup(D_INVOICE_MESSAGE_NOT_ISSUED);
    c_util_set_code_and_name_on_combo (D_INVOICE_NOT_ISSUED,temp,GTK_COMBO(combo));
    g_free(temp);
    temp = g_strdup(D_INVOICE_MESSAGE_NOT_PAID);
    c_util_set_code_and_name_on_combo (D_INVOICE_NOT_PAID,temp,GTK_COMBO(combo));
    g_free(temp);
    temp = g_strdup(D_INVOICE_MESSAGE_PAID);
    c_util_set_code_and_name_on_combo (D_INVOICE_PAID,temp,GTK_COMBO(combo));
    g_free(temp);
    temp = g_strdup(D_INVOICE_MESSAGE_MANAGING);
    c_util_set_code_and_name_on_combo (D_INVOICE_MANAGING,temp,GTK_COMBO(combo));
    g_free(temp);
#endif

}

static void on_invoice_type_combo_entry_changed(GtkEntry *entry,
								 gpointer user_data) {
    GtkWidget * label = glade_xml_get_widget(xml,"label_numero_gestion_facturas");
    gchar * temp =g_strdup(gtk_entry_get_text(entry));
    GtkWidget * state_combo = glade_xml_get_widget(xml,"combo_estado_factura_gestion_facturas");
    if ((!strcmp(temp,D_INVOICE_MESSAGE)) || (!strcmp(temp,D_INVOICE_SPLIT_INVOICE))) {
	gtk_label_set_text(GTK_LABEL(label),_("Invoice number"));
        if (!GTK_WIDGET_SENSITIVE(state_combo)) gtk_widget_set_sensitive(state_combo,TRUE);
    } else {
        gtk_label_set_text(GTK_LABEL(label),_("Payment note number"));
        if GTK_WIDGET_SENSITIVE(state_combo) gtk_widget_set_sensitive(state_combo,FALSE);
    }
    do_search();
    g_free(temp);

}

static void fill_shop_combo() {
    GtkWidget * combo;
    Shop ** shops;
    gchar * temp;

    combo = glade_xml_get_widget (xml, "combo_tienda_gestion_facturas");

    temp = g_strdup(_("All the shops"));
    c_util_set_code_and_name_on_combo (C_ALL_SHOPS_CODE,temp,GTK_COMBO(combo));
    shops = m_shop_find_all ();
    g_free(temp);

    c_util_fill_combo_with_shops (combo,shops);

}

static void fill_company_combo() {
    GtkWidget * combo;
    Company ** companies;
    gchar * temp;

    combo = glade_xml_get_widget (xml, "combo_compania_gestion_facturas");

    temp = g_strdup(_("Without company filter"));
    c_util_set_code_and_name_on_combo (C_COMPANY_FILTER_NONE,temp,GTK_COMBO(combo));
    companies= m_company_find_by_code (ULONG_MAX);
    g_free(temp);

    c_util_fill_combo_with_companies (combo,companies);

}

static void on_delete_button_clicked(GtkButton *button, gpointer data) {
    clean_screen();
}
static void on_accept_button_clicked(GtkButton *button, gpointer data) {
    view_invoice();
}

static void on_print_button_clicked(GtkButton *button, gpointer data) {
    Invoice * invoice_total= d_invoice_create_empty ();

    gchar * temp_total = g_strdup(gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(xml,"entry_total_lista"))));
    d_invoice_set_total (invoice_total,atof(temp_total));

    if (invoices != NULL) {
	Listing list;
        Listing * listing = (Listing *) g_malloc0(sizeof(Listing));
        gulong    rows = 0;

	list.content_title = d_invoice_get_content_title ();
	list.listing_title = d_invoice_get_listing_title ();
	list.content       = d_invoice_array_to_content (invoices,invoice_total, &rows);
	list.width_of_cols = d_invoice_get_listing_columns_width ();
	list.alignment_of_cols = d_invoice_get_listing_columns_alignment ();
	list.n_rows        = rows;
	list.n_columns     = d_invoice_get_content_size ();

        // Fixme!!!
        // Hardcoded -> get this from a combo
	list.orientation   = 4;
	list.type_of_paper = 2;

	*listing = list;
        p_listing_wrap_print (listing);
    } else {
	c_util_open_error_window (_("Search the invoices to print"));
	}
    d_invoice_clear (invoice_total);
}

static void on_search_button_clicked(GtkButton *button, gpointer data) {
    do_search();
}

static void on_ok_do_direct_debit(gint reply,
				  gpointer user_data) {
    GArray * selected;
    GtkCList * list;

    if (!reply) {
	list = GTK_CLIST(glade_xml_get_widget (xml, "lista_facturas"));
	selected = c_util_get_selected_rows_from_clist (list);

	if (selected->len>=1) {
	    i_book58_generate_from_invoice_list (invoices, selected);
	    gtk_clist_clear(GTK_CLIST(list));

	    fill_invoice_list(GTK_CLIST(list),invoices);
	}
	else {
	    c_util_open_error_window (_("Select at least one invoice to process"));
	}
	g_array_free(selected,TRUE);
    }

}


static void on_generate_cuaderno58_button_clicked(GtkButton *button, gpointer data) {
    GArray * selected;
    gint do_continue=0;
    GtkCList * list;
    guint i;
    guint index;
    gboolean exist_error=FALSE;
    gchar * message =g_strconcat(_("There are invoices that have been paid by direct debit and have these errors:\n\n"),
				 _("     Code     | Description\n"),NULL);
    gchar * message_aux=NULL;
    gulong invoice_number_error=ULONG_MAX;

    list = GTK_CLIST(glade_xml_get_widget (xml, "lista_facturas"));
    selected = c_util_get_selected_rows_from_clist (list);

    for (i=0; i< selected->len; i++) {
	gulong invoice_code ;
	gulong enterprise_code ;
	gulong shop_code ;
	gint state;
	gint number;

	gint actual_instalment;
	index  = g_array_index(selected,gint,i); /* Index takes only the selected invoices */

	invoice_code=d_invoice_get_invoice_code (invoices[index]);
        enterprise_code=d_participant_get_code (d_enterprise_get_participant (d_invoice_get_enterprise (invoices[index])));
	shop_code=d_participant_get_code (d_shop_get_participant (d_invoice_get_shop (invoices[index])));

	number = m_payment_find_number_of_operation (d_payment_get_payment_code (d_invoice_get_payment (invoices[index])),
						      shop_code,enterprise_code);
        actual_instalment = m_direct_debit_find_number_of_instalment (invoice_code,shop_code,enterprise_code);
	if (number == actual_instalment) {
            gint limit;
	    state = m_direct_debit_get_state_from_invoice (invoice_code,enterprise_code,shop_code);
	    if (number>=1) {
                limit = D_DIRECT_DEBIT_PAID;
	    } else {
                limit = D_DIRECT_DEBIT_GENERATED;
	    }
	    if ((state>=D_DIRECT_DEBIT_TOTAL_ZERO) && (state <limit) && (do_continue<2)) {
		gchar * aux=d_direct_debit_get_state_message (state);
		gchar * invoice_code_char = g_strdup_printf("%010lu",invoice_code);
		message_aux = g_strconcat(message,invoice_code_char,"   ",aux,"\n",NULL);
		g_free(aux);
		g_free(message);
		message = message_aux;
		exist_error = TRUE;
		message_aux=NULL;
		do_continue = 1;
	    } else if (state>=D_DIRECT_DEBIT_GENERATED) {
		do_continue = 2;
	    } else if (state<D_DIRECT_DEBIT_TOTAL_ZERO) {
		do_continue = 1;
	    }
	} else {
            invoice_number_error = d_invoice_get_invoice_number (invoices[index]);
            do_continue = 3;
	}


    }

    message_aux = g_strconcat(message,_("Are you sure you want to continue?"),NULL);
    g_free(message);
    message = message_aux;

    if (do_continue==1) {
        if (exist_error !=FALSE)
	    c_util_open_question_dialog(message,
				 (*on_ok_do_direct_debit));
	else
	    on_ok_do_direct_debit(0,NULL);

    } else if (do_continue == 3) {
        gchar * invoicce_number_char = g_strdup_printf("%lu",invoice_number_error);
	gchar * mes = g_strconcat(_("There is, at least, one invoice on an instalment plan (the "),invoicce_number_char,
				  _(") and one of the instalments is in management, you have to pay it before"),NULL);
	c_util_open_error_window (mes);
	g_free(mes);
        g_free(invoicce_number_char);
    } else {
        c_util_open_error_window (_("You can't pay by direct debit an invoice that is paid or in management"));
    }

    g_free(message);
    g_array_free(selected,TRUE);

}


static void clean_screen() {

    GtkWidget *vbox;
    gchar *s_name =NULL;
    time_t expiry_date = time(NULL) + 86400;
    GtkWidget * date_dateedit= NULL;
    time_t init_date = time(NULL) - 86400*30;
    GtkWidget * init_date_dateedit= NULL;

    vbox = GTK_WIDGET(glade_xml_get_widget(xml, widget_name));

    gtk_container_foreach(GTK_CONTAINER(vbox),&c_util_clean_widget,NULL);
      s_name = u_parameter_manager_get_parameter ("tienda");
      if (s_name!=NULL) {
	  gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget (xml, "combo_entry_tienda_gestion_facturas")), s_name);
      }
      g_free(s_name);

      date_dateedit = glade_xml_get_widget (xml, "dateedit_vencimiento_gestion_facturas");
      gnome_date_edit_set_time(GNOME_DATE_EDIT(date_dateedit), expiry_date);
      init_date_dateedit = glade_xml_get_widget (xml, "dateedit_desde_gestion_facturas");
      gnome_date_edit_set_time(GNOME_DATE_EDIT(init_date_dateedit), init_date);


}

static void on_press_key(GtkWidget *widget,
					   GdkEventKey *event,
					   gpointer user_data) {
    if (event->keyval==GDK_Return)
	do_search();
                                           
}

static void on_invoice_list_select_row (GtkCList        *list,
					  gint             row,
					  gint             column,
					  GdkEvent        *event,
					  gpointer         user_data) {

    if (event!=NULL) {
	if (event->type == GDK_2BUTTON_PRESS)
	{
	    view_invoice();
	}
    }
}

static void do_search() {
    if (first != TRUE) {

	GtkWidget * wid =NULL;
	gchar * temp;
	gchar ** stop;
	gulong invoice_state;
	ConditionFindInvoice * con;
	time_t expiry_date;
	time_t init_date;
	gint no_error = 1;
	GtkWidget * clist = glade_xml_get_widget(xml,"lista_facturas");
	GtkWidget * window;
	GdkCursor * cursor;
	window = lookup_widget(GTK_WIDGET(clist),"app1");
	cursor = gdk_cursor_new(GDK_WATCH);
	gdk_window_set_cursor(GTK_WIDGET(window)->window, cursor);
	gdk_cursor_destroy(cursor);

	while (gtk_events_pending())
	    gtk_main_iteration();

	con = (ConditionFindInvoice * ) g_malloc0(sizeof(ConditionFindInvoice));
	stop = (gchar **) g_malloc0(sizeof(gchar *));
	wid = glade_xml_get_widget(xml,"combo_estado_factura_gestion_facturas");

	invoice_state = c_util_get_code_from_combo (wid);
	if (invoice_state!=C_FIND_ALL_STATES) {
	    con -> invoice_state = invoice_state;
	} else {
	    con -> invoice_state = -1;
	}
	wid = glade_xml_get_widget(xml,"dateedit_vencimiento_gestion_facturas");
	expiry_date = gnome_date_edit_get_date(GNOME_DATE_EDIT(wid));
	/*I DON'T SURE THE COMPARATION HAS TO BE THE FOLLOWING.*/
	//expiry_date_aux=time(NULL);
	if (expiry_date>-1) {
	    con -> ending_issue_date = (time_t *) g_malloc0(sizeof(time_t));
	    *(con -> ending_issue_date) = expiry_date;
	} else {
	    con->ending_issue_date=NULL;
	}

	wid = glade_xml_get_widget(xml,"dateedit_desde_gestion_facturas");
	init_date = gnome_date_edit_get_date(GNOME_DATE_EDIT(wid));
	if (init_date>-1) {
	    con -> initial_issue_date = (time_t *) g_malloc0(sizeof(time_t));
	    *(con -> initial_issue_date) = init_date;
	} else {
	    con->initial_issue_date=NULL;
	}

	wid = glade_xml_get_widget(xml,"combo_tipo_factura_gestion_facturas");
	con->type= c_util_get_code_from_combo (wid);

	wid = glade_xml_get_widget(xml,"combo_tienda_gestion_facturas");
	con->shop_code= c_util_get_code_from_combo (wid);
	con->enterprise_code =  ULONG_MAX;

	wid = glade_xml_get_widget(xml,"combo_pagos_gestion_facturas");
	con->payment_type= c_util_get_code_from_combo (wid);

	wid = glade_xml_get_widget(xml,"spinbutton_cantidad_gestion_facturas");
	gtk_spin_button_update(GTK_SPIN_BUTTON(wid));
	con->amount = gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(wid));
	wid = glade_xml_get_widget(xml,"checkbutton_iva_gestion_facturas");
	con->vat = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wid));

	wid = glade_xml_get_widget(xml,"entry_matricula_gestion_facturas");
	con ->license_number = g_strdup(gtk_entry_get_text(GTK_ENTRY(wid)));

	if (con->type!=D_PAYMENT_NOTE_CODE) {
	    wid = glade_xml_get_widget(xml,"entry_siniestro");
	    con ->accident_reference = g_strdup(gtk_entry_get_text(GTK_ENTRY(wid)));
	}

	wid = glade_xml_get_widget(xml,"combo_compania_gestion_facturas");
	con->company_code= c_util_get_code_from_combo (wid);

	wid = glade_xml_get_widget(xml,"entry_nombre_cliente_gestion_facturas");
	con ->client_name = g_strdup(gtk_entry_get_text(GTK_ENTRY(wid)));
	wid = glade_xml_get_widget(xml,"entry_cif_gestion_facturas");
	temp = g_strdup(gtk_entry_get_text(GTK_ENTRY(wid)));
	if (strcmp(temp,"")) {
	    no_error = u_person_legal_id_is_correct (temp);
	    con ->client_legal_id = g_strdup(temp);
	    if (no_error == 0) {
		gchar * enterprise_legal_id;
		gchar * enterprise_legal_id_aux;
		enterprise_legal_id_aux = (gchar *) g_malloc0(sizeof(gchar)*8);
		enterprise_legal_id = (gchar *) g_malloc0(sizeof(gchar)*9);
		enterprise_legal_id = g_strdup(&temp[1]);
		enterprise_legal_id_aux = strncpy(enterprise_legal_id_aux,enterprise_legal_id,8);
		no_error = u_enterprise_legal_id_is_correct (strtoul(enterprise_legal_id_aux,stop,10));
		g_free(enterprise_legal_id);
		g_free(enterprise_legal_id_aux);
		if (no_error==0) {
		    c_util_open_error_window (_("Tax code incorrect"));
		}
	    }
	}
	g_free(temp);
	wid = glade_xml_get_widget(xml,"entry_numero_factura_gestion_facturas");
	temp = g_strdup(gtk_entry_get_text(GTK_ENTRY(wid)));
	con->invoice_number = strtoul(temp,stop,10);
	if (strcmp(*stop,"")) {
	    no_error = 0;
	    c_util_open_error_window (_("This field should have a number"));
	}
	g_free(temp);
	g_free(stop);

	if (invoices!=NULL) {
	    if (invoices[0]!=NULL) {
		m_invoice_clear_array (invoices);
		invoices =NULL;
		payment_notes = NULL;
		gtk_clist_clear(GTK_CLIST(clist));
	    }
	} else if (payment_notes!=NULL){
	    if (payment_notes[0]!=NULL) {
		m_payment_note_clear_array (payment_notes);
		payment_notes =NULL;
		invoices = NULL;
		gtk_clist_clear(GTK_CLIST(clist));
	    }
	}
	if ((con->type == D_INVOICE_CODE) || (con->type == D_INVOICE_SPLIT_INVOICE_CODE)) {
	    if (no_error == 1) {
		invoices = m_invoice_find_by_condition (con);
		fill_invoice_list(GTK_CLIST(clist),invoices);
	    }
	} else if (con->type==D_PAYMENT_NOTE_CODE) {
	    if (no_error == 1) {
		payment_notes = m_payment_note_find_by_condition (con);
		fill_payment_note_list_on_clist(GTK_CLIST(clist),payment_notes);
	    }
	}
	if (con->ending_issue_date!=NULL) g_free(con->ending_issue_date);
	if (con->initial_issue_date!=NULL) g_free(con->initial_issue_date);
	if (con->client_name!=NULL) g_free(con->client_name);
	if (con->client_legal_id!=NULL) g_free(con->client_legal_id);
	if (con->license_number!=NULL) g_free(con->license_number);
	cursor = gdk_cursor_new(GDK_LEFT_PTR);
	gdk_window_set_cursor(GTK_WIDGET(window)->window, cursor);
	gdk_cursor_destroy(cursor);

	g_free(con);
    }
}

static InvoiceId * obtain_invoice_id_from_list(gint value,GtkCList * list) {
    InvoiceId * invoice_id = (InvoiceId *) g_malloc0(sizeof(InvoiceId));

    
    
    invoice_id -> invoice_number = g_strdup_printf("%lu",d_invoice_get_invoice_code (invoices[value]));
    invoice_id -> enterprise_code = g_strdup_printf("%lu",
						 d_participant_get_code (d_enterprise_get_participant (d_invoice_get_enterprise (invoices[value])))
						);
    invoice_id -> shop_code =  g_strdup_printf("%lu",
						 d_participant_get_code (d_shop_get_participant (d_invoice_get_shop (invoices[value])))
						);

    return invoice_id;

}

static PaymentNoteId * obtain_payment_note_id_from_list(gint value,GtkCList * list) {
    PaymentNoteId * payment_note_id = (PaymentNoteId *) g_malloc0(sizeof(PaymentNoteId));

    
    
    payment_note_id -> payment_note_code = g_strdup_printf("%lu",d_payment_note_get_note_code (payment_notes[value]));
    payment_note_id -> enterprise_code = g_strdup_printf("%lu",
						 d_participant_get_code (d_enterprise_get_participant (d_repair_get_enterprise (d_payment_note_get_repair_detail (payment_notes[value]))))
						);
    payment_note_id -> shop_code =  g_strdup_printf("%lu",
						 d_participant_get_code (d_shop_get_participant (d_repair_get_shop (d_payment_note_get_repair_detail (payment_notes[value]))))
						);

    return payment_note_id;

}


static void view_invoice() {
    GArray * selected;
    gint value;
    GtkCList * list = GTK_CLIST(glade_xml_get_widget (xml, "lista_facturas"));
    GtkWidget * type_invoice_control = glade_xml_get_widget (xml, "combo_tipo_factura_gestion_facturas");
    gulong code_combo = c_util_get_code_from_combo (type_invoice_control);

    selected = c_util_get_selected_rows_from_clist (list);

    if (selected->len==1) {

	value = g_array_index(selected,gint,0);

	if ((code_combo==D_INVOICE_CODE) || (code_combo==D_INVOICE_SPLIT_INVOICE_CODE)) {
	    InvoiceId * invoice_id = (InvoiceId *)g_malloc0(sizeof(InvoiceId));

	    invoice_id = obtain_invoice_id_from_list(value,list);

	    g_array_free(selected,TRUE);

	    i_shortcut_change_dialog (GTK_WIDGET(list),1,0);

	    c_invoice_load(invoice_id);
	    if (invoice_id !=NULL) {
		if (invoice_id->shop_code!=NULL)
		    g_free(invoice_id->shop_code);
		if (invoice_id->enterprise_code!=NULL)
		    g_free(invoice_id->enterprise_code);
		if (invoice_id->invoice_number!=NULL)
		g_free(invoice_id->invoice_number);
		g_free(invoice_id);
	    }
	} else {
	    PaymentNoteId * payment_note_id= (PaymentNoteId *)g_malloc0(sizeof(PaymentNoteId));

            payment_note_id = obtain_payment_note_id_from_list(value,list);

	    g_array_free(selected,TRUE);

	    i_shortcut_change_dialog (GTK_WIDGET(list),1,0);

            c_invoice_load_payment_note(payment_note_id);
	}
    } else {
	c_util_open_error_window (_("Select an invoice"));
    }

}

static void fill_payments_combo(void) {
    GtkWidget * combo=glade_xml_get_widget(xml,"combo_pagos_gestion_facturas");
    gchar * temp;

#ifndef IS_SHOP
    temp = g_strdup(_("All the types of payments"));
    c_util_set_code_and_name_on_combo (ULONG_MAX,temp,GTK_COMBO(combo));
    g_free(temp);
    temp = g_strdup(D_PAYMENT_MESSAGE_TRANSFER);
    c_util_set_code_and_name_on_combo (D_PAYMENT_TRANSFER_CODE,temp,GTK_COMBO(combo));
    g_free(temp);
#endif

    temp = g_strdup(D_PAYMENT_MESSAGE_IN_CASH);
    c_util_set_code_and_name_on_combo (D_PAYMENT_IN_CASH,temp,GTK_COMBO(combo));
    g_free(temp);
#ifndef IS_SHOP
    temp = g_strdup(D_PAYMENT_MESSAGE_CHEQUE);
    c_util_set_code_and_name_on_combo (D_PAYMENT_CHEQUE,temp,GTK_COMBO(combo));
    g_free(temp);
    temp = g_strdup(D_PAYMENT_MESSAGE_DIRECT_DEBIT);
    c_util_set_code_and_name_on_combo (D_PAYMENT_DIRECT_DEBIT_CODE,temp,GTK_COMBO(combo));
    g_free(temp);
    temp = g_strdup(D_PAYMENT_MESSAGE_CREDIT_CARD);
    c_util_set_code_and_name_on_combo (D_PAYMENT_CREDIT_CARD,temp,GTK_COMBO(combo));
    g_free(temp);
#endif


}

static void proccess_filename(GtkFileSelection *selector, gpointer user_data) {
    GtkWidget * file_selector = (GtkWidget *) user_data;
    /*no liberar selected_filename*/
    gchar * selected_filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION(file_selector));

    gchar * error = i_book58_proccess_devolution_file (selected_filename);

    if (error!=NULL) {
	c_util_open_error_window (error);
        g_free(error);
    } else {
	c_util_open_info_window (_("Process finished correctly"));
    }

}
static void on_repayment_button_clicked(GtkButton *button, gpointer data) {
    GtkWidget * file_selector = gtk_file_selection_new(_("Select the files"));
    gchar * path = g_strconcat(AA_DATA_DIR,"/",NULL);

    gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION(file_selector)->ok_button),
			"clicked", GTK_SIGNAL_FUNC (proccess_filename), (gpointer) file_selector);
     
   
    gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION(file_selector)->ok_button),
			       "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy),
			       (gpointer) file_selector);

   gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION(file_selector)->cancel_button),
			      "clicked", GTK_SIGNAL_FUNC (gtk_widget_destroy),
			      (gpointer) file_selector);

   gtk_file_selection_set_filename(GTK_FILE_SELECTION(file_selector),path);
   g_free(path);
   gtk_widget_show (file_selector);
}

static gint on_toolbar_button_press_event(GtkWidget *widget,
					  GdkEventButton *event, gpointer user_data) {

#ifndef IS_SHOP
    GtkWidget * menu_item1 = gtk_menu_item_new_with_label(_("Show cuaderno 58 buttons"));
    GtkWidget * menu_item2 = gtk_menu_item_new_with_label(_("Hide cuaderno 58 buttons"));

    GtkWidget * menu = gtk_menu_new();
    GdkEventButton *event_button;

    gtk_menu_item_activate(GTK_MENU_ITEM(menu_item1));
    gtk_menu_item_activate(GTK_MENU_ITEM(menu_item2));

    gtk_menu_set_title(GTK_MENU(menu),_("Buttons selection menu"));
    gtk_menu_prepend(GTK_MENU(menu),menu_item1);
    gtk_menu_prepend(GTK_MENU(menu),menu_item2);

    if (!GTK_WIDGET_VISIBLE(glade_xml_get_widget(xml,"button_devolucion")))
	gtk_widget_show(menu_item1);
    else
	gtk_widget_show(menu_item2);

    gtk_signal_connect_object(GTK_OBJECT(menu_item1), "activate",
       		       GTK_SIGNAL_FUNC (on_activate_buttons), NULL);
    gtk_signal_connect_object(GTK_OBJECT(menu_item2), "activate",
       		       GTK_SIGNAL_FUNC (on_hide_buttons), NULL);

    if (event->type == GDK_BUTTON_PRESS)
    {
	event_button = (GdkEventButton *) event;
	if (event_button->button == 3)
	{

	    gtk_menu_popup (GTK_MENU(menu), NULL, NULL, NULL, NULL,
			    event_button->button, event_button->time);
	    return TRUE;
	}
    }
#endif

    return FALSE;

}

static void on_activate_buttons(GtkMenuItem *menuitem,
				gpointer user_data) {

    gtk_widget_show(glade_xml_get_widget(xml,"generate_cuaderno58"));
    gtk_widget_show(glade_xml_get_widget(xml,"button_devolucion"));

}

static void on_hide_buttons(GtkMenuItem *menuitem,
			      gpointer user_data) {

    gtk_widget_hide(glade_xml_get_widget(xml,"generate_cuaderno58"));
    gtk_widget_hide(glade_xml_get_widget(xml,"button_devolucion"));
    gtk_widget_hide(glade_xml_get_widget(xml,"toolbar7"));
    gtk_widget_show(glade_xml_get_widget(xml,"toolbar7"));

}

static void on_by_totals_button_clicked(GtkButton *button, gpointer data) {
    Invoice * invoice_total= d_invoice_create_empty ();

    gchar * temp_total = g_strdup(gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(xml,"entry_total_lista"))));
    d_invoice_set_total (invoice_total,atof(temp_total));

    if (invoices != NULL) {
	Listing list;
        Listing * listing = (Listing *) g_malloc0(sizeof(Listing));
        gulong    rows = 0;

	list.content_title = d_invoice_get_content_title ();
	list.listing_title = d_invoice_get_listing_title ();
	list.content       = d_invoice_array_to_content_by_company (invoices,invoice_total, &rows);
	list.width_of_cols = d_invoice_get_listing_columns_width ();
	list.alignment_of_cols = d_invoice_get_listing_columns_alignment ();
	list.n_rows        = rows;
	list.n_columns     = d_invoice_get_content_size ();

        // Fixme!!!
        // Hardcoded -> get this from a combo
	list.orientation   = 4;
	list.type_of_paper = 2;

	*listing = list;
        p_listing_wrap_print (listing);
    } else {
	c_util_open_error_window (_("Search the invoices to print"));
	}
    d_invoice_clear (invoice_total);
}


