
/* find_stocked_pieces_control.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include <gnome.h>
#include <glade/glade.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <glib.h>
#include <time.h>

#include "a-shortcut.h"
#include "support.h"
#include "repairs_control.h"
#include "stock_control.h"
#include "order_control.h"
#include "find_stocked_pieces_control.h"
#include "find_stocked_pieces_model.h"
#include "supplier_model.h"
#include "stock.h"
#include "warehouse_piece_history.h"
#include "util.h"
#include "parameter_manager.h"
#include "control_util.h"
#include "warehouse_model.h"
#include "find_product_cost.h"
#include "enterprise_model.h"
#include "piece_control.h"
#include "find_pieces_model.h"

#define C_EUROCODE_COLUMN             0
#define C_FAMILY_COLUMN              1
#define C_DESCRIPTION_COLUMN          2
#define C_STOCK_COLUMN                3
#define C_WIDTH_COLUMN                4
#define C_LONG_COLUMN                5
#define C_VAT_COLUMN                  6
#define C_COST_PRICE_COLUMN      7
#define C_PRICE_COLUMN               8
#define C_SUPPLIER_NONE                  0
#define C_ALL_ENTERPRISES              0
#define C_ALL_WAREHOUSES_OF_ENTERPRISE 0


static GtkWidget *find_pieces=NULL;
static GladeXML *xml = NULL;
static gchar dialog_file[]="/src/interface/glade/store_view.glade";
static gchar widget_name[]="vbox-listado-piezas";
static Stock ** stocked_pieces_array;
static gdouble size_of_array;
static void fill_CList(GtkCList * list,gulong size,gdouble * total_entry,gboolean show_stocked);
static void move_stocked_pieces(GtkCList * list);
static void fill_total_entry(GtkWidget * entry,gdouble * total);
static void on_stock_checkbutton_toggled(GtkToggleButton *togglebutton, gpointer user_data);

static void on_search_button_clicked(GtkButton *button, gpointer data);
static gulong get_array_index_from_pieces_list(GtkCList * clist,gulong index);
static void on_stock_button_clicked(GtkButton *button, gpointer data);
static void on_modify_button_clicked(GtkButton *button, gpointer data);
static void on_delete_button_clicked(GtkButton *button, gpointer data);
static void on_repair_button_clicked(GtkButton *button, gpointer data);
static void on_order_button_clicked(GtkButton *button, gpointer data);
static void on_clean_button_clicked(GtkButton *button, gpointer data);
static void on_cost_price_button_clicked(GtkButton *button, gpointer data);
static void on_press_key(GtkWidget *widget,
				       GdkEventKey *event,
				       gpointer user_data);
static void on_combo_entry_empresa_filtrado_listadopiezas_changed(void);
static void on_combo_entry_almacen_filtrado_listadopiezas_changed(void);
static void obtain_stocked_pieces(void);
static gint obtain_stocked_pieces_ids(GArray * selected, Stock **pieces, gulong *pieces_ids);
static gint obtain_selected_stocked_pieces(GArray * selected, Stock **pieces, Stock **selected_pieces);
static void fill_enterprise_combo(void);
static void fill_warehouse_combo(void);
static void clean_warehouse_combo(void);
static void clean_combos(void);
static void clean_screen(void);
static void view_history_piece();
static void on_clist_listadopiezas_select_row(GtkCList        *list,
					  gint             row,
					  gint             column,
					  GdkEvent        *event,
					  gpointer         user_data);

#ifndef IS_SHOP
static void fill_supplier_combo(void);
static void move_stocked_pieces_order(GtkCList * list);
#endif

GtkWidget *c_find_stocked_pieces_open_window(GtkWidget *app1, GtkWidget *vbox)
{

    gchar *directory = g_strconcat(AA_DATA_DIR,dialog_file,NULL);
    gchar *e_name = NULL;
    gchar *w_name = NULL;

    if (find_pieces==NULL)
    {
	GtkWidget *dialog;
#ifdef IS_SHOP
	GtkWidget * modify_button = NULL;
	GtkWidget * order_button  = NULL;
	GtkWidget * supplier_label = NULL;
	GtkWidget * supplier_combo = NULL;
#endif

	xml = glade_xml_new (directory, widget_name);
	dialog = glade_xml_get_widget (xml, widget_name);

#ifdef IS_SHOP
	modify_button = glade_xml_get_widget (xml, "boton-modificar-listadopiezas");
	order_button  = glade_xml_get_widget (xml, "boton-pedido-listadopiezas");
	supplier_label  = glade_xml_get_widget (xml, "label_proveedor_listado_piezas");
	supplier_combo  = glade_xml_get_widget (xml, "combo_proveedor_listado_piezas");
	gtk_widget_hide(GTK_WIDGET(modify_button));
	gtk_widget_hide(GTK_WIDGET(order_button));
	gtk_widget_hide(GTK_WIDGET(supplier_label));
	gtk_widget_hide(GTK_WIDGET(supplier_combo));
#endif


	gtk_box_pack_start (GTK_BOX (vbox), dialog, TRUE, TRUE, 0);

	glade_xml_signal_connect (xml, "on_entry-eurocode-filtrado-listadopiezas_key_press_event",
				  GTK_SIGNAL_FUNC (on_press_key));

	glade_xml_signal_connect (xml, "on_entry-descripcionpieza-filtrado-listadopiezas_key_press_event",
				  GTK_SIGNAL_FUNC (on_press_key));

	glade_xml_signal_connect (xml, "on_clist-listadopiezas_select_row",
				  GTK_SIGNAL_FUNC (on_clist_listadopiezas_select_row));
            
	glade_xml_signal_connect (xml, "on_Buscar-listadopiezas_clicked",
				  GTK_SIGNAL_FUNC (on_search_button_clicked));

	glade_xml_signal_connect (xml, "on_boton-modificar-listadopiezas_clicked",
				  GTK_SIGNAL_FUNC (on_modify_button_clicked));

	glade_xml_signal_connect (xml, "on_boton-eliminar-listadopiezas_clicked",
				  GTK_SIGNAL_FUNC (on_delete_button_clicked));

	glade_xml_signal_connect (xml, "on_Limpiar-listadopiezas_clicked",
				  GTK_SIGNAL_FUNC (on_clean_button_clicked));
	glade_xml_signal_connect (xml, "on_button_precios_coste_clicked",
				  GTK_SIGNAL_FUNC (on_cost_price_button_clicked));

	glade_xml_signal_connect (xml, "on_boton-reparar-listadopiezas_clicked",
				  GTK_SIGNAL_FUNC (on_repair_button_clicked));
	glade_xml_signal_connect (xml, "on_boton-pedido-listadopiezas_clicked",
				  GTK_SIGNAL_FUNC (on_order_button_clicked));
	glade_xml_signal_connect (xml, "on_boton-stock-listadopiezas_clicked",
				  GTK_SIGNAL_FUNC (on_stock_button_clicked));
	glade_xml_signal_connect (xml, "on_combo-entry-empresa-filtrado-listadopiezas_changed",
				  GTK_SIGNAL_FUNC (on_combo_entry_empresa_filtrado_listadopiezas_changed));
	glade_xml_signal_connect (xml, "on_combo-entry-almacen-filtrado-listadopiezas_changed",
				  GTK_SIGNAL_FUNC (on_combo_entry_almacen_filtrado_listadopiezas_changed));
	glade_xml_signal_connect (xml, "on_checkbutton_stock_toggled",
				  GTK_SIGNAL_FUNC (on_stock_checkbutton_toggled));

                                        
	fill_enterprise_combo();
	fill_supplier_combo();



	find_pieces =dialog;
    }

    e_name = u_parameter_manager_get_parameter ("empresa");
    if (e_name!=NULL) {
	gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget (xml, "combo-entry-empresa-filtrado-listadopiezas")), e_name);
    }
    g_free(e_name);

    w_name = u_parameter_manager_get_parameter ("almacen");
    if (w_name!=NULL) {
	gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget (xml, "combo-entry-almacen-filtrado-listadopiezas")), w_name);
    }
    g_free(w_name);

    clean_screen();
    clean_combos();

    m_stock_clear_array (stocked_pieces_array);
    stocked_pieces_array=NULL;

    g_free(directory);
    return find_pieces;

}

void c_find_stocked_pieces_set_combos(gchar *enterprise, gchar *warehouse) {
    if (enterprise!=NULL) {
	gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget (xml, "combo-entry-empresa-filtrado-listadopiezas")), enterprise);
	if (warehouse!=NULL) {
	    gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget (xml, "combo-entry-almacen-filtrado-listadopiezas")), warehouse);
	}
    }
}

static void fill_CList(GtkCList * list,gulong size,gdouble * total_entry,gboolean show_stocked) {

    gulong i=0;
    gulong index=0;
    gchar * temp;
    gchar *empty_row[]={"","","","","","","","",""};
    gboolean ent;


    while (stocked_pieces_array[index]) {
//	if (i==30) gtk_clist_freeze ((GtkCList *) list);


	if (d_stock_get_quantity (stocked_pieces_array[index])!=G_MINDOUBLE) {
	    ent = TRUE;
	} else {
	    if (show_stocked!=FALSE) {
		ent=TRUE;
	    } else {
                ent=FALSE;
	    }
	}

	if (ent!=FALSE) {
	    gtk_clist_append ((GtkCList *) list,empty_row);
	    if (d_product_get_is_piece ((Product *)d_stock_get_piece (stocked_pieces_array[index]))) {
		temp = d_piece_get_euroCode (d_stock_get_piece (stocked_pieces_array[index]));
		gtk_clist_set_text((GtkCList *) list, i, C_EUROCODE_COLUMN, temp);
		g_free(temp);

		temp = g_strdup_printf("%d",d_piece_get_width (d_stock_get_piece (stocked_pieces_array[index])));
		gtk_clist_set_text((GtkCList *) list, i, C_WIDTH_COLUMN, temp);
		g_free(temp);

		temp = g_strdup_printf("%d",d_piece_get_length (d_stock_get_piece (stocked_pieces_array[index])));
		gtk_clist_set_text((GtkCList *) list, i, C_LONG_COLUMN, temp);
		g_free(temp);

	    }

	    temp = d_product_get_family_code ((Product *)d_stock_get_piece (stocked_pieces_array[index]));
	    gtk_clist_set_text((GtkCList *) list, i, C_FAMILY_COLUMN, temp);
	    g_free(temp);

	    temp = d_product_get_description ((Product *)d_stock_get_piece (stocked_pieces_array[index]));
	    gtk_clist_set_text((GtkCList *) list, i, C_DESCRIPTION_COLUMN, temp);
	    g_free(temp);

	    temp = g_strdup_printf("%.2f",d_product_get_vat_type ((Product *)d_stock_get_piece (stocked_pieces_array[index])));
	    gtk_clist_set_text((GtkCList *) list, i, C_VAT_COLUMN, temp);
	    g_free(temp);

	    if (d_piece_get_cost_price (d_stock_get_piece (stocked_pieces_array[index]))!=G_MINDOUBLE) {
		temp = g_strdup_printf("%.2f",d_piece_get_cost_price (d_stock_get_piece (stocked_pieces_array[index])));
		gtk_clist_set_text((GtkCList *) list, i, C_COST_PRICE_COLUMN, temp);
		g_free(temp);
	    }
	    if (d_product_get_price ((Product *)d_stock_get_piece (stocked_pieces_array[index]))!=G_MINDOUBLE) {
		temp = g_strdup_printf("%.2f",d_product_get_price ((Product *)d_stock_get_piece (stocked_pieces_array[index])));
		gtk_clist_set_text((GtkCList *) list, i, C_PRICE_COLUMN, temp);
		g_free(temp);
	    }

	    if (d_stock_get_quantity (stocked_pieces_array[index])!=G_MINDOUBLE)
		temp = g_strdup_printf("%.3f",d_stock_get_quantity (stocked_pieces_array[index]));
	    else
		temp = g_strdup("");

	    if ((d_piece_get_cost_price (d_stock_get_piece (stocked_pieces_array[index])) != G_MINDOUBLE) &&
		(d_stock_get_quantity (stocked_pieces_array[index]) != G_MINDOUBLE)) {

		*total_entry = *total_entry + (u_round_to_decimal (d_piece_get_cost_price (d_stock_get_piece (stocked_pieces_array[index])),2)*
					       u_round_to_decimal (d_stock_get_quantity (stocked_pieces_array[index]),2));

	    }
	    gtk_clist_set_text((GtkCList *) list, i, C_STOCK_COLUMN, temp);
	    g_free(temp);

	    i++;
	}
	index++;
    }

    gtk_clist_thaw ((GtkCList *) list);

}

static void on_search_button_clicked(GtkButton *button, gpointer data) {
    GdkCursor * cursor;
    GtkWidget * window;
    GtkWidget * list;

    list = glade_xml_get_widget(xml,"clist-listadopiezas");
    window = lookup_widget(GTK_WIDGET(list),"app1");
    cursor = gdk_cursor_new(GDK_WATCH);
    gdk_window_set_cursor(GTK_WIDGET(window)->window, cursor);
    gdk_cursor_destroy(cursor);

    while (gtk_events_pending())
	gtk_main_iteration();

    gtk_clist_clear(GTK_CLIST(glade_xml_get_widget(xml,"clist-listadopiezas")));

    m_stock_clear_array (stocked_pieces_array);
    stocked_pieces_array=NULL;

    obtain_stocked_pieces();

    cursor = gdk_cursor_new(GDK_LEFT_PTR);
    gdk_window_set_cursor(GTK_WIDGET(window)->window, cursor);
    gdk_cursor_destroy(cursor);

}

static void fill_supplier_combo() {
    GtkWidget * combo = glade_xml_get_widget(xml,"combo_proveedor_listado_piezas");
    Supplier ** suppliers = m_supplier_find_all ();

    c_util_set_code_and_name_on_combo (C_SUPPLIER_NONE,_("There is not assigned filter"),GTK_COMBO(combo));
    c_util_fill_combo_with_suppliers (combo,suppliers);

    m_supplier_clear_array (suppliers);
}


void obtain_stocked_pieces() {
    GtkWidget * list;
    GtkWidget * entry;
    GtkWidget * checkbutton;
    ConditionFindStock con;
    gchar * content;
    gulong code;
    GtkWidget *check       = NULL;
    gulong stocked_pieces_array_len = 0;
    gboolean minimun_stock = FALSE;
    gboolean show_stocked;
    gdouble * total = (gdouble *) g_malloc0(sizeof(gdouble));


    list = glade_xml_get_widget(xml,"clist-listadopiezas");
    entry = glade_xml_get_widget(xml,"entry_total_piezas");
    checkbutton= glade_xml_get_widget(xml,"checkbutton_stock_listadopiezas");

    content=gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "entry-eurocode-filtrado-listadopiezas")),0,-1);
    if (!strcmp(content,"")) {
	con.eurocode=NULL;
    } else {
	con.eurocode   = (gchar *) g_malloc (strlen(content)+1);
	strcpy(con.eurocode, content);
    }
    g_free(content);

    content=gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "entry-descripcionpieza-filtrado-listadopiezas")),0,-1);
    if (!strcmp(content,"")) {
	con.description=NULL;
    } else {
	con.description   = (gchar *) g_malloc (strlen(content)+1);
	strcpy(con.description, content);
    }
    g_free(content);

    code = c_util_get_code_from_combo (glade_xml_get_widget (xml, "combo-empresa-filtrado-listadopiezas"));
    con.enterprise_code = code;

    code = c_util_get_code_from_combo (glade_xml_get_widget (xml, "combo-almacen-filtrado-listadopiezas"));
    con.warehouse_code = code;
    con.supplier_code = c_util_get_code_from_combo (glade_xml_get_widget (xml, "combo_proveedor_listado_piezas"));

    check = glade_xml_get_widget(xml, "checkbutton-stockminimo-listadopiezas");
    minimun_stock = (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check))==TRUE);
    con.minimun_stock = minimun_stock;

    gtk_clist_clear ((GtkCList *) list);
    m_stock_clear_array (stocked_pieces_array);
    stocked_pieces_array=NULL;

    stocked_pieces_array = m_stock_find_pieces_by_condition (&con,&stocked_pieces_array_len);

    if ((stocked_pieces_array!=NULL) && (stocked_pieces_array_len>0)) {
	show_stocked = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton));
	fill_CList(GTK_CLIST(list), stocked_pieces_array_len,total,!show_stocked);
    } else {
        c_util_open_info_window (_("There is not any piece"));
    }
    fill_total_entry(entry,total);
    g_free(total);

    size_of_array = stocked_pieces_array_len;

}

static gint obtain_stocked_pieces_ids(GArray * selected, Stock **pieces, gulong *pieces_ids) {

    gint index=0;
    gint indexOfPiece=0;
    gulong real_index=0;
    GtkWidget* clist = glade_xml_get_widget(xml,"clist-listadopiezas");


    while (index<selected->len) {
	indexOfPiece = g_array_index(selected,gint,index);

	real_index = get_array_index_from_pieces_list(GTK_CLIST(clist),indexOfPiece);

	pieces_ids[index] =
	    d_product_get_product_code ((Product *)d_stock_get_piece (pieces[real_index]));
	index++;
    }
    pieces_ids[index]=-1;
    return index;
}

static gint obtain_selected_stocked_pieces(GArray * selected, Stock **stocked_pieces, Stock **selected_pieces) {

    gint index=0;
    gint indexOfPiece=0;
    gulong real_index=0;
    GtkWidget* clist = glade_xml_get_widget(xml,"clist-listadopiezas");

    while (index<selected->len) {
	indexOfPiece = g_array_index(selected,gint,index);

	real_index = get_array_index_from_pieces_list(GTK_CLIST(clist),indexOfPiece);

	selected_pieces[index] = stocked_pieces[real_index];
	index++;
    }
    selected_pieces[index]=NULL;
    return index;
}

static void move_stocked_pieces(GtkCList * list) {
    GtkWidget *app1 = lookup_widget(GTK_WIDGET(list), "app1");
    GtkWidget *vbox  = lookup_widget(GTK_WIDGET(list), "main_container");
    GArray    *selected = c_util_get_selected_rows_from_clist (list);
    gulong     number;
    gulong     *pieces_ids;
    gint       index;
    Stock    **selected_pieces;
    GtkWidget *combo_enterprise = glade_xml_get_widget (xml, "combo-empresa-filtrado-listadopiezas");
    GtkWidget *combo_warehouse  = glade_xml_get_widget (xml, "combo-almacen-filtrado-listadopiezas");
    gulong    enterprise_code = (gulong) c_util_get_code_from_combo (combo_enterprise);
    gulong    warehouse_code  = (gulong) c_util_get_code_from_combo (combo_warehouse);

    if (selected->len>0) {

	if ((enterprise_code!=C_ALL_ENTERPRISES)
	    && (warehouse_code!=C_ALL_WAREHOUSES_OF_ENTERPRISE)) {
	  gchar *a_name = u_parameter_manager_get_parameter ("almacen");
	  gchar *a_combo = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "combo-entry-almacen-filtrado-listadopiezas")),0,-1);	  

#ifdef IS_SHOP
	  if (!g_strcasecmp(a_name,a_combo)) {	  	  
#endif
	    number = (gulong)(selected->len+1);
	    pieces_ids = (gulong *) g_malloc(sizeof(gulong)*(selected->len+1));
            selected_pieces = (Stock **)g_malloc(sizeof(Stock *)*(selected->len+1));

	    obtain_stocked_pieces_ids(selected, stocked_pieces_array, pieces_ids);


	    obtain_selected_stocked_pieces(selected, stocked_pieces_array, selected_pieces);



	    c_repair_open_window(app1, vbox);

	    c_repair_load_pieces(selected_pieces);
	    i_shortcut_change_dialog (GTK_WIDGET(list),0,1);

	    gtk_clist_unselect_all(GTK_CLIST(list));

	    clean_screen();
//	    clean_combos();
	    
	    g_free(pieces_ids);

            index = 0;

 	    g_free(selected_pieces); 

	    selected_pieces=NULL;
#ifdef IS_SHOP
	  }
	  else c_util_open_error_window (_("Repairs can't be done with pieces from others warehouses"));
#endif

	  g_free(a_name);
	  g_free(a_combo);

    	} else {
	    c_util_open_error_window (_("Select the enterprise and the warehouse where the repair will be done"));
	}
    } else {
	c_util_open_error_window (_("Select a piece to insert it in the repair"));
    }

    g_array_free(selected, FALSE);
}

#ifndef IS_SHOP
static void move_stocked_pieces_order(GtkCList * list) {
    GtkWidget *app1 = lookup_widget(GTK_WIDGET(list), "app1");
    GtkWidget *vbox  = lookup_widget(GTK_WIDGET(list), "main_container");
    GArray    *selected = c_util_get_selected_rows_from_clist (list);
    gulong     number;
    gulong     *pieces_ids;
    GtkWidget *combo_enterprise = glade_xml_get_widget (xml, "combo-empresa-filtrado-listadopiezas");
    GtkWidget *combo_warehouse  = glade_xml_get_widget (xml, "combo-almacen-filtrado-listadopiezas");
    gulong    enterprise_code = (gulong) c_util_get_code_from_combo (combo_enterprise);
    gulong    warehouse_code  = (gulong) c_util_get_code_from_combo (combo_warehouse);


    if (selected->len>0) {
	if ((enterprise_code!=C_ALL_ENTERPRISES)
	    && (warehouse_code!=C_ALL_WAREHOUSES_OF_ENTERPRISE)) {
	    number = (gulong)(selected->len+1);
	    pieces_ids = (gulong *) g_malloc(sizeof(gulong)*(selected->len+1));

	    obtain_stocked_pieces_ids(selected, stocked_pieces_array, pieces_ids);

	    c_order_open_window(app1, vbox, NULL);

	    c_order_load_pieces(pieces_ids, &number, enterprise_code,
			      warehouse_code);
	    i_shortcut_change_dialog (GTK_WIDGET(list),2,3);

	    gtk_clist_unselect_all(GTK_CLIST(list));

	    clean_screen();
//	    clean_combos();

	    g_free(pieces_ids);
    	} else {
	    c_util_open_error_window (_("Select the enterprise and the warehouse for the order"));
	}
    } else {
	c_util_open_error_window (_("Select at least one piece"));
    }

    g_array_free(selected, FALSE);
}
#endif

static void on_modify_button_clicked(GtkButton *button,
						     gpointer data) {
#ifndef IS_SHOP
    GtkWidget * list = glade_xml_get_widget (xml, "clist-listadopiezas");
    GtkWidget * app1 = lookup_widget(GTK_WIDGET(GTK_CLIST(list)), "app1");
    GtkWidget * vbox = lookup_widget(GTK_WIDGET(GTK_CLIST(list)), "main_container");
    GArray *selected = c_util_get_selected_rows_from_clist (GTK_CLIST(list));
    gulong  n=2;
    gulong  *pieces_ids;
    Piece **pieces;

    if ((selected->len)==1) {
	pieces_ids = (gulong *) g_malloc(sizeof(gulong)*(1+1));

	obtain_stocked_pieces_ids(selected, stocked_pieces_array, pieces_ids);

	c_piece_open_window(app1, vbox);

	// Only load the first one. (Really, there is only one)
	if (pieces_ids[0]!=-1) {
	    pieces = m_piece_find_by_product_code (pieces_ids, &n);
	    c_piece_load(pieces[0]);
	}

	i_shortcut_change_dialog (GTK_WIDGET(list),2,0);

	gtk_clist_unselect_all(GTK_CLIST(list));
	g_free(pieces_ids);
    } else {
	if ((selected->len)==0) {
	    c_util_open_error_window (_("Select a piece to be modified"));
	} else {
	    c_util_open_error_window (_("Select just one piece to be modified"));
	}
    }

    g_array_free(selected, FALSE);
#endif

}

static void on_stock_button_clicked(GtkButton *button,
						 gpointer data) {
    GtkWidget *list  = glade_xml_get_widget (xml, "clist-listadopiezas");
    GtkWidget *combo_enterprise = glade_xml_get_widget (xml, "combo-empresa-filtrado-listadopiezas");
    GtkWidget *combo_warehouse  = glade_xml_get_widget (xml, "combo-almacen-filtrado-listadopiezas");
    GtkWidget *update_stock_window = NULL;

    GArray    *selected = c_util_get_selected_rows_from_clist (GTK_CLIST(list));
    gchar     *enterprise_name = g_strdup(gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget (xml, "combo-entry-empresa-filtrado-listadopiezas"))));
    gchar     *warehouse_name = g_strdup(gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget (xml, "combo-entry-almacen-filtrado-listadopiezas"))));
    gulong    enterprise_code = (gulong) c_util_get_code_from_combo (combo_enterprise);
    gulong    warehouse_code  = (gulong) c_util_get_code_from_combo (combo_warehouse);
    GtkWidget * window;
    GdkCursor * cursor;


#ifdef IS_SHOP
    gchar     *e_name=NULL;
    gchar     *w_name=NULL;
    gulong    e_code=ULONG_MAX;
    gulong    w_code=ULONG_MAX;

    e_name = u_parameter_manager_get_parameter ("empresa");
    if (e_name!=NULL) {
	e_code = m_enterprise_get_code_from_name (e_name);
    }
    g_free(e_name);

    w_name = u_parameter_manager_get_parameter ("almacen");
    if (w_name!=NULL) {
	w_code = m_warehouse_get_code_from_name (w_name);
    }
    g_free(w_name);
#endif
    window = lookup_widget(GTK_WIDGET(list),"app1");
    cursor = gdk_cursor_new(GDK_WATCH);
    gdk_window_set_cursor(GTK_WIDGET(window)->window, cursor);
    gdk_cursor_destroy(cursor);
    while (gtk_events_pending())
	gtk_main_iteration();

    window = lookup_widget(GTK_WIDGET(list),"app1");
    cursor = gdk_cursor_new(GDK_WATCH);
    gdk_window_set_cursor(GTK_WIDGET(window)->window, cursor);
    gdk_cursor_destroy(cursor);

    while (gtk_events_pending())
	gtk_main_iteration();

    if ((selected->len)==1) {
#ifdef IS_SHOP
	if ((enterprise_code==e_code)
	    && (warehouse_code==w_code)) {
#else
	if ((enterprise_code!=C_ALL_ENTERPRISES)
	    && (warehouse_code!=C_ALL_WAREHOUSES_OF_ENTERPRISE)) {
#endif
	    gulong     n = 2;
	    gulong     *stocked_pieces_ids = (gulong *) g_malloc(sizeof(gulong)*(1+1));
	    Stock    **stocked_pieces = NULL;
	    GtkWidget *app1  = lookup_widget(GTK_WIDGET(GTK_CLIST(list)), "app1");

	    obtain_stocked_pieces_ids(selected, stocked_pieces_array, stocked_pieces_ids);

	    // Only load the first one. (Really, there is only one)
	    if (stocked_pieces_ids[0]!=-1) {

		stocked_pieces =
		    m_stock_find_pieces_by_product_code (stocked_pieces_ids,
								enterprise_code,
								warehouse_code, &n);
		if(stocked_pieces[0]!=NULL) {
		    update_stock_window = c_piece_stock_open_window(app1,obtain_stocked_pieces);
		    c_piece_stock_load(stocked_pieces[0], enterprise_name, warehouse_name);

		    gtk_window_set_modal(GTK_WINDOW(update_stock_window),1);
		    gtk_window_set_position(GTK_WINDOW(update_stock_window),GTK_WIN_POS_CENTER);
		    gtk_window_set_transient_for(GTK_WINDOW(update_stock_window),
						 GTK_WINDOW(app1));
		    gtk_widget_show(update_stock_window);

		    gtk_clist_unselect_all(GTK_CLIST(list));
		}
	    }
//	    clean_screen();
//	    clean_combos();
	    g_free(stocked_pieces_ids);
	    m_stock_clear_array (stocked_pieces);

	    stocked_pieces = NULL;
	} else {
#ifdef IS_SHOP
	    c_util_open_error_window (_("You just can modify the stock of your own warehouse"));
#else
	    c_util_open_error_window (_("Select an enterprise and a warehouse where you want to modify the stock"));
#endif
	}
    } else {
	if ((selected->len)==0) {
	    c_util_open_error_window (_("Select a piece to be modified"));
	} else {
	    c_util_open_error_window (_("Select just one piece to be modified"));
	}
    }
    g_free(enterprise_name);
    g_free(warehouse_name);
    cursor = gdk_cursor_new(GDK_LEFT_PTR);
    gdk_window_set_cursor(GTK_WIDGET(window)->window, cursor);
    gdk_cursor_destroy(cursor);
    g_array_free(selected, FALSE);
}

static void on_delete_button_clicked(GtkButton *button,
						    gpointer data) {

/*    GtkWidget *list = glade_xml_get_widget (xml, "clist-listadopiezas");
    GtkWidget *warning_dialog = NULL;
    GtkWidget *app1  = lookup_widget(GTK_WIDGET(GTK_CLIST(list)), "app1");
    GArray *selected = c_util_get_selected_rows_from_clist (GTK_CLIST(list));
    Piece  **pieces_array;
    gulong   *pieces_ids;
    gulong    number = 1;
    gint ret=-1, i=0;

    if ((selected->len)>0) {
	pieces_ids = (gulong *) g_malloc(sizeof(gulong)*(1+1));

	obtain_stocked_pieces_ids(selected, stocked_pieces_array, pieces_ids);
//	pieces_array = m_piece_find_by_product_code (pieces_ids, &number);

	warning_dialog =
	    gnome_message_box_new ("Est seguro de que quiere ELIMINAR lo seleccionado de la Base de Datos?",
				   GNOME_MESSAGE_BOX_WARNING,
				   GNOME_STOCK_BUTTON_OK,
				   GNOME_STOCK_BUTTON_CANCEL,
				   NULL);
	if (warning_dialog !=NULL) {
	    gtk_window_set_modal(GTK_WINDOW(warning_dialog),1);
	    gtk_window_set_position(GTK_WINDOW(warning_dialog),GTK_WIN_POS_CENTER);
	    gtk_window_set_transient_for(GTK_WINDOW(warning_dialog),
	    GTK_WINDOW(app1));
	    ret = gnome_dialog_run_and_close(GNOME_DIALOG (warning_dialog));
	}

	switch (ret){
	case 0:
//	    while(pieces_array[i]!=NULL) {
//		m_piece_delete (pieces_array[i]);
//		i++;
//	    }
//	    gtk_clist_unselect_all(GTK_CLIST(list));
	    clean_screen();
	    break;
	case 1:
	    break;
	default:
*/	    /* User closed dialog with window manager,
	     * assume case 1 (cancel)
	     */
/*	    break;
	}
	g_free(pieces_ids);
//	m_piece_clear_array (pieces_array);
        pieces_array = NULL;
    } else {
	c_util_open_error_window ("Seleccione al menos una pieza a eliminar de la Base de Datos");
    }
    g_array_free(selected, FALSE);
*/
}

static void on_repair_button_clicked(GtkButton *button,
						   gpointer data)
{
    GtkWidget * list = glade_xml_get_widget (xml, "clist-listadopiezas");
    move_stocked_pieces(GTK_CLIST(list));
}

static void on_order_button_clicked(GtkButton *button,
						   gpointer data)
{
#ifndef IS_SHOP
    GtkWidget * list = glade_xml_get_widget (xml, "clist-listadopiezas");
    move_stocked_pieces_order(GTK_CLIST(list));
#endif
}

static void on_clean_button_clicked(GtkButton *button, gpointer data) {
    clean_screen();
    clean_combos();
}

static void clean_screen() {

    GtkWidget *vbox;
    GtkWidget *list;

    list = glade_xml_get_widget(xml,"clist-listadopiezas");
    vbox = GTK_WIDGET(glade_xml_get_widget(xml, widget_name));
    gtk_container_foreach(GTK_CONTAINER(vbox),&c_util_clean_widget,NULL);

    if (stocked_pieces_array!=NULL) {
	gtk_clist_clear ((GtkCList *) list);
	m_stock_clear_array (stocked_pieces_array);
	stocked_pieces_array=NULL;
    }

}

static void on_press_key(GtkWidget *widget,
				       GdkEventKey *event,
				       gpointer user_data) {
    GtkWidget * wid;
    GtkContainer * container;
    GList * list=NULL;
    GList * last_list;
    GList * first_focusable_of_list;


    if (event->keyval==GDK_Return) {
	on_search_button_clicked(NULL,user_data);
    } else if (event->keyval==GDK_Tab) {

	container = (GtkContainer *) widget->parent;
	list = gtk_container_children(container);
	wid=GTK_WIDGET((list)->data);

	while (((GTK_WIDGET_CAN_FOCUS(wid))==0) ||
	       ((GTK_WIDGET_VISIBLE(wid))==0) ||
	       ((GTK_WIDGET_SENSITIVE(wid))==0)) {
	    list=g_list_next(list);
	    wid=GTK_WIDGET((list)->data);
	}

	first_focusable_of_list = list;

	last_list = (GList *) g_list_last(list);
	while (last_list!=list) {
	    wid=GTK_WIDGET((list)->data);
	    if (GTK_WIDGET_HAS_FOCUS(wid)==1) {
		break;
	    }
	    list=g_list_next(list);

	}
	list=g_list_next(list);
	wid=GTK_WIDGET((list)->data);
	while ((((GTK_WIDGET_CAN_FOCUS(wid))==0) ||
		((GTK_WIDGET_VISIBLE(wid))==0) ||
		((GTK_WIDGET_SENSITIVE(wid))==0)) && (last_list!=list)){
	    list=g_list_next(list);
	    wid=GTK_WIDGET((list)->data);
	}
	if (last_list==list) {
	    list=first_focusable_of_list;
	    wid=GTK_WIDGET((list)->data);
	}

	gtk_signal_emit_stop_by_name(GTK_OBJECT (widget),"key_press_event");

	gtk_widget_grab_focus(wid);

	/*  	g_print("Nombre activable: %s\n",wid->name);  */
    }

}

static void on_combo_entry_empresa_filtrado_listadopiezas_changed() {
    fill_warehouse_combo();
}

static void on_combo_entry_almacen_filtrado_listadopiezas_changed() {
    clean_screen();
}

static void fill_warehouse_combo() {
    GtkWidget * warehouse_combo;
    GtkWidget * enterprise_combo;
    Warehouse ** warehouses;
    gint code_combo;

    enterprise_combo = glade_xml_get_widget (xml, "combo-empresa-filtrado-listadopiezas");
    warehouse_combo = glade_xml_get_widget (xml, "combo-almacen-filtrado-listadopiezas");
    code_combo = c_util_get_code_from_combo (enterprise_combo);

    warehouses = m_warehouse_find_by_enterprise_code (code_combo);

    clean_warehouse_combo();

    c_util_set_code_and_name_on_combo (C_ALL_WAREHOUSES_OF_ENTERPRISE,g_strdup(_("All the warehouses")),GTK_COMBO(warehouse_combo));
    c_util_fill_combo_with_warehouses(warehouse_combo,warehouses);
    m_warehouse_clear_array (warehouses);
    warehouses = NULL;

}

static void fill_enterprise_combo() {
    GtkWidget * combo;
    Enterprise ** enterprises;

    combo = glade_xml_get_widget (xml, "combo-empresa-filtrado-listadopiezas");

    c_util_set_code_and_name_on_combo (C_ALL_ENTERPRISES,g_strdup(_("All the enterprises")),GTK_COMBO(combo));
    enterprises = m_enterprise_find_all ();

    c_util_fill_combo_with_enterprises (combo,enterprises);
}

static void clean_warehouse_combo() {
    GtkWidget * combo;
    GtkWidget * entry;

    entry = glade_xml_get_widget (xml, "combo-entry-almacen-filtrado-listadopiezas");
    gtk_entry_set_text(GTK_ENTRY(entry),"");

    combo = glade_xml_get_widget (xml, "combo-almacen-filtrado-listadopiezas");
    c_util_clean_code_and_name_combo (combo);
}

static void clean_enterprise_combo() {
    GtkWidget * combo;
    GtkWidget * entry;

    entry = glade_xml_get_widget (xml, "combo-entry-empresa-filtrado-listadopiezas");
    gtk_entry_set_text(GTK_ENTRY(entry),"");

    combo = glade_xml_get_widget (xml, "combo-empresa-filtrado-listadopiezas");
    c_util_clean_code_and_name_combo (combo);

}

static void clean_combos() {
    gchar *e_name = NULL;
    gchar *w_name = NULL;

    clean_enterprise_combo();
    clean_warehouse_combo();
    fill_enterprise_combo();
    e_name = u_parameter_manager_get_parameter ("empresa");
    if (e_name!=NULL) {
	gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget (xml, "combo-entry-empresa-filtrado-listadopiezas")), e_name);
    }

    w_name = u_parameter_manager_get_parameter ("almacen");
    if (w_name!=NULL) {
	gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget (xml, "combo-entry-almacen-filtrado-listadopiezas")), w_name);
    }

    g_free(e_name);
    g_free(w_name);
}

static void view_history_piece() {
    GArray * selected;
    gint value;
    GtkCList * list = GTK_CLIST(glade_xml_get_widget (xml, "clist-listadopiezas"));



    selected = c_util_get_selected_rows_from_clist (list);

    if (selected->len==1) {
	GtkWidget * history =  c_warehouse_history_open_window();
        gulong enterprise_code = c_util_get_code_from_combo (glade_xml_get_widget(xml,"combo-empresa-filtrado-listadopiezas"));
        gulong warehouse_code = c_util_get_code_from_combo (glade_xml_get_widget(xml,"combo-almacen-filtrado-listadopiezas"));
	if ((enterprise_code!=0) && (warehouse_code!=0)) {
	    gchar ** temp_name = (gchar **)g_malloc0(sizeof(gchar *));

//	    gtk_window_set_modal(GTK_WINDOW(history),TRUE);
	    gtk_window_set_position(GTK_WINDOW(history),GTK_WIN_POS_CENTER);
//	    gtk_window_set_transient_for(GTK_WINDOW(history),
//					 GTK_WINDOW(app1));
	    value = g_array_index(selected,gint,0);
 	    gtk_clist_get_text(GTK_CLIST(glade_xml_get_widget(xml,"clist-listadopiezas")),value,C_EUROCODE_COLUMN,temp_name);
	    c_warehouse_history_load_data(warehouse_code,enterprise_code,g_strdup(temp_name[0]));
	    gtk_widget_show(history);
	    g_free(temp_name);
	    g_array_free(selected,TRUE);
	} else {
	    c_util_open_error_window (_("Select a warehouse of a enterprise"));
	}
    } else if (selected->len==0) {
	c_util_open_error_window (_("Select a piece to be modified"));
    } else {
	c_util_open_error_window (_("Select just one piece to be modified"));
    }

}

static void on_clist_listadopiezas_select_row(GtkCList        *list,
					  gint             row,
					  gint             column,
					  GdkEvent        *event,
					  gpointer         user_data) {


    if (event!=NULL) {
	if (event->type == GDK_2BUTTON_PRESS)
	{
	    view_history_piece();
	}
    }
}

static void on_cost_price_button_clicked(GtkButton *button, gpointer data) {
#ifndef IS_SHOP
    GArray * selected;
    GtkWidget * app1 = lookup_widget(GTK_WIDGET(button), "app1");
    GtkWidget * vbox  = lookup_widget(GTK_WIDGET(button), "main_container");
    GtkCList * list = GTK_CLIST(glade_xml_get_widget (xml, "clist-listadopiezas"));

    selected = c_util_get_selected_rows_from_clist (list);

    if (selected->len==1) {
	gint index = g_array_index(selected,gint,0);
        gulong real_index;

	real_index = get_array_index_from_pieces_list(GTK_CLIST(list),index);
	c_find_product_cost_open_window(app1, vbox);

        c_load_piece(d_piece_duplicate (d_stock_get_piece(stocked_pieces_array[real_index])));

	i_shortcut_change_dialog (GTK_WIDGET(vbox),2,6);

    } else {
        c_util_open_info_window (_("Select at least one piece"));
    }
#else
    c_util_open_info_window (_("Operation allowed just in head office"));
#endif

}

static void fill_total_entry(GtkWidget * entry,gdouble * total) {
    gchar * content = g_strdup_printf("%.2f",*total);
    gtk_entry_set_text(GTK_ENTRY(entry),content);
    g_free(content);
    *total = 0;
}

static void on_stock_checkbutton_toggled(GtkToggleButton *togglebutton, gpointer user_data) {
    gboolean show_stocked;
    GtkWidget * list;
    GtkWidget * checkbutton;
    gdouble * total = (gdouble *) g_malloc0(sizeof(gdouble));

    if (stocked_pieces_array!=NULL) {
	list = glade_xml_get_widget(xml,"clist-listadopiezas");
	checkbutton= glade_xml_get_widget(xml,"checkbutton_stock_listadopiezas");

	show_stocked = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton));
	gtk_clist_clear (GTK_CLIST(list));

	fill_CList(GTK_CLIST(list), size_of_array,total,!show_stocked);
    }
    g_free(total);

}

static gulong get_array_index_from_pieces_list(GtkCList * clist,gulong index) {

    gchar ** content = (gchar **) g_malloc0(sizeof(gchar *));
    gulong i = 0;
    gchar * eurocode;
    gtk_clist_get_text(clist,index,0,content);
    eurocode = g_strdup(content[0]);
    g_strstrip(eurocode);
    if (stocked_pieces_array!=NULL) {
	while (stocked_pieces_array[i]!=NULL) {
	    gchar *tmp = d_piece_get_euroCode (d_stock_get_piece (stocked_pieces_array[i]));
	    g_strstrip(tmp);
	    if (!strcmp(eurocode,tmp)) {
                break;
	    }
	    i++;
            g_free(tmp);
	}
    }
    g_free(eurocode);
    g_free(content);
    return i;
}
