/* insert_account_control.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include <gnome.h>
#include <glade/glade.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <glib.h>
#include <stdlib.h>


#include "a-shortcut.h"
#include "support.h"
#include "util.h"
#include "office.h"
#include "control_util.h"
#include "insert_account_control.h"
#include "account_model.h"
#include "insert_clients_control.h"
#include "edit_clients_control.h"

static GtkWidget *insert_accounts=NULL;
static GladeXML *xml;
static gboolean from_client=C_ACCOUNT_NOT_FROM_CLIENTS;
static gchar widget_name[]="cuentas_bancarias";
static gchar url[]="/src/interface/glade/invoice_view.glade";
static void (*function_to_call)(gpointer value);
static gboolean update = FALSE;
static gulong  participant_code_to_update = FALSE;
static gchar * previous_account_number=NULL;
static gchar * previous_bank_code=NULL;
static gchar * previous_office_code=NULL;
static void on_accept_button_clicked(GtkButton *button, gpointer data);
static void on_cancel_button_clicked(GtkButton *button, gpointer data);
static void on_account_window_destroy(GtkWidget * widget,gpointer user_data);
static void inicialize_balance_entry(void);
static gboolean insert_account(void);
static void clean_screen(void);
static gboolean modify_account(void);
static void fill_spins_with_account_number(gchar * complete_account);

GtkWidget *c_insert_account_open_window(gpointer f_to_call,gint from_clients)
{
   gchar * directory = g_strconcat(AA_DATA_DIR,url,NULL);
   
   if (insert_accounts==NULL)
   {
       GtkWidget *dialog;

       xml = glade_xml_new (directory, widget_name);
       dialog = glade_xml_get_widget (xml, widget_name);
       glade_xml_signal_connect (xml, "on_button_aceptar_cuenta_clicked",
				 GTK_SIGNAL_FUNC (on_accept_button_clicked));
       glade_xml_signal_connect (xml, "on_button_cancelar_cuenta_clicked",
				 GTK_SIGNAL_FUNC (on_cancel_button_clicked));
       glade_xml_signal_connect (xml, "on_cuentas_bancarias_destroy",
				 GTK_SIGNAL_FUNC (on_account_window_destroy));

       insert_accounts=dialog;

   }
   else {
       clean_screen();
   }
   g_free(directory);
   function_to_call = f_to_call;
   from_client = from_clients;
   inicialize_balance_entry();
   return insert_accounts;

}

void c_generic_repair_line_set_update(gboolean do_update,gulong participant_code,gchar * complete_account) {
    GtkWidget * spin1 = glade_xml_get_widget(xml,"spinbutton_primer_cuentas");
    GtkWidget * spin2 = glade_xml_get_widget(xml,"spinbutton_segundo_cuentas");
    GtkWidget * entry_ccc = glade_xml_get_widget(xml,"entry_ccc_cuentas");

    if (do_update!=FALSE)
	fill_spins_with_account_number(complete_account);
    update = do_update;
    participant_code_to_update=participant_code;

    if (update !=FALSE) {
	previous_bank_code=g_strdup(gtk_entry_get_text(GTK_ENTRY(spin1)));
	previous_office_code=g_strdup(gtk_entry_get_text(GTK_ENTRY(spin2)));
	previous_account_number=g_strdup(gtk_entry_get_text(GTK_ENTRY(entry_ccc)));
    }

}


static void clean_screen() {

    GtkWidget *vbox;

    vbox = GTK_WIDGET(glade_xml_get_widget(xml, widget_name));

    gtk_container_foreach(GTK_CONTAINER(vbox),&c_util_clean_widget,NULL);

}

static void on_accept_button_clicked(GtkButton *button, gpointer data) {
    gboolean no_error;
    gboolean * error_pointer = (gint *)g_malloc0(sizeof(gint));
    gpointer value;

    if (update == FALSE)
	no_error =  insert_account();
    else
        no_error = modify_account();

    *error_pointer = !no_error;
    value = (gpointer) error_pointer;

    if (no_error == FALSE) {
	if (function_to_call!=NULL) function_to_call(value);
	gtk_widget_destroy(insert_accounts);
	insert_accounts = NULL;

    } else {
        c_util_open_error_window (_("Error inserting the account"));
    }

}
static void on_cancel_button_clicked(GtkButton *button, gpointer data){
    gtk_widget_destroy(insert_accounts);
    insert_accounts = NULL;
}

static gboolean insert_account() {
    GtkWidget * spin1 = glade_xml_get_widget(xml,"spinbutton_primer_cuentas");
    GtkWidget * spin2 = glade_xml_get_widget(xml,"spinbutton_segundo_cuentas");
    GtkWidget * spin3 = glade_xml_get_widget(xml,"spinbutton_tercer_cuentas");
    GtkWidget * entry_ccc = glade_xml_get_widget(xml,"entry_ccc_cuentas");
    GtkWidget * balance = glade_xml_get_widget(xml,"entry_saldo_inicial");
    gchar * bank_code; 
    gchar * office_code; 
    gchar * dc; 
    gchar * account_number; 
    gchar * init_balance; 
    gboolean error=FALSE;

    gtk_spin_button_update(GTK_SPIN_BUTTON(spin1));
    gtk_spin_button_update(GTK_SPIN_BUTTON(spin2));
    gtk_spin_button_update(GTK_SPIN_BUTTON(spin3));

    bank_code = g_strdup(gtk_entry_get_text(GTK_ENTRY(spin1)));
    office_code = g_strdup(gtk_entry_get_text(GTK_ENTRY(spin2)));
    dc = g_strdup(gtk_entry_get_text(GTK_ENTRY(spin3)));
    account_number = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry_ccc)));
    init_balance = g_strdup(gtk_entry_get_text(GTK_ENTRY(balance)));

    if (d_account_is_correct(bank_code,office_code,dc,account_number)!=FALSE) {
	Office * office = d_office_create_empty ();
	Account * account = d_account_create_empty ();
        d_office_set_bank_code (office,atoi(bank_code));
        d_office_set_office_code (office,atoi(office_code));
	d_account_set_office (account,office);
	d_account_set_account_number (account,account_number);
	d_account_set_control_digit (account,atoi(dc));
	if (from_client==C_ACCOUNT_NOT_FROM_CLIENTS) {
            u_points_to_commas (init_balance);
	    d_account_set_initial_balance (account,atof(init_balance));
	    m_account_insert (account);
	} else if (from_client==C_ACCOUNT_FROM_INSERT_CLIENTS) {
	    c_clients_insert(account);
	} else if (from_client==C_ACCOUNT_FROM_EDIT_CLIENTS) {
	    c_edit_clients_add_account_client(account);
	}
    } else {
	error = TRUE;
	c_util_open_error_window (_("Wrong account number"));
    }
    return error;

}

static void on_account_window_destroy(GtkWidget * widget,gpointer user_data) {
    gtk_widget_destroy(insert_accounts);
    insert_accounts = NULL;
}

static void inicialize_balance_entry(void) {
    GtkWidget * entry_balance = glade_xml_get_widget(xml,"entry_saldo_inicial");
    GtkWidget * label_balance = glade_xml_get_widget(xml,"label_saldo_inicial_cuentas");
    if (from_client!=C_ACCOUNT_NOT_FROM_CLIENTS) {
        if GTK_WIDGET_VISIBLE(entry_balance) gtk_widget_hide(entry_balance);
        if GTK_WIDGET_VISIBLE(label_balance) gtk_widget_hide(label_balance);
    } else {
        if (!GTK_WIDGET_VISIBLE(entry_balance)) gtk_widget_show(entry_balance);
        if (!GTK_WIDGET_VISIBLE(label_balance)) gtk_widget_show(label_balance);

    }
}

static gboolean modify_account() {
    GtkWidget * spin1 = glade_xml_get_widget(xml,"spinbutton_primer_cuentas");
    GtkWidget * spin2 = glade_xml_get_widget(xml,"spinbutton_segundo_cuentas");
    GtkWidget * spin3 = glade_xml_get_widget(xml,"spinbutton_tercer_cuentas");
    GtkWidget * entry_ccc = glade_xml_get_widget(xml,"entry_ccc_cuentas");
    gchar * bank_code;
    gchar * office_code;
    gchar * dc; 
    gchar * account_number;
    gboolean error=FALSE;

    gtk_spin_button_update(GTK_SPIN_BUTTON(spin1));
    gtk_spin_button_update(GTK_SPIN_BUTTON(spin2));
    gtk_spin_button_update(GTK_SPIN_BUTTON(spin3));

    bank_code = g_strdup(gtk_entry_get_text(GTK_ENTRY(spin1)));
    office_code = g_strdup(gtk_entry_get_text(GTK_ENTRY(spin2)));
    dc = g_strdup(gtk_entry_get_text(GTK_ENTRY(spin3)));
    account_number = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry_ccc)));

    if (d_account_is_correct(bank_code,office_code,dc,account_number)!=FALSE) {
	Office * office = d_office_create_empty ();
	Account * account = d_account_create_empty ();
        d_office_set_bank_code (office,atoi(bank_code));
        d_office_set_office_code (office,atoi(office_code));
	d_account_set_office (account,office);
	d_account_set_account_number (account,account_number);
	d_account_set_control_digit (account,atoi(dc));
	if (from_client==C_ACCOUNT_NOT_FROM_CLIENTS) {

	} else if (from_client==C_ACCOUNT_FROM_INSERT_CLIENTS) {
	} else if (from_client==C_ACCOUNT_FROM_EDIT_CLIENTS) {
	    m_account_update (account,participant_code_to_update,previous_account_number,previous_office_code,previous_bank_code);
	}
    } else {
	error = TRUE;
	c_util_open_error_window (_("Wrong account number"));
    }
    return error;

}

static void fill_spins_with_account_number(gchar * complete_account) {
    GtkWidget * spin1 = glade_xml_get_widget(xml,"spinbutton_primer_cuentas");
    GtkWidget * spin2 = glade_xml_get_widget(xml,"spinbutton_segundo_cuentas");
    GtkWidget * spin3 = glade_xml_get_widget(xml,"spinbutton_tercer_cuentas");
    GtkWidget * entry_ccc = glade_xml_get_widget(xml,"entry_ccc_cuentas");
    gchar ** splits = g_strsplit(complete_account," ",4);

    gtk_entry_set_text(GTK_ENTRY(spin1),splits[0]);
    gtk_entry_set_text(GTK_ENTRY(spin2),splits[1]);
    gtk_entry_set_text(GTK_ENTRY(spin3),splits[2]);
    gtk_entry_set_text(GTK_ENTRY(entry_ccc),splits[3]);
    gtk_spin_button_update(GTK_SPIN_BUTTON(spin1));
    gtk_spin_button_update(GTK_SPIN_BUTTON(spin2));
    gtk_spin_button_update(GTK_SPIN_BUTTON(spin3));

    g_strfreev(splits);
}
