
/* insert_clients_control.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"

#include <gnome.h>
#include <glade/glade.h>
#include <glib.h>

#include "a-shortcut.h"
#include "support.h"
#include "invoice.h"
#include "insert_clients_control.h"
#include "insert_account_control.h"
#include "account_model.h"
#include "client_model.h"
#include "clients_control.h"
#include "control_util.h"
#include "consortium_model.h"
#include "client.h"
#include "participant.h"
#include "enterprise.h"
#include "supplier.h"
#include "consortium.h"
#include "company.h"
#include "company_model.h"
#include "particular_object.h"
#include "garage.h"
#include "post_address_control.h"


static GtkWidget *client_insertion=NULL;
static GladeXML *xml;
static gboolean event_changed = TRUE;
static gchar url[]="/src/interface/glade/clients_view.glade";
static gchar widget_name[]="insercion_persona";
static GList * accounts_client;
/* static PostAddress ** post_addresses=NULL; */

static void show_particular(void);
static void show_garage(void);
static void show_general_enterprise(void);  
static void show_enterprise(void);
static void show_company(void);
static void on_person_type_entry_changed(GtkEntry *entry,gpointer user_data);
static void on_trade_name_entry_insert_text(GtkEntry *entry, gchar *new_text, gint new_text_length, gint *position,gpointer user_data);
static void on_trade_name_entry_delete_text(GtkEntry *entry, gint start_pos, gint end_pos, gpointer user_data);	
static void on_trade_name_entry_move_cursor(GtkEntry *entry, gint x, gint y, gpointer user_data);
					    
static void on_company_entry_changed(GtkEntry *entry,gpointer user_data);
static void fill_person_combo(void);
//static void open_post_insert(GtkWidget *widget,gpointer user_data);
static void on_client_insertion_window_destroy(GtkWidget *widget,gpointer user_data);
static void close_insertion_window(GtkWidget *widget,gpointer user_data);
static void fill_consortium_combo(void);
static void fill_template_combo(void);
static void add_to_account_combo(Account * account);
static void set_account_checkbutton_state(gboolean new_state);
static void clean_screen(void);
static void insert_person(void);
static void repaint_accounts(gpointer value);
static void on_client_account_checkbutton_toggled(GtkToggleButton *togglebutton, gpointer user_data);
static void on_insert_button_clicked(void);
static void search_existent_clients(gint reply, gpointer user_data);


GtkWidget *c_clients_open_insertion_window(GtkWidget *app1)
{
   gchar * directory = g_strconcat(AA_DATA_DIR,url,NULL);
   
   if (client_insertion==NULL)
     {
	GtkWidget *window;
	
	xml = glade_xml_new (directory, widget_name);
	window = glade_xml_get_widget (xml, widget_name);
	
	glade_xml_signal_connect (xml, "on_close_button_clicked",
				  GTK_SIGNAL_FUNC (close_insertion_window));

	glade_xml_signal_connect (xml, "on_insercion_persona_destroy",
				  GTK_SIGNAL_FUNC (on_client_insertion_window_destroy));

	glade_xml_signal_connect (xml, "on_entry_insert_changed",
				  GTK_SIGNAL_FUNC (on_person_type_entry_changed));

	glade_xml_signal_connect (xml, "on_entry_company_changed",
				  GTK_SIGNAL_FUNC (on_company_entry_changed));

	glade_xml_signal_connect (xml, "on_limpiar_button_clicked",
				  GTK_SIGNAL_FUNC (clean_screen));

	glade_xml_signal_connect (xml, "on_insertar_button_clicked",
				  GTK_SIGNAL_FUNC (on_insert_button_clicked));
				        
	glade_xml_signal_connect (xml, "on_razonsocial_entry_insert_text",
				  GTK_SIGNAL_FUNC (on_trade_name_entry_insert_text));

	glade_xml_signal_connect (xml, "on_razonsocial_entry_move_cursor",
				  GTK_SIGNAL_FUNC (on_trade_name_entry_move_cursor));

	glade_xml_signal_connect (xml, "on_razonsocial_entry_delete_text",
				  GTK_SIGNAL_FUNC (on_trade_name_entry_delete_text));

	glade_xml_signal_connect (xml, "on_checkbutton_cuenta_cliente_toggled",
				  GTK_SIGNAL_FUNC (on_client_account_checkbutton_toggled));

	fill_consortium_combo();
	
	fill_person_combo();

	client_insertion=window;
	
     }

   g_free(directory);
   return client_insertion;
   
}


static void fill_consortium_combo() {
    GtkWidget * combo;
    Consortium ** consortiums;

    combo = glade_xml_get_widget (xml, "compania_combo_insert");

    c_util_set_code_and_name_on_combo (D_CONSORTIUM_NONE,g_strdup(_("<None>")),GTK_COMBO(combo));
    consortiums = m_consortium_find_by_code (ULONG_MAX);

    c_util_fill_combo_with_consortiums (combo,consortiums);    

}

static void fill_template_combo() {
    GtkWidget * combo = glade_xml_get_widget (xml, "template_combo_insert");

    c_util_clean_code_and_name_combo (combo);

    c_util_set_code_and_name_on_combo (D_INVOICE_NORMAL_CODE,D_INVOICE_MESSAGE_NORMAL,GTK_COMBO(combo));
}


static void fill_person_combo() {
    GtkWidget * combo;

    combo = glade_xml_get_widget (xml, "person_combo_insertion_person");

    c_util_set_code_and_name_on_combo (M_CLIENT_PERSON_CODE,(C_PERSON_MESSAGE),GTK_COMBO(combo));
    c_util_set_code_and_name_on_combo (M_CLIENT_ENTERPRISE_CODE,(C_ENTERPRISE_MESSAGE),GTK_COMBO(combo));
#ifndef IS_SHOP
    c_util_set_code_and_name_on_combo (M_CLIENT_SUPPLIER_CODE,(C_SUPPLIER_MESSAGE),GTK_COMBO(combo));
    c_util_set_code_and_name_on_combo (M_CLIENT_COMPANY_CODE,(C_COMPANY_MESSAGE),GTK_COMBO(combo));
    c_util_set_code_and_name_on_combo (M_CLIENT_CONSORTIUM_CODE,(C_CONSORTIUM_MESSAGE),GTK_COMBO(combo));
#endif
    c_util_set_code_and_name_on_combo (M_CLIENT_GARAGE_CODE,(C_GARAGE_MESSAGE),GTK_COMBO(combo));

}


static void on_client_insertion_window_destroy(GtkWidget *widget,gpointer user_data){
    if (client_insertion!=NULL)
	gtk_widget_destroy(client_insertion);
    client_insertion = NULL;

/*   c_clients_control_find_persons (); */

}


static void close_insertion_window(GtkWidget *widget,gpointer user_data){
    if (accounts_client!=NULL)
	g_list_free(accounts_client);
    if (client_insertion!=NULL)
	gtk_widget_destroy(client_insertion);
  accounts_client = NULL;
  client_insertion = NULL;
}

static void on_person_type_entry_changed(GtkEntry *entry,gpointer user_data){
  GtkWidget * combo;
  gulong code_combo;

  combo = glade_xml_get_widget (xml,"person_combo_insertion_person");	
  code_combo = c_util_get_code_from_combo (combo);

  switch (code_combo)
    {
    case M_CLIENT_PERSON_CODE: show_particular();break;
    case M_CLIENT_ENTERPRISE_CODE: show_enterprise();break;
    case M_CLIENT_SUPPLIER_CODE: show_general_enterprise();break;
    case M_CLIENT_COMPANY_CODE: show_company();break;
    case M_CLIENT_CONSORTIUM_CODE: show_general_enterprise();break;
    case M_CLIENT_GARAGE_CODE: show_garage();break;
    }
}

static void show_particular(){
  GtkWidget *name_label = glade_xml_get_widget(xml, "nombre_label");
  GtkWidget *surname1_label = glade_xml_get_widget(xml, "apellido1_label");
  GtkWidget *surname2_label = glade_xml_get_widget(xml, "apellido2_label");
  GtkWidget *name_entry = glade_xml_get_widget(xml, "nombre_entry");
  GtkWidget *surname1_entry = glade_xml_get_widget(xml, "apellido1_entry");
  GtkWidget *surname2_entry = glade_xml_get_widget(xml, "apellido2_entry");
  GtkWidget *personal_legal_id_label = glade_xml_get_widget(xml, "dni_label");
  GtkWidget *personal_legal_id_entry = glade_xml_get_widget(xml, "dni_entry");
  GtkWidget *trade_name_label = glade_xml_get_widget(xml, "razonsocial_label");
  GtkWidget *trade_name_entry = glade_xml_get_widget(xml, "razonsocial_entry");
  GtkWidget *short_name_label = glade_xml_get_widget(xml, "nombre_corto_label");
  GtkWidget *short_name_entry = glade_xml_get_widget(xml, "nombre_corto_entry");
  GtkWidget *enterprise_legal_id_label = glade_xml_get_widget(xml, "cif_label");
  GtkWidget *enterprise_legal_id_entry = glade_xml_get_widget(xml, "cif_entry");
  GtkWidget *parent_company_checkbutton = glade_xml_get_widget(xml, "parent_company_checkbutton");
  GtkWidget *company_combo_insert = glade_xml_get_widget(xml, "compania_combo_insert");
  GtkWidget *consortium_label = glade_xml_get_widget(xml, "consorcio_label");
  GtkWidget *template_combo_insert = glade_xml_get_widget(xml, "template_combo_insert");
  GtkWidget *template_label = glade_xml_get_widget(xml, "template_label");
  GtkWidget *commission_entry = glade_xml_get_widget(xml, "comision_entry");
  GtkWidget *commission_label = glade_xml_get_widget(xml, "comision_label");
  GtkWidget *discount_company_label = glade_xml_get_widget(xml, "dto_company_label");
  GtkWidget *discount_company_entry = glade_xml_get_widget(xml, "dto_company_entry");
  GtkWidget *ck_label = glade_xml_get_widget(xml, "ck_label");
  GtkWidget *ck_code_entry = glade_xml_get_widget(xml, "ck_code_entry");


  gtk_widget_show(name_label);
  gtk_widget_show(name_entry);

  gtk_widget_show(surname1_label);
  gtk_widget_show(surname1_entry);

  gtk_widget_show(surname2_label);
  gtk_widget_show(surname2_entry);

  gtk_widget_show(personal_legal_id_label);
  gtk_widget_show(personal_legal_id_entry);
  
  gtk_widget_hide(trade_name_label);
  gtk_widget_hide(trade_name_entry);

  gtk_widget_hide(short_name_label);
  gtk_widget_hide(short_name_entry);

  gtk_widget_hide(enterprise_legal_id_label);
  gtk_widget_hide(enterprise_legal_id_entry);

  gtk_widget_hide(parent_company_checkbutton);
  gtk_widget_hide(company_combo_insert);

  gtk_widget_hide(consortium_label);
  gtk_widget_hide(commission_entry);
  gtk_widget_hide(commission_label);

  gtk_widget_hide(discount_company_label);
  gtk_widget_hide(discount_company_entry);
  
  gtk_widget_hide(template_combo_insert);
  gtk_widget_hide(template_label);
  
  gtk_widget_hide(ck_label);
  gtk_widget_hide(ck_code_entry);  
}


static void on_company_entry_changed(GtkEntry *entry,gpointer user_data){
 
  GtkWidget *ck_label = glade_xml_get_widget(xml, "ck_label");
  GtkWidget *ck_entry = glade_xml_get_widget(xml, "ck_code_entry");
  GtkWidget * combo = glade_xml_get_widget (xml,"compania_combo_insert");
  gulong code_combo = c_util_get_code_from_combo (combo);


  
  
  gtk_widget_hide(ck_label);
  gtk_widget_hide(ck_entry);
  gtk_entry_set_text(GTK_ENTRY(ck_entry),"");
     
}

static void show_enterprise(){
#ifndef IS_SHOP
  GtkWidget *parent_company_checkbutton = glade_xml_get_widget(xml, "parent_company_checkbutton");
#endif

  show_general_enterprise();

#ifndef IS_SHOP
  gtk_widget_show(parent_company_checkbutton);
#endif
}

static void show_company(){
  GtkWidget *company_combo_insert = glade_xml_get_widget(xml, "compania_combo_insert");
  gulong company_combo_code = c_util_get_code_from_combo (company_combo_insert);
  GtkWidget *template_combo_insert = glade_xml_get_widget(xml, "template_combo_insert");
  GtkWidget *consortium_label = glade_xml_get_widget(xml, "consorcio_label");
  GtkWidget *template_label = glade_xml_get_widget(xml, "template_label");
  GtkWidget *discount_company_label = glade_xml_get_widget(xml, "dto_company_label");
  GtkWidget *discount_company_entry = glade_xml_get_widget(xml, "dto_company_entry");
  
  fill_template_combo();
    	
  show_general_enterprise();
  gtk_widget_show(company_combo_insert);
  gtk_widget_show(consortium_label);
  gtk_widget_show(discount_company_label);
  gtk_widget_show(discount_company_entry);
  gtk_widget_show(template_combo_insert);
  gtk_widget_show(template_label);
  
	
}

static void show_garage(){
  GtkWidget *commission_entry = glade_xml_get_widget(xml, "comision_entry");
  GtkWidget *commission_label = glade_xml_get_widget(xml, "comision_label");

  show_general_enterprise();
  gtk_widget_show(commission_entry);
  gtk_widget_show(commission_label);
}

static void show_general_enterprise(){
  GtkWidget *name_label = glade_xml_get_widget(xml, "nombre_label");
  GtkWidget *surname1_label = glade_xml_get_widget(xml, "apellido1_label");
  GtkWidget *surname2_label = glade_xml_get_widget(xml, "apellido2_label");
  GtkWidget *name_entry = glade_xml_get_widget(xml, "nombre_entry");
  GtkWidget *surname1_entry = glade_xml_get_widget(xml, "apellido1_entry");
  GtkWidget *surname2_entry = glade_xml_get_widget(xml, "apellido2_entry");
  GtkWidget *personal_legal_id_label = glade_xml_get_widget(xml, "dni_label");
  GtkWidget *personal_legal_id_entry = glade_xml_get_widget(xml, "dni_entry");
  GtkWidget *trade_name_label = glade_xml_get_widget(xml, "razonsocial_label");
  GtkWidget *trade_name_entry = glade_xml_get_widget(xml, "razonsocial_entry");
  GtkWidget *short_name_label = glade_xml_get_widget(xml, "nombre_corto_label");
  GtkWidget *short_name_entry = glade_xml_get_widget(xml, "nombre_corto_entry");  
  GtkWidget *enterprise_legal_id_label = glade_xml_get_widget(xml, "cif_label");
  GtkWidget *enterprise_legal_id_entry = glade_xml_get_widget(xml, "cif_entry");
  GtkWidget *parent_company_checkbutton = glade_xml_get_widget(xml, "parent_company_checkbutton");
  GtkWidget *company_combo_insert = glade_xml_get_widget(xml, "compania_combo_insert");
  GtkWidget *consortium_label = glade_xml_get_widget(xml, "consorcio_label");
  GtkWidget *commission_entry = glade_xml_get_widget(xml, "comision_entry");
  GtkWidget *commission_label = glade_xml_get_widget(xml, "comision_label");
  GtkWidget *discount_company_label = glade_xml_get_widget(xml, "dto_company_label");
  GtkWidget *discount_company_entry = glade_xml_get_widget(xml, "dto_company_entry");
  GtkWidget *ck_label = glade_xml_get_widget(xml, "ck_label");
  GtkWidget *ck_code_entry = glade_xml_get_widget(xml, "ck_code_entry");
  GtkWidget *template_combo_insert = glade_xml_get_widget(xml, "template_combo_insert");
  GtkWidget *template_label = glade_xml_get_widget(xml, "template_label");

  gtk_widget_hide(name_label);
  gtk_widget_hide(name_entry);

  gtk_widget_hide(surname1_label);
  gtk_widget_hide(surname1_entry);

  gtk_widget_hide(surname2_label);
  gtk_widget_hide(surname2_entry);

  gtk_widget_hide(personal_legal_id_label);
  gtk_widget_hide(personal_legal_id_entry);
  
  gtk_widget_show(trade_name_label);
  gtk_widget_show(trade_name_entry);
  gtk_widget_show(short_name_label);
  gtk_widget_show(short_name_entry);

  gtk_widget_show(enterprise_legal_id_label);
  gtk_widget_show(enterprise_legal_id_entry);

  gtk_widget_hide(parent_company_checkbutton);
  gtk_widget_hide(company_combo_insert);

  gtk_widget_hide(consortium_label);
  gtk_widget_hide(commission_entry);
  gtk_widget_hide(commission_label);

  gtk_widget_hide(discount_company_label);
  gtk_widget_hide(discount_company_entry);


  gtk_widget_hide(template_combo_insert);
  gtk_widget_hide(template_label);
  
  gtk_widget_hide(ck_label);
  gtk_widget_hide(ck_code_entry);  
}

static void clean_screen() {                                                                

  GtkWidget *container = GTK_WIDGET(glade_xml_get_widget(xml, widget_name));
  
  gtk_container_foreach(GTK_CONTAINER(container),&c_util_clean_widget,NULL);

}

static void search_existent_clients(gint reply,
				    gpointer user_data) {
    if (!reply) {
	Participant ** participants = (Participant **) user_data;
	c_clients_control_fill_window (participants);
	if (client_insertion!=NULL)
	    gtk_widget_destroy(client_insertion);
	if (accounts_client!=NULL)
	    g_list_free(accounts_client);
        accounts_client = NULL;
	client_insertion=NULL;

    } else {
        insert_person();
    }
}

static void on_trade_name_entry_insert_text(GtkEntry *entry, gchar *new_text, gint new_text_length, 
	gint *position, gpointer user_data) {
	    
	    
	GtkWidget * short_name_entry = glade_xml_get_widget (xml,"nombre_corto_entry");

	gint * new_pos= g_malloc0(sizeof(gint));
	*new_pos = *position;
				
	gtk_editable_insert_text(GTK_EDITABLE(short_name_entry), new_text, new_text_length, new_pos);			
}


static void on_trade_name_entry_delete_text(GtkEntry *entry, gint start_pos, gint end_pos, gpointer user_data) {	

	    
	GtkWidget * short_name_entry = glade_xml_get_widget (xml,"nombre_corto_entry");	
							
	gtk_editable_delete_text(GTK_EDITABLE(short_name_entry), start_pos, end_pos);				
}



static void on_trade_name_entry_move_cursor(GtkEntry *entry, gint x, gint y, gpointer user_data) {

	    
	GtkWidget * short_name_entry = glade_xml_get_widget (xml,"nombre_corto_entry");		
					
	gtk_editable_set_position(GTK_EDITABLE(short_name_entry), x);				
}



static void on_insert_button_clicked() {
    GtkWidget *combo = glade_xml_get_widget (xml, "person_combo_insertion_person");
    GtkWidget *entry_enterprise_legal_id;
    gulong type = c_util_get_code_from_combo (combo);
    gchar * enterprise_legal_id;
    gchar ** tokens;
    Participant ** participants;

    g_print("Insertando\n");
    switch (type)
    {
    case M_CLIENT_PERSON_CODE:{
	entry_enterprise_legal_id = glade_xml_get_widget (xml, "dni_entry");
	break;}
    default: {
	entry_enterprise_legal_id = glade_xml_get_widget (xml, "cif_entry");
    }
    }

    enterprise_legal_id = gtk_editable_get_chars(GTK_EDITABLE(entry_enterprise_legal_id),0,-1);

    if (strcmp(enterprise_legal_id,"")) {
	tokens = u_cif_get_tokens (enterprise_legal_id);

	participants = m_client_get_participants_by_legal_id (tokens);

	if (participants!=NULL) {
	  c_util_open_question_dialog_with_param (_("There is already one client with that tax code, do you want to search it or to insert it?\nIf you press \"NO\" a new client will be inserted, posibly duplicated"), (*search_existent_clients),participants);

	} else {
	    insert_person();
	}
    } else {
	insert_person();

    }
}


static void insert_person() {
  
  Client *client = d_client_create_empty ();
  Participant *participant = d_participant_create_empty ();
  GtkWidget *combo = glade_xml_get_widget (xml, "person_combo_insertion_person");
  gulong type = c_util_get_code_from_combo (combo);
  gchar *street = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "calle_entry")),0,-1);
  gchar *number = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "numero_entry")),0,-1);
  gchar *floor = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "piso_entry")),0,-1);
  gchar *cp = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "cp_entry")),0,-1);
  gchar *town = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "municipio_entry")),0,-1);
  gchar *province = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "provincia_entry")),0,-1);
  gchar *phone1 = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "telefono1_entry")),0,-1);
  gchar *phone2 = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "telefono2_entry")),0,-1);
  gchar *fax = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "fax_entry")),0,-1);
  gchar *email = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "email_entry")),0,-1);
  gchar *discount = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "dto_entry")),0,-1);
  gchar *short_name = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "nombre_corto_entry")),0,-1);
  gboolean result = FALSE;
  gchar ** stop;

  stop = (gchar **) g_malloc0(sizeof(gchar *));

  
  d_client_set_participant (client, participant);
 
  if (g_strcasecmp(short_name,"")) 
    d_participant_set_short_name (participant, short_name);
 
  if (g_strcasecmp(email,""))
    d_participant_set_email (participant, email);
  if (g_strcasecmp(phone1,""))
    d_participant_set_phone (participant, phone1);
  if (g_strcasecmp(phone2,""))
    d_participant_set_phone2 (participant,phone2);
  if (g_strcasecmp(fax,""))
    d_participant_set_fax (participant,fax);
  if (g_strcasecmp(street,""))
    d_participant_set_street (participant,street);
  if (g_strcasecmp(number,""))
    d_participant_set_number (participant,number);
  if (g_strcasecmp(floor,""))
    d_participant_set_floor (participant, floor);
  if (g_strcasecmp(cp,""))
    d_participant_set_postal_code (participant, strtoul(cp,stop,10));
  if (g_strcasecmp(town,""))
    d_participant_set_town (participant, town);
  if (g_strcasecmp(province,""))
      d_participant_set_province (participant, province);
  if (g_strcasecmp(discount,"")) {
      u_points_to_commas (discount);
      d_client_set_discount (client,atof(discount));
  }
  switch (type)
    {
    case M_CLIENT_PERSON_CODE:{
      ParticularObject *particular = d_particular_create_empty ();
      gchar *personal_legal_id = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "dni_entry")),0,-1);
      gchar *name = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "nombre_entry")),0,-1);
      gchar *surname1 = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "apellido1_entry")),0,-1);
      gchar *surname2 = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "apellido2_entry")),0,-1);      
      gchar *nick = g_strconcat(name," ",surname1," ",surname2);

      if (g_strcasecmp(personal_legal_id,""))
	d_participant_set_legal_id (participant, personal_legal_id);
      
      if (g_strcasecmp(nick,"")) {
	d_participant_set_nick (participant, nick);
     	d_participant_set_short_name (participant, nick);
      }

      if (g_strcasecmp(name,""))
	d_participant_set_name (participant, name);
      if (g_strcasecmp(surname1,""))
	d_particular_set_first_surname (particular, surname1);
      if (g_strcasecmp(surname2,""))
	d_particular_set_second_surname (particular, surname2);

      d_particular_set_participant (particular, participant);
      
      result = m_client_insert ((gpointer) particular, client, type,accounts_client);
      
      d_particular_clear (particular);      
      g_free(personal_legal_id);
      g_free(name);
      g_free(surname1);
      g_free(surname2);
      g_free(nick);
      break;}

    case M_CLIENT_ENTERPRISE_CODE:{

      Enterprise *enterprise = d_enterprise_create_empty ();
      gchar *enterprise_legal_id = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "cif_entry")),0,-1);

      gchar *trade_name = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "razonsocial_entry")),0,-1);
      
      gboolean parent_company_checkbutton = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget (xml, "parent_company_checkbutton")));

      if (g_strcasecmp(enterprise_legal_id,""))
	d_participant_set_legal_id (participant, enterprise_legal_id);
      if (g_strcasecmp(trade_name,""))
	d_participant_set_nick (participant, trade_name);
      if (g_strcasecmp(trade_name,""))
	d_participant_set_name (participant, trade_name);
	
      d_enterprise_set_belong_to_principal (enterprise, parent_company_checkbutton);
	
      d_enterprise_set_participant (enterprise, participant);

      result = m_client_insert ((gpointer) enterprise, client, type,accounts_client);

      d_enterprise_clear (enterprise);
      g_free(enterprise_legal_id);
      g_free(trade_name);
      break;}
    
    case M_CLIENT_SUPPLIER_CODE:{
      Supplier *supplier = d_supplier_create_empty ();
      gchar *enterprise_legal_id = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "cif_entry")),0,-1);
      gchar *trade_name = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "razonsocial_entry")),0,-1);
      

      if (g_strcasecmp(enterprise_legal_id,""))
	d_participant_set_legal_id (participant, enterprise_legal_id);
      if (g_strcasecmp(trade_name,""))
	d_participant_set_nick (participant, trade_name);
      if (g_strcasecmp(trade_name,""))
	d_participant_set_name (participant, trade_name);

      d_supplier_set_participant (supplier, participant);

      result = m_client_insert ((gpointer) supplier, client, type,accounts_client);

      d_supplier_clear (supplier);
      g_free(enterprise_legal_id);
      g_free(trade_name);
      break;}
   
    case M_CLIENT_COMPANY_CODE:{
   
      Company *company = d_company_create_empty ();
      gchar *enterprise_legal_id = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "cif_entry")),0,-1);
      gchar *trade_name = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "razonsocial_entry")),0,-1);
      GtkWidget *combo_consortium = glade_xml_get_widget (xml, "compania_combo_insert");
      GtkWidget *combo_plantilla = glade_xml_get_widget (xml, "template_combo_insert");      
      gchar *ck_code = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "ck_code_entry")),0,-1);
      gchar *discount = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "dto_company_entry")),0,-1);
      gulong consortium = c_util_get_code_from_combo (combo_consortium);
      gint template = c_util_get_code_from_combo (combo_plantilla);
      
      
      if (g_strcasecmp(enterprise_legal_id,""))
	d_participant_set_legal_id (participant, enterprise_legal_id);
	
      if (g_strcasecmp(trade_name,""))
	d_participant_set_nick (participant, trade_name);

      if (g_strcasecmp(trade_name,""))
	d_participant_set_name (participant, trade_name);
	
      d_company_set_consortium_code (company, consortium);
      d_company_set_template (company, template);
      
      if (g_strcasecmp(discount,"")) {
	  u_points_to_commas (discount);
	  d_company_set_discount (company, atof(discount));
      }
      if (g_strcasecmp(ck_code,""))
	d_company_set_ck_code (company, ck_code);            
                    
      d_company_set_participant (company, participant);

      result = m_client_insert ((gpointer) company, client, type,accounts_client);

      d_company_clear (company);
      g_free(enterprise_legal_id);
      g_free(trade_name);
      g_free(discount);
      break;}

    case M_CLIENT_CONSORTIUM_CODE:{

      Consortium *consortium = d_consortium_create_empty ();
      gchar *enterprise_legal_id = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "cif_entry")),0,-1);
      gchar *trade_name = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "razonsocial_entry")),0,-1);

      if (g_strcasecmp(enterprise_legal_id,""))
	d_participant_set_legal_id (participant, enterprise_legal_id);
      if (g_strcasecmp(trade_name,""))
	d_participant_set_nick (participant, trade_name);	
      if (g_strcasecmp(trade_name,""))
	d_participant_set_name (participant, trade_name);

      d_consortium_set_participant (consortium, participant);

      result = m_client_insert ((gpointer) consortium, client, type,accounts_client);

      d_consortium_clear (consortium);
      g_free(enterprise_legal_id);
      g_free(trade_name);
      break;}
 
    case M_CLIENT_GARAGE_CODE:{
 
      Garage *garage = d_garage_create_empty ();
      gchar *enterprise_legal_id = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "cif_entry")),0,-1);
      gchar *trade_name = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "razonsocial_entry")),0,-1);
      gchar *commission = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "comision_entry")),0,-1);

      if (g_strcasecmp(enterprise_legal_id,""))
	d_participant_set_legal_id (participant, enterprise_legal_id);

      if (g_strcasecmp(trade_name,""))
	d_participant_set_nick (participant, trade_name);	
      if (g_strcasecmp(trade_name,""))
	d_participant_set_name (participant, trade_name);

      if (g_strcasecmp(commission,"")) {
	  u_points_to_commas (commission);
	  d_garage_set_commission (garage, atof(commission));
      }
      d_garage_set_participant (garage, participant);

      result = m_client_insert ((gpointer) garage, client, type,accounts_client);

      d_garage_clear (garage);
      g_free(enterprise_legal_id);
      g_free(trade_name);
      g_free(commission);
      break;}
    }

  g_free(short_name);      
  g_free(stop);
  g_free(street);
  g_free(number);
  g_free(discount);
  g_free(email);
  g_free(fax);
  g_free(phone2);
  g_free(phone1);
  g_free(province);
  g_free(town);
  g_free(cp);
  g_free(floor);
  d_client_set_participant (client, NULL);
  d_client_clear (client);

  if (result) {
      if (accounts_client!=NULL)
	  g_list_free(accounts_client);
      if (client_insertion!=NULL)
	  gtk_widget_destroy(client_insertion);
      accounts_client=NULL;
      client_insertion = NULL;
  }
  else c_util_open_error_window (_("The client couldn't be inserted, check the data"));
}

static void repaint_accounts(gpointer value) {
}

static void set_account_checkbutton_state(gboolean new_state) {
    GtkWidget * checkbutton_cuenta = glade_xml_get_widget(xml,"checkbutton_cuenta_cliente");

    event_changed = FALSE;
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbutton_cuenta),new_state);

    event_changed = TRUE;
}

static void on_client_account_checkbutton_toggled(GtkToggleButton *togglebutton,
						  gpointer user_data) {

    if (event_changed!=FALSE) {
	GtkWidget * open_insert = c_insert_account_open_window(repaint_accounts,1);
	GtkWidget * app1 = lookup_widget(GTK_WIDGET(open_insert), "app1");
	if (open_insert!=NULL) {
	    gtk_window_set_modal(GTK_WINDOW(open_insert),C_ACCOUNT_FROM_INSERT_CLIENTS);
	    gtk_window_set_position(GTK_WINDOW(open_insert),GTK_WIN_POS_CENTER);
	    gtk_window_set_transient_for(GTK_WINDOW(open_insert),
					 GTK_WINDOW(app1));
	    gtk_widget_show(GTK_WIDGET(open_insert));
	}
        set_account_checkbutton_state(FALSE);
    }
}

void c_clients_insert(Account * account) {
    accounts_client = g_list_append(accounts_client,account);
    add_to_account_combo(account);

}

static void add_to_account_combo(Account * account) {
    GtkWidget * combo;
    gchar * complet_account_number=NULL;

    combo = glade_xml_get_widget (xml, "combo_cuentas_inserta_clientes");
    complet_account_number=d_account_get_complete_number(account);
    c_util_set_code_and_name_on_combo (g_list_length(accounts_client),complet_account_number,GTK_COMBO(combo));
    g_free(complet_account_number);

}


