
/* insert_document_control.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include <gnome.h>
#include <glade/glade.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <glib.h>
#include <time.h>

#include "a-shortcut.h"
#include "support.h"
#include "document_model.h"
#include "insert_document_control.h"
#include "associated_comp_documents_control.h"

#ifndef IS_SHOP
static GtkWidget *widget_document=NULL;
static gchar url[]="/src/interface/glade/documents_view.glade";
static gchar widget_name[]="vbox_introducciondocumentos";
static GladeXML *xml = NULL;

static Document *document_loaded = NULL;

static void clean_screen(void);
static void on_save_button_clicked(GtkButton *button, gpointer data);
static void on_clean_button_clicked(GtkButton *button, gpointer data);

static void associate_company(GtkWidget *app1);
#endif

GtkWidget *c_document_open_insertion_window(GtkWidget *app1, GtkWidget *vbox) {

#ifndef IS_SHOP

    gchar * directory = g_strconcat(AA_DATA_DIR,url,NULL);

    if (widget_document==NULL) {
	GtkWidget *dialog;

	xml = glade_xml_new (directory, widget_name);
	dialog = glade_xml_get_widget (xml, widget_name);


	gtk_box_pack_start (GTK_BOX (vbox), dialog, TRUE, TRUE, 0);

	glade_xml_signal_connect (xml, "on_boton-guardar-insertardocumentos_clicked",
				  GTK_SIGNAL_FUNC (on_save_button_clicked));
	glade_xml_signal_connect (xml, "on_boton-limpiar-insertardocumentos_clicked",
				  GTK_SIGNAL_FUNC (on_clean_button_clicked));

	widget_document = dialog;
    }

    clean_screen();

    g_free(directory);
    return widget_document;
#else
    return NULL;
#endif

}

void c_document_load(Document *document) {
#ifndef IS_SHOP
    gchar * temp;
    gboolean from=FALSE;
    gint  position = 0;
    gint  size = 0;

    if(document!=NULL) {
	d_document_clear (document_loaded);

        temp = d_document_get_name (document);
	gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(xml, "entry-nombre-introducciondocumentos")), temp);
	gtk_entry_set_position(GTK_ENTRY(glade_xml_get_widget(xml, "entry-nombre-introducciondocumentos")), 0);
        g_free(temp);

	temp = d_document_get_description (document);
        size = strlen(temp);
	gtk_editable_delete_text(GTK_EDITABLE(glade_xml_get_widget(xml,  "text-descripcion-introducciondocumentos")), 0, -1);
	gtk_editable_insert_text(GTK_EDITABLE(glade_xml_get_widget(xml,  "text-descripcion-introducciondocumentos")), temp, size, &position);
	gtk_editable_set_position(GTK_EDITABLE(glade_xml_get_widget(xml, "text-descripcion-introducciondocumentos")), 0);

	from = d_document_get_from_invoice(document);
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(xml, "checkbutton-facturacin-introducciondocumentos")),from);

	from = d_document_get_from_repair(document);
        gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(xml, "checkbutton-reparacin-introducciondocumentos")),from);	
        document_loaded = d_document_duplicate (document);
    }
#endif
}

#ifndef IS_SHOP
static void on_save_button_clicked(GtkButton *button, gpointer data) {


    GtkWidget *app1  = lookup_widget(GTK_WIDGET(button), "app1");
    GtkWidget *facturation_checkbutton = glade_xml_get_widget(xml, "checkbutton-facturacin-introducciondocumentos");
    GtkWidget *repair_checkbutton = glade_xml_get_widget(xml, "checkbutton-reparacin-introducciondocumentos");
    gboolean   active = FALSE;
    gboolean   correct = FALSE;
    gchar     *name        = g_strstrip(gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "entry-nombre-introducciondocumentos")),0,-1));
    gchar     *description = g_strstrip(gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "text-descripcion-introducciondocumentos")),0,-1));
    gboolean  new_document = FALSE;

    if ((strlen(name)>1) && (strlen(description)>1) ) {

	gtk_entry_set_position(GTK_ENTRY(glade_xml_get_widget(xml, "entry-nombre-introducciondocumentos")), 0);

	if(document_loaded == NULL) {
	    document_loaded = d_document_create_empty ();
            new_document = TRUE;
	} 

	d_document_set_name (document_loaded, name);
	d_document_set_description (document_loaded, description);

	active = (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(repair_checkbutton))==TRUE);
	if (active) {
	    d_document_set_from_repair (document_loaded, TRUE);
	} else {
	    d_document_set_from_repair (document_loaded, FALSE);
	}

	active = (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(facturation_checkbutton))==TRUE);
	if (active) {
	    d_document_set_from_invoice (document_loaded, TRUE);
	} else {
	    d_document_set_from_invoice (document_loaded, FALSE);
	}

	if(new_document) {
	    correct = m_document_insert (document_loaded);
	} else {
	    correct = m_document_update (document_loaded);
	}

	if(!correct) {
	    c_util_open_error_window (_("The update of the document couldn't be done correctly. There has been a problem with the database.\n Try it again\n"));
	}
	associate_company(app1);

    } else {
	c_util_open_error_window (_("Fill the fields Name and Description to insert a document"));
    }

    g_free(name);
    g_free(description);

}

static void on_clean_button_clicked(GtkButton *button, gpointer data) {
    clean_screen();
}

static void associate_company(GtkWidget *app1) {
    GtkWidget *open_associated = c_associated_companies_open (app1,NULL, document_loaded);

    if (open_associated!=NULL) {
	gtk_window_set_modal(GTK_WINDOW(open_associated),1);
	gtk_window_set_position(GTK_WINDOW(open_associated),GTK_WIN_POS_CENTER);
	gtk_window_set_transient_for(GTK_WINDOW(open_associated),
				     GTK_WINDOW(app1));
	gtk_widget_show(GTK_WIDGET(open_associated));
    }

    clean_screen();
}

static void clean_screen() {

    GtkWidget *vbox;

    vbox = GTK_WIDGET(glade_xml_get_widget(xml, widget_name));
    gtk_container_foreach(GTK_CONTAINER(vbox),&c_util_clean_widget,NULL);
    d_document_clear (document_loaded);
    document_loaded = NULL;
}

#endif

