/* insert_measure_control.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include <gnome.h>
#include <glade/glade.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <glib.h>
#include <stdlib.h>


#include "a-shortcut.h"
#include "support.h"
#include "control_util.h"
#include "insert_measure_control.h"
#include "measure_model.h"
#include "invoice_control.h"
#include "piece_control.h"

static GtkWidget *widget_insert_measure=NULL;
static GladeXML *xml;
static gchar widget_name[]="window_medida";
static gchar url[]="/src/interface/glade/store_view.glade";

static void on_cancel_button_clicked(GtkButton *button, gpointer user_data);
static void on_accept_button_clicked(GtkButton *button,  gpointer user_data);
static Measure * create_measure_from_screen(void);
static void on_meassure_window_destroy(GtkWidget *wid,gpointer user_data);

GtkWidget *c_meassure_open_window(GtkWidget *app1) {
    gchar * directory = g_strconcat(AA_DATA_DIR,url,NULL);

    if (widget_insert_measure==NULL)
    {
	GtkWidget *dialog;
    
	xml = glade_xml_new (directory, widget_name);
	dialog = glade_xml_get_widget (xml, widget_name);


	glade_xml_signal_connect (xml, "on_cancelar_medida_clicked",
				  GTK_SIGNAL_FUNC (on_cancel_button_clicked));
	glade_xml_signal_connect (xml, "on_aceptar_medida_clicked",
			          GTK_SIGNAL_FUNC (on_accept_button_clicked));
	glade_xml_signal_connect (xml, "on_window_medida_destroy",
			          GTK_SIGNAL_FUNC (on_meassure_window_destroy));

	widget_insert_measure=dialog;
    }
    g_free(directory);
    return widget_insert_measure;
}


static void on_cancel_button_clicked(GtkButton *button,gpointer user_data) {

    gtk_widget_destroy(widget_insert_measure);

    widget_insert_measure = NULL;
}

static Measure * create_measure_from_screen() {
    Measure * result = d_measure_create_empty ();
    gchar * name = g_strdup(gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(xml,"entry_nombre_medida"))));
    gchar * abrev = g_strdup(gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(xml,"entry_abreviatura_medida"))));

    if (!strcmp(name,"")) {
	g_free(name);
        name=NULL;
    } else {
	d_measure_set_name (result,name);
    }
    if (!strcmp(abrev,"")) {
	g_free(abrev);
        abrev=NULL;
    } else {
	d_measure_set_abbreviation (result,abrev);
    }
    return result;
}

static void on_accept_button_clicked(GtkButton *button,gpointer user_data) {
    gboolean result =FALSE;
    Measure * measure = create_measure_from_screen();
    result = m_meassure_insert (measure);
    if (result == FALSE) c_util_open_error_window (_("Measure insertion error"));
    else {
        c_piece_repaint_measures();
	c_util_open_info_window (_("Measure inserted correctly"));
    }
    gtk_widget_destroy(widget_insert_measure);
    widget_insert_measure = NULL;
	

}

static void on_meassure_window_destroy(GtkWidget *wid,gpointer user_data) {
    gtk_widget_destroy(widget_insert_measure);

    widget_insert_measure = NULL;

}



