/* insert_operation_control.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include <gnome.h>
#include <glade/glade.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <glib.h>
#include <stdlib.h>


#include "a-shortcut.h"
#include "support.h"
#include "util.h"
#include "control_util.h"
#include "insert_operation_control.h"
#include "insert_account_control.h"
#include "account_model.h"
#include "operation_model.h"

static GtkWidget *insert_operations=NULL;
static GladeXML *xml;
static void (*function_to_call)(gpointer value);
static void (*function2_to_call)(gpointer value);
static gdouble total_operation;
static gboolean refactura=FALSE;
static gboolean from_aceptar=FALSE;
static gchar widget_name[]="introduccion_de_movimientos";
static gchar url[]="/src/interface/glade/invoice_view.glade";
static void on_insert_account_checkbutton_toggled(GtkToggleButton *togglebutton,
						 gpointer user_data);
static void on_accept_button_clicked(GtkButton *button, gpointer data);
static void on_cancel_button_clicked(GtkButton *button, gpointer data);
static void on_account_combo_entry_changed(GtkEntry *entry,
					  gpointer user_data);
static void fill_accounts_combo(void);
static void on_window_destroy(GtkWidget * widget,gpointer user_data);
static gboolean insert_operation(void);
static void fill_total_data(gdouble total_data);
static void fill_payment_code(void);
void repaint(gpointer value);

static void clean_screen(void);

GtkWidget *c_insert_operation_open_window(GtkWidget *app1,gpointer f_to_call,gpointer f2_to_call,gdouble total)
{
   gchar * directory = g_strconcat(AA_DATA_DIR,url,NULL);
   
  if (insert_operations==NULL)
    {
      GtkWidget *dialog;

      xml = glade_xml_new (directory, widget_name);
      dialog = glade_xml_get_widget (xml, widget_name);

      glade_xml_signal_connect (xml, "on_checkbutton_inserta_cuenta_toggled",
				GTK_SIGNAL_FUNC (on_insert_account_checkbutton_toggled));
      glade_xml_signal_connect (xml, "on_button_aceptar_movimientos_clicked",
				GTK_SIGNAL_FUNC (on_accept_button_clicked));
      glade_xml_signal_connect (xml, "on_button_cancelar_movimientos_clicked",
				GTK_SIGNAL_FUNC (on_cancel_button_clicked));
      glade_xml_signal_connect (xml, "on_combo_entry_cuenta_changed",
				GTK_SIGNAL_FUNC (on_account_combo_entry_changed));
                                      
      glade_xml_signal_connect (xml, "on_introduccion_de_movimientos_destroy",
				GTK_SIGNAL_FUNC (on_window_destroy));
                                                 
      fill_accounts_combo();
      insert_operations=dialog;          
      
    }
  else {
      clean_screen();
  }
  total_operation = total;
  fill_total_data(total);
  g_free(directory);
  function_to_call = f_to_call;
  function2_to_call = f2_to_call;
  fill_payment_code();
  return insert_operations;
  
}

static void fill_total_data(gdouble total_data) {
    GtkWidget * entry_total = glade_xml_get_widget(xml,"entry_cantidad_movimiento");
    gchar * total_char = g_strdup_printf("%.2f",total_data);
    gtk_entry_set_text(GTK_ENTRY(entry_total),total_char);
}

static void clean_screen() {

    GtkWidget *vbox;

    vbox = GTK_WIDGET(glade_xml_get_widget(xml, widget_name));

    gtk_container_foreach(GTK_CONTAINER(vbox),&c_util_clean_widget,NULL);

}



static void on_insert_account_checkbutton_toggled(GtkToggleButton *togglebutton,
						  gpointer user_data) {
//    GtkWidget * app1 = lookup_widget(GTK_WIDGET(togglebutton), "app1");
    GtkWidget * t_button =glade_xml_get_widget(xml,"checkbutton_inserta_cuenta");
    GtkWidget * open_insert;
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(t_button))!=FALSE) {
	open_insert = c_insert_account_open_window(repaint,0);
	if (open_insert!=NULL) {
	    gtk_window_set_modal(GTK_WINDOW(open_insert),1);
	    gtk_window_set_position(GTK_WINDOW(open_insert),GTK_WIN_POS_CENTER);
	    gtk_widget_show(GTK_WIDGET(open_insert));
	}
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(t_button),FALSE);

    }

}

void c_insert_operation_from_split_invoice(gboolean refactura_aux) {

    if (refactura_aux != FALSE) {
        GtkWidget * button_cancelar = glade_xml_get_widget(xml,"button_cancelar_movimientos");
	if GTK_WIDGET_VISIBLE(button_cancelar) gtk_widget_hide(button_cancelar);
	refactura = refactura_aux;

    }

}


static void on_accept_button_clicked(GtkButton *button, gpointer data) {
    GtkWidget * combo=glade_xml_get_widget(xml,"combo_cuentas_movimientos");
    GtkWidget * entry_combo = glade_xml_get_widget(xml,"combo_entry_cuenta_movimiento");
    gboolean error = FALSE;
    gboolean * pointer_error = (gboolean *) g_malloc0(sizeof(gboolean));
    gpointer data1 ;
    gchar * account_data = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry_combo)));

    g_strstrip(account_data);
    if (!strcmp(account_data,"")) {
	error =TRUE;
    }

    g_free(account_data);

    if (error==FALSE) {
	error = insert_operation();
	*pointer_error = error;
	data1 = (gpointer) pointer_error;
	//    if (function2_to_call!=NULL) {
	//	function2_to_call(data1);
	//    }
	if (error == FALSE) {

	    c_util_clean_code_and_name_combo (combo);
	    from_aceptar=TRUE;
	    gtk_widget_destroy(insert_operations);
	    insert_operations = NULL;
	    if (function2_to_call!=NULL) {
		function2_to_call(data1);
	    }

	} else {
	    c_util_open_error_window (_("Error inserting the bank movement"));
	}

    } else {
	    c_util_open_error_window (_("Insert at least a bank account"));
    }

}
static void on_cancel_button_clicked(GtkButton *button, gpointer data){
    GtkWidget * combo=glade_xml_get_widget(xml,"combo_cuentas_movimientos");

    from_aceptar=FALSE;
    c_util_clean_code_and_name_combo (combo);
    gtk_widget_destroy(insert_operations);
    insert_operations = NULL;
}

static void fill_accounts_combo(void) {
    GtkWidget * combo = glade_xml_get_widget(xml,"combo_cuentas_movimientos");

    Account ** accounts = m_account_find_all ();
    gint index=0;
    if (accounts!=NULL) {
	while (accounts[index]!=NULL) {
	    gchar * complete_account_number=NULL;
            complete_account_number = d_account_get_complete_number(accounts[index]);
	    c_util_set_code_and_name_on_combo (index,complete_account_number,GTK_COMBO(combo));
	    g_free(complete_account_number);
	    index++;
	}
    }
}

void repaint(gpointer value) {
    GtkWidget * combo=glade_xml_get_widget(xml,"combo_cuentas_movimientos");
    c_util_clean_code_and_name_combo (combo);
    fill_accounts_combo();
}

static void on_account_combo_entry_changed(GtkEntry *entry,
					  gpointer user_data) {

    GtkWidget * entry_ccc = glade_xml_get_widget(xml,"combo_entry_cuenta_movimiento");
    GtkWidget * entry_bank = glade_xml_get_widget(xml,"entry_nombre_banco");
    gchar * account_number ;
    gchar * bank_number ;
    gchar * bank_name ;
    gchar ** tokens;

    account_number= g_strdup(gtk_entry_get_text(GTK_ENTRY(entry_ccc)));
    tokens = g_strsplit(account_number," ",1);
    bank_number = g_strdup(tokens[0]);
    g_strfreev(tokens);
    bank_name = m_account_find_bank_by_code (bank_number);
    g_free(bank_number);
    gtk_entry_set_text(GTK_ENTRY(entry_bank),bank_name);
    g_free(bank_name);

}

static void on_window_destroy(GtkWidget * widget,gpointer user_data) {
            
    GtkWidget * combo=glade_xml_get_widget(xml,"combo_cuentas_movimientos");
    if (refactura!=TRUE) {
	c_util_clean_code_and_name_combo (combo);
	gtk_widget_destroy(insert_operations);
	insert_operations = NULL;
    } else {
	if (from_aceptar==FALSE) {
//        gtk_signal_emit_stop_by_name(GTK_OBJECT (insert_operations),"destroy");
	}
    }
    from_aceptar=FALSE;
}

static gboolean insert_operation() {
    GtkWidget * entry_numero_cuenta = glade_xml_get_widget(xml,"combo_entry_cuenta_movimiento");
    GtkWidget * text_description = glade_xml_get_widget(xml,"text_descripcion");
    GtkWidget * movement_date_dateedit = glade_xml_get_widget(xml,"dateedit_fecha_movimiento");
    GtkWidget * notification_dateedit = glade_xml_get_widget(xml,"dateedit_notificacion");
    GtkWidget * quantity_movement_entry = glade_xml_get_widget(xml,"entry_cantidad_movimiento");
    GtkWidget * payment_code_entry = glade_xml_get_widget(xml,"entry_codigo_pago_movimientos");
    GtkWidget * reference_entry = glade_xml_get_widget(xml,"entry_recibo_bancario");
    gchar * complete_account_numner = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry_numero_cuenta)));
    gchar * description;
    gchar * recepit_reference=NULL;
    time_t time_operation = gnome_date_edit_get_date(GNOME_DATE_EDIT(movement_date_dateedit));
    time_t time_notification = gnome_date_edit_get_date(GNOME_DATE_EDIT(notification_dateedit));
    gchar * quantity_char = g_strdup(gtk_entry_get_text(GTK_ENTRY(quantity_movement_entry)));
    gdouble quantity;
    gchar * payment_code = g_strdup(gtk_entry_get_text(GTK_ENTRY(payment_code_entry)));
    gulong payment_gulong;
    gchar ** stop = (gchar **)g_malloc0(sizeof(gchar *));
    gchar ** tokens;
    Office * office = d_office_create_empty ();
    Account * account = d_account_create_empty ();
    Operation * operation = d_operation_create_empty ();
    gboolean error = FALSE;
    guint length;

    length = gtk_text_get_length(GTK_TEXT(text_description));
    if (length<200) {
	description = gtk_editable_get_chars(GTK_EDITABLE(text_description),0,length);
    } else {
	description = gtk_editable_get_chars(GTK_EDITABLE(text_description),0,199);
    }

    recepit_reference = g_strdup(gtk_entry_get_text(GTK_ENTRY(reference_entry)));

    u_points_to_commas (quantity_char);
    quantity = atof(quantity_char);
    {
	Payment * payment = d_payment_create_empty ();
	tokens = g_strsplit(complete_account_numner," ",4);
	d_office_set_bank_code (office,atoi(tokens[0]));
	d_office_set_office_code (office,atoi(tokens[1]));
	d_account_set_control_digit (account,atoi(tokens[2]));
	d_account_set_account_number (account,tokens[3]);
	d_account_set_office (account,office);
	d_operation_set_account (operation,account);
	payment_gulong = strtoul(payment_code,stop,10);
	d_payment_set_payment_code (payment,payment_gulong);
	d_operation_set_payment (operation,payment);
	d_operation_set_section (operation,description);
	d_operation_set_receipt_reference (operation,recepit_reference);
	d_operation_set_date (operation,time_operation);
	d_operation_set_notification_date (operation,time_notification);
	d_operation_set_amount (operation,quantity);
	error = !m_operation_insert (operation);
    }
    return error;
}

static void fill_payment_code(void) {
    GtkWidget * payment_code_entry = glade_xml_get_widget(xml,"entry_codigo_pago_movimientos");
    gpointer value = (gulong *)g_malloc0(sizeof(gulong));
    gulong * value_gulong;
    gchar * value_gchar;
    if (function_to_call!=NULL) {
	function_to_call(value);
    }
    if (value !=NULL) {
	value_gulong = (gulong *) value;
	value_gchar = g_strdup_printf("%lu",*value_gulong);
	gtk_entry_set_text(GTK_ENTRY(payment_code_entry),value_gchar);

	g_free(value_gchar);
    }
}

