
/* insurance_data_control.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */
#include "config.h"
#include <gnome.h>
#include <glade/glade.h>
#include <glib.h>

#include "insurance_data_control.h"
#include "control_util.h"
#include "province_model.h"
#include "company_model.h"
#include "consortium_model.h"
#include "policy_model.h"
#include "receipt_model.h"
#include "repairs_control.h"
#include "participant.h"
#include "garage.h"
#include "company.h"
#include "policy.h"
#include "receipt.h"
#include "parameter_manager.h"

#define C_LEGAL_ID_COLUMN 0
#define C_NICK_COLUMN 1

#define C_CODIGO_INSERCION ULONG_MAX
#define C_INSERTION_MESSAGE N_("New")

static GtkWidget *insurance_data=NULL;
static GladeXML *xml;
static Policy *policy = NULL;
static gboolean new_policy = TRUE;
static Receipt *receipt = NULL;
static gboolean new_receipt = TRUE;
static Accident *accident = NULL;
static Company *general_company = NULL;
static gchar *general_client_code = NULL;

static gchar url[]="/src/interface/glade/repairs_view.glade";
static gchar widget_name[]="datos_seguro";
static gchar photo1[]="foto1_pixmapentry";
static gchar photo2[]="foto2_pixmapentry";
static gchar dir_photos[]="../../../"DIR_SPOOL"/fotos";

static void on_accept_button_clicked(GtkButton *button,gpointer user_data);
static void on_cancel_button_clicked(GtkButton *button,gpointer user_data);
static void on_window_destroy(GtkWidget *widget,gpointer user_data);
static void on_company_entry_changed(GtkEntry *entry, gpointer user_data);
static void on_policy_combo_entry_changed(GtkEntry *entry, gpointer user_data);
static void on_without_insurance_button_clicked(GtkButton *button,gpointer user_data);
static void on_receipt_combo_entry_changed(GtkEntry *entry, gpointer user_data);
static void fill_policy_combo(Insurance *insurance);
static void insert_company_discount(gdouble dtocompany);
static void insert_consortium(gchar *nombre);
static void fill_company_combo(void);
static void fill_policy(Policy * local_policy);
static void fill_receipt(Receipt * local_receipt);
static void fill_accident(Accident* local_accident);
static void fill_receipt_combo(Insurance *insurance);


GtkWidget *c_insurance_open_window(GtkWidget *app1, gpointer data)
{
   gchar * directory = g_strconcat(AA_DATA_DIR,url,NULL);
   if (insurance_data==NULL)
     {
	GtkWidget *window;	
	GtkWidget *pixmap_photo1 = NULL;
	GtkWidget *pixmap_photo2 = NULL; 
	Insurance *insurance = (Insurance *) data;
	
	xml = glade_xml_new (directory, widget_name);

	pixmap_photo1 = glade_xml_get_widget (xml, photo1);
	pixmap_photo2 = glade_xml_get_widget (xml, photo2);
	window = glade_xml_get_widget (xml, widget_name);
	
	glade_xml_signal_connect(xml, "on_accept_button_clicked",
				 GTK_SIGNAL_FUNC (on_accept_button_clicked));
	glade_xml_signal_connect(xml, "on_cancel_button_clicked",
				 GTK_SIGNAL_FUNC (on_cancel_button_clicked));
	glade_xml_signal_connect(xml, "on_datos_seguro_destroy",
				 GTK_SIGNAL_FUNC (on_window_destroy));
	glade_xml_signal_connect(xml, "on_sseguro_button_clicked",
				 GTK_SIGNAL_FUNC (on_without_insurance_button_clicked));
	glade_xml_signal_connect(xml, "on_company_entry_changed",
				 GTK_SIGNAL_FUNC (on_company_entry_changed));
	glade_xml_signal_connect(xml, "on_poliza_combo_entry_changed",
				 GTK_SIGNAL_FUNC (on_policy_combo_entry_changed));
	glade_xml_signal_connect(xml, "on_recibo_combo_entry_changed",
				 GTK_SIGNAL_FUNC (on_receipt_combo_entry_changed));

	fill_company_combo();


	if (insurance!=NULL){

	    fill_accident(insurance->accident);
	    accident = insurance->accident;

	    if (insurance -> client_code != NULL)
		general_client_code = insurance -> client_code;

	    new_policy = insurance -> new_policy;
	    new_receipt = insurance -> new_receipt;
	}
	
	fill_policy_combo(insurance);

	fill_receipt_combo(insurance);	

	if (insurance!=NULL){
	  fill_policy(insurance->policy); 
	  policy = insurance->policy;
	  fill_receipt(insurance->receipt); 
	  receipt = insurance->receipt;    
	}

	gnome_pixmap_entry_set_pixmap_subdir((GnomePixmapEntry *) pixmap_photo1, dir_photos);

 	gnome_pixmap_entry_set_pixmap_subdir((GnomePixmapEntry *) pixmap_photo2, dir_photos);

	insurance_data = window;
	
     }

   g_free(directory);

   return insurance_data;
   
}

static void fill_policy_combo(Insurance *insurance) {

  GtkWidget * combo;
  Policy **policies;

  combo = glade_xml_get_widget (xml, "poliza_combo");

  c_util_set_code_and_name_on_combo (C_CODIGO_INSERCION,_(C_INSERTION_MESSAGE),GTK_COMBO(combo));

  if (insurance->client_code != NULL) {    
  
    policies = m_policy_find_by_name (NULL,insurance->client_code,NULL);
    c_util_fill_combo_with_policies (combo,policies);

   if (!insurance -> new_policy){
      gchar *policy_number = d_policy_get_policy_number (insurance->policy);

      if (policy_number != NULL) {
	GtkWidget *combo_entry =glade_xml_get_widget (xml, "poliza_combo_entry");
	gtk_entry_set_text (GTK_ENTRY(combo_entry), policy_number);
	g_free(policy_number);
      }
    }
   
   m_policy_clear_array (policies);
   policies = NULL;
  }

}

static void fill_receipt_combo(Insurance *insurance) {

  Policy *local_policy = NULL;
  Receipt *local_receipt = NULL;
  gboolean local_new_receipt = TRUE;
  gchar * local_client = NULL;
  GtkWidget * combo;
   
  if (insurance != NULL) {
    local_policy = insurance -> policy;
    local_receipt = insurance -> receipt;
    local_new_receipt = insurance-> new_receipt;
    local_client = insurance -> client_code;
  }
  else {
    local_receipt = receipt;
    local_policy = policy;
    local_new_receipt = new_receipt;
    local_client = general_client_code;
  }

  combo = glade_xml_get_widget (xml, "recibo_combo");

  c_util_clean_code_and_name_combo (combo);

  c_util_set_code_and_name_on_combo (C_CODIGO_INSERCION,_(C_INSERTION_MESSAGE),GTK_COMBO(combo));
      
  if (local_policy != NULL) {
    gchar * policy_number = d_policy_get_policy_number (local_policy);
    
    if (policy_number != NULL) {
      Receipt **receipts;     
      gulong company_gulong = d_policy_get_company_code (local_policy);
      gchar *company = NULL;
      
      if (company_gulong!=ULONG_MAX)
	company = g_strdup_printf("%lu",company_gulong);
      
      receipts = m_receipts_find_by_name (NULL, policy_number, local_client,company);
      c_util_fill_combo_with_receipts (combo,receipts);
    
      if (!local_new_receipt) {
	gchar *receipt_number = NULL;

	receipt_number = d_receipt_get_receipt_number (local_receipt);
      
	if (receipt_number != NULL) {
	  GtkWidget *combo_entry =glade_xml_get_widget (xml, "recibo_combo_entry");
	  gtk_entry_set_text (GTK_ENTRY(combo_entry), receipt_number);
	  g_free(receipt_number);
	}
      }
      
      m_receipts_clear_array (receipts);
      receipts = NULL;
      
      if (company != NULL) g_free(company);

      g_free(policy_number);
    }
  }

}

static void on_company_entry_changed(GtkEntry *entry, gpointer user_data) {

  GtkWidget * combo;
  gulong code_combo;
  Company ** companies;

  combo = glade_xml_get_widget (xml,"company_combo");

  code_combo = c_util_get_code_from_combo (combo);
    
  if (code_combo != C_COMPANY_NONE) {
    companies = m_company_find_by_code (code_combo);
  
    if (companies[0]!=NULL){
      gulong codigoConsorcio = d_company_get_consortium_code (companies[0]);
/*       gdouble descuento = -1; */

      if (codigoConsorcio != ULONG_MAX){
	Consortium **consortiums = m_consortium_find_by_code (codigoConsorcio);

	gchar * nick = d_participant_get_nick (d_consortium_get_participant (consortiums[0]));
/* 	descuento = d_consortium_get_discount (consortiums[0]); */

	insert_consortium(nick);
	
	d_consortium_clear (consortiums[0]);

	g_free(consortiums);

	g_free(nick);
	
      }
      else insert_consortium(NULL);
      
/*       if (descuento != -1) d_company_set_discount (companies[0], descuento); */

      insert_company_discount(d_company_get_discount (companies[0]));
      
      if (general_company != NULL) d_company_clear (general_company);
      
      general_company=d_company_duplicate (companies[0]);
      
      companies = NULL;
    } else  {
      insert_company_discount(-1);
      insert_consortium(NULL);
      
      if (general_company != NULL) d_company_clear (general_company);
      
      general_company = NULL;   
    }
  } else {
    insert_company_discount(-1);
    insert_consortium(NULL);
    
    if (general_company != NULL) d_company_clear (general_company);
      
    general_company = NULL;   
  }
}

static void on_accept_button_clicked(GtkButton *button,gpointer user_data) {
  
  GtkWidget *policy_number_entry =glade_xml_get_widget (xml, "npoliza_entry");
  gchar *policy_number = gtk_editable_get_chars(GTK_EDITABLE(policy_number_entry),0,-1);
  GtkWidget *combo = glade_xml_get_widget (xml,"company_combo");
  gulong code_combo = c_util_get_code_from_combo (combo);
  
  if ((policy!=NULL)&&(receipt!=NULL)&&(accident!=NULL)) {
    if (code_combo != C_COMPANY_NONE) {
      if (strcasecmp(policy_number,"")) {
	gchar *receipt_number = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "nrecibo_entry")),0,-1);
	
	if (strcasecmp(receipt_number,"")) {	  
	  gchar *mediator_branch_office = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "sucursal_entry")),0,-1);
	  time_t init_date = gnome_date_edit_get_date(GNOME_DATE_EDIT(glade_xml_get_widget (xml, "inicio_dateedit")));
	  time_t expiry_date = gnome_date_edit_get_date(GNOME_DATE_EDIT(glade_xml_get_widget (xml, "validez_dateedit")));
	  
	  gchar *accident_reference = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "ref_entry")),0,-1);
	  time_t fecha_siniestro = gnome_date_edit_get_date(GNOME_DATE_EDIT(glade_xml_get_widget (xml, "siniestro_dateedit")));
	  time_t accident_open_date = gnome_date_edit_get_date(GNOME_DATE_EDIT(glade_xml_get_widget (xml, "dateedit_apertura")));
	  gchar *place = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "lugar_entry")),0,-1);
	  gchar *field = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "ramo_entry")),0,-1);
	  gchar *damages = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "danos_entry")),0,-1);
	  gchar *mode = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "modo_entry")),0,-1);
//	  gchar *cprov = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "cprov_entry")),0,-1);
          gchar *cprov =NULL;
	  gchar *nprov = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "entry_provincias")),0,-1);
	  gchar *ccause = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "ccausa_entry")),0,-1);
	  gchar *photo1 = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "foto1_entry")),0,-1);
	  gchar *photo2 = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "foto2_entry")),0,-1);
	  gchar *descripcion = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "descripcion_text")),0,-1);
	  gulong codProv = m_province_get_code_from_name (nprov);

	  d_policy_set_policy_number (policy, policy_number);
	  
	  if (code_combo!=ULONG_MAX)
	    d_policy_set_company_code (policy, code_combo);    
	  
	  
	  d_receipt_set_receipt_number (receipt, receipt_number);

	  if (strcasecmp(policy_number,""))
	    d_receipt_set_policy_number (receipt, policy_number);
	  else d_receipt_set_policy_number (receipt, "");

	  if (strcasecmp(mediator_branch_office,""))
	    d_receipt_set_subsidiary_mediator (receipt, mediator_branch_office);
	  else d_receipt_set_subsidiary_mediator (receipt, NULL);
	  
	  d_receipt_set_validity_start (receipt, init_date);
	  
	  d_receipt_set_validity_end (receipt, expiry_date);
	  
	  if (code_combo!=ULONG_MAX)
	    d_receipt_set_company_code (receipt, code_combo);
	  
	  
	  if (strcasecmp(accident_reference,""))      
	    d_accident_set_accident_reference (accident, accident_reference);
	  else d_accident_set_accident_reference (accident, NULL);
	  
	  d_accident_set_date (accident, fecha_siniestro);
	  d_accident_set_open_date (accident, accident_open_date);
	  
	  if (strcasecmp(place,""))      
	    d_accident_set_place (accident, place);
	  else d_accident_set_place (accident, NULL);
	  if (strcasecmp(field,""))      
	    d_accident_set_field (accident, field);
	  else d_accident_set_field (accident, NULL);
	  if (strcasecmp(damages,""))      
	    d_accident_set_damages (accident, damages);
	  else d_accident_set_damages (accident, NULL);
	  if (strcasecmp(mode,""))      
	    d_accident_set_way (accident, mode);
	  else d_accident_set_way (accident, NULL);
          cprov = g_strdup_printf("%lu",codProv);
	  if (strcasecmp(cprov,""))
	    d_accident_set_province_code (accident, cprov);
	  else d_accident_set_province_code (accident, NULL);
	  if (strcasecmp(ccause,""))      
	    d_accident_set_reason_code (accident, ccause);
	  else d_accident_set_reason_code (accident, NULL);
	  if (strcasecmp(photo1,""))
	    d_accident_set_photo1 (accident, photo1);
	  else d_accident_set_photo1 (accident, NULL);
	  if (strcasecmp(photo2,""))      
	    d_accident_set_photo2 (accident, photo2);
	  else d_accident_set_photo2 (accident, NULL);
	  if (strcasecmp(descripcion,""))      
	    d_accident_set_description (accident, descripcion);
	  else d_accident_set_description (accident, NULL);
	  
	  
	  g_free(policy_number);
	  g_free(mediator_branch_office);    
	  g_free(receipt_number);	    
	  g_free(accident_reference);
	  g_free(place);
	  g_free(field);
	  g_free(damages);
	  g_free(mode);
	  g_free(cprov);
	  g_free(ccause);
	  g_free(photo1);
	  g_free(photo2);
	  g_free(descripcion);	   
	  
	  c_repair_toggle_insurance(TRUE,new_policy, new_receipt);

	  c_repair_insert_insurance(TRUE);	 
	  
	  gtk_widget_destroy(insurance_data);
	} else c_util_open_error_window (_("A repair with an insurance must have a receipt number to be inserted"));
      } else c_util_open_error_window (_("A repair with an insurance must have a policy number to be inserted"));
    } else c_util_open_error_window (_("A repair with an insurance must have a company to be inserted"));
  }
}

static void on_cancel_button_clicked(GtkButton *button,gpointer user_data){
  
  c_repair_toggle_contrary_insurance();
  
  gtk_widget_destroy(insurance_data);
}

static void on_without_insurance_button_clicked(GtkButton *button,gpointer user_data){  
  
  c_repair_toggle_insurance(FALSE, TRUE, TRUE);

  c_repair_insert_insurance(FALSE);
  
  gtk_widget_destroy(insurance_data);
}

static void fill_company_combo() {
  GtkWidget * combo;
  Company ** companies;
  
  combo = glade_xml_get_widget (xml, "company_combo");
  
  c_util_set_code_and_name_on_combo (C_COMPANY_NONE,g_strdup(_("<None>")),GTK_COMBO(combo));
  companies = m_company_find_by_code (ULONG_MAX);
  
  c_util_fill_combo_with_companies (combo,companies);    
  
}

static void insert_company_discount(gdouble dtocompany){
  
  if (dtocompany!=-1){
    GtkWidget *dto_entry =glade_xml_get_widget (xml, "dtocompany_entry");
    gchar *dtocompany_gchar = NULL;
    
    dtocompany_gchar = g_strdup_printf("%.2f",dtocompany);
    
    gtk_entry_set_text (GTK_ENTRY(dto_entry),dtocompany_gchar);
    g_free(dtocompany_gchar);
  } else{
    GtkWidget *dto_entry =glade_xml_get_widget (xml, "dtocompany_entry");
    
    gtk_entry_set_text (GTK_ENTRY(dto_entry),"");
  }
  
}

static void insert_consortium(gchar *nombre){
  
  if (nombre!=NULL){
    GtkWidget *consortium_entry =glade_xml_get_widget (xml, "consorcio_entry"); 
    
    gtk_entry_set_text (GTK_ENTRY(consortium_entry),nombre);
    
  } else{
    GtkWidget *consortium_entry =glade_xml_get_widget (xml, "consorcio_entry");
    
    gtk_entry_set_text (GTK_ENTRY(consortium_entry),"");
  }
}

static void on_window_destroy(GtkWidget *widget,gpointer user_data){
  
  
  if (general_company != NULL) d_company_clear (general_company);
  if (general_client_code != NULL) g_free(general_client_code);

  general_client_code = NULL; 
  general_company = NULL;
  insurance_data = NULL;
  policy = NULL;
  accident = NULL;
  receipt = NULL;
  xml = NULL;
  
}

static void fill_policy(Policy * local_policy) {
  
  if (local_policy!=NULL){
    
    gchar *policy_number = d_policy_get_policy_number (local_policy);
    gulong codigo_company = d_policy_get_company_code (local_policy);
    
    if (policy_number != NULL) {
      GtkWidget *policy_number_entry =glade_xml_get_widget (xml, "npoliza_entry");
      gtk_entry_set_text (GTK_ENTRY(policy_number_entry), policy_number);
      g_free(policy_number);
    }
    
    if (codigo_company!=ULONG_MAX){
      GtkWidget *combo_entry = glade_xml_get_widget (xml,"company_entry");
      Company **companies = m_company_find_by_code (codigo_company);
      
      if (companies[0] !=NULL){
	gchar *name = d_participant_get_name (d_company_get_participant (companies[0]));
	gtk_entry_set_text(GTK_ENTRY(combo_entry),name);
	
	d_company_clear (companies[0]);
	
	g_free(companies);
	g_free(name);
      }
    }    
  }
}


static void fill_receipt(Receipt * local_receipt){
  
  if (local_receipt!=NULL){
    gchar *receipt_number = d_receipt_get_receipt_number (local_receipt);
    gchar *branch_office = d_receipt_get_subsidiary_mediator (local_receipt);
    time_t expiry_date = d_receipt_get_validity_end (local_receipt);
    GtkWidget *expiry_dateedit = glade_xml_get_widget (xml, "validez_dateedit");
    time_t init_date = d_receipt_get_validity_start (local_receipt);
    GtkWidget *init_date_dateedit = glade_xml_get_widget (xml, "inicio_dateedit");
    
    if (receipt_number != NULL) {
      GtkWidget *receipt_number_entry = glade_xml_get_widget (xml, "nrecibo_entry");
      gtk_entry_set_text (GTK_ENTRY(receipt_number_entry), receipt_number);
      g_free(receipt_number);
    }
    
    if (branch_office!= NULL) {
      GtkWidget *branch_office_entry = glade_xml_get_widget (xml, "sucursal_entry");
      gtk_entry_set_text (GTK_ENTRY(branch_office_entry), branch_office);
      g_free(branch_office);
    }
    
    gnome_date_edit_set_time(GNOME_DATE_EDIT(init_date_dateedit),init_date);
    
    gnome_date_edit_set_time(GNOME_DATE_EDIT(expiry_dateedit),expiry_date);
    
  }
  
}

static void fill_accident(Accident* local_accident){
  
  if (local_accident!=NULL){
    gchar *accident_reference = d_accident_get_accident_reference (local_accident);
    time_t fecha_siniestro = d_accident_get_date (local_accident);
    time_t accident_open_date = d_accident_get_open_date (local_accident);
    gchar *place = d_accident_get_place (local_accident);
    gchar *field = d_accident_get_field (local_accident);
    gchar *damages = d_accident_get_damages (local_accident);
    gchar *mode = d_accident_get_way (local_accident);
    gchar *cprov = d_accident_get_province_code (local_accident);
    gchar *ccause = d_accident_get_reason_code (local_accident);
    gchar *photo1 = d_accident_get_photo1 (local_accident);
    gchar *photo2 = d_accident_get_photo2 (local_accident);
    gchar *descripcion = d_accident_get_description (local_accident);
    Province ** provinces = m_province_find_all ();
    GtkWidget * combo_provinces = glade_xml_get_widget(xml,"combo_provincias");
    gchar * p_name=NULL;


    c_util_fill_combo_with_provinces (combo_provinces,provinces);
    p_name = u_parameter_manager_get_parameter ("provincia");
    if (p_name!=NULL) {
	gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget (xml, "entry_provincias")), p_name);
    }
    g_free(p_name);

    if (accident_reference != NULL) {
      GtkWidget *accident_reference_entry = glade_xml_get_widget (xml, "ref_entry");
      gtk_entry_set_text (GTK_ENTRY(accident_reference_entry), accident_reference);
      g_free(accident_reference);
    }
    
    gnome_date_edit_set_time(GNOME_DATE_EDIT(glade_xml_get_widget (xml, "siniestro_dateedit")),fecha_siniestro);
    gnome_date_edit_set_time(GNOME_DATE_EDIT(glade_xml_get_widget (xml, "dateedit_apertura")),accident_open_date);
    
    if (place != NULL) {
      GtkWidget *place_entry = glade_xml_get_widget (xml, "lugar_entry");
      gtk_entry_set_text (GTK_ENTRY(place_entry), place);
      g_free(place);
    }
    
    if (field != NULL) {
      GtkWidget *field_entry = glade_xml_get_widget (xml, "ramo_entry");
      gtk_entry_set_text (GTK_ENTRY(field_entry), field);
      g_free(field);
    }
    
    if (damages != NULL) {
      GtkWidget *damages_entry = glade_xml_get_widget (xml, "danos_entry");
      gtk_entry_set_text (GTK_ENTRY(damages_entry), damages);
      g_free(damages);
    }
    
    if (mode != NULL) {
      GtkWidget *mode_entry = glade_xml_get_widget (xml, "modo_entry");
      gtk_entry_set_text (GTK_ENTRY(mode_entry), mode);
      g_free(mode);
    }


    if (cprov != NULL) {
      GtkWidget *province_entry = glade_xml_get_widget(xml,"entry_provincias");
      GtkWidget *cprov_entry = glade_xml_get_widget (xml, "cprov_entry");
      gchar ** stop = (gchar **) g_malloc0(sizeof(gchar *));
      gulong codigo = strtoul(cprov,stop,10);
      gchar * province_name = m_province_get_name_from_code (codigo);

      gtk_entry_set_text (GTK_ENTRY(cprov_entry), cprov);
      gtk_entry_set_text (GTK_ENTRY(province_entry), province_name);
      g_free(province_name);
      g_free(stop);
      g_free(cprov);
    }
    
    if (ccause != NULL) {
      GtkWidget *ccause_entry = glade_xml_get_widget (xml, "ccausa_entry");
      gtk_entry_set_text (GTK_ENTRY(ccause_entry), ccause);
      g_free(ccause);
    }
    
    if (photo1 != NULL) {
      GtkWidget *photo1_entry = glade_xml_get_widget (xml, "foto1_entry");
      gtk_entry_set_text (GTK_ENTRY(photo1_entry), photo1);
      g_free(photo1);
    }
    
    if (photo2 != NULL) {
      GtkWidget *photo2_entry = glade_xml_get_widget (xml, "foto2_entry");
      gtk_entry_set_text (GTK_ENTRY(photo2_entry), photo2);
      g_free(photo2);
    }
    
    if (descripcion != NULL) {
      GtkWidget *descripcion_entry = glade_xml_get_widget (xml, "descripcion_text");
      gint *position = (gint *) g_malloc0(sizeof(gint));
      gint len = strlen(descripcion);
      
      *position = 0;
      
      gtk_editable_insert_text (GTK_EDITABLE(descripcion_entry), descripcion, len, position);
      
      g_free(descripcion);
      g_free(position);
    }
    
  }
}


static void on_policy_combo_entry_changed(GtkEntry *entry, gpointer user_data) {

  GtkWidget * combo;
  GtkWidget * local_entry;
  gulong code_combo_gulong;
  GtkWidget * policy_number = glade_xml_get_widget (xml, "npoliza_entry");  
  GtkWidget *vbox = GTK_WIDGET(glade_xml_get_widget(xml, "seguro_frame"));

  local_entry = glade_xml_get_widget (xml, "poliza_combo_entry");
  
  combo = glade_xml_get_widget (xml, "poliza_combo");
  
  code_combo_gulong = c_util_get_code_from_combo (combo);

  gtk_container_foreach(GTK_CONTAINER(vbox),&c_util_clean_widget,NULL);
  
  if (code_combo_gulong != C_CODIGO_INSERCION) {
    gchar *policy_combo_number = gtk_editable_get_chars(GTK_EDITABLE(local_entry),0,-1);
    Policy **policies = NULL;

    policies = m_policy_find_by_name (policy_combo_number, NULL, general_client_code);

    if (policies != NULL) {

      fill_policy(policies[0]);
      
      if (policy != NULL) {
	d_policy_set_company_code (policy, d_policy_get_company_code (policies[0]));
	d_policy_set_policy_number (policy, d_policy_get_policy_number (policies[0]));
	d_policy_set_holder_code (policy, d_policy_get_holder_code (policies[0]));
	d_policy_set_old_company_code (policy, d_policy_get_old_company_code (policies[0]));
	d_policy_set_old_policy_number (policy, d_policy_get_old_policy_number (policies[0]));
	d_policy_set_old_holder_code (policy, d_policy_get_old_holder_code (policies[0]));
      }
            
      fill_receipt_combo(NULL);

      policies[0]=NULL;
      
      g_free(policies);

      policies = NULL;
    }

    g_free(policy_combo_number);      

    gtk_widget_set_sensitive (policy_number, FALSE);
    new_policy = FALSE;
  }
  else {
    GtkWidget *combo_entry = glade_xml_get_widget (xml,"company_entry");
    GtkWidget *recibo_combo_entry = glade_xml_get_widget (xml,"recibo_combo_entry");

    gtk_entry_set_text (GTK_ENTRY(combo_entry), _("<None>"));
    gtk_entry_set_text (GTK_ENTRY(recibo_combo_entry), _(C_INSERTION_MESSAGE));
    
    if (policy != NULL) d_policy_init (policy);

    gtk_widget_set_sensitive (policy_number, TRUE);
    new_policy = TRUE;
  }
  
}

static void on_receipt_combo_entry_changed(GtkEntry *entry, gpointer user_data) {

  GtkWidget * combo;
  GtkWidget * local_entry;
  gulong code_combo_gulong;
  GtkWidget * receipt_number = glade_xml_get_widget (xml, "nrecibo_entry");  

  local_entry = glade_xml_get_widget (xml, "recibo_combo_entry");
  
  combo = glade_xml_get_widget (xml, "recibo_combo");
  
  code_combo_gulong = c_util_get_code_from_combo (combo);

  if (code_combo_gulong != C_CODIGO_INSERCION) {
    gchar *receipt_combo_number = gtk_editable_get_chars(GTK_EDITABLE(local_entry),0,-1);
    Receipt **receipts = NULL;
/*     gulong company_gulong = d_policy_get_company_code (policy); */
    gchar *company = NULL;
      
/*     if (company_gulong!=ULONG_MAX) */
/*       company = g_strdup_printf("%lu",company_gulong); */
      
    receipts = m_receipts_find_by_name (receipt_combo_number, NULL, general_client_code,company);

    if (receipts != NULL) {
	
      fill_receipt(receipts[0]);

      if (receipt != NULL) {
	d_receipt_set_company_code (receipt, d_receipt_get_company_code (receipts[0]));
	d_receipt_set_policy_number (receipt, d_receipt_get_policy_number (receipts[0]));
	d_receipt_set_holder_code (receipt, d_receipt_get_holder_code (receipts[0]));
      }
      
      receipts[0]=NULL;
      
      g_free(receipts);
      
      receipts = NULL;
      
    }

    gtk_widget_set_sensitive (receipt_number, FALSE);
    new_receipt = FALSE;

    if (company != NULL) g_free(company);

  }
  else {
    GtkWidget * branch_office_entry = glade_xml_get_widget (xml, "sucursal_entry");
    GtkWidget * init_date_dateedit = glade_xml_get_widget (xml, "inicio_dateedit");
    GtkWidget * expiry_dateedit = glade_xml_get_widget (xml, "validez_dateedit");

    if (receipt != NULL) d_receipt_init (receipt);

    gtk_editable_delete_text(GTK_EDITABLE(receipt_number),0,-1);
    gtk_editable_delete_text(GTK_EDITABLE(branch_office_entry),0,-1);
    gnome_date_edit_set_time(GNOME_DATE_EDIT(init_date_dateedit),time(NULL));
    gnome_date_edit_set_time(GNOME_DATE_EDIT(expiry_dateedit),time(NULL));

    gtk_widget_set_sensitive (receipt_number, TRUE);
    new_receipt = TRUE;
    
  }

}
