/* invoice_control.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */


#define D_INVOICE_REPAIR_CODE_COLUMN 0
#define C_DELIVERY_PAYMENT_COLUMN 1
#define C_CK_CODE_COLUMN 2
#define C_EUROCODE_COLUMN 3
#define C_DESCRIPTION_COLUMN 4
#define C_FAMILY_COLUMN 5
#define C_PRICE_COLUMN 6
#define C_QUANTITY_COLUMN 7
#define C_DISCOUNT_COLUMN 8
#define C_VAT_COLUMN 9
#define C_TOTAL_COLUMN 10


#define C_NEW_INVOICE -1

#include "config.h"
#include <gnome.h>
#include <glade/glade.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <glib.h>
#include <stdlib.h>

#include "time.h"
#include "a-shortcut.h"
#include "support.h"
#include "parameter_manager.h"
#include "control_util.h"
#include "shop_model.h"
#include "enterprise_model.h"
#include "company_model.h"

#include "enterprise.h"
#include "shop.h"
#include "account.h"
#include "account_model.h"
#include "product.h"
#include "piece.h"
#include "consortium.h"
#include "consortium_model.h"
#include "repair_object.h"
#include "invoice_control.h"
#include "select_fields_control.h"
#include "update_repair_line_control.h"
#include "update_entry_control.h"
#include "invoice_receiver_control.h"
#include "insert_operation_control.h"
#include "insert_cash_operation_control.h"
#include "delivery_payment_print_control.h"
#include "util.h"
#include "payment_note_model.h"
#include "emited_payment_control.h"
#include "documents_control.h"
#include <../interface/printing/invoice-print-wrapper.h>
#include <../interface/printing/listing-print-wrapper.h>


static GtkWidget *invoices=NULL;
static GladeXML *xml;
static GList * accounts=NULL;
static gchar widget_name[]="vbox_facturas";
//static gchar url[]="/src/interface/glade/invoice_view.glade";
static gchar url[]="/src/interface/glade/invoice_third_glade.glade";
Invoice * actual_invoice=NULL;
PaymentNote * payment_note=NULL;
static gboolean negative_price=FALSE;
static gboolean from_garage=FALSE;
static gboolean entra=TRUE;
static GtkWidget *documents_window = NULL;

static DeliveryPayment ** delivery_payments = NULL;

static void clean_screen(void);
static void on_save_button_clicked(GtkButton *button, gpointer data);
static void on_cancel_button_clicked(GtkButton *button, gpointer data);
static void on_delivery_payment_button_clicked(GtkButton *button, gpointer data);
static void on_pay_invoice_button_clicked(GtkButton *button, gpointer data);
static void on_recover_button_clicked(GtkButton *button, gpointer data);
static void on_modify_button_clicked(GtkButton *button, gpointer data);

static void on_modify_invoice_button_clicked(GtkButton *button, gpointer data);
static void on_invoice_name_entry_button_press_event(GtkButton *button,gpointer user_data);
static gint on_invoice_vat_entry_button_press_event(GtkWidget *widget,
							 GdkEventButton *event, gpointer user_data);
static void on_print_invoice_button_clicked(GtkButton *button, gpointer data);
static void fill_shop_combo(void);
static void put_buttons_sensible(gint option);
static void fill_enterprise_from_combo(void);
static void on_shop_entry_changed(GtkEntry *entry,
                                    gpointer user_data);
static void payment_mode_combo_init(void);
static void payment_type_combo_init(void);
static void on_ok_create_delivery_payment(gint reply,gpointer user_data);
static void invoice_type_combo_init(void);
static void delete_factura(gint reply,
			   gpointer user_data);
static void template_combo_init(void);
static void fill_invoice_data(Invoice * invoice,gint actual_state);
static void fill_list_with_invoice(GtkCList * list,Invoice * invoice);
static void fill_list_with_payment_note(GtkCList  * list,PaymentNote * payment_note_aux);
static void on_payment_mean_combo_entry_hanged(GtkEntry *entry,
					     gpointer user_data);
static void add_delivery_payment_line(GtkCList * list,gint index,DeliveryPayment * delivery_payment,gulong repair_code);
static gdouble get_garage_total_entry(void);
static void set_dates_on_payment(void);
static void fill_account_number_combo(void);
static void set_split_invoice_state(gboolean checked,gboolean sensitive);
static void on_money_order_days_changed(GtkEntry *entry,
				 gpointer user_data);
static void on_payment_means_combo_entry_changed(GtkEntry *entry,
					      gpointer user_data);
static void on_type_invoice_combo_entry_changed(GtkEntry *entry,
					      gpointer user_data);
static gboolean get_is_from_garage(DeliveryPaymentId ** delivery_payments_ids);
static void fill_client_data(Participant * participant);
static void fill_accident_data(Repair * repair);
static void fill_parent_company_data(gchar * shopName);
static void fill_state_data(Invoice * invoice);
static void fill_total_data(gdouble subtotal,gdouble vat,gdouble total);
static void fill_payment_note_data(PaymentNote * payment_note_aux);
static void fill_payment_data(Payment * payment,gboolean recarga);
static void fill_payment_initial_data(gboolean with_insurance);
static void view_garage_total_entry(gboolean sensitive);
static void fill_garage_total_entry(gdouble total_taller);
static void on_detail_invoice_list_select_row (GtkCList        *list,
				  gint             row,
				  gint             column,
				  GdkEvent        *event,
				  gpointer         user_data);
static gint on_detail_invoice_list_button_press_event(GtkWidget *widget, 
	GdkEventButton *event, gpointer user_data);
					  
//static void d_payment_note_calculate_total  (gdouble * subtotal,gdouble * vat,
//					      gdouble * total,PaymentNote * payment_note);
static void open_update_repair_line_invoice(GtkWidget * wid);
static void restore_to_initial_purchase(gboolean no_sensitive);
static void fill_invoice_number_data(Invoice * invoice);
static void modify_client_invoice(gpointer data);
gboolean on_subtotal_entry_press_event(GtkWidget *widget,
                                            GdkEventButton *event,
                                            gpointer user_data);
gboolean on_discount_entry_press_event(GtkWidget *widget,
                                            GdkEventButton *event,
                                            gpointer user_data);
static void inicialize_payment_screen(gboolean sensitive,gboolean note);
void set_subtotal(gpointer value);
void pay_split_invoice_from_invoice	(void);
void print_normal_delivery_payment(gpointer value,gpointer value2);
static void do_pay(void);
void set_discount_value(gpointer value);
static void data_note_fill(PaymentNote * paymentNote);
static void modify_invoice(void);
//static gboolean correct_account_number(void);
static Payment * create_payment_from_screen(gint split_invoice);
static void on_spliting_invoice_checkbutton_toggled(GtkToggleButton *togglebutton,
						 gpointer user_data);
static void add_list_line(GtkCList * list,gint index,Repair_line * repair_line,gulong repair_code,gulong delivery_payment_number);
void get_payment_code(gpointer value);
void save_invoice(gpointer data);
static void print_invoice(gpointer value,gpointer value2);
void do_second_operation(gpointer data);
void save_garage_invoice(void);
static void open_document_window(GtkWidget *app1);
static void on_facturas_hide(GtkWidget *widget, gpointer user_data);
static void charge_documents(void);
static void fill_date_payment_note(PaymentNote * payment_note);
static void set_template(Invoice * invoice);
static void confirm_update(gint reply, gpointer data);
static void on_activate_with_vat(GtkMenuItem *menuitem,
				gpointer user_data);
static void on_without_vat_activate(GtkMenuItem *menuitem,
				gpointer user_data);

void disassociate_delivery_payments(gpointer user_data, gint len);
		

GtkWidget *c_invoice_open_window(GtkWidget *app1, GtkWidget *vbox)
{
   gchar * directory = g_strconcat(AA_DATA_DIR,url,NULL);
      
  if (invoices==NULL)
    {
      GtkWidget *dialog;
      gchar * s_name = NULL;
      xml = glade_xml_new (directory, widget_name);
      dialog = glade_xml_get_widget (xml, widget_name);
      
      gtk_box_pack_start (GTK_BOX (vbox), dialog, TRUE, TRUE, 0);  

      glade_xml_signal_connect (xml, "on_guardar_clicked",
				GTK_SIGNAL_FUNC (on_save_button_clicked));
      glade_xml_signal_connect (xml, "on_anular_clicked",
				GTK_SIGNAL_FUNC (on_cancel_button_clicked));
      glade_xml_signal_connect (xml, "on_entry_tienda_changed",
				GTK_SIGNAL_FUNC (on_shop_entry_changed));
      glade_xml_signal_connect (xml, "on_combo_entry_modo_pago_changed",
				GTK_SIGNAL_FUNC (on_payment_mean_combo_entry_hanged));
      glade_xml_signal_connect (xml, "on_combo_entry_medio_changed",
				GTK_SIGNAL_FUNC (on_payment_means_combo_entry_changed));
      glade_xml_signal_connect (xml, "on_entry_tipo_combo_factura_changed",
				GTK_SIGNAL_FUNC (on_type_invoice_combo_entry_changed));
      glade_xml_signal_connect (xml, "on_lista_detalle_factura_select_row",
				GTK_SIGNAL_FUNC (on_detail_invoice_list_select_row));
      glade_xml_signal_connect (xml, "on_lista_detalle_factura_button_press_event",
				GTK_SIGNAL_FUNC (on_detail_invoice_list_button_press_event));
      glade_xml_signal_connect (xml, "on_entry_base_imponible_factura_button_press_event",
				GTK_SIGNAL_FUNC (on_subtotal_entry_press_event));
      glade_xml_signal_connect (xml, "on_entry_descuento_factura_button_press_event",
				GTK_SIGNAL_FUNC (on_discount_entry_press_event));
      glade_xml_signal_connect (xml, "on_checkbutton_refacturacion_toggled",
				GTK_SIGNAL_FUNC (on_spliting_invoice_checkbutton_toggled));
      glade_xml_signal_connect (xml, "on_button_albaran_factura_clicked",
				GTK_SIGNAL_FUNC (on_delivery_payment_button_clicked));
      glade_xml_signal_connect (xml, "on_button_pagar_factura_clicked",
				GTK_SIGNAL_FUNC (on_pay_invoice_button_clicked));
      glade_xml_signal_connect (xml, "on_button_imprimir_factura_clicked",
				GTK_SIGNAL_FUNC (on_print_invoice_button_clicked));
      glade_xml_signal_connect (xml, "on_button_modificar_clicked",
				GTK_SIGNAL_FUNC (on_modify_button_clicked));
      glade_xml_signal_connect (xml, "on_button_recuperar_clicked",
				GTK_SIGNAL_FUNC (on_recover_button_clicked));
      glade_xml_signal_connect (xml, "on_dias_giro_changed",
				GTK_SIGNAL_FUNC (on_money_order_days_changed));
      glade_xml_signal_connect (xml, "on_button_modificar_factura_clicked",
				GTK_SIGNAL_FUNC (on_modify_invoice_button_clicked));
      glade_xml_signal_connect (xml, "on_facturas_hide",
				GTK_SIGNAL_FUNC (on_facturas_hide));
      glade_xml_signal_connect (xml, "on_entry_nombre_factura_button_press_event",
				GTK_SIGNAL_FUNC (on_invoice_name_entry_button_press_event));
      glade_xml_signal_connect (xml, "on_entry_iva_factura_button_press_event",
				GTK_SIGNAL_FUNC (on_invoice_vat_entry_button_press_event));
 

      fill_shop_combo();
      s_name = u_parameter_manager_get_parameter ("tienda");
      if (s_name!=NULL) {
	  gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget (xml, "entry_tienda_facturas")), s_name);
      }
      g_free(s_name);

      /* Inicializa el combo de modo de pago*/
      payment_mode_combo_init();
      /* Inicializa el combo de tipos de pago*/
      payment_type_combo_init();
      /*Inicializa el combo de tipos de factura*/
      invoice_type_combo_init();
      /*Inicializa el combo de templates de factura*/
      template_combo_init();

      invoices=dialog;

    }
  else {
      if (actual_invoice!=NULL) {
	  d_invoice_clear (actual_invoice);
	  if (accounts!=NULL) {
	      g_list_free(accounts);
	      accounts =NULL;
	  }
	  actual_invoice=NULL;

      } else if (payment_note!=NULL) {
	  d_payment_note_clear (payment_note);
	  if (accounts!=NULL) {
	      g_list_free(accounts);
	      accounts =NULL;
	  }
	  payment_note=NULL;
      }
      clean_screen();

  }
  negative_price=FALSE;
  inicialize_payment_screen(TRUE,FALSE);
  put_buttons_sensible(4);
  g_free(directory);

  return invoices;

}

static void open_document_window(GtkWidget *app1) {

  gboolean repair = FALSE;

  if (!(GTK_IS_WINDOW(documents_window))) {
    documents_window = c_documents_open(app1, (gpointer) &repair);
    
    gtk_window_set_position(GTK_WINDOW(documents_window),GTK_WIN_POS_NONE);
    
  }
}
/* CArga los docuemtos asociados a la compaa de seguros de la reparacin de la factura.*/
static void charge_documents(){

    if (actual_invoice != NULL) {
	Repair * local_repair;
	if (d_invoice_get_repair(actual_invoice)!=NULL)
	    local_repair = d_invoice_get_repair (actual_invoice);
	else {
            DeliveryPayment ** delivery_payments = d_invoice_get_delivery_payments(actual_invoice);
	    local_repair = d_delivery_payment_get_repair (delivery_payments[0]);
	}
	if (local_repair != NULL) {
	    Receipt *local_receipt = d_repair_get_receipt (local_repair);
	    if (local_receipt != NULL) {
		gulong company_code = d_receipt_get_company_code (local_receipt);
		if (company_code != ULONG_MAX) {
		    Company **companies = m_company_find_by_code (company_code);
		    if (companies[0]!=NULL){
			open_document_window(NULL);
			c_documents_load_company(companies[0]);
			gtk_widget_show(documents_window);
			companies[0] = NULL;
			g_free(companies);
		    }
		}
	    }
	    local_receipt = NULL;
	}
	local_repair = NULL;
    }
}


void c_invoice_load_delivery_payments(DeliveryPaymentId ** delivery_payments_from_client) {
    GtkWidget * invoice_type_combo;
    GtkWidget * invoice_type_entry;
    Invoice * invoice=NULL;
    gboolean with_insurance=FALSE;
    GtkWidget * entry_discount = glade_xml_get_widget (xml, "entry_descuento_factura");
    DeliveryPayment ** delivery_payments=NULL;

    from_garage=get_is_from_garage(delivery_payments_from_client);
    invoice = m_invoice_obtain_data (delivery_payments_from_client);
    invoice_type_combo = glade_xml_get_widget(xml,"combo_tipo_factura");
    invoice_type_entry = glade_xml_get_widget(xml,"entry_tipo_combo_factura");

    put_buttons_sensible(5);
    gtk_widget_set_sensitive(glade_xml_get_widget(xml,"dateedit_emision"),TRUE);
    if (invoice!=NULL)
	fill_invoice_data(invoice,C_NEW_INVOICE);

    if (!GTK_WIDGET_SENSITIVE(entry_discount)) gtk_widget_set_sensitive(entry_discount,TRUE);
    
    fill_account_number_combo();
    delivery_payments = d_invoice_get_delivery_payments(actual_invoice);

    if (from_garage==FALSE) {
	if (delivery_payments!=NULL) {
	    if (delivery_payments[1]==NULL) {
		fill_garage_total_entry(d_delivery_payment_get_garage_total (delivery_payments[0]));
		view_garage_total_entry(TRUE);
	    } else {
		view_garage_total_entry(FALSE);
	    }
	}
	set_split_invoice_state(FALSE,TRUE);
    }
    else {
	view_garage_total_entry(FALSE);
	set_split_invoice_state(FALSE,FALSE);
    }
    if (delivery_payments!=NULL) {
	if ((delivery_payments[0]!=NULL) && (delivery_payments[1]==NULL)) {
            Repair * repair = d_delivery_payment_get_repair(delivery_payments[0]);
	    with_insurance=d_repair_is_with_insurance(repair);

	} else {
	    gtk_widget_set_sensitive(glade_xml_get_widget(xml,"combo_tipo_factura"),FALSE);
	}
    }


    fill_payment_initial_data(with_insurance);
    set_template(invoice);


#ifndef IS_SHOP
    charge_documents();
#endif

}

void c_invoice_load_repair(Repair * repair){
    gboolean with_insurance=FALSE;
    GtkWidget * invoice_type_entry;
//    GtkWidget * invoice_type_combo;

#ifndef IS_SHOP
    GtkWidget * entry_discount = glade_xml_get_widget (xml, "entry_descuento_factura");
#endif

    from_garage=FALSE;
    invoice_type_entry = glade_xml_get_widget(xml,"entry_tipo_combo_factura");
    gtk_widget_set_sensitive(glade_xml_get_widget(xml,"dateedit_emision"),TRUE);
    if (repair!=NULL) {
	Invoice * invoice = d_invoice_create_empty ();
	d_invoice_set_repair (invoice,repair,TRUE);
	fill_invoice_data(invoice,C_NEW_INVOICE);
#ifndef IS_SHOP
	put_buttons_sensible(2);
	if (!GTK_WIDGET_SENSITIVE(entry_discount)) gtk_widget_set_sensitive(entry_discount,TRUE);
#else
	put_buttons_sensible(1);
#endif
	fill_account_number_combo();
	view_garage_total_entry(TRUE);
	set_split_invoice_state(FALSE,TRUE);
	if (!GTK_WIDGET_SENSITIVE(invoice_type_entry)) gtk_widget_set_sensitive(invoice_type_entry,TRUE);

        with_insurance = d_invoice_get_with_insurance(invoice);
	fill_payment_initial_data(with_insurance);
        set_template(invoice);

#ifndef IS_SHOP
	if (actual_invoice != NULL) {

	  open_document_window(NULL);
	  
	  gtk_widget_show(documents_window);
	  
	}
#endif

    }
#ifndef IS_SHOP
    if (!GTK_WIDGET_SENSITIVE(glade_xml_get_widget(xml,"combo_tipo_factura"))) gtk_widget_set_sensitive(glade_xml_get_widget(xml,"combo_tipo_factura"),TRUE);
#endif

}

void c_invoice_load(InvoiceId * invoiceId) {
    
#ifndef IS_SHOP
    GtkWidget * entry_discount = glade_xml_get_widget (xml, "entry_descuento_factura");

    Invoice * invoice = m_invoice_find_by_id (invoiceId);
    gulong garage_code =ULONG_MAX;
    from_garage=TRUE;

    garage_code = d_participant_get_code (d_invoice_get_participant (invoice));

    if (GTK_WIDGET_SENSITIVE(entry_discount)) gtk_widget_set_sensitive(entry_discount,TRUE);

    from_garage = m_delivery_payment_client_is_not_from_garage (garage_code,
						  garage_code,
						  ULONG_MAX,
						  ULONG_MAX,
						  ULONG_MAX
						 );

    put_buttons_sensible(1);
    fill_invoice_data(invoice,d_invoice_get_state (invoice));
    fill_payment_data(d_invoice_get_payment (invoice),FALSE);
    inicialize_payment_screen(FALSE,FALSE);
    fill_state_data(invoice);
    fill_account_number_combo();
    view_garage_total_entry(FALSE);
    if (d_invoice_get_shop_code_split_invoice (actual_invoice)!=ULONG_MAX) {
	set_split_invoice_state(TRUE,FALSE);
    } else {
	set_split_invoice_state(FALSE,FALSE);
    }
    gtk_widget_set_sensitive(glade_xml_get_widget(xml,"combo_tipo_factura"),FALSE);
    gtk_widget_set_sensitive(glade_xml_get_widget(xml,"dateedit_emision"),TRUE);
    set_template(invoice);

    charge_documents();
#endif
    

}

void c_invoice_load_payment_note(PaymentNoteId * paymentNoteId) {

    from_garage=FALSE;
    payment_note = m_payment_note_find_by_id (paymentNoteId);
    fill_payment_note_data(payment_note);
    fill_payment_data(d_payment_note_get_payment (payment_note),FALSE);
    put_buttons_sensible(6);
    fill_account_number_combo();
    set_split_invoice_state(FALSE,FALSE);
    gtk_widget_set_sensitive(glade_xml_get_widget(xml,"combo_tipo_factura"),FALSE);
    gtk_widget_set_sensitive(glade_xml_get_widget(xml,"dateedit_emision"),FALSE);
    set_template(NULL);

}

/*Rellena los datos de una nota en la pantalla*/
static void fill_payment_note_data(PaymentNote * payment_note_aux) {
    GtkWidget * list = glade_xml_get_widget(xml, "lista_detalle_factura");
    GtkWidget * paymentFrame = glade_xml_get_widget(xml, "frame_datos_pago");
    GtkWidget * assuranceFrame = glade_xml_get_widget(xml, "frame_parte_siniestro");
    GtkWidget * wid = glade_xml_get_widget(xml,"entry_tipo_combo_factura");
    GtkWidget * wid2 = glade_xml_get_widget(xml,"combo_tipo_factura");
    GtkWidget * check = glade_xml_get_widget(xml,"checkbutton_refacturacion");

    gchar * temp;
    gdouble * subtotal = (gdouble *) g_malloc0(sizeof(gdouble));
    gdouble * vat = (gdouble *) g_malloc0(sizeof(gdouble));
    gdouble * total = (gdouble *) g_malloc0(sizeof(gdouble));

    payment_note = payment_note_aux;

    if (GTK_WIDGET_VISIBLE(assuranceFrame)) {
	gtk_widget_hide(assuranceFrame);
    }
    if (!GTK_WIDGET_VISIBLE(paymentFrame)) {
	gtk_widget_show(paymentFrame);
    }

    temp = g_strdup(gtk_entry_get_text(GTK_ENTRY(wid)));
    if (strcmp(temp,D_PAYMENT_NOTE))  gtk_entry_set_text(GTK_ENTRY(wid),D_PAYMENT_NOTE);
    g_free(temp);

    gtk_widget_set_sensitive(check,FALSE);
    gtk_widget_set_sensitive(wid2,FALSE);

    if (d_payment_note_get_repair_detail (payment_note)!=NULL) {
	fill_client_data(d_client_get_participant (d_repair_get_client (d_payment_note_get_repair_detail (payment_note))));
    }
    data_note_fill(payment_note);
    d_payment_note_calculate_total  (subtotal,vat,total,payment_note);

    /*Rellena la lista con la nota*/
    fill_list_with_payment_note(GTK_CLIST(list),payment_note);
    /*Rellena los datos de los totales*/
    fill_total_data(*subtotal,*vat,*total);

    fill_date_payment_note(payment_note);
    g_free(subtotal);
    g_free(vat);
    g_free(total);


}

/*Carga en la pantalla todos los datos de la factura*/
static void fill_invoice_data(Invoice * invoice,gint actual_state){
    gboolean conSeguro=0;
    gdouble * subtotal = (gdouble *) g_malloc0(sizeof(gdouble));
    gdouble * vat = (gdouble *) g_malloc0(sizeof(gdouble));
    gdouble * total = (gdouble *) g_malloc0(sizeof(gdouble));
    GtkWidget * entry;
    GtkWidget * list = glade_xml_get_widget(xml, "lista_detalle_factura");
    GtkWidget * paymentFrame = glade_xml_get_widget(xml, "frame_datos_pago");
    GtkWidget * assuranceFrame = glade_xml_get_widget(xml, "frame_parte_siniestro");
    Repair * repair=NULL;
    gchar * name_of_shop=NULL;
    gint error_repair=0;
    actual_invoice = invoice;

    /*Establece el estado de la factura*/
    d_invoice_set_state (invoice,actual_state);


    repair = d_invoice_get_first_repair(invoice);
    if (repair==NULL) {
        error_repair=1;
	c_util_open_error_window (_("Error inserting repair data"));
    }

    /*Si no hay error al obtener la reparacin sigue*/
    if (error_repair!=1) {

	if ((d_repair_get_company (repair) !=NULL) && (from_garage==FALSE)) {
	    /*Hay seguro si la compaa tiene un cdigo correcto*/
            conSeguro = d_repair_is_with_insurance(repair);
	} else {
	    conSeguro = FALSE;
	}

	if ((conSeguro!=FALSE) && (repair!=NULL)) {
	    GtkWidget * wid = glade_xml_get_widget(xml,"entry_tipo_combo_factura");
	    GtkWidget * combo_tipo = glade_xml_get_widget(xml,"combo_tipo_factura");
	    gchar * temp;
	    temp = g_strdup(gtk_entry_get_text(GTK_ENTRY(wid)));
	    if (strcmp(temp,D_INVOICE_MESSAGE))  gtk_entry_set_text(GTK_ENTRY(wid),D_INVOICE_MESSAGE);
	    g_free(temp);

	    gtk_widget_set_sensitive(combo_tipo,TRUE);
            if (!GTK_WIDGET_VISIBLE(paymentFrame)) {
		gtk_widget_show(paymentFrame);
	    }
	    if (!GTK_WIDGET_VISIBLE(assuranceFrame)) {
		gtk_widget_show(assuranceFrame);
	    }
	    fill_accident_data(repair);

	} else if (repair!=NULL) {
	    GtkWidget * wid = glade_xml_get_widget(xml,"entry_tipo_combo_factura");
	    GtkWidget * combo_tipo = glade_xml_get_widget(xml,"combo_tipo_factura");

	    gtk_widget_set_sensitive(combo_tipo,TRUE);
	    gtk_entry_set_text(GTK_ENTRY(wid),D_PAYMENT_NOTE);
	    if (GTK_WIDGET_VISIBLE(assuranceFrame)) {
		gtk_widget_hide(assuranceFrame);
	    }
	    if (!GTK_WIDGET_VISIBLE(paymentFrame)) {
		gtk_widget_show(paymentFrame);
	    }
	} else {
	    c_util_open_error_window ("Reparacion incorrecta");
	}

	entry = glade_xml_get_widget(xml,"entry_tipo_combo_factura");
	gtk_entry_set_text(GTK_ENTRY(entry),D_INVOICE_MESSAGE);

	fill_list_with_invoice(GTK_CLIST(list),invoice);
	if (from_garage==FALSE) {
	    if (negative_price==FALSE) {
            /*Si el precio de todas las lneas de factura es correcto calculamos los valores totales */
		d_invoice_calculate_total (subtotal,vat,total,actual_invoice,FALSE);

		fill_total_data(*subtotal,*vat,*total);
	    } else {
            /*Si el precio de alguna pieza no existe sacamos un mesaje y dehabilitamos los botones*/
	      gchar * message = g_strdup(_("WARNING:You just could print delivery payment in this window.\nTo do other opetations you must:\n      * Go back to the repair window.\n      * Insert the price in the product or in the line\n        for the lines that already don't have it."));
	      c_util_open_info_window (message);
#ifndef IS_SHOP
		put_buttons_sensible(3);
#endif
		g_free(message);
	    }
	} else {
	    /* CASO DE FACTURA DE TALLER: es un caso especial ya que las facturas de taller pueden llevar
	     * lneas de factura diferentes a lneas de reparacin*/
		d_invoice_calculate_total_to_garage (subtotal,vat,total,actual_invoice,FALSE);
		fill_total_data(*subtotal,*vat,*total);
	}

	if (d_invoice_get_shop (invoice)!=NULL) {
	    if (d_participant_get_code (d_shop_get_participant (d_invoice_get_shop (invoice)))!=ULONG_MAX) {
		name_of_shop = d_participant_get_name (d_shop_get_participant (
									   d_invoice_get_shop (invoice)
									  ));
	    } else {
		name_of_shop = d_participant_get_name (d_shop_get_participant (
									   d_repair_get_shop (repair)
									  ));
	    }
	} else {
	    name_of_shop = d_participant_get_name (d_shop_get_participant (
								       d_repair_get_shop (repair)
								      ));
	}

	if (actual_state==C_NEW_INVOICE) {
	    GtkWidget * outstanding_button;
            /*Carga los datos del cliente, dependiendo del caso que se trate*/
	    if (from_garage==FALSE) {
		fill_client_data(d_client_get_participant (d_repair_get_client (repair)));
	    } else {
		fill_client_data(d_delivery_payment_obtain_garage(d_invoice_get_delivery_payments(actual_invoice)));
	    }
	    if (d_repair_get_shop (repair)!=NULL) {
		fill_parent_company_data(name_of_shop);
	    }
	    outstanding_button = glade_xml_get_widget(xml, "radiobutton_pendiente_factura");
	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(outstanding_button),TRUE);


	} else {
	  GtkWidget * outstanding_button=NULL;
	  
	  fill_client_data(d_invoice_get_participant (invoice));
	  if (d_repair_get_shop (repair)!=NULL) {
	    fill_parent_company_data(name_of_shop);
	  }
	  
	  switch (actual_state) {
	  case D_INVOICE_NOT_ISSUED: 
	    outstanding_button = glade_xml_get_widget(xml, "radiobutton_pendiente_factura");
	    break;
	  case D_INVOICE_NOT_PAID:
	    outstanding_button = glade_xml_get_widget(xml, "radiobutton_emitida_factura");
	    break;
	  case D_INVOICE_PAID: 
	    outstanding_button = glade_xml_get_widget(xml, "radiobutton_pagada_factura");
	    break;
	  case D_INVOICE_MANAGING:
	    outstanding_button = glade_xml_get_widget(xml, "radiobutton_engestion_factura");
	    break;
	  default:
	    c_util_open_error_window (_("Wrong invoice state"));
	    break;
	  }
	  
	  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(outstanding_button),TRUE);
	  
	  fill_invoice_number_data(invoice);
	}

	g_free(subtotal);
	g_free(vat);
	g_free(total);


    }
    invoice = NULL;
}

/*Establece el nmero de la factura y la fecha de emisin */
static void fill_invoice_number_data(Invoice * invoice) {
    GtkWidget * entry = glade_xml_get_widget(xml,"entry_numero_factura");
    GtkWidget * issue_date = glade_xml_get_widget(xml,"dateedit_emision");
    gulong invoice_number = d_invoice_get_invoice_number (invoice);

    if ((invoice_number!=ULONG_MAX) && (d_invoice_get_shop (invoice))!=NULL) {
        gchar * numero = d_invoice_get_invoice_number_with_group_code(invoice);
	gtk_entry_set_text(GTK_ENTRY(entry),numero);
	g_free(numero);
    }
    gnome_date_edit_set_time(GNOME_DATE_EDIT(issue_date),d_invoice_get_issue_date (invoice));
}

/*Establece el nmero de nota y la fecha de emisin*/
static void data_note_fill(PaymentNote * paymentNote) {
    GtkWidget * entry = glade_xml_get_widget(xml,"entry_numero_factura");
    GtkWidget * issue_date = glade_xml_get_widget(xml,"dateedit_emision");
    gulong payment_number = d_payment_note_get_note_code (paymentNote);

    if (payment_number!=ULONG_MAX) {
	gchar * payment_number_char = g_strdup_printf("%lu",payment_number);
	gtk_entry_set_text(GTK_ENTRY(entry),payment_number_char);
	g_free(payment_number_char);
    }
    if (d_payment_note_get_payment (paymentNote)!=NULL) {
	gnome_date_edit_set_time(GNOME_DATE_EDIT(issue_date),d_payment_get_init_date (d_payment_note_get_payment (paymentNote)));
    }

}
/*Aade una lnea de reparcin a la lista de facturas que se muestra en la pantalla.*/
static void add_list_line(GtkCList * list,gint index,Repair_line * repair_line,gulong repair_code,gulong delivery_payment_number) {
    gboolean is_piece;
    gchar * content;
    Piece * piece;
    Product * product;
    gdouble price=G_MINDOUBLE;
    gdouble line_price=G_MINDOUBLE;
    gdouble discount;
    gdouble vat;


    if (repair_code!=ULONG_MAX) {
	content = g_strdup_printf("%lu",repair_code);
	gtk_clist_set_text((GtkCList *) list, index, D_INVOICE_REPAIR_CODE_COLUMN, content);
	g_free(content);
    }
    if (delivery_payment_number!=ULONG_MAX) {
	content = g_strdup_printf("%lu",delivery_payment_number);
	gtk_clist_set_text((GtkCList *) list, index, C_DELIVERY_PAYMENT_COLUMN, content);
	g_free(content);
    } else {
        gtk_clist_set_column_visibility(list,C_DELIVERY_PAYMENT_COLUMN,FALSE);
    }

    gtk_clist_set_column_visibility(list,C_CK_CODE_COLUMN,FALSE);
    product = d_repair_line_get_product (repair_line);
    is_piece = d_product_get_is_piece (product);

    if (is_piece==TRUE) {
	piece = (Piece *) product;
	content = d_piece_get_euroCode (piece);
	gtk_clist_set_text((GtkCList *) list, index, C_EUROCODE_COLUMN, content);
	g_free(content);
    }

    content = d_product_get_description (product);
    if (content!=NULL) {
	gtk_clist_set_text((GtkCList *) list, index, C_DESCRIPTION_COLUMN, content);
	g_free(content);
    }

    content=d_product_get_family_code (product);
    if (content!=NULL) {
	gtk_clist_set_text((GtkCList *) list, index, C_FAMILY_COLUMN, content);
	g_free(content);
    }

    line_price = d_repair_line_get_price (repair_line);
    price = d_product_get_price (product);

    if ((line_price==G_MINDOUBLE) && (price==G_MINDOUBLE)) negative_price=TRUE;

    if (price!=G_MINDOUBLE) {
	content = g_strdup_printf("%10.2f",price);
	gtk_clist_set_text((GtkCList *) list, index, C_PRICE_COLUMN, content);
	g_free(content);
    }

    if (d_repair_line_get_quantity (repair_line)!=G_MINDOUBLE) {
	content = g_strdup_printf("%.3f",d_repair_line_get_quantity (repair_line));
	gtk_clist_set_text((GtkCList *) list, index, C_QUANTITY_COLUMN, content);
	g_free(content);
    }

    discount = d_repair_line_get_discount (repair_line);
    if (discount>=0) {
	content = g_strdup_printf("%.2f",discount);
	gtk_clist_set_text((GtkCList *) list, index, C_DISCOUNT_COLUMN, content);
	g_free(content);
    }
    vat = d_repair_line_get_historical_vat_type (repair_line);
    if (vat!=G_MINDOUBLE) {
	content = g_strdup_printf("%.2f",vat);
	gtk_clist_set_text((GtkCList *) list, index, C_VAT_COLUMN, content);
	g_free(content);
    }
    if (line_price!=G_MINDOUBLE) {
	content = g_strdup_printf("%10.2f",line_price*u_round_to_decimal (d_repair_line_get_quantity (repair_line),3));
	gtk_clist_set_text((GtkCList *) list, index, C_TOTAL_COLUMN, content);
	g_free(content);
    } else {
	if (price!=G_MINDOUBLE) {
            if (discount<0) discount=0;
	    content = g_strdup_printf("%10.2f",(price*(u_round_to_decimal (d_repair_line_get_quantity (repair_line),3))*(1-(0.01*discount))));
	    gtk_clist_set_text((GtkCList *) list, index, C_TOTAL_COLUMN, content);
	    g_free(content);
	}
    }
}

/*Aade una lnea de albarn para el caso de facturas de taller que llevan lneas especiales por albarn.*/
static void add_delivery_payment_line(GtkCList * list,gint index,DeliveryPayment * delivery_payment,gulong repair_code) {
    gchar * content;
    gdouble price=G_MINDOUBLE;
    gchar * descripcion_aux=NULL;
    gulong delivery_payment_number=ULONG_MAX;
    gchar ** description = (gchar **) g_malloc0(sizeof(gchar *));
    gchar ** vat = (gchar **) g_malloc0(sizeof(gchar *));
    gchar ** quantity = (gchar **) g_malloc0(sizeof(gchar *));

    m_invoice_get_data_to_delivery_payment_line(description,vat,quantity);

    repair_code = d_repair_get_repair_code (d_delivery_payment_get_repair (delivery_payment));
    if (repair_code!=ULONG_MAX) {
	content = g_strdup_printf("%lu",repair_code);
	gtk_clist_set_text((GtkCList *) list, index, D_INVOICE_REPAIR_CODE_COLUMN, content);
	g_free(content);
    }

    delivery_payment_number = d_delivery_payment_get_number (delivery_payment);
    if (delivery_payment_number!=ULONG_MAX) {
	content = g_strdup_printf("%lu",delivery_payment_number);
	gtk_clist_set_text((GtkCList *) list, index, C_DELIVERY_PAYMENT_COLUMN, content);
	descripcion_aux = g_strconcat(*description,content, NULL);
	g_free(content);
    }

    if (descripcion_aux!=NULL) {
	gtk_clist_set_text((GtkCList *) list, index, C_DESCRIPTION_COLUMN, descripcion_aux);
	g_free(descripcion_aux);
    }

    price = d_delivery_payment_get_garage_total (delivery_payment);
 
    content = g_strdup(*quantity);
    gtk_clist_set_text((GtkCList *) list, index, C_QUANTITY_COLUMN, content);
    g_free(content);

    content = g_strdup(*vat);
    gtk_clist_set_text((GtkCList *) list, index, C_VAT_COLUMN, content);
    g_free(content);

    if (price!=G_MINDOUBLE) {
	content = g_strdup_printf("-%.2f",price);
	gtk_clist_set_text((GtkCList *) list, index, C_PRICE_COLUMN, content);
	gtk_clist_set_text((GtkCList *) list, index, C_TOTAL_COLUMN, content);
	g_free(content);
    }
    g_free(description);
    g_free(vat);
    g_free(quantity);

}

/*Carga las lneas de una factura en la pantalla*/
static void fill_list_with_invoice(GtkCList  * list,Invoice * invoice) {
    gint index=0;
    gint indexDelivery=0;
    gchar *empty_row[]={"","","","","","","","","","",""};
    Repair * repair=NULL;
    DeliveryPayment ** delivery_payments = d_invoice_get_delivery_payments(invoice);
    if (delivery_payments!=NULL) {
	/*CASO 1: Factura con albaranes*/
	while (delivery_payments[indexDelivery]!=NULL) {
	    gulong repair_code;
	    gulong delivery_payment_number;
            gboolean do_add_delivery_payment_line=FALSE;
	    gulong garage_code=ULONG_MAX;
	    gulong codigoCliente=ULONG_MAX;
	    repair = d_delivery_payment_get_repair (delivery_payments[indexDelivery]);
	    repair_code = d_repair_get_repair_code (repair);
            delivery_payment_number=d_delivery_payment_get_number (delivery_payments[indexDelivery]);

	    if (d_repair_get_garage (repair)!=NULL) {
		if (d_invoice_get_participant (invoice)!=NULL) {
		    codigoCliente = d_participant_get_code (d_client_get_participant (d_repair_get_client (repair)));
		    garage_code = d_participant_get_code (d_invoice_get_participant (invoice));
		    if (codigoCliente == garage_code) {
			codigoCliente = ULONG_MAX;
                        garage_code =ULONG_MAX;
		    }
		}
		else {
		    codigoCliente = d_participant_get_code (d_client_get_participant (d_repair_get_client (repair)));
		    garage_code = d_participant_get_code (d_garage_get_participant (d_repair_get_garage (repair)));
		}
	    } 
	    if (m_delivery_payment_client_is_not_from_garage (garage_code,
                                               codigoCliente,
					       delivery_payment_number,
					       d_participant_get_code (d_shop_get_participant (d_repair_get_shop (repair))),
					       d_participant_get_code (d_enterprise_get_participant (d_repair_get_enterprise (repair)))
					      )!=FALSE) {
                do_add_delivery_payment_line = TRUE;
	    } else
		do_add_delivery_payment_line = FALSE;

	    if ((from_garage==FALSE) || (do_add_delivery_payment_line==FALSE)) {
		Repair_line ** repair_lines = d_repair_get_repair_lines(repair);
		    /*CASO 1.1: Factura sin taller */
		if (repair_lines!=NULL) {
		    gint indexLinea = 0;
		    while (repair_lines[indexLinea]) {
			if (index==30) gtk_clist_freeze ((GtkCList *) list);
                        /*Se aaden lneas de reparacin normales.*/
			gtk_clist_append ((GtkCList *) list,empty_row);
			add_list_line( list,index,repair_lines[indexLinea],repair_code,delivery_payment_number);
			index++;
			indexLinea++;
		    }
		}
	    } else {
		/*CASO 1.2: Factura de taller */
		if (index==30) gtk_clist_freeze ((GtkCList *) list);
		gtk_clist_append ((GtkCList *) list,empty_row);
                        /*Se aaden lneas de reparacin normales y en caso especial lneas por albarn.*/
		add_delivery_payment_line( list,index,delivery_payments[indexDelivery],repair_code);
                index++;
	    }
	    indexDelivery++;
	    gtk_clist_thaw((GtkCList *) list);

	}
    } else {
	Repair_line ** repair_lines = NULL;
	/*CASO 2: Factura a partir de reparacin sin albarn*/
	gulong delivery_payment_number = ULONG_MAX;
	gint indexLinea = 0;
	gulong repair_code;
	repair = d_invoice_get_repair (invoice);
        repair_lines = d_repair_get_repair_lines(repair);
	repair_code = d_repair_get_repair_code (repair);
	if (repair_lines[indexLinea]!=NULL) {
	    while (repair_lines[indexLinea]!=NULL) {
		if (index==30) gtk_clist_freeze ((GtkCList *) list);
		gtk_clist_append ((GtkCList *) list,empty_row);
		add_list_line( list,index,repair_lines[indexLinea],repair_code,delivery_payment_number);
		index++;
		indexLinea++;
	    }
	}
    }
}
/*Rellena la clist con las lneas de reparacin de la nota.*/
static void fill_list_with_payment_note(GtkCList  * list,PaymentNote * payment_note_aux) {
    gint index=0;
    gchar *empty_row[]={"","","","","","","","","","",""};
    Repair * repair=NULL;
    gulong delivery_payment_number = ULONG_MAX;
    gint indexLinea = 0;
    gulong repair_code;
    Repair_line ** repair_lines;
    repair = d_payment_note_get_repair_detail (payment_note);
    repair_lines = d_repair_get_repair_lines(repair);
    repair_code = d_repair_get_repair_code (repair);
    while (repair_lines[indexLinea]!=NULL) {
	if (index==30) gtk_clist_freeze ((GtkCList *) list);
	gtk_clist_append ((GtkCList *) list,empty_row);
        /*Aade lneas de reparacin normales*/
	add_list_line( list,index,repair_lines[indexLinea],repair_code,delivery_payment_number);
	index++;
	indexLinea++;
    }

}

/*Repinta las lnes de la factura/nota en pantalla*/
void c_invoice_repaint_repair_lines() {
    gdouble * subtotal = (gdouble *) g_malloc0(sizeof(gdouble));
    gdouble * vat = (gdouble *) g_malloc0(sizeof(gdouble));
    gdouble * total = (gdouble *) g_malloc0(sizeof(gdouble));

    GtkWidget * list = glade_xml_get_widget(xml, "lista_detalle_factura");

    gtk_clist_clear(GTK_CLIST(list));
    if (actual_invoice!=NULL) {
	fill_list_with_invoice(GTK_CLIST(list),actual_invoice);
	d_invoice_calculate_total (subtotal,vat,total,actual_invoice,FALSE);
    } else {
	fill_list_with_payment_note(GTK_CLIST(list),payment_note);
	d_payment_note_calculate_total  (subtotal,vat,total,payment_note);

    }
    if (negative_price==FALSE) {
	fill_total_data(*subtotal,*vat,*total);
    }

    g_free(subtotal);
    g_free(vat);
    g_free(total);

}


/*Establece los datos del receptor de la factura en pantalla.*/
static void fill_client_data(Participant * participant){
    if (participant!=NULL) {
	GtkWidget * entry;
	gchar * content;
        gchar * comp = d_participant_get_nick (participant);

        if (comp!=NULL) {
	    content = d_participant_get_nick (participant);
	    entry =glade_xml_get_widget (xml, "entry_nombre_factura");
	    gtk_entry_set_text (GTK_ENTRY(entry),content);
	    g_free(content);
            g_free(comp);
	}

        comp = d_participant_get_legal_id (participant);
	if (comp!=NULL) {
	    content = d_participant_get_legal_id (participant);
	    entry =glade_xml_get_widget (xml, "entry_identificador_factura");
	    gtk_entry_set_text (GTK_ENTRY(entry),content);
	    g_free(content);
            g_free(comp);
	}
    }
}
/*Establece los datos de la empresa del grupo en eb la pantalla*/
static void fill_parent_company_data(gchar * shopName){
    if (shopName!=NULL) {
	GtkWidget * entry = glade_xml_get_widget (xml, "entry_tienda_facturas");
        gtk_entry_set_text(GTK_ENTRY(entry),shopName);
    }
}

/*Rellena las entradas de texto de totales con los datos pasados por parmetros*/
static void fill_total_data(gdouble subtotal,gdouble vat,gdouble total){

    GtkWidget * entry;
    gchar * content;
    gchar * temp_total_char=NULL;
    gchar * temp_subtotal_char = NULL;
    entry = glade_xml_get_widget (xml, "entry_base_imponible_factura");
    content = g_strdup_printf("%10.2f",subtotal);
    g_strstrip(content);
    gtk_entry_set_text(GTK_ENTRY(entry),content);
    g_free(content);

    entry = glade_xml_get_widget (xml, "entry_descuento_factura");
    if (actual_invoice!=NULL) {
	if (d_invoice_get_invoice_discount (actual_invoice)!=-1) {
	    content = g_strdup_printf("%.2f",1.0*d_invoice_get_invoice_discount (actual_invoice));
	} else {
	    content = g_strconcat("0.0",NULL);
	}
    } else {
        content=g_strdup("");
    }
    gtk_entry_set_text(GTK_ENTRY(entry),content);
    g_free(content);

    entry = glade_xml_get_widget (xml, "entry_iva_factura");
    content = g_strdup_printf("%10.2f",vat);
    g_strstrip(content);
    gtk_entry_set_text(GTK_ENTRY(entry),content);
    g_free(content);

    if (actual_invoice!=NULL) {
	if (d_invoice_get_total (actual_invoice)!=G_MINDOUBLE) {
	    temp_subtotal_char = g_strdup_printf("%.2f",total);
	    temp_total_char = g_strdup_printf("%.2f",d_invoice_get_total (actual_invoice));
	    if (strcmp(temp_subtotal_char,temp_total_char)) {
		d_invoice_set_total (actual_invoice,total);
		c_util_open_question_dialog(_("The total amount of the invoice has been modified.\nDo you want to update the total amount of the invoice?"), 
					    (*confirm_update));
	    }
	    g_free(temp_subtotal_char);
            g_free(temp_total_char);
	}
    }
    entry = glade_xml_get_widget (xml, "entry_total_factura");
    content = g_strdup_printf("%10.2f",total);
    g_strstrip(content);
    gtk_entry_set_text(GTK_ENTRY(entry),content);
    g_free(content);


}

/*Ejecutado al solicitar la confirmacin de modificacin y pulsar "Si"*/
static void confirm_update(gint reply, gpointer data){
  if (!reply) modify_invoice();
}


/*Rellena los datos del siniestro en la pantalla. */
static void fill_accident_data(Repair * repair) {

    GtkWidget * entry;
    GtkWidget * gnome_date_edit;
    gchar * content;

    content = d_participant_get_name (d_company_get_participant (d_repair_get_company (repair)));
    /*Compaa de seguros*/
    if (content!=NULL) {
	entry =glade_xml_get_widget (xml, "entry_compania_factura");
	gtk_entry_set_text (GTK_ENTRY(entry),content);
	g_free(content);
    }
    /*Datos del siniestro*/
    if (d_repair_get_accident (repair)!=NULL) {
	content = d_accident_get_place (d_repair_get_accident (repair));
	if (content!=NULL) {
	    entry =glade_xml_get_widget (xml, "entry_lugar_factura");
	    gtk_entry_set_text (GTK_ENTRY(entry),content);
	    g_free(content);
	}

	content = d_accident_get_accident_reference (d_repair_get_accident (repair));
	if  (content!=NULL) {
	    entry =glade_xml_get_widget (xml, "entry_referencia_factura");
	    gtk_entry_set_text (GTK_ENTRY(entry),content);
	    g_free(content);

	}

	gnome_date_edit = glade_xml_get_widget(xml,"dateedit_siniestro_factura");
	gnome_date_edit_set_time(GNOME_DATE_EDIT(gnome_date_edit),d_accident_get_date (d_repair_get_accident (
													 repair
													)));
    }
    /*Datos del titular.*/
    if (d_repair_get_client (repair)!=NULL) {
	content = d_participant_get_nick (d_client_get_participant (d_repair_get_client (repair)));
	if (content!=NULL) {
	    entry =glade_xml_get_widget (xml, "entry_titular_factura");
	    gtk_entry_set_text (GTK_ENTRY(entry),content);
	    g_free(content);
	}
    }
    /*Datos de la pliza*/
    if (d_repair_get_policy (repair)!=NULL) {
	content = d_policy_get_policy_number(d_repair_get_policy (repair));
	if (content!=NULL) {
	    entry =glade_xml_get_widget (xml, "entry_poliza_factura");
	    gtk_entry_set_text (GTK_ENTRY(entry),content);
	    g_free(content);
	}
    }
    if (d_repair_get_vehicle (repair)!=NULL) {
	content = d_vehicle_get_license_number (d_repair_get_vehicle (repair));
	if (content!=NULL) {
	    entry =glade_xml_get_widget (xml, "entry_matricula_factura");
	    gtk_entry_set_text (GTK_ENTRY(entry),content);
	    g_free(content);

	}
	content = d_vehicle_get_chassis_number (d_repair_get_vehicle (repair));
	if (content!=NULL) {
	    entry =glade_xml_get_widget (xml, "entry_bastidor_factura");
	    gtk_entry_set_text (GTK_ENTRY(entry),content);
	    g_free(content);
	}
    }
    /*Datos del recibo*/
    if (d_repair_get_receipt (repair)!=NULL) {
	content = d_receipt_get_receipt_number (d_repair_get_receipt (repair));
	if (content!=NULL) {
	    entry =glade_xml_get_widget (xml, "entry_recibo_factura");
	    gtk_entry_set_text (GTK_ENTRY(entry),content);
	    g_free(content);
	}

	content = d_receipt_get_subsidiary_mediator (d_repair_get_receipt (repair));
	if (content!=NULL) {
	    entry =glade_xml_get_widget (xml, "entry_sucursal_mediadora_factura");
	    gtk_entry_set_text (GTK_ENTRY(entry),content);
	    g_free(content);
	}
	gnome_date_edit = glade_xml_get_widget(xml,"dateedit_validez_factura");
	gnome_date_edit_set_time(GNOME_DATE_EDIT(gnome_date_edit),d_receipt_get_validity_end (d_repair_get_receipt (
                                                                                                            repair
													   )));
    }
    /*Datos del agente*/
    if (d_repair_get_agent (repair)!=NULL) {
        content = d_participant_get_nick (d_agent_get_participant (d_repair_get_agent (repair)));
	if (content!=NULL) {
	    entry =glade_xml_get_widget (xml, "entry_nombre_agente_factura");
	    gtk_entry_set_text (GTK_ENTRY(entry),content);
	    g_free(content);
	}
    }
}

static void payment_mode_combo_init() {
    GtkWidget * combo;

    combo = glade_xml_get_widget (xml, "combo_modo_pago");

    c_util_set_code_and_name_on_combo (D_PAYMENT_AT_MOMENT,D_PAYMENT_MESSAGE_AT_MOMENT,GTK_COMBO(combo));
#ifndef IS_SHOP
    c_util_set_code_and_name_on_combo (D_PAYMENT_MONEY_ORDER,D_PAYMENT_MESSAGE_MONEY_ORDER,GTK_COMBO(combo));
    c_util_set_code_and_name_on_combo (D_PAYMENT_MONEY_ORDER_AT_END_OF_MONTH,D_PAYMENT_MESSAGE_MONEY_ORDER_AT_END_OF_MONTH,GTK_COMBO(combo));
    c_util_set_code_and_name_on_combo (D_PAYMENT_HIRE_PURCHASE,D_PAYMENT_MESSAGE_HIRE_PURCHASE,GTK_COMBO(combo));
#endif
}
static void payment_type_combo_init() {
    GtkWidget * combo;

    combo = glade_xml_get_widget (xml, "combo_medio_pago");

#ifndef IS_SHOP
    c_util_set_code_and_name_on_combo (D_PAYMENT_TRANSFER_CODE,D_PAYMENT_MESSAGE_TRANSFER,GTK_COMBO(combo));
#endif

    c_util_set_code_and_name_on_combo (D_PAYMENT_IN_CASH,D_PAYMENT_MESSAGE_IN_CASH,GTK_COMBO(combo));
#ifndef IS_SHOP
    c_util_set_code_and_name_on_combo (D_PAYMENT_CHEQUE,D_PAYMENT_MESSAGE_CHEQUE,GTK_COMBO(combo));
    c_util_set_code_and_name_on_combo (D_PAYMENT_CREDIT_CARD,D_PAYMENT_MESSAGE_CREDIT_CARD,GTK_COMBO(combo));
    c_util_set_code_and_name_on_combo (D_PAYMENT_DIRECT_DEBIT_CODE,D_PAYMENT_MESSAGE_DIRECT_DEBIT,GTK_COMBO(combo));
#endif
}
static void invoice_type_combo_init() {
    GtkWidget * combo;

    combo = glade_xml_get_widget (xml, "combo_tipo_factura");

    c_util_set_code_and_name_on_combo (D_PAYMENT_NOTE_CODE,D_PAYMENT_NOTE,GTK_COMBO(combo));
    c_util_set_code_and_name_on_combo (D_INVOICE_CODE,D_INVOICE_MESSAGE,GTK_COMBO(combo));



}

static void template_combo_init() {
    GtkWidget * combo;

    
    combo = glade_xml_get_widget (xml, "combo_seguros");

    c_util_set_code_and_name_on_combo (D_INVOICE_NORMAL_CODE,D_INVOICE_MESSAGE_NORMAL,GTK_COMBO(combo));

}

static void fill_shop_combo() {
    GtkWidget * combo;
    Shop ** shops;

    combo = glade_xml_get_widget (xml, "combo-tienda");

    shops = m_shop_find_all ();

    c_util_fill_combo_with_shops (combo,shops);

}

static void fill_enterprise_from_combo() {
    GtkWidget * entry;
    GtkWidget * combo;
    gchar * aux=NULL;
    gint code_combo;
    Enterprise * enterprise = NULL;

    entry = glade_xml_get_widget (xml, "entry_empresa_facturas");

    combo = glade_xml_get_widget (xml, "combo-tienda");

	code_combo = c_util_get_code_from_combo (combo);

	enterprise = m_enterprise_find_by_shop (code_combo);
	if (enterprise!=NULL) {
	    aux = d_participant_get_name (d_enterprise_get_participant (enterprise));
	    gtk_entry_set_text(GTK_ENTRY(entry),aux);
	    d_enterprise_clear (enterprise);
            enterprise=NULL;
	    g_free(aux);
	}
}

/*Abre la ventana de modificacin de lneas de factura.*/
static void open_update_repair_line_invoice(GtkWidget *  wid) {

    GtkWidget * app1 = lookup_widget(wid, "app1");
    GtkWidget * line_edit = NULL;

    line_edit = c_update_repair_line_open_window(app1,C_FROM_INVOICE);
    if (line_edit!=NULL) {
	gtk_window_set_modal(GTK_WINDOW(line_edit),1);
	gtk_window_set_position(GTK_WINDOW(line_edit),GTK_WIN_POS_CENTER);
	gtk_window_set_transient_for(GTK_WINDOW(line_edit),
				     GTK_WINDOW(app1));
	gtk_widget_show(GTK_WIDGET(line_edit));
    }

}

/*FIXME: Se debera realizar la bsqueda por cdigo de reparacin, numeroAlbarn y nmero de lnea en
  vez de por posicin en la factura*/
/*En caso de seleccionar una lnea de la lista de lneas recorre las lneas de la factura buscando la
 * lnea que coincide en posicin con la lnea que se seleccion en la pantalla.*/
static void on_detail_invoice_list_select_row (GtkCList        *list,
				  gint             row,
				  gint             column,
				  GdkEvent        *event,
				  gpointer         user_data)
{
    if (event!=NULL) {
	if (event->type == GDK_2BUTTON_PRESS) {
            /*Busca la lnea de la factura que coincide con la factura seleccionada en la clist*/
            Repair_line * repair_line = NULL;
	    gint count=0;
	    gint countAux=0;
	    gulong delivery_payment_number=ULONG_MAX;
            gulong repair_number=ULONG_MAX;
	    open_update_repair_line_invoice(GTK_WIDGET(list));
	    if (actual_invoice!=NULL) {
		DeliveryPayment ** dPS = d_invoice_get_delivery_payments (actual_invoice);
		if (dPS!=NULL) {
                    /*Recorre los albaranes de la factura y dentro de los albaranes las lneas de la reparacin del albarn*/
		    while (dPS[countAux]!=NULL) {
			gint exit = 0;
			DeliveryPayment * dP = dPS[countAux];
			Repair * repair = d_delivery_payment_get_repair (dP);
                        Repair_line ** repair_lines = d_repair_get_repair_lines (repair);
			if (repair_lines!=NULL) {
			    gint countAux2=0;
			    while (repair_lines[countAux2]) {
				repair_line = repair_lines[countAux2];
                                /*Comprueba si el ndice que ocupa la lnea de reparacin es el ndice de la lnea que aparace en la pantalle*/
				if (count == row) {
                                    DeliveryPayment ** delivery_payments = d_invoice_get_delivery_payments (actual_invoice);
				    delivery_payment_number=d_delivery_payment_get_number (delivery_payments[countAux]);
				    repair_number=d_repair_get_repair_code (d_delivery_payment_get_repair (delivery_payments[countAux]));
				    exit = 1;
				    break;
				}
				countAux2++;
				count++;
			    }
			}
			if (exit==1) {
			    break;
			}
			countAux++;
		    }
		} else {
		    Repair * repair = d_invoice_get_repair (actual_invoice);
                    Repair_line ** repair_lines = d_repair_get_repair_lines (repair);
		    while (repair_lines[countAux]!=NULL) {
			gint exit = 0;
			repair_line = repair_lines[countAux];
			if (count == row) {
			    repair_number=d_repair_get_repair_code (d_invoice_get_repair (actual_invoice));
			    exit = 1;
			    break;
			}
			countAux++;
			count++;
			if (exit==1) {
			    break;
			}
		    }
		}
	    } else {
		Repair * repair = d_payment_note_get_repair_detail (payment_note);
                Repair_line ** repair_lines = d_repair_get_repair_lines (repair);
		while (repair_lines[countAux]!=NULL) {
		    gint exit = 0;
		    repair_line = repair_lines[countAux];
		    /*Comprueba si el ndice que ocupa la lnea de reparacin es el ndice de la lnea que aparace en la pantalle*/
		    if (count == row) {
			repair_number=d_repair_get_repair_code (d_payment_note_get_repair_detail (payment_note));
			exit = 1;
			break;
		    }
		    countAux++;
		    count++;
		    if (exit==1) {
			break;
		    }
		}

	    }
	    c_update_repair_line_load_data(repair_line,ULONG_MAX,repair_number);
	    //	  i_shortcut_change_dialog (GTK_WIDGET(list),1,2);
	}
    }
}
/* Guarda la factura en la base de datos. Primero obtiene los datos adecuados de pantalla y
 * despus inserta la factura. En caso de que se haya insertado correctamente se mostrar en la
 * pantalla el nmero de la factura, en otro caso no se mostrar nada.
 * A partir de data obtiene el receptor de la factura ya que segn seleccione el usuario se
 * pone como receptor al cliente, a la compaa de seguros o al consorcio */
void save_invoice(gpointer data) {
    gulong invoice_number;
    gchar * content;
    GtkWidget * entry = glade_xml_get_widget (xml, "entry_numero_factura");
    GtkWidget * label = glade_xml_get_widget (xml, "label_numero_factura");
    gchar * label_char;

    content = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry)));
    if (!strcmp(content,"")) {
	GtkWidget * entry_combo_factura=glade_xml_get_widget(xml,"entry_tipo_combo_factura");
	GtkWidget * issue_date = glade_xml_get_widget(xml,"dateedit_emision");
	gchar * invoice_type = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry_combo_factura)));
	Payment * payment = NULL;
#ifndef IS_SHOP
	put_buttons_sensible(1);
#else
	put_buttons_sensible(6);
#endif
	view_garage_total_entry(FALSE);
	g_free(content);
	if (!strcmp(invoice_type,D_INVOICE_MESSAGE)) {
            /*INSERCIN DE UNA FACTURA*/
	    GtkWidget * check_invoice = glade_xml_get_widget(xml,"checkbutton_refacturacion");
	    GtkWidget * entry_total_invoice = glade_xml_get_widget(xml,"entry_total_factura");
            gchar * total = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry_total_invoice)));
	    gulong * receiver = (gulong *) data;

	    Enterprise * enterprise_to_dup=NULL;
	    Shop * shop_to_dup=NULL;
            Participant * participant_to_dup=NULL;
	    Repair * repair;
	    u_points_to_commas (total);

            d_invoice_set_issue_date (actual_invoice,gnome_date_edit_get_date(GNOME_DATE_EDIT(issue_date)));
            d_invoice_set_state (actual_invoice,D_INVOICE_NOT_ISSUED);
	    d_invoice_set_garage_is_receiver (actual_invoice,FALSE);
            repair = d_invoice_get_first_repair(actual_invoice);

	    enterprise_to_dup= d_repair_get_enterprise (repair);
	    shop_to_dup = d_repair_get_shop (repair);
            /*selecciona el receptor de la factura*/
	    if ((*receiver) == D_INVOICE_CLIENT_SELECTION) {
		participant_to_dup = d_client_get_participant (d_repair_get_client (repair));
	    } else if ((*receiver) == D_INVOICE_COMPANY_SELECTION) {
		participant_to_dup = d_company_get_participant (d_repair_get_company (repair));
	    } else if ((*receiver) == D_INVOICE_CONSORTIUM_SELECTION) {
		Consortium ** consortium;
		gulong consortium_code = d_company_get_consortium_code (d_repair_get_company (repair));
		consortium = m_consortium_find_by_code (consortium_code);
		participant_to_dup = d_consortium_get_participant (consortium[0]);
	    }


	    d_invoice_set_enterprise (actual_invoice,d_enterprise_duplicate (enterprise_to_dup));
	    d_invoice_set_shop (actual_invoice,d_shop_duplicate (shop_to_dup));
	    d_invoice_set_participant (actual_invoice,d_participant_duplicate (participant_to_dup));
            modify_client_invoice(NULL);
            /*Crea el pago a partir de la pantalla de pago.*/
	    payment = create_payment_from_screen(0);
            /*Comprueba si se desea realizar una refactura*/
	    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_invoice))!=FALSE) {
                GtkWidget * split_invoice_combo = glade_xml_get_widget(xml,"combo_tienda_refacturacion");
		gulong split_invoice_shop_code = c_util_get_code_from_combo (split_invoice_combo);
		d_invoice_set_shop_code_split_invoice (actual_invoice,split_invoice_shop_code);
	    }

	    if (payment!=NULL) {
                /*Establece el pago en la factura*/
		d_invoice_set_payment (actual_invoice,payment);

		if (d_invoice_get_modified_total (actual_invoice)==FALSE)
		    d_invoice_set_total (actual_invoice,atof(total));
                /*Inserta la factura en la base de datos*/
		invoice_number = m_invoice_insert (actual_invoice);
		g_print("%lu",invoice_number);
		if (invoice_number!=ULONG_MAX) {
		    /*Insercin correcta. Imprimimos el nmero de la factura con la letra en la entrada
		     * de texto reservada para el nmero de factura*/
		    GtkWidget * combo;

		    d_invoice_set_invoice_number (actual_invoice,invoice_number);
		    combo = glade_xml_get_widget(xml,"combo_tipo_factura");
		    if (GTK_WIDGET_SENSITIVE(combo)) gtk_widget_set_sensitive(combo,FALSE);
		    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_invoice))!=FALSE)
			set_split_invoice_state(TRUE,FALSE);
                    fill_invoice_number_data(actual_invoice);
		    label_char = g_strconcat("Nmero factura: ",NULL);
		    gtk_label_set_text (GTK_LABEL(label),label_char);
		    g_free(label_char);
		}
	    } else {
		/*Fallo al insertar: Limpiamos los datos de la factura*/
		d_invoice_clear (actual_invoice);
                actual_invoice=NULL;
	    }
	} else {
            /*INSERCIN DE UNA NOTA*/
	    Repair * repair;
	    Enterprise * enterprise_to_dup=NULL;
	    Shop * shop_to_dup=NULL;
	    GtkWidget * combo;
	    GtkWidget * entry_total_nota = glade_xml_get_widget(xml,"entry_base_imponible_factura");
            gchar * total = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry_total_nota)));
	    gulong payment_note_number;
            gchar * payment_note_number_char;
	    combo = glade_xml_get_widget(xml,"combo_tipo_factura");

            /*Se transforma la factura que hay en memoris por una nota*/
	    payment_note = d_invoice_to_payment_note (actual_invoice);

            /*desreferenciamos la factura, a partir de ahora solo existe la nota*/
	    actual_invoice=NULL;
	    u_points_to_commas (total);

            /*Obtiene el resto de los datos necesarios de pantalla*/
	    d_payment_note_set_emission_date (payment_note,gnome_date_edit_get_date(GNOME_DATE_EDIT(issue_date)));

	    repair = d_payment_note_get_repair_detail (payment_note);
	    enterprise_to_dup= d_repair_get_enterprise (repair);
	    shop_to_dup = d_repair_get_shop (repair);
	    payment = create_payment_from_screen(0);
	    d_payment_note_set_payment (payment_note,payment);
            d_payment_note_set_total (payment_note,atof(total));

            /*INSERTA LA NOTA*/
	    payment_note_number = m_payment_note_insert (payment_note);

            /*Se imprime el nmero de nota en la pantalla.*/
	    payment_note_number_char = g_strdup_printf("%lu",payment_note_number);
	    g_print("\n%s\n",payment_note_number_char);
	    d_payment_note_set_note_code (payment_note,payment_note_number);
	    gtk_entry_set_text (GTK_ENTRY(entry),payment_note_number_char);
	    label_char = g_strconcat(_("Payment note number: "),NULL);

	    gtk_label_set_text (GTK_LABEL(label),label_char);
            g_free(label_char);

	    gtk_widget_set_sensitive(combo,FALSE);
	    g_free(payment_note_number_char);

	}
        g_free(invoice_type);

    } else {
	c_util_open_info_window (_("The invoice already exists"));
    }

}

/* Guarda la factura DEL TALLER en la base de datos. Obtiene los
 * datos necesarios de pantalla y la inserta en la base de datos. El mtodo
 * de insercin es diferente ya que las facturas de taller son un caso distinto
 * a lpoder llevar lneas de factura que no pertenecen a lneas de reparacin
 * sino que pertenecen a lneas por albarn*/
void save_garage_invoice(void) {
    gulong invoice_number;
    gchar * content;
    GtkWidget * entry = glade_xml_get_widget (xml, "entry_numero_factura");
    GtkWidget * label = glade_xml_get_widget (xml, "label_numero_factura");
    gchar * label_char=NULL;

    content = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry)));
    if (!strcmp(content,"")) {
	GtkWidget * combo_tienda = glade_xml_get_widget(xml,"combo-tienda");
	GtkWidget * entry_total_invoice = glade_xml_get_widget(xml,"entry_total_factura");
	gchar * total = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry_total_invoice)));
	Enterprise * enterprise_to_dup=NULL;
	Shop * shop_to_dup=NULL;
	Participant * participant_to_dup=NULL;
	
	gulong shopCode;
	Payment * payment = NULL;
	put_buttons_sensible(1);
	g_free(content);
        u_points_to_commas (total);
	d_invoice_set_state (actual_invoice,D_INVOICE_NOT_ISSUED);
        d_invoice_set_garage_is_receiver (actual_invoice,TRUE);
        shopCode = c_util_get_code_from_combo (combo_tienda);
	shop_to_dup= m_shop_find_by_shop_code (shopCode);
	enterprise_to_dup = m_enterprise_find_by_shop (shopCode);
	participant_to_dup = d_delivery_payment_obtain_garage (d_invoice_get_delivery_payments (actual_invoice));
	d_invoice_set_enterprise (actual_invoice,enterprise_to_dup);
	d_invoice_set_shop (actual_invoice,shop_to_dup);
	d_invoice_set_participant (actual_invoice,d_participant_duplicate (participant_to_dup));

	payment = create_payment_from_screen(0);

	if (payment!=NULL) {
	    d_invoice_set_payment (actual_invoice,payment);

	    if (d_invoice_get_modified_total (actual_invoice)==FALSE)
		d_invoice_set_total (actual_invoice,atof(total));
	    invoice_number = m_invoice_insert (actual_invoice);
	    if (invoice_number!=ULONG_MAX) {
		d_invoice_set_invoice_number (actual_invoice,invoice_number);

		fill_invoice_number_data(actual_invoice);
		label_char = g_strconcat(_("Invoice number: "),NULL);
		gtk_label_set_text (GTK_LABEL(label),label_char);
		if (label_char!=NULL)
		    g_free(label_char);
	    }
	} else {
	    d_invoice_clear (actual_invoice);
	    actual_invoice=NULL;
	}

    } else {
	c_util_open_info_window (_("The invoice already exists"));
    }
}

/*Realiza los pasos previos a la insercin de la factura en la BD.
 *En caso de insercin de factura normal se abre el dilogo de seleccin
 *receptor de la factura, comprueba la correccin de la cuenta corriente en caso de domiciliacin,
 *y abre una ventana de insercin de movimiento de caja en caso de ser una factura pagada en caja */
static void on_save_button_clicked(GtkButton *button, gpointer data) {
    if ((actual_invoice!=NULL) || (payment_note!=NULL)) {
	GtkWidget * entry = glade_xml_get_widget (xml, "entry_numero_factura");
	GtkWidget * direct_debit_entry = glade_xml_get_widget (xml, "entry_combo_cuenta_factura");
	GtkWidget * direct_debit_combo= glade_xml_get_widget (xml, "combo_medio_pago");
	gchar * content;
	GtkWidget * app1 = lookup_widget(GTK_WIDGET(button), "app1");
	GtkWidget * open_ir;
	gboolean incorrect_account=FALSE;

	if (c_util_get_code_from_combo (direct_debit_combo)==D_PAYMENT_DIRECT_DEBIT_CODE) {
	    gchar * account_direct_debit = g_strdup(gtk_entry_get_text(GTK_ENTRY(direct_debit_entry)));
	    if (!strcmp(account_direct_debit,""))
		incorrect_account=TRUE;
            g_free(account_direct_debit);
	}

	if (incorrect_account==FALSE) {
	    content = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry)));
	    if (!strcmp(content,"")) {
		GtkWidget * entry_combo_invoice=glade_xml_get_widget(xml,"entry_tipo_combo_factura");
		gchar * invoice_type = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry_combo_invoice)));
		if (!strcmp(invoice_type,D_INVOICE_MESSAGE)) {
		    if (from_garage==FALSE) {
			open_ir = c_invoice_receiver_open_window(app1,save_invoice,actual_invoice);
			if (c_invoice_receiver_open_window!=NULL) {
			    gtk_window_set_modal(GTK_WINDOW(open_ir),1);
			    gtk_window_set_position(GTK_WINDOW(open_ir),GTK_WIN_POS_CENTER);
			    gtk_window_set_transient_for(GTK_WINDOW(open_ir),
							 GTK_WINDOW(app1));
			    gtk_widget_show(GTK_WIDGET(open_ir));
			}
		    } else {
			save_garage_invoice();
		    }
		} else {
		    Payment * payment_aux =NULL;

		    save_invoice(NULL);
		    payment_aux= d_payment_note_get_payment (payment_note);
		    open_ir = c_cash_operation_open_window(app1,NULL,TRUE);
		    load_payment_cash_operation(payment_aux,d_payment_note_get_total (payment_note));
                    hide_cancelar_button_on_cash_operation();
		    if (open_ir!=NULL) {
			gtk_window_set_modal(GTK_WINDOW(open_ir),1);
			gtk_window_set_position(GTK_WINDOW(open_ir),GTK_WIN_POS_CENTER);
			gtk_window_set_transient_for(GTK_WINDOW(open_ir),
						     GTK_WINDOW(app1));
			gtk_widget_show(GTK_WIDGET(open_ir));
		    }
		}
	    } else {
		c_util_open_info_window (_("The invoice already exists"));
	    }
	} else {
		c_util_open_error_window (_("The account number doesn't exist.\n Insert an account in the clients modification window"));
	}
    } else {
	c_util_open_info_window (_("Empty invoice"));
    }
}

static void on_shop_entry_changed(GtkEntry *entry,
				    gpointer user_data) {
    fill_enterprise_from_combo();
}


static void clean_screen() {

    GtkWidget *vbox;
    gchar * s_name=NULL;
    vbox = GTK_WIDGET(glade_xml_get_widget(xml, widget_name));

    gtk_container_foreach(GTK_CONTAINER(vbox),&c_util_clean_widget,NULL);
    s_name = u_parameter_manager_get_parameter ("tienda");
      if (s_name!=NULL) {
	  gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget (xml, "entry_tienda_facturas")), s_name);
      }
      g_free(s_name);


}
/* Establece el valor que inserta el usuario como base imponible.
 * Esta funcin es muy delicada ya que se puede dar varios casos,
 * que el cliente crea conveniente modificar la BI de la factura
 * a un valor pactado con la compaa de seguros ,que el usuario
 * quiera reestablecer el valor anterior a cambiar base imponible o bien
 * que el usuario desee combiar la base imponible pactada con la compaa
 * por otra cantidad tambien pactada con la compaa.*/
void set_subtotal(gpointer value) {
    gdouble * value_gdouble = value;

    if (actual_invoice!=NULL) {
        gdouble value_to_invoice = G_MINDOUBLE;
	gdouble * subtotal = (gdouble *) g_malloc0(sizeof(gdouble));
	gdouble * vat = (gdouble *) g_malloc0(sizeof(gdouble));
	gdouble * total = (gdouble *) g_malloc0(sizeof(gdouble));
	gboolean with_discount = d_invoice_get_with_insurance (actual_invoice);
	/* Guarda el tipo de base imponible antes de realizar operaciones para
	 * reestablecerlo ms tarde */
        gboolean estado_ant=d_invoice_get_modified_total (actual_invoice);

	gdouble value_gdouble_aux;


	d_invoice_set_modified_total (actual_invoice,FALSE);
        /*CALCULA LOS TOTALES COMO SI NO FUESE MODULO PACTADO*/
	d_invoice_calculate_total (subtotal, vat,total,actual_invoice,FALSE);
        /*Almacena el valor de la base obtenido*/
	value_gdouble_aux = *subtotal;
        /*Restablece el estado del tipo de base imponible*/
        d_invoice_set_modified_total (actual_invoice,estado_ant);

        value_to_invoice = m_invoice_modify_subtotal (actual_invoice,*value_gdouble,with_discount);
	d_invoice_set_total(actual_invoice,value_to_invoice);

	if (value_gdouble_aux == *value_gdouble) {
            GtkWidget * entry = glade_xml_get_widget(xml,"entry_descuento_factura");
	    d_invoice_set_modified_total (actual_invoice,FALSE);
	    gtk_widget_set_sensitive(entry,TRUE);
	} else {
            GtkWidget * entry = glade_xml_get_widget(xml,"entry_descuento_factura");
	    d_invoice_set_modified_total (actual_invoice,TRUE);
            gtk_widget_set_sensitive(entry,TRUE);

	}
        /*CALCULA LOS VALORES REALES DE LOS TOTALES*/
	d_invoice_calculate_total (subtotal, vat,total,actual_invoice,TRUE);

        /*CARGA LOS VALORES REALES DE LOS TOTALES EN LA PANTALLA*/
	fill_total_data(*subtotal,* vat,*total);

    } else if (payment_note!=NULL) {
        /*EN LA NOTA CALCULA LOS TOTALES Y LOS ESTABLECE EN LA PANTALLA*/
	gdouble * subtotal = (gdouble *) g_malloc0(sizeof(gdouble));
	gdouble * vat= (gdouble *) g_malloc0(sizeof(gdouble));
	gdouble * total = (gdouble *) g_malloc0(sizeof(gdouble));
	d_payment_note_set_total (payment_note,*value_gdouble);

	d_payment_note_calculate_total  (subtotal, vat,total,payment_note);

	fill_total_data(*subtotal,* vat,*total);

    } else {
        c_util_open_error_window (_("The value can't be modified"));
    }
    g_free(value_gdouble);
    value = NULL;
}

void set_discount_value(gpointer value) {
    gdouble * value_gulong = value;

    if (actual_invoice!=NULL) {
	gdouble * subtotal = (gdouble *) g_malloc0(sizeof(gdouble));
	gdouble * vat= (gdouble *) g_malloc0(sizeof(gdouble));
	gdouble * total = (gdouble *) g_malloc0(sizeof(gdouble));

	d_invoice_set_invoice_discount (actual_invoice,*value_gulong);
        /*Recalcula los valores de los totales*/
	d_invoice_calculate_total (subtotal, vat,total,actual_invoice,TRUE);
	fill_total_data(*subtotal,* vat,*total);

    } else {
        c_util_open_error_window (_("The discount value can't be modified"));
    }
    value = NULL;
}


gboolean on_subtotal_entry_press_event(GtkWidget *widget,
                                            GdkEventButton *event,
							gpointer user_data) {
    if (negative_price==FALSE) {
	GtkWidget * app1 = lookup_widget(widget, "app1");
	gchar * temp = g_strdup(_("Taxable income"));
	c_update_entry_open_window(app1,temp,C_GDOUBLE_TYPE,set_subtotal);
	g_free(temp);
    }
    return FALSE;
}

gboolean on_discount_entry_press_event(GtkWidget *widget,
                                            GdkEventButton *event,
							gpointer user_data) {
    if (negative_price==FALSE) {
	GtkWidget * app1 = lookup_widget(widget, "app1");
	gchar * temp = g_strdup(_("Discount"));
	c_update_entry_open_window(app1,temp,C_GDOUBLE_TYPE,set_discount_value);
	g_free(temp);
    } 
    return FALSE;
}

static Listing * obtain_listing_from_array(DeliveryPayment ** delivery_payments) {

	gulong rows;
		
	Listing * listado = p_listing_init ();
				
	listado->content_title = d_delivery_payment_get_content_title ();
	listado->content       = d_delivery_payment_array_to_content (delivery_payments, &rows);
	listado->width_of_cols = d_delivery_payment_get_listing_columns_width ();
	listado->alignment_of_cols = d_delivery_payment_get_listing_columns_alignment ();
	listado->n_rows        = rows;
	listado->n_columns     = d_delivery_payment_get_content_size ();
	listado->orientation   = 4;
	listado->type_of_paper = 2;
	
	return listado;
}
					

void disassociate_delivery_payments(gpointer user_data, gint len) {

	gchar ** selected = (gchar **) user_data;
	
	if (len > 0) {
	
		InvoiceId *id;			
		gint index=0;
		
		while (index < len) {			
			
			gchar *shop_char = m_invoice_get_shop_code_as_gchar (actual_invoice);
			gchar *enterprise_char = m_invoice_get_enterprise_code_as_gchar (actual_invoice);
			gchar *num_invoice = g_strdup_printf("%lu" , d_invoice_get_invoice_code (actual_invoice));
			gchar *num = selected[index];
			
			m_delivery_payment_dissasociate_invoice (num, shop_char, enterprise_char, num_invoice);
			
			index++;
		}
		id = m_invoice_get_id (actual_invoice);	
			
		clean_screen();
		c_invoice_load(id);
								
	} else {
	
		c_util_open_error_window (_("You must select at least one delivery payment"));
	}		
}
	

static void on_activate_disassociate_delivery_payment(GtkMenuItem *menuitem,
				   gpointer user_data) {

    if ((d_invoice_get_state (actual_invoice)==D_INVOICE_NOT_ISSUED) || (d_invoice_get_state (actual_invoice)==D_INVOICE_NOT_PAID)) {
	gint number_delivery_payments = d_invoice_get_number_of_delivery_payments (actual_invoice);
	delivery_payments = d_invoice_get_delivery_payments (actual_invoice);

	if (number_delivery_payments > 1) {

	    Listing * listado = obtain_listing_from_array(delivery_payments);
	    char * message = _("Select the delivery payments that you want to unlink");

	    c_util_open_listing_window (listado, message, disassociate_delivery_payments);

	} else  {

	    c_util_open_info_window (_("At least one delivery payment must be linked to this invoice"));
	}
    } else {
	c_util_open_info_window (_("You just can unlink delivery payments from unpaid invoices"));

    }

}

static gint on_detail_invoice_list_button_press_event(GtkWidget *widget, 
	GdkEventButton *event, gpointer user_data) {	
	
	if (actual_invoice != NULL) {
	
		GtkWidget * menu_item1 = gtk_menu_item_new_with_label(_("Unlink delivery payments"));
		GtkWidget * menu = gtk_menu_new();
		GdkEventButton *event_button;

		gtk_menu_item_activate(GTK_MENU_ITEM(menu_item1));
		gtk_menu_set_title(GTK_MENU(menu),_("Select the operation"));
		gtk_menu_prepend(GTK_MENU(menu),menu_item1);
		gtk_widget_show(menu_item1);
		gtk_signal_connect_object(GTK_OBJECT(menu_item1), "activate",
			GTK_SIGNAL_FUNC (on_activate_disassociate_delivery_payment), NULL);
			    
	        if (event->type == GDK_BUTTON_PRESS) {
			event_button = (GdkEventButton *) event;
			if (event_button->button == 3) {
				gtk_menu_popup (GTK_MENU(menu), NULL, NULL, NULL, NULL,
					event_button->button, event_button->time);
				return TRUE;
		  	}
		}	
	}
	return FALSE;
}

static void on_payment_mean_combo_entry_hanged(GtkEntry *entry,
					     gpointer user_data) {

    GtkWidget * combo = glade_xml_get_widget(xml,"combo_modo_pago");
    GtkWidget * money_order_days_label = glade_xml_get_widget(xml,"label_dias_giro");
    GtkWidget * spinButton = glade_xml_get_widget(xml,"dias_giro");
    gulong code_combo = 0;
    gfloat f = 1.0;
    code_combo = c_util_get_code_from_combo (combo);
    /*Valores por defecto en caso de los diferentes tipos de pago*/
    if (code_combo == D_PAYMENT_AT_MOMENT) {

        restore_to_initial_purchase(TRUE);
        gtk_widget_set_sensitive(GTK_WIDGET(spinButton),FALSE);
        f=0.0;
        gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinButton),f);
        gtk_label_set_text(GTK_LABEL(money_order_days_label),"");

    } else if (code_combo == D_PAYMENT_MONEY_ORDER) {
        restore_to_initial_purchase(TRUE);
	gtk_widget_set_sensitive(GTK_WIDGET(spinButton),TRUE);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinButton),f);
	gtk_label_set_text(GTK_LABEL(money_order_days_label),_("Draft days"));

    } else if (code_combo == D_PAYMENT_MONEY_ORDER_AT_END_OF_MONTH) {
        restore_to_initial_purchase(TRUE);
        gtk_widget_set_sensitive(GTK_WIDGET(spinButton),FALSE);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinButton),f);
	gtk_label_set_text(GTK_LABEL(money_order_days_label),_("Deferment days"));

    }
    else if (code_combo == D_PAYMENT_HIRE_PURCHASE) {
        restore_to_initial_purchase(FALSE);
        gtk_widget_set_sensitive(GTK_WIDGET(spinButton),TRUE);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinButton),f);
	gtk_label_set_text(GTK_LABEL(money_order_days_label),_("Number of instalments"));

    }

    set_dates_on_payment();

}

static void on_payment_means_combo_entry_changed(GtkEntry *entry,
					      gpointer user_data) {
    GtkWidget * combo = glade_xml_get_widget(xml,"combo_medio_pago");
    gint code_combo = c_util_get_code_from_combo (combo);
    GtkWidget * label_tarjeta = glade_xml_get_widget(xml,"label_valida_hasta_factura");
    GtkWidget * date_edit_valida = glade_xml_get_widget(xml,"dateedit_valida_hasta_factura");
    GtkWidget * label_numero_tarjeta = glade_xml_get_widget(xml,"label_numero_identificador_factura");
    GtkWidget * combo_numero_cuenta = glade_xml_get_widget(xml,"combo_numero_cuenta_factura");
    GtkWidget * spin;
    GtkAdjustment * adjustment;
    GtkWidget * entry_numero_cuenta;


    if (GTK_WIDGET_VISIBLE(label_tarjeta)) gtk_widget_hide(label_tarjeta);
    if (GTK_WIDGET_VISIBLE(date_edit_valida)) gtk_widget_hide(date_edit_valida);
    if (GTK_WIDGET_VISIBLE(label_numero_tarjeta)) gtk_widget_hide(label_numero_tarjeta);

    /*Esconde o ensea y desactiva o activa botones, entradas de text y etiquetas segun el medio de
     * pago*/
    if (code_combo == D_PAYMENT_TRANSFER_CODE) {

	spin = glade_xml_get_widget(xml,"spinbutton_primer_numero");
	if (GTK_WIDGET_VISIBLE(spin)) gtk_widget_hide(spin);
	spin = glade_xml_get_widget(xml,"spinbutton_segundo_numero");
	if (GTK_WIDGET_VISIBLE(spin)) gtk_widget_hide(spin);
	spin = glade_xml_get_widget(xml,"spinbutton_tercer_numero");
	if (GTK_WIDGET_VISIBLE(spin)) gtk_widget_hide(spin);
	spin = glade_xml_get_widget(xml,"spinbutton_cuarto_numero");
	if (GTK_WIDGET_VISIBLE(spin)) gtk_widget_hide(spin);
	entry_numero_cuenta = glade_xml_get_widget(xml,"entry_numero_cuenta");
	if (GTK_WIDGET_VISIBLE(entry_numero_cuenta)) gtk_widget_hide(entry_numero_cuenta);
	if (GTK_WIDGET_VISIBLE(combo_numero_cuenta)) gtk_widget_hide(combo_numero_cuenta);

    } else if (code_combo == D_PAYMENT_IN_CASH) {

	spin = glade_xml_get_widget(xml,"spinbutton_primer_numero");
	if (GTK_WIDGET_VISIBLE(spin)) gtk_widget_hide(spin);
	spin = glade_xml_get_widget(xml,"spinbutton_segundo_numero");
	if (GTK_WIDGET_VISIBLE(spin)) gtk_widget_hide(spin);
	spin = glade_xml_get_widget(xml,"spinbutton_tercer_numero");
	if (GTK_WIDGET_VISIBLE(spin)) gtk_widget_hide(spin);
	spin = glade_xml_get_widget(xml,"spinbutton_cuarto_numero");
	if (GTK_WIDGET_VISIBLE(spin)) gtk_widget_hide(spin);
	entry_numero_cuenta = glade_xml_get_widget(xml,"entry_numero_cuenta");
	if (GTK_WIDGET_VISIBLE(entry_numero_cuenta)) gtk_widget_hide(entry_numero_cuenta);
	if (GTK_WIDGET_VISIBLE(combo_numero_cuenta)) gtk_widget_hide(combo_numero_cuenta);

    } else if (code_combo == D_PAYMENT_CREDIT_CARD) {
        GtkWidget * label = glade_xml_get_widget(xml,"label_valida_hasta_factura");
        GtkWidget * label_numero_tarjeta = glade_xml_get_widget(xml,"label_numero_identificador_factura");
	GtkWidget * date_edit = glade_xml_get_widget(xml,"dateedit_valida_hasta_factura");
	if (!GTK_WIDGET_VISIBLE(label_numero_tarjeta)) gtk_widget_show(label_numero_tarjeta);
	if (GTK_WIDGET_VISIBLE(label)==FALSE) gtk_widget_show(label);
	if (GTK_WIDGET_VISIBLE(date_edit)==FALSE) gtk_widget_show(date_edit);
	gtk_label_set_text(GTK_LABEL(label_numero_tarjeta),_("Card number"));
	spin = glade_xml_get_widget(xml,"spinbutton_primer_numero");
	if (!GTK_WIDGET_VISIBLE(spin)) gtk_widget_show(spin);
        adjustment = gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(spin));
	adjustment->upper=9999;
        gtk_adjustment_changed(adjustment);
	gtk_spin_button_update(GTK_SPIN_BUTTON(spin));
	spin = glade_xml_get_widget(xml,"spinbutton_segundo_numero");
	if (!GTK_WIDGET_VISIBLE(spin)) gtk_widget_show(spin);
        adjustment = gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(spin));
	adjustment->upper=9999;
        gtk_adjustment_changed(adjustment);
	gtk_spin_button_update(GTK_SPIN_BUTTON(spin));
	spin = glade_xml_get_widget(xml,"spinbutton_tercer_numero");
	if (!GTK_WIDGET_VISIBLE(spin)) gtk_widget_show(spin);
        adjustment = gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(spin));
	adjustment->upper=9999;
        gtk_adjustment_changed(adjustment);
	gtk_spin_button_update(GTK_SPIN_BUTTON(spin));
	spin = glade_xml_get_widget(xml,"spinbutton_cuarto_numero");
	if (!GTK_WIDGET_VISIBLE(spin)) gtk_widget_show(spin);
        adjustment = gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(spin));
	adjustment->upper=9999;
        gtk_adjustment_changed(adjustment);
	gtk_spin_button_update(GTK_SPIN_BUTTON(spin));
	entry_numero_cuenta = glade_xml_get_widget(xml,"entry_numero_cuenta");
	if (GTK_WIDGET_VISIBLE(entry_numero_cuenta)) gtk_widget_hide(entry_numero_cuenta);
	if (GTK_WIDGET_VISIBLE(combo_numero_cuenta)) gtk_widget_hide(combo_numero_cuenta);

    } else if (code_combo == D_PAYMENT_CHEQUE) {
        GtkWidget * label_numero_talon = glade_xml_get_widget(xml,"label_numero_identificador_factura");
	if (!GTK_WIDGET_VISIBLE(label_numero_talon)) gtk_widget_show(label_numero_talon);
        gtk_label_set_text(GTK_LABEL(label_numero_tarjeta),_("Check number"));
	spin = glade_xml_get_widget(xml,"spinbutton_primer_numero");
	if (!GTK_WIDGET_VISIBLE(spin)) gtk_widget_show(spin);
        adjustment = gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(spin));
	adjustment->upper=9;
        gtk_adjustment_changed(adjustment);
	gtk_spin_button_update(GTK_SPIN_BUTTON(spin));
	spin = glade_xml_get_widget(xml,"spinbutton_segundo_numero");
	if (!GTK_WIDGET_VISIBLE(spin)) gtk_widget_show(spin);
        adjustment = gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(spin));
	adjustment->upper=999;
        gtk_adjustment_changed(adjustment);
	gtk_spin_button_update(GTK_SPIN_BUTTON(spin));
	spin = glade_xml_get_widget(xml,"spinbutton_tercer_numero");
	if (!GTK_WIDGET_VISIBLE(spin)) gtk_widget_show(spin);
        adjustment = gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(spin));
	adjustment->upper=999;
        gtk_adjustment_changed(adjustment);
	gtk_spin_button_update(GTK_SPIN_BUTTON(spin));
	spin = glade_xml_get_widget(xml,"spinbutton_cuarto_numero");
	if (!GTK_WIDGET_VISIBLE(spin)) gtk_widget_show(spin);
        adjustment = gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(spin));
	adjustment->upper=9;
        gtk_adjustment_changed(adjustment);
	gtk_spin_button_update(GTK_SPIN_BUTTON(spin));
	entry_numero_cuenta = glade_xml_get_widget(xml,"entry_numero_cuenta");
	if (GTK_WIDGET_VISIBLE(entry_numero_cuenta)) gtk_widget_hide(entry_numero_cuenta);
	if (GTK_WIDGET_VISIBLE(combo_numero_cuenta)) gtk_widget_hide(combo_numero_cuenta);

    } else if (code_combo == D_PAYMENT_DIRECT_DEBIT_CODE) {
        GtkWidget * label_numero_cuenta = glade_xml_get_widget(xml,"label_numero_identificador_factura");

	if (!GTK_WIDGET_VISIBLE(label_numero_cuenta)) gtk_widget_show(label_numero_cuenta);
	gtk_label_set_text(GTK_LABEL(label_numero_cuenta),_("Current account"));
	spin = glade_xml_get_widget(xml,"spinbutton_primer_numero");
	if (GTK_WIDGET_VISIBLE(spin)) gtk_widget_hide(spin);
	spin = glade_xml_get_widget(xml,"spinbutton_segundo_numero");
	if (GTK_WIDGET_VISIBLE(spin)) gtk_widget_hide(spin);
	spin = glade_xml_get_widget(xml,"spinbutton_tercer_numero");
	if (GTK_WIDGET_VISIBLE(spin)) gtk_widget_hide(spin);
	spin = glade_xml_get_widget(xml,"spinbutton_cuarto_numero");
	if (GTK_WIDGET_VISIBLE(spin)) gtk_widget_hide(spin);
	entry_numero_cuenta = glade_xml_get_widget(xml,"entry_numero_cuenta");
	if (GTK_WIDGET_VISIBLE(entry_numero_cuenta)) gtk_widget_hide(entry_numero_cuenta);

	if (!GTK_WIDGET_VISIBLE(combo_numero_cuenta)) gtk_widget_show(combo_numero_cuenta);


    } else {
        g_print(_("Not yet implemented this mode of payment"));
    }


}

/* Reestablece ciertos valores*/
static void restore_to_initial_purchase(gboolean no_sensitive) {
    gfloat f1 = 100;
    gfloat fl = 1;
    GtkWidget * spin1 = glade_xml_get_widget(xml,"spinbutton_entrada");
    GtkWidget * spin2 = glade_xml_get_widget(xml,"spinbutton_intervalo");
    GtkWidget * date_edit = glade_xml_get_widget(xml,"dateedit_primer_pago");
    if (no_sensitive) {
	gtk_widget_set_sensitive(spin2,FALSE);
	gtk_widget_set_sensitive(spin1,FALSE);
	gtk_widget_set_sensitive(date_edit,FALSE);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin1),f1);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin2),fl);
    } else {
	gtk_widget_set_sensitive(spin2,TRUE);
	gtk_widget_set_sensitive(spin1,TRUE);
	gtk_widget_set_sensitive(date_edit,TRUE);
    }


}

static Payment * create_payment_from_screen(gint split_invoice) {

    GtkWidget * payment_mean_combo = glade_xml_get_widget(xml,"combo_medio_pago");
    GtkWidget * payment_mode_combo = glade_xml_get_widget(xml,"combo_modo_pago");
    GtkWidget * dateedit_primerPago=  glade_xml_get_widget(xml,"dateedit_primer_pago");
    GtkWidget * spinbutton_entrada = glade_xml_get_widget(xml,"spinbutton_entrada");
    GtkWidget * spinbutton_intervalo = glade_xml_get_widget(xml,"dias_giro");
    GtkWidget * spinbutton_vencimientos = glade_xml_get_widget(xml,"spinbutton_intervalo");
    GtkWidget * spinbutton_primer_numero = glade_xml_get_widget(xml,"spinbutton_primer_numero");
    GtkWidget * spinbutton_segundo_numero = glade_xml_get_widget(xml,"spinbutton_segundo_numero");
    GtkWidget * spinbutton_tercer_numero = glade_xml_get_widget(xml,"spinbutton_tercer_numero");
    GtkWidget * spinbutton_cuarto_numero = glade_xml_get_widget(xml,"spinbutton_cuarto_numero");
    GtkWidget * valida_hasta = glade_xml_get_widget(xml,"dateedit_valida_hasta_factura");
    GtkWidget * dateedit_fecha_pago_factura = glade_xml_get_widget(xml,"dateedit_fecha_pago_factura");
    gboolean is_hire_purchase=FALSE;
    gshort totel_expiries=1;
    gdouble deposit_percentage=100;
    time_t fechaInicio=time(NULL);
    gshort diasIntervalo=0;
    time_t fechaRealizado=time(NULL);
    gboolean esTransferencia=FALSE;
    gboolean esTalon=FALSE;
    gchar * numeroTalon=NULL;
    gboolean esTarjeta=FALSE;
    gboolean esDomiciliacion=FALSE;
    gint bank_code=-1;
    gint office_code=-1;
    gchar *  numeroCuenta=NULL;
    gchar * numeroTarjeta=NULL;
    time_t validaHasta=time(NULL);
    gboolean is_pay_cash=FALSE;
    gint error = 0;
    
    gint code_combo_payment_mean = c_util_get_code_from_combo (payment_mean_combo);
    gint payment_mode_combo_code = c_util_get_code_from_combo (payment_mode_combo);
    Payment * payment = d_payment_create_empty ();
    Enterprise * enterprise;
    Shop * shop;


    if (actual_invoice!=NULL) {
	enterprise =d_enterprise_duplicate (d_invoice_get_enterprise (actual_invoice));
	shop =  d_shop_duplicate (d_invoice_get_shop (actual_invoice));
    } else {
	enterprise =d_enterprise_duplicate (d_repair_get_enterprise (d_payment_note_get_repair_detail (payment_note)));
	shop =  d_shop_duplicate (d_repair_get_shop (d_payment_note_get_repair_detail (payment_note)));
    }

    d_payment_set_enterprise (payment,enterprise);
    d_payment_set_shop (payment,shop);
    if (payment_mode_combo_code == D_PAYMENT_AT_MOMENT) {
	is_hire_purchase=FALSE;
	totel_expiries=1;
	deposit_percentage=100;
	fechaInicio=time(NULL);
	diasIntervalo=0;
        is_pay_cash = TRUE;
	d_payment_set_achieved_payments (payment,1);
    } else if (payment_mode_combo_code == D_PAYMENT_MONEY_ORDER) {
	is_hire_purchase=FALSE;
	totel_expiries=1;
	deposit_percentage=100;
	diasIntervalo=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton_intervalo));
	fechaInicio= gnome_date_edit_get_date(GNOME_DATE_EDIT(dateedit_primerPago));
	is_pay_cash = FALSE;
	fechaRealizado = gnome_date_edit_get_date(GNOME_DATE_EDIT(dateedit_fecha_pago_factura));
	d_payment_set_achieved_payments (payment,0);
    } else if (payment_mode_combo_code == D_PAYMENT_MONEY_ORDER_AT_END_OF_MONTH) {
	time_t * fecha = (time_t *) g_malloc0(sizeof(time_t));
	is_hire_purchase=FALSE;
	totel_expiries=1;
	deposit_percentage=100;
	fechaInicio=gnome_date_edit_get_date(GNOME_DATE_EDIT(dateedit_primerPago));
	*fecha = fechaInicio;
	diasIntervalo= c_util_get_days_to_last_day_of_month (fecha);
        g_free(fecha);
        is_pay_cash = FALSE;
	fechaRealizado = gnome_date_edit_get_date(GNOME_DATE_EDIT(dateedit_fecha_pago_factura));
	d_payment_set_achieved_payments (payment,0);

    } else if (payment_mode_combo_code == D_PAYMENT_HIRE_PURCHASE) {
	is_hire_purchase=TRUE;
	diasIntervalo=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton_vencimientos));
	deposit_percentage=gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(spinbutton_entrada));;
	fechaInicio=gnome_date_edit_get_date(GNOME_DATE_EDIT(dateedit_primerPago));
        totel_expiries=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton_intervalo));
        is_pay_cash = FALSE;
	fechaRealizado = gnome_date_edit_get_date(GNOME_DATE_EDIT(dateedit_fecha_pago_factura));
        d_payment_set_achieved_payments (payment,0);
    } else {
	g_print(_("Not yet implemented this mode of payment"));
    }

    if (code_combo_payment_mean == D_PAYMENT_TRANSFER_CODE) {
	esTransferencia=TRUE;
    } else if (code_combo_payment_mean == D_PAYMENT_IN_CASH) {
        /*THAT OPTION DON'T NEED ANY VARIABLE TO TRUE */
    } else if (code_combo_payment_mean == D_PAYMENT_CREDIT_CARD) {
	gint value1 = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton_primer_numero));
	gint value2 = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton_segundo_numero));
	gint value3 = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton_tercer_numero));
	gint value4 = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton_cuarto_numero));
	validaHasta = gnome_date_edit_get_date(GNOME_DATE_EDIT(valida_hasta));

	numeroTarjeta = c_util_get_card_number (value1,value2,value3,value4);
	esTarjeta=TRUE;
    } else if (code_combo_payment_mean == D_PAYMENT_CHEQUE) {
	gint value1 = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton_primer_numero));
	gint value2 = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton_segundo_numero));
	gint value3 = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton_tercer_numero));
	gint value4 = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinbutton_cuarto_numero));
	esTalon=TRUE;
	numeroTalon = c_util_get_cheque_number (value1,value2,value3,value4);
    } else if (code_combo_payment_mean == D_PAYMENT_DIRECT_DEBIT_CODE) {
        GtkWidget * entry_combo_cuenta = glade_xml_get_widget(xml,"entry_combo_cuenta_factura");
        gchar * bank_gchar;
        gchar * office_gchar;
        gchar * account_gchar;
        gchar * complete_account;
	gint bank;
	gint office;

        complete_account = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry_combo_cuenta)));
	if (!strcmp(complete_account,"")) {
            error=1;
	}
	if (error!=1) {
	    bank_gchar = u_extract_element (complete_account,0," ",4);
	    office_gchar = u_extract_element (complete_account,1," ",4);
	    account_gchar = u_extract_element (complete_account,3," ",4);
	    bank =atoi(bank_gchar);
	    office = atoi(office_gchar);
	    bank_code = bank;
	    office_code = office;
	    numeroCuenta = g_strdup(account_gchar);
	    g_free(bank_gchar);
	    g_free(account_gchar);
	    g_free(office_gchar);
	    g_free(complete_account);
	}
	esDomiciliacion = TRUE;

    }
    if (error==0) {
	d_payment_set_is_hire_purchase (payment,is_hire_purchase);
	d_payment_set_number_expiries (payment,totel_expiries);
	d_payment_set_entry_percentage (payment,deposit_percentage);
	d_payment_set_init_date (payment,fechaInicio);
	d_payment_set_interval_days (payment,diasIntervalo);
	d_payment_set_is_paid_cash (payment,is_pay_cash);
	d_payment_set_achieved_date ( payment,fechaRealizado);
	d_payment_set_is_hire_purchase (payment,is_hire_purchase);
	d_payment_set_is_transfer ( payment,esTransferencia);
	d_payment_set_is_cheque ( payment,esTalon);
	d_payment_set_cheque_number ( payment,numeroTalon);
	d_payment_set_is_credit_card ( payment,esTarjeta);
	d_payment_set_is_direct_debit ( payment,esDomiciliacion);
	d_payment_set_credit_card_number ( payment,numeroTarjeta);
	d_payment_set_expiry_date ( payment,validaHasta);
	d_payment_set_is_paid_cash ( payment,is_pay_cash);
	d_payment_set_bank_code ( payment,bank_code);
	d_payment_set_office_code ( payment,office_code);
	d_payment_set_account_number ( payment,numeroCuenta);
        if (numeroTalon!=NULL)  g_free(numeroTalon);
        if (numeroCuenta!=NULL)  g_free(numeroCuenta);
        if (numeroTarjeta!=NULL)  g_free(numeroTarjeta);
    } else {
 
	d_payment_clear (payment);
        payment=NULL;
    }

return payment;

}
static void delete_factura(gint reply,
			   gpointer user_data) {
    if (!reply) {
	if (actual_invoice!=NULL) {
	    gchar * message = m_invoice_delete (actual_invoice);
	    if (message == NULL) {
		d_invoice_clear (actual_invoice);
		actual_invoice=NULL;
		payment_note=NULL;
		clean_screen();
		c_util_open_info_window (_("Deleted invoice"));

	    } else {
		c_util_open_error_window (message);
	    }
            if (message!=NULL)
		g_free(message);
	} else {
	    gchar * message = m_payment_note_delete (payment_note);
	    if (message == NULL) {
		d_payment_note_clear (payment_note);
		payment_note=NULL;
		actual_invoice=NULL;
		clean_screen();
		c_util_open_info_window (_("Deleted payment note"));

	    } else {
		c_util_open_error_window (message);
	    }
            if (message!=NULL)
		g_free(message);
	}
    } else {

	c_util_open_info_window (_("Cancelled operation"));
    }
}
static void on_cancel_button_clicked(GtkButton *button, gpointer data) {
    if (actual_invoice!=NULL) {
	gulong codigoFactura = d_invoice_get_invoice_code (actual_invoice);
	if (codigoFactura!=ULONG_MAX) {
	    gchar * invoice_number_char=g_strdup_printf("%lu",d_invoice_get_invoice_number (actual_invoice));
	    gchar * my_message = g_strconcat(_("Are you sure you want to remove the INVOICE: "),invoice_number_char, "?",
					     _("\nYou have to consider\n:       * You just can remove unpaid invoices.\n       * If you remove invoices that come from split invoice processes, you will remove both.\n      * When you remove an invoice you must correct the next invoice number,\n in the <accountability/invoice number modification> window you could find the appropiate number"),NULL);
            g_free(invoice_number_char);
	    c_util_open_question_dialog(my_message,
				 (*delete_factura));
	    g_free(my_message);
	} else {
	    c_util_open_error_window (_("You can't remove a nonexistent invoice"));
	}
    } else if (payment_note!=NULL) {
        gulong codigoNota=d_payment_note_get_note_code (payment_note);
	if (codigoNota!=ULONG_MAX) {
            gchar * codigoNota_char=g_strdup_printf("%lu",codigoNota);
	    gchar * my_message = g_strconcat(_("Are you sure that you want to remove the PAYMENT NOTE: "),codigoNota_char, "?\n",
					     _("When you remove a payment note the linked cash movement will be removed too\n and the data couldn't be restored"),NULL);
            g_free(codigoNota_char);
	    c_util_open_question_dialog(my_message,
				 (*delete_factura));
            g_free(my_message);
	} else {
	    c_util_open_error_window (_("Nonexisting payment note can't be removed"));
	}

    } else {
        c_util_open_info_window (_("Empty invoice/payment note"));
    }

}

static void on_spliting_invoice_checkbutton_toggled(GtkToggleButton *togglebutton,
						 gpointer user_data) {
    GtkWidget * wid;
    Shop ** shops;

    wid = glade_xml_get_widget(xml,"checkbutton_refacturacion");
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(wid))==TRUE) {
	wid = glade_xml_get_widget(xml,"label_datos_empresa_refacturacion");
	if (d_invoice_get_invoice_number (actual_invoice)==ULONG_MAX) {
	    if (!GTK_WIDGET_VISIBLE(wid)) gtk_widget_show(wid);
	}
	else {
	    if (GTK_WIDGET_VISIBLE(wid)) gtk_widget_hide(wid);
	}

	wid = glade_xml_get_widget(xml,"label_refacturacion_refacturacion");
	if (d_invoice_get_invoice_number (actual_invoice)==ULONG_MAX) {
	    if (!GTK_WIDGET_VISIBLE(wid)) gtk_widget_show(wid);
	}
	else {
	    if (GTK_WIDGET_VISIBLE(wid)) gtk_widget_hide(wid);
	}

	wid = glade_xml_get_widget(xml,"label_tienda_refacturacion");
	if (d_invoice_get_invoice_number (actual_invoice)==ULONG_MAX) {
	    if (!GTK_WIDGET_VISIBLE(wid)) gtk_widget_show(wid);
	}
	else {
	    if (GTK_WIDGET_VISIBLE(wid)) gtk_widget_hide(wid);
	}

	wid = glade_xml_get_widget(xml,"combo_tienda_refacturacion");
	if (d_invoice_get_invoice_number (actual_invoice)==ULONG_MAX) {
	    if (!GTK_WIDGET_VISIBLE(wid)) gtk_widget_show(wid);
	}
	else {
	    if (GTK_WIDGET_VISIBLE(wid)) gtk_widget_hide(wid);
	}
	shops= m_shop_find_all ();
	c_util_fill_combo_with_shops (wid,shops);

    } else {
        GtkWidget * entry;
	wid = glade_xml_get_widget(xml,"label_datos_empresa_refacturacion");
        if (GTK_WIDGET_VISIBLE(wid)) gtk_widget_hide(wid);
	wid = glade_xml_get_widget(xml,"label_refacturacion_refacturacion");
        if (GTK_WIDGET_VISIBLE(wid)) gtk_widget_hide(wid);
	wid = glade_xml_get_widget(xml,"label_tienda_refacturacion");
        if (GTK_WIDGET_VISIBLE(wid)) gtk_widget_hide(wid);
	wid = glade_xml_get_widget(xml,"combo_tienda_refacturacion");
        if (GTK_WIDGET_VISIBLE(wid)) gtk_widget_hide(wid);
	entry = glade_xml_get_widget (xml, "combo_entry_empresa_refacturacion");
	gtk_entry_set_text(GTK_ENTRY(entry),"");
	c_util_clean_code_and_name_combo (wid);
    }
}

static void on_type_invoice_combo_entry_changed(GtkEntry *entry,
						gpointer user_data) {
    GtkWidget * combo = glade_xml_get_widget(xml,"combo_tipo_factura");
    GtkWidget * label = glade_xml_get_widget(xml,"label_numero_factura");
    GtkWidget * entry_vat= glade_xml_get_widget(xml,"entry_iva_factura");
    GtkWidget * entry_discount = glade_xml_get_widget(xml,"entry_descuento_factura");
    GtkWidget * label_vat= glade_xml_get_widget(xml,"label_iva_factura");
    GtkWidget * label_discount = glade_xml_get_widget(xml,"label_descuento_factura");
    GtkWidget * combo_template = glade_xml_get_widget(xml,"combo_seguros");
    GtkWidget * button_albaran = glade_xml_get_widget(xml,"button_albaran_factura");
    GtkWidget * button_pagar = glade_xml_get_widget(xml,"button_pagar_factura");
    GtkWidget * label_subtotal = glade_xml_get_widget(xml,"label_base_imponible_factura");
    GtkWidget * label_total = glade_xml_get_widget(xml,"label_total_factura");
    GtkWidget * entry_total = glade_xml_get_widget(xml,"entry_total_factura");
    GtkWidget * clist = glade_xml_get_widget(xml,"lista_detalle_factura");
    gint code_combo = c_util_get_code_from_combo (combo);
    GtkWidget * wid;
    if (code_combo==D_INVOICE_CODE) {
	wid = glade_xml_get_widget(xml,"frame_estado_factura_factura");
	if (!GTK_WIDGET_VISIBLE(wid)) gtk_widget_show(wid);
	gtk_label_set_text(GTK_LABEL(label),_("Invoice number"));
        if (!GTK_WIDGET_VISIBLE(entry_vat)) gtk_widget_show(entry_vat);
	if (!GTK_WIDGET_VISIBLE(entry_discount)) gtk_widget_show(entry_discount);
        if (!GTK_WIDGET_VISIBLE(label_vat)) gtk_widget_show(label_vat);
	if (!GTK_WIDGET_VISIBLE(label_discount)) gtk_widget_show(label_discount);
	if (!GTK_WIDGET_VISIBLE(button_albaran)) gtk_widget_show(button_albaran);
	if (!GTK_WIDGET_VISIBLE(button_pagar)) gtk_widget_show(button_pagar);
	if (!GTK_WIDGET_VISIBLE(label_subtotal)) gtk_widget_show(label_subtotal);
	if (!GTK_WIDGET_VISIBLE(label_total)) gtk_widget_show(label_total);
	if (!GTK_WIDGET_VISIBLE(entry_total)) gtk_widget_show(entry_total);
	if (!GTK_WIDGET_SENSITIVE(combo_template)) gtk_widget_set_sensitive(combo_template,TRUE);
	gtk_label_set_text(GTK_LABEL(label_subtotal),_("Taxable income"));
	inicialize_payment_screen(TRUE,FALSE);
	gtk_clist_set_column_visibility ((GtkCList *) clist, C_VAT_COLUMN,TRUE);
#ifdef IS_SHOP
        put_buttons_sensible(0);
#endif
    } else {
	wid = glade_xml_get_widget(xml,"frame_estado_factura_factura");
	if (GTK_WIDGET_VISIBLE(wid)) gtk_widget_hide(wid);
	gtk_label_set_text(GTK_LABEL(label),_("Payment note number"));
        if (GTK_WIDGET_VISIBLE(entry_vat)) gtk_widget_hide(entry_vat);
	if (GTK_WIDGET_VISIBLE(entry_discount)) gtk_widget_hide(entry_discount);
        if (GTK_WIDGET_VISIBLE(label_vat)) gtk_widget_hide(label_vat);
	if (GTK_WIDGET_VISIBLE(label_discount)) gtk_widget_hide(label_discount);
	if (GTK_WIDGET_VISIBLE(button_albaran)) gtk_widget_hide(button_albaran);
	if (GTK_WIDGET_VISIBLE(button_pagar)) gtk_widget_hide(button_pagar);
	if (GTK_WIDGET_VISIBLE(label_total)) gtk_widget_hide(label_total);
	if (GTK_WIDGET_VISIBLE(entry_total)) gtk_widget_hide(entry_total);
	if (GTK_WIDGET_SENSITIVE(combo_template)) gtk_widget_set_sensitive(combo_template,FALSE);
	gtk_label_set_text(GTK_LABEL(label_subtotal),_("Total"));
	inicialize_payment_screen(FALSE,TRUE);
	gtk_clist_set_column_visibility ((GtkCList *) clist, C_VAT_COLUMN,FALSE);
#ifdef IS_SHOP
        put_buttons_sensible(2);
#endif
    }

}

static void inicialize_payment_screen(gboolean sensitive,gboolean note) {
    GtkWidget * payment_mode_combo = glade_xml_get_widget(xml,"combo_modo_pago");
    GtkWidget * payment_mean_combo = glade_xml_get_widget(xml,"combo_medio_pago");
    GtkWidget * payment_mean_entry = glade_xml_get_widget(xml,"combo_entry_medio_factura");
    GtkWidget * entry_modo_pago = glade_xml_get_widget(xml,"combo_entry_modo_pago");
    GtkWidget * dateedit_fecha_pago_factura = glade_xml_get_widget(xml,"dateedit_fecha_pago_factura");
    GtkWidget * spinbutton_primer_numero = glade_xml_get_widget(xml,"spinbutton_primer_numero");
    GtkWidget * spinbutton_segundo_numero = glade_xml_get_widget(xml,"spinbutton_segundo_numero");
    GtkWidget * spinbutton_tercer_numero = glade_xml_get_widget(xml,"spinbutton_tercer_numero");
    GtkWidget * spinbutton_cuarto_numero = glade_xml_get_widget(xml,"spinbutton_cuarto_numero");
    GtkWidget * spinbutton_dias_giro = glade_xml_get_widget(xml,"dias_giro");
    GtkWidget * entry_numero_cuenta = glade_xml_get_widget(xml,"entry_numero_cuenta");
    GtkWidget * dateedit_valida_hasta_factura= glade_xml_get_widget(xml,"dateedit_valida_hasta_factura");
    GtkWidget * spinbutton_entrada=glade_xml_get_widget(xml,"spinbutton_entrada");
    GtkWidget * dateedit_primer_pago=glade_xml_get_widget(xml,"dateedit_primer_pago");
    GtkWidget * spinbutton_vencimientos=glade_xml_get_widget(xml,"spinbutton_intervalo");

    if (sensitive==FALSE) {
	if (GTK_WIDGET_SENSITIVE(payment_mode_combo)) gtk_widget_set_sensitive(payment_mode_combo,FALSE);
	if (GTK_WIDGET_SENSITIVE(payment_mean_combo)) gtk_widget_set_sensitive(payment_mean_combo,FALSE);
	if (GTK_WIDGET_SENSITIVE(dateedit_fecha_pago_factura)) gtk_widget_set_sensitive(dateedit_fecha_pago_factura,FALSE);
	if (GTK_WIDGET_SENSITIVE(spinbutton_primer_numero)) gtk_widget_set_sensitive(spinbutton_primer_numero,FALSE);
	if (GTK_WIDGET_SENSITIVE(spinbutton_segundo_numero)) gtk_widget_set_sensitive(spinbutton_segundo_numero,FALSE);
	if (GTK_WIDGET_SENSITIVE(spinbutton_tercer_numero)) gtk_widget_set_sensitive(spinbutton_tercer_numero,FALSE);
	if (GTK_WIDGET_SENSITIVE(spinbutton_cuarto_numero)) gtk_widget_set_sensitive(spinbutton_cuarto_numero,FALSE);
	if (GTK_WIDGET_SENSITIVE(entry_numero_cuenta)) gtk_widget_set_sensitive(entry_numero_cuenta,FALSE);
	if (GTK_WIDGET_SENSITIVE(dateedit_valida_hasta_factura)) gtk_widget_set_sensitive(dateedit_valida_hasta_factura,FALSE);
	if (GTK_WIDGET_SENSITIVE(spinbutton_vencimientos)) gtk_widget_set_sensitive(spinbutton_vencimientos,FALSE);
	if (GTK_WIDGET_SENSITIVE(spinbutton_entrada)) gtk_widget_set_sensitive(spinbutton_entrada,FALSE);
	if (GTK_WIDGET_SENSITIVE(dateedit_primer_pago)) gtk_widget_set_sensitive(dateedit_primer_pago,FALSE);
	if (GTK_WIDGET_SENSITIVE(spinbutton_dias_giro)) gtk_widget_set_sensitive(spinbutton_dias_giro,FALSE);
	if (note!=FALSE) {
	    if (strcmp(gtk_entry_get_text(GTK_ENTRY(payment_mean_entry)),D_PAYMENT_MESSAGE_IN_CASH)) gtk_entry_set_text(GTK_ENTRY(payment_mean_entry),D_PAYMENT_MESSAGE_IN_CASH);
	    if (strcmp(gtk_entry_get_text(GTK_ENTRY(entry_modo_pago)),D_PAYMENT_MESSAGE_AT_MOMENT)) gtk_entry_set_text(GTK_ENTRY(payment_mean_entry),D_PAYMENT_MESSAGE_AT_MOMENT);
	}
    } else {
	if (!GTK_WIDGET_SENSITIVE(payment_mode_combo)) gtk_widget_set_sensitive(payment_mode_combo,TRUE);
	if (!GTK_WIDGET_SENSITIVE(payment_mean_combo)) gtk_widget_set_sensitive(payment_mean_combo,TRUE);
	if (!GTK_WIDGET_SENSITIVE(dateedit_fecha_pago_factura)) gtk_widget_set_sensitive(dateedit_fecha_pago_factura,TRUE);
	if (!GTK_WIDGET_SENSITIVE(spinbutton_primer_numero)) gtk_widget_set_sensitive(spinbutton_primer_numero,TRUE);
	if (!GTK_WIDGET_SENSITIVE(spinbutton_segundo_numero)) gtk_widget_set_sensitive(spinbutton_segundo_numero,TRUE);
	if (!GTK_WIDGET_SENSITIVE(spinbutton_tercer_numero)) gtk_widget_set_sensitive(spinbutton_tercer_numero,TRUE);
	if (!GTK_WIDGET_SENSITIVE(spinbutton_cuarto_numero)) gtk_widget_set_sensitive(spinbutton_cuarto_numero,TRUE);
	if (!GTK_WIDGET_SENSITIVE(entry_numero_cuenta)) gtk_widget_set_sensitive(entry_numero_cuenta,TRUE);
	if (!GTK_WIDGET_SENSITIVE(dateedit_valida_hasta_factura)) gtk_widget_set_sensitive(dateedit_valida_hasta_factura,TRUE);
	if (!GTK_WIDGET_SENSITIVE(spinbutton_dias_giro)) gtk_widget_set_sensitive(spinbutton_dias_giro,TRUE);
        if (strcmp(gtk_entry_get_text(GTK_ENTRY(payment_mean_entry)),D_PAYMENT_MESSAGE_IN_CASH)) gtk_entry_set_text(GTK_ENTRY(payment_mean_entry),D_PAYMENT_MESSAGE_IN_CASH);
        if (strcmp(gtk_entry_get_text(GTK_ENTRY(entry_modo_pago)),D_PAYMENT_MESSAGE_AT_MOMENT)) gtk_entry_set_text(GTK_ENTRY(payment_mean_entry),D_PAYMENT_MESSAGE_AT_MOMENT);
    }
}

static void fill_payment_initial_data(gboolean with_insurance) {
    GtkWidget * payment_mean_entry = glade_xml_get_widget(xml,"combo_entry_medio_factura");
    GtkWidget * entry_modo_pago = glade_xml_get_widget(xml,"combo_entry_modo_pago");
    GtkWidget * spinbutton_dias_giro = glade_xml_get_widget(xml,"dias_giro");
    gchar * medio_temp = g_strdup(gtk_entry_get_text(GTK_ENTRY(payment_mean_entry)));
    gchar * modo_temp = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry_modo_pago)));
#ifndef IS_SHOP
    if (with_insurance!=FALSE) {
	if (strcmp(medio_temp,D_PAYMENT_MESSAGE_TRANSFER)) gtk_entry_set_text(GTK_ENTRY(payment_mean_entry),D_PAYMENT_MESSAGE_TRANSFER);
	if (strcmp(modo_temp,D_PAYMENT_MESSAGE_MONEY_ORDER)) gtk_entry_set_text(GTK_ENTRY(entry_modo_pago),D_PAYMENT_MESSAGE_MONEY_ORDER);
	gtk_entry_set_text(GTK_ENTRY(spinbutton_dias_giro),"60");
        gtk_spin_button_update(GTK_SPIN_BUTTON(spinbutton_dias_giro));
    } else {
#endif
	if (strcmp(medio_temp,D_PAYMENT_MESSAGE_IN_CASH)) gtk_entry_set_text(GTK_ENTRY(payment_mean_entry),D_PAYMENT_MESSAGE_IN_CASH);
	if (strcmp(modo_temp,D_PAYMENT_MESSAGE_AT_MOMENT)) gtk_entry_set_text(GTK_ENTRY(entry_modo_pago),D_PAYMENT_MESSAGE_AT_MOMENT);

#ifndef IS_SHOP
    }
#endif
    g_free(medio_temp);
    g_free(modo_temp);
}

static void fill_payment_data(Payment * payment,gboolean recarga){
    GtkWidget * payment_mode_combo = glade_xml_get_widget(xml,"combo_modo_pago");
    GtkWidget * payment_mean_combo = glade_xml_get_widget(xml,"combo_medio_pago");
    GtkWidget * payment_mean_entry = glade_xml_get_widget(xml,"combo_entry_medio_factura");
    GtkWidget * entry_modo_pago = glade_xml_get_widget(xml,"combo_entry_modo_pago");
    GtkWidget * dateedit_fecha_pago_factura = glade_xml_get_widget(xml,"dateedit_fecha_pago_factura");
    GtkWidget * spinbutton_dias_giro = glade_xml_get_widget(xml,"dias_giro");
    GtkWidget * spinbutton_primer_numero = glade_xml_get_widget(xml,"spinbutton_primer_numero");
    GtkWidget * spinbutton_segundo_numero = glade_xml_get_widget(xml,"spinbutton_segundo_numero");
    GtkWidget * spinbutton_tercer_numero = glade_xml_get_widget(xml,"spinbutton_tercer_numero");
    GtkWidget * spinbutton_cuarto_numero = glade_xml_get_widget(xml,"spinbutton_cuarto_numero");
    GtkWidget * entry_numero_cuenta = glade_xml_get_widget(xml,"entry_numero_cuenta");
    GtkWidget * dateedit_valida_hasta_factura= glade_xml_get_widget(xml,"dateedit_valida_hasta_factura");
    GtkWidget * spinbutton_entrada=glade_xml_get_widget(xml,"spinbutton_entrada");
    GtkWidget * dateedit_primer_pago=glade_xml_get_widget(xml,"dateedit_primer_pago");
    GtkWidget * spinbutton_vencimientos=glade_xml_get_widget(xml,"spinbutton_intervalo");
    gchar * medio_temp = g_strdup(gtk_entry_get_text(GTK_ENTRY(payment_mean_entry)));
    gchar * modo_temp = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry_modo_pago)));


    if (payment!=NULL) {
	time_t * time_aux = (time_t *) g_malloc0(sizeof(time_t));
        time_t * fecha_realizado = (time_t *) g_malloc0(sizeof(time_t));
	gchar * dias_giro = g_strdup_printf("%d",d_payment_get_interval_days (payment));
	*fecha_realizado = d_payment_get_achieved_date (payment);
        if (actual_invoice!=NULL)
	    *time_aux = d_invoice_get_issue_date (actual_invoice);
	else
	    *time_aux = d_payment_note_get_emission_date (payment_note);

        /* PAYMENT MODE*/
	if (d_payment_get_interval_days (payment)==0) {
	    gchar * vencimientos = g_strdup_printf("%d",d_payment_get_number_expiries (payment));
            gchar * porcentaje = g_strdup_printf("%.2f",d_payment_get_entry_percentage (payment));
	    if (strcmp(modo_temp,D_PAYMENT_MESSAGE_AT_MOMENT)) gtk_entry_set_text(GTK_ENTRY(entry_modo_pago),D_PAYMENT_MESSAGE_AT_MOMENT);
	    gtk_entry_set_text(GTK_ENTRY(spinbutton_entrada),porcentaje);
	    gtk_spin_button_update(GTK_SPIN_BUTTON(spinbutton_entrada));
	    gtk_entry_set_text(GTK_ENTRY(spinbutton_vencimientos),vencimientos);
	    gtk_spin_button_update(GTK_SPIN_BUTTON(spinbutton_vencimientos));
	    g_free(vencimientos);
            g_free(porcentaje);

	} else if (d_payment_get_is_hire_purchase (payment)!=FALSE) {
	    gchar * vencimientos = NULL;// g_strdup_printf("%d",1.0*d_payment_get_number_expiries (payment));
            gchar * porcentaje = g_strdup_printf("%.2f",d_payment_get_entry_percentage (payment));
	    if (strcmp(modo_temp,D_PAYMENT_MESSAGE_HIRE_PURCHASE)) gtk_entry_set_text(GTK_ENTRY(entry_modo_pago),D_PAYMENT_MESSAGE_HIRE_PURCHASE);

	    vencimientos = dias_giro;
	    dias_giro = g_strdup_printf("%d",d_payment_get_number_expiries (payment));

	    gtk_entry_set_text(GTK_ENTRY(spinbutton_entrada),porcentaje);
	    gtk_spin_button_update(GTK_SPIN_BUTTON(spinbutton_entrada));

	    gtk_entry_set_text(GTK_ENTRY(spinbutton_vencimientos),vencimientos);
	    gtk_spin_button_update(GTK_SPIN_BUTTON(spinbutton_vencimientos));

	    g_free(vencimientos);
            g_free(porcentaje);

	} else if ((c_util_get_days_to_last_day_of_month (time_aux))==d_payment_get_interval_days (payment)) {
	    gchar * vencimientos = g_strdup_printf("%d",d_payment_get_number_expiries (payment));
	    gchar * porcentaje = g_strdup_printf("%.2f",d_payment_get_entry_percentage (payment));
	    if (strcmp(modo_temp,D_PAYMENT_MESSAGE_MONEY_ORDER_AT_END_OF_MONTH)) gtk_entry_set_text(GTK_ENTRY(entry_modo_pago),D_PAYMENT_MESSAGE_MONEY_ORDER_AT_END_OF_MONTH);
	    gtk_entry_set_text(GTK_ENTRY(spinbutton_entrada),porcentaje);
	    gtk_spin_button_update(GTK_SPIN_BUTTON(spinbutton_entrada));
	    gtk_entry_set_text(GTK_ENTRY(spinbutton_vencimientos),vencimientos);
	    gtk_spin_button_update(GTK_SPIN_BUTTON(spinbutton_vencimientos));

	    g_free(vencimientos);
	    g_free(porcentaje);

	} else {
	    gchar * vencimientos = g_strdup_printf("%d",d_payment_get_number_expiries (payment));
            gchar * porcentaje = g_strdup_printf("%.2f",d_payment_get_entry_percentage (payment));
	    if (strcmp(modo_temp,D_PAYMENT_MESSAGE_MONEY_ORDER)) gtk_entry_set_text(GTK_ENTRY(entry_modo_pago),D_PAYMENT_MESSAGE_MONEY_ORDER);
	    gtk_entry_set_text(GTK_ENTRY(spinbutton_entrada),porcentaje);
	    gtk_spin_button_update(GTK_SPIN_BUTTON(spinbutton_entrada));

	    gtk_entry_set_text(GTK_ENTRY(spinbutton_vencimientos),vencimientos);
	    gtk_spin_button_update(GTK_SPIN_BUTTON(spinbutton_vencimientos));

	    g_free(vencimientos);
            g_free(porcentaje);
	}

        /* PAYMENT WAY*/
	if (d_payment_get_is_transfer (payment)) {
	    if (strcmp(medio_temp,D_PAYMENT_MESSAGE_TRANSFER)) gtk_entry_set_text(GTK_ENTRY(payment_mean_entry),D_PAYMENT_MESSAGE_TRANSFER);

	} else if (d_payment_get_is_cheque (payment)) {
            gchar * number = d_payment_c_util_get_cheque_number (payment);
	    gchar ** numbers = c_util_get_card_numbers (number);
	    if (strcmp(medio_temp,D_PAYMENT_MESSAGE_CHEQUE)) gtk_entry_set_text(GTK_ENTRY(payment_mean_entry),D_PAYMENT_MESSAGE_CHEQUE);
	    gtk_entry_set_text(GTK_ENTRY(spinbutton_primer_numero),numbers[0]);
	    gtk_spin_button_update(GTK_SPIN_BUTTON(spinbutton_primer_numero));
            gtk_entry_set_text(GTK_ENTRY(spinbutton_segundo_numero),numbers[1]);
	    gtk_spin_button_update(GTK_SPIN_BUTTON(spinbutton_segundo_numero));
            gtk_entry_set_text(GTK_ENTRY(spinbutton_tercer_numero),numbers[2]);
	    gtk_spin_button_update(GTK_SPIN_BUTTON(spinbutton_tercer_numero));
	    gtk_entry_set_text(GTK_ENTRY(spinbutton_cuarto_numero),numbers[3]);
	    gtk_spin_button_update(GTK_SPIN_BUTTON(spinbutton_cuarto_numero));
	    g_free(number);
	    g_strfreev(numbers);


	} else if (d_payment_get_is_direct_debit (payment)) {
	    if (strcmp(medio_temp,D_PAYMENT_MESSAGE_DIRECT_DEBIT)) gtk_entry_set_text(GTK_ENTRY(payment_mean_entry),D_PAYMENT_MESSAGE_DIRECT_DEBIT);

	} else if (d_payment_get_is_credit_card (payment)) {
            gchar * number = d_payment_get_credit_card_number (payment);
	    gchar ** numbers = c_util_get_card_numbers (number);
	    time_t * valida_hasta = (time_t *) g_malloc0(sizeof(time_t));
            *valida_hasta = d_payment_get_expiry_date (payment);
	    if (strcmp(medio_temp,D_PAYMENT_MESSAGE_CREDIT_CARD)) gtk_entry_set_text(GTK_ENTRY(payment_mean_entry),D_PAYMENT_MESSAGE_CREDIT_CARD);
            gtk_entry_set_text(GTK_ENTRY(spinbutton_primer_numero),numbers[0]);
	    gtk_spin_button_update(GTK_SPIN_BUTTON(spinbutton_primer_numero));
	    gtk_entry_set_text(GTK_ENTRY(spinbutton_segundo_numero),numbers[1]);
	    gtk_spin_button_update(GTK_SPIN_BUTTON(spinbutton_segundo_numero));
	    gtk_entry_set_text(GTK_ENTRY(spinbutton_tercer_numero),numbers[2]);
	    gtk_spin_button_update(GTK_SPIN_BUTTON(spinbutton_tercer_numero));
	    gtk_entry_set_text(GTK_ENTRY(spinbutton_cuarto_numero),numbers[3]);
	    gtk_spin_button_update(GTK_SPIN_BUTTON(spinbutton_cuarto_numero));
 
	     gnome_date_edit_set_time(GNOME_DATE_EDIT(dateedit_valida_hasta_factura),*valida_hasta);
	    g_free(number);
	    g_strfreev(numbers);
            g_free(valida_hasta);
	} else {
            /* ES EFECTIVO*/
	    if (strcmp(medio_temp,D_PAYMENT_MESSAGE_IN_CASH)) gtk_entry_set_text(GTK_ENTRY(payment_mean_entry),D_PAYMENT_MESSAGE_IN_CASH);

	}
        /* HE / SHE CAN'T MODIFY DATA WHE THE PAYMENT WAS DONE */
	if ((d_payment_get_achieved_payments (payment))==(d_payment_get_number_expiries (payment))) {
	    if (GTK_WIDGET_SENSITIVE(payment_mode_combo)) gtk_widget_set_sensitive(payment_mode_combo,FALSE);
	    if (GTK_WIDGET_SENSITIVE(payment_mean_combo)) gtk_widget_set_sensitive(payment_mean_combo,FALSE);
	    if (GTK_WIDGET_SENSITIVE(dateedit_fecha_pago_factura)) gtk_widget_set_sensitive(dateedit_fecha_pago_factura,FALSE);
	    if (GTK_WIDGET_SENSITIVE(spinbutton_dias_giro)) gtk_widget_set_sensitive(spinbutton_dias_giro,FALSE);
	    if (GTK_WIDGET_SENSITIVE(spinbutton_primer_numero)) gtk_widget_set_sensitive(spinbutton_primer_numero,FALSE);
	    if (GTK_WIDGET_SENSITIVE(spinbutton_segundo_numero)) gtk_widget_set_sensitive(spinbutton_segundo_numero,FALSE);
	    if (GTK_WIDGET_SENSITIVE(spinbutton_tercer_numero)) gtk_widget_set_sensitive(spinbutton_tercer_numero,FALSE);
	    if (GTK_WIDGET_SENSITIVE(spinbutton_cuarto_numero)) gtk_widget_set_sensitive(spinbutton_cuarto_numero,FALSE);
	    if (GTK_WIDGET_SENSITIVE(entry_numero_cuenta)) gtk_widget_set_sensitive(entry_numero_cuenta,FALSE);
	    if (GTK_WIDGET_SENSITIVE(dateedit_valida_hasta_factura)) gtk_widget_set_sensitive(dateedit_valida_hasta_factura,FALSE);
	    if (GTK_WIDGET_SENSITIVE(spinbutton_vencimientos)) gtk_widget_set_sensitive(spinbutton_vencimientos,FALSE);
	    if (GTK_WIDGET_SENSITIVE(spinbutton_entrada)) gtk_widget_set_sensitive(spinbutton_entrada,FALSE);
	    if (GTK_WIDGET_SENSITIVE(dateedit_primer_pago)) gtk_widget_set_sensitive(dateedit_primer_pago,FALSE);

	} else {
	    if (!GTK_WIDGET_SENSITIVE(payment_mode_combo)) gtk_widget_set_sensitive(payment_mode_combo,TRUE);
	    if (!GTK_WIDGET_SENSITIVE(payment_mean_combo)) gtk_widget_set_sensitive(payment_mean_combo,TRUE);
	    if (!GTK_WIDGET_SENSITIVE(dateedit_fecha_pago_factura)) gtk_widget_set_sensitive(dateedit_fecha_pago_factura,TRUE);
	    if (!GTK_WIDGET_SENSITIVE(spinbutton_dias_giro)) gtk_widget_set_sensitive(spinbutton_dias_giro,TRUE);
	    if (!GTK_WIDGET_SENSITIVE(spinbutton_primer_numero)) gtk_widget_set_sensitive(spinbutton_primer_numero,TRUE);
	    if (!GTK_WIDGET_SENSITIVE(spinbutton_segundo_numero)) gtk_widget_set_sensitive(spinbutton_segundo_numero,TRUE);
	    if (!GTK_WIDGET_SENSITIVE(spinbutton_tercer_numero)) gtk_widget_set_sensitive(spinbutton_tercer_numero,TRUE);
	    if (!GTK_WIDGET_SENSITIVE(spinbutton_cuarto_numero)) gtk_widget_set_sensitive(spinbutton_cuarto_numero,TRUE);
	    if (!GTK_WIDGET_SENSITIVE(entry_numero_cuenta)) gtk_widget_set_sensitive(entry_numero_cuenta,TRUE);
	    if (!GTK_WIDGET_SENSITIVE(dateedit_valida_hasta_factura)) gtk_widget_set_sensitive(dateedit_valida_hasta_factura,TRUE);
	}

	if (d_payment_get_is_hire_purchase (payment)!=FALSE) {
	    if (GTK_WIDGET_SENSITIVE(spinbutton_dias_giro)) gtk_widget_set_sensitive(spinbutton_dias_giro,FALSE);
	}
	if (recarga==FALSE) {
	    gtk_entry_set_text(GTK_ENTRY(spinbutton_dias_giro),dias_giro);
	    gtk_spin_button_update(GTK_SPIN_BUTTON(spinbutton_dias_giro));
	}


	*time_aux = d_payment_get_init_date (payment);

	gnome_date_edit_set_time(GNOME_DATE_EDIT(dateedit_primer_pago),*time_aux);
	gnome_date_edit_set_time(GNOME_DATE_EDIT(dateedit_fecha_pago_factura),*fecha_realizado);

	if (recarga!=FALSE) {
	    gtk_entry_set_text(GTK_ENTRY(spinbutton_dias_giro),dias_giro);
	    gtk_spin_button_update(GTK_SPIN_BUTTON(spinbutton_dias_giro));
	}
 

	g_free(time_aux);
	g_free(fecha_realizado);
        g_free(dias_giro);
    } else {

    }
    g_free(medio_temp);
    g_free(modo_temp);
}


static void on_ok_create_delivery_payment(gint reply,
				gpointer user_data) {
    GtkWidget * entry = glade_xml_get_widget(xml,"entry_numero_factura");
    gchar * temp = gtk_entry_get_text(GTK_ENTRY(entry));
    GtkWidget * combo_template = glade_xml_get_widget(xml,"combo_seguros");
    GtkWidget * invoice_type_combo = glade_xml_get_widget(xml,"combo_tipo_factura");
    gulong code = c_util_get_code_from_combo (combo_template);
    gboolean imprime=FALSE;
    DeliveryPayment * delivery_payment_to_print=NULL;

    if (!reply) {
	if (negative_price==FALSE) {
	    if (actual_invoice!=NULL) {
		Repair * repair;
		gulong invoice_number = d_invoice_get_invoice_number (actual_invoice);
		repair = d_invoice_get_repair (actual_invoice);
		if ((repair!=NULL) && (invoice_number==ULONG_MAX) && (!strcmp(temp,""))) {
		    DeliveryPayment * delivery_payment=NULL;
		    DeliveryPayment ** delivery_payments=NULL;
		    gulong numeroAlbaran;
		    delivery_payment = d_delivery_payment_create_empty ();
		    d_repair_clear (d_delivery_payment_get_repair (delivery_payment));
		    d_delivery_payment_set_repair (delivery_payment,repair);
		    d_delivery_payment_set_garage_total (delivery_payment,get_garage_total_entry());
		    numeroAlbaran = m_delivery_payment_insert (delivery_payment);
		    if (numeroAlbaran != ULONG_MAX) {
			d_delivery_payment_set_number (delivery_payment,numeroAlbaran);
			gtk_widget_set_sensitive(invoice_type_combo,FALSE);
                        d_invoice_set_repair(actual_invoice,NULL,FALSE);
			if (delivery_payments==NULL) {
			    delivery_payments = (DeliveryPayment **) g_malloc0(2*sizeof(DeliveryPayment*));
			    delivery_payments[0] = delivery_payment;
			    delivery_payments[1] = NULL;
			    d_invoice_set_delivery_payments (actual_invoice,delivery_payments);
			}
			imprime = TRUE;
		    } else {
                        imprime = FALSE;
		    }


		    repair = NULL;
                    if (delivery_payments!=NULL)
			delivery_payment_to_print = delivery_payments[0];
		} else {
		    if (repair!=NULL) {
			c_util_open_info_window (_("Not allowed operation. Print the invoice"));
		    } else {
                        DeliveryPayment ** delivery_payments = d_invoice_get_delivery_payments(actual_invoice);
			if (delivery_payments[0]!=NULL) {
			    if (delivery_payments[1]!=NULL) {
				c_util_open_info_window (_("The invoice has many delivery payments. Print the invoice"));
			    } else {
				delivery_payment_to_print = delivery_payments[0];
				imprime = TRUE;
			    }
			} else {
			    c_util_open_info_window (_("Null delivery payment"));
			}
		    }
		}
	    } else {
		c_util_open_error_window (_("Not allowed operation"));
	    }
	} else {
	    Repair * repair;
	    DeliveryPayment ** delivery_payments = d_invoice_get_delivery_payments(actual_invoice);
	    DeliveryPayment * delivery_payment=NULL;
	    gulong numeroAlbaran;
	    gulong invoice_number = d_invoice_get_invoice_number (actual_invoice);
	    repair = d_invoice_get_repair (actual_invoice);
	    if ((repair!=NULL) && (invoice_number==ULONG_MAX) && (!strcmp(temp,""))) {
		delivery_payment = d_delivery_payment_create_empty ();
		d_repair_clear (d_delivery_payment_get_repair (delivery_payment));
		d_delivery_payment_set_repair (delivery_payment,repair);
		numeroAlbaran = m_delivery_payment_insert (delivery_payment);
		if (numeroAlbaran != ULONG_MAX) {
		    d_delivery_payment_set_number (delivery_payment,numeroAlbaran);
		    imprime = TRUE;
		} else {
                    imprime = FALSE;
		}
		delivery_payment_to_print = delivery_payment;
	    } else if ((delivery_payments!=NULL) && (invoice_number==ULONG_MAX) && (!strcmp(temp,""))) {
		delivery_payment_to_print = delivery_payments[0];
		imprime = TRUE;
	    }
	}
	if (imprime != FALSE) {
	    if (code ==D_INVOICE_NORMAL_CODE) {
		GtkWidget * app1=lookup_widget(GTK_WIDGET(entry), "app1");
		GtkWidget * open_print = c_delivery_payment_print_open(app1,print_normal_delivery_payment,(gpointer) delivery_payment_to_print);
		if (open_print!=NULL) {
		    gtk_window_set_modal(GTK_WINDOW(open_print),1);
		    gtk_window_set_position(GTK_WINDOW(open_print),GTK_WIN_POS_CENTER);
		    gtk_window_set_transient_for(GTK_WINDOW(open_print),
						 GTK_WINDOW(app1));
		    gtk_widget_show(GTK_WIDGET(open_print));
		}
	    } else {
                c_util_open_info_window(_("Not allowed operation"));
	    }
	} else {
	    c_util_open_error_window (_("It has not been posible to generate the delivery payment.\nSearch again the repair and generate the new delivery payment.\nIf the error persist contact with the head office"));
	}
    }

}
static void on_delivery_payment_button_clicked(GtkButton *button, gpointer data) {
    gchar * shop_name= u_parameter_manager_get_parameter ("tienda");
    gulong shop_code = m_shop_get_code_from_name (shop_name);
    gulong actual_shop_code = c_util_get_code_from_combo (glade_xml_get_widget(xml,"combo-tienda"));

    if (shop_code!=actual_shop_code) {
	c_util_open_question_dialog(_("Warning! If you generate the delivery payment will commit errors in the replication process.\nDo you want to continue?"),
		       (*on_ok_create_delivery_payment));

    } else {
	on_ok_create_delivery_payment(0,NULL);
    }

}

void get_payment_code(gpointer value) {
    gulong payment_code;
    if (d_invoice_get_payment (actual_invoice)!=NULL) {
	payment_code = d_payment_get_payment_code (d_invoice_get_payment (actual_invoice));
	*((gulong *) value) =payment_code;
    }
}

void c_invoice_set_state_paid(gpointer data) {
    GtkWidget * wid = glade_xml_get_widget(xml,"radiobutton_pagada_factura");
    gboolean * error =(gboolean *) data;

    if (*error==FALSE) {
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wid),TRUE);
	m_invoice_update_state (actual_invoice,D_INVOICE_PAID);
	d_invoice_set_state (actual_invoice,D_INVOICE_PAID);
    } else {
        c_util_open_error_window (_("Cancelled payment"));
    }
}

void do_second_operation(gpointer data) {
    GtkWidget * wid = glade_xml_get_widget(xml,"radiobutton_pagada_factura");
    GtkWidget * app1 = lookup_widget(wid, "app1");
    GtkWidget * open_insert_op=NULL;
    gdouble total;


    total = d_invoice_get_total (actual_invoice);
    open_insert_op = c_insert_operation_open_window(app1,get_payment_code,c_invoice_set_state_paid,total);
    c_insert_operation_from_split_invoice(TRUE);
    if (open_insert_op!=NULL) {
	gtk_window_set_modal(GTK_WINDOW(open_insert_op),1);
	gtk_window_set_position(GTK_WINDOW(open_insert_op),GTK_WIN_POS_CENTER);
	gtk_window_set_transient_for(GTK_WINDOW(open_insert_op),
				     GTK_WINDOW(app1));
	gtk_widget_show(GTK_WIDGET(open_insert_op));
    }

}

void pay_split_invoice_from_invoice (void) {
    gulong shop_code = d_participant_get_code (d_shop_get_participant (d_invoice_get_shop (actual_invoice)));
    gulong enterprise_code = d_participant_get_code (d_enterprise_get_participant (d_invoice_get_enterprise (actual_invoice)));
    gulong codigoFactura = d_invoice_get_invoice_code (actual_invoice);
    InvoiceId * invoiceId = m_invoice_get_split_invoice (codigoFactura,shop_code,enterprise_code);

    if (invoiceId!=NULL) {
	Invoice * re_invoice = m_invoice_find_by_id (invoiceId);
	Invoice ** invoices = (Invoice **) g_malloc0(sizeof(Invoice *)*2);
	GArray * selected = g_array_new(TRUE,FALSE,sizeof(gint));
	gint i=0;
	g_array_append_val(selected,i);
	invoices[0] = re_invoice;
	m_invoice_set_state_selected  (invoices, selected, d_invoice_get_state (re_invoice), D_INVOICE_PAID);

	g_array_free(selected,FALSE);
	d_invoice_clear (invoices[0]);
	invoices[0]=NULL;
	g_free(invoices);
        if (invoiceId->invoice_number!=NULL) g_free(invoiceId->invoice_number);
        if (invoiceId->shop_code!=NULL) g_free(invoiceId->shop_code);
	if (invoiceId->enterprise_code!=NULL) g_free(invoiceId->enterprise_code);
        g_free(invoiceId);
    }




}

static void do_pay() {
    GtkWidget * wid = glade_xml_get_widget(xml,"radiobutton_pagada_factura");
    GtkWidget * app1 = lookup_widget(wid, "app1");
    if ((payment_note==NULL) && (actual_invoice!=NULL)) {
	if (actual_invoice!=NULL) {
	    if (d_invoice_get_state (actual_invoice)==D_INVOICE_NOT_PAID) {
		if (d_payment_get_is_hire_purchase (d_invoice_get_payment (actual_invoice))==FALSE) {
		    if (d_invoice_get_split_invoice (actual_invoice)!=TRUE) {
			GtkWidget * open_insert;
			Payment * payment_aux;
			gdouble total;

			total = d_invoice_get_total (actual_invoice);

			payment_aux = d_invoice_get_payment (actual_invoice);

			if (d_payment_get_is_paid_in_cash(payment_aux)!=FALSE) {
			    open_insert = c_cash_operation_open_window(app1,c_invoice_set_state_paid,TRUE);
			    load_payment_cash_operation(payment_aux,d_invoice_get_total (actual_invoice));
			} else {
			    open_insert = c_insert_operation_open_window(app1,get_payment_code,c_invoice_set_state_paid,total);
			    c_insert_operation_from_split_invoice(FALSE);
			}
			pay_split_invoice_from_invoice	();
			if (open_insert!=NULL) {
			    gtk_window_set_modal(GTK_WINDOW(open_insert),1);
			    gtk_window_set_position(GTK_WINDOW(open_insert),GTK_WIN_POS_CENTER);
			    gtk_window_set_transient_for(GTK_WINDOW(open_insert),
							 GTK_WINDOW(app1));
			    gtk_widget_show(GTK_WIDGET(open_insert));
			}
		    } else {
			GtkWidget * app1 = lookup_widget(wid, "app1");
			GtkWidget * open_insert;

			gdouble total;
			total = d_invoice_get_total (actual_invoice);

			open_insert = c_insert_operation_open_window(app1,get_payment_code,do_second_operation,(-total));
			c_insert_operation_from_split_invoice(FALSE);
			if (open_insert!=NULL) {
			    gtk_window_set_modal(GTK_WINDOW(open_insert),1);
			    gtk_window_set_position(GTK_WINDOW(open_insert),GTK_WIN_POS_CENTER);
			    gtk_window_set_transient_for(GTK_WINDOW(open_insert),
							 GTK_WINDOW(app1));
			    gtk_widget_show(GTK_WIDGET(open_insert));
			}

		    }
		} else {
                    Payment * payment = d_invoice_get_payment (actual_invoice);
		    GtkWidget * open_emited_payment_control = c_emited_payment_open_window(app1,&payment,FALSE,d_participant_get_code (d_invoice_get_participant (actual_invoice)));
		    if (open_emited_payment_control!=NULL) {
			gtk_window_set_modal(GTK_WINDOW(open_emited_payment_control),1);
			gtk_window_set_position(GTK_WINDOW(open_emited_payment_control),GTK_WIN_POS_CENTER);
			gtk_window_set_transient_for(GTK_WINDOW(open_emited_payment_control),
						     GTK_WINDOW(app1));
			gtk_widget_show(GTK_WIDGET(open_emited_payment_control));
		    }

		}
	    } else if (d_invoice_get_state (actual_invoice)==D_INVOICE_NOT_ISSUED) {
	      c_util_open_info_window (_("You must generate the invoice before"));
	    } else if (d_invoice_get_state (actual_invoice)==D_INVOICE_PAID) {
	      c_util_open_info_window (_("Invoice already paid"));
	    } else if (d_invoice_get_state (actual_invoice)==D_INVOICE_MANAGING) {
	      c_util_open_info_window (_("Invoice in management"));
	    } else {
	      c_util_open_info_window (_("Not generated invoice"));
	    }
	}
    }

}
static void on_pay_invoice_button_clicked(GtkButton *button, gpointer data) {
    if (((d_payment_get_is_hire_purchase (d_invoice_get_payment (actual_invoice)))!=FALSE) &&
	(d_payment_get_is_direct_debit (d_invoice_get_payment (actual_invoice))!=FALSE)) {
        c_util_open_info_window (_("The payment process of the invoices paid by direct debit it is done in the search window"));
    } else {
	do_pay();
    }

}

static void fill_state_data(Invoice * invoice) {
    GtkWidget * wid=NULL;

    switch (d_invoice_get_state (actual_invoice)) {
    case D_INVOICE_NOT_ISSUED: 
      wid = glade_xml_get_widget(xml,"radiobutton_pendiente_factura");
      break;
    case D_INVOICE_NOT_PAID:
      wid = glade_xml_get_widget(xml,"radiobutton_emitida_factura");
      break;
    case D_INVOICE_PAID:
      wid = glade_xml_get_widget(xml,"radiobutton_pagada_factura");
      break;
    case D_INVOICE_MANAGING: 
      wid = glade_xml_get_widget(xml,"radiobutton_engestion_factura");
      break;
    }
    
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wid),TRUE);
}

static void print_invoice(gpointer value,gpointer value2) {

    GtkWidget * wid = glade_xml_get_widget(xml,"radiobutton_emitida_factura");
    gboolean * valor = (gboolean *) value;
    GeneralCondition * general_condition = (GeneralCondition *) value2;
    gboolean code;

    code = * valor;
    if (code ==D_INVOICE_NORMAL_CODE) {
	gulong garage_code =ULONG_MAX;
	from_garage=TRUE;

	garage_code = d_participant_get_code (d_invoice_get_participant (actual_invoice));
	from_garage = m_delivery_payment_client_is_not_from_garage (garage_code,
						     garage_code,
						     ULONG_MAX,
						     ULONG_MAX,
						     ULONG_MAX
						    );
	p_invoice_wrap_print (actual_invoice,D_INVOICE_NORMAL_CODE,general_condition,from_garage);
    } else {
	c_util_open_info_window (_("Invalid option"));

    }

    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wid),TRUE);
    if (d_invoice_get_state (actual_invoice)==D_INVOICE_NOT_ISSUED) {
	m_invoice_update_state (actual_invoice,D_INVOICE_NOT_PAID);
	d_invoice_set_state (actual_invoice,D_INVOICE_NOT_PAID);
    }

}

static void on_print_invoice_button_clicked(GtkButton *button, gpointer data) {


    if (payment_note!=NULL) {
	if (d_payment_note_get_note_code (payment_note)!=ULONG_MAX) {

	    p_payment_note_wrap_print (payment_note);
            /* IMPRIMIR */
	} else {
            c_util_open_info_window (_("You must save the payment note before print it"));
	}
    } else if (actual_invoice!=NULL) {
	if (d_invoice_get_invoice_number (actual_invoice)!=ULONG_MAX) {
            GtkWidget * combo_template = glade_xml_get_widget(xml,"combo_seguros");

	    gulong code = c_util_get_code_from_combo (combo_template);
	    GtkWidget * dialog ;
            gint error =0;

	    if (d_invoice_get_state (actual_invoice)==D_INVOICE_NOT_ISSUED) {
		GtkWidget * label;
                gchar * message=g_strdup(_("The invoice will pass to the waiting for payment state"));
		dialog=gnome_dialog_new(_("State change"),_("Ok"),_("Cancel"),NULL);
		label = gtk_label_new (message);
                gtk_widget_show(label);
		gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (dialog)->vbox), label, TRUE, TRUE, 0);
		switch (gnome_dialog_run_and_close(GNOME_DIALOG(dialog)))
		{
		case -1:
		    error = 1;
		    break;
		case 0:
		    error = 0;
		    break;
		case 1:
		    error = 1;
		    break;
		}
	    }
	    if (error==0) {
		GtkWidget * open_select;

		open_select = c_select_fields_open_window(print_invoice,code);

		if (open_select!=NULL) {
		    gtk_window_set_modal(GTK_WINDOW(open_select),1);
		    gtk_window_set_position(GTK_WINDOW(open_select),GTK_WIN_POS_CENTER);
		    gtk_window_set_transient_for(GTK_WINDOW(open_select),
						 GTK_WINDOW(lookup_widget(GTK_WIDGET(open_select), "app1")));
		    gtk_widget_show(GTK_WIDGET(open_select));
		}
	    }
	    else {
		c_util_open_info_window (_("Invoice printing has been cancelled"));
	    }
	} else {
	    c_util_open_info_window (_("You must save the invoice before print it"));
	}
    } else {
	c_util_open_info_window (_("There is nothing to print"));
    }
}

static void set_dates_on_payment() {
    struct tm* time_tm ;
    time_t *time_aux = (time_t *) g_malloc0(sizeof(time_t));
    GtkWidget * dateedit_primer_pago = glade_xml_get_widget(xml,"dateedit_primer_pago");
    GtkWidget * dateedit_pago_realizado = glade_xml_get_widget(xml,"dateedit_fecha_pago_factura");
    GtkWidget * combo = glade_xml_get_widget(xml,"combo_modo_pago");
    GtkWidget * spinButton = glade_xml_get_widget(xml,"dias_giro");
    gint code_combo = c_util_get_code_from_combo (combo);

    if (!GTK_WIDGET_SENSITIVE(dateedit_pago_realizado))
	gtk_widget_set_sensitive(dateedit_pago_realizado,TRUE);

    if (entra==TRUE) {
	entra=FALSE;
	*time_aux = time(NULL);

	if (code_combo == D_PAYMENT_MONEY_ORDER) {
	    GDate * date;
	    gint days;
	    time_tm = localtime(time_aux);
	    date = g_date_new_dmy(time_tm->tm_mday,time_tm->tm_mon+1,time_tm->tm_year+1900);
	    days = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinButton));
	    g_date_add_days(date,days);
	    time_tm ->tm_mday = g_date_day(date);
	    time_tm ->tm_mon= g_date_month(date)-1;
	    time_tm ->tm_year = g_date_year(date)-1900;
	    g_free(date);
	    *time_aux = mktime(time_tm);

	} else if (code_combo == D_PAYMENT_MONEY_ORDER_AT_END_OF_MONTH) {
	    gint days;

	    *time_aux = time(NULL);
	    time_tm = localtime(time_aux);
	    days = c_util_get_days_to_last_day_of_month (time_aux);
	    gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinButton),1.0*days);
	    time_tm->tm_mday = time_tm->tm_mday + days;
	    *time_aux = mktime(time_tm);

	} else if (code_combo == D_PAYMENT_HIRE_PURCHASE) {
	    time_tm = localtime(time_aux);
	    if (GTK_WIDGET_SENSITIVE(dateedit_pago_realizado)) gtk_widget_set_sensitive(dateedit_pago_realizado,FALSE);
	}

	gtk_calendar_display_options(GTK_CALENDAR(GNOME_DATE_EDIT(dateedit_primer_pago)->calendar),GTK_CALENDAR_SHOW_DAY_NAMES);
	gnome_date_edit_set_time(GNOME_DATE_EDIT(dateedit_primer_pago),*time_aux);
	gnome_date_edit_set_time(GNOME_DATE_EDIT(dateedit_pago_realizado),*time_aux);
	g_free(time_aux);
        entra=TRUE;
    }


}
static void on_money_order_days_changed(GtkEntry *entry,
				 gpointer user_data) {

    set_dates_on_payment();
}

static gdouble get_garage_total_entry(){
    GtkWidget * garage_total_entry = glade_xml_get_widget(xml,"entry_taller_factura");
    gchar * total_taller_char = g_strdup(gtk_entry_get_text(GTK_ENTRY(garage_total_entry)));
    gdouble result=0.0;

    u_points_to_commas (total_taller_char);

    result = atof(total_taller_char);
    return result;
}

static void modify_invoice(void) {
    gboolean no_error = m_invoice_update (actual_invoice);
    if (no_error ==TRUE)
	c_util_open_info_window (_("Data has been updated correctly"));
    else
	c_util_open_info_window (_("Data has not been modified"));
}

static void on_modify_invoice_button_clicked(GtkButton *button, gpointer data) {
    if (actual_invoice!=NULL) {
	if (d_invoice_get_invoice_number (actual_invoice)<(ULONG_MAX-1)) {
	    GtkWidget * issue_date = glade_xml_get_widget(xml,"dateedit_emision");
	    d_invoice_set_issue_date (actual_invoice,gnome_date_edit_get_date(GNOME_DATE_EDIT(issue_date)));
            modify_invoice();
	} else {
	    GtkWidget * garage_total_entry = glade_xml_get_widget(xml,"entry_taller_factura");
            DeliveryPayment ** delivery_payments = d_invoice_get_delivery_payments(actual_invoice);
	    if (delivery_payments!=NULL) {
		if (delivery_payments[1]==NULL) {
		    if (GTK_WIDGET_SENSITIVE(garage_total_entry)!=FALSE) {
                        gboolean no_error=FALSE;
                        d_delivery_payment_set_garage_total (delivery_payments[0],get_garage_total_entry());
			no_error = m_delivery_payment_update_garage_total (delivery_payments[0]);
			if (no_error==TRUE)
			    c_util_open_info_window (_("Garage total correctly modified"));
                        else
			    c_util_open_info_window (_("The garage hasn't been modified correctly"));
		    }
		}
	    }
	}
    }
}

static void put_buttons_sensible(gint option) {
    GtkWidget * button_imprimir = glade_xml_get_widget(xml,"button_imprimir_factura");
    GtkWidget * button_albaran = glade_xml_get_widget(xml,"button_albaran_factura");
    GtkWidget * button_guardar = glade_xml_get_widget(xml,"button_guardar_facturas");
    GtkWidget * button_anular = glade_xml_get_widget(xml,"button_anular_factura");
    GtkWidget * button_modificar =glade_xml_get_widget(xml,"button_modificar_factura");
    GtkWidget * button_pagar = glade_xml_get_widget(xml,"button_pagar_factura");

#ifndef IS_SHOP
    if (option==0) {
        gtk_widget_set_sensitive(button_imprimir,FALSE);
        gtk_widget_set_sensitive(button_albaran,FALSE);
    } else if (option==1) {
	gtk_widget_set_sensitive(button_imprimir,TRUE);
	gtk_widget_set_sensitive(button_albaran,FALSE);
	gtk_widget_set_sensitive(button_anular,TRUE);
	gtk_widget_set_sensitive(button_modificar,TRUE);
	gtk_widget_set_sensitive(button_pagar,TRUE);

    } else if (option==2) {
	gtk_widget_set_sensitive(button_imprimir,FALSE);
	gtk_widget_set_sensitive(button_albaran,TRUE);
    } else if (option==3) {
	gtk_widget_set_sensitive(button_guardar,FALSE);
	gtk_widget_set_sensitive(button_anular,TRUE);
	gtk_widget_set_sensitive(button_modificar,FALSE);
	gtk_widget_set_sensitive(button_pagar,FALSE);
	gtk_widget_set_sensitive(button_imprimir,FALSE);
	gtk_widget_set_sensitive(button_albaran,TRUE);
    } else if (option==4) {
	gtk_widget_set_sensitive(button_imprimir,FALSE);
	gtk_widget_set_sensitive(button_albaran,FALSE);
	gtk_widget_set_sensitive(button_modificar,TRUE);
	gtk_widget_set_sensitive(button_pagar,FALSE);
	gtk_widget_set_sensitive(button_guardar,TRUE);
	gtk_widget_set_sensitive(button_anular,FALSE);

    } else if (option==5) {
	gtk_widget_set_sensitive(button_guardar,TRUE);
	gtk_widget_set_sensitive(button_anular,TRUE);
	gtk_widget_set_sensitive(button_modificar,TRUE);
	gtk_widget_set_sensitive(button_pagar,FALSE);
	gtk_widget_set_sensitive(button_imprimir,FALSE);
	gtk_widget_set_sensitive(button_albaran,TRUE);
    } else if (option==6) {
	gtk_widget_set_sensitive(button_guardar,FALSE);
	gtk_widget_set_sensitive(button_modificar,TRUE);
	gtk_widget_set_sensitive(button_imprimir,TRUE);
	gtk_widget_set_sensitive(button_anular,TRUE);

    }
#else
    if (option==1) {
	gtk_widget_set_sensitive(button_imprimir,FALSE);
	gtk_widget_set_sensitive(button_guardar,FALSE);
	gtk_widget_set_sensitive(button_anular,FALSE);
	gtk_widget_set_sensitive(button_modificar,FALSE);
	gtk_widget_set_sensitive(button_pagar,FALSE);
	gtk_widget_set_sensitive(button_albaran,TRUE);

    } else if (option==2) {
        if (negative_price==FALSE)
	    gtk_widget_set_sensitive(button_guardar,TRUE);
	else
	    gtk_widget_set_sensitive(button_guardar,FALSE);
	gtk_widget_set_sensitive(button_anular,FALSE);
	gtk_widget_set_sensitive(button_modificar,FALSE);
	gtk_widget_set_sensitive(button_pagar,FALSE);
	gtk_widget_set_sensitive(button_imprimir,FALSE);
	gtk_widget_set_sensitive(button_albaran,TRUE);
    } else if (option==3) {
        if (negative_price==FALSE)
	    gtk_widget_set_sensitive(button_guardar,TRUE);
	else
	    gtk_widget_set_sensitive(button_guardar,FALSE);
	gtk_widget_set_sensitive(button_anular,FALSE);
	gtk_widget_set_sensitive(button_modificar,FALSE);
	gtk_widget_set_sensitive(button_pagar,FALSE);
	gtk_widget_set_sensitive(button_imprimir,TRUE);
	gtk_widget_set_sensitive(button_albaran,FALSE);
    } else if (option==6) {
	gtk_widget_set_sensitive(button_guardar,FALSE);
	gtk_widget_set_sensitive(button_modificar,FALSE);
	gtk_widget_set_sensitive(button_imprimir,TRUE);
	gtk_widget_set_sensitive(button_anular,FALSE);

    } else {
	gtk_widget_set_sensitive(button_guardar,FALSE);
	gtk_widget_set_sensitive(button_anular,FALSE);
	gtk_widget_set_sensitive(button_modificar,FALSE);
	gtk_widget_set_sensitive(button_pagar,FALSE);
	gtk_widget_set_sensitive(button_imprimir,FALSE);
	gtk_widget_set_sensitive(button_albaran,TRUE);
    }
#endif
}

static void fill_account_number_combo() {
    GtkWidget * combo;
    gulong codigo_gulong=ULONG_MAX;
    GList * actual_account;
    if (actual_invoice!=NULL) {
	if (d_invoice_get_participant (actual_invoice)!=NULL) {
	    codigo_gulong = d_participant_get_code (d_invoice_get_participant (actual_invoice));
	} else {
	    if (d_invoice_get_repair (actual_invoice)!=NULL) {
		codigo_gulong = d_participant_get_code (d_client_get_participant (d_repair_get_client (d_invoice_get_repair (actual_invoice))));
	    } else {
                DeliveryPayment ** delivery_payments = d_invoice_get_delivery_payments(actual_invoice);
		codigo_gulong = d_participant_get_code (d_client_get_participant (d_repair_get_client (d_delivery_payment_get_repair (delivery_payments[0]))));
	    }
	}
    } else {
        codigo_gulong = d_participant_get_code (d_client_get_participant (d_repair_get_client (d_payment_note_get_repair_detail (payment_note))));
    }
    accounts = m_account_find_by_participant_code (codigo_gulong);

    combo = glade_xml_get_widget (xml, "combo_numero_cuenta_factura");

    c_util_clean_code_and_name_combo (combo);

    actual_account = g_list_first(accounts);

    while (actual_account!=NULL) {
	gchar * complete_account_number_gchar=NULL;
	Account * account = (Account *) actual_account->data;

	complete_account_number_gchar = d_account_get_complete_number(account);
	c_util_set_code_and_name_on_combo (g_list_length(accounts),complete_account_number_gchar,GTK_COMBO(combo));
	g_free(complete_account_number_gchar);
        actual_account=g_list_next(actual_account);
    }
}

static void view_garage_total_entry(gboolean sensitive) {
    gboolean visible = FALSE;
#ifndef IS_SHOP
    if (actual_invoice!=NULL)
	visible = d_invoice_get_is_from_garage(actual_invoice);

    if (visible!=FALSE) {
	GtkWidget * entry = glade_xml_get_widget(xml,"entry_taller_factura");
	GtkWidget * label = glade_xml_get_widget(xml,"label_taller_factura");
	if (!GTK_WIDGET_VISIBLE(entry)) gtk_widget_show(entry);
	if (!GTK_WIDGET_VISIBLE(label)) gtk_widget_show(label);
	gtk_widget_set_sensitive(entry,sensitive);
	gtk_widget_set_sensitive(label,sensitive);
    } else {
        GtkWidget * entry = glade_xml_get_widget(xml,"entry_taller_factura");
	GtkWidget * label = glade_xml_get_widget(xml,"label_taller_factura");
	if (GTK_WIDGET_VISIBLE(entry)) gtk_widget_hide(entry);
	if (GTK_WIDGET_VISIBLE(label)) gtk_widget_hide(label);
    }
#else
    {
	GtkWidget * entry = glade_xml_get_widget(xml,"entry_taller_factura");
	GtkWidget * label = glade_xml_get_widget(xml,"label_taller_factura");
	if (GTK_WIDGET_VISIBLE(entry)) gtk_widget_hide(entry);
	if (GTK_WIDGET_VISIBLE(label)) gtk_widget_hide(label);
    }
#endif
}

static void set_split_invoice_state(gboolean checked,gboolean sensitive) {
    GtkWidget * wid = glade_xml_get_widget(xml,"checkbutton_refacturacion");
    GtkWidget * split_invoice_combo = glade_xml_get_widget(xml,"combo_tienda_refacturacion");
#ifndef IS_SHOP
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wid),checked);
    gtk_widget_set_sensitive(wid,sensitive);
    gtk_widget_set_sensitive(split_invoice_combo,sensitive);
#else
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wid),FALSE);
    gtk_widget_set_sensitive(wid,FALSE);
    gtk_widget_set_sensitive(split_invoice_combo,FALSE);
#endif
}

static gboolean get_is_from_garage(DeliveryPaymentId ** delivery_payments_ids) {
    gint index=0;
    gboolean result=FALSE;
    if (delivery_payments_ids!=NULL)
	while (delivery_payments_ids[index]!=NULL) {
	    if (delivery_payments_ids[index]->from_garage!=FALSE)
                result=TRUE;
	    index++;
	}
    return result;
}


static void on_facturas_hide(GtkWidget *widget, gpointer user_data){
  
    if (documents_window != NULL) {
	gtk_widget_destroy(documents_window);
        documents_window=NULL;
    }
}

void c_invoice_document_destroyed() {
  documents_window = NULL;
}

static void fill_garage_total_entry(gdouble total_taller) {
    GtkWidget * garage_total_entry=glade_xml_get_widget(xml,"entry_taller_factura");
    gchar * total=NULL ;
    if (total_taller>0) {
	total = g_strdup_printf("%.2f",total_taller);
	gtk_entry_set_text(GTK_ENTRY(garage_total_entry),total);
        g_free(total);
    }
}


void print_normal_delivery_payment(gpointer value,gpointer value2) {
    gboolean * valued = (gboolean *) value2;
    p_delivery_payment_wrap_print ((DeliveryPayment *) value,D_INVOICE_NORMAL_CODE,*valued);
}

static void fill_date_payment_note(PaymentNote * payment_note) {
    GtkWidget * note_dateedit=glade_xml_get_widget(xml,"dateedit_emision");
    gnome_date_edit_set_time(GNOME_DATE_EDIT(note_dateedit),d_payment_note_get_emission_date (payment_note));
}

static void set_template(Invoice * invoice) {

    GtkWidget * entry_template = glade_xml_get_widget(xml,"entry_plantilla");
    Company * company=NULL;

    if (invoice!=NULL) {
	company = d_repair_get_company (d_invoice_get_first_repair(invoice));
    }

    if (company!=NULL) {
	gint company_template = d_company_get_template (company);
	if (company_template == D_INVOICE_NORMAL_CODE) {
	    gtk_entry_set_text(GTK_ENTRY(entry_template),D_INVOICE_MESSAGE_NORMAL);
	} else {
	    c_util_open_info_window (_("Regular template will be used to print"));
	}
    } else {
	gtk_entry_set_text(GTK_ENTRY(entry_template),D_INVOICE_MESSAGE_NORMAL);
    }
}
/*Modifica el cliente de la factura */
static void modify_client_invoice(gpointer data) {
    gulong * receiver = (gulong *) data;
    Participant * participant_to_dup=NULL;
    Repair * repair = NULL;

    if (data!=NULL) {
        repair = d_invoice_get_first_repair(actual_invoice);
	if ((*receiver) == D_INVOICE_CLIENT_SELECTION) {
	    participant_to_dup = d_client_get_participant (d_repair_get_client (repair));
	} else if ((*receiver) == D_INVOICE_COMPANY_SELECTION) {
	    participant_to_dup = d_company_get_participant (d_repair_get_company (repair));
	} else if ((*receiver) == D_INVOICE_CONSORTIUM_SELECTION) {
	    Consortium ** consortium;
	    gulong consortium_code = d_company_get_consortium_code (d_repair_get_company (repair));
	    consortium = m_consortium_find_by_code (consortium_code);
	    participant_to_dup = d_consortium_get_participant (consortium[0]);
	}
	d_participant_clear (d_invoice_get_participant (actual_invoice));
	d_invoice_set_participant (actual_invoice,d_participant_duplicate (participant_to_dup));
        modify_invoice();
    }
    fill_client_data(d_invoice_get_participant (actual_invoice));

}
static void on_invoice_name_entry_button_press_event(GtkButton *button,gpointer user_data) {

    GtkWidget * open_ir;
    GtkWidget * app1 = lookup_widget(GTK_WIDGET(button), "app1");

    if (actual_invoice!=NULL) {
	if (d_invoice_get_invoice_number (actual_invoice)<(ULONG_MAX)) {
	    open_ir = c_invoice_receiver_open_window(app1,modify_client_invoice,actual_invoice);
	    if (c_invoice_receiver_open_window!=NULL) {
		gtk_window_set_modal(GTK_WINDOW(open_ir),1);
		gtk_window_set_position(GTK_WINDOW(open_ir),GTK_WIN_POS_CENTER);
		gtk_window_set_transient_for(GTK_WINDOW(open_ir),
					     GTK_WINDOW(app1));
		gtk_widget_show(GTK_WIDGET(open_ir));
	    }
	}
    }
}


static gint on_invoice_vat_entry_button_press_event(GtkWidget *widget,
							 GdkEventButton *event, gpointer user_data) {

    GtkWidget * menu_item1 = gtk_menu_item_new_with_label(_("Set up an invoice without VAT"));
    GtkWidget * menu_item2 = gtk_menu_item_new_with_label(_("Set up an invoice with VAT"));
    GtkWidget * menu = gtk_menu_new();

    if (actual_invoice!=NULL) {
	    gtk_menu_item_activate(GTK_MENU_ITEM(menu_item1));
	    gtk_menu_item_activate(GTK_MENU_ITEM(menu_item2));
	    gtk_menu_set_title(GTK_MENU(menu),_("Select the operation"));
	    gtk_menu_prepend(GTK_MENU(menu),menu_item1);
	    gtk_menu_prepend(GTK_MENU(menu),menu_item2);


	    if (d_invoice_get_vat_type (actual_invoice)!=0) {
		gtk_widget_set_sensitive(menu_item2,FALSE);
		gtk_widget_set_sensitive(menu_item1,TRUE);
	    } else {
		gtk_widget_set_sensitive(menu_item1,FALSE);
		gtk_widget_set_sensitive(menu_item2,TRUE);
	    }

	    gtk_widget_show(menu_item1);
	    gtk_widget_show(menu_item2);

	    gtk_signal_connect_object(GTK_OBJECT(menu_item2), "activate",
				      GTK_SIGNAL_FUNC (on_activate_with_vat), NULL);
	    gtk_signal_connect_object(GTK_OBJECT(menu_item1), "activate",
				      GTK_SIGNAL_FUNC (on_without_vat_activate), NULL);

	    if (event->button == 3)
	    {

		gtk_menu_popup (GTK_MENU(menu), NULL, NULL, NULL, NULL,
				event->button, event->time);
		return TRUE;
	    }
    }
    return FALSE;
}

static void on_activate_with_vat(GtkMenuItem *menuitem,
				gpointer user_data) {

    if (actual_invoice!=NULL) {
	gdouble * subtotal = (gdouble *) g_malloc0(sizeof(gdouble));
	gdouble * vat = (gdouble *) g_malloc0(sizeof(gdouble));
	gdouble * total = (gdouble *) g_malloc0(sizeof(gdouble));

	d_invoice_set_vat_type (actual_invoice,-1);
	d_invoice_calculate_total (subtotal,vat,total,actual_invoice,TRUE);
	fill_total_data(*subtotal,*vat,*total);

    } else {
        c_util_open_info_window ("An invoice without VAT can't be set up");
    }

}

static void on_without_vat_activate(GtkMenuItem *menuitem,
				gpointer user_data) {
    if (actual_invoice!=NULL) {
	gdouble * subtotal = (gdouble *) g_malloc0(sizeof(gdouble));
	gdouble * vat= (gdouble *) g_malloc0(sizeof(gdouble));
	gdouble * total = (gdouble *) g_malloc0(sizeof(gdouble));

	d_invoice_set_vat_type (actual_invoice,0.0);
	d_invoice_calculate_total (subtotal,vat,total,actual_invoice,TRUE);
	fill_total_data(*subtotal,*vat,*total);
    } else {
        c_util_open_info_window (_("An invoice without VAT can't be set up"));
    }

}

static void on_recover_button_clicked(GtkButton *button, gpointer data) {
    if (actual_invoice!=NULL) {
	if (d_invoice_get_invoice_code (actual_invoice)!=ULONG_MAX) {
	    PaymentId * paymentId = m_invoice_find_last_payment_from_client (actual_invoice);
	    Payment * payment = NULL;
	    Payment * payment_to_invoice = NULL;
	    gulong payment_code =d_payment_get_payment_code (d_invoice_get_payment (actual_invoice));

	    if (paymentId!=NULL) {
		payment = m_payment_find_by_id (paymentId);
		fill_payment_data(payment,TRUE);
		payment_to_invoice = create_payment_from_screen(0);
		d_payment_clear (payment);
		d_payment_set_payment_code (payment_to_invoice,payment_code);
		d_invoice_set_payment (actual_invoice,payment_to_invoice);

		if (paymentId->payment_code!=NULL)
		    g_free(paymentId->payment_code);
		if (paymentId->shop_code!=NULL)
		    g_free(paymentId->shop_code);
		if (paymentId->enterprise_code!=NULL)
		    g_free(paymentId->enterprise_code);
	    } else {
                c_util_open_info_window (_("Error recovering data.\n Probably this will be the first invoice of this client"));
	    }
	}
    }

}

static void on_modify_button_clicked(GtkButton *button, gpointer data) {

    if (actual_invoice!=NULL) {
	if (d_invoice_get_invoice_code (actual_invoice) != ULONG_MAX) {
	    if (d_invoice_get_state (actual_invoice)<D_INVOICE_PAID) {
		gboolean can_do = FALSE;
		if (d_payment_get_is_hire_purchase (d_invoice_get_payment (actual_invoice))!=FALSE) {
		    if (d_payment_get_achieved_payments (d_invoice_get_payment (actual_invoice))==0) {
			can_do = TRUE;
		    }
		} else {
		    can_do = TRUE;
		}
		if (can_do!=FALSE) {
		    GtkWidget * wid = glade_xml_get_widget(xml,"radiobutton_pagada_factura");
		    GtkWidget * app1 = lookup_widget(wid, "app1");

		    Payment * payment = d_invoice_get_payment (actual_invoice);
		    GtkWidget * open_emited_payment_control = c_emited_payment_open_window(app1,&payment,TRUE,d_participant_get_code (d_invoice_get_participant (actual_invoice)));
		    if (open_emited_payment_control!=NULL) {
			gtk_window_set_modal(GTK_WINDOW(open_emited_payment_control),1);
			gtk_window_set_position(GTK_WINDOW(open_emited_payment_control),GTK_WIN_POS_CENTER);
			gtk_window_set_transient_for(GTK_WINDOW(open_emited_payment_control),
						     GTK_WINDOW(app1));
			gtk_widget_show(GTK_WIDGET(open_emited_payment_control));
		    }
		} else {
		    c_util_open_info_window (_("You just can modify the payment of unpaid invoices"));
		}
	    } else {
		c_util_open_info_window (_("You just can modify the payment of unpaid invoices"));
	    }
	} else {
	    c_util_open_info_window (_("There is not any invoice"));
	}
    } else {
	c_util_open_info_window (_("There is not any invoice"));
    }
}

void c_invoice_repaint_payment(void) {
    fill_payment_data(d_invoice_get_payment (actual_invoice),FALSE);
    inicialize_payment_screen(FALSE,FALSE);
}
