
/* invoice_control.h
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */


#ifndef _INVOICE_CONTROL_
#define _INVOICE_CONTROL_


#include "delivery_payment.h"
#include "invoice_model.h"
#include "payment_note_model.h"

/**
 * c_invoice_open_window:
 * @app1: #Widget que contiene la aplicacin.
 * @vbox: Contenedor que est dentro de la ventana de contabilidad.
 *
 * Inicializa la ventana de introduccin de factura. Esta pantalla permite introducir facturas
 * emitidas por cualquier empresa en la base de datos. Esta pantalla no puede ser utilizada por s sola,
 * ya que no permite realizar ningua operacin si no se ha cargado antes una reparacin, uno o varios
 * albaranes o una factura ya existente. Cuando se realiza la insercin de una factura el sistema se encarga
 * de proporcionar el nmero adecuado para la misma. En esta pantalla tambin se gestionan las notas.
 * A partir de esta pantalla se puede imprimir una factura con cualquier plantilla o una nota con la plantilla normal.
 *
 * Returns:
 * 
 */
GtkWidget *c_invoice_open_window(GtkWidget *app1, GtkWidget *vbox);
/**
 * c_invoice_load_delivery_payments:
 * @delivery_payments_from_client: Lista de identificadores de albaranes de un cliente que se desea
 * agrupar en la misma factura.
 *
 * Carga la lista de albaranes de una cliente que se desean agrupar en una factura. Este metodo es llamado
 * desde la bsqueda de albaranes para facturarlos.
 *
 */
void c_invoice_load_delivery_payments(DeliveryPaymentId ** delivery_payments_from_client);
/**
 * c_invoice_load_repair:
 * @repair: Reparacin que se desea facturar.
 *
 * Carga una reparacin en la pantalla de factura. Este caso se produce cuando se desea facturar directamente
 * desde una reparacin sin emitir un albarn.
 *
 */
void c_invoice_load_repair(Repair * repair);
/**
 * c_invoice_load:
 * @invoiceId: Identificador de la factura que se desea cargar de la base de datos.
 *
 * Carga una factura de la base de datos a partir del identificador de la misma.
 *
 */
void c_invoice_load(InvoiceId * invoiceId);
/**
 * c_invoice_load_payment_note:
 * @paymentNoteId: Identificador de la nota que se desea cargar en la pantalla.
 *
 * Carga la nota emitida por una empresa en la pantalla. Las notas se gestionan tambin en esta
 * pantalla.
 *
 */
void c_invoice_load_payment_note(PaymentNoteId * paymentNoteId);
/**
 * c_invoice_repaint_repair_lines:
 *
 * Repinta las lneas de factura de la factura que est en memoria o bien de la nota.
 *
 */
void c_invoice_repaint_repair_lines(void);
/**
 * c_invoice_document_destroyed:
 * 
 * Desreferencia la ventana de documentos.
 *
 */
void c_invoice_document_destroyed(void);
/**
 * c_invoice_set_state_paid:
 * @data: Indica si se acarrea error.
 *
 * Establece como pagada la factura si no se pasa un error.
 *
 */
void c_invoice_set_state_paid(gpointer data);
/**
 * c_invoice_repaint_payment:
 * 
 * Repinta el pago.
 *
 */
void c_invoice_repaint_payment(void);

#endif

