/* update_entry_control.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */


#include <gnome.h>
#include <glade/glade.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <glib.h>
#include <stdlib.h>


#include "a-shortcut.h"
#include "support.h"
#include "control_util.h"
#include "invoice_receiver_control.h"
#include "consortium_model.h"
#include "invoice_control.h"
#include "post_address.h"
#include "post_address_model.h"

static GtkWidget *invoice_receiver=NULL;
static GladeXML *xml;
static void (*function_to_call)(gpointer value);
static gchar widget_name[]="dialog_receptor_factura";
static gchar url[]="/src/interface/glade/invoice_view.glade";


static void on_cancel_button_clicked(GtkButton *button, gpointer user_data);
static void on_accept_button_clicked(GtkButton *button,  gpointer user_data);
static void fill_combo_with_invoice(Invoice * invoice);
//static void on_receiver_combo_entry_hanged(GtkEntry *entry, gpointer user_data);

GtkWidget *c_invoice_receiver_open_window(GtkWidget *app1, gpointer f_to_call,Invoice * invoice) {
    gchar * directory = g_strconcat(AA_DATA_DIR,url,NULL);

    if (invoice_receiver==NULL)
    {
	GtkWidget *dialog;

	xml = glade_xml_new (directory, widget_name);
	dialog = glade_xml_get_widget (xml, widget_name);


	glade_xml_signal_connect (xml, "on_button_cancelar_dialogo_receptor_clicked",
				  GTK_SIGNAL_FUNC (on_cancel_button_clicked));
	glade_xml_signal_connect (xml, "on_button_aceptar_dialogo_receptor_clicked",
			          GTK_SIGNAL_FUNC (on_accept_button_clicked));

        fill_combo_with_invoice	(invoice);
	invoice_receiver=dialog;
    }
    function_to_call = f_to_call;
    g_free(directory);
    return invoice_receiver;
}


static void on_cancel_button_clicked(GtkButton *button,gpointer user_data) {
    gtk_widget_destroy(invoice_receiver);
    function_to_call = NULL;
    invoice_receiver = NULL;
}

static void on_accept_button_clicked(GtkButton *button,gpointer user_data) {
    GtkWidget * combo = glade_xml_get_widget(xml,"combo_receptor_dialogo_receptor");
    gpointer data = NULL;
    gulong * code_pointer = (gulong *) g_malloc0(sizeof(gulong));
    gulong code = c_util_get_code_from_combo (combo);

    * code_pointer = code;
    data = (gpointer) code_pointer;
    function_to_call(data);
    if (invoice_receiver!=NULL)
	gtk_widget_destroy(invoice_receiver);
    function_to_call = NULL;
    invoice_receiver = NULL;

}

static void fill_combo_with_invoice(Invoice * invoice) {
    GtkWidget * combo = glade_xml_get_widget(xml,"combo_receptor_dialogo_receptor");
    gchar * nombre = NULL;                        
    gulong consortium_code=ULONG_MAX;
    gchar * company_name = NULL;
    gchar * consortium_name = NULL;
    Consortium ** consortium = NULL;
    DeliveryPayment ** delivery_payments = d_invoice_get_delivery_payments (invoice);

    if (delivery_payments!=NULL) {
	if (delivery_payments[0]!=NULL) {
            Repair * repair = d_delivery_payment_get_repair (delivery_payments[0]);
	    nombre = d_participant_get_nick (d_client_get_participant (d_repair_get_client (repair)));
	    if ((d_repair_get_company (repair))!=NULL) {
		    company_name = d_participant_get_nick (d_company_get_participant (d_repair_get_company (repair)));
		    consortium_code =  d_company_get_consortium_code (d_repair_get_company (repair));
	    }

	} else {
	    nombre = NULL;
	}
    } else {
	nombre = d_participant_get_nick (d_client_get_participant (d_repair_get_client (d_invoice_get_repair (invoice))));
	if (d_repair_get_company (d_invoice_get_repair (invoice))!=NULL) {
	    company_name = d_participant_get_nick (d_company_get_participant (d_repair_get_company (d_invoice_get_repair (invoice))));
	    consortium_code = d_company_get_consortium_code (d_repair_get_company (d_invoice_get_repair (invoice)));
	}
    }
    if (consortium_code!=ULONG_MAX) {
	consortium = m_consortium_find_by_code (consortium_code);
    }
    if (consortium!=NULL) {
	consortium_name = d_participant_get_nick (d_consortium_get_participant (consortium[0]));
    } else {
    }

    c_util_set_code_and_name_on_combo (D_INVOICE_CLIENT_SELECTION,nombre,GTK_COMBO(combo));
    if (company_name!=NULL) {
	c_util_set_code_and_name_on_combo (D_INVOICE_COMPANY_SELECTION,company_name,GTK_COMBO(combo));
    }
    if (consortium_name!=NULL) {
	c_util_set_code_and_name_on_combo (D_INVOICE_CONSORTIUM_SELECTION,consortium_name,GTK_COMBO(combo));
    }
    g_free(nombre);
    if (company_name!=NULL) g_free(company_name);
    if (consortium_name!=NULL) g_free(consortium_name);
}



