 /* listing_operation_control.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include <gnome.h>
#include <glade/glade.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <glib.h>
#include <stdlib.h>

#define P_LISTING_OPERATION_NOTIFICATION_DATE_COLUMN 0
#define C_ACCOUNT_NUMBER_COLUMN 1
#define C_COMPANY_COLUMN 2
#define C_AMOUNT_COLUMN 4
#define C_DEPOSIT_DATE_COLUMN 3
#define C_ALL_SHOPS_CODE 0

#include "a-shortcut.h"
#include "support.h"
#include "control_util.h"
#include <../interface/printing/listing-print-wrapper.h>
#include "listing_operation.h"
#include "listing_operation_model.h"
#include "listing_operation_control.h"
#include "parameter_manager.h"
#include "../interface/printing/listing-print-wrapper.h"
#include "invoice.h"
#include "charge_invoice.h"
#include "shop_model.h"
#include "account.h"
#include "account_model.h"
#include "company_model.h"
#include "supplier_model.h"

static GtkWidget *open_listing_operations=NULL;
static ListingOperation ** listing_operations;
static GladeXML *xml;
static gchar widget_name[]="vbox1_listado_de_pagos";
static gchar url[]="/src/interface/glade/listing_view.glade";

static void clean_screen(void);
static void fill_shop_combo(void);
static void fill_invoice_state_combo(gint type);
static void fill_payments_combo(void);
static void on_listing_management_press_key(GtkWidget *widget,
					   GdkEventKey *event,
					   gpointer user_data);
static void on_delete_button_clicked(GtkButton *button, gpointer data);
static void on_print_button_clicked(GtkButton *button, gpointer data);
static void on_search_button_clicked(GtkButton *button, gpointer data);
static void do_search(void);
static void fill_accounts_combo(void);
static void fill_general_enterprise_combo(gint type);
static void on_radiobutton_ingresos_toggled(GtkToggleButton *togglebutton,
						 gpointer user_data);
static void on_radiobutton_gastos_toggled(GtkToggleButton *togglebutton,
						 gpointer user_data);
static void on_radiobutton_ambos_toggled(GtkToggleButton *togglebutton,
						 gpointer user_data);
static void set_window_values(gint type);
static gint get_type_of_search(void);

GtkWidget *c_listing_operation_open_window(GtkWidget *app1, GtkWidget *vbox)

{
   
    gchar * directory = g_strconcat(AA_DATA_DIR,url,NULL);
    time_t expiry_date = time(NULL) + 86400;
    GtkWidget * dateedit_date= NULL;
    
   
  if (open_listing_operations==NULL)
    {
      GtkWidget *dialog;
      gchar * s_name = NULL;

      xml = glade_xml_new (directory, widget_name);
      dialog = glade_xml_get_widget (xml, widget_name);
      
      gtk_box_pack_start (GTK_BOX (vbox), dialog, TRUE, TRUE, 0);  
                                      
      glade_xml_signal_connect (xml, "on_button_borrar_gestion_listados_clicked",
				GTK_SIGNAL_FUNC (on_delete_button_clicked));
      glade_xml_signal_connect (xml, "on_button_imprimir_gestion_listados_clicked",
				GTK_SIGNAL_FUNC (on_print_button_clicked));
      glade_xml_signal_connect (xml, "on_button_buscar_gestion_listados_clicked",
				GTK_SIGNAL_FUNC (on_search_button_clicked));
      glade_xml_signal_connect (xml, "on_press_key_gestion_listados",
				GTK_SIGNAL_FUNC (on_listing_management_press_key));
      glade_xml_signal_connect (xml, "on_radiobutton_ingresos_toggled",
				GTK_SIGNAL_FUNC (on_radiobutton_ingresos_toggled));
      glade_xml_signal_connect (xml, "on_radiobutton_gastos_toggled",
				GTK_SIGNAL_FUNC (on_radiobutton_gastos_toggled));
      glade_xml_signal_connect (xml, "on_radiobutton_ambos_toggled",
				GTK_SIGNAL_FUNC (on_radiobutton_ambos_toggled));



      fill_invoice_state_combo(M_LISTING_OPERATION_INCOMING);
      fill_shop_combo();
      fill_general_enterprise_combo(M_LISTING_OPERATION_INCOMING);
      fill_payments_combo();

//      gtk_widget_set_sensitive(glade_xml_get_widget (xml, "combo_tienda_gestion_listados"),FALSE);

      s_name = u_parameter_manager_get_parameter ("tienda");
      if (s_name!=NULL) {
	  gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget (xml, "combo_entry_tienda_gestion_listados")), s_name);
      }
      g_free(s_name);

      open_listing_operations=dialog;

    } else {

    }
  dateedit_date = glade_xml_get_widget (xml, "dateedit_vencimiento_gestion_listados");
  gnome_date_edit_set_time(GNOME_DATE_EDIT(dateedit_date), expiry_date);
  fill_accounts_combo();
  return open_listing_operations;
  
}





static void fill_listing_operation_list_on_clist(GtkCList * list,ListingOperation ** listing_operation_array) {
    gchar * content=NULL;
    gint index=0;
    gchar *empty_row[]={"","","","",""};

    if (listing_operation_array!=NULL) {
	while (listing_operation_array[index]!=NULL) {
            gchar *date_char = (gchar *) g_malloc(sizeof(gchar)*100);
	    size_t stored_ing;
            gchar *notification_date_char = (gchar *) g_malloc(sizeof(gchar)*100);
	    size_t stored_not;
	    time_t date = d_listing_operation_get_date (listing_operation_array[index]);
	    time_t notification_date =d_listing_operation_get_notification_date (listing_operation_array[index]);

	    if (index==30) gtk_clist_freeze ((GtkCList *) list);
	    gtk_clist_append ((GtkCList *) list,empty_row);
	    	    
	    
	    stored_ing=strftime(date_char,sizeof(gchar)*100,"%Y-%m-%d", localtime(&date));
	    stored_not=strftime(notification_date_char,sizeof(gchar)*100,"%Y-%m-%d", localtime(&notification_date));

	    gtk_clist_set_text((GtkCList *) list, index, C_DEPOSIT_DATE_COLUMN, date_char);

	    gtk_clist_set_text((GtkCList *) list, index, P_LISTING_OPERATION_NOTIFICATION_DATE_COLUMN, notification_date_char);

	    
	    content =d_listing_operation_get_company_name (
						    listing_operation_array[index]
						   );

	    if (content!=NULL) {
		gtk_clist_set_text((GtkCList *) list, index, C_COMPANY_COLUMN,content);
		g_free(content);
	    }

	    content = d_listing_operation_get_complete_account_number (listing_operation_array[index]);
	    if ((content!=NULL) && (c_util_get_code_from_combo (glade_xml_get_widget(xml,"combo_cuenta_corriente"))==ULONG_MAX)) {
		gtk_clist_set_text((GtkCList *) list, index, C_ACCOUNT_NUMBER_COLUMN,content);
		g_free(content);
	    }


            content = g_strdup_printf("%.2f",d_listing_operation_get_amount (listing_operation_array[index]));
	    if (content!=NULL) {
		gtk_clist_set_text((GtkCList *) list, index, C_AMOUNT_COLUMN,content);
		g_free(content);
	    }
            g_free(date_char);
            g_free(notification_date_char);
	    index++;
	}
    }
    gtk_clist_thaw ((GtkCList *) list);

}





static void fill_invoice_state_combo(gint type) {
    GtkWidget * combo=glade_xml_get_widget(xml,"combo_estado_factura_gestion_listados");
    gchar * temp;

    c_util_clean_code_and_name_combo (combo);

    switch (type) {
    case M_LISTING_OPERATION_INCOMING: {
	temp = g_strdup(D_INVOICE_MESSAGE_PAID);
	c_util_set_code_and_name_on_combo (D_INVOICE_PAID,temp,GTK_COMBO(combo));
	g_free(temp);
        break;
    }
    case M_LISTING_OPERATION_EXPENSE: {
	temp = g_strdup(D_CHARGE_INVOICE_MESSAGE_PAID);
	c_util_set_code_and_name_on_combo (D_CHARGE_INVOICE_PAID,temp,GTK_COMBO(combo));
	g_free(temp);
        break;
    }
    }

}


static void fill_shop_combo() {
    GtkWidget * combo;
    Shop ** shops;
    gchar * temp;

    combo = glade_xml_get_widget (xml, "combo_tienda_gestion_listados");

    temp = g_strdup(_("All the shops"));
    c_util_set_code_and_name_on_combo (C_ALL_SHOPS_CODE,temp,GTK_COMBO(combo));
    shops = m_shop_find_all ();
    g_free(temp);

    c_util_fill_combo_with_shops (combo,shops);

}

static void fill_general_enterprise_combo(gint type) {
    GtkWidget * combo;
    gchar * temp;


    combo = glade_xml_get_widget (xml, "combo_compania");

    c_util_clean_code_and_name_combo (combo);

    if (type==M_LISTING_OPERATION_INCOMING) {
	Company ** companies;

	temp = g_strdup(_("All the companies"));
	c_util_set_code_and_name_on_combo (ULONG_MAX,temp,GTK_COMBO(combo));
	g_free(temp);
	temp = g_strdup(_("Without company"));
	c_util_set_code_and_name_on_combo (0,temp,GTK_COMBO(combo));
	g_free(temp);
	companies = m_company_find_by_code (ULONG_MAX);

	c_util_fill_combo_with_companies (combo,companies);
    } else if (type==M_LISTING_OPERATION_EXPENSE) {
	Supplier ** suppliers;

	temp = g_strdup(_("All the suppliers"));
	c_util_set_code_and_name_on_combo (ULONG_MAX,temp,GTK_COMBO(combo));
	g_free(temp);
	suppliers = m_supplier_find_all ();

	c_util_fill_combo_with_suppliers (combo,suppliers);
    } else {
	temp = g_strdup(_("All"));
	c_util_set_code_and_name_on_combo (ULONG_MAX,temp,GTK_COMBO(combo));
    }


}


static void on_delete_button_clicked(GtkButton *button, gpointer data) {
    clean_screen();
}

static void on_print_button_clicked(GtkButton *button, gpointer data) {

    if (listing_operations != NULL) {
	Listing list;
        Listing * listing = (Listing *) g_malloc0(sizeof(Listing));
        gulong    rows = 0;

	list.content_title = d_listing_operation_get_content_title ();
	list.listing_title = d_listing_operation_get_listing_title (c_util_get_code_from_combo (glade_xml_get_widget(xml,"combo_pagos_gestion_listados")));
	list.content       = d_listing_d_operation_array_to_content (listing_operations, &rows);
	list.width_of_cols = d_listing_operation_get_listing_columns_width ();
	list.alignment_of_cols = d_listing_operation_get_listing_columns_alignment ();
	list.n_rows        = rows;
	list.n_columns     = d_listing_operation_get_content_size ();

        // Fixme!!!
        // Hardcoded -> get this from a combo
	list.orientation   = 4;
	list.type_of_paper = 2;

	*listing = list;
        p_listing_wrap_print (listing);
    } else {
	c_util_open_error_window (_("Search the invoices to print"));
	}

}

static void on_search_button_clicked(GtkButton *button, gpointer data) {
    GtkWidget * vbox = glade_xml_get_widget(xml,widget_name);
    GtkWidget * window;
    GdkCursor * cursor;

    window = lookup_widget(GTK_WIDGET(vbox),"app1");
    cursor = gdk_cursor_new(GDK_WATCH);
    gdk_window_set_cursor(GTK_WIDGET(window)->window, cursor);
    gdk_cursor_destroy(cursor);

    do_search();

    cursor = gdk_cursor_new(GDK_LEFT_PTR);
    gdk_window_set_cursor(GTK_WIDGET(window)->window, cursor);
    gdk_cursor_destroy(cursor);

}


static void clean_screen() {

    GtkWidget *vbox;
    gchar *s_name =NULL;

    vbox = GTK_WIDGET(glade_xml_get_widget(xml, widget_name));

    gtk_container_foreach(GTK_CONTAINER(vbox),&c_util_clean_widget,NULL);
      s_name = u_parameter_manager_get_parameter ("tienda");
      if (s_name!=NULL) {
	  gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget (xml, "combo_entry_tienda_gestion_listados")), s_name);
      }
      g_free(s_name);


}

static void on_listing_management_press_key(GtkWidget *widget,
					   GdkEventKey *event,
					  gpointer user_data) {

    GtkWidget * vbox = glade_xml_get_widget(xml,widget_name);
    GtkWidget * window;
    GdkCursor * cursor;

    window = lookup_widget(GTK_WIDGET(vbox),"app1");
    cursor = gdk_cursor_new(GDK_WATCH);
    gdk_window_set_cursor(GTK_WIDGET(window)->window, cursor);
    gdk_cursor_destroy(cursor);

    if (event->keyval==GDK_Return)
	do_search();
                                           

    cursor = gdk_cursor_new(GDK_LEFT_PTR);
    gdk_window_set_cursor(GTK_WIDGET(window)->window, cursor);
    gdk_cursor_destroy(cursor);
}


static void do_search() {
    GtkWidget * wid =NULL;
    gulong invoice_state;
    ConditionFindListingOperation * con;
    time_t init_date;
    time_t end_date;
    GtkWidget * clist = glade_xml_get_widget(xml,"lista_pagos");
    GtkWidget * entry_combo = glade_xml_get_widget(xml,"entry_combo_cuenta");
    gint type;
    gchar ** account_numbers = NULL;
    gchar * temp_amount=NULL;
    gchar * account_number=NULL;
    gulong code_combo=ULONG_MAX;
    GdkCursor * cursor;
    GtkWidget * window;

    window = lookup_widget(GTK_WIDGET(clist),"app1");
    cursor = gdk_cursor_new(GDK_WATCH);
    gdk_window_set_cursor(GTK_WIDGET(window)->window, cursor);
    gdk_cursor_destroy(cursor);

    while (gtk_events_pending())
	gtk_main_iteration();

    con = (ConditionFindListingOperation * ) g_malloc0(sizeof(ConditionFindListingOperation));

    con -> amount = G_MINDOUBLE;
    code_combo = c_util_get_code_from_combo (glade_xml_get_widget(xml,"combo_cuenta_corriente"));
    if (code_combo!=ULONG_MAX) {
	account_number = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry_combo)));
	account_numbers = c_util_get_card_numbers (account_number);

	con -> bank_code = g_strdup(account_numbers[0]);
	con -> office_code = g_strdup(account_numbers[1]);
	con -> account_number = g_strdup(account_numbers[3]);

	g_strfreev(account_numbers);
	g_free(account_number);
    }

    wid = glade_xml_get_widget(xml,"combo_compania");
    con -> company_code = c_util_get_code_from_combo (wid);

    wid = glade_xml_get_widget(xml,"combo_estado_factura_gestion_listados");

    invoice_state = c_util_get_code_from_combo (wid);
    con -> invoice_state = invoice_state;

    wid = glade_xml_get_widget(xml,"dateedit_vencimiento_gestion_listados");
    init_date = gnome_date_edit_get_date(GNOME_DATE_EDIT(wid));
    if (init_date>-1) {
	con -> initial_done_date = (time_t *) g_malloc0(sizeof(time_t));
	*(con -> initial_done_date) = init_date;
    } else {
        con->initial_done_date=NULL;
    }

    wid = glade_xml_get_widget(xml,"dateedit_desde_gestion_listados");
    end_date = gnome_date_edit_get_date(GNOME_DATE_EDIT(wid));
    if (end_date>-1) {
	con -> ending_done_date = (time_t *) g_malloc0(sizeof(time_t));
	*(con -> ending_done_date) = end_date;
    } else {
        con->ending_done_date=NULL;
    }

    wid = glade_xml_get_widget(xml,"combo_pagos_gestion_listados");
    con->type= c_util_get_code_from_combo (wid);

    wid = glade_xml_get_widget(xml,"combo_tienda_gestion_listados");
    con->shop_code= c_util_get_code_from_combo (wid);

    wid = glade_xml_get_widget(xml,"entry_cantidad_gestion_listados");
    temp_amount = g_strdup(gtk_entry_get_text(GTK_ENTRY(wid)));
    if (strcmp(temp_amount,"")) {
	u_points_to_commas (temp_amount);
        con->amount=atof(temp_amount);
    }
    g_free(temp_amount);
    if (listing_operations!=NULL) {
	if (listing_operations[0]!=NULL) {
	    m_listing_operation_clear_array (listing_operations);
	    listing_operations =NULL;
            gtk_clist_clear(GTK_CLIST(clist));
	}
    }
    type = get_type_of_search();
    listing_operations = m_listing_operation_find_by_condition (con,type);
    fill_listing_operation_list_on_clist(GTK_CLIST(clist),listing_operations);

    if (con->initial_done_date!=NULL) g_free(con->initial_done_date);
    if (con->ending_done_date!=NULL) g_free(con->ending_done_date);

    g_free(con);

    cursor = gdk_cursor_new(GDK_LEFT_PTR);
    gdk_window_set_cursor(GTK_WIDGET(window)->window, cursor);
    gdk_cursor_destroy(cursor);

}


static void fill_payments_combo(void) {
    GtkWidget * combo=glade_xml_get_widget(xml,"combo_pagos_gestion_listados");
    gchar * temp;

    temp = g_strdup(_("All the types of payments"));
    c_util_set_code_and_name_on_combo (ULONG_MAX,temp,GTK_COMBO(combo));
    g_free(temp);
    temp = g_strdup(D_PAYMENT_MESSAGE_TRANSFER);
    c_util_set_code_and_name_on_combo (D_PAYMENT_TRANSFER_CODE,temp,GTK_COMBO(combo));
    g_free(temp);
    temp = g_strdup(D_PAYMENT_MESSAGE_CHEQUE);
    c_util_set_code_and_name_on_combo (D_PAYMENT_CHEQUE,temp,GTK_COMBO(combo));
    g_free(temp);
    temp = g_strdup(D_PAYMENT_MESSAGE_DIRECT_DEBIT);
    c_util_set_code_and_name_on_combo (D_PAYMENT_DIRECT_DEBIT_CODE,temp,GTK_COMBO(combo));
    g_free(temp);
    temp = g_strdup(D_PAYMENT_MESSAGE_CREDIT_CARD);
    c_util_set_code_and_name_on_combo (D_PAYMENT_CREDIT_CARD,temp,GTK_COMBO(combo));
    g_free(temp);


}

static void fill_accounts_combo(void) {
    GtkWidget * combo;
    Account ** accounts_of_parent_company=m_account_find_all ();
    gint index=0;

    combo = glade_xml_get_widget (xml, "combo_cuenta_corriente");
    c_util_clean_code_and_name_combo (combo);
    c_util_set_code_and_name_on_combo (ULONG_MAX,_("All the current accounts of the enterprise"),GTK_COMBO(combo));

    if (accounts_of_parent_company!=NULL) {
	while (accounts_of_parent_company[index]!=NULL) {
	    gchar * complete_account_number=NULL;
	    Account * account = accounts_of_parent_company[index];

            complete_account_number = d_account_get_complete_number (account);
	    c_util_set_code_and_name_on_combo (index,complete_account_number,GTK_COMBO(combo));
	    g_free(complete_account_number);
	    index++;
	}

    }
}
static void on_radiobutton_ingresos_toggled(GtkToggleButton *togglebutton,
					    gpointer user_data) {
    set_window_values(M_LISTING_OPERATION_INCOMING);
}
static void on_radiobutton_gastos_toggled(GtkToggleButton *togglebutton,
					  gpointer user_data) {
    set_window_values(M_LISTING_OPERATION_EXPENSE);
}
static void on_radiobutton_ambos_toggled(GtkToggleButton *togglebutton,
					 gpointer user_data) {
    set_window_values(M_LISTING_OPERATION_BOTH);
}

static void set_window_values(gint type) {
    fill_invoice_state_combo(type);
    fill_general_enterprise_combo(type);
}

static gint get_type_of_search() {

    GtkWidget * radiobutton ;
    gint type = M_LISTING_OPERATION_BOTH;
    radiobutton = glade_xml_get_widget(xml,"radiobutton_ingresos");
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(radiobutton))) {
        type = M_LISTING_OPERATION_INCOMING;
    } else {
	radiobutton = glade_xml_get_widget(xml,"radiobutton_gastos");
	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(radiobutton))) {
            type = M_LISTING_OPERATION_EXPENSE;
	} 
    }

    return type;

}
