
/* listing_window_control.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#define P_MAXIMUN_NUMBER_OF_COLUMNS 8

#include <gnome.h>
#include <glade/glade.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <glib.h>
#include <time.h>


#include "delivery_payment_model.h"
#include "util.h"
#include "parameter_manager.h"
#include "control_util.h"

/*
#include "enterprise_model.h"
#include "accounting_period_model.h"
*/

#include "../interface/printing/listing-print-wrapper.h"


static GtkWidget *window = NULL;
static GladeXML *xml = NULL;
static gchar widget_name[]="general_listing_window";
static gchar url[]="/src/interface/glade/general_listing_window.glade";
static void (*ok_callback)(gpointer selected, gint len);

static void on_cancel_clicked(GtkButton *button, gpointer data);
static void on_accept_clicked(GtkButton *button, gpointer data);

static void inicialize_clist(gint);
void c_listing_fill_generic_list(Listing *listado, gint rows);
static void clean_screen();

 
GtkWidget *c_listing_open_window(char *message, gpointer callback) {
    
    gchar * directory = g_strconcat(AA_DATA_DIR,url,NULL);
    ok_callback = callback;
        
    if (window == NULL) {
	
	GtkWidget *message_entry;
	gint * position = (gint *) g_malloc0(sizeof(gint));
	
	xml = glade_xml_new (directory, widget_name);
	window = glade_xml_get_widget (xml, widget_name);
	message_entry = glade_xml_get_widget (xml, "message");
	gtk_editable_insert_text(GTK_EDITABLE(message_entry), message, strlen(message), position);
	    
	glade_xml_signal_connect (xml, "on_cancel_clicked", GTK_SIGNAL_FUNC (on_cancel_clicked));
	glade_xml_signal_connect (xml, "on_accept_clicked", GTK_SIGNAL_FUNC (on_accept_clicked));
    }

    inicialize_clist(P_MAXIMUN_NUMBER_OF_COLUMNS);   
    g_free(directory);

    return window;
}


static void inicialize_clist(gint num_cols) {
    GtkWidget *list;
    gint       column;

    list = glade_xml_get_widget(xml,"clist_general_listing");

    gtk_clist_clear(GTK_CLIST(list));

    for(column=0; column < P_MAXIMUN_NUMBER_OF_COLUMNS; column++) {
    
	gtk_clist_set_column_title((GtkCList *)list, column, "");
	
	if (column < num_cols) 
		gtk_clist_set_column_visibility((GtkCList *)list, column, TRUE);
	else
		gtk_clist_set_column_visibility((GtkCList *)list, column, FALSE);
    }
}


void c_listing_fill_generic_list(Listing *listado, gint rows) {

    gint i;
    gint column = 0;
    gchar * temp;
    gchar **data;
    GtkWidget *list = glade_xml_get_widget(xml,"clist_general_listing");
    gchar *empty_row[]={"","","","","","","",""};

	
    if (listado!=NULL) {
    
    	inicialize_clist(listado->n_columns);
	
	if (listado->content_title!=NULL) {
	    
	    for(column=0; listado->content_title[column] != NULL; column++){
		temp = g_strdup_printf("%s", listado->content_title[column]);
		gtk_clist_set_column_title((GtkCList *)list, column, temp);
		gtk_clist_set_column_width ((GtkCList *)list,column, listado->width_of_cols[column]*7);
		g_free(temp);
	    }
	}

	if (listado->content!=NULL) {
	   
	    for (i=0; i < rows; i++) {
		
		if (i==30) gtk_clist_freeze ((GtkCList *) list);

		column = 0;
		data = listado->content[i];
		gtk_clist_append ((GtkCList *) list, empty_row);

		for (column=0; data[column]!=NULL; column++) {
		    temp = g_strdup(data[column]);
		    gtk_clist_set_text((GtkCList *) list, i, column, temp);
		    g_free(temp);
		}
	    }
	    
	    gtk_clist_thaw ((GtkCList *) list);

	}
    }
}


static void on_accept_clicked(GtkButton *button, gpointer data) {

	GtkWidget *list = glade_xml_get_widget(xml,"clist_general_listing");	
	GArray * selected = c_util_get_selected_rows_from_clist (GTK_CLIST(list));
        char **ids = (gchar **) g_malloc(sizeof(gchar *) * selected->len);
	gint index = 0;
	
	while (index < selected->len) {
	
		gtk_clist_get_text(GTK_CLIST(list), g_array_index(selected, gulong, index), 0, ids);
		index++;
	}
	
	ok_callback(ids, selected->len);
		
	g_array_free(selected, TRUE);
	g_free(ids);
	clean_screen();			
}


static void on_cancel_clicked(GtkButton *button, gpointer data) {
	
	clean_screen();
}


static void clean_screen() {

    GtkWidget *container = GTK_WIDGET(glade_xml_get_widget(xml, widget_name));

    gtk_container_foreach(GTK_CONTAINER(container),&c_util_clean_widget,NULL);

    if (window != NULL) {
      	
	gtk_widget_destroy(window);
      	window=NULL; 
    }    
}

