
/* order_control.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include <gnome.h>
#include <glade/glade.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <glib.h>
#include <time.h>
#include <stdlib.h>

#include "a-shortcut.h"
#include "support.h"
#include "stock.h"
#include "util.h"
#include "order_line.h"
#include "order_model.h"
#include "enterprise_model.h"
#include "warehouse_model.h"
#include "order_line_model.h"
#include "find_pieces_model.h"

#include "order_control.h"
#include "clients_control.h"
#include "update_entry_control.h"
#include "find_stocked_pieces_control.h"
#include <../interface/printing/order-print-wrapper.h>


#define C_EUROCODE_COLUMN          0
#define C_FAMILY_COLUMN           1
#define C_DESCRIPTION_COLUMN       2
#define C_AMOUNT_COLUMN          3
#define C_WIDTH_COLUMN             4
#define C_LONG_COLUMN             5
#define C_ALL_ENTERPRISES              0
#define C_ALL_WAREHOUSES_OF_ENTERPRISE 0


#ifndef IS_SHOP
static GtkWidget *order=NULL;
static GladeXML *xml = NULL;

static gchar url[]="/src/interface/glade/store_view.glade";
static gchar widget_name[]="vbox_pedidos";

GList  *order_line_list = NULL;
gint    updated_row = -1;
gulong  warehouse_code  = ULONG_MAX;
gulong  enterprise_code = ULONG_MAX;
gulong number_of_lines=ULONG_MAX;
static  Participant *supplier = NULL;
Order  *order_loaded = NULL;

static void load_supplier_order(Participant *participant);
static void on_search_supplier_by_order_button_clicked(GtkButton *button,
							     gpointer data);
static void on_clean_orders_button_clicked(GtkButton *button,
					      gpointer data);
static void on_insert_piece_button_clicked(GtkButton *button,
						    gpointer data);
            
static void on_delete_piece_button_clicked(GtkButton *button,
						    gpointer data);
static void on_piece_quantity_clicked(GtkButton *button,
						    gpointer data);
static void on_save_order_button_clicked(GtkButton *button,
					     gpointer data);
static void on_print_order_button_clicked(GtkButton *button,
					      gpointer data);
static void set_value_in_list(GtkCList *list, gint row, gint column, gdouble value);
static void set_value_in_order_line(gint row, gdouble value);
static void set_quantity(gpointer value);
static gdouble get_quantity(gint index);
static gint obtain_pieces_quantity(GArray * selected, gdouble *amounts);
static void load_order_lines_list(void);
static OrderLine **array_from_order_lines_list(GList *list);
static void clean_screen(void);
static void free_objects(void);
#endif

GtkWidget *c_order_open_window(GtkWidget *app1, GtkWidget *vbox, Participant *supplier_param) {

#ifndef IS_SHOP
    gchar * directory = g_strconcat(AA_DATA_DIR,url,NULL);

    if (order==NULL) {
	GtkWidget *dialog;

	xml = glade_xml_new (directory, widget_name);
	dialog = glade_xml_get_widget (xml, widget_name);


	gtk_box_pack_start (GTK_BOX (vbox), dialog, TRUE, TRUE, 0);

	glade_xml_signal_connect (xml, "on_button-buscar-buscarproveedor-pedidos_clicked",
				  GTK_SIGNAL_FUNC (on_search_supplier_by_order_button_clicked));
	glade_xml_signal_connect (xml, "on_button-insertarpieza-pedidos_clicked",
				  GTK_SIGNAL_FUNC (on_insert_piece_button_clicked));
	glade_xml_signal_connect (xml, "on_button-eliminarpieza-pedidos_clicked",
				  GTK_SIGNAL_FUNC (on_delete_piece_button_clicked));
	glade_xml_signal_connect (xml, "on_button-unidadespieza-pedidos_clicked",
				  GTK_SIGNAL_FUNC (on_piece_quantity_clicked));
	glade_xml_signal_connect (xml, "on_button-limpiar-pedidos_clicked",
				  GTK_SIGNAL_FUNC (on_clean_orders_button_clicked));
	glade_xml_signal_connect (xml, "on_boton-guardar-pedidos_clicked",
				  GTK_SIGNAL_FUNC (on_save_order_button_clicked));
	glade_xml_signal_connect (xml, "on_boton-imprimir-pedidos_clicked",
				  GTK_SIGNAL_FUNC (on_print_order_button_clicked));
                                                   
	order =dialog;
    }

    if (supplier_param != NULL) {
        load_supplier_order(supplier_param);
    }

    g_free(directory);

    return order;
#else
    return NULL;
#endif

}

void c_order_load(Order *order) {
#ifndef IS_SHOP
    gulong e_code = d_order_get_enterprise_code (order);
    gulong w_code  = d_order_get_warehouse_code (order);
    GList *element = NULL;

    warehouse_code  = w_code;
    enterprise_code = e_code;

    d_order_clear (order_loaded);
    order_loaded = NULL;
    order_loaded = d_order_duplicate (order);

    if (order_line_list!=NULL) {

	element = g_list_first (order_line_list);
	while (element != NULL){
	    d_order_line_clear (element->data);
	    element->data = NULL;
	    element = g_list_next(element);
	}
	if (order_line_list!=NULL) {
	    g_list_free(order_line_list);
	    order_line_list=NULL;
	}
    }

    m_order_line_get_lines_from_order (&order_line_list, order_loaded);
    load_order_lines_list();
#endif
}

void c_order_load_pieces(gulong *pieces_ids, gulong *number,
		       gulong e_code, gulong w_code) {

#ifndef IS_SHOP
    if((warehouse_code!=ULONG_MAX)&&(enterprise_code!=ULONG_MAX)
       && ((warehouse_code!=w_code) || (enterprise_code!=e_code))) {
	c_util_open_error_window (_("Select the same warehouse for the entire order"));
    } else {
	warehouse_code=w_code;
	enterprise_code=e_code;
	m_order_line_get_lines_from_pieces (&order_line_list, pieces_ids, number,
				     e_code, w_code);
	load_order_lines_list();
    }
#endif
}


#ifndef IS_SHOP
static void load_supplier_order(Participant *participant) {
    gchar *enterprise_legal_id = d_participant_get_legal_id (participant);
    gchar *nick = d_participant_get_nick (participant);
    gchar *phone1 = d_participant_get_phone (participant);
    gchar *phone2 = d_participant_get_phone2 (participant);
    gchar *fax       = d_participant_get_fax (participant);
    gchar *email = d_participant_get_email (participant);

    if (supplier != NULL){
	d_participant_clear (supplier);
	supplier = NULL;
    }

    if (enterprise_legal_id != NULL) {
	GtkWidget *enterprise_legal_id_entry =glade_xml_get_widget (xml, "entry-dni-pedidos");
	gtk_entry_set_text (GTK_ENTRY(enterprise_legal_id_entry),enterprise_legal_id);
	g_free(enterprise_legal_id);
    }

    if (nick != NULL) {
	GtkWidget *nick_entry =glade_xml_get_widget (xml, "entry-nombre-pedidos");
	gtk_entry_set_text (GTK_ENTRY(nick_entry),nick);
	g_free(nick);
    }

    if (phone1 != NULL) {
	GtkWidget *phone1_entry =glade_xml_get_widget (xml, "entry-telefono1-pedidos");
	gtk_entry_set_text (GTK_ENTRY(phone1_entry),phone1);
	g_free(phone1);
    }

    if (phone2 != NULL) {
	GtkWidget *phone2_entry =glade_xml_get_widget (xml, "entry-telefono2-pedidos");
	gtk_entry_set_text (GTK_ENTRY(phone2_entry),phone2);
	g_free(phone2);
    }

    if (fax != NULL) {
	GtkWidget *fax_entry =glade_xml_get_widget (xml, "entry-fax-pedidos");
	gtk_entry_set_text (GTK_ENTRY(fax_entry),fax);
	g_free(fax);
    }

    if (email != NULL) {
	GtkWidget *email_entry =glade_xml_get_widget (xml, "entry-email-pedidos");
	gtk_entry_set_text (GTK_ENTRY(email_entry),email);
	g_free(email);
    }

    supplier = d_participant_duplicate (participant);

}

static void on_search_supplier_by_order_button_clicked(GtkButton *button,
							     gpointer data) {
    GtkWidget * app1 = lookup_widget(GTK_WIDGET(button), "app1");
    GtkWidget * vbox;

    if (supplier!=NULL){
	vbox = GTK_WIDGET(glade_xml_get_widget(xml, "frame-proveedor-pedidos"));

	gtk_container_foreach(GTK_CONTAINER(vbox),&c_util_clean_widget,NULL);

	d_participant_clear (supplier);

	supplier = NULL;
    }

    vbox = lookup_widget(GTK_WIDGET(button), "main_container");
    c_clients_control_open_supplier (app1, vbox, c_order_open_window,2,3);

    i_shortcut_change_dialog (GTK_WIDGET(button),0,0);

}


static void on_insert_piece_button_clicked(GtkButton *button,
						    gpointer data) {
    GtkWidget *list = glade_xml_get_widget(xml,"clist-buscarpiezas-pedido");
    gchar *enterprise = NULL;
    gchar *warehouse = NULL;

    gtk_clist_unselect_all(GTK_CLIST(list));

    i_shortcut_change_dialog (GTK_WIDGET(list),2,1);

    if ((enterprise_code!=0)&& (enterprise_code!=ULONG_MAX)
	&& (warehouse_code!=0)&& (warehouse_code!=ULONG_MAX)) {
	enterprise = m_enterprise_get_name_from_code (enterprise_code);
	warehouse  = m_warehouse_get_name_from_code (warehouse_code);

	c_find_stocked_pieces_set_combos(enterprise, warehouse);

	g_free(enterprise);
	g_free(warehouse);
    }
}

static void on_delete_piece_button_clicked(GtkButton *button,
						    gpointer data) {


    GtkWidget * list = glade_xml_get_widget(xml,"clist-buscarpiezas-pedido");
    GArray *selected = c_util_get_selected_rows_from_clist (GTK_CLIST(list));
    gint line = selected->len-1;

    while (line != -1) {
	gint nindex = g_array_index(selected,gint,line);
	gpointer data = g_list_nth_data (order_line_list, nindex);

	gtk_clist_remove(GTK_CLIST(list),nindex);

	order_line_list = g_list_remove(order_line_list, data);

	line--;
    }

    if ((selected->len-1) != -1){
	load_order_lines_list();
    }

    g_array_free(selected, FALSE);

}

static void on_piece_quantity_clicked(GtkButton *button,
						    gpointer data) {

    GtkWidget *app1 = lookup_widget(GTK_WIDGET(button), "app1");
    gchar     *temp = g_strdup(_("Units"));
    GtkCList  *list  = GTK_CLIST(glade_xml_get_widget (xml, "clist-buscarpiezas-pedido"));
    GArray    *selected = c_util_get_selected_rows_from_clist (list);
    gint       index = 0;
    if (selected->len==1) {
	updated_row = g_array_index(selected,gint,index);
	c_update_entry_open_window(app1,temp,C_GDOUBLE_TYPE, set_quantity);
    } else {
	if (selected->len>1) {
	    c_util_open_error_window (_("Select one piece"));
	} else {
	    c_util_open_error_window (_("Select at least one piece"));
	}
    }
    g_free(temp);
    g_array_free(selected, FALSE);
}

static void set_value_in_list(GtkCList *list, gint row, gint column, gdouble value) {

    gchar *temp = g_strdup_printf("%.3f",value);
    gtk_clist_set_text(list, row, column, temp);
    g_free(temp);

}

static void set_value_in_order_line(gint row, gdouble value) {

    gchar *temp = g_strdup_printf("%.0f",value);
    gpointer data = g_list_nth_data (order_line_list, row);
    OrderLine *line = (OrderLine *)data;
    d_order_line_set_quantity (line, value);
    g_free(temp);

}

static void set_quantity(gpointer value) {
    gdouble *gulong_value = value;
    GtkCList *list = GTK_CLIST(glade_xml_get_widget(xml,"clist-buscarpiezas-pedido"));

    if (*gulong_value>=0) {
	set_value_in_list(list, updated_row, C_AMOUNT_COLUMN, *gulong_value);
	set_value_in_order_line(updated_row, *gulong_value);
    }
    updated_row = -1;
}

static gdouble get_quantity(gint index) {

    // Do not free text, is a reference (see gtkclist code)
    gchar **text = (gchar **) g_malloc(sizeof(gchar*)*1);
    gdouble amount = G_MINDOUBLE;
    GtkCList *clist = GTK_CLIST(glade_xml_get_widget(xml,"clist-buscarpiezas-pedido"));
    gtk_clist_get_text(clist, index, C_AMOUNT_COLUMN, text);
    //amount = strtoul(*text, NULL, 10);
    amount = atof(*text);

    return amount;

}

static gint obtain_pieces_quantity(GArray * selected, gdouble *amounts) {

    gint index=0;
    gint indexOfPiece=0;

    while (index<selected->len) {
	indexOfPiece = g_array_index(selected,gint,index);

	amounts[index] = get_quantity(indexOfPiece);
	index++;
    }
    amounts[index]=ULONG_MAX;
    return index;
}


static void on_save_order_button_clicked(GtkButton *button,
					     gpointer data) {
    GtkCList  *list  = GTK_CLIST(glade_xml_get_widget (xml, "clist-buscarpiezas-pedido"));
    GArray    *rows = c_util_get_rows_from_clist (list);
    Order     *order    = NULL;
    gulong     number, participant_code;
    gulong     *pieces_ids = NULL;
    gdouble    *amounts = NULL;
    gint       index = 0;
    OrderLine **order_lines = NULL;
    gboolean   correct = FALSE;

    if (rows->len>0) {
	if (supplier != NULL) {
	    if (order_loaded!=NULL) {
		order = order_loaded;
	    } else {
		order   = d_order_create_empty ();
	    }
	    number = (gulong)(rows->len+1);
            number_of_lines = number;
	    pieces_ids = (gulong *) g_malloc(sizeof(gulong)*(rows->len+1));
	    amounts    = (gdouble *) g_malloc(sizeof(gdouble)*(rows->len+1));

	    order_lines = array_from_order_lines_list(order_line_list);

	    obtain_pieces_quantity(rows, amounts);
	    d_order_set_enterprise_code (order, enterprise_code);

	    d_order_set_warehouse_code (order, warehouse_code);

	    while (order_lines[index]!=NULL) {
		d_order_line_set_quantity ((OrderLine*)order_lines[index],
					amounts[index]);
		index++;
	    }

	    participant_code =  d_participant_get_code (supplier);
	    d_order_set_supplier_code (order, participant_code);

	    d_order_set_order_lines (order, order_lines);

	    if (order_loaded!=NULL) {
		correct = m_order_update (order);
	    } else {
		correct = m_order_insert (order);
		g_free(order);
                order = NULL;
	    }

	    if(!correct) {
		c_util_open_error_window (_("Order update hasn't been done successfully. There has been a problem in the database.\n Try it again"));
	    }

	    clean_screen();

	    g_free(pieces_ids);
	    g_free(amounts);
	    g_array_free(rows, FALSE);

	} else {
	    c_util_open_error_window (_("Select a supplier to make the order"));
	}
    } else {
	c_util_open_error_window (_("Select a piece to make the order"));
    }
}

static void on_print_order_button_clicked(GtkButton *button,
					      gpointer data) {
    if (order_loaded!=NULL) {
	if (number_of_lines!=ULONG_MAX) {

	    if (d_order_get_code (order_loaded)!=ULONG_MAX) {
		p_order_wrap_print (order_loaded,number_of_lines);
	    } else {
		c_util_open_error_window (_("You must save the order"));
	    }
	} else {
            c_util_open_error_window (_("Wrong number of lines"));
	}
    } else {
        c_util_open_error_window (_("Empty order or not saved"));
    }

}

static void load_order_lines_list() {

    if (order_line_list!=NULL) {
	GtkWidget * list = glade_xml_get_widget (xml, "clist-buscarpiezas-pedido");
	gchar *empty_row[]={"","","","","",""};
	GList * element = g_list_first(order_line_list);
	GArray    *rows; 
	gint index = 0;
        gdouble quantity = 0;
	gchar * temp;

	gtk_clist_clear(GTK_CLIST(list));
	
	while (element != NULL) {
	    OrderLine *line =(OrderLine *) element->data;

	    if (index==30)
		gtk_clist_freeze ((GtkCList *) list);

	    gtk_clist_append ((GtkCList *) list,empty_row);

	    if (d_product_get_is_piece ((Product *)d_order_line_get_piece (line))) {
		gtk_clist_set_text((GtkCList *) list, index, C_EUROCODE_COLUMN, d_piece_get_euroCode (d_order_line_get_piece (line)));

		temp = g_strdup_printf("%d",d_piece_get_width (d_order_line_get_piece (line)));
		gtk_clist_set_text((GtkCList *) list, index, C_WIDTH_COLUMN, temp);
		g_free(temp);

		temp = g_strdup_printf("%d",d_piece_get_length (d_order_line_get_piece (line)));
		gtk_clist_set_text((GtkCList *) list, index, C_LONG_COLUMN, temp);
		g_free(temp);

	    }

            quantity = d_order_line_get_quantity (line);
	    if (quantity != G_MINDOUBLE) {
		temp = g_strdup_printf("%.3f",quantity);
		gtk_clist_set_text((GtkCList *) list, index, C_AMOUNT_COLUMN, temp);
		g_free(temp);
	    }

            temp = d_product_get_family_code ((Product *)d_order_line_get_piece (line));
	    gtk_clist_set_text((GtkCList *) list, index, C_FAMILY_COLUMN, temp);
	    g_free(temp);

            temp = d_product_get_description ((Product *)d_order_line_get_piece (line));
	    gtk_clist_set_text((GtkCList *) list, index, C_DESCRIPTION_COLUMN, temp);
            g_free(temp);

	    element = g_list_next(element);
	    index++;
	}
	
	gtk_clist_thaw ((GtkCList *) list);
        rows = c_util_get_rows_from_clist ((GtkCList *) list);
        number_of_lines = (gulong)(rows->len+1);
    }

}

static OrderLine **array_from_order_lines_list(GList *list) {

    OrderLine **order_lines = NULL;

    if (list != NULL){
	GList *element = g_list_last(list);
	gint number_elements = g_list_position(list, element);

	guint line;

	order_lines = (OrderLine **) g_malloc0(sizeof(OrderLine*)*(number_elements+2));

	order_lines[number_elements+1] = NULL;

	for (line = 0; line <= number_elements; line ++){
	    order_lines[line] = g_list_nth_data(list,line);
	}

    }

    return order_lines;
}

static void on_clean_orders_button_clicked(GtkButton *button,
					      gpointer data) {
    clean_screen();
    free_objects();
}

static void clean_screen() {

    GtkWidget *vbox = GTK_WIDGET(glade_xml_get_widget(xml, widget_name));
    GtkWidget *list = glade_xml_get_widget(xml,"clist-buscarpiezas-pedido");

    free_objects();
    if (list!=NULL) {
	gtk_clist_clear ((GtkCList *) list);
    }

    gtk_container_foreach(GTK_CONTAINER(vbox),&c_util_clean_widget,NULL);
}

static void free_objects() {

    number_of_lines=ULONG_MAX;
    if (supplier!=NULL) {
	d_participant_clear (supplier);
        supplier = NULL;
    }

    if (order_loaded!=NULL) {
	d_order_clear (order_loaded);
	order_loaded = NULL;
    }


    warehouse_code  = ULONG_MAX;
    enterprise_code = ULONG_MAX;

    if (order_line_list!=NULL) {
	g_list_free(order_line_list);
	order_line_list=NULL;
    }

}
            

#endif


