/* payment_control.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include <gnome.h>
#include <glade/glade.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <glib.h>
#include <stdlib.h>

#include "a-shortcut.h"
#include "support.h"
#include "control_util.h"
#include "payment.h"
#include "account.h"
#include "payment_control.h"
#include "payment_model.h"
#include "account_model.h"
#include "charge_invoice_control.h"
#include "insert_operation_control.h"
#include "insert_cash_operation_control.h"

static GtkWidget *payments=NULL;
static Payment * payment_to_manage;
static GladeXML *xml;
static void (*function_payment_to_call)(gpointer value);
static gboolean managing;
static gchar widget_name[]="gestion_pagos";
static gchar url[]="/src/interface/glade/invoice_view.glade";

static void on_cancel_button_clicked(GtkButton *button, gpointer user_data);
static void on_accept_button_clicked(GtkButton *button,  gpointer user_data);
static void on_pay_button_clicked(GtkButton *button,gpointer user_data);
static void on_exit_button_clicked(GtkButton *button,gpointer user_data);
static void payment_mode_combo_init(void);
static void payment_type_combo_init(void);
static void clean_screen(void);
static void on_payment_mode_combo_entry_changed(GtkEntry *entry,
					     gpointer user_data);
static void on_payment_means_combo_entry_changed(GtkEntry *entry,
					      gpointer user_data);
static void restore_to_initial_purchase(gboolean no_sensitive);
static void on_days_money_order_entry_changed(GtkEntry *entry,
				 gpointer user_data);
static void set_dates_on_payment(void);
static void on_payment_window_destroy(GtkWidget * widget,gpointer user_data);
static void set_button_visible(void);
static void fill_payment_on_screen(void);
static gboolean set_payment_from_screen(Payment * payment_to_fill);
static void get_code_from_payment(gpointer value);
static void generate_operation(void);
static void pay(gpointer value);
static void fill_account_number_combo(Payment * payment);

GtkWidget *c_payment_open_window(GtkWidget *app1,Payment * new_payment,gpointer f_to_call,gboolean is_new,gboolean open_managing) {
    gchar * directory = g_strconcat(AA_DATA_DIR,url,NULL);

    if (payments==NULL)
    {
	GtkWidget *dialog;

	xml = glade_xml_new (directory, widget_name);
	dialog = glade_xml_get_widget (xml, widget_name);


	glade_xml_signal_connect (xml, "on_button_cancelar_gestion_pagos_clicked",
				  GTK_SIGNAL_FUNC (on_cancel_button_clicked));
	glade_xml_signal_connect (xml, "on_button_aceptar_gestion_pagos_clicked",
				  GTK_SIGNAL_FUNC (on_accept_button_clicked));
	glade_xml_signal_connect (xml, "on_button_pagar_gestion_pagos_clicked",
				  GTK_SIGNAL_FUNC (on_pay_button_clicked));
	glade_xml_signal_connect (xml, "on_button_salir_gestion_pagos_clicked",
				  GTK_SIGNAL_FUNC (on_exit_button_clicked));
	glade_xml_signal_connect (xml, "on_combo_entry_modo_pago_gestion_pagos_changed",
			          GTK_SIGNAL_FUNC (on_payment_mode_combo_entry_changed));
	glade_xml_signal_connect (xml, "on_combo_entry_medio_pago_gestion_pagos_changed",
			          GTK_SIGNAL_FUNC (on_payment_means_combo_entry_changed));
	glade_xml_signal_connect (xml, "on_dias_giro_gestion_pagos_changed",
				  GTK_SIGNAL_FUNC (on_days_money_order_entry_changed));
	glade_xml_signal_connect (xml, "on_gestion_pagos_destroy",
				  GTK_SIGNAL_FUNC (on_payment_window_destroy));
	payment_mode_combo_init();
	payment_type_combo_init();
	payments=dialog;
    } else {
	if (payment_to_manage!=NULL)
            /* DOESN'T FREE THE PAYMENT*/
	    payment_to_manage = NULL;
    }
    if (is_new!=FALSE) {
	clean_screen();
	restore_to_initial_purchase(TRUE);
	
    }

    fill_account_number_combo(new_payment);
    payment_to_manage = new_payment;
    function_payment_to_call = f_to_call;
    managing = open_managing;
    if (payment_to_manage!=NULL) {
	if ((d_payment_get_entry_percentage (payment_to_manage)!=-1) &&
            (d_payment_get_number_expiries (payment_to_manage)!=-1))
	    fill_payment_on_screen();
    }
    g_free(directory);
    return payments;
}


static void on_cancel_button_clicked(GtkButton *button,gpointer user_data) {
    gboolean * from_managing = (gboolean *) g_malloc0(sizeof(gboolean));
    payment_to_manage = NULL;
    * from_managing = managing;
    function_payment_to_call((gpointer) from_managing);
    gtk_widget_destroy(payments);
    payments = NULL;
}

static void on_payment_window_destroy(GtkWidget * widget,gpointer user_data) {
    payment_to_manage = NULL;
    gtk_widget_destroy(payments);
    payments = NULL;
}


static void on_accept_button_clicked(GtkButton *button,gpointer user_data) {

    gboolean done = FALSE;
    gulong code;
    done = set_payment_from_screen(payment_to_manage);

    if (done ==TRUE) {
	if (d_payment_get_payment_code (payment_to_manage)!=0) {
	    code = m_payment_update (payment_to_manage);
            d_payment_set_payment_code (payment_to_manage,code);
	}
    } else {
	gboolean * from_managing = (gboolean *) g_malloc0(sizeof(gboolean));
	payment_to_manage = NULL;
	* from_managing = managing;
        function_payment_to_call((gpointer) from_managing);
    }

    gtk_widget_destroy(payments);
    payments = NULL;
}

static void pay(gpointer value) {
    GtkWidget * done_payments_spin = glade_xml_get_widget(xml,"spinbutton_realizados_gestion_pagos");
    GtkWidget * next_done_dateedit = glade_xml_get_widget(xml,"dateedit_fecha_siguiente_gestion_pagos");
    GtkWidget * button = glade_xml_get_widget(xml,"button_pagado_gestion_pagos");
    gint pagos_realizados;

    gint expiries;
    struct tm * time_tm = (struct tm *) g_malloc0(sizeof(struct tm));
    time_t * time_aux = (time_t *) g_malloc0(sizeof(time_t));

    
    pagos_realizados = d_payment_get_achieved_payments (payment_to_manage);
    expiries = d_payment_get_number_expiries (payment_to_manage);
    if (expiries>pagos_realizados) {
	if (expiries == 1) {
	    if (pagos_realizados == 0) {
		pagos_realizados++;

		gtk_spin_button_set_value(GTK_SPIN_BUTTON(done_payments_spin),1.0*pagos_realizados);
                gtk_spin_button_update(GTK_SPIN_BUTTON(done_payments_spin));
		gtk_widget_set_sensitive(GTK_WIDGET(button),FALSE);
	    }
	} else {
	    if (pagos_realizados==(expiries-1)) {
		pagos_realizados++;
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(done_payments_spin),1.0*pagos_realizados);
                gtk_spin_button_update(GTK_SPIN_BUTTON(done_payments_spin));
		gtk_widget_set_sensitive(GTK_WIDGET(button),FALSE);
	    } else if (pagos_realizados<expiries){
		GDate * date;
		gint days;
                *time_aux = gnome_date_edit_get_date(GNOME_DATE_EDIT(next_done_dateedit));
		pagos_realizados++;
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(done_payments_spin),1.0*pagos_realizados);
		gtk_spin_button_update(GTK_SPIN_BUTTON(done_payments_spin));

		time_tm = localtime(time_aux);
		date = g_date_new_dmy(time_tm->tm_mday,time_tm->tm_mon+1,time_tm->tm_year+1900);
		days = d_payment_get_interval_days (payment_to_manage);
		g_date_add_days(date,days);

		time_tm ->tm_mday = g_date_day(date);
		time_tm ->tm_mon= g_date_month(date)-1;
		time_tm ->tm_year = g_date_year(date)-1900;
		*time_aux = mktime(time_tm);
		gnome_date_edit_set_time(GNOME_DATE_EDIT(next_done_dateedit),*time_aux);
		d_payment_set_init_date (payment_to_manage,*time_aux);

	    }
	}
	if (pagos_realizados == expiries)
	    c_charge_invoice_is_paid_toggle_button_change_state (TRUE);

	d_payment_set_achieved_payments (payment_to_manage,pagos_realizados);
	m_payment_update (payment_to_manage);
    } else {
        c_util_open_info_window (_("Paid invoice"));
    }
    gtk_widget_destroy(payments);
    payments=NULL;

}
static void on_pay_button_clicked(GtkButton *button,gpointer user_data) {
    generate_operation();
}


static void payment_mode_combo_init() {
    GtkWidget * combo;

    combo = glade_xml_get_widget (xml, "combo_modo_pago_gestion_pagos");

    c_util_set_code_and_name_on_combo (D_PAYMENT_AT_MOMENT,D_PAYMENT_MESSAGE_AT_MOMENT,GTK_COMBO(combo));
    c_util_set_code_and_name_on_combo (D_PAYMENT_HIRE_PURCHASE,D_PAYMENT_MESSAGE_HIRE_PURCHASE,GTK_COMBO(combo));
    c_util_set_code_and_name_on_combo (D_PAYMENT_MONEY_ORDER_AT_END_OF_MONTH,D_PAYMENT_MESSAGE_MONEY_ORDER_AT_END_OF_MONTH,GTK_COMBO(combo));
    c_util_set_code_and_name_on_combo (D_PAYMENT_MONEY_ORDER,D_PAYMENT_MESSAGE_MONEY_ORDER,GTK_COMBO(combo));
}
static void payment_type_combo_init() {
    GtkWidget * combo;

    combo = glade_xml_get_widget (xml, "combo_medio_pago_gestion_pagos");

    c_util_set_code_and_name_on_combo (D_PAYMENT_TRANSFER_CODE,D_PAYMENT_MESSAGE_TRANSFER,GTK_COMBO(combo));
    c_util_set_code_and_name_on_combo (D_PAYMENT_IN_CASH,D_PAYMENT_MESSAGE_IN_CASH,GTK_COMBO(combo));
    c_util_set_code_and_name_on_combo (D_PAYMENT_CHEQUE,D_PAYMENT_MESSAGE_CHEQUE,GTK_COMBO(combo));
    c_util_set_code_and_name_on_combo (D_PAYMENT_CREDIT_CARD,D_PAYMENT_MESSAGE_CREDIT_CARD,GTK_COMBO(combo));
    c_util_set_code_and_name_on_combo (D_PAYMENT_DIRECT_DEBIT_CODE,D_PAYMENT_MESSAGE_DIRECT_DEBIT,GTK_COMBO(combo));
}

static void on_payment_mode_combo_entry_changed(GtkEntry *entry,
							   gpointer user_data) {

    GtkWidget * combo = glade_xml_get_widget(xml,"combo_modo_pago_gestion_pagos");
    GtkWidget * money_order_days_label = glade_xml_get_widget(xml,"label_dias_giro_gestion_pagos");
    GtkWidget * spinButton = glade_xml_get_widget(xml,"dias_giro_gestion_pagos");
    GtkWidget * entry_codigo_pago = glade_xml_get_widget(xml,"entry_codigo_pago_gestion_pagos");
    gint code_combo = c_util_get_code_from_combo (combo);
    gfloat f = 1;

    if (code_combo == D_PAYMENT_AT_MOMENT) {
        restore_to_initial_purchase(TRUE);
	gtk_widget_set_sensitive(GTK_WIDGET(spinButton),FALSE);
	f=1;
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinButton),f);
	gtk_spin_button_update(GTK_SPIN_BUTTON(spinButton));

	gtk_label_set_text(GTK_LABEL(money_order_days_label),"");

    } else if (code_combo == D_PAYMENT_MONEY_ORDER) {
        restore_to_initial_purchase(TRUE);
	gtk_widget_set_sensitive(GTK_WIDGET(spinButton),TRUE);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinButton),f);
	gtk_spin_button_update(GTK_SPIN_BUTTON(spinButton));
        
	gtk_label_set_text(GTK_LABEL(money_order_days_label),_("Draft days"));

    } else if (code_combo == D_PAYMENT_MONEY_ORDER_AT_END_OF_MONTH) {
    
        restore_to_initial_purchase(TRUE);
        gtk_widget_set_sensitive(GTK_WIDGET(spinButton),FALSE);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinButton),f);

	gtk_label_set_text(GTK_LABEL(money_order_days_label),_("Deferment days"));

    } else if (code_combo == D_PAYMENT_HIRE_PURCHASE) {
        restore_to_initial_purchase(FALSE);
        gtk_widget_set_sensitive(GTK_WIDGET(spinButton),TRUE);
        gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinButton),f);
	gtk_spin_button_update(GTK_SPIN_BUTTON(spinButton));

	gtk_label_set_text(GTK_LABEL(money_order_days_label),_("Number of instalments"));

    }
    if (strcmp(gtk_entry_get_text(GTK_ENTRY(entry_codigo_pago)),"")) {
	set_button_visible();
    }
    set_dates_on_payment();
}

static void on_payment_means_combo_entry_changed(GtkEntry *entry,
					 gpointer user_data) {

    GtkWidget * combo = glade_xml_get_widget(xml,"combo_medio_pago_gestion_pagos");
    gint code_combo = c_util_get_code_from_combo (combo);
    GtkWidget * valid_card_label = glade_xml_get_widget(xml,"label_valida_hasta_gestion_pagos");
    GtkWidget * valid_date_edit = glade_xml_get_widget(xml,"dateedit_valida_hasta_gestion_pagos");
    GtkWidget * card_number_label = glade_xml_get_widget(xml,"label_numero_gestion_pagos");
    GtkWidget * spin;
    GtkWidget * combo_parent_company = glade_xml_get_widget(xml,"parent_company_accounts_combo");
    GtkAdjustment * adjustment;
    GtkWidget * account_number_entry;


    if (GTK_WIDGET_VISIBLE(valid_card_label)) gtk_widget_hide(valid_card_label);
    if (GTK_WIDGET_VISIBLE(valid_date_edit)) gtk_widget_hide(valid_date_edit);
    if (GTK_WIDGET_VISIBLE(card_number_label)) gtk_widget_hide(card_number_label);

    if (code_combo == D_PAYMENT_TRANSFER_CODE) {

	spin = glade_xml_get_widget(xml,"spinbutton_primero_gestion_pagos");
	if (GTK_WIDGET_VISIBLE(spin)) gtk_widget_hide(spin);
	spin = glade_xml_get_widget(xml,"spinbutton_segundo_gestion_pagos");
	if (GTK_WIDGET_VISIBLE(spin)) gtk_widget_hide(spin);
	spin = glade_xml_get_widget(xml,"spinbutton_tercero_gestion_pagos");
	if (GTK_WIDGET_VISIBLE(spin)) gtk_widget_hide(spin);
	spin = glade_xml_get_widget(xml,"spinbutton_cuarto_gestion_pagos");
	if (GTK_WIDGET_VISIBLE(spin)) gtk_widget_hide(spin);
	if (GTK_WIDGET_VISIBLE(combo_parent_company)) gtk_widget_hide(combo_parent_company);
	account_number_entry = glade_xml_get_widget(xml,"entry_numero_cuenta_gestion_pagos");
	if (GTK_WIDGET_VISIBLE(account_number_entry)) gtk_widget_hide(account_number_entry);

    } else if (code_combo == D_PAYMENT_IN_CASH) {

	spin = glade_xml_get_widget(xml,"spinbutton_primero_gestion_pagos");
	if (GTK_WIDGET_VISIBLE(spin)) gtk_widget_hide(spin);
	spin = glade_xml_get_widget(xml,"spinbutton_segundo_gestion_pagos");
	if (GTK_WIDGET_VISIBLE(spin)) gtk_widget_hide(spin);
	spin = glade_xml_get_widget(xml,"spinbutton_tercero_gestion_pagos");
	if (GTK_WIDGET_VISIBLE(spin)) gtk_widget_hide(spin);
	spin = glade_xml_get_widget(xml,"spinbutton_cuarto_gestion_pagos");
	if (GTK_WIDGET_VISIBLE(spin)) gtk_widget_hide(spin);
	if (GTK_WIDGET_VISIBLE(combo_parent_company)) gtk_widget_hide(combo_parent_company);
	account_number_entry = glade_xml_get_widget(xml,"entry_numero_cuenta_gestion_pagos");
	if (GTK_WIDGET_VISIBLE(account_number_entry)) gtk_widget_hide(account_number_entry);

    } else if (code_combo == D_PAYMENT_CREDIT_CARD) {
	if (!GTK_WIDGET_VISIBLE(card_number_label)) gtk_widget_show(card_number_label);
	if (GTK_WIDGET_VISIBLE(valid_card_label)==FALSE) gtk_widget_show(valid_card_label);
	if (GTK_WIDGET_VISIBLE(valid_date_edit)==FALSE) gtk_widget_show(valid_date_edit);
	gtk_label_set_text(GTK_LABEL(card_number_label),_("Card Number"));
	spin = glade_xml_get_widget(xml,"spinbutton_primero_gestion_pagos");
	if (!GTK_WIDGET_VISIBLE(spin)) gtk_widget_show(spin);
        adjustment = gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(spin));
	adjustment->upper=9999;
        gtk_adjustment_changed(adjustment);
	spin = glade_xml_get_widget(xml,"spinbutton_segundo_gestion_pagos");
	if (!GTK_WIDGET_VISIBLE(spin)) gtk_widget_show(spin);
        adjustment = gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(spin));
	adjustment->upper=9999;
        gtk_adjustment_changed(adjustment);
	spin = glade_xml_get_widget(xml,"spinbutton_tercero_gestion_pagos");
	if (!GTK_WIDGET_VISIBLE(spin)) gtk_widget_show(spin);
        adjustment = gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(spin));
	adjustment->upper=9999;
        gtk_adjustment_changed(adjustment);
	spin = glade_xml_get_widget(xml,"spinbutton_cuarto_gestion_pagos");
	if (!GTK_WIDGET_VISIBLE(spin)) gtk_widget_show(spin);
        adjustment = gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(spin));
	adjustment->upper=9999;
        gtk_adjustment_changed(adjustment);
	account_number_entry = glade_xml_get_widget(xml,"entry_numero_cuenta_gestion_pagos");
	if (GTK_WIDGET_VISIBLE(account_number_entry)) gtk_widget_hide(account_number_entry);
	if (GTK_WIDGET_VISIBLE(combo_parent_company)) gtk_widget_hide(combo_parent_company);

    } else if (code_combo == D_PAYMENT_CHEQUE) {
        GtkWidget * label_numero_talon = glade_xml_get_widget(xml,"label_numero_gestion_pagos");
	if (!GTK_WIDGET_VISIBLE(label_numero_talon)) gtk_widget_show(label_numero_talon);
        gtk_label_set_text(GTK_LABEL(card_number_label),_("Check number"));
	spin = glade_xml_get_widget(xml,"spinbutton_primero_gestion_pagos");
	if (!GTK_WIDGET_VISIBLE(spin)) gtk_widget_show(spin);
        adjustment = gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(spin));
	adjustment->upper=9;
        gtk_adjustment_changed(adjustment);
	spin = glade_xml_get_widget(xml,"spinbutton_segundo_gestion_pagos");
	if (!GTK_WIDGET_VISIBLE(spin)) gtk_widget_show(spin);
        adjustment = gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(spin));
	adjustment->upper=999;
        gtk_adjustment_changed(adjustment);
	spin = glade_xml_get_widget(xml,"spinbutton_tercero_gestion_pagos");
	if (!GTK_WIDGET_VISIBLE(spin)) gtk_widget_show(spin);
        adjustment = gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(spin));
	adjustment->upper=999;
        gtk_adjustment_changed(adjustment);
	spin = glade_xml_get_widget(xml,"spinbutton_cuarto_gestion_pagos");
	if (!GTK_WIDGET_VISIBLE(spin)) gtk_widget_show(spin);
        adjustment = gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(spin));
	adjustment->upper=9;
        gtk_adjustment_changed(adjustment);
	account_number_entry = glade_xml_get_widget(xml,"entry_numero_cuenta_gestion_pagos");
	if (GTK_WIDGET_VISIBLE(account_number_entry)) gtk_widget_hide(account_number_entry);
	if (GTK_WIDGET_VISIBLE(combo_parent_company)) gtk_widget_hide(combo_parent_company);

    } else if (code_combo == D_PAYMENT_DIRECT_DEBIT_CODE) {
        GtkWidget * label_numero_cuenta = glade_xml_get_widget(xml,"label_numero_gestion_pagos");

	spin = glade_xml_get_widget(xml,"spinbutton_primero_gestion_pagos");
	if (!GTK_WIDGET_VISIBLE(label_numero_cuenta)) gtk_widget_show(label_numero_cuenta);
	gtk_label_set_text(GTK_LABEL(label_numero_cuenta),_("Current account"));

	if (GTK_WIDGET_VISIBLE(spin)) gtk_widget_hide(spin);
	spin = glade_xml_get_widget(xml,"spinbutton_segundo_gestion_pagos");
	if (GTK_WIDGET_VISIBLE(spin)) gtk_widget_hide(spin);

	spin = glade_xml_get_widget(xml,"spinbutton_tercero_gestion_pagos");
	if (GTK_WIDGET_VISIBLE(spin)) gtk_widget_hide(spin);
	if (!GTK_WIDGET_VISIBLE(combo_parent_company)) gtk_widget_show(combo_parent_company);

	account_number_entry = glade_xml_get_widget(xml,"entry_numero_cuenta_gestion_pagos");
	if (GTK_WIDGET_VISIBLE(account_number_entry)) gtk_widget_hide(account_number_entry);

    } else {
        g_print(_("Not yet implemented"));
    }


}

static void restore_to_initial_purchase(gboolean no_sensitive) {
    gfloat f1 = 100;
    gfloat f = 1;
    GtkWidget * spin1 = glade_xml_get_widget(xml,"spinbutton_entrada_gestion_pagos");
    GtkWidget * spin2 = glade_xml_get_widget(xml,"spinbutton_intervalos_gestion_pagos");
    GtkWidget * date_edit = glade_xml_get_widget(xml,"dateedit_fecha_siguiente_gestion_pagos");
    GtkWidget * spin3 = glade_xml_get_widget(xml,"spinbutton_realizados_gestion_pagos");
    GtkWidget * spin4 = glade_xml_get_widget(xml,"dias_giro_gestion_pagos");

    gtk_widget_set_sensitive(spin3,FALSE);
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin3),0);
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin4),1);


    if (no_sensitive) {
	gtk_widget_set_sensitive(spin2,FALSE);
	gtk_widget_set_sensitive(spin1,FALSE);
	gtk_widget_set_sensitive(date_edit,FALSE);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin1),f1);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin2),f);

    } else {
	gtk_widget_set_sensitive(spin2,TRUE);
	gtk_widget_set_sensitive(spin1,TRUE);
	gtk_widget_set_sensitive(date_edit,TRUE);
    }
}


static void set_dates_on_payment() {
    struct tm * time_tm = (struct tm *) g_malloc0(sizeof(struct tm));
    time_t * time_aux = (time_t *) g_malloc0(sizeof(time_t));
    GtkWidget * next_done_dateedit = glade_xml_get_widget(xml,"dateedit_fecha_siguiente_gestion_pagos");
    GtkWidget * combo = glade_xml_get_widget(xml,"combo_modo_pago_gestion_pagos");
    GtkWidget * spinButton = glade_xml_get_widget(xml,"dias_giro_gestion_pagos");
    gint code_combo = c_util_get_code_from_combo (combo);

    *time_aux = time(NULL);

    if (code_combo == D_PAYMENT_MONEY_ORDER) {
        GDate * date;
        gint days;
	time_tm = localtime(time_aux);
	date = g_date_new_dmy(time_tm->tm_mday,time_tm->tm_mon+1,time_tm->tm_year+1900);
	days = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinButton));
	g_date_add_days(date,days);
	time_tm ->tm_mday = g_date_day(date);
	time_tm ->tm_mon= g_date_month(date)-1;
	time_tm ->tm_year = g_date_year(date)-1900;
	*time_aux = mktime(time_tm);
    } else if (code_combo == D_PAYMENT_MONEY_ORDER_AT_END_OF_MONTH) {
	gint days;
        time_tm = localtime(time_aux);
	days = c_util_get_days_to_last_day_of_month (time_aux);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinButton),1.0*days);
        gtk_spin_button_update(GTK_SPIN_BUTTON(spinButton));
        time_tm->tm_mday = time_tm->tm_mday + days;
	*time_aux = mktime(time_tm);
    } else if (code_combo == D_PAYMENT_HIRE_PURCHASE) {
	time_tm = localtime(time_aux);


    }

    gtk_calendar_display_options(GTK_CALENDAR(GNOME_DATE_EDIT(next_done_dateedit)->calendar),GTK_CALENDAR_SHOW_DAY_NAMES);
    gnome_date_edit_set_time(GNOME_DATE_EDIT(next_done_dateedit),*time_aux);
}

static void on_days_money_order_entry_changed(GtkEntry *entry,
				 gpointer user_data) {

    if (strcmp(gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(xml,"combo_entry_modo_pago_gestion_pagos"))),D_PAYMENT_MESSAGE_MONEY_ORDER_AT_END_OF_MONTH))
	set_dates_on_payment();
}

static void clean_screen() {

    GtkWidget *vbox;

    vbox = GTK_WIDGET(glade_xml_get_widget(xml, widget_name));

    gtk_container_foreach(GTK_CONTAINER(vbox),&c_util_clean_widget,NULL);


}

static void set_button_visible() {
    GtkWidget * entry_modo = glade_xml_get_widget(xml,"combo_entry_modo_pago_gestion_pagos");
    GtkWidget * button_varios_plazos = glade_xml_get_widget(xml,"button_siguiente_pago_gestion_pagos");
    GtkWidget * button_un_plazo = glade_xml_get_widget(xml,"button_pagado_gestion_pagos");
    gchar * value;

    value = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry_modo)));
    if (!strcmp(value,D_PAYMENT_MESSAGE_HIRE_PURCHASE)) {
        if (!GTK_WIDGET_VISIBLE(button_varios_plazos)) gtk_widget_show(button_varios_plazos);
        if (GTK_WIDGET_VISIBLE(button_un_plazo)) gtk_widget_hide(button_un_plazo);
    } else {
        if (GTK_WIDGET_VISIBLE(button_varios_plazos)) gtk_widget_hide(button_varios_plazos);
        if (!GTK_WIDGET_VISIBLE(button_un_plazo)) gtk_widget_show(button_un_plazo);
    }

}
static void fill_payment_on_screen(void) {
    GtkWidget * payment_mode_combo = glade_xml_get_widget(xml,"combo_modo_pago_gestion_pagos");
    GtkWidget * payment_mean_combo = glade_xml_get_widget(xml,"combo_medio_pago_gestion_pagos");
    GtkWidget * entry_medio_pago = glade_xml_get_widget(xml,"combo_entry_medio_pago_gestion_pagos");
    GtkWidget * entry_modo_pago = glade_xml_get_widget(xml,"combo_entry_modo_pago_gestion_pagos");
    GtkWidget * entry_codigo_pago = glade_xml_get_widget(xml,"entry_codigo_pago_gestion_pagos");
    GtkWidget * spin_realizados_pago = glade_xml_get_widget(xml,"spinbutton_realizados_gestion_pagos");
    GtkWidget * dateedit_next_date = glade_xml_get_widget(xml,"dateedit_fecha_siguiente_gestion_pagos");
    GtkWidget * money_order_days_spinbutton = glade_xml_get_widget(xml,"dias_giro_gestion_pagos");
    GtkWidget * first_number_spinbutton = glade_xml_get_widget(xml,"spinbutton_primero_gestion_pagos");
    GtkWidget * second_number_spinbutton = glade_xml_get_widget(xml,"spinbutton_segundo_gestion_pagos");
    GtkWidget * third_number_spinbutton = glade_xml_get_widget(xml,"spinbutton_tercero_gestion_pagos");
    GtkWidget * fourth_number_spinbutton = glade_xml_get_widget(xml,"spinbutton_cuarto_gestion_pagos");
    GtkWidget * account_number_entry = glade_xml_get_widget(xml,"entry_numero_cuenta_gestion_pagos");
    GtkWidget * invoice_validity_dateedit= glade_xml_get_widget(xml,"dateedit_valida_hasta_gestion_pagos");
    GtkWidget * entry_spinbutton=glade_xml_get_widget(xml,"spinbutton_entrada_gestion_pagos");
    GtkWidget * interval_spinbutton=glade_xml_get_widget(xml,"spinbutton_intervalos_gestion_pagos");
    GtkWidget * spinbutton_expiries=glade_xml_get_widget(xml,"dias_giro_gestion_pagos");
    GtkWidget * pay_all_button=glade_xml_get_widget(xml,"button_siguiente_pago_gestion_pagos");
    GtkWidget * instalment_pay_button=glade_xml_get_widget(xml,"button_pagado_gestion_pagos");
    GtkWidget * accept_button=glade_xml_get_widget(xml,"button_aceptar_gestion_pagos");
    GtkWidget * cancel_button=glade_xml_get_widget(xml,"button_cancelar_gestion_pagos");
    GtkWidget * button_salir=glade_xml_get_widget(xml,"button_salir_gestion_pagos");
    gchar * medio_temp = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry_medio_pago)));
    gchar * modo_temp = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry_modo_pago)));
    gint insensible = 0;

    if (payment_to_manage!=NULL) {
	gchar * dias_giro=NULL;
	time_t * time_aux = (time_t *) g_malloc0(sizeof(time_t));
	gchar * codigo_pago = g_strdup_printf("%lu",d_payment_get_payment_code (payment_to_manage));
	gchar * pagos_realizados =  g_strdup_printf("%lu",d_payment_get_achieved_payments (payment_to_manage));
	*time_aux = d_payment_get_init_date (payment_to_manage);
	dias_giro =  g_strdup_printf("%d",d_payment_get_interval_days (payment_to_manage));

	if (d_payment_get_payment_code (payment_to_manage)!=ULONG_MAX) {
	    set_button_visible();
	    if (strcmp(codigo_pago,"0")) {
		gtk_entry_set_text(GTK_ENTRY(entry_codigo_pago),codigo_pago);
	    }
	}


	g_free(codigo_pago);
	/* PAYMENT MODE*/
	if (d_payment_get_interval_days (payment_to_manage)==0) {
	    gchar * expiries = g_strdup_printf("%d",d_payment_get_number_expiries (payment_to_manage));
	    gchar * percentaje = g_strdup_printf("%.2f",d_payment_get_entry_percentage (payment_to_manage));
	    gchar * interval = g_strdup_printf("%d",d_payment_get_interval_days (payment_to_manage));

	    if (strcmp(modo_temp,D_PAYMENT_MESSAGE_AT_MOMENT)) gtk_entry_set_text(GTK_ENTRY(entry_modo_pago),D_PAYMENT_MESSAGE_AT_MOMENT);
	    gtk_entry_set_text(GTK_ENTRY(entry_spinbutton),percentaje);
	    gtk_entry_set_text(GTK_ENTRY(spinbutton_expiries),expiries);
	    gtk_entry_set_text(GTK_ENTRY(interval_spinbutton),interval);
	    g_free(expiries);
	    g_free(percentaje);
	    g_free(interval);

	} else if (d_payment_get_number_expiries (payment_to_manage)>1) {
	    gchar * expiries = g_strdup_printf("%d",d_payment_get_number_expiries (payment_to_manage));
	    gchar * percentaje = g_strdup_printf("%.2f",d_payment_get_entry_percentage (payment_to_manage));
	    gchar * interval = g_strdup_printf("%d",d_payment_get_interval_days (payment_to_manage));

	    if (strcmp(modo_temp,D_PAYMENT_MESSAGE_HIRE_PURCHASE)) gtk_entry_set_text(GTK_ENTRY(entry_modo_pago),D_PAYMENT_MESSAGE_HIRE_PURCHASE);
	    gtk_entry_set_text(GTK_ENTRY(entry_spinbutton),percentaje);
	    gtk_entry_set_text(GTK_ENTRY(spinbutton_expiries),expiries);
	    gtk_entry_set_text(GTK_ENTRY(interval_spinbutton),interval);
	    g_free(expiries);
	    g_free(percentaje);
	    g_free(interval);



	} else if ((c_util_get_days_to_last_day_of_month (time_aux))==d_payment_get_interval_days (payment_to_manage)) {
	    gchar * expiries = g_strdup_printf("%d",d_payment_get_number_expiries (payment_to_manage));
	    gchar * percentaje = g_strdup_printf("%.2f",d_payment_get_entry_percentage (payment_to_manage));
	    gchar * interval = g_strdup_printf("%d",d_payment_get_interval_days (payment_to_manage));

            gchar * temp_vista=g_strconcat(D_PAYMENT_MESSAGE_MONEY_ORDER_AT_END_OF_MONTH,"\0",NULL);
 
	    if (strcmp(modo_temp,temp_vista))
	                    	    gtk_entry_set_text(GTK_ENTRY(entry_modo_pago),temp_vista);

 
            g_free(temp_vista);
	    gtk_entry_set_text(GTK_ENTRY(entry_spinbutton),percentaje);
	    gtk_entry_set_text(GTK_ENTRY(spinbutton_expiries),expiries);
	    gtk_entry_set_text(GTK_ENTRY(interval_spinbutton),interval);
	    g_free(expiries);
	    g_free(percentaje);
	    g_free(interval);


	} else {
	    gchar * expiries = g_strdup_printf("%d",d_payment_get_number_expiries (payment_to_manage));
	    gchar * percentaje = g_strdup_printf("%.2f",d_payment_get_entry_percentage (payment_to_manage));
	    gchar * interval = g_strdup_printf("%d",d_payment_get_interval_days (payment_to_manage));

	    if (strcmp(modo_temp,D_PAYMENT_MESSAGE_MONEY_ORDER)) gtk_entry_set_text(GTK_ENTRY(entry_modo_pago),D_PAYMENT_MESSAGE_MONEY_ORDER);
	    gtk_entry_set_text(GTK_ENTRY(entry_spinbutton),percentaje);
	    gtk_entry_set_text(GTK_ENTRY(spinbutton_expiries),interval);
	    gtk_entry_set_text(GTK_ENTRY(interval_spinbutton),expiries);
	    g_free(expiries);
	    g_free(percentaje);
	    g_free(interval);
	}

	gtk_spin_button_update(GTK_SPIN_BUTTON(entry_spinbutton));
	gtk_spin_button_update(GTK_SPIN_BUTTON(spinbutton_expiries));
	gtk_spin_button_update(GTK_SPIN_BUTTON(interval_spinbutton));

	/* PAYMENT WAY*/
	if (d_payment_get_is_transfer (payment_to_manage)) {
	    if (strcmp(medio_temp,D_PAYMENT_MESSAGE_TRANSFER)) gtk_entry_set_text(GTK_ENTRY(entry_medio_pago),D_PAYMENT_MESSAGE_TRANSFER);

	} else if (d_payment_get_is_cheque (payment_to_manage)) {
	    gchar * number = d_payment_c_util_get_cheque_number (payment_to_manage);
	    gchar ** numbers = c_util_get_card_numbers (number);
	    if (strcmp(medio_temp,D_PAYMENT_MESSAGE_CHEQUE)) gtk_entry_set_text(GTK_ENTRY(entry_medio_pago),D_PAYMENT_MESSAGE_CHEQUE);
	    gtk_entry_set_text(GTK_ENTRY(first_number_spinbutton),numbers[0]);
	    gtk_entry_set_text(GTK_ENTRY(second_number_spinbutton),numbers[1]);
	    gtk_entry_set_text(GTK_ENTRY(third_number_spinbutton),numbers[2]);
	    gtk_entry_set_text(GTK_ENTRY(fourth_number_spinbutton),numbers[3]);
	    g_free(number);
	    g_strfreev(numbers);


	} else if (d_payment_get_is_direct_debit (payment_to_manage)) {
	    if (strcmp(medio_temp,D_PAYMENT_MESSAGE_DIRECT_DEBIT)) gtk_entry_set_text(GTK_ENTRY(entry_medio_pago),D_PAYMENT_MESSAGE_DIRECT_DEBIT);
	} else if (d_payment_get_is_credit_card (payment_to_manage)) {
	    gchar * number = d_payment_get_credit_card_number (payment_to_manage);
	    gchar ** numbers = c_util_get_card_numbers (number);
	    time_t * valid_until = (time_t *) g_malloc0(sizeof(time_t));
	    *valid_until = d_payment_get_expiry_date (payment_to_manage);
	    if (strcmp(medio_temp,D_PAYMENT_MESSAGE_CREDIT_CARD)) gtk_entry_set_text(GTK_ENTRY(entry_medio_pago),D_PAYMENT_MESSAGE_CREDIT_CARD);
	    gtk_entry_set_text(GTK_ENTRY(first_number_spinbutton),numbers[0]);
	    gtk_entry_set_text(GTK_ENTRY(second_number_spinbutton),numbers[1]);
	    gtk_entry_set_text(GTK_ENTRY(third_number_spinbutton),numbers[2]);
	    gtk_entry_set_text(GTK_ENTRY(fourth_number_spinbutton),numbers[3]);
	    gnome_date_edit_set_time(GNOME_DATE_EDIT(invoice_validity_dateedit),*valid_until);
	    g_free(number);
	    g_strfreev(numbers);
	    g_free(valid_until);
	} else {
	    /* ES EFECTIVO*/
	    if (strcmp(medio_temp,D_PAYMENT_MESSAGE_IN_CASH)) gtk_entry_set_text(GTK_ENTRY(entry_medio_pago),D_PAYMENT_MESSAGE_IN_CASH);

	}
	if (GTK_WIDGET_VISIBLE(button_salir)) gtk_widget_hide(button_salir);
	/* HE / SHE CAN'T MODIFY DATA WHE THE PAYMENT WAS DONE */
	if (d_payment_get_payment_code (payment_to_manage)!=ULONG_MAX) {
	    if ((d_payment_get_payment_code (payment_to_manage)!=0) && (d_payment_get_payment_code (payment_to_manage)!=ULONG_MAX) &&
		(d_payment_get_is_hire_purchase (payment_to_manage))) {
		insensible=1;
	    }
	    if ((d_payment_get_achieved_payments (payment_to_manage))==(d_payment_get_number_expiries (payment_to_manage))) {
		insensible=1;
		if (GTK_WIDGET_SENSITIVE(pay_all_button)) gtk_widget_set_sensitive(pay_all_button,FALSE);
		if (GTK_WIDGET_SENSITIVE(instalment_pay_button)) gtk_widget_set_sensitive(instalment_pay_button,FALSE);
		if (GTK_WIDGET_SENSITIVE(accept_button)) gtk_widget_set_sensitive(accept_button,FALSE);
		if (GTK_WIDGET_SENSITIVE(cancel_button)) gtk_widget_set_sensitive(cancel_button,FALSE);
		if (!GTK_WIDGET_VISIBLE(button_salir)) gtk_widget_show(button_salir);
	    } else {
		if (!GTK_WIDGET_SENSITIVE(pay_all_button)) gtk_widget_set_sensitive(pay_all_button,TRUE);
		if (!GTK_WIDGET_SENSITIVE(instalment_pay_button)) gtk_widget_set_sensitive(instalment_pay_button,TRUE);
		if (!GTK_WIDGET_SENSITIVE(accept_button)) gtk_widget_set_sensitive(accept_button,TRUE);
		if (!GTK_WIDGET_SENSITIVE(cancel_button)) gtk_widget_set_sensitive(cancel_button,TRUE);
	    }
	    if (insensible==1) {
		if (GTK_WIDGET_SENSITIVE(payment_mode_combo)) gtk_widget_set_sensitive(payment_mode_combo,FALSE);
		if (GTK_WIDGET_SENSITIVE(payment_mean_combo)) gtk_widget_set_sensitive(payment_mean_combo,FALSE);
		if (GTK_WIDGET_SENSITIVE(dateedit_next_date)) gtk_widget_set_sensitive(dateedit_next_date,FALSE);
		if (GTK_WIDGET_SENSITIVE(money_order_days_spinbutton)) gtk_widget_set_sensitive(money_order_days_spinbutton,FALSE);
		if (GTK_WIDGET_SENSITIVE(first_number_spinbutton)) gtk_widget_set_sensitive(first_number_spinbutton,FALSE);
		if (GTK_WIDGET_SENSITIVE(second_number_spinbutton)) gtk_widget_set_sensitive(second_number_spinbutton,FALSE);
		if (GTK_WIDGET_SENSITIVE(third_number_spinbutton)) gtk_widget_set_sensitive(third_number_spinbutton,FALSE);
		if (GTK_WIDGET_SENSITIVE(fourth_number_spinbutton)) gtk_widget_set_sensitive(fourth_number_spinbutton,FALSE);
		if (GTK_WIDGET_SENSITIVE(account_number_entry)) gtk_widget_set_sensitive(account_number_entry,FALSE);
		if (GTK_WIDGET_SENSITIVE(invoice_validity_dateedit)) gtk_widget_set_sensitive(invoice_validity_dateedit,FALSE);
		if (GTK_WIDGET_SENSITIVE(spinbutton_expiries)) gtk_widget_set_sensitive(spinbutton_expiries,FALSE);
		if (GTK_WIDGET_SENSITIVE(entry_spinbutton)) gtk_widget_set_sensitive(entry_spinbutton,FALSE);

	    } else {
		if (!GTK_WIDGET_SENSITIVE(payment_mode_combo)) gtk_widget_set_sensitive(payment_mode_combo,TRUE);
		if (!GTK_WIDGET_SENSITIVE(payment_mean_combo)) gtk_widget_set_sensitive(payment_mean_combo,TRUE);
		if (!GTK_WIDGET_SENSITIVE(dateedit_next_date)) gtk_widget_set_sensitive(dateedit_next_date,TRUE);
		if (!GTK_WIDGET_SENSITIVE(money_order_days_spinbutton)) gtk_widget_set_sensitive(money_order_days_spinbutton,TRUE);
		if (!GTK_WIDGET_SENSITIVE(first_number_spinbutton)) gtk_widget_set_sensitive(first_number_spinbutton,TRUE);
		if (!GTK_WIDGET_SENSITIVE(second_number_spinbutton)) gtk_widget_set_sensitive(second_number_spinbutton,TRUE);
		if (!GTK_WIDGET_SENSITIVE(third_number_spinbutton)) gtk_widget_set_sensitive(third_number_spinbutton,TRUE);
		if (!GTK_WIDGET_SENSITIVE(fourth_number_spinbutton)) gtk_widget_set_sensitive(fourth_number_spinbutton,TRUE);
		if (!GTK_WIDGET_SENSITIVE(account_number_entry)) gtk_widget_set_sensitive(account_number_entry,TRUE);
		if (!GTK_WIDGET_SENSITIVE(invoice_validity_dateedit)) gtk_widget_set_sensitive(invoice_validity_dateedit,TRUE);
	    }
	} else {
	    if (GTK_WIDGET_VISIBLE(instalment_pay_button)) gtk_widget_hide(instalment_pay_button);
	    if (GTK_WIDGET_VISIBLE(pay_all_button)) gtk_widget_hide(pay_all_button);

	}
	gtk_entry_set_text(GTK_ENTRY(spin_realizados_pago),pagos_realizados);

	gnome_date_edit_set_time(GNOME_DATE_EDIT(dateedit_next_date),*time_aux);
	//	    gnome_date_edit_set_time(GNOME_DATE_EDIT(dateedit_fecha_pago_factura),*fecha_realizado);
	//	    gtk_entry_set_text(GTK_ENTRY(money_order_days_spinbutton),dias_giro);

	g_free(time_aux);
	//	    g_free(fecha_realizado);
	g_free(dias_giro);
    }
    if (d_payment_get_payment_code (payment_to_manage)==ULONG_MAX) {
        set_dates_on_payment();
    }
    g_free(medio_temp);
    g_free(modo_temp);
}

static gboolean set_payment_from_screen(Payment * payment_to_fill) {

    GtkWidget * payment_mean_combo = glade_xml_get_widget(xml,"combo_medio_pago_gestion_pagos");
    GtkWidget * payment_mode_combo = glade_xml_get_widget(xml,"combo_modo_pago_gestion_pagos");
    GtkWidget * entry_spinbutton = glade_xml_get_widget(xml,"spinbutton_entrada_gestion_pagos");
    GtkWidget * interval_spinbutton = glade_xml_get_widget(xml,"spinbutton_intervalos_gestion_pagos");
    GtkWidget * money_order_days_spinbutton = glade_xml_get_widget(xml,"dias_giro_gestion_pagos");
    GtkWidget * done_spinbutton = glade_xml_get_widget(xml,"spinbutton_realizados_gestion_pagos");
    GtkWidget * first_number_spinbutton = glade_xml_get_widget(xml,"spinbutton_primero_gestion_pagos");
    GtkWidget * second_number_spinbutton = glade_xml_get_widget(xml,"spinbutton_segundo_gestion_pagos");
    GtkWidget * third_number_spinbutton = glade_xml_get_widget(xml,"spinbutton_tercero_gestion_pagos");
    GtkWidget * fourth_number_spinbutton = glade_xml_get_widget(xml,"spinbutton_cuarto_gestion_pagos");
    GtkWidget * dateedit_valid_until = glade_xml_get_widget(xml,"dateedit_valida_hasta_gestion_pagos");
    GtkWidget * dateedit_next_date = glade_xml_get_widget(xml,"dateedit_fecha_siguiente_gestion_pagos");
    gboolean is_hire_purchase=FALSE;
    gshort expiries_number=1;
    gdouble entry_percentaje=100;
    time_t next_date=time(NULL);
    gshort interval_days=0;
    gboolean is_transfer=FALSE;
    gboolean is_cheque=FALSE;
    gchar * cheque_number=NULL;
    gboolean is_card=FALSE;
    gboolean is_direct_debit=FALSE;
    gint bank_code=-1;
    gint office_code=-1;
    gchar *  account_number_gchar=NULL;
    gchar * card_number_gchar=NULL;
    time_t valid_until=time(NULL);
    gboolean is_pay_cash=FALSE;
    gint error = 0;
    
    gint payment_mean_code_combo = c_util_get_code_from_combo (payment_mean_combo);
    gint payment_mode_code_combo = c_util_get_code_from_combo (payment_mode_combo);

    if (payment_to_fill==NULL) payment_to_fill=d_payment_create_empty ();

    if (payment_mode_code_combo == D_PAYMENT_AT_MOMENT) {
	is_hire_purchase=FALSE;
	expiries_number=1;
	entry_percentaje=100;
	next_date=time(NULL);
	interval_days=0;
        is_pay_cash = TRUE;
        d_payment_set_achieved_payments (payment_to_fill,gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(done_spinbutton)));

    } else if (payment_mode_code_combo == D_PAYMENT_MONEY_ORDER) {
	is_hire_purchase=FALSE;
	expiries_number=1;
	entry_percentaje=100;
        gtk_spin_button_update(GTK_SPIN_BUTTON(money_order_days_spinbutton));
	interval_days=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(money_order_days_spinbutton));
	next_date= gnome_date_edit_get_date(GNOME_DATE_EDIT(dateedit_next_date));
	is_pay_cash = FALSE;
        d_payment_set_achieved_payments (payment_to_fill,gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(done_spinbutton)));

    } else if (payment_mode_code_combo == D_PAYMENT_MONEY_ORDER_AT_END_OF_MONTH) {
	time_t * fecha = (time_t *) g_malloc0(sizeof(time_t));
	is_hire_purchase=FALSE;
	expiries_number=1;
	entry_percentaje=100;
	next_date=gnome_date_edit_get_date(GNOME_DATE_EDIT(dateedit_next_date));
	*fecha = next_date;
	interval_days= c_util_get_days_to_last_day_of_month (fecha);
        g_free(fecha);
        is_pay_cash = FALSE;
        d_payment_set_achieved_payments (payment_to_fill,gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(done_spinbutton)));

    } else if (payment_mode_code_combo == D_PAYMENT_HIRE_PURCHASE) {
	is_hire_purchase=TRUE;
        gtk_spin_button_update(GTK_SPIN_BUTTON(money_order_days_spinbutton));
	expiries_number=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(money_order_days_spinbutton));
        gtk_spin_button_update(GTK_SPIN_BUTTON(entry_spinbutton));
	entry_percentaje=gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(entry_spinbutton));;
	next_date=gnome_date_edit_get_date(GNOME_DATE_EDIT(dateedit_next_date));
        gtk_spin_button_update(GTK_SPIN_BUTTON(interval_spinbutton));
	interval_days=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(interval_spinbutton));
	is_pay_cash = FALSE;
	gtk_spin_button_update(GTK_SPIN_BUTTON(done_spinbutton));
        d_payment_set_achieved_payments (payment_to_fill,gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(done_spinbutton)));

    } else {
	g_print(_("Not yet implemented"));
    }

    if (payment_mean_code_combo == D_PAYMENT_TRANSFER_CODE) {
	is_transfer=TRUE;
    } else if (payment_mean_code_combo == D_PAYMENT_IN_CASH) {
        /*THAT OPTION DON'T NEED ANY VARIABLE TO TRUE */
    } else if (payment_mean_code_combo == D_PAYMENT_CREDIT_CARD) {
	gint value1;
	gint value2;
	gint value3;
	gint value4;

        gtk_spin_button_update(GTK_SPIN_BUTTON(first_number_spinbutton));
        gtk_spin_button_update(GTK_SPIN_BUTTON(second_number_spinbutton));
        gtk_spin_button_update(GTK_SPIN_BUTTON(third_number_spinbutton));
        gtk_spin_button_update(GTK_SPIN_BUTTON(fourth_number_spinbutton));

	value1 = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(first_number_spinbutton));
	value2 = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(second_number_spinbutton));
	value3 = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(third_number_spinbutton));
	value4 = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(fourth_number_spinbutton));

	valid_until = gnome_date_edit_get_date(GNOME_DATE_EDIT(dateedit_valid_until));

	card_number_gchar = c_util_get_card_number (value1,value2,value3,value4);

	is_card=TRUE;
    } else if (payment_mean_code_combo == D_PAYMENT_CHEQUE) {
	gint value1 ;
	gint value2 ;
	gint value3 ;
	gint value4 ;

	gtk_spin_button_update(GTK_SPIN_BUTTON(first_number_spinbutton));
        gtk_spin_button_update(GTK_SPIN_BUTTON(second_number_spinbutton));
        gtk_spin_button_update(GTK_SPIN_BUTTON(third_number_spinbutton));
        gtk_spin_button_update(GTK_SPIN_BUTTON(fourth_number_spinbutton));

	value1 = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(first_number_spinbutton));
	value2 = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(second_number_spinbutton));
	value3 = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(third_number_spinbutton));
	value4 = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(fourth_number_spinbutton));
	is_cheque=TRUE;
	cheque_number = c_util_get_cheque_number (value1,value2,value3,value4);

    } else if (payment_mean_code_combo == D_PAYMENT_DIRECT_DEBIT_CODE) {
	gchar * complete_account_number = g_strdup(gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(xml,"parent_company_accounts_entry"))));
        gchar ** tokens = (gchar **) g_malloc0(sizeof(gchar *));

	if (strcmp(complete_account_number,"")) {
	    tokens = g_strsplit(complete_account_number," ",4);
	    bank_code=atoi(tokens[0]);
	    office_code=atoi(tokens[1]);
	    account_number_gchar=g_strdup(tokens[3]);
	    g_strfreev(tokens);
	    is_direct_debit = TRUE;
	} else {
            c_util_open_error_window (_("There is not any current account"));
	    error = 1;
	}
    }
    if (error==0) {
	d_payment_set_payment_code (payment_to_fill,0);
	d_payment_set_is_hire_purchase (payment_to_fill,is_hire_purchase);
	d_payment_set_number_expiries (payment_to_fill,expiries_number);
	d_payment_set_entry_percentage (payment_to_fill,entry_percentaje);
	d_payment_set_init_date (payment_to_fill,next_date);
	d_payment_set_interval_days (payment_to_fill,interval_days);
	d_payment_set_is_paid_cash (payment_to_fill,is_pay_cash);
	d_payment_set_is_hire_purchase (payment_to_fill,is_hire_purchase);
	d_payment_set_is_transfer (payment_to_fill,is_transfer);
	d_payment_set_is_cheque (payment_to_fill,is_cheque);
	d_payment_set_cheque_number (payment_to_fill,cheque_number);
	d_payment_set_is_credit_card (payment_to_fill,is_card);
	d_payment_set_is_direct_debit (payment_to_fill,is_direct_debit);
	d_payment_set_credit_card_number (payment_to_fill,card_number_gchar);
	d_payment_set_expiry_date (payment_to_fill,valid_until);
	d_payment_set_is_paid_cash (payment_to_fill,is_pay_cash);
	d_payment_set_bank_code (payment_to_fill,bank_code);
	d_payment_set_office_code (payment_to_fill,office_code);
	d_payment_set_account_number (payment_to_fill,account_number_gchar);
        if (cheque_number!=NULL)  g_free(cheque_number);
        if (account_number_gchar!=NULL)  g_free(account_number_gchar);
	if (card_number_gchar!=NULL)  g_free(card_number_gchar);
        return TRUE;
    } else {
        return FALSE;
    }

}


static void get_code_from_payment(gpointer value) {
    gulong payment_code;
    if (payment_to_manage!=NULL) {
        payment_code = d_payment_get_payment_code (payment_to_manage);
	*((gulong *) value) =payment_code;
    }
}

static void generate_operation(void) {
    GtkWidget * wid = glade_xml_get_widget(xml,"combo_modo_pago_gestion_pagos");
    GtkWidget * app1 = lookup_widget(wid, "app1");
    GtkWidget * open_insert;
    gdouble total=G_MINDOUBLE;
    PaymentId * paymentId = (PaymentId *) g_malloc0(sizeof(PaymentId));

    paymentId ->payment_code = g_strdup_printf("%lu",d_payment_get_payment_code (payment_to_manage));
    paymentId ->enterprise_code = g_strdup_printf("%lu",d_participant_get_code (d_enterprise_get_participant (d_payment_get_enterprise (payment_to_manage))));
    paymentId ->shop_code = g_strdup_printf("%lu",d_participant_get_code (d_shop_get_participant (d_payment_get_shop (payment_to_manage))));


    if (d_payment_get_is_paid_in_cash(payment_to_manage)!=FALSE) {
	total = (m_payment_calculate_next_total (paymentId,FALSE));

	open_insert = c_cash_operation_open_window(app1,pay,FALSE);
	load_payment_cash_operation(payment_to_manage,total);

	if (open_insert!=NULL) {
	    gtk_window_set_modal(GTK_WINDOW(open_insert),1);
	    gtk_window_set_position(GTK_WINDOW(open_insert),GTK_WIN_POS_CENTER);
	    gtk_window_set_transient_for(GTK_WINDOW(open_insert),
					 GTK_WINDOW(app1));
	    gtk_widget_show(GTK_WIDGET(open_insert));
	}

    } else {
	total = -(m_payment_calculate_next_total (paymentId,FALSE));

	open_insert = c_insert_operation_open_window(app1,get_code_from_payment,pay,total);
	if (open_insert!=NULL) {
	    gtk_window_set_modal(GTK_WINDOW(open_insert),1);
	    gtk_window_set_position(GTK_WINDOW(open_insert),GTK_WIN_POS_CENTER);
	    gtk_window_set_transient_for(GTK_WINDOW(open_insert),
					 GTK_WINDOW(app1));
	    gtk_widget_show(GTK_WIDGET(open_insert));
	}
    }
}


static void on_exit_button_clicked(GtkButton *button,gpointer user_data) {
    payment_to_manage = NULL;
    gtk_widget_destroy(payments);
    c_charge_invoice_is_paid_toggle_button_change_state (TRUE);
    payments = NULL;
}

static void fill_account_number_combo(Payment * payment) {
    GtkWidget * combo;
    Account ** accounts;


    accounts = m_account_find_all ();

    combo = glade_xml_get_widget (xml, "parent_company_accounts_combo");

    c_util_clean_code_and_name_combo (combo);

    if (accounts!=NULL) {
        gint index_account=0;
	while (accounts[index_account]!=NULL) {
	    gchar * complete_account_number=NULL;
	    Account * account = accounts[index_account];

            complete_account_number = d_account_get_complete_number(account);
	    c_util_set_code_and_name_on_combo (index_account,complete_account_number,GTK_COMBO(combo));
	    g_free(complete_account_number);
            index_account++;
	}
    }
    if ((d_payment_get_payment_code (payment) != ULONG_MAX) && (d_payment_get_is_direct_debit (payment))) {
	GtkWidget * entry = glade_xml_get_widget(xml,"parent_company_accounts_entry");
	Account * account = d_account_create_empty ();
        Office * office = d_office_create_empty ();
	gchar * complete_account_number=NULL;
        gchar * number_aux=NULL;
	d_office_set_bank_code (office,d_payment_get_bank_code (payment));
	d_office_set_office_code (office,d_payment_get_office_code (payment));
	number_aux = d_payment_get_account_number (payment);
        d_account_set_office (account,office);
	d_account_set_account_number (account,number_aux);
        g_free(number_aux);
        complete_account_number = d_account_get_complete_number (account);
        gtk_entry_set_text(GTK_ENTRY(entry),complete_account_number);
	g_free(complete_account_number);
	d_account_clear (account);
        gtk_widget_set_sensitive(combo,FALSE);
    } else {
	gtk_widget_set_sensitive(combo,TRUE);

    }
    if (accounts!=NULL) {
	m_account_clear_array (accounts);
	accounts =NULL;
    }


}

