

/* repairs_control.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#define C_LINE_NUMBER_COLUMN      0
#define C_EUROCODE_COLUMN          1
#define C_DESCRIPTION_COLUMN       2
#define C_PRICE_COLUMN            3
#define C_AMOUNT_COLUMN          4
#define C_COST_PRICE_COLUMN   5
#define C_DISCOUNT_COLUMN         6
#define C_MAXIMUN_DISCOUNT_COLUMN     7
#define C_TOTAL_COLUMN           8

#define C_CODIGO_INSERCION ULONG_MAX
#define C_INSERTION_MESSAGE "Nuevo"

#include "config.h"
#include <gnome.h>
#include <glade/glade.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <glib.h>
#include <stdlib.h>

#include "vehicle.h"
#include "company.h"
#include "policy.h"
#include "receipt.h"
#include "accident.h"
#include "repair_line_object.h"
#include "exception.h"
#include "util.h"
#include "parameter_manager.h"

#include "find_manodeobra.h"
#include "find_stocked_pieces_control.h"
#include "invoice_control.h"
#include "generic_repair_line.h"
#include "find_garage.h"
#include "find_agent.h"
#include "repairs_control.h"
#include "clients_control.h"
#include "insurance_data_control.h"
#include "documents_control.h"
#include "repair_line_data.h"
#include "a-shortcut.h"
#include "support.h"
#include "control_util.h"

#include "vehicle_model.h"
#include "repair_line_model.h"
#include "repair_object_model.h"
#include "company_model.h"
#include "find_pieces_model.h"
#include "shop_model.h"
#include "enterprise_model.h"

static GtkWidget *repairs=NULL;
static GladeXML *xml = NULL;
static GtkWidget *documents_window = NULL;

static Repair *general_repair = NULL;
static Client *general_client = NULL;
static Garage *general_garage = NULL;
static Agent *general_agent = NULL;
static Receipt *general_receipt = NULL;
static gboolean new_receipt = TRUE;
static Policy *general_policy = NULL;
static gboolean new_policy = TRUE;
static Accident *general_accident = NULL;
static Vehicle *general_vehicle = NULL;
static gboolean new_vehicle = TRUE;
static GList *repair_lines = NULL;
static gboolean cleaning = FALSE;
static gboolean is_change = FALSE;

static gchar url[]="/src/interface/glade/repairs_view.glade";
static gchar widget_name[]="repairs_vbox";

static void on_insert_client_button_clicked(GtkButton *button, gpointer data);
static void on_change_client_button_clicked(GtkButton *button, gpointer data);
static void on_garage_checkbutton_toggled(GtkToggleButton *togglebutton, gpointer user_data);
static void on_agent_checkbutton_toggled(GtkToggleButton *togglebutton, gpointer user_data);
static void on_insurance_checkbutton_toggled(GtkToggleButton *togglebutton, gpointer user_data);
static void on_clean_button_clicked(GtkButton *button, gpointer data);
static void on_insert_line_button_clicked(GtkButton *button, gpointer data);
static void on_edit_linea_button_clicked(GtkButton *button, gpointer data);
static void on_delete_line_button_clicked(GtkButton *button, gpointer data);
static void on_insert_piece_button_clicked(GtkButton *button, gpointer data);
static void on_insert_man_power_button_clicked(GtkButton *button, gpointer data);
static void on_vehicle_combo_entry_changed(GtkEntry *entry, gpointer user_data);
static void on_shop_entry_changed(GtkEntry *entry, gpointer user_data);
static void on_manage_button_clicked(GtkButton *button, gpointer data);
static void on_repairs_insertion_destroy(GtkWidget *widget,gpointer user_data);
static gboolean on_save_button_clicked(GtkButton *button, gpointer data);
static void clean_screen(void);
static void load_lines_list(void);
static void fill_shop_combo(void);
static void fill_enterprise_from_combo_repairs(void);
static void fill_vehicle_combo(gchar *number_plate);
static void clean_vehicle_combo(void);
static void recover_vehicle_data(void);
static void recover_repair_data(void);
static void free_objects(void);
static void unlink_objects(void);
static void fill_vehicle(Vehicle *vehicle);
static void on_repairs_insertion_window_hide(GtkWidget *widget, gpointer user_data);
static void open_doc_window(GtkWidget *app1);
static void confirm_clean (gint reply, gpointer data);

GtkWidget *c_repair_open_window(GtkWidget *app1, GtkWidget *vbox)
{
    gchar * directory = g_strconcat(AA_DATA_DIR,url,NULL);
    gchar *s_name =NULL;

    if (repairs==NULL)
    {
	GtkWidget *dialog;
	GtkWidget *clist ;
#ifdef IS_SHOP
	GtkWidget * shop_combo= NULL;
#endif

	xml = glade_xml_new (directory, widget_name);
	dialog = glade_xml_get_widget (xml, widget_name);
	clist =glade_xml_get_widget (xml, "lineas_clist");

#ifdef IS_SHOP
	shop_combo = glade_xml_get_widget (xml, "tienda_combo");
      
	gtk_widget_set_sensitive(shop_combo,FALSE);
#endif
      
	gtk_box_pack_start (GTK_BOX (vbox), dialog, TRUE, TRUE, 0);


	glade_xml_signal_connect (xml, "on_insert_client_button_clicked",
				  GTK_SIGNAL_FUNC (on_insert_client_button_clicked));
	glade_xml_signal_connect (xml, "on_taller_checkbutton_toggled",
				  GTK_SIGNAL_FUNC (on_garage_checkbutton_toggled));
	glade_xml_signal_connect (xml, "on_agente_checkbutton_toggled",
				  GTK_SIGNAL_FUNC (on_agent_checkbutton_toggled));
	glade_xml_signal_connect (xml, "on_seguro_checkbutton_toggled",
				  GTK_SIGNAL_FUNC (on_insurance_checkbutton_toggled));
	glade_xml_signal_connect (xml, "on_clean_button_clicked",
				  GTK_SIGNAL_FUNC (on_clean_button_clicked));
	glade_xml_signal_connect (xml, "on_insertar_pieza_button_clicked",
				  GTK_SIGNAL_FUNC (on_insert_piece_button_clicked));
	glade_xml_signal_connect (xml, "on_editar_linea_button_clicked",
				  GTK_SIGNAL_FUNC (on_edit_linea_button_clicked));
	glade_xml_signal_connect (xml, "on_eliminar_linea_button_clicked",
				  GTK_SIGNAL_FUNC (on_delete_line_button_clicked));
	glade_xml_signal_connect (xml, "on_insertar_linea_button_clicked",
				  GTK_SIGNAL_FUNC (on_insert_line_button_clicked));
	glade_xml_signal_connect (xml, "on_insertar_mo_button_clicked",
				  GTK_SIGNAL_FUNC (on_insert_man_power_button_clicked));
	glade_xml_signal_connect (xml, "on_tienda_entry_changed",
				  GTK_SIGNAL_FUNC (on_shop_entry_changed));
	glade_xml_signal_connect (xml, "on_gestionar_button_clicked",
				  GTK_SIGNAL_FUNC (on_manage_button_clicked));
	glade_xml_signal_connect (xml, "on_insercion_reparaciones_destroy",
				  GTK_SIGNAL_FUNC (on_repairs_insertion_destroy));
	glade_xml_signal_connect (xml, "on_guardar_button_clicked",
				  GTK_SIGNAL_FUNC (on_save_button_clicked));
	glade_xml_signal_connect (xml, "on_vehiculo_combo_entry_changed",
				  GTK_SIGNAL_FUNC (on_vehicle_combo_entry_changed));
	glade_xml_signal_connect (xml, "on_insercion_reparaciones_hide",
				  GTK_SIGNAL_FUNC (on_repairs_insertion_window_hide));
	glade_xml_signal_connect (xml, "on_change_client_button_clicked",
				  GTK_SIGNAL_FUNC (on_change_client_button_clicked));



	gtk_clist_set_column_visibility ((GtkCList *) clist, C_COST_PRICE_COLUMN,FALSE);

	fill_shop_combo();
	s_name = u_parameter_manager_get_parameter ("tienda");
	if (s_name!=NULL) {
	    gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget (xml, "tienda_entry")), s_name);
	}
        g_free(s_name);

	fill_vehicle_combo(NULL);

	repairs=dialog;
	
    }

    if (general_policy != NULL) {

      open_doc_window(app1);
      
      gtk_widget_show(documents_window);
      
    }

    g_free(directory);

    return repairs;

}

static void open_doc_window(GtkWidget *app1) {

  gboolean repair = TRUE;

  if (!(GTK_IS_WINDOW(documents_window))) {
    documents_window = c_documents_open(app1, (gpointer) &repair);
    
    gtk_window_set_position(GTK_WINDOW(documents_window),GTK_WIN_POS_NONE);
    
  }
}

void c_repair_load_repair(Repair *repair){

  if (repair != NULL) {
    if (d_repair_get_repair_code (repair) != ULONG_MAX) {
      time_t init_date = d_repair_get_in_date (repair);
      time_t end_date = d_repair_get_out_date (repair);
      gchar *observations = d_repair_get_observations (repair);
      gchar *place = d_repair_get_mobile_unit_place (repair);
      gulong distance_gulong = d_repair_get_mobile_unit_distance (repair);
      gchar * distance = NULL;
      Agent *agent = d_repair_get_agent (repair);
      Garage *garage = d_repair_get_garage (repair);
      GtkWidget *vehicle_entry = glade_xml_get_widget (xml, "vehiculo_combo_entry");
      GtkWidget *tienda_combo_entry = glade_xml_get_widget (xml, "tienda_entry");
      gchar * nombre_tienda = d_participant_get_name (d_shop_get_participant (d_repair_get_shop (repair)));

      if (general_repair != NULL) clean_screen();
      
      cleaning = TRUE;
      
      general_repair = repair;

      if (nombre_tienda!=NULL) {
          gtk_entry_set_text(GTK_ENTRY(tienda_combo_entry),nombre_tienda);
      }

      if (distance_gulong!=ULONG_MAX)
	distance = g_strdup_printf("%lu",distance_gulong);
      
      gnome_date_edit_set_time(GNOME_DATE_EDIT(glade_xml_get_widget (xml, "fecha_entrada_dateedit")), init_date);
      
      gnome_date_edit_set_time(GNOME_DATE_EDIT(glade_xml_get_widget (xml, "fecha_salida_dateedit")), end_date);
      
      if (observations != NULL) {
	GtkWidget *observations_entry = glade_xml_get_widget (xml, "observaciones_text");
	gint *position = (gint *) g_malloc0(sizeof(gint));
	gint len = strlen(observations);
	
	*position = 0;
	
	gtk_editable_insert_text (GTK_EDITABLE(observations_entry), observations, len, position);
	
	g_free(observations);
	g_free(position);
      }
      
      if (place != NULL) {
	GtkWidget *place_entry = glade_xml_get_widget (xml, "lugar_entry");
	gtk_entry_set_text (GTK_ENTRY(place_entry), place);
	g_free(place);
      }
      
      if (distance != NULL) {
	GtkWidget *distance_entry = glade_xml_get_widget (xml, "distancia_entry");
	gtk_entry_set_text (GTK_ENTRY(distance_entry), distance);
	g_free(distance);
      }

      is_change = FALSE;

      c_repair_load_client(d_client_duplicate (d_repair_get_client (repair)), FALSE);
      
      gtk_entry_set_text (GTK_ENTRY(vehicle_entry), d_vehicle_get_license_number (d_repair_get_vehicle (repair)));
      
      if (d_participant_get_code (d_agent_get_participant (agent)) != ULONG_MAX) {
	c_repair_toggle_agent(TRUE);
	c_repair_insert(agent); 
      } else d_repair_set_agent (repair, NULL);
      
      if (d_participant_get_code (d_garage_get_participant (garage)) != ULONG_MAX) {
	c_repair_toggle_garage(TRUE);
	c_repair_insert_garage(garage);
      } else d_repair_set_garage (repair, NULL);
      
      if (d_repair_get_policy (repair) != NULL) {
	if ((d_policy_get_policy_number(d_repair_get_policy (repair))) != NULL) {
	  general_policy = d_policy_duplicate(d_repair_get_policy (repair));
	  
	  if ((d_receipt_get_receipt_number (d_repair_get_receipt (repair))) != NULL)
	    general_receipt = d_receipt_duplicate (d_repair_get_receipt (repair));
	  
	  general_accident = d_accident_duplicate (d_repair_get_accident (repair));
	  
	  c_repair_toggle_insurance(TRUE,FALSE,FALSE);
	  c_repair_insert_insurance(TRUE);
	}
      }
      else {
	c_repair_toggle_insurance(FALSE,TRUE,TRUE);
	c_repair_insert_insurance(FALSE);
      }
      
      repair_lines = m_repair_line_array_to_glist (d_repair_get_repair_lines (repair));
      
      load_lines_list();
      
      cleaning = FALSE;
      
    }
  }    
}

static void on_shop_entry_changed(GtkEntry *entry, gpointer user_data){
  fill_enterprise_from_combo_repairs();
}


static void fill_enterprise_from_combo_repairs() {
  GtkWidget * entry;
  GtkWidget * combo;
  gchar * aux=NULL;
  gulong code_combo;
  Enterprise * enterprise = NULL;
  
  entry = glade_xml_get_widget (xml, "empresa_entry");
  
  combo = glade_xml_get_widget (xml, "tienda_combo");
  
  code_combo = c_util_get_code_from_combo (combo);
  
  enterprise = m_enterprise_find_by_shop (code_combo);
  if (enterprise!=NULL) {
    aux = d_participant_get_name (d_enterprise_get_participant (enterprise));
    gtk_entry_set_text(GTK_ENTRY(entry),aux);
    d_enterprise_clear (enterprise);
    enterprise=NULL;
    g_free(aux);
  }
}

static void fill_shop_combo() {
  GtkWidget * combo;
  Shop ** shops;
  
  combo = glade_xml_get_widget (xml, "tienda_combo");
    
  shops = m_shop_find_all ();
  
  c_util_fill_combo_with_shops (combo,shops);
  
}

void c_repair_load_pieces(Stock **stocks) {

/*   m_repair_line_convert_from_pieces (&repair_lines, pieces_ids, number); */

  m_repair_line_convert_from_stocks (&repair_lines, stocks);

  if (general_policy == NULL){
    if (general_client!=NULL) {
      gdouble discount_gint = d_client_get_discount (general_client);
  
      m_repair_line_apply_discount (repair_lines, discount_gint);
    }
  }
  else{
    if (general_receipt!=NULL) {
      gulong codigo_company = d_receipt_get_company_code (general_receipt);

      if (codigo_company!=ULONG_MAX){
	Company **companies = m_company_find_by_code (codigo_company);
	
	if (companies[0]!=NULL){
	  gdouble discount_gint = d_company_get_discount (companies[0]);
	  
	  if (discount_gint != -1) {	    
	    m_repair_line_apply_discount (repair_lines, discount_gint);
	    load_lines_list();
	  }
	  
	  d_company_clear (companies[0]);
	  
	  g_free(companies);
	}
      }
    }
  }
  
  load_lines_list();

}


static void load_lines_list(){
  
  if (repair_lines!=NULL) {
    GtkWidget * list = glade_xml_get_widget (xml, "lineas_clist");      
    gchar *empty_row[]={"","","","","","","","",""};
    GList * element = g_list_first(repair_lines);   
    gint index = 0;

    gtk_clist_clear(GTK_CLIST(list));
    
    while (element != NULL) {
      Repair_line *line =(Repair_line *) element->data;
      Product *product = d_repair_line_get_product (line);
      gchar *description = d_product_get_description (product);
      gdouble price_gdouble = d_product_get_price (product);
      gdouble total_gdouble = d_repair_line_calculate_total(line);//d_repair_line_get_price (line);
      gdouble quantity_gulong = u_round_to_decimal (d_repair_line_get_quantity(line),3);
      gulong nline_gulong = d_repair_line_get_line_position_number (line);
      gdouble dto_line_gint = d_repair_line_get_discount (line);
      gdouble dtomax_gint = d_product_get_max_discount (product);


      if (index==30) 
	gtk_clist_freeze ((GtkCList *) list);

      gtk_clist_append ((GtkCList *) list,empty_row);
	
      
      
      if (d_product_get_is_piece (product)){
	gchar *eurocode = d_piece_get_euroCode ((Piece *) product);
	gfloat cost_price = d_piece_get_cost_price ((Piece *) product);

	if (eurocode!=NULL) {
	  gtk_clist_set_text((GtkCList *) list, index, C_EUROCODE_COLUMN, eurocode);	    
	  g_free(eurocode);
	}
		
	if (cost_price!=G_MINDOUBLE) {
	  gchar *pcoste =  g_strdup_printf("%.2f",cost_price);
	  gtk_clist_set_text((GtkCList *) list, index, C_COST_PRICE_COLUMN, pcoste);
	  g_free(pcoste);

	}
      }
      
      if (description!=NULL) {
	gtk_clist_set_text((GtkCList *) list, index, C_DESCRIPTION_COLUMN, description);
	g_free(description);
      }


      if (dtomax_gint > 0){
	gchar *dto_max =  g_strdup_printf("%.2f",dtomax_gint);
	gtk_clist_set_text((GtkCList *) list, index, C_MAXIMUN_DISCOUNT_COLUMN, dto_max);
	g_free(dto_max);
      }

      if (total_gdouble!=G_MINDOUBLE) {
	gchar *total =  g_strdup_printf("%.2f",total_gdouble);
	gtk_clist_set_text((GtkCList *) list, index, C_TOTAL_COLUMN, total);
	g_free(total);
      }

      if (price_gdouble!=G_MINDOUBLE) {
	gchar *price =  g_strdup_printf("%.2f",price_gdouble);
	gtk_clist_set_text((GtkCList *) list, index, C_PRICE_COLUMN, price);
	g_free(price);
      }

      if (dto_line_gint != -1) {
	gchar *dto_line =  g_strdup_printf("%.2f",dto_line_gint);
	gtk_clist_set_text((GtkCList *) list, index, C_DISCOUNT_COLUMN, dto_line);
	g_free(dto_line);
      }
      else gtk_clist_set_text((GtkCList *) list, index, C_DISCOUNT_COLUMN,"0");
      
      if (quantity_gulong != G_MINDOUBLE) {
	gchar *quantity =  g_strdup_printf("%.3f",quantity_gulong);
	gtk_clist_set_text((GtkCList *) list, index, C_AMOUNT_COLUMN, quantity);
	g_free(quantity);
      }
      
      if (nline_gulong != ULONG_MAX) {
	gchar *nline =  g_strdup_printf("%lu",nline_gulong);
	gtk_clist_set_text((GtkCList *) list, index, C_LINE_NUMBER_COLUMN, nline);
	g_free(nline);
      }
      
      element = g_list_next(element);
      index++;
    }
    
    gtk_clist_thaw ((GtkCList *) list);
  }
  
}

static void on_change_client_button_clicked(GtkButton *button, gpointer data){
  
  GtkWidget * app1 = lookup_widget(GTK_WIDGET(button), "app1");
  GtkWidget * vbox  = lookup_widget(GTK_WIDGET(button), "main_container");
  
  is_change = TRUE;

  c_clients_control_open (app1, vbox);
  
  i_shortcut_change_dialog (GTK_WIDGET(button),0,0);  

}

static void on_insert_client_button_clicked(GtkButton *button, gpointer data){
  
  GtkWidget * app1 = lookup_widget(GTK_WIDGET(button), "app1");
  GtkWidget * vbox  = lookup_widget(GTK_WIDGET(button), "main_container");
  
  is_change = FALSE;

  if (general_client!=NULL){
    
    clean_screen();
  }
  
  c_clients_control_open (app1, vbox);
  
  i_shortcut_change_dialog (GTK_WIDGET(button),0,0);


}

void c_repair_load_client(Client *client, gboolean clean) {
  
  if (client != NULL) {
    Participant *participant = d_client_get_participant (client);
    gchar *legal_id = d_participant_get_legal_id (participant);
    gchar *nick = d_participant_get_nick (participant);
    gchar *phone1 = d_participant_get_phone (participant);
    gchar *email = d_participant_get_email (participant);
    gulong participant_code_gulong = d_participant_get_code (participant);
    gchar *participant_code = NULL;
    gdouble discount_gint = d_client_get_discount (client);
    gchar *discount = NULL;
    
    if (clean&&(!is_change)) clean_screen(); 

    if (general_client != NULL){    
      d_client_clear (general_client);
      general_client = NULL;
    }
    
    if (discount_gint!=-1)
      discount = g_strdup_printf("%.2f",discount_gint);
    
    if (legal_id != NULL) {
      GtkWidget *legal_id_entry =glade_xml_get_widget (xml, "cif_entry");
      gtk_entry_set_text (GTK_ENTRY(legal_id_entry),legal_id);
      g_free(legal_id);
    }
    
    if (nick != NULL) {
      GtkWidget *nick_entry =glade_xml_get_widget (xml, "nombre_entry");
      gtk_entry_set_text (GTK_ENTRY(nick_entry),nick);
      g_free(nick);
    }
    
    if (phone1 != NULL) {
      GtkWidget *phone1_entry =glade_xml_get_widget (xml, "telefono_entry");
      gtk_entry_set_text (GTK_ENTRY(phone1_entry),phone1);
      g_free(phone1);
    }
    
    if (discount != NULL) {
      GtkWidget *discount_entry =glade_xml_get_widget (xml, "dto_entry");
      gtk_entry_set_text (GTK_ENTRY(discount_entry),discount);
      g_free(discount);
      
      if (general_policy == NULL){
	m_repair_line_apply_discount (repair_lines, discount_gint);
	load_lines_list();
      }
      
    }
    
    if (email != NULL) {
      GtkWidget *email_entry =glade_xml_get_widget (xml, "email_entry");
      gtk_entry_set_text (GTK_ENTRY(email_entry),email);
      g_free(email);
    }
    
    if (participant_code_gulong != ULONG_MAX)
      participant_code = g_strdup_printf("%lu",participant_code_gulong);
    
    if (!is_change) fill_vehicle_combo(participant_code);
    
    if (participant_code != NULL) g_free(participant_code);
    
    general_client = client;      
    
  }
}

static void on_garage_checkbutton_toggled(GtkToggleButton *togglebutton, gpointer user_data){
  if (!cleaning){
    GtkWidget * app1 = lookup_widget(GTK_WIDGET(togglebutton), "app1");
    GtkWidget * find_garage = NULL;
    
    find_garage = c_find_garage_open_window(app1, NULL);
    
    gtk_window_set_modal(GTK_WINDOW(find_garage),1);
    gtk_window_set_position(GTK_WINDOW(find_garage),GTK_WIN_POS_CENTER);
    gtk_widget_show(find_garage);
  }
}

void c_repair_toggle_garage(gboolean active){

  GtkWidget *button =glade_xml_get_widget (xml, "taller_checkbutton");

  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), active);

}

void c_repair_toggle_contrary_garage(){

  GtkWidget *button =glade_xml_get_widget (xml, "taller_checkbutton");
  gboolean active = !gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button));

  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button),active);

}

void c_repair_insert_garage(Garage *garage){

  if (general_garage!=NULL) 
    d_garage_clear (general_garage); 

  general_garage = NULL;

  if (garage != NULL){
    Participant *participant = d_garage_get_participant (garage);
    gchar *nombre = d_participant_get_name (participant);	  

    if (nombre != NULL) {
      GtkWidget *nombre_entry =glade_xml_get_widget (xml, "taller_entry");
      gtk_entry_set_text (GTK_ENTRY(nombre_entry),nombre);
      g_free(nombre);
    }

    general_garage = d_garage_duplicate (garage);
  }
  else {
    GtkWidget *nombre_entry =glade_xml_get_widget (xml, "taller_entry");
    gtk_entry_set_text (GTK_ENTRY(nombre_entry),"");    
  }
  
}

static void on_agent_checkbutton_toggled(GtkToggleButton *togglebutton, gpointer user_data){
  if (!cleaning){  
    GtkWidget * app1 = lookup_widget(GTK_WIDGET(togglebutton), "app1");
    GtkWidget * find_agent = NULL;
    
    find_agent = c_find_agent_open_window(app1, NULL);
    
    gtk_window_set_modal(GTK_WINDOW(find_agent),1);
    gtk_window_set_position(GTK_WINDOW(find_agent),GTK_WIN_POS_CENTER);
    gtk_widget_show(find_agent);
  }
}

void c_repair_toggle_agent(gboolean active){

  GtkWidget *button =glade_xml_get_widget (xml, "agente_checkbutton");

  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), active);

}

void c_repair_toggle_contrary_agent(){

  GtkWidget *button =glade_xml_get_widget (xml, "agente_checkbutton");
  gboolean active = !gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button));

  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button),active);

}

void c_repair_insert(Agent *agent){

  if (general_agent!=NULL) 
    d_agent_clear (general_agent); 

  general_agent = NULL;

  if (agent != NULL){
    Participant *participant = d_agent_get_participant (agent);
    gchar *nick = d_participant_get_nick (participant);	  

    if (nick != NULL) {
      GtkWidget *nick_entry =glade_xml_get_widget (xml, "agente_entry");
      gtk_entry_set_text (GTK_ENTRY(nick_entry),nick);
      g_free(nick);
    }

    general_agent = d_agent_duplicate (agent);
  }
  else {
    GtkWidget *nick_entry =glade_xml_get_widget (xml, "agente_entry");
    gtk_entry_set_text (GTK_ENTRY(nick_entry),"");    
  }
  
}

static void on_insurance_checkbutton_toggled(GtkToggleButton *togglebutton, gpointer user_data){

  if (!cleaning){
    GtkWidget * app1 = lookup_widget(GTK_WIDGET(togglebutton), "app1");
    GtkWidget * insurance_data = NULL;
    gchar *client_code = NULL;
    Insurance *insurance = (Insurance *) g_malloc (sizeof(Insurance));
    
    if (general_policy==NULL)
      general_policy = d_policy_create_empty ();

    if (general_receipt==NULL)
      general_receipt = d_receipt_create_empty ();
    
    if (general_accident==NULL)
      general_accident = d_accident_create_empty ();

    if (general_client!=NULL){
      gulong client_code_gulong = d_participant_get_code (d_client_get_participant (general_client));
      
      client_code = g_strdup_printf("%lu", client_code_gulong);
    } 

    insurance -> policy = general_policy;
    insurance -> new_policy = new_policy;
    insurance -> receipt = general_receipt;
    insurance -> new_receipt = new_receipt;
    insurance -> accident = general_accident;
    insurance -> client_code = client_code;
    
    insurance_data = c_insurance_open_window(app1, insurance);
    
    g_free(insurance);
    
    gtk_window_set_modal(GTK_WINDOW(insurance_data),1);
    gtk_window_set_position(GTK_WINDOW(insurance_data),GTK_WIN_POS_CENTER);
    gtk_widget_show(insurance_data);
  }
}


void c_repair_toggle_insurance(gboolean active, gboolean local_new_policy, gboolean local_new_receipt){

  GtkWidget *button =glade_xml_get_widget (xml, "seguro_checkbutton");

  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), active);

  new_policy = local_new_policy;
  new_receipt = local_new_receipt;
  
  if (active){
    if (general_receipt!=NULL){
      gulong codigo_company = d_receipt_get_company_code (general_receipt);

      if (codigo_company!=ULONG_MAX){
	Company **companies = m_company_find_by_code (codigo_company);

	if (companies[0]!=NULL){
	  gdouble discount_gint = d_company_get_discount (companies[0]);
	  
	  if (discount_gint != -1) {	    
	    m_repair_line_apply_discount (repair_lines, discount_gint);
	    load_lines_list();
	  }

	  if (documents_window == NULL){
	    GtkWidget * app1 = lookup_widget(GTK_WIDGET(button), "app1");

	    open_doc_window(app1 );
	    
	    gtk_widget_show(documents_window);
	  }
	  
	  c_documents_load_company(companies[0]);
	  
	  companies[0] = NULL;

	  g_free(companies);
	}
      }
    }    
  }
  else  {
    if (documents_window != NULL){
      c_documents_load_company(NULL);

      gtk_widget_hide(documents_window);
    }

    if (general_client != NULL){
      gdouble discount_gint = d_client_get_discount (general_client);

      c_repair_free_insurance();

      if (discount_gint!=-1){
	GtkWidget *discount_entry =glade_xml_get_widget (xml, "dto_entry");
	gchar *discount = NULL;
	
	discount = g_strdup_printf("%.2f",discount_gint);
	gtk_entry_set_text (GTK_ENTRY(discount_entry),discount);

	m_repair_line_apply_discount (repair_lines, discount_gint);

	load_lines_list();

	g_free(discount);
      }
    }
    else {
      c_repair_free_insurance();
      m_repair_line_apply_discount (repair_lines, -1);
      load_lines_list();
    }
  }
}

void c_repair_toggle_contrary_insurance(){

  GtkWidget *button =glade_xml_get_widget (xml, "seguro_checkbutton");
  gboolean active = !gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button));

  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button),active);

  if (!active) c_repair_free_insurance();

}

void c_repair_free_insurance() {

  if (general_policy!=NULL)
      d_policy_clear (general_policy);

  general_policy = NULL;

  if (general_receipt!=NULL)
    d_receipt_clear (general_receipt);
    
  general_receipt = NULL;

  if (general_accident!=NULL)
    d_accident_clear (general_accident);
  
  general_accident = NULL;

}

void c_repair_insert_insurance(gboolean insurance) {

  if (!insurance){
    GtkWidget *number_entry =glade_xml_get_widget (xml, "seguro_entry");
    
    gtk_entry_set_text (GTK_ENTRY(number_entry),"");
    
  }
  else {

    if (general_policy != NULL) {
      gchar *number = d_policy_get_policy_number (general_policy);
      
      if (number != NULL) {
	GtkWidget *number_entry =glade_xml_get_widget (xml, "seguro_entry");
	gtk_entry_set_text (GTK_ENTRY(number_entry),number);
	g_free(number);
      }
    }
  }
  
}

static void clean_screen() {

    GtkWidget *container = GTK_WIDGET(glade_xml_get_widget(xml, widget_name));
    GtkWidget *garage_button =glade_xml_get_widget (xml, "taller_checkbutton");
    GtkWidget *agent_button =glade_xml_get_widget (xml, "agente_checkbutton");
    GtkWidget *insurance_button =glade_xml_get_widget (xml, "seguro_checkbutton");
    gchar     *s_name =NULL;
    cleaning = TRUE;

    s_name = u_parameter_manager_get_parameter ("tienda");
    if (s_name!=NULL) {
	gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget (xml, "tienda_entry")), s_name);
    }
    g_free(s_name);
        
    free_objects();
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(garage_button),FALSE);
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(agent_button),FALSE);
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(insurance_button),FALSE);
    
    gtk_container_foreach(GTK_CONTAINER(container),&c_util_clean_widget,NULL);

    fill_enterprise_from_combo_repairs();

    clean_vehicle_combo(); 

    if (documents_window != NULL){
      gtk_widget_hide(documents_window);
    }    
    
    cleaning = FALSE;

}

static void confirm_clean (gint reply, gpointer data){
  if (!reply) clean_screen();
}

static void on_clean_button_clicked(GtkButton *button, gpointer data) {
  
  c_util_open_question_dialog(_("Are you sure you want to clean the window?"), (*confirm_clean));

}

static void on_insert_piece_button_clicked(GtkButton *button, gpointer data){

  GtkWidget * list = glade_xml_get_widget (xml, "lineas_clist");
  GtkWidget * app1 = lookup_widget(GTK_WIDGET(button), "app1");
  GtkWidget * vbox  = lookup_widget(GTK_WIDGET(button), "main_container");
  GtkWidget * entry = glade_xml_get_widget (xml, "empresa_entry");

  gchar * enterprise = NULL;
  gchar * w_name = u_parameter_manager_get_parameter ("almacen");

  enterprise = gtk_entry_get_text (GTK_ENTRY(entry));

  gtk_clist_unselect_all(GTK_CLIST(list));
#ifndef IS_SHOP
  i_shortcut_change_dialog (GTK_WIDGET(list),2,1);
#else
  i_shortcut_change_dialog (GTK_WIDGET(list),2,0);
#endif

  c_find_stocked_pieces_open_window(app1, vbox);
  c_find_stocked_pieces_set_combos(enterprise, w_name);
  
  g_free(w_name);

}


static void on_edit_linea_button_clicked(GtkButton *button, gpointer data){
  
  GtkWidget * list = glade_xml_get_widget (xml, "lineas_clist");
  GArray *selected = c_util_get_selected_rows_from_clist (GTK_CLIST(list));
  
  if (selected->len>0){
    GtkWidget * app1 = lookup_widget(GTK_WIDGET(button), "app1");
    GtkWidget * repair_line_data = NULL;
    gint line = g_array_index(selected,gint,0);
    
    repair_line_data = c_repair_line_open_window(app1,(gpointer) g_list_nth_data(repair_lines, line));
    
    if (general_policy != NULL) {
      gulong codigo_company = d_policy_get_company_code (general_policy);
      Company *company = NULL;
      
      if (codigo_company!=ULONG_MAX) {
	Company **companies = m_company_find_by_code (codigo_company);
	
	if (companies[0]!=NULL){
	  company = companies[0];
	  g_free(companies);
	}
	c_repair_line_set_inssurance_company(company);
	company = NULL;
      }
    }
    gtk_window_set_modal(GTK_WINDOW(repair_line_data),1);
    gtk_window_set_position(GTK_WINDOW(repair_line_data),GTK_WIN_POS_CENTER);
    gtk_widget_show(repair_line_data);
    
  }
  
  g_array_free(selected, FALSE);  
  
}

void c_repair_on_line_changed(){
  
  load_lines_list();
  
}

static void on_delete_line_button_clicked(GtkButton *button, gpointer data){
  
  GtkWidget * list = glade_xml_get_widget (xml, "lineas_clist");
  GArray *selected = c_util_get_selected_rows_from_clist (GTK_CLIST(list));
  gint line = selected->len-1;
  gboolean one = FALSE;
  
  while (line != -1) {
      if (g_list_length(repair_lines)>1) {
	  gint nindex = g_array_index(selected,gint,line);
	  gpointer data = g_list_nth_data (repair_lines, nindex);

	  if (general_repair != NULL) {
	      gulong repair_code_gulong = d_repair_get_repair_code (general_repair);
	      gchar *repair_code = g_strdup_printf("%lu", repair_code_gulong);

	      if ( repair_code_gulong != ULONG_MAX) {
		  GtkWidget *combo = glade_xml_get_widget (xml, "tienda_combo");
		  gulong code_combo = c_util_get_code_from_combo (combo);
		  gchar * shop_code = g_strdup_printf("%lu", code_combo);
		  Enterprise *enterprise = m_enterprise_find_by_shop (code_combo);
		  gulong enterprise_code_gulong = d_participant_get_code (d_enterprise_get_participant (enterprise));
		  gchar *enterprise_code = g_strdup_printf("%lu", enterprise_code_gulong);
		  gulong line_number_gulong = d_repair_line_get_line_number ((Repair_line *) data);
		  gchar *line_number = g_strdup_printf("%lu", line_number_gulong);
		  gdouble quantity_gulong = u_round_to_decimal (d_repair_line_get_earlier_quantity ((Repair_line*) data),3);
		  Stock *stock = d_repair_line_get_stock ((Repair_line *) data);

		  if (line_number_gulong != ULONG_MAX)
		      m_repair_line_delete (repair_code, shop_code, enterprise_code, line_number, quantity_gulong, stock);

		  g_free(enterprise_code);
		  g_free(line_number);
		  g_free(shop_code);
		  d_enterprise_clear (enterprise);

	      }

	      g_free(repair_code);
	  }



	  gtk_clist_remove(GTK_CLIST(list),nindex);

	  repair_lines = g_list_remove (repair_lines, data);

      } else {
          one= TRUE;
      }
      line--;
  }
  
  if ((selected->len-1) != -1){
    m_repair_line_reallocate (repair_lines);
    load_lines_list();
  }
  if (one!=FALSE) {
      c_util_open_info_window (_("Repairs can't be emptied.\nThe first line won't be removed"));
  }
  g_array_free(selected, FALSE);
  
}

static void on_insert_line_button_clicked(GtkButton *button, gpointer data){

  GtkWidget * app1 = lookup_widget(GTK_WIDGET(button), "app1");

  c_generic_repair_line_open_window(app1, NULL);

}

static void on_insert_man_power_button_clicked(GtkButton *button, gpointer data){

  GtkWidget * app1 = lookup_widget(GTK_WIDGET(button), "app1");

  c_find_man_power_open_window(app1, NULL);

  if (general_policy != NULL) {
    gulong codigo_company = d_policy_get_company_code (general_policy);
    Company *company = NULL;
      
    if (codigo_company!=ULONG_MAX) {
      Company **companies = m_company_find_by_code (codigo_company);
      
      if (companies[0]!=NULL){
	company = companies[0];
	
	g_free(companies);
      }
         
      c_find_man_power_insuranced_repair(company);
      
      company = NULL;
      
    }
  }

}

void c_repair_add_line(Repair_line * repair_line) {
 
  repair_lines = g_list_append(repair_lines, repair_line);
  m_repair_line_reallocate (repair_lines);
  load_lines_list();

}

static void recover_vehicle_data(){

  gchar *number_plate = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "matricula_entry")),0,-1);
  gchar *bastidor = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "bastidor_entry")),0,-1);
  gchar *trademark = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "marca_entry")),0,-1);
  gchar *model = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "modelo_entry")),0,-1);
  gchar *kms = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "kms_entry")),0,-1);
  gchar *color = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "color_entry")),0,-1);
  gchar ** stop;
  gulong aux;

/*   if (general_vehicle != NULL){ */
/*     d_vehicle_clear (general_vehicle); */
/*     general_vehicle = NULL; */
/*   } */

  if (general_vehicle == NULL) general_vehicle = d_vehicle_create_empty ();
  
  stop = (gchar **) g_malloc0(sizeof(gchar *));

  if (strcasecmp(number_plate,""))
    d_vehicle_set_license_number (general_vehicle, number_plate);
  else d_vehicle_set_license_number (general_vehicle, NULL);

  if (strcasecmp(bastidor,""))
    d_vehicle_set_chassis_number (general_vehicle, bastidor);
  else d_vehicle_set_chassis_number (general_vehicle, NULL);

  if (strcasecmp(kms,"")){
    aux = strtoul(kms,stop,10);
    d_vehicle_set_kilometres (general_vehicle, aux);
  } else d_vehicle_set_kilometres (general_vehicle, ULONG_MAX);

  if (strcasecmp(model,""))
    d_vehicle_set_model (general_vehicle, model);
  else d_vehicle_set_model (general_vehicle, NULL);

  if (strcasecmp(trademark,""))
    d_vehicle_set_make (general_vehicle, trademark);
  else d_vehicle_set_make (general_vehicle, NULL);

  if (strcasecmp(color,""))
    d_vehicle_set_colour (general_vehicle, color);
  else d_vehicle_set_colour (general_vehicle, NULL);

  g_free(number_plate);
  g_free(bastidor);
  g_free(trademark);
  g_free(model);
  g_free(kms);
  g_free(color);
  g_free(stop);
}

static void recover_repair_data(){

  time_t init_date = gnome_date_edit_get_date(GNOME_DATE_EDIT(glade_xml_get_widget (xml, "fecha_entrada_dateedit")));
  time_t end_date = gnome_date_edit_get_date(GNOME_DATE_EDIT(glade_xml_get_widget (xml, "fecha_salida_dateedit")));
  gchar *observations = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "observaciones_text")),0,-1);
  gchar *distance = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "distancia_entry")),0,-1);
  gchar *place = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "lugar_entry")),0,-1);
  gchar ** stop;
  gulong aux;
  GtkWidget * combo;
  gulong code_combo;
  Enterprise * enterprise = NULL;
  gulong codigo_company = ULONG_MAX;
  Company *company = NULL;
  Shop *shop = NULL;
  gdouble total = 0.0;
  Repair_line ** lines = NULL;
  gchar * probe = NULL;
  
  combo = glade_xml_get_widget (xml, "tienda_combo");

  code_combo = c_util_get_code_from_combo (combo);

  enterprise = m_enterprise_find_by_shop (code_combo);

  if (general_repair == NULL) general_repair = d_repair_create_empty ();

  stop = (gchar **) g_malloc0(sizeof(gchar *));

  d_repair_set_in_date (general_repair, init_date);
  probe = ctime(&init_date);
  d_repair_set_out_date (general_repair, end_date);
  probe = ctime(&end_date);

  if (strcasecmp(observations,""))
    d_repair_set_observations (general_repair, observations);
  else d_repair_set_observations (general_repair, NULL);

  if (strcasecmp(place,""))
    d_repair_set_mobile_unit_place (general_repair, place);
  else d_repair_set_mobile_unit_place (general_repair, NULL);
  
  if (strcasecmp(distance,"")){
    aux = strtoul(distance,stop,10);
    d_repair_set_mobile_unit_distance (general_repair, aux);
  }
  else d_repair_set_mobile_unit_distance (general_repair, ULONG_MAX);

  shop = m_shop_find_by_shop_code (code_combo);

  if (general_policy != NULL)
    codigo_company = d_policy_get_company_code (general_policy);

  if (codigo_company!=ULONG_MAX) {
    Company **companies = m_company_find_by_code (codigo_company);
    
    if (companies[0]!=NULL){
      company = companies[0];
            
      g_free(companies);
    }
  }

  lines = m_repair_line_glist_to_array (repair_lines, &total);

  d_repair_set_enterprise (general_repair, enterprise);
  d_repair_set_shop (general_repair, shop);
  d_repair_set_company (general_repair, company);

  if (general_garage != NULL)
    d_repair_set_garage (general_repair, d_garage_duplicate (general_garage));
  else  d_repair_set_garage (general_repair, NULL);
  
  if (general_receipt != NULL) 
    d_repair_set_receipt (general_repair, d_receipt_duplicate (general_receipt));
  else d_repair_set_receipt (general_repair, NULL);
  if (general_policy != NULL) 
    d_repair_set_policy (general_repair, d_policy_duplicate(general_policy));
  else d_repair_set_policy (general_repair, NULL);
  if (general_accident != NULL) 
    d_repair_set_accident (general_repair,d_accident_duplicate (general_accident));
  else d_repair_set_accident (general_repair, NULL);
  if (general_client != NULL) 
    d_repair_set_client (general_repair, d_client_duplicate (general_client));
  else d_repair_set_client (general_repair, NULL);
  if (general_vehicle != NULL) 
    d_repair_set_vehicle (general_repair, d_vehicle_duplicate (general_vehicle));
  else d_repair_set_vehicle (general_repair, NULL);
  if (general_agent != NULL) 
    d_repair_set_agent (general_repair, d_agent_duplicate (general_agent));
  else d_repair_set_agent (general_repair, NULL);

  d_repair_set_repair_lines (general_repair, lines);
  d_repair_set_total (general_repair, total);

  g_free(stop);
  g_free(observations);
  g_free(distance);
  g_free(place);

}

static void on_manage_button_clicked(GtkButton *button, gpointer data){

    if (general_repair != NULL)
	if (repair_lines != NULL)
	    if (d_repair_get_repair_code (general_repair) != ULONG_MAX)
		if (!m_repair_is_managed (general_repair)) {
		    gboolean dialog = FALSE;

		    /*   c_invoice_open_window(app1, vbox); */

		    if (on_save_button_clicked(button, (gpointer) &dialog)) {

			gtk_widget_destroy(documents_window);
			documents_window = NULL;

			i_shortcut_change_dialog (GTK_WIDGET(button),1,0);

			c_invoice_load_repair(general_repair);

			unlink_objects();

			clean_screen();
		    }
		} else c_util_open_error_window (_("This repair has already been managed.\nYou can access the invoice through the option: \n * Search invoices. \n * Serch delivery payments"));
	    else c_util_open_error_window (_("Before the management of the repair you hace to save it"));
	else  c_util_open_error_window (_("You must insert a repair line to manage the repair"));
    else c_util_open_error_window (_("Before the management of the repair you hace to save it"));

}

static void unlink_objects(){

  general_repair = NULL;
  general_agent = NULL;
  general_garage = NULL;
  general_receipt = NULL;
  general_policy = NULL;
  general_accident = NULL;
  general_client = NULL;
  general_vehicle = NULL;
  g_list_free(repair_lines);
  repair_lines = NULL;

}

static void free_objects(){

  if (general_repair != NULL){ 
    d_repair_clear (general_repair);
    general_repair = NULL;
  } else {
    GList *element = g_list_first (repair_lines);

    if (general_agent != NULL)
      d_agent_clear (general_agent);
    if (general_garage != NULL)
      d_garage_clear (general_garage);
    if (general_receipt != NULL)
      d_receipt_clear (general_receipt);
    if (general_policy != NULL)
      d_policy_clear (general_policy);
    if (general_accident != NULL)
      d_accident_clear (general_accident);
    if (general_client != NULL)
      d_client_clear (general_client);
    if (general_vehicle != NULL)
      d_vehicle_clear (general_vehicle);

    while (element != NULL) {
      d_repair_line_clear (element->data);
      element->data = NULL;
      element = g_list_next(element);
    }

  }

  general_agent = NULL;
  general_garage = NULL;
  general_receipt = NULL;
  general_policy = NULL;
  general_accident = NULL;
  general_client = NULL;
  general_vehicle = NULL;
  repair_lines = NULL;
  new_receipt = TRUE;
  new_policy = TRUE;
  new_vehicle = TRUE;
  
}

static void on_repairs_insertion_destroy(GtkWidget *widget,gpointer user_data){

  free_objects();
}


static gboolean on_save_button_clicked(GtkButton *button, gpointer data){

  gboolean result = FALSE;

  if (general_client != NULL) {
    Exception *exception = d_exception_create ();
    
    recover_vehicle_data();
    
    recover_repair_data();

    
    
    if (!(m_repair_insert (general_repair, new_policy, new_receipt, new_vehicle, exception))){
      gchar *message = d_exception_get_message (exception);
      gchar *aux = g_strconcat(_("The repair couldn't be saved: \n"),message,"\n",NULL);
      
      c_util_open_error_window (aux);

      g_free(message);
      g_free(aux);
      
      d_exception_destroy (exception);
    }
    else {
	 GList *element = g_list_first (repair_lines);
	 
	 while (element != NULL){
	   d_repair_line_clear (element->data);
	   element->data = NULL;
	   element = g_list_next(element);
	 }

	 repair_lines = m_repair_line_array_to_glist (general_repair->repair_line);
	 
	 d_vehicle_clear (general_vehicle);
	 
	 general_vehicle = d_vehicle_duplicate (general_repair->vehicle);
	 
	 new_vehicle = FALSE;

	 if (general_policy != NULL) {
	   d_policy_set_old_key (general_policy);
	   new_policy = FALSE;
	 }

	 if (general_receipt != NULL) new_receipt = FALSE;
	 
	 result = TRUE;

	 if (data == NULL){	   
	   gchar *message = d_exception_get_message (exception);
	   gchar *aux = NULL;
	   
	   if (message != NULL) 
	     aux = g_strconcat(_("Repair successfully saved. WARNING: \n"),message,"\n",NULL);
	   else aux = g_strdup(_("Repair successfully saved"));
	   
	   c_util_open_info_window (aux);

	   g_free(aux);
	   g_free(message);
	 }

	 d_exception_destroy (exception);

	 exception = NULL;
    }
  }
  else c_util_open_error_window (_("To insert a repair you must select a client"));
  
  return result;
}


static void fill_vehicle_combo(gchar *codigo) {

  GtkWidget * combo;
  Vehicle **vehicles;

  combo = glade_xml_get_widget (xml, "vehiculo_combo");

  clean_vehicle_combo();

  if (codigo != NULL){
    vehicles = m_vehicle_find_by_name (NULL,codigo);
    c_util_fill_combo_with_vehicles (combo,vehicles);
  }

}


static void clean_vehicle_combo() {
    GtkWidget * combo;

    combo = glade_xml_get_widget (xml, "vehiculo_combo");
    c_util_clean_code_and_name_combo (combo); 

    c_util_set_code_and_name_on_combo (C_CODIGO_INSERCION,C_INSERTION_MESSAGE,GTK_COMBO(combo));

}

static void fill_vehicle(Vehicle *vehicle) {

  if (vehicle!=NULL) {
    gchar *number_plate = d_vehicle_get_license_number (vehicle);
    gchar *chasis_number = d_vehicle_get_chassis_number (vehicle);
    gchar *model = d_vehicle_get_model (vehicle);
    gchar *trademark = d_vehicle_get_make (vehicle);
    gchar *color = d_vehicle_get_colour (vehicle);
    gulong kms_gulong = d_vehicle_get_kilometres (vehicle);
    gchar *kms = NULL;   
   
    if (kms_gulong != ULONG_MAX)
      kms = g_strdup_printf("%lu",kms_gulong);
    
    if (number_plate != NULL) {
      GtkWidget *number_plate_entry = glade_xml_get_widget (xml, "matricula_entry");
      gtk_entry_set_text (GTK_ENTRY(number_plate_entry), number_plate);
      g_free(number_plate);
      }
    
    if (chasis_number != NULL) {
      GtkWidget *chasis_number_entry = glade_xml_get_widget (xml, "bastidor_entry");
      gtk_entry_set_text (GTK_ENTRY(chasis_number_entry), chasis_number);
      g_free(chasis_number);
    }
    
      if (model != NULL) {
	GtkWidget *model_entry = glade_xml_get_widget (xml, "modelo_entry");
	gtk_entry_set_text (GTK_ENTRY(model_entry), model);
	g_free(model);
      }
      
      if (trademark != NULL) {
	GtkWidget *trademark_entry = glade_xml_get_widget (xml, "marca_entry");
	gtk_entry_set_text (GTK_ENTRY(trademark_entry), trademark);
	g_free(trademark);
      }
      
      if (color != NULL) {
	GtkWidget *color_entry = glade_xml_get_widget (xml, "color_entry");
	gtk_entry_set_text (GTK_ENTRY(color_entry), color);
	g_free(color);
      }
      
      if (kms != NULL) {
	GtkWidget *kms_entry = glade_xml_get_widget (xml, "kms_entry");
	gtk_entry_set_text (GTK_ENTRY(kms_entry), kms);
	g_free(kms);
      }
      
      if (general_vehicle != NULL) 
	d_vehicle_clear (general_vehicle);

      general_vehicle = vehicle;
    }
  
}


static void on_vehicle_combo_entry_changed(GtkEntry *entry, gpointer user_data){
  
  GtkWidget * local_entry;
  GtkWidget * combo;
  gulong code_combo = ULONG_MAX;
  
  local_entry = glade_xml_get_widget (xml, "vehiculo_combo_entry");
  
  combo = glade_xml_get_widget (xml, "vehiculo_combo");
  
  code_combo = c_util_get_code_from_combo (combo);
  
  if (code_combo != C_CODIGO_INSERCION) {
    Vehicle ** vehicles = NULL;
    gchar *vehicle_code = NULL;
    
    vehicle_code = g_strdup_printf("%lu",code_combo);
    
    vehicles = m_vehicle_find_by_name (vehicle_code, NULL);
    
    if (vehicles != NULL) {
      fill_vehicle(vehicles[0]);    

      g_free(vehicles);
    }
    
    g_free(vehicle_code);

    new_vehicle = FALSE;
  }
  else{
    GtkWidget * frame = glade_xml_get_widget (xml, "vehiculo_frame");
    
    gtk_container_foreach(GTK_CONTAINER(frame),&c_util_clean_widget,NULL);

    if (general_vehicle != NULL) {
      d_vehicle_clear (general_vehicle);
      general_vehicle = NULL;
    }
    
    new_vehicle = TRUE;
  }
  
}

static void on_repairs_insertion_window_hide(GtkWidget *widget, gpointer user_data) {

  if (documents_window != NULL){
	gtk_widget_destroy(documents_window);
        documents_window=NULL;
  }
  
}

void c_repair_document_destroyed() {

  documents_window = NULL;

}


void c_repair_clean_screen() {

	clean_screen();
}
