
/* stock_control.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include <gnome.h>
#include <glade/glade.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <glib.h>
#include <time.h>

#include "a-shortcut.h"
#include "support.h"
#include "stock_control.h"
#include "stock_model.h"
#include "regularization.h"
#include "regularization_model.h"
#include "find_stocked_pieces_model.h"
#include "warehouse_model.h"
#include "enterprise_model.h"

static GtkWidget *widget_stock=NULL;
static GladeXML *xml = NULL;
static void (*repaint_pieces_to_call)(void);

static gchar url[]="/src/interface/glade/store_view.glade";
static gchar widget_name[]="dialog-stock";
static Stock *stock_loaded;

static void on_dialog_stock_destroy(GtkWidget *widget,gpointer user_data);
static void close_stock_window();
static void on_cancel_button_stock_clicked(GtkButton *button, gpointer data);
static void on_accept_button_stock_clicked(GtkButton *button, gpointer data);


GtkWidget *c_piece_stock_open_window(GtkWidget *app1, gpointer repaint_pieces) {

    gchar * directory = g_strconcat(AA_DATA_DIR,url,NULL);

    if (widget_stock==NULL) {
	GtkWidget *dialog;

	xml = glade_xml_new (directory, widget_name);
	dialog = glade_xml_get_widget (xml, widget_name);

	glade_xml_signal_connect (xml, "on_button-aceptar-stock_clicked",
				  GTK_SIGNAL_FUNC (on_accept_button_stock_clicked));
	glade_xml_signal_connect (xml, "on_button-cancelar-stock_clicked",
				  GTK_SIGNAL_FUNC (on_cancel_button_stock_clicked));
	glade_xml_signal_connect (xml, "on_dialog-stock_destroy",
				  GTK_SIGNAL_FUNC (on_dialog_stock_destroy));

	widget_stock =dialog;
    }
    repaint_pieces_to_call=repaint_pieces;
    d_stock_clear (stock_loaded);
    stock_loaded = NULL;
    g_free(directory);
    return widget_stock;
}

void c_piece_stock_load(Stock *stock, gchar *enterprise_name,
			gchar *warehouse_name) {

    GtkWidget *spinbutton =  NULL;
    gchar *eurocode    = g_strstrip(d_piece_get_euroCode (d_stock_get_piece (stock)));
    gchar *description = g_strstrip(d_product_get_description ((Product *)d_stock_get_piece (stock)));
    gchar *family_codetype = g_strstrip(d_product_get_family_code ((Product *)d_stock_get_piece (stock)));
    gchar *location   = d_stock_get_location (stock);

    Piece *piece = d_piece_create_empty ();

    if (location!=NULL) {
        location = g_strstrip(location);
	gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget (xml, "entry-ubicacion-stock")),location);
	gtk_entry_set_position(GTK_ENTRY(glade_xml_get_widget (xml, "entry-ubicacion-stock")),0);
    }

    d_stock_clear (stock_loaded);
    stock_loaded = NULL;

    d_piece_set_euroCode (piece, g_strdup(eurocode));
    d_piece_set_width (piece, d_piece_get_width (d_stock_get_piece (stock)));
    d_piece_set_length (piece, d_piece_get_length (d_stock_get_piece (stock)));
    d_piece_set_cost_price (piece, d_piece_get_cost_price (d_stock_get_piece (stock)));

    d_product_set_family_code ((Product *)piece, family_codetype);
    d_product_set_description ((Product *)piece, description);
    d_product_set_product_code ((Product *)piece, d_product_get_product_code ((Product *) d_stock_get_piece (stock)));
    d_product_set_vat_type ((Product *)piece, d_product_get_vat_type ((Product *) d_stock_get_piece (stock)));
    d_product_set_is_piece ((Product *)piece, d_product_get_is_piece ((Product *) d_stock_get_piece (stock)));
    d_product_set_is_man_power ((Product *)piece, d_product_get_is_man_power ((Product *) d_stock_get_piece (stock)));
    d_product_set_price ((Product *)piece, d_product_get_price ((Product *) d_stock_get_piece (stock)));

    stock_loaded = d_stock_create_empty ();
    d_stock_set_piece (stock_loaded, piece);
    d_stock_set_location (stock_loaded, location);
    d_stock_set_warehouse_code (stock_loaded, d_stock_get_warehouse_code (stock));
    d_stock_set_enterprise_code (stock_loaded, d_stock_get_enterprise_code (stock));
    d_stock_set_quantity (stock_loaded, d_stock_get_quantity (stock));
    d_stock_set_previous_quantity (stock_loaded, d_stock_get_previous_quantity (stock));
    d_stock_set_minimum_stock (stock_loaded, d_stock_get_minimum_stock (stock));

    gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(xml, "entry-eurocode-stock")), eurocode);
    gtk_entry_set_position(GTK_ENTRY(glade_xml_get_widget(xml, "entry-eurocode-stock")), 0);

    gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget (xml, "entry-descripcion-stock")),description);
    gtk_entry_set_position(GTK_ENTRY(glade_xml_get_widget (xml, "entry-descripcion-stock")),0);

    gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget (xml, "entry-empresa-stock")),g_strdup(enterprise_name));
    gtk_entry_set_position(GTK_ENTRY(glade_xml_get_widget (xml, "entry-empresa-stock")),0);

    gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget (xml, "entry-almacen-stock")),g_strdup(warehouse_name));
    gtk_entry_set_position(GTK_ENTRY(glade_xml_get_widget (xml, "entry-almacen-stock")),0);

    spinbutton =  glade_xml_get_widget(xml, "spinbutton-unidades-stock");
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbutton), (gfloat) d_stock_get_quantity (stock_loaded));

    spinbutton =  glade_xml_get_widget(xml, "spinbutton-stockminimo-stock");
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbutton), (gfloat) d_stock_get_minimum_stock (stock_loaded));

    g_free(eurocode);
    g_free(description);
    g_free(location);
}

static void on_dialog_stock_destroy(GtkWidget *widget,gpointer user_data){
    g_print(_("Update stock window closed\n"));
}


static void close_stock_window() {
    gtk_widget_destroy(widget_stock);
    widget_stock = NULL;
}

static void on_cancel_button_stock_clicked(GtkButton *button,
					     gpointer data) {
    close_stock_window();
    if (repaint_pieces_to_call!=NULL)
        repaint_pieces_to_call();
}

static void on_accept_button_stock_clicked(GtkButton *button,
					    gpointer data) {

    GtkSpinButton *spinbutton =  NULL;
    gchar     *location = g_strstrip(gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "entry-ubicacion-stock")),0,-1));
    gdouble     quantity;
    gdouble     quantity_regularization;
    gdouble     minimun_stock;
    gboolean   correct = FALSE;
    Regularization *regularization;

    if (stock_loaded!=NULL) {
        Product product = d_piece_get_product (d_stock_get_piece (stock_loaded));
	gtk_entry_set_position(GTK_ENTRY(glade_xml_get_widget(xml, "entry-ubicacion-stock")), 0);

	spinbutton =  GTK_SPIN_BUTTON(glade_xml_get_widget(xml, "spinbutton-unidades-stock"));
	gtk_spin_button_update(spinbutton);
	quantity      = (gdouble)gtk_spin_button_get_value_as_float(spinbutton);

	spinbutton =  GTK_SPIN_BUTTON(glade_xml_get_widget(xml, "spinbutton-stockminimo-stock"));
	gtk_spin_button_update(spinbutton);
	minimun_stock = (gdouble)gtk_spin_button_get_value_as_float(spinbutton);

	quantity_regularization = quantity - d_stock_get_quantity (stock_loaded);
	regularization = d_regularization_create_empty ();

        d_regularization_set_enterprise_code (regularization, d_stock_get_enterprise_code (stock_loaded));
        d_regularization_set_warehouse_code (regularization, d_stock_get_warehouse_code (stock_loaded));
        d_regularization_set_product_code (regularization, d_product_get_product_code(&product));
	d_regularization_set_quantity (regularization, quantity_regularization);

	d_stock_set_location (stock_loaded, location);
	d_stock_set_quantity (stock_loaded, quantity);
	d_stock_set_minimum_stock (stock_loaded, minimun_stock);

	correct = m_regularization_insert (regularization, stock_loaded);

	if(!correct) {
	    c_util_open_error_window (_("Stock regularization error. There has been a problem accessing the database.\n Try it again"));
	}

	d_stock_clear (stock_loaded);
	stock_loaded = NULL;
    }
    g_free(location);
    close_stock_window();
    if (repaint_pieces_to_call!=NULL)
	repaint_pieces_to_call();

}

