/* supplier_price_control.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include <gnome.h>
#include <glade/glade.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <glib.h>
#include <stdlib.h>


#include "a-shortcut.h"
#include "support.h"
#include "parameter_manager.h"
#include "control_util.h"
#include "supplier_price_control.h"
#include "invoice_control.h"

static GtkWidget *supplier_price_widget=NULL;
static GladeXML *xml;
static SupplierPrice * general_supplier_price;
static void (*function_to_call)();
static gchar widget_name[]="window_provider_product";
static gchar url[]="/src/interface/glade/product_view.glade";


static void on_cancel_cost_price_button_clicked(GtkButton *button, gpointer user_data);
static void on_accept_cost_price_button_clicked(GtkButton *button,  gpointer user_data);
static void fill_supplier_combo(void);

GtkWidget * c_supplier_price_open_window(GtkWidget *app1,gpointer f_to_call) {
    gchar * directory = g_strconcat(AA_DATA_DIR,url,NULL);

    if (supplier_price_widget==NULL)
    {
	GtkWidget *dialog;

	xml = glade_xml_new (directory, widget_name);
	dialog = glade_xml_get_widget (xml, widget_name);


	glade_xml_signal_connect (xml, "on_button_cancelar_precio_coste_clicked",
				  GTK_SIGNAL_FUNC (on_cancel_cost_price_button_clicked));
	glade_xml_signal_connect (xml, "on_button_aceptar_precio_coste_clicked",
			          GTK_SIGNAL_FUNC (on_accept_cost_price_button_clicked));
                             
        fill_supplier_combo();
	supplier_price_widget=dialog;
    }
    function_to_call = f_to_call;
    g_free(directory);
    return supplier_price_widget;
}


static void on_cancel_cost_price_button_clicked(GtkButton *button,gpointer user_data) {
    gtk_widget_destroy(supplier_price_widget);
    d_supplier_price_clear (general_supplier_price);
    function_to_call = NULL;
    supplier_price_widget = NULL;
}

static void on_accept_cost_price_button_clicked(GtkButton *button,gpointer user_data) {
    gchar * price = g_strdup(gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(xml,"entry_precio_coste"))));
    gulong supplier_code = c_util_get_code_from_combo (glade_xml_get_widget(xml,"combo_proveedor"));
    gboolean result = FALSE;
    if (strcmp(price,"")) {
	if (d_supplier_price_get_participant (general_supplier_price)!=NULL) {
	    if (supplier_code != d_participant_get_code (d_supplier_price_get_participant (general_supplier_price))) {
		Participant * party = m_client_get_participant_by_code (supplier_code);
		d_supplier_price_set_participant (general_supplier_price,party);
	    }
	} else {
	    Participant * party = m_client_get_participant_by_code (supplier_code);
	    d_supplier_price_set_participant (general_supplier_price,party);
	}

        u_points_to_commas (price);
	d_supplier_price_set_cost_price (general_supplier_price,atof(price));

	result = m_supplier_price_insert (general_supplier_price);
	if (result ==TRUE) {
	    gtk_widget_destroy(supplier_price_widget);
	    d_supplier_price_clear (general_supplier_price);
	    function_to_call();
            function_to_call=NULL;
	    supplier_price_widget = NULL;
	} else {
	    c_util_open_info_window (_("Error inserting in the database"));

	}
    } else {
        c_util_open_info_window (_("Insert a cost price"));
    }


}




static void fill_supplier_combo(void) {
    GtkWidget * combo = glade_xml_get_widget(xml,"combo_proveedor");
    Supplier ** suppliers = m_supplier_find_all ();

    c_util_fill_combo_with_suppliers (combo,suppliers);

    m_supplier_clear_array (suppliers);
}


void c_supplier_price_load_data(SupplierPrice * supplier_price,gboolean modify) {
    gchar * eurocode = NULL;
    gchar * name = NULL;
    gchar * price = NULL;
    GtkWidget * price_entry = glade_xml_get_widget(xml,"entry_precio_coste");
    GtkWidget * product_entry = glade_xml_get_widget(xml,"entry_producto");
    GtkWidget * supplier_entry = glade_xml_get_widget(xml,"entry_combo_proveedor");
    GtkWidget * supplier_combo = glade_xml_get_widget(xml,"combo_proveedor");
    GtkWidget * button = glade_xml_get_widget(xml,"button_buscar_producto");
    
    if (general_supplier_price!=NULL) {
	general_supplier_price = NULL;
    }
    general_supplier_price = d_supplier_price_duplicate (supplier_price);

    eurocode = d_piece_get_euroCode (d_supplier_price_get_piece (supplier_price));
    gtk_entry_set_text(GTK_ENTRY(product_entry),eurocode);
    g_free(eurocode);

    if (modify!=FALSE) {

	name = d_participant_get_nick (d_supplier_price_get_participant (supplier_price));
	gtk_entry_set_text(GTK_ENTRY(supplier_entry),name);
	g_free(name);

	if (d_supplier_price_get_cost_price (supplier_price)!=G_MINDOUBLE) {
	    price = g_strdup_printf("%.2f",d_supplier_price_get_cost_price (supplier_price));
	} else {
	    price = g_strdup("0.0");
	}
	gtk_entry_set_text(GTK_ENTRY(price_entry),price);
	g_free(price);

	if GTK_WIDGET_SENSITIVE(supplier_entry) gtk_widget_set_sensitive(supplier_entry,FALSE);
	if GTK_WIDGET_SENSITIVE(supplier_combo) gtk_widget_set_sensitive(supplier_combo,FALSE);
    } else {
	if (!GTK_WIDGET_SENSITIVE(supplier_entry)) gtk_widget_set_sensitive(supplier_entry,TRUE);
	if (!GTK_WIDGET_SENSITIVE(supplier_combo)) gtk_widget_set_sensitive(supplier_combo,TRUE);
    }

}


