/* cuaderno58.c
 *  *
 *  * Copyright (C) 2001, 2002 Igalia, S.L..
 *  *
 *  * This program is free software; you can redistribute it and/or
 *  * modify it under the terms of version 2 of the GNU General Public
 *  * License as published by the Free Software Foundation.
 *  *
 *  * This program is distributed in the hope that it will be useful,
 *  * but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  * General Public License for more details.
 *  *
 *  * You should have received a copy of the GNU General Public
 *  * License along with this program; if not, write to the
 *  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *  * Boston, MA 02111-1307, USA.
 *  *
 * Author: Igalia S.L.
 *  */
#include "config.h"
#include "cuaderno58.h"

#include "enterprise_model.h"
#include "office_model.h"
#include "invoice.h"
#include "control_util.h"
#include "parameter_manager.h"
#include "client_model.h"
#include "account_model.h"
#include "invoice_model.h"
#include "payment_model.h"
#include "util.h"
#include "direct_debit_model.h"

#include <glib.h>
#include <time.h>


/**
 * I_BOOK_PRESENTER_HEADER_CODE:
 *
 * Define el cdigo de la cabecera de presentador del cuaderno 58.
 *
 */
#define I_BOOK_PRESENTER_HEADER_CODE "5170"
/**
 * I_BOOK_ISSUER_HEADER_CODE:
 *
 * Define el cdigo de la cabecera de ordenante del cuaderno 58.
 *
 */
#define I_BOOK_ISSUER_HEADER_CODE "5370"
/**
 * I_BOOK_INDIVIDUAL_MANDATORY_CODE:
 *
 * Define el cdigo del registro individual del cuaderno 58.
 *
 */
#define I_BOOK_INDIVIDUAL_MANDATORY_CODE "5670"
/**
 * I_BOOK_ISSUER_TOTAL_CODE:
 *
 * Define el cdigo del total por ordenante del cuaderno 58.
 *
 */
#define I_BOOK_ISSUER_TOTAL_CODE "5870"
/**
 * I_BOOK_GENERAL_TOTAL_CODE:
 *
 * Define el cdigo del total por general del cuaderno 58.
 *
 */
#define I_BOOK_GENERAL_TOTAL_CODE "5970"

/**
 * I_BOOK_ISSUER_TOTAL:
 *
 * Define el cdigo del total por ordenante del cuaderno 58. DUPLICADO.
 *
 */
#define I_BOOK_ISSUER_TOTAL "5870"
/**
 * I_BOOK_GENERAL_TOTAL:
 *
 * Define el cdigo del total por general del cuaderno 58. DUPLICADO.
 *
 */
#define I_BOOK_GENERAL_TOTAL "5970"
/**
 * I_BOOK_STANDARD_SUFFIX:
 *
 * Indica el nmero de sufijo estndar que se inserta despues de los cdigos de cabeceras.
 *
 */
#define I_BOOK_STANDARD_SUFFIX "000"
/**
 * I_BOOK_FIRST_PROCEDURE:
 *
 * Cdigo que identifica el procedimiento primero.
 *
 */

#define I_BOOK_FIRST_PROCEDURE "01"

/**
 * I_BOOK_SIXTH_PROCEDURE:
 *
 * Cdigo que identifica el procedimiento sexto.
 *
 */
#define I_BOOK_SIXTH_PROCEDURE "06"

/**
 * I_BOOK58_FILE:
 *
 * Parte general del nombre del fichero en el que se generar el cuaderno 58.
 *
 */
#define I_BOOK58_FILE "cuaderno58"


/**
 * I_BOOK_DECIMAL_CHARACTER:
 *
 * Indica cual es el carcter utilizado para los decimales.
 *
 */
#define I_BOOK_DECIMAL_CHARACTER ","

typedef struct _file_and_total {
  FILE * out_file;
  GList * total_list;
  gulong n_direct_debit;
} FileAndTotal;

typedef struct _total_data_ {
    FILE * out_file;
    gchar * issuer_code;
    gdouble total_sum;
} TotalData;


static void book58_write_each_issuer_hash(gchar* company_id, GHashTable * client_invoices, FileAndTotal * file_and_total);
static void book58_write_each_individual_list(gulong client_id,GList * invoices, TotalData * total_data);
static gchar * convert_to_string_without_decimal_character(gdouble number, gint length);
static gdouble calculate_total_from_gdouble_glist(GList * list);
static gint get_first_number_of_line(gchar * line);
static gchar * get_caracters_from_position(gchar * line,gint position,gint number_of_caracters);
static gchar * get_out_first_zeros(gchar * nif);
static void fill_struct_code_from_line(DirectDebit * dev,gchar * temp,gulong enterprise_code);
static void count_records(gulong issuer_code, GList * dev, gint * total);
static void proccess_direct_debits(gulong issuer_code, GList * dev, gboolean * result);
static gboolean book58_free_each_issuer_hash(gchar* company_id, GHashTable * client_invoices,gpointer user_data);
static gboolean book58_free_each_individual_list(gulong client_id,GList * invoices, TotalData * total_data);
static gboolean return_free_each_issuer_hash(gulong issuer_code, GList * dev,gpointer data);

/* Returns the gdouble with the sum over all the elements in the GList */
/* It is assumed that the list is not NULL */
static gdouble calculate_total_from_gdouble_glist(GList * list) {
  GList * element;
  gdouble total = 0;
  gdouble * total_pointer;

  element = g_list_first(list);

  if (element == NULL) {
    return 0;
  }
  else {
    do {
      total_pointer = (gdouble *) element->data;
      total = total + (*total_pointer);
      element = g_list_next(element);    
    } 
    while (element != NULL);
  }
  
  return total;
} 


/* Converts a gdouble to a string of size 'length' without decimal delimiter */
static gchar * convert_to_string_without_decimal_character(gdouble number, gint length)
{
  gchar * string;
  gchar ** total_part;
  gchar * total_without_decimal_character;

  string = g_strdup_printf("%.2f", number);
  total_part = g_strsplit (string,I_BOOK_DECIMAL_CHARACTER,2); 
  g_free(string);

  string = g_strconcat(total_part[0], total_part[1],NULL);
  g_strfreev(total_part);

  total_without_decimal_character = u_string_fill_left ('0', length, string);

  return total_without_decimal_character;
}


static void free_gdouble_list (GList * list) {
  GList * element;

  element = g_list_first(list);

  do {
    g_free(element->data);
    element = g_list_next(element);
  }
  while (element != NULL);

  g_list_free(list);
}


/**
 * i_book58_generate_from_invoice_list:
 * @invoices: Arra de facturas que contiene todas las facturas que estn actualmente en memoria.
 * @selected: #GArray que contiene la lista de facturas seleccionadas.
 *
 * Genera el cuaderno 58 a partir de la lista de facturas y de las seleccionadas de la misma. El fichero
 * generado es cuaderno58-fecha_de_hoy. La lista de facturas se recorre y las que se han seleccionado se almacenan
 * en una hash segn el cliente ordenante, adems se almacenan en una hash por receptor de factura, ya que si
 * existen varias facturas del mismo cliente se metern en una sola domiciliacin. Se procesan y se escribe
 * el fichero con el cuaderno 58.
 *
 */
void i_book58_generate_from_invoice_list (Invoice ** invoices, GArray * selected) {
  
  g_print(_("(cuaderno58.c) -cuaderno58_generate_from_invoice_list- Generating Cuaderno58...\n"));
  
  if (invoices != NULL) {
    FILE * out_file;
    gint index = 0; 
    gchar * company_id;
    gchar * client_id;
    GList * content = NULL;

    gchar * presenter_header;
    gchar * presenter_client_name;
    gchar * presenter_client_name_filled;
    gchar * today_char;
    gchar * receiver_entity;
    gchar * office;    
    time_t timer;
    gchar * spaces6;
    gchar * spaces20;
    gchar * spaces66;
    GHashTable *hash_table = g_hash_table_new(g_str_hash, g_str_equal);
    GHashTable *hash_table_by_client ;
    GList * account_list;
    GList * element;
    Participant * participant;
    gchar * presenter_legal_id;    
    FileAndTotal * file_and_total;
    gchar * general_total;
    gchar * spaces52;
    gchar * issuers_number;
    gchar * spaces16;
    gchar * total_sum;
    gchar * direct_debit_total_num;
    gchar * spaces38;
    gchar * records_total_number;

    gchar * cuaderno58_file;
    gint i;

    /* Obtaining the date for today as a string in today_char */
    today_char=(gchar *) g_malloc(sizeof(gchar)*10);
    timer=time(NULL);
    strftime(today_char,sizeof(gchar)*10,"%d%m%y",localtime(&timer));
    
    cuaderno58_file = g_strconcat(DIR_SPOOL,"/",I_BOOK58_FILE,"/",I_BOOK58_FILE,"-",today_char,NULL);
    g_free(today_char);

    /*Open the file to write*/
    out_file = fopen(cuaderno58_file, "w");
    if (out_file!=NULL) {
	g_print(_("(cuaderno58.c) -cuaderno58_generate_from_invoice_list- The file will be written at: %s"), cuaderno58_file);

	g_print(_("(cuaderno58.c) -cuaderno58_generate_from_invoice_list- Obtaining account data\n"));

	/* We take the PRESENTADOR from the configuration parameters of the app */
	presenter_client_name = u_parameter_manager_get_parameter ("empresa");
	participant = m_client_get_participant_by_name (presenter_client_name,M_CLIENT_ENTERPRISE_CODE,TRUE);
	presenter_legal_id= u_string_fill_left ('0',9,g_strdup_printf("%s",d_participant_get_legal_id (participant)));


	/*Search the client's bank account*/
	account_list = m_account_find_by_participant_code (d_participant_get_code (participant));
	d_participant_clear (participant); /* Was created by get_participant_by_name */

	/*FIXME: We suppose the first bank account is the account to direct debit*/
	element = g_list_first(account_list);

	if (element != NULL) {
	    receiver_entity = g_strdup_printf("%04d",d_office_get_bank_code (d_account_get_office (element->data)));
	    office = g_strdup_printf("%04d",d_office_get_office_code (d_account_get_office (element->data)));
	    d_account_free_glist (account_list); /* Was created by model_find_account_by_party_code */
	}
	else {
	    c_util_open_info_window (_("The host being processed hasn't got bank account information\n Asterisks are going to be added at the correspondent place in the debits file"));
	    receiver_entity = g_strdup("****");
	    office = g_strdup("****");
	}

	/* Obtaining the date for today as a string in today_char */
	today_char=(gchar *) g_malloc(sizeof(gchar)*10);
	timer=time(NULL);
	strftime(today_char,sizeof(gchar)*10,"%d%m%y",localtime(&timer));

	spaces6 = g_strnfill(6,' ');
	spaces20 = g_strnfill(20,' ');
	spaces66 = g_strnfill(66,' ');

	presenter_client_name_filled = g_strdup_printf("%-40s",presenter_client_name);
	g_free(presenter_client_name);

	g_print(_("(cuaderno58.c) -cuaderno58_generate_from_invoice_list- Generating 'presenter_header'\n"));

	/* Standard head for the PRESENTADOR in 'Cuaderno 58' */
	presenter_header =
	    g_strconcat(
			I_BOOK_PRESENTER_HEADER_CODE, /* (4) */
			presenter_legal_id, /* (9) */
			I_BOOK_STANDARD_SUFFIX, /* (3) */
			today_char, /* Current time (6) DDMMAA */
			spaces6, /* (6) */
			presenter_client_name_filled, /* Nombre del cliente presentador (40) */
			spaces20, /* */
			receiver_entity, /* */
			office,
			spaces66, /* (12+40+14) */
			"\r\n",
			NULL);

	g_free(today_char);
	g_free(presenter_client_name_filled);
	g_free(spaces20);
	g_free(receiver_entity);
	g_free(office);
	g_free(spaces66);

	fprintf(out_file, presenter_header);
	g_free(presenter_header);

	g_print(_("(cuaderno58.c) -cuaderno58_generate_from_invoice_list- 'presenter_header' generated successfully\n"));

	g_print(_("(cuaderno58.c) -cuaderno58_generate_from_invoice_list- The list of invoices is not empty\n"));

	for (i=0; i< selected->len; i++) {
	    index  = g_array_index(selected,gint,i); /* Index takes only the selected invoices */

	    if (d_payment_get_is_direct_debit (d_invoice_get_payment (invoices[index]))
		&&
		(d_invoice_get_state (invoices[index]) != D_INVOICE_PAID)
		&&
		(d_invoice_get_state (invoices[index]) != D_INVOICE_MANAGING)
	       ) {
		g_print(_("(cuaderno58.c) -cuaderno58_generate_from_invoice_list-  The invoice number %d is a payment by 'domiciliacion'\n"), index);

		/* Gets the identifier of the enterprise that created the current invoice */
		company_id = g_strdup_printf("%lu",d_participant_get_code (d_enterprise_get_participant (d_invoice_get_enterprise (invoices[index]))));
		/* We create a hash by issuer*/
		hash_table_by_client = g_hash_table_lookup(hash_table, company_id);
		if (hash_table_by_client==NULL) {
		    hash_table_by_client = g_hash_table_new(g_str_hash, g_str_equal);

		}
		client_id = g_strdup_printf("%lu",d_participant_get_code (d_invoice_get_participant (invoices[index])));
		/*Each hash "by ordenante" has a hash by client_id*/
		content = g_hash_table_lookup(hash_table_by_client, client_id);
		g_hash_table_insert (hash_table_by_client, client_id, g_list_append(content,invoices[index]));
		g_hash_table_insert (hash_table,company_id,hash_table_by_client);
	    }
	}

	g_print(_("(cuaderno58.c) -cuaderno58_generate_from_invoice_list- After processing the invoices, the size of the hash table: %d\n"),
		g_hash_table_size(hash_table));

	if (g_hash_table_size(hash_table) == 0) c_util_open_error_window (_("There aren't any selected invoices are debits don't paid\n cuaderno58 will be generated empty."));

	/* Now we are going to process each of the entries in the hash table and their associated list of invoices */

	/* file_and_total is used for passing foreach functional argument one parameter and to obtain a total as result */
	file_and_total = (FileAndTotal *) g_malloc0(sizeof(FileAndTotal));
	file_and_total->out_file = out_file;
	file_and_total->total_list = NULL;
	file_and_total->n_direct_debit = 0;

	/*Aplicamos la funcin de escritura por ordenante a cada elemento de la hash de ordenantes.*/
	g_hash_table_foreach(hash_table, (void *) (*book58_write_each_issuer_hash), file_and_total);

	g_print(_("(cuaderno58.c) -cuaderno58_generate_from_invoice_list- Generating 'total general'\n"));

	spaces52 = g_strnfill(52, ' ');
	issuers_number = g_strdup_printf("%04d",g_hash_table_size(hash_table));
	spaces16 = g_strnfill(16, ' ');

	/* Adds all the 'importes' returned by the different 'write_each_ordenante_hash' functions */
	total_sum = convert_to_string_without_decimal_character(calculate_total_from_gdouble_glist(file_and_total->total_list), 10);

	direct_debit_total_num = g_strdup_printf("%010lu", file_and_total->n_direct_debit);
	records_total_number = g_strdup_printf("%010lu",  file_and_total->n_direct_debit+ g_hash_table_size(hash_table)*2 + 2 );
	/* FIXME: Could change if using the 'complex' method */

	spaces38 = g_strnfill(38, ' ');

	general_total =
	    g_strconcat(
			I_BOOK_GENERAL_TOTAL_CODE,
			presenter_legal_id, /* (9) */
			I_BOOK_STANDARD_SUFFIX, /* (3) */
			spaces52,
			issuers_number,
			spaces16,
			total_sum,
			spaces6,
			direct_debit_total_num,
			records_total_number,
			spaces38,
			"\r\n",
			NULL);


	g_free(presenter_legal_id);
	g_free(spaces52);
	g_free(issuers_number);
	g_free(spaces16);
	g_free(total_sum);
	g_free(spaces6);
	g_free(direct_debit_total_num);
	g_free(records_total_number);
	g_free(spaces38);

	fprintf(out_file, general_total);
	g_free(general_total);

	g_print(_("(cuaderno58.c) -cuaderno58_generate_from_invoice_list- 'total general' successfully generated\n"));

	g_print(_("(cuaderno58.c) -cuaderno58_generate_from_invoice_list- Cuaderno58 generated\n"));

	fclose(out_file);
	if (file_and_total->total_list) free_gdouble_list(file_and_total->total_list); /* NULL if empty cuaderno58 file */
	g_free(file_and_total);

    } else {
	c_util_open_info_window (_("It couldn't open the file"));
    }
    g_free(cuaderno58_file);

    g_hash_table_foreach_remove(hash_table, (void *) (*book58_free_each_issuer_hash), NULL);
    g_hash_table_destroy(hash_table);
  }
  else c_util_open_info_window (_("Impossible generate Cuaderno58 file from a empty invoices list"));
}


/* Para elemento de la tabla hash de ordenante trata la hash por cliente para obtener los registros del
 * fichero por cada cliente. */
static void book58_write_each_issuer_hash(gchar* company_id,GHashTable * client_invoices, FileAndTotal * file_and_total) {
  gchar * issuer_header;
  Participant * company;
  time_t timer;
  gchar * today_char;
  Enterprise * enterprise;
  gulong enterprise_code;
  gchar ** stop = (gchar **) g_malloc0(sizeof(gchar *));
  gchar * filled_bank_code ;
  gchar * filled_office_code ;
  gchar * number_filled ;
  gchar * account_number;
  gchar * control_digit;
  TotalData * total_data = (TotalData *) g_malloc0(sizeof(TotalData));
  gchar * issuer_code;
  gchar * ine_code=NULL;
  gulong ine_code_gulong ;
  guint direct_debit_number;
  gchar * spaces6;
  gchar * spaces8;
  gchar * spaces10;
  gchar * spaces40;
  gchar * spaces2;
  gchar * spaces3;
  gchar * issuer_client_name;
  gint counter;
  gchar * issuer_total_sum;

  gdouble * total_sum_pointer;
  gdouble total;

  gchar * direct_debit_number_char;
  gchar * record_number;
  gchar * issuer_total;
  gchar * spaces72;
  gchar * spaces38;

  GList * account_list;
  GList * element_account;

  g_print(_("(cuaderno58.c) -write_each_issuer_hash- Starting to process one of the 'ordenantes'\n"));

  /* Getting the shop code in order to be able to obtain the enterprise and company for this invoice */
  /* In 'element->data' we have the invoice object */

//  element = g_list_first(invoices);
  enterprise_code = strtoul(company_id,stop,10);
  g_free(stop);

  //shop_code = d_participant_get_code (d_shop_get_participant (d_invoice_get_shop (element->data)));

  /* We need this because the Enterprise is not in memory (no included in de do_search result set) */
  enterprise = m_enterprise_find_by_code (enterprise_code);

  company = d_enterprise_get_participant (enterprise);

  /* Obtaining the date for today as a string in today_char */
  today_char=(gchar *) g_malloc(sizeof(gchar)*10);
  timer=time(NULL);
  strftime(today_char,sizeof(gchar)*10,"%d%m%y",localtime(&timer));

  
 /* Obtaining the CCC for this client (included the calculation of the control code) */
  account_list = m_account_find_by_participant_code (d_participant_get_code (company));
  if (account_list == NULL) 
    {
      g_print(_("(cuaderno58.c) -write_each_issuer_hash- This orderer hasn't got accounts!!!\n"));
      c_util_open_info_window (_("The orderer that is being processed has no bank account information\n Question marks are going to be pintroduced in the corresponding place inside the generated file"));
      account_number = g_strdup("????????????????????");
    }
  else {
    element_account = g_list_first(account_list);
    filled_bank_code = g_strdup_printf("%04d",d_office_get_bank_code (d_account_get_office (element_account->data)));
    filled_office_code = g_strdup_printf("%04d",d_office_get_office_code (d_account_get_office (element_account->data)));   
    number_filled = u_string_fill_left ('0',10,g_strdup_printf("%s",d_account_get_account_number (element_account->data)));  

    ine_code_gulong = m_office_find_ine_code (d_office_get_bank_code (d_account_get_office (element_account->data)),
						    d_office_get_office_code (d_account_get_office (element_account->data)));
    

    ine_code = u_string_fill_left ('0',9,g_strdup_printf("%lu",ine_code_gulong));

    g_print("\nine_code: %s\n",ine_code);


    control_digit = u_calculate_control_digit (filled_bank_code,filled_office_code,number_filled);
    account_number = g_strconcat(filled_bank_code, filled_office_code, control_digit, number_filled, NULL);
    d_account_free_glist (account_list); /* Was created by model_find_account_by_party_code */
    
    g_free(filled_bank_code);
    g_free(filled_office_code);
    g_free(number_filled);
    
    g_free(control_digit);
  }

  issuer_code = u_string_fill_left ('0',9,d_participant_get_legal_id (company));
  spaces6 = g_strnfill(6,' ');
  spaces8 = g_strnfill(8,' ');
  spaces10 = g_strnfill(10,' ');
  spaces40 = g_strnfill(40,' ');
  spaces2 = g_strnfill(2,' ');
  spaces3 = g_strnfill(3,' ');
  
  issuer_client_name = g_strdup_printf("%-40s",d_participant_get_nick (company));
  
  g_print(_("(cuaderno58.c) -write_each_issuer_hash- Generating issuer header\n"));
  
  /* Standard head for the ORDENANTE in 'Cuaderno 58' */
  issuer_header =
    g_strconcat(
		I_BOOK_ISSUER_HEADER_CODE, /* (4) */
		issuer_code, /* (9) */
		I_BOOK_STANDARD_SUFFIX, /* (3) */
		today_char, /* Current time (6) DDMMAA */
		spaces6, /* Fecha de cargo (6) DDMMAA */
		issuer_client_name, /* Nombre del cliente ordenante (40) */
		account_number, /* CCC (4+4+2+10) */
		spaces8, /* (8) */
		I_BOOK_SIXTH_PROCEDURE, /* (2) */
		spaces10, /* (10) */
		spaces40, /* (40) */
		spaces2, /* (2) */
                ine_code,
		spaces3, /* (3) */
		"\r\n",
		NULL);   

  g_free(issuer_client_name);
  g_free(account_number);
  g_free(spaces8);
  g_free(spaces10);
  g_free(spaces40);
  g_free(spaces2);
  g_free(spaces3);
  
  fprintf(file_and_total->out_file, issuer_header);
  g_free(issuer_header);
  g_free(ine_code);
  
  g_free(today_char);
  g_print(_("(cuaderno58.c) -write_each_issuer_hash- Issuer header succesfully generated\n"));
  
  counter = 0;
  total = 0;
  g_print(_("(cuaderno58.c) -write_each_issuer_hash- Organizing the invoices in the hash_table\n"));

  total_data->out_file =file_and_total->out_file;
  total_data->total_sum=0.0;
  total_data->issuer_code = issuer_code;

  direct_debit_number = g_hash_table_size(client_invoices);
  g_hash_table_foreach(client_invoices, (void *) (*book58_write_each_individual_list), total_data);

  g_print(_("(cuaderno58.c) -write_each_issuer_hash- Organized the invoices in the hash_table\n"));

  total_sum_pointer = (gdouble *) g_malloc0(sizeof(gdouble));  
  (*total_sum_pointer) = total_data->total_sum;
  file_and_total->total_list = g_list_append(file_and_total->total_list , total_sum_pointer);
  file_and_total->n_direct_debit = file_and_total->n_direct_debit + direct_debit_number;
  issuer_total_sum = convert_to_string_without_decimal_character(total_data->total_sum, 10);
  direct_debit_number_char = g_strdup_printf("%010d", direct_debit_number);
  record_number =  g_strdup_printf("%010d", direct_debit_number+2); /* FIXME: This could change if using the complex method */
  /* +2 because the header and total are included */
  
  spaces72 = g_strnfill(72,' ');  

  spaces38 = g_strnfill(38,' ');

  g_print(_("(cuaderno58.c) -write_each_issuer_hash- Generating total line for this issuer\n"));
  
  issuer_total = 
    g_strconcat(
		I_BOOK_ISSUER_TOTAL_CODE,
		issuer_code, /* (9) */
		I_BOOK_STANDARD_SUFFIX, /* (3) */		
		spaces72, /* (12 + 40 +20) */
		issuer_total_sum, /* (10) */
		spaces6, /* (6) */
		direct_debit_number_char, /* (10) */
		record_number, /* (10) */
		spaces38,
		"\r\n",
		NULL);		

  g_free(issuer_code);
  g_free(spaces72);
  g_free(issuer_total_sum);
  g_free(spaces6);
  g_free(direct_debit_number_char);
  g_free(record_number);
  g_free(spaces38);
  
  fprintf(file_and_total->out_file, issuer_total);
  g_print(_("(cuaderno58.c) -write_each_issuer_hash- Generated total line for this issuer\n"));  

  g_free(issuer_total);
  
  
  d_enterprise_clear (enterprise);
}


/* Escribe en el registro segn la lista de facturas de un cliente*/
static void book58_write_each_individual_list(gulong client_id,GList * invoices, TotalData * total_data)
{
  gchar * individual_obligatorio;
  gchar * filled_bank_code ;
  gchar * filled_office_code ;
  gchar * number_filled ;
  gchar * account_number;
  gchar * control_digit;
  gchar * reference_code;
  gchar * holder_name;
  gchar * total;
  gdouble direct_debit_total = 0;
  gdouble invoice_total = 0;
  gchar * total_without_dot;
  gchar * direct_debit_code;
  gchar * section;
  gchar * spaces8;
  gchar * spaces2;
  gchar * expire_date_char;
  DirectDebit ** direct_debits;
  gchar ** total_part;
  gboolean result;
  gchar * issue_date;
  gchar * desc;
  time_t date;
  time_t expire_date;
  GList * element= g_list_first(invoices);
  Invoice * invoice = (Invoice *) element->data;

  filled_bank_code = g_strdup_printf("%04d",d_payment_get_bank_code (d_invoice_get_payment (invoice)));
  filled_office_code = g_strdup_printf("%04d",d_payment_get_office_code (d_invoice_get_payment (invoice)));
  number_filled = u_string_fill_left ('0',10,g_strdup_printf("%s",d_payment_get_account_number (d_invoice_get_payment (invoice))));  
  control_digit = u_calculate_control_digit (filled_bank_code,filled_office_code,number_filled);
  account_number = g_strconcat(filled_bank_code, filled_office_code, control_digit, number_filled, NULL);

  g_free(filled_bank_code);
  g_free(filled_office_code);
  g_free(number_filled);
  g_free(control_digit);

  reference_code = g_strdup_printf("%012lu", d_participant_get_code (d_invoice_get_participant (invoice)));
  holder_name = g_strdup_printf("%-40s", d_participant_get_nick (d_invoice_get_participant (invoice)));

  

  while (element !=NULL) {
      if (d_payment_get_is_hire_purchase (d_invoice_get_payment (invoice))!=FALSE) {
	  PaymentId * paymentId = (PaymentId *) g_malloc0(sizeof(PaymentId));
	  paymentId -> payment_code = g_strdup_printf("%lu",d_payment_get_payment_code (d_invoice_get_payment (invoice)));
	  paymentId -> enterprise_code= g_strdup_printf("%lu",d_participant_get_code (d_enterprise_get_participant (d_payment_get_enterprise (d_invoice_get_payment (invoice)))));
	  paymentId -> shop_code= g_strdup_printf("%lu",d_participant_get_code (d_shop_get_participant (d_payment_get_shop (d_invoice_get_payment (invoice)))));

	  invoice_total = (m_payment_calculate_next_total (paymentId,FALSE));

      } else {
	  invoice_total = d_invoice_get_total (invoice);
      }
      direct_debit_total = direct_debit_total + invoice_total;
      total_data -> total_sum = total_data -> total_sum + d_invoice_get_total (invoice); /* Return parameter of the function */

      if (d_payment_get_is_hire_purchase (d_invoice_get_payment (invoice))!=FALSE) {
          Payment * payment = d_invoice_get_payment (invoice);
//	  gboolean error = m_payment_is_pay_instalment (&payment);
	  if (d_payment_get_number_expiries (payment)==d_payment_get_achieved_payments (payment)) {
	      m_invoice_update_state (invoice, D_INVOICE_MANAGING);
	  }
      } else {
	  m_invoice_update_state (invoice, D_INVOICE_MANAGING);
      }
      element = g_list_next(element);
      if (element!=NULL)
	  invoice = (Invoice *) element -> data;
  }

  direct_debit_code = m_direct_debit_get_next_code ();
  direct_debit_code = u_string_fill_left ('0',16,direct_debit_code);

  direct_debits = m_direct_debit_generate_from_invoice_array (direct_debit_code,invoices,D_DIRECT_DEBIT_GENERATED);
  result = m_direct_debit_insert_array (direct_debits);
  if (result == FALSE)
    g_print(_("An error was detected in database while generating Cuaderno 59"));
  
  total = g_strdup_printf("%011.2f", direct_debit_total);
  total_part = g_strsplit (total,",",2); 
  g_free(total);
  total_without_dot = g_strconcat(total_part[0], total_part[1],NULL);
  g_strfreev(total_part);



  issue_date=(gchar *) g_malloc(sizeof(gchar)*20);
  date = d_invoice_get_issue_date (invoice);
  strftime(issue_date,sizeof(gchar)*20,"%d-%m-%Y",localtime(&date));
  desc = g_strconcat(_("Repair."), 
		     issue_date, 
		     d_participant_get_nick (d_shop_get_participant (d_invoice_get_shop (invoice))), 
		     NULL);

  g_free(issue_date);
  section = g_strdup_printf("%-40s", desc);  /* FIXMENOW: Hablar con Tere para seleccionar el section */
  g_free(desc);

  g_print(section);

  spaces8 = g_strnfill(8,' ');
  spaces2 = g_strnfill(2,' ');
  expire_date_char=(gchar *) g_malloc0(sizeof(gchar)*20);
  expire_date = d_payment_get_init_date (d_invoice_get_payment (invoice));
  strftime(expire_date_char,sizeof(gchar)*20,"%d%m%y",localtime(&expire_date));


  individual_obligatorio =    
    g_strconcat(
		I_BOOK_INDIVIDUAL_MANDATORY_CODE,
		total_data->issuer_code, /* NIF */
		I_BOOK_STANDARD_SUFFIX, /* Sufijo */	
		reference_code, /* Codigo de referencia (12) */
		holder_name, /* Nombre del titular (40)  */				
		account_number, /* CCC (20) */				
		total_without_dot, /* Importe (10) */		
		direct_debit_code, /* Codigo Devoluciones  & Codigo Referencia (6+10) */		  		
		section, /* Primer campo de concepto (40) */
                expire_date_char, /*(DDMMAA)*/
		spaces2, /* Libre (8) */
		"\r\n",
		NULL);

  g_free(reference_code);
  g_free(holder_name);
  g_free(account_number);
  g_free(total_without_dot);
  g_free(direct_debit_code);
  g_free(section);
  g_free(spaces8);
  g_free(spaces2);

  fprintf(total_data->out_file, individual_obligatorio);

  g_free(individual_obligatorio);
  g_list_free(invoices);
  /* Update the invoice in the DB as 'en gestion' intermediate state */
  
}

static gint get_first_number_of_line(gchar * line) {
    gchar * value_char = g_strndup(line,I_BOOK_A_SIZE_CODE);
    return atoi(value_char);
}

static gchar * get_caracters_from_position(gchar * line,gint position,gint number_of_caracters) {
    gchar * line_aux = NULL;
    gchar * value_char;
    line_aux = &line[position-1];
    value_char = g_strndup(line_aux,number_of_caracters);
    return value_char;
}

static gchar * get_out_first_zeros(gchar * nif) {
    gchar * result;
    gchar * aux;
    gint index=0;
    while (nif[index]=='0') {
        index++;
    }
    aux = &nif[index];
    result = g_strndup(aux,9-index);
    return result;
}

static void fill_struct_code_from_line(DirectDebit * dev,gchar * temp,gulong enterprise_code) {
    gchar * direct_debit_code = get_caracters_from_position(temp,99,16);
    gchar * reason = get_caracters_from_position(temp,155,1);
    gchar ** stop=(gchar **) g_malloc0(sizeof(gchar *));

    d_direct_debit_set_code (dev,strtoul(direct_debit_code,stop,10));
    d_direct_debit_set_enterprise_code (dev,enterprise_code);
    d_direct_debit_set_state (dev,atoi(reason));

    g_free(stop);
    g_free(direct_debit_code);
    g_free(reason);

}

/**
 * i_book58_proccess_devolution_file:
 * @file_name: Fichero a procesar.
 *
 * Procesa el fichero de devoluciones de domiciliaciones. El fichero se recorr y se procesa, almacenando
 * en una tabla hash en memoria los datos por cliente ordenante. Despus se procesan las domiciliaciones
 * devoviendo las facturas de cada domiciliacin a su estado original. En caso de producirse un error en el procesado
 * se devolver un mensaje de error.
 *
 * Returns:
 * Devuelve %NULL si el procesado ha terminado correctamente, un mensaje de error en otro caso.
 */
gchar * i_book58_proccess_devolution_file (gchar * file_name) {
    FILE * in_file  = fopen(file_name, "r");
    gchar * line;
    gboolean * result_boolean = (gboolean *)g_malloc0(sizeof(gboolean));
    gchar * issuer_id=NULL;
    gchar * result=NULL;
    gchar *temp_aux = (gchar *) g_malloc0(sizeof(gchar)*I_BOOK_MAXIMUN_BUFFER_SIZE);
    gulong current_enterprise_code = ULONG_MAX;
    GHashTable *hash_table_dev = g_hash_table_new(g_str_hash, g_str_equal);
    GList * content = NULL;
    gint * return_total = (gint *) g_malloc0(sizeof(gint));

    if (in_file!=NULL) {
	line = fgets(temp_aux, I_BOOK_MAXIMUN_BUFFER_SIZE, in_file);
	/*Recorremos el fichero y obtenemos la informacin que nos interesa para realizar las devoluciones*/
	while (line!=NULL) {
	    gint codigo = get_first_number_of_line(line);
            /* COmprobamos los datos correctos para el caso del presentador.*/
	    if (codigo == I_BOOK58_PRESENTER_RETURN_CODE) {
		gchar * presenter_client_name = u_parameter_manager_get_parameter ("empresa");
		Participant * participant = m_client_get_participant_by_name (presenter_client_name,M_CLIENT_ENTERPRISE_CODE,TRUE);
		gchar * presenter_legal_id= u_string_fill_left ('0',9,g_strdup_printf("%s",d_participant_get_legal_id (participant)));
		gchar * presenter_legal_id_file =get_caracters_from_position(line,5,9);
		/*Comparamos los cifs, pero como pueden tener un guin o espacio interfiriendo se utiliza la comparacin
		  especfica para el cif*/
		if (u_cif_strcmp (presenter_legal_id,presenter_legal_id_file)) {
		    result = g_strdup(_("Incorrect presenter tax code in refunds file"));
		    break;
		}
		g_free(presenter_legal_id);
		g_free(presenter_legal_id_file);
            /* COmprobamos los datos correctos para cada cliente ordenante.*/
	    } else if (codigo ==  I_BOOK_ISSUER_RETURN_CODE) {
		gchar * legal_id_issuer = get_caracters_from_position(line,5,9);
		gchar * legal_id_aux = get_out_first_zeros(legal_id_issuer);
		current_enterprise_code = m_enterprise_get_code_from_legal_id (legal_id_aux);
		if (issuer_id!=NULL)
                    g_free(issuer_id);
 	        issuer_id= g_strdup_printf("%016lu",current_enterprise_code);
		g_free(legal_id_aux);
		g_free(legal_id_issuer);
		if (current_enterprise_code ==ULONG_MAX) {
		    result = g_strdup(_("Incorrect issuer tax code in refunds file"));
		    break;
		}
            /*Almacenamos los datos de cada devolucin*/
	    } else if (codigo ==  I_BOOK_INDIVIDUAL_RETURN_CODE) {
		DirectDebit * dev = d_direct_debit_create_empty ();
		content = g_hash_table_lookup(hash_table_dev, issuer_id);

		fill_struct_code_from_line(dev,line,current_enterprise_code);
                content = g_list_append(content,(gpointer)(dev));
		g_hash_table_insert (hash_table_dev,(gpointer) issuer_id, content);

               /*Calculamos los totales por ordenante y comprobamos los valores.*/
	    } else if (codigo ==  I_BOOK_TOTAL_RETURN_CODE) {
		gchar * issuer_total_file;
		gchar * issuer_total_record_file;
		gint issuer_total_file_gint;
		gint issuer_total_record_file_gint;
		gint * total_ordenante = (gint *) g_malloc0(sizeof(gint));
		content = g_hash_table_lookup(hash_table_dev, issuer_id);
		count_records(current_enterprise_code,content,total_ordenante);
		issuer_total_file = get_caracters_from_position(line,105,10);
		issuer_total_file_gint = atoi(issuer_total_file);
		issuer_total_record_file = get_caracters_from_position(line,115,10);
		issuer_total_record_file_gint = atoi(issuer_total_record_file);
		if (issuer_total_file_gint != (*total_ordenante)) {
		  result = g_strdup(_("Number of refunds is not equal to the total refunds value in the header"));
		    break;
		}

		if (issuer_total_record_file_gint != ((*total_ordenante)+2)) {
		    result = g_strdup(_("Number of issuer registers is not equal to total registers value of the issuer header"));
		    break;
		}

		g_free(issuer_total_file);
		g_free(issuer_total_record_file);
		g_free(total_ordenante);
                /* calculamos los registros totales del fichero y comprobamos los valores*/
	    } else if (codigo ==  I_BOOK_GENERAL_RETURN_CODE) {
		gchar * general_total_file;
		gchar * total_reg_general_file;
		gint general_total_file_gint;
		gint total_reg_general_file_gint;
		gint size_of_hash = g_hash_table_size(hash_table_dev);
		general_total_file = get_caracters_from_position(line,105,10);
		general_total_file_gint = atoi(general_total_file);
		total_reg_general_file = get_caracters_from_position(line,115,10);
		total_reg_general_file_gint = atoi(total_reg_general_file);

		g_hash_table_foreach(hash_table_dev,(void *)(*count_records),return_total);
		if (general_total_file_gint != (*return_total)) {
		  result = g_strdup(_("Number of refunds is not equal to the total refunds value in the header"));
		    break;
		}

		if (total_reg_general_file_gint != ((*return_total)+(2*size_of_hash)+2)) {
		    result = g_strdup(_("Number of registers is not equal to the total register value in the general header"));
		    break;
		}

		g_free(general_total_file);
		g_free(total_reg_general_file);

	    }


	    if (codigo !=  I_BOOK_GENERAL_RETURN_CODE)
		line = fgets(temp_aux, I_BOOK_MAXIMUN_BUFFER_SIZE, in_file);
	    else
		line = NULL;
	}

	if (issuer_id!=NULL)
	    g_free(issuer_id);

	g_free(return_total);
	g_free(temp_aux);
    } else {
	result = g_strdup(_("File doesn't exist"));
	
    }

    /*Si no existe ningn mensaje de error realizamos el procesamiento de las devoluciones*/
    if (result == NULL) {
	*result_boolean = TRUE;
        /*para cada lista de domiciliaciones de cada cliente ordenante se procesan las mismas.*/
	g_hash_table_foreach(hash_table_dev,(void *)(*proccess_direct_debits),result_boolean);

	if (*result_boolean == FALSE) {
            result = g_strdup(_("Error while processing refunds in database"));
	}

    }

    g_hash_table_foreach_remove(hash_table_dev,(void *)(*return_free_each_issuer_hash),NULL);
    g_hash_table_destroy(hash_table_dev);
    return result;
}

static void count_records(gulong issuer_code, GList * dev, gint * total) {
    GList * element = g_list_first(dev);
    while (element!=NULL) {
        *total = (*total) + 1;
        element = g_list_next(element);
    }
}

static void proccess_direct_debits(gulong issuer_code, GList * direct_debits, gboolean * result_boolean) {
    *result_boolean = *result_boolean && m_direct_debit_return_array (direct_debits);
}

static gboolean book58_free_each_issuer_hash(gchar* company_id, GHashTable * client_invoices,gpointer user_data) {
    g_hash_table_foreach_remove(client_invoices, (void *) (*book58_free_each_individual_list), NULL);
    g_hash_table_destroy(client_invoices);
    return TRUE;
}
static gboolean book58_free_each_individual_list(gulong client_id,GList * invoices, TotalData * total_data) {
    /*NO VALE???
     GList * list = g_list_first(invoices);
    while (list!=NULL) {
        g_list_remove_link(list,list->next);
        list = g_list_next(list);
    }
    g_list_free(invoices);
*/    return TRUE;
}
static gboolean return_free_each_issuer_hash(gulong issuer_code, GList * dev,gpointer data) {
/*    GList * list = g_list_first(dev);
    while (list!=NULL) {
        g_list_remove(list,list->data);
        list = g_list_next(list);
    }
    */
    //    g_list_free(dev);

    return TRUE;
}
