/* extract_delivery_payment.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */


#include "extract_delivery_payment.h"
#include "extract_repair.h"
#include "glib.h"


/***********************/
/* FIXME: esta funcin debera estar dentro de dataobjects */
static GList * repair_lines_get_string_lines(Repair_line ** repair_lines);


/*******************************************/

/**
 * p_delivery_payment_get_stringify_client_number:
 * @delivery_payment: Albarn del que se extraer el atributo.
 *
 * Obtiene el nmero de cliente del albarn de abono.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres en una cadena de caracteres.
 *
 */
gchar * p_delivery_payment_get_stringify_client_number (DeliveryPayment * delivery_payment)
{
   gulong client_number_gulong=ULONG_MAX;
   gchar * client_number = NULL;


//   repair = d_delivery_payment_get_repair (delivery_payment);
//   client_number_gulong = d_participant_get_code (d_client_get_participant (d_repair_get_client (repair)));

   if (d_delivery_payment_get_repair (delivery_payment)!=NULL) {
       if (d_repair_get_client (d_delivery_payment_get_repair (delivery_payment))!=NULL) {
	   client_number_gulong = d_participant_get_code (d_client_get_participant (d_repair_get_client (d_delivery_payment_get_repair (delivery_payment))));
       }
   }
   if (client_number_gulong!=ULONG_MAX)
       client_number = g_strdup_printf("%lu",client_number_gulong);

   if (client_number!=NULL) {
       return client_number;
   } else {
       return g_strdup("");
   }
}


/**
 * p_delivery_payment_get_stringify_legal_id:
 * @delivery_payment: Albarn del que se extraer el atributo.
 *
 * Devuelve el cif/nif del cliente del albarn.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * p_delivery_payment_get_stringify_legal_id (DeliveryPayment * delivery_payment)
{
   Repair * repair;
   gchar * legal_id = NULL;
   
   repair = d_delivery_payment_get_repair (delivery_payment);
   if (repair!=NULL) {
       if  (d_repair_get_client (repair)!=NULL) {
	   legal_id = d_participant_get_legal_id (d_client_get_participant (d_repair_get_client (repair)));
       }
   }
   if (legal_id!=NULL) {
       return legal_id;
   } else {
       return g_strdup("");
   }
}

/**
 * p_delivery_payment_get_stringify_date:
 * @delivery_payment: Albarn del que se extraer el atributo.
 *
 * Devuelve la fecha de emisin del albarn en formato aaaa-mm-dd.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * p_delivery_payment_get_stringify_date (DeliveryPayment * delivery_payment)
{
   gchar *timer_char = (gchar *) g_malloc(sizeof(gchar)*100);
   size_t stored;
   time_t date;

   date = d_delivery_payment_get_issue_date (delivery_payment);  
   stored = strftime(timer_char,sizeof(gchar)*100,"%Y-%m-%d", localtime(&date));

   if (timer_char!=NULL) {
       return timer_char;
   } else {
       return g_strdup("");
   }

}

/**
 * p_delivery_payment_get_stringify_number:
 * @delivery_payment: Albarn del que se extraer el atributo.
 *
 * Devuelve el nmero de albarn.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * p_delivery_payment_get_stringify_number (DeliveryPayment * delivery_payment)
{
   gulong invoice_number_gulong;
   gchar * invoice_number = NULL;

   invoice_number_gulong =  d_delivery_payment_get_number (delivery_payment);

   invoice_number = g_strdup_printf("%lu",invoice_number_gulong);

   if (invoice_number!=NULL) {
       return invoice_number;
   } else {
       return g_strdup("");
   }
}

/**
 * p_delivery_payment_get_stringify_agent_number:
 * @delivery_payment: Albarn del que se extraer el atributo.
 *
 * Devuelve el nmero de agente del agente de la reparacin.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * p_delivery_payment_get_stringify_agent_number (DeliveryPayment * delivery_payment)
{
    if (d_repair_get_agent ((d_delivery_payment_get_repair (delivery_payment)))!=NULL) {
	if (d_participant_get_number (d_agent_get_participant (d_repair_get_agent (d_delivery_payment_get_repair (delivery_payment))))!=NULL) {
	    return d_participant_get_number (d_agent_get_participant (d_repair_get_agent ((d_delivery_payment_get_repair (delivery_payment)))));
	} else {
	    return g_strdup(" Sin agente ");
	}
    } else {
	return g_strdup(" ");
    }
}

/**
 * p_delivery_payment_get_stringify_post_address:
 * @delivery_payment: Albarn del que se extraer el atributo.
 *
 * Devuelve la direccin a imprimir en el albarn.
 *
 * Returns:
 * Devuelve una lista de cadenas que contienen la direccin en cuatro lneas.
 *
 */
gchar ** p_delivery_payment_get_stringify_post_address (DeliveryPayment * delivery_payment) /* FIXME: 4 LINES */
{
   Repair * repair;
   Participant * participant=NULL;
   gchar **dest_address = NULL;
   gchar * name=NULL;
   gchar * street=NULL;
   gchar * floor=NULL;
   gchar * number=NULL;
   gchar * postal_code=NULL;
   gchar * town=NULL;
   gchar * province=NULL;
   gchar * temp_name=NULL;
   gulong aux_code = ULONG_MAX;

   /* 
    Si la factura es para una compaa (clientes con seguro), la direccin
    ser la de dicha compaa en caso contrario ser la del cliente
    asociado a la reparacin --> Para comprobar si es un cliente con
    seguro, se mira si el nmero de pliza contiene algo 
    */
      
   dest_address = (gchar **) g_malloc0 (sizeof(gchar *)*4);

   repair = d_delivery_payment_get_repair (delivery_payment);

   if (d_repair_get_client (d_delivery_payment_get_repair (delivery_payment))!=NULL) {
       participant = d_client_get_participant (d_repair_get_client (d_delivery_payment_get_repair (delivery_payment)));
   }
   if (participant!=NULL) {
       aux_code = d_participant_get_code (participant);
       //       temp_name = d_participant_get_nick (participant);
       temp_name = d_participant_get_short_name (participant);
   }
   //name = d_company_get_short_name (NULL,aux_code,temp_name);
   name = d_participant_get_short_name (participant);
   if (participant!=NULL)
       g_free(temp_name);
   if (name == NULL) {
       g_free(name);
       name = g_strdup(" ");
   }
   if (participant!=NULL)
       street = d_participant_get_street (participant);
   if (street == NULL) {
       g_free(street);
       street = g_strdup("");
   }
   if (participant!=NULL)
       floor = d_participant_get_floor (participant);
   if (floor == NULL) {
       g_free(floor);
       floor = g_strdup("");
   }

   if (participant!=NULL)
       number = d_participant_get_number (participant);

   if (number == NULL) {
       g_free(number);
       number = g_strdup("");
   }
   if (participant!=NULL)
       postal_code = g_strdup_printf("%lu, ", d_participant_get_postal_code (participant)); /* FIXME: liberar el number?? */

   if ((participant==NULL) || (d_participant_get_postal_code (participant) == ULONG_MAX)) {
       g_free(postal_code);
       postal_code = g_strdup("");
   }
   if (participant!=NULL)
       town = d_participant_get_town (participant);
   if (town == NULL) {
       g_free(town);
       town = g_strdup("");
   }
   if (participant!=NULL)
       province = d_participant_get_province (participant);
   if (province == NULL) {
       g_free(province);
       province = g_strdup(" ");
   }
   
   dest_address[0] = name;
   dest_address[1] = g_strconcat(street, ", ", number, ", ", floor, NULL);
   dest_address[2] = g_strconcat(postal_code, town, NULL);
   dest_address[3] = province; 
   
   /* Liberamos las cadenas que han sido duplicadas */
   g_free(street);
   g_free(number);
   g_free(floor);
   g_free(postal_code);
   g_free(town);
   
   return dest_address;
}


/**
 * p_delivery_payment_get_stringify_company:
 * @delivery_payment: Albarn del que se extraer el atributo.
 *
 * Devuelve el nombre de la compaa de seguros de la reparacin.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * p_delivery_payment_get_stringify_company (DeliveryPayment * delivery_payment)
{
    gchar * result=NULL;
    
    if (d_repair_get_company (d_delivery_payment_get_repair (delivery_payment))!=NULL) {
//	result = d_participant_get_name (d_company_get_participant (d_repair_get_company (d_delivery_payment_get_repair (delivery_payment))));
	result = d_participant_get_short_name (d_company_get_participant (d_repair_get_company (d_delivery_payment_get_repair (delivery_payment))));
    }

    if (result !=NULL) return result;
    else return g_strdup("");
}

/**
 * p_delivery_payment_get_stringify_policy:
 * @delivery_payment: Albarn del que se extraer el atributo.
 *
 * Devuelve el nmero de pliza del cliente de la reparacin.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * p_delivery_payment_get_stringify_policy (DeliveryPayment * delivery_payment)
{
    gchar * result=NULL;
    if (d_repair_get_policy (d_delivery_payment_get_repair (delivery_payment))!=NULL) {
	result = d_policy_get_policy_number (d_repair_get_policy (d_delivery_payment_get_repair (delivery_payment)));
    }
    if (result !=NULL) return result;
    else return g_strdup("");
}

/**
 * repair_lines_get_string_lines:
 * @delivery_payment: Albarn del que se extraer el atributo.
 *
 * Devuelve una #Glist con la lista de datos que saldrn en las lneas del albarn.
 *
 * Returns:
 * Lista con los elementos de una lnea de reparacin en cadenas de caracteres.
 *
 */
static GList * repair_lines_get_string_lines(Repair_line ** repair_lines)
{  
   gint j;
   GList * list = NULL;
   gchar ** line;
   gdouble quantity;
   gint index_line=0;
   gdouble price;
   gdouble line_price;
   gdouble discount;
   gboolean negative=FALSE;

   for (j=0; (repair_lines[j])!=NULL; j++)
   {
        gchar * description;
        line = (gchar **) g_malloc (sizeof(gchar *)*7); /* 7 columnas de strings */
	
	quantity = d_repair_line_get_quantity (repair_lines[j]);
	line_price = d_repair_line_get_price (repair_lines[j]);
	price = d_product_get_price (d_repair_line_get_product(repair_lines[j]));
	discount = d_repair_line_get_discount (repair_lines[j]);
	
	/* 7 is the number of cols in the delivery_payment table for the repair_lines */

	if (d_product_get_is_piece(d_repair_line_get_product (repair_lines[j]))) {
	    Piece * piece = (Piece *) d_repair_line_get_product (repair_lines[j]);
	    line[0] = d_piece_get_euroCode (piece);

	} else {
	    line[0] =  g_strdup_printf("%lu",
	    d_product_get_product_code(d_repair_line_get_product (repair_lines[j])));
	}
	/* Referencia */

	description = d_product_get_description (d_repair_line_get_product(repair_lines[j]));

	if (description!=NULL) {
	    g_strup(description);
	} else {
	    description=g_strdup("");
	}
	line[1] = g_strdup(description);
	g_free(description);

	line[2] = g_strdup_printf("%.3f", quantity); /* Cantidad */
	if (price!=G_MINDOUBLE) {
            if (line_price==G_MINDOUBLE)
		line[3] = g_strdup_printf("%.2f", price); /* Precio */
	    else
                line[3] = g_strdup(" "); /* Precio */
	} else {
	    line[3] = g_strdup(" "); /* Precio */
	}
	if (line_price!=G_MINDOUBLE) {
	    line[4] = g_strdup_printf("%.2f", line_price*quantity); /* Importe */
	} else if (price!=G_MINDOUBLE) {
	    line[4] = g_strdup_printf("%.2f", price*quantity); /* Importe */
	} else {
	    line[4] = g_strdup(" "); /* Importe */
            negative = TRUE;
	}
	if (discount>0) {
            if (line_price==G_MINDOUBLE)
		line[5] = g_strdup_printf("%.2f", discount); /* Dcto. */
	    else
		line[5] = g_strdup(" ");
	} else {
	    line[5] = g_strdup(" ");
	}
	if (line_price!=G_MINDOUBLE) {
	    line[6] = g_strdup_printf("%.2f", line_price*quantity);
	} else if (price!=G_MINDOUBLE) {
            line[6] = g_strdup_printf("%.2f", price*quantity*(1-(0.01*discount))); /* Total */
	} else {
	    line[6] = g_strdup(" ");
            negative = TRUE;
	}

	for (index_line=0;index_line<7;index_line++)
	    if (line[index_line]==NULL) line[index_line] = g_strdup("");
	list = g_list_append(list,line);
        /* creo que non se debera liberar*/
	//g_free(line); /* FIXME: SHOULD FREE ALSO EACH ELEMENT OF THE LINE */
     }
  
  return list;
}


/**
 * p_delivery_payment_get_stringify_lines:
 * @delivery_payment: Albarn del que se extraer el atributo.
 *
 * Devuelve las lneas de reparacin de la reparacin del albarn.
 *
 * Returns:
 * Lista de lneas de reparacin del albarn.
 *
 */
GList * p_delivery_payment_get_stringify_lines (DeliveryPayment * delivery_payment) /* FIXME: totales... */
{
   Repair_line ** repair_lines;
   Repair * repair;
   GList * list = NULL;
   
   repair = d_delivery_payment_get_repair (delivery_payment);
   repair_lines = d_repair_get_repair_lines (repair);
   list = repair_lines_get_string_lines (repair_lines);

  return list;
}

/**
 * p_delivery_payment_get_number_of_lines:
 * @delivery_payment: Albarn del que se extraer el atributo.
 *
 * Devuelve el nmero de lneas de reparacin del albarn de abono.
 *
 * Returns:
 * Devuelve el atributo solicitado.
 *
 */
gint p_delivery_payment_get_number_of_lines (DeliveryPayment * delivery_payment)
{
   Repair_line ** repair_lines;
   Repair * repair;
   gint number_of_lines = 0;
   
   repair = d_delivery_payment_get_repair (delivery_payment);
   repair_lines = d_repair_get_repair_lines (repair);
   while (repair_lines[number_of_lines]!=NULL) {
       number_of_lines++;
   }

  return number_of_lines;
}




/**
 * p_delivery_payment_get_stringify_company_name:
 * @delivery_payment: Albarn del que se extraer el atributo.
 *
 * Devuelve el nombre de la empresa que gener el albarn de abono.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * p_delivery_payment_get_stringify_company_name (DeliveryPayment * delivery_payment)
{
    gchar * result=NULL;
    if (d_repair_get_enterprise (d_delivery_payment_get_repair (delivery_payment))!=NULL) {
	//	result = d_participant_get_name (d_enterprise_get_participant (d_repair_get_enterprise (d_delivery_payment_get_repair (delivery_payment))));
	result = d_participant_get_short_name (d_enterprise_get_participant (d_repair_get_enterprise (d_delivery_payment_get_repair (delivery_payment))));
    }
    if (result !=NULL)
	return result;
    else
	return g_strdup("");
}

/**
 * p_delivery_payment_get_stringify_company_legal_id:
 * @delivery_payment: Albarn del que se extraer el atributo.
 *
 * Devuelve el cif de la empresa que gener el albarn de abono.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * p_delivery_payment_get_stringify_company_legal_id (DeliveryPayment * delivery_payment)
{
    gchar * result=NULL;
    if (d_repair_get_enterprise (d_delivery_payment_get_repair (delivery_payment))!=NULL) {
	result = d_participant_get_legal_id (d_enterprise_get_participant (d_repair_get_enterprise (d_delivery_payment_get_repair (delivery_payment))));
    }
    if (result !=NULL)
	return result;
    else
	return g_strdup("");
}

/**
 * p_delivery_payment_calculate_stringify_total:
 * @delivery_payment: Albarn del que se extraer el atributo.
 * @total_char: Doble puntero a char, para rellenar el total del albarn.
 *
 * Rellena el total del albarn en la variable total_char.
 *
 *
 */
void p_delivery_payment_calculate_stringify_total (DeliveryPayment * delivery_payment,gchar ** total_char)
{
    gdouble * total = (gdouble *) g_malloc0(sizeof(gdouble));

    d_delivery_payment_calculate_total_data (total,delivery_payment);

    *total_char = g_strdup_printf("%.2f",*total);
}

/**
 * p_delivery_payment_get_repair:
 * @delivery_payment: Albarn del que se extraer el atributo.
 *
 * Devuelve la reparacin del albarn
 *
 * Returns:
 * Devuelve el atributo solicitado.
 *
 */
Repair * p_delivery_payment_get_repair (DeliveryPayment * delivery_payment) {
    Repair * result = NULL;
    if (delivery_payment!=NULL) {
	result = d_delivery_payment_get_repair (delivery_payment);
    }
    return result;
}
