/* extract_order.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */


#include "extract_order.h"
#include "glib.h"


/***********************/
/* FIXME: esta funcin debera estar dentro de dataobjects */

static GList * order_lines_get_string_lines(OrderLine ** order_lines,gboolean show,gulong from_lines,gulong n_lines);


/**
 * p_order_get_stringify_supplier_number:
 * @order: Pedido del que se extraer el atributo.
 * @show: Permite elegir si se muestra el atributo o no.
 *
 * Devuelve el nmero del proveedor al que se le solicitar el pedido.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * p_order_get_stringify_supplier_number (Order * order,gboolean show) {

    gulong supplier_number_gulong;
    gchar * supplier_number = NULL;

    supplier_number_gulong = d_order_get_supplier_code (order);
    if (supplier_number_gulong!=ULONG_MAX)
	supplier_number = g_strdup_printf("%lu",supplier_number_gulong);

    if ((supplier_number!=NULL) && (show!=FALSE)) {
	return supplier_number;
    } else {
       if (supplier_number!=NULL)
	   g_free(supplier_number);
       return g_strdup("");
    }

}

/**
 * p_order_get_stringify_legal_id:
 * @participant: Pedido del que se extraer el atributo.
 * @show: Permite elegir si se muestra el atributo o no.
 *
 * Devuelve el cif de un participante. Esta funcin debera moverse a extract_participant.c
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * p_order_get_stringify_legal_id (Participant * participant,gboolean show) {
    gchar * legal_id = NULL;

    if (participant!=NULL) {
	legal_id = d_participant_get_legal_id (participant);
    }
    if ((legal_id!=NULL) && (show!=FALSE)) {
	return legal_id;
    } else {
	if (legal_id!=NULL)
	    g_free(legal_id);
	return g_strdup("");
    }
}

/**
 * p_order_get_stringify_date:
 * @order: Pedido del que se extraer el atributo.
 * @show: Permite elegir si se muestra el atributo o no.
 *
 * Devuelve la fecha en la que se solicita el pedido.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * p_order_get_stringify_date (Order * order,gboolean show) {
   gchar *timer_char = (gchar *) g_malloc0(sizeof(gchar)*100);
   size_t stored;
   time_t date;
   
   date = d_order_get_date (order);
   stored = strftime(timer_char,sizeof(gchar)*100,"%Y-%m-%d", localtime(&date));

   if ((timer_char!=NULL) && (show!=FALSE)) {
       return timer_char;
   } else {
       if (timer_char!=NULL)
	   g_free(timer_char);
       return g_strdup("");
   }

}

/**
 * p_order_get_stringify_order_number:
 * @order: Pedido del que se extraer el atributo.
 * @show: Permite elegir si se muestra el atributo o no.
 *
 * Devuelve el nmero de pedido.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * p_order_get_stringify_order_number (Order * order,gboolean show) {
   gulong order_number_gulong;
   gchar * order_number = NULL;

   order_number_gulong =  d_order_get_code (order);

   if (order_number_gulong!=ULONG_MAX) {
       order_number = g_strdup_printf("%lu",order_number_gulong);
   }

   if ((order_number!=NULL) && (show!=FALSE)) {
       return order_number;
   } else {
       if (order_number!=NULL)
	   g_free(order_number);
       return g_strdup("");
   }
}

/**
 * p_order_get_stringify_name:
 * @participant: Pedido del que se extraer el atributo.
 * @show: Permite elegir si se muestra el atributo o no.
 *
 * Devuelve el nombre de un participante. Esta funcin debera moverse a extract_participant.c
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * p_order_get_stringify_name (Participant * participant,gboolean show) {
    gchar * result=NULL;
    if (participant!=NULL) {
	//	result = d_participant_get_nick (participant);
	result = d_participant_get_short_name (participant);
    }

    if ((result !=NULL) && (show!=FALSE))
	return result;
    else {
	if (result!=NULL)
	    g_free(result);
	return g_strdup("");
    }
}



/**
 * p_order_get_stringify_post_address:
 * @participant: Pedido del que se extraer el atributo.
 * @show: Permite elegir si se muestra el atributo o no.
 *
 * Devuelve la direccin de envo de un participante. Esta funcin debera moverse a extract_participant.c
 *
 * Returns:
 * Devuelve el atributo solicitado en un array de cadenas de caracteres en los que se devuelven las lneas
 * de la direccin.
 *
 */
gchar ** p_order_get_stringify_post_address (Participant * participant,gboolean show)
{

   gchar **dest_address = NULL;
   gchar * name= NULL;
   gchar * street= NULL;
   gchar * floor= NULL;
   gchar * numero= NULL;
   gchar * postal_code= NULL;
   gchar * town= NULL;
   gchar * province= NULL;
     
   dest_address = (gchar **) g_malloc0 (sizeof(gchar *)*4);

   if (participant!=NULL) {
       //gchar * temp = d_participant_get_nick (participant);
       gulong aux_code = d_participant_get_code (participant);
       //name = d_company_get_short_name (NULL,aux_code,temp);
       name = d_participant_get_short_name (participant);
       //g_free(temp);
   }
   if (name == NULL) {
       
       name = g_strdup(" ");
   }
   if (participant!=NULL)
       street = d_participant_get_street (participant);
   if (street == NULL) {
       street = g_strdup("");
   }
   if (participant!=NULL)
       floor = d_participant_get_floor (participant);
   if (floor == NULL) {
       floor = g_strdup("");
   }
   if (participant!=NULL)
       numero = d_participant_get_number (participant);
   if (numero == NULL) {
       
       numero = g_strdup("");
   }
   if (participant!=NULL)
       postal_code = g_strdup_printf("%lu, ", d_participant_get_postal_code (participant));
   if ((participant==NULL) || (d_participant_get_postal_code (participant) == ULONG_MAX)) {
       if (postal_code!=NULL)
	   g_free(postal_code);
       postal_code = g_strdup("");
   }
   if (participant!=NULL)
       town = d_participant_get_town (participant);
   if (town == NULL) {
       town = g_strdup("");
   }
   if (participant!=NULL)
       province = d_participant_get_province (participant);
   if (province == NULL) {
       province = g_strdup(" ");
   }
   if (show!=FALSE)
       dest_address[0] = name;
   else {
       if (name!=NULL)
	   g_free(name);
       dest_address[0] = g_strdup("");
   }

   if (show!=FALSE)
       dest_address[1] = g_strconcat(street, "   ", numero, "   ", floor, NULL);
   else
       dest_address[1] = g_strdup("");

   if (show!=FALSE)
       dest_address[2] = g_strconcat(postal_code, " " ,town, NULL);
   else
       dest_address[2] = g_strdup("");

   if (show!=FALSE)
       dest_address[3] = province;
   else {
       if (province!=NULL)
	   g_free(province);
       dest_address[3] = g_strdup("");
   }
   /* Liberamos las cadenas que han sido duplicadas */
   g_free(street);
   g_free(numero);
   g_free(floor);
   g_free(postal_code);
   g_free(town);
   
   return dest_address;
}


static GList * order_lines_get_string_lines(OrderLine ** order_lines,gboolean show,gulong from_lines,gulong n_lines)
{  
   gint j;
   GList * list = NULL;
   gchar ** line;
   gdouble quantity;
   gint index_line=0;
   gint index_order_line=from_lines;

   j=from_lines;
   while ((j<n_lines) || (order_lines[index_order_line]!=NULL))
   {
       gchar * description;
       Product product;
       line = (gchar **) g_malloc0 (sizeof(gchar *)*7); /* 7 columnas de strings */

       if (order_lines[index_order_line]!=NULL) {
	   quantity = d_order_line_get_quantity (order_lines[index_order_line]);

	   if (show!=FALSE) {
	       Piece * piece = d_order_line_get_piece (order_lines[index_order_line]);
	       line[0] = d_piece_get_euroCode (piece);
	   } else {
	       line[0] = g_strdup("");
	   }

	   product=d_piece_get_product (d_order_line_get_piece (order_lines[index_order_line]));
	   description = d_product_get_description (&product);
	   if (show!=FALSE) {
	       if (description!=NULL)
		   g_strup(description);
	       line[1] = g_strdup(description);
	   } else {
	       line[1] = g_strdup("");
	   }

	   if (description!=NULL) {
	       g_free(description);
	   }

	   if (show!=FALSE)
	       line[2] = g_strdup_printf("%.3f", quantity); /* Cantidad */
	   else
	       line[2] = g_strdup("");
	   index_order_line++;
       }
       for (index_line=0;index_line<7;index_line++)
	   if (line[index_line]==NULL) line[index_line] = g_strdup("");
       list = g_list_append(list,line);
       j++;
   }
  
  return list;
}

/**
 * p_order_get_lines:
 * @order: Pedido del que se extraer el atributo.
 * @show: Permite elegir si se muestra el atributo o no.
 * @from_lines: Indica la lnea del pedido desde la que se empezar a imprimir en la pgina actual.
 * @n_lines: Indica el nmero de lneas que se imprimirn en la pgina actual.
 *
 * Devuelve las lneas de pedido para imprimir en una pgina del pedido.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
GList * p_order_get_lines (Order * order,gboolean show,gulong from_lines,gulong n_lines) {

   OrderLine ** order_lines;
   GList * list = NULL;

   if (d_order_get_order_lines (order)!=NULL)
       {
	   order_lines = d_order_get_order_lines (order);
	   list = order_lines_get_string_lines (order_lines,show,from_lines,n_lines);
       }

   return list;
}

