 /* extract_post_address.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */


#include "extract_post_address.h"
#include <gnome.h>
#include "glib.h"

/**
 * p_post_address_get_stringify_second_line:
 * @post_address: Direccin de envo.
 *
 * Obtiene la segunda lnea de una direccin de envo, que contiene la calle, el nmero y el piso.
 *
 * Returns:
 * Devuelve una cadena de caracteres con la parte solicitada de una direccin.
 *
 */
gchar * p_post_address_get_stringify_second_line (PostAddress * post_address){
    gchar * street;
    gchar * number;
    gchar * floor;
    gchar * result;

    if (post_address!=NULL) {
	street = d_post_address_get_street (post_address);
	number =d_post_address_get_number (post_address);
	floor = d_post_address_get_floor (post_address);

	if (street == NULL) street = g_strdup("");
	if (number == NULL) number = g_strdup("");
	if (floor == NULL) floor = g_strdup("");

	result = g_strconcat(street," ",number," ",floor," ",NULL);
	g_free(street);
	g_free(number);
	g_free(floor);
    } else {
        result = g_strdup(_("Problems with the address"));
    }
    return result;
}

/**
 * p_post_address_get_stringify_third_line:
 * @post_address: Direccin de envo.
 *
 * Devuelve el atributo solicitado en una cadena de caracteres. 
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * p_post_address_get_stringify_third_line (PostAddress * post_address){
    gulong postal_code;
    gchar * town;
    gchar * postal_code_char;
    gchar * result;
    if (post_address!=NULL) {
	postal_code = d_post_address_get_postal_code (post_address);
	town =  d_post_address_get_town (post_address);
	if (postal_code!=ULONG_MAX) {
	    postal_code_char = g_strdup_printf("%lu",postal_code);
	} else {
	    postal_code_char = g_strdup("");
	}
	if (town == NULL) town = g_strdup("");
	result = g_strconcat(town," ",postal_code_char,NULL);
	g_free(town);
    } else {
        result = g_strdup(_("Problems with the address"));
    }
    return result;
}

/**
 * p_post_address_get_stringify_fourth_line:
 * @post_address: Direccin de envo.
 *
 * Obtiene la cuarta lnea de una direccin de envo, que contiene la provincia.
 *
 * Returns:
 * Devuelve una cadena de caracteres con la parte solicitada de una direccin.
 *
 */
gchar * p_post_address_get_stringify_fourth_line (PostAddress * post_address){
    gchar * province=NULL;
    if (post_address!=NULL) {
	province =  d_post_address_get_province (post_address);
	if (province == NULL) province = g_strdup("");
    }
    return province;
}

