/* invoice-print-wrapper.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */


#include "invoice-print-wrapper.h"
#include "locale.h"

static int do_preview;
static int copies=1, collate;
static void (*function_to_call)(void) = NULL;

/*static struct poptOption options [] =
{   
     {"copies", 0, POPT_ARG_INT, &copies },
     { "collate", 0, POPT_ARG_NONE, &collate },
     { NULL }
};
*/
/**
 * p_invoice_close_window:
 * @o: Objeto a liberar.
 * @data: Datos a utilizar por el deserrollador.
 * 
 * Cierra correctamente la ventana de vista preliminar.
 *
 */
void p_invoice_close_window (GtkObject *o, void *data)
{
    if (function_to_call !=NULL) {
        function_to_call();
    }
        
    gtk_widget_destroy(GTK_WIDGET(o));
//    exit(0);
}

/**
 * p_payment_note_wrap_print:
 * @payment_note:
 * 
 * Esta funcin abre un widget que permite seleccionar la impresora y las copias de impresin. Adems,
 * permite seleccionar previsualizar una nota o bien imprimirla sin visualizarla previamente.
 *
 */
void p_payment_note_wrap_print (PaymentNote * payment_note) {
   GnomePrintContext *pc;
   GnomePrintMaster *gpm = NULL;
   GnomePrintDialog *gpd;
   const gchar * papername = "A4";
   const GnomePaper * paper;

   /*Crea un dilogo de impresin*/
   gpd = GNOME_PRINT_DIALOG (gnome_print_dialog_new(_("Print options"), GNOME_PRINT_DIALOG_COPIES));
   gnome_print_dialog_set_copies(gpd, copies, collate);
   /*Elige por defecto el papel A4*/
   paper= gnome_paper_with_name(papername);
      gtk_signal_connect(GTK_OBJECT(gpd), "destroy", p_invoice_close_window , NULL);

      /*Ejecuta el dilogo de impresin que permite elegir la impresora, las copias y la previsualizacin del
       * documento.*/
   switch (gnome_dialog_run(GNOME_DIALOG(gpd))) 
     {	
      case GNOME_PRINT_PRINT:
	do_preview = 0;
	break;
      case GNOME_PRINT_PREVIEW:
	do_preview = 1;
	break;
      case GNOME_PRINT_CANCEL:
	  gtk_widget_destroy(GTK_WIDGET(gpd));
	  do_preview=2;
          break;
     default:
	 do_preview=2;
	 break;
     }
   
   if (do_preview<2) {
       /*    printf("printer = %p\n", gnome_print_dialog_get_printer(gpd));*/

       gpm = gnome_print_master_new();

       /*Obtiene las copias seleccionadas*/
       gnome_print_dialog_get_copies(gpd, &copies, &collate);
       /*Establece la copias en el "master" de impresin*/
       gnome_print_master_set_copies(gpm, copies, collate);
       /*Establece la impresora seleccionada*/
       gnome_print_master_set_printer(gpm, gnome_print_dialog_get_printer(gpd));

       /*Cierra el dilogo*/
       gnome_dialog_close (GNOME_DIALOG(gpd));

       /*Obtiene el contexto de impresin*/
       pc = gnome_print_master_get_context(gpm);

       /* Pages to print: */

       /*Imprime una nota en varias pginas si es necesario*/
       p_payment_note_multipage (pc,payment_note);

       gnome_print_master_close(gpm);
       gnome_print_master_set_paper(gpm,paper);
       if (do_preview==1)
       {
	   GnomePrintMasterPreview *pmp;
           /*Caso de querer previsualizar.*/
	   pmp = gnome_print_master_preview_new(gpm, "Test preview");
	   gtk_signal_connect(GTK_OBJECT(pmp), "destroy", p_invoice_close_window , NULL);
	   gtk_widget_show(GTK_WIDGET(pmp));
	   // gtk_main ();
       }
       else
       {
           /*Imprime*/
	   gnome_print_master_print(gpm);
       }
   }

/* Fixed in gnome-print-0.37, this line is needed if you want to use 
 * the program and don't loose your locale when printing or print previewing:
 *       setlocale(LC_ALL,"es_ES@euro");
 */
}

/**
 * p_invoice_wrap_print:
 * @invoice: Factura que se desea imprimir.
 * @invoice_template: Plantilla con la que se imprimir la factura.
 * @general_condition: Condicin de seleccin de atributos que se desean mostrar en la impresin.
 * @from_garage: Indica si la factura se le realiza a un taller.
 * 
 * Esta funcin abre un widget que permite seleccionar la impresora y las copias de impresin. Adems,
 * permite seleccionar previsualizar una factura o bien imprimirla sin visualizarla previamente.
 *
 */
void p_invoice_wrap_print (Invoice * invoice,gint invoice_template,GeneralCondition * general_condition,gboolean from_garage)
{

   GnomePrintContext *pc;
   GnomePrintMaster *gpm = NULL;
   GnomePrintDialog *gpd;
   const gchar * papername = "A4";
   const GnomePaper * paper;


   //Linea que estaba na inicializacin que non sei se  neceSARIA
//   gnome_init_with_popt_table ("TestPrint", VERSION, 0, NULL, options, 0, NULL);


   gpd = GNOME_PRINT_DIALOG (gnome_print_dialog_new(_("Print options"), GNOME_PRINT_DIALOG_COPIES));
   gnome_print_dialog_set_copies(gpd, copies, collate);
   /*Elige por defecto el papel A4*/
   paper= gnome_paper_with_name(papername);

   gtk_signal_connect(GTK_OBJECT(gpd), "destroy", p_invoice_close_window , NULL);
   
      /*Ejecuta el dilogo de impresin que permite elegir la impresora, las copias y la previsualizacin del
       * documento.*/
   switch (gnome_dialog_run(GNOME_DIALOG(gpd))) 
     {	
     case GNOME_PRINT_PRINT:
	do_preview = 0;
	break;
      case GNOME_PRINT_PREVIEW:
	do_preview = 1;
	break;
      case GNOME_PRINT_CANCEL:
	  gtk_widget_destroy(GTK_WIDGET(gpd));
	  do_preview=2;
          break;
     default:
	 do_preview=2;
	 break;
     }

   if (do_preview<2) {
       /*    printf("printer = %p\n", gnome_print_dialog_get_printer(gpd));*/

       gpm = gnome_print_master_new();
       /*Obtiene las copias seleccionadas*/
       gnome_print_dialog_get_copies(gpd, &copies, &collate);
       /*Establece la copias en el "master" de impresin*/
       gnome_print_master_set_copies(gpm, copies, collate);
       /*Establece la impresora seleccionada*/
       gnome_print_master_set_printer(gpm, gnome_print_dialog_get_printer(gpd));

       gnome_dialog_close (GNOME_DIALOG(gpd));

       pc = gnome_print_master_get_context(gpm);

       /* Pages to print: */

       /* Imprime la factura segn la plantilla que se haya seleccionado                    *
        * Lo que hace una  llamada a imprimir una factura es rellenar los datos             *
	* a imprimir de la factura.                                                         */
       if (invoice_template==D_INVOICE_NORMAL_CODE) {
	   p_invoice_print_multipage (pc,invoice,FALSE,general_condition,from_garage);
       }
       gnome_print_master_close(gpm);
       gnome_print_master_set_paper(gpm,paper);
       if (do_preview==1)
       {
	   GnomePrintMasterPreview *pmp;
	   pmp = gnome_print_master_preview_new(gpm, "Test preview");
	   gtk_signal_connect(GTK_OBJECT(pmp), "destroy", p_invoice_close_window , NULL);
	   gtk_widget_show(GTK_WIDGET(pmp));
	   // gtk_main ();
       }
       else
       {
	 gnome_print_master_print(gpm); 
       }
   }

/* Fixed in gnome-print-0.37, this line is needed if you want to use 
 *    the program and don't loose your locale when printing or print previewing:
 *        setlocale(LC_ALL,"es_ES@euro");
 *  */
     
}

/**
 * p_delivery_payment_wrap_print:
 * @delivery_payment: Albarn que se desea imprimir.
 * @delivery_payment_template: Plantilla con la que se imprimir el albarn.
 * @valorado: Indica si el albarn se imprime con los precios de las lneas o no.
 * 
 * Esta funcin abre un widget que permite seleccionar la impresora y las copias de impresin. Adems,
 * permite seleccionar previsualizar un albarn o bien imprimirlo sin visualizarla previamente.
 *
 */
void p_delivery_payment_wrap_print (DeliveryPayment * delivery_payment,gint delivery_payment_template,gboolean valorado)
{
   
   GnomePrintContext *pc;
   GnomePrintMaster *gpm = NULL;
   GnomePrintDialog *gpd;
   const gchar * papername = "A4";
   const GnomePaper * paper;

   gpd = GNOME_PRINT_DIALOG (gnome_print_dialog_new(_("Print options"), GNOME_PRINT_DIALOG_COPIES));
   gnome_print_dialog_set_copies(gpd, copies, collate);
   paper= gnome_paper_with_name(papername);
   gtk_signal_connect(GTK_OBJECT(gpd), "destroy", p_invoice_close_window , NULL);

   switch (gnome_dialog_run(GNOME_DIALOG(gpd)))
     {	
      case GNOME_PRINT_PRINT:
	do_preview = 0;
	break;
      case GNOME_PRINT_PREVIEW:
	do_preview = 1;
	break;
      case GNOME_PRINT_CANCEL:
	  gtk_widget_destroy(GTK_WIDGET(gpd));
	  do_preview=2;
	  break;
     default:
	 do_preview=2;
	 break;
     }

   if (do_preview<2) {
       gpm = gnome_print_master_new();
       /*Obtiene las copias seleccionadas*/
       gnome_print_dialog_get_copies(gpd, &copies, &collate);
       /*Establece la copias en el "master" de impresin*/
       gnome_print_master_set_copies(gpm, copies, collate);
       /*Establece la impresora seleccionada*/
       gnome_print_master_set_printer(gpm, gnome_print_dialog_get_printer(gpd));

       gnome_dialog_close (GNOME_DIALOG(gpd));

       pc = gnome_print_master_get_context(gpm);

       /* Pages to print: */

       /* Imprime el albarn segn la plantilla que se haya seleccionado                    *
        * Lo que hace una  llamada a imprimir una albarn es rellenar los datos             *
	* a imprimir de la factura.                                                         */
       if (delivery_payment_template==D_INVOICE_NORMAL_CODE) {
	   p_delivery_payment_print_multipage (pc,delivery_payment,valorado);
       }
       gnome_print_master_close(gpm);
       gnome_print_master_set_paper(gpm,paper);
       if (do_preview==1)
       {
	   GnomePrintMasterPreview *pmp;
	   pmp = gnome_print_master_preview_new(gpm, "Test preview");
	   gtk_signal_connect(GTK_OBJECT(pmp), "destroy", p_invoice_close_window , NULL);
	   gtk_widget_show(GTK_WIDGET(pmp));
	   // gtk_main ();
       }
       else
       {
	   gnome_print_master_print(gpm);
       }
   }

   /* Fixed in gnome-print-0.37, this line is needed if you want to use 
    *    the program and don't loose your locale when printing or print previewing:
    *        setlocale(LC_ALL,"es_ES@euro");
    *  */
  
}

/**
 * p_envelope_wrap_print:
 * @post_address: Direccin de correo que se quiere imprimir en el sobre.
 * @f_to_call: Funcin a asignar a la variable #function_to_call que se llamar
 * al destruir la ventana de impresin.
 * @client_name: Nombre completo del cliente que ir en la direccin de envo.
 * 
 * Abre una ventana que permite imprimir un sobre con la direccin de envo.
 *
 */
void p_envelope_wrap_print (PostAddress * post_address,gpointer f_to_call,gchar * client_name)
{
   
   GnomePrintContext *pc;
   GnomePrintMaster *gpm = NULL;
   GnomePrintDialog *gpd;
//   const gchar * papername = "Envelope-Commercial";
//   const GnomePaper * paper;


   function_to_call=f_to_call;
//   paper= gnome_paper_with_name(papername);
//   p_invoice_list_papers ();
   gpd = GNOME_PRINT_DIALOG (gnome_print_dialog_new(_("Print options"), GNOME_PRINT_DIALOG_COPIES));
   gnome_print_dialog_set_copies(gpd, copies, collate);
   gtk_signal_connect(GTK_OBJECT(gpd), "destroy", p_invoice_close_window , NULL);

   switch (gnome_dialog_run(GNOME_DIALOG(gpd)))
     {	
      case GNOME_PRINT_PRINT:
	do_preview = 0;
	break;
      case GNOME_PRINT_PREVIEW:
	do_preview = 1;
	break;
      case GNOME_PRINT_CANCEL:
	  gtk_widget_destroy(GTK_WIDGET(gpd));
	  do_preview=2;
	  break;
     default:
	 do_preview=2;
	 break;

     }

   if (do_preview<2) {
       gpm = gnome_print_master_new();
       gnome_print_dialog_get_copies(gpd, &copies, &collate);
       gnome_print_master_set_copies(gpm, copies, collate);
       gnome_print_master_set_printer(gpm, gnome_print_dialog_get_printer(gpd));

       gnome_dialog_close (GNOME_DIALOG(gpd));

       pc = gnome_print_master_get_context(gpm);

       p_invoice_print_envelope (pc,post_address,client_name);

       gnome_print_master_close(gpm);
       if (do_preview==1)
       {
       } else
       {
	   gnome_print_master_print(gpm);
       }
   }

     
/* Fixed in gnome-print-0.37, this line is needed if you want to use 
 *    the program and don't loose your locale when printing or print previewing:
 *        setlocale(LC_ALL,"es_ES@euro");
 */
   }


/**
 * p_invoice_list_papers:
 * 
 * Lista los tipos de papeles que acepta el gnome-print.
 *
 */
void p_invoice_list_papers (void) {
    GList * list = gnome_paper_name_list();
    GList * a_list = g_list_first(list);
    while (a_list!=NULL) {
	gchar * name = (gchar *) a_list->data;
        g_print("\n%s\n",name);
        a_list = g_list_next(a_list);
    }

}

