
/* listing-print.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */


#include "listing-print.h"

/**
 * p_listing_print_one_page:
 * @pc: Contexto en el que se indicar lo que se quiere imprimir.
 * @listing: Listado a imprimir.
 * @listing_parameters: Parmetros para imprimir el listado.
 * @next_row: Proxima fila a imprimir.
 * 
 * Indica al contexto lo que hay que imprimir para sacar una pgina del listado.
 *
 */
void p_listing_print_one_page (GnomePrintContext *pc,Listing * listing,ListingParameters * listing_parameters,gint next_row)
{
    GnomeFont *font=NULL;
    gint number_of_rows=0;
    gint n_cols = listing->n_columns;
    gint vertical_jump = 0; /* Espacio que debe saltar en caso de ser la primera hoja */
    gint n_col_to_paint=0;
    gint desp_by_landscape=0;
    gint dif_font_by_landscape=0;

    if (listing->orientation==P_LANSCAPE) desp_by_landscape=820;
    if (listing->orientation==P_LANSCAPE) dif_font_by_landscape=2;

    if (next_row<listing_parameters->n_rows_on_first_page) {
	font = gnome_font_new_closest ("Helvetica", GNOME_FONT_BOLD, 1, 12-dif_font_by_landscape);
	if (font == NULL)
	{
	    g_warning(_("Font cannot be found"));
	}

	gnome_print_setfont (pc, font);

	gnome_print_gsave(pc);
	p_text_centered_in_box (pc, font, P_LEFT_LIMIT, 800-desp_by_landscape, strlen(listing->listing_title)*10,25 , listing->listing_title);
//	p_print_centered_text (pc, font, 10, 800, strlen(listing->listing_title)*10, listing->listing_title);
	gnome_print_grestore(pc);

	vertical_jump = 30;
    } else {
	vertical_jump = 0;
    }
    font = gnome_font_new_closest ("Times", GNOME_FONT_BOLD, 1, 10-dif_font_by_landscape);
    if (font == NULL)
    {
	g_warning(_("Font cannot be found"));
    }

    gnome_print_setfont (pc, font);

    if (listing->content_title!=NULL) {
	gint index_contents = 0;
        gint desp_acum=P_LEFT_LIMIT;
	while (listing->content_title[index_contents]!=NULL) {
            gint new_desp=((listing_parameters->horizontal_space)*((listing->width_of_cols[index_contents])/100.0));
            gint desp_acum_post = desp_acum +new_desp ;
            p_text_centered_in_box (pc,font,desp_acum,800-vertical_jump-desp_by_landscape,new_desp,17,listing->content_title[index_contents]);
	    desp_acum =  desp_acum_post;
            index_contents++;
	}
    }
    if (next_row<listing_parameters->n_rows_on_first_page) {
	n_col_to_paint=listing_parameters->n_rows_on_first_page;
        number_of_rows = listing_parameters->n_rows_on_first_page;
    } else {
        n_col_to_paint=listing_parameters->n_rows_on_page;
        number_of_rows = listing_parameters->n_rows_on_page;
    }
    font = gnome_font_new_closest ("Times", GNOME_FONT_BOOK, 1, 10-dif_font_by_landscape);
    if (font == NULL)
    {
	g_warning(_("Font cannot be found"));
    }

    gnome_print_setfont (pc, font);

    if (listing->content!=NULL) {
	gint index_row=next_row;
	gint index_row_actual_page=0;
	while ((index_row_actual_page<number_of_rows) && (listing->content[index_row]!=NULL)) {
	    gint indexCol=0;
	    gint desp_acum=P_LEFT_LIMIT;
	    while ((indexCol<n_cols) && (listing->content[index_row][indexCol]!=NULL)) {
		gint new_desp=((listing_parameters->horizontal_space)*((listing->width_of_cols[indexCol])/100.0));
		gint desp_acum_post = desp_acum +new_desp ;
		if (listing->content[index_row][indexCol]!=NULL) {
		    if (listing->alignment_of_cols[indexCol]==U_LEFT) {
			gnome_print_moveto(pc, desp_acum,770-vertical_jump*1-(index_row_actual_page*listing_parameters->row_heigth)-desp_by_landscape);
			p_print_show (pc,listing->content[index_row][indexCol]);
		    } else if (listing->alignment_of_cols[indexCol]==U_CENTER) {
			p_print_centered_text (pc, font, desp_acum, 770-vertical_jump*1-(index_row_actual_page*listing_parameters->row_heigth)-desp_by_landscape, new_desp, listing->content[index_row][indexCol]);
		    } else if (listing->alignment_of_cols[indexCol]==U_RIGHT) {
			p_print_text_righted (pc, font, desp_acum, 770-vertical_jump*1-(index_row_actual_page*listing_parameters->row_heigth)-desp_by_landscape, new_desp, listing->content[index_row][indexCol]);
 
		    }
		}
		desp_acum=desp_acum_post;
		indexCol++;
	    }
	    index_row++;
	    index_row_actual_page++;

	}
        
    }
    

}

/**
 * p_listing_multipage:
 * @pc: Contexto en el que se indicar lo que se quiere imprimir.
 * @listing: Listado a imprimir.
 * @listing_parameters: Parmetros para imprimir el listado.
 * 
 * Permite imprimir un listado en varias pginas.
 *
 */
void p_listing_multipage (GnomePrintContext *pc,Listing * listing,ListingParameters * listing_parameters) {
    gint index_page = 0;
    gchar * temp=NULL;
    gint next_row=0;

    g_print(_("\nNumber of pages: %d\n"),listing_parameters->n_pages);
    while (index_page<(listing_parameters->n_pages)) {
	temp = g_strdup_printf(_("Listing, page %d"),index_page);
	gnome_print_beginpage (pc, temp);
	if (listing->orientation==P_LANSCAPE)
            gnome_print_rotate(pc,90);
	g_free(temp);
	if (index_page == 0) {
	    next_row = 0;
	} else {
	    next_row = (listing_parameters->n_rows_on_first_page) + ((listing_parameters->n_rows_on_page)*(index_page-1));
	}
	p_listing_print_one_page (pc,listing,listing_parameters,next_row);
	gnome_print_showpage (pc);
        index_page++;
    }
    gnome_print_context_close(pc);

}

