/* select_fields.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "select_fields.h"
#include "invoice.h"

/**
 * p_create_condition:
 * @invoiceTemplate: Plantilla de la factura para la que se quiere conseguir.
 *
 * Crea una condicin inicializada para imprimir los datos de cada plantilla. Inicialmente
 * se ponen a %TRUE todas los datos de una plantilla.
 *
 * Returns:
 * La condicin con la memoria reservada y inicializados a %TRUE los datos segn la plantilla seleccionada.
 *
 */
GeneralCondition * p_create_condition (gint invoiceTemplate) {
    GeneralCondition * general_condition = (GeneralCondition *) g_malloc0(sizeof(GeneralCondition));

    switch (invoiceTemplate) {
   case D_INVOICE_NORMAL_CODE:{
	general_condition->date=TRUE;
	general_condition->page_number=TRUE;
	general_condition->enterprise_legal_id_template_1=TRUE;
	general_condition->client_number=TRUE;
	general_condition->invoice_number=TRUE;
	general_condition->agent=TRUE;
	general_condition->nick=TRUE;
	general_condition->street=TRUE;
	general_condition->town=TRUE;
	general_condition->province=TRUE;
	general_condition->company=TRUE;
	general_condition->policy_number=TRUE;
	general_condition->work_number=TRUE;
	general_condition->reference=TRUE;
	general_condition->description=TRUE;
	general_condition->quantity=TRUE;
	general_condition->prices=TRUE;
	general_condition->total=TRUE;
	general_condition->discount=TRUE;
	general_condition->invoice_total=TRUE;
	general_condition->payment_way=TRUE;
	general_condition->account=TRUE;
	general_condition->payment_entity=TRUE;
	general_condition->instalments=TRUE;
	general_condition->amount=TRUE;
	general_condition->enterprise_name=TRUE;
	general_condition->enterprise_legal_id=TRUE;
	general_condition->garage_data=FALSE;
	general_condition->name=FALSE;
	general_condition->surnames=FALSE;
	general_condition->postal_code=FALSE;
	general_condition->phone=FALSE;
	general_condition->accident_number=FALSE;
	general_condition->accident_date=FALSE;
	general_condition->field=FALSE;
	general_condition->province_code=FALSE;
	general_condition->reason_code=FALSE;
	general_condition->made=FALSE;
	general_condition->model=FALSE;
	general_condition->license_number=FALSE;
	general_condition->chassis=FALSE;
	general_condition->kilometers=FALSE;
	general_condition->colour=FALSE;
	general_condition->in_date=FALSE;
	general_condition->dossier=FALSE;
	general_condition->out_date=FALSE;
	general_condition->accident_description=FALSE;
	general_condition->repairer_centre_code=FALSE;
	general_condition->discounts=FALSE;
	general_condition->repairer_centre_data=FALSE;
	general_condition->material_quantity=FALSE;
	general_condition->material_description=FALSE;
	general_condition->line_total=FALSE;
	general_condition->man_power_description=FALSE;
	general_condition->man_power_total=FALSE;
	general_condition->subtotal=TRUE;
	general_condition->vat=TRUE;
	general_condition->receiver_legal_id=FALSE;
	general_condition->observations=FALSE;
	general_condition->totals=FALSE;
	general_condition->validity_period=FALSE;
	general_condition->holder=FALSE;
	general_condition->place=FALSE;
	general_condition->way=FALSE;
	general_condition->damages=FALSE;
	general_condition->receiver_nick=FALSE;
	general_condition->receipt_number=FALSE;
	general_condition->branch=FALSE;
	general_condition->invoice_lines=TRUE;

        break;
    }
    default:{
	general_condition->province=FALSE;
	general_condition->invoice_number=FALSE;
	general_condition->work_number=FALSE;
	general_condition->garage_data=FALSE;
	general_condition->name=FALSE;
	general_condition->nick=FALSE;
	general_condition->surnames=FALSE;
	general_condition->street=FALSE;
	general_condition->town=FALSE;
	general_condition->postal_code=FALSE;
	general_condition->phone=FALSE;
	general_condition->total=FALSE;
	general_condition->accident_number=FALSE;
	general_condition->policy_number=FALSE;
	general_condition->accident_date=FALSE;
	general_condition->field=FALSE;
	general_condition->province_code=FALSE;
	general_condition->reason_code=FALSE;
	general_condition->branch=FALSE;
	general_condition->made=FALSE;
	general_condition->model=FALSE;
	general_condition->license_number=FALSE;
	general_condition->chassis=FALSE;
	general_condition->kilometers=FALSE;
	general_condition->colour=FALSE;
	general_condition->in_date=FALSE;
	general_condition->out_date=FALSE;
	general_condition->accident_description=FALSE;
	general_condition->repairer_centre_code=FALSE;
	general_condition->enterprise_legal_id_template_1=FALSE;
	general_condition->repairer_centre_data=FALSE;
	general_condition->reference=FALSE;
	general_condition->material_quantity=FALSE;
	general_condition->material_description=FALSE;
	general_condition->line_total=FALSE;
	general_condition->man_power_description=FALSE;
	general_condition->man_power_total=FALSE;
	general_condition->subtotal=FALSE;
	general_condition->discounts=FALSE;
	general_condition->discount=FALSE;
	general_condition->vat=FALSE;
	general_condition->invoice_total=FALSE;
	general_condition->date=FALSE;
	general_condition->receiver_legal_id=FALSE;
	general_condition->observations=FALSE;
	general_condition->description=FALSE;
	general_condition->place=FALSE;
	general_condition->way=FALSE;
	general_condition->damages=FALSE;
	general_condition->quantity=FALSE;
	general_condition->prices=FALSE;
	general_condition->totals=FALSE;
	general_condition->validity_period=FALSE;
	general_condition->holder=FALSE;
	general_condition->receiver_nick=FALSE;
	general_condition->page_number=FALSE;
	general_condition->client_number=FALSE;
	general_condition->agent=FALSE;
	general_condition->payment_way=FALSE;
	general_condition->payment_entity=FALSE;
	general_condition->account=FALSE;
	general_condition->instalments=FALSE;
	general_condition->amount=FALSE;
	general_condition->enterprise_name=FALSE;
	general_condition->enterprise_legal_id=FALSE;
	general_condition->invoice_lines=FALSE;

        break;
    }
    }
    return general_condition;
}


