/* account_model.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include "account_model.h"
#include "office_model.h"
#include "enterprise_model.h"
#include "shop_model.h"
#include "warehouse_model.h"
#include "parameter_manager.h"

#define TABLE         "CuentaCorriente "
#define ATTRIBUTES    "CuentaCorriente.codigoBanco as codigoBancoAccount , \
                       CuentaCorriente.codigoOficina as codigoOficinaAccount , \
                       CuentaCorriente.numerocuenta as numerocuentaAccount , \
                       CuentaCorriente.dc as dcAccount , \
                       CuentaCorriente.saldoInicial as saldoInicialAccount "



/* GNOME-DOC aadiendo * al inicio de este comentario
 * insert_account - Inserta el objeto account en la BD. ESTA FUNCIN NO DEBERA SER UTILIZADA.
 * @account: Objeto que representa la cuenta corriente
 * Return value: %TRUE si se pudo completar la insercin.
 * Description:
 * Funcin que se utiliza para insertar en la base de datos una cuenta corriente.
 * OJO! En la BD existen dos tablas diferentes en las cuales se podran insertar
 * cuentas corrientes. La cuenta corriente de esta funcin se corresponde con una
 * cuenta de la empresa con lo debe llevar un saldo inicial.
 * Error: no es posible insertar la cuenta.
 * desc: Si falla la conexin de la Bd o bien ya existe la cuenta.
 * source: mltiples orgenes
 */

/**
 * m_account_get_table:
 *
 * Devuelve un string con la tabla correspondiente al objeto de la base de datos. Si el
 * objeto se corresponde con varias tablas se devolveran los nombres separados por comas.
 *
 * Returns:
 * Cadena de caracteres con la tabla o lista de tablas a las que se refiere el objeto.
 */
gchar *m_account_get_table () {
    return TABLE;
}

/**
 * m_account_get_attributes:
 *
 * Funcin que obtiene los atributos de un objeto, estos atributos son los que corresponden al
 * objeto en la base de datos.
 *
 * Returns:
 * Cadena de caracteres con los atributos a los que se refiere el objeto con un alias.
 */
gchar *m_account_get_attributes () {
    return ATTRIBUTES;
}
/**
 * m_account_create:
 * @rs: Record set que contiene los campos de la BD.
 *
 * Crea un objeto a partir de los valores de un record set de la base de datos.
 *
 * Returns:
 * Un objeto #Account con los campos rellenados con los datos de la base de datos.
 */
Account *m_account_create (GdaRecordset *rs) {
    GdaField *field;
    gchar    *temp;
    Account  *account = d_account_create_empty();



    field = gda_recordset_field_name(rs,"codigoOficinaaccount");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    account->office=m_office_create (rs);

	    field = gda_recordset_field_name(rs,"dcaccount");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    account->control_digit = gda_field_get_smallint_value(field);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs,"numeroCuentaaccount");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    temp=gda_stringify_value(NULL, 0, field);
		    account-> account_number = g_strdup(temp);
		    g_free(temp);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs,"saldoInicialaccount");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    temp=gda_stringify_value(NULL, 0, field);
		    account->initial_balance = atof(temp);
		    g_free(temp);
		}
		gda_field_free(field);
	    }
	} else {
	    gda_field_free(field);
	}
    }

    return account;
}



/**
 * m_account_insert: 
 * @account: Objeto que representa la cuenta corriente
 *
 * Inserta el objeto account en la BD. ESTA FUNCIN NO DEBERA SER UTILIZADA.
 * Funcin que se utiliza para insertar en la base de datos una cuenta corriente.
 * OJO! En la BD existen dos tablas diferentes en las cuales se podran insertar
 * cuentas corrientes. La cuenta corriente de esta funcin se corresponde con una
 * cuenta de la empresa con lo debe llevar un saldo inicial.
 *
 * Returns: %TRUE si se pudo completar la insercin.
 */
gboolean m_account_insert (Account * account) {
    gboolean result=TRUE;
    gchar * query=NULL;
    gchar * query_aux=NULL;
    gchar * values=NULL;
    gchar * values_aux;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    GdaRecordset *record_set=NULL;
    GdaConnection * connection = m_model_get_connection ();
    GdaCommand * command = gda_command_new();
    gint bank_code = d_office_get_bank_code (d_account_get_office (account));
    gint office_code = d_office_get_office_code (d_account_get_office (account));
    gchar * account_number = d_account_get_account_number (account);
    gulong logged_shop_code;
    gulong logged_enterprise_code;
    gulong logged_warehouse_code;
    gchar * temp_name;

    temp_name = u_parameter_manager_get_parameter ("empresa");
    logged_enterprise_code = m_enterprise_get_code_from_name (temp_name);
    g_free(temp_name);
    temp_name = u_parameter_manager_get_parameter ("tienda");
    logged_shop_code = m_shop_get_code_from_name (temp_name);
    g_free(temp_name);
    temp_name = u_parameter_manager_get_parameter ("almacen");
    logged_warehouse_code = m_warehouse_get_code_from_name (temp_name);

    g_free(temp_name);
    gda_command_set_connection(command,connection);
    m_model_connection_begin_mutex_transaction_logged (connection,logged_enterprise_code,logged_shop_code,logged_warehouse_code);

    if ((bank_code!=-1) && (office_code!=-1) && (account_number!=NULL)) {
        gchar * temp;
	query = g_strconcat(" INSERT INTO CUENTACORRIENTE VALUES ",NULL);
	values = g_strconcat(" (",NULL);

	temp = g_strdup_printf("%d",bank_code);
	values_aux = g_strconcat(values,temp,",",NULL);
	g_free(values);
	g_free(temp);
	values = values_aux;

	temp = g_strdup_printf("%d",office_code);
	values_aux = g_strconcat(values,temp,",",NULL);
	g_free(values);
	g_free(temp);
	values = values_aux;


	values_aux = g_strconcat(values,account_number,",",NULL);
	g_free(values);
	values = values_aux;

	if (d_account_get_control_digit (account)!=-1) {
	    temp = g_strdup_printf("%d",d_account_get_control_digit (account));
	    values_aux = g_strconcat(values,temp,",",NULL);
	    g_free(values);
	    g_free(temp);
	    values = values_aux;
	} else {
	    values_aux = g_strconcat(values,"NULL,",NULL);
	    g_free(values);
	    values = values_aux;
	}

	if (d_account_get_initial_balance (account)!=-1) {
	    temp = g_strdup_printf("%.2f",d_account_get_initial_balance (account));
	    values_aux = g_strconcat(values,"to_number('",temp,"','99999999D99')",NULL);
	    g_free(values);
	    g_free(temp);                         
	    values = values_aux;
	} else {
	    values_aux = g_strconcat(values,"NULL",NULL);
	    g_free(values);
	    values = values_aux;
	}

	query_aux=g_strconcat(query,values,")",NULL);
	g_free(query);
	query=query_aux;
	query_aux=NULL;

	result = m_model_command_execute (&record_set, query, connection,
				       command,tuples,0, FALSE);

    }
    result = m_model_execute_mutex_modification_logged (connection) && result;

    if (query!=NULL) g_free(query);
    if (tuples!=NULL) g_free(tuples);
    if (account_number!=NULL) g_free(account_number);

    if (record_set!=NULL)
	gda_recordset_free(record_set);




    igalia_command_free(command);

    return result;
}

/**
 * m_account_update: 
 * @account: Cuenta a modificar. Datos nuevos.
 * @participant_code: Cdigo del cliente que se modificar.
 * @previous_account_number: Nmero de cuenta antiguo.
 * @previous_office_code: Cdigo de oficina antiguo
 * @previous_code_code: Cdigo de banco antiguo
 *
 * Modifica la cuenta de un cliente en la BD.
 *
 * Returns: %TRUE si se pudo completar la modificacin.
 */
gboolean m_account_update (Account * account,gulong participant_code,gchar * previous_account_number,gchar * previous_office_code,gchar * previous_code_code) {
    gboolean result=TRUE;
    gchar * query=NULL;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    GdaRecordset *record_set=NULL;
    GdaConnection * connection = m_model_get_connection ();
    GdaCommand * command = gda_command_new();
    gint bank_code = d_office_get_bank_code (d_account_get_office (account));
    gint office_code = d_office_get_office_code (d_account_get_office (account));
    gchar *  bank_code_char = NULL;
    gchar *  office_code_char = NULL;
    gchar *  participant_code_char=NULL;
    gchar * account_number = d_account_get_account_number (account);
    gulong logged_shop_code;
    gulong logged_enterprise_code;
    gulong logged_warehouse_code;
    gchar * temp_name;

    temp_name = u_parameter_manager_get_parameter ("empresa");
    logged_enterprise_code = m_enterprise_get_code_from_name (temp_name);
    g_free(temp_name);
    temp_name = u_parameter_manager_get_parameter ("tienda");
    logged_shop_code = m_shop_get_code_from_name (temp_name);
    g_free(temp_name);
    temp_name = u_parameter_manager_get_parameter ("almacen");
    logged_warehouse_code = m_warehouse_get_code_from_name (temp_name);

    g_free(temp_name);
    gda_command_set_connection(command,connection);
    m_model_connection_begin_mutex_transaction_logged (connection,logged_enterprise_code,logged_shop_code,logged_warehouse_code);

    if ((bank_code!=-1) && (office_code!=-1) && (account_number!=NULL)) {

	bank_code_char = g_strdup_printf("%d",bank_code);

	office_code_char = g_strdup_printf("%d",office_code);

	participant_code_char = g_strdup_printf("%lu",participant_code);



	query = g_strconcat(" UPDATE CCPARTY SET ",
			    " codigoBanco = ",bank_code_char,
			    " ,codigoOficina = ",office_code_char,
			    " ,numeroCuenta = ",account_number,
			    " where codigoPArty = ",participant_code_char,
			    " and codigoBanco = ",previous_code_code,
			    " and codigoOficina = ",previous_office_code,
			    " and numeroCuenta = ",previous_account_number,
			    NULL);

	result = m_model_command_execute (&record_set, query, connection,
				       command,tuples,0, FALSE);
                      
	g_print(_("\nUpdate: %s\n"),query);
    }
    result = m_model_execute_mutex_modification_logged (connection) && result;

    if (query!=NULL) g_free(query);
    if (tuples!=NULL) g_free(tuples);
    if (account_number!=NULL) g_free(account_number);
    if (bank_code_char!=NULL) g_free(bank_code_char);
    if (office_code_char!=NULL) g_free(office_code_char);
/*    if (previous_code_code!=NULL) g_free(previous_code_code);
    if (previous_office_code!=NULL) g_free(previous_office_code);
    if (previous_account_number!=NULL) g_free(previous_account_number);
 */
    if (record_set!=NULL)
	gda_recordset_free(record_set);




    igalia_command_free(command);

    return result;
}


/**
 * m_account_insert_in_transaction:
 * @account: Objeto que representa la cuenta corriente
 * @connection: Conexin en la que se ejetuca la actual transaccin.

 * Inserta el objeto account en la BD,dentro de una transaccin.
 * Este mtodo es llamado desde la insercin de cuentas de un cliente, para que se
 * inserte la cuenta en caso de que el cliente sea una empresa de la empresa.
 * OJO! En la BD existen dos tablas diferentes en las cuales se podran insertar
 * cuentas corrientes. La cuenta corriente de esta funcin se corresponde con una
 * cuenta de la empresa con lo debe llevar un saldo inicial.

 * Returns: %TRUE si se pudo completar la insercin.
 */
gboolean m_account_insert_in_transaction (Account * account,GdaConnection * connection) {
    gboolean result=TRUE;
    gchar * query=NULL;
    gchar * query_select=NULL;
    gchar * query_aux=NULL;
    gchar * values=NULL;
    gchar * values_aux;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    GdaRecordset *record_set=NULL;
    GdaRecordset *record_set_select=NULL;
    GdaCommand * command = gda_command_new();
    gint bank_code = d_office_get_bank_code (d_account_get_office (account));
    gint office_code = d_office_get_office_code (d_account_get_office (account));
    gchar * account_number = d_account_get_account_number (account);
    gulong logged_shop_code;
    gulong logged_enterprise_code;
    gulong logged_warehouse_code;
    gchar * bank_code_char = NULL;
    gchar * office_code_char = NULL;
    gchar * account_number_char = NULL;
    gchar * temp_name;

    temp_name = u_parameter_manager_get_parameter ("empresa");
    logged_enterprise_code = m_enterprise_get_code_from_name (temp_name);
    g_free(temp_name);
    temp_name = u_parameter_manager_get_parameter ("tienda");
    logged_shop_code = m_shop_get_code_from_name (temp_name);
    g_free(temp_name);
    temp_name = u_parameter_manager_get_parameter ("almacen");
    logged_warehouse_code = m_warehouse_get_code_from_name (temp_name);

    g_free(temp_name);
    gda_command_set_connection(command,connection);

    if ((bank_code!=-1) && (office_code!=-1) && (account_number!=NULL)) {
	gchar * temp;

	bank_code_char = g_strdup_printf("%d",bank_code);
	office_code_char =g_strdup_printf("%d",office_code);
        account_number_char=g_strdup(account_number);

	query_select = g_strconcat("SELECT * FROM CUENTACORRIENTE WHERE ",
				   " codigoBanco=",bank_code_char,
				   " and codigoOficina=",office_code_char,
				   " and numeroCuenta=",account_number_char,
				   NULL);
	result = m_model_command_execute (&record_set_select, query_select, connection,
				       command,tuples,0,TRUE);
	if ((*tuples) == 0) {

	    query = g_strconcat(" INSERT INTO CUENTACORRIENTE VALUES ",NULL);
	    values = g_strconcat(" (",NULL);

	    temp = g_strdup_printf("%d",bank_code);
	    values_aux = g_strconcat(values,temp,",",NULL);
	    g_free(values);
	    g_free(temp);
	    values = values_aux;

	    temp = g_strdup_printf("%d",office_code);
	    values_aux = g_strconcat(values,temp,",",NULL);
	    g_free(values);
	    g_free(temp);
	    values = values_aux;


	    values_aux = g_strconcat(values,account_number,",",NULL);
	    g_free(values);
	    values = values_aux;

	    if (d_account_get_control_digit (account)!=-1) {
		temp = g_strdup_printf("%d",d_account_get_control_digit (account));
		values_aux = g_strconcat(values,temp,",",NULL);
		g_free(values);
		g_free(temp);
		values = values_aux;
	    } else {
		values_aux = g_strconcat(values,"NULL,",NULL);
		g_free(values);
		values = values_aux;
	    }

	    if (d_account_get_initial_balance (account)!=-1) {
		temp = g_strdup_printf("%.2f",d_account_get_initial_balance (account));
		values_aux = g_strconcat(values,"to_number('",temp,"','99999999D99')",NULL);
		g_free(values);
		g_free(temp);
		values = values_aux;
	    } else {
		values_aux = g_strconcat(values,"0.0",NULL);
		g_free(values);
		values = values_aux;
	    }

	    query_aux=g_strconcat(query,values,")",NULL);
	    g_free(query);
	    query=query_aux;
	    query_aux=NULL;

	    result = m_model_command_execute (&record_set, query, connection,
					   command,tuples,0, FALSE);
	}
        if (bank_code_char !=NULL)
	    g_free(bank_code_char);
        if (office_code_char!=NULL)
	    g_free(office_code_char);
        if (account_number_char !=NULL)
	    g_free(account_number_char);

    }


    if (query!=NULL) g_free(query);
    if (tuples!=NULL) g_free(tuples);
    if (account_number!=NULL) g_free(account_number);

    if (record_set!=NULL)
	gda_recordset_free(record_set);




    igalia_command_free(command);

    return result;
}




/**
 * m_account_find_bank_by_code: 
 * @bank_code: El cdigo del banco, es el primer nmero de un nmero de cuenta.
 *
 * Obtiene el nombre del banco a partir del cdigo.
 *
 * Returns: El nombre del banco. %NULL si no lo encuentra.
 */

gchar * m_account_find_bank_by_code (gchar * bank_code) {
    GdaCommand         *my_command;
    gchar              *gchar_query=NULL;
    GdaConnection      *my_connection = m_model_get_connection ();
    GdaRecordset       *record_set = NULL;
    gchar * name;
    gint index;
    gulong * tuples;

    my_command = gda_command_new();
    tuples = (gulong *) g_malloc0(sizeof(gulong));
    gda_command_set_connection(my_command,my_connection);

    gchar_query = g_strconcat(" SELECT nombre from banco where codigoBanco=",bank_code,
			       NULL );

    m_model_command_execute (&record_set, gchar_query, my_connection, my_command,tuples,0, TRUE);

    if (record_set!=NULL) {
	if (*tuples==1) {
            GdaField * field;
	    index = gda_recordset_move_first(record_set);
	    if (index==GDA_RECORDSET_INVALID_POSITION) return NULL;
	    field = gda_recordset_field_idx(record_set,0);
	    name = gda_stringify_value(NULL, 0, field);
	    if (field!=NULL)
                gda_field_free(field);
	} else {
	    name=NULL;
	}
	gda_recordset_close(record_set);
    } else {
        name = NULL;
    }


    g_free(gchar_query);
    g_free(tuples);


    igalia_command_free(my_command);
    g_print(_("Freeing database completed\n"));

    return name;
}

/**
 * m_account_clear_array:
 * @accounts: El array de cuentas a liberar
 *
 * Libera la memoria ocupada por el array de cuentas
 *
 */

void m_account_clear_array (Account ** accounts) {
    if (accounts!=NULL) {
	gint index = 0;
	while (accounts[index]!=NULL) {
	    d_account_clear (accounts[index]);
	    index++;
	}
    }
    g_free(accounts);
    accounts=NULL;
}


/**
 * m_account_find_by_participant_code: 
 * @participant_code: Cdigo del cliente del que se quiere obtener las cuentas
 *
 * Devuelve la lista de cuentas corrientes
 * de un cliente de la empresa identificado por @participant_code
 *
 * Returns: Una lista de cuentas corrientes del cliente, es decir, una lista
 * de objetos account.
 */
GList * m_account_find_by_participant_code (gulong participant_code) {
    GdaCommand         *my_command;
    gchar              *gchar_query=NULL;
    gchar              *gchar_select=NULL;
    gchar              *gchar_from=NULL;
    gchar              *gchar_where=NULL;
    GdaConnection      *my_connection = m_model_get_connection ();
    GdaRecordset       *record_set = NULL;

    GList * accounts=NULL;
    Account * account;
    gint index;
    gulong * tuples;
    gchar * participant_code_char = g_strdup_printf("%lu",participant_code);
    my_command = gda_command_new();
    tuples = (gulong *) g_malloc0(sizeof(gulong));
    gda_command_set_connection(my_command,my_connection);

    gchar_select = g_strconcat(" SELECT ",
                               "CCParty.codigoBanco as codigoBancoOffice,",
                               "CCParty.codigoOficina as codigoOficinaOffice,",
			       "CCParty.codigoBanco as codigoBancoAccount,",
			       "CCParty.codigoOficina as codigoOficinaAccount,",
			       "CCParty.numeroCuenta as numeroCuentaAccount",
			       NULL );


    gchar_from = g_strconcat(" from CCParty left outer join oficina on",NULL);
    gchar_where = g_strconcat(" (CCParty.codigoBanco = oficina.codigoBanco and ",
			      " CCParty.codigoOficina = oficina.codigoOficina) where ",
			      " CCParty.codigoParty=",participant_code_char,NULL);

    g_free(participant_code_char);

    gchar_query = g_strconcat(gchar_select,gchar_from,gchar_where,NULL);

    g_print(_("\nQuery: %s\n"),gchar_query);

    m_model_command_execute (&record_set, gchar_query, my_connection, my_command,tuples,0, TRUE);

    if (record_set!=NULL) {
	if (*tuples!=0) {
		index = gda_recordset_move_first(record_set);
		while (!gda_recordset_eof(record_set)) {
		    if (index==GDA_RECORDSET_INVALID_POSITION) return NULL;
		    account = m_account_create (record_set);
		    accounts = g_list_append(accounts,account);
		    index = gda_recordset_move_next(record_set);
		}
	    } else {
		accounts=NULL;
	    }
	    gda_recordset_close(record_set);
	} else {
	    accounts=NULL;
	}


    g_free(gchar_from);
    g_free(gchar_query);
    g_free(gchar_select);
    g_free(gchar_where);
    g_free(tuples);


    igalia_command_free(my_command);
    g_print(_("Freeing database completed\n"));

    return accounts;


}

/**
 * m_account_find_all: 
 *
 * Devuelve la lista de cuentas corrientes
 * de todas las empresas de la empresa
 *
 * Returns: Un array de cuentas corrientes de las empresas de la empresa.
 */
Account ** m_account_find_all () {
    GdaCommand         *my_command;
    gchar              *gchar_query=NULL;
    gchar              *gchar_select=NULL;
    gchar              *gchar_from=NULL;
    gchar              *gchar_where=NULL;
    GdaConnection      *my_connection = m_model_get_connection ();
    GdaRecordset       *record_set = NULL;

    Account ** accounts=NULL;
    Account * account;
    gint index;
    gulong * tuples;

    my_command = gda_command_new();
    tuples = (gulong *) g_malloc0(sizeof(gulong));
    gda_command_set_connection(my_command,my_connection);

    gchar_select = g_strconcat(" SELECT ",
                               "oficina.codigoBanco as codigoBancoOffice,",
                               "oficina.codigoOficina as codigoOficinaOffice,",
			       "cuentaCorriente.codigoBanco as codigoBancoAccount,",
			       "cuentaCorriente.codigoOficina as codigoOficinaAccount,",
			       "cuentaCorriente.numeroCuenta as numeroCuentaAccount,",
			       "cuentaCorriente.dc as dcAccount,",
			       "cuentaCorriente.saldoInicial as saldoInicialAccount",
			       NULL );


    gchar_from = g_strconcat(" from cuentaCorriente,oficina ",NULL);
    gchar_where = g_strconcat(" where cuentaCorriente.codigoBanco = oficina.codigoBanco and ",
			      " cuentaCorriente.codigoOficina = oficina.codigoOficina ",NULL);



    gchar_query = g_strconcat(gchar_select,gchar_from,gchar_where,NULL);

    g_print(_("\nQuery: %s\n"),gchar_query);

    m_model_command_execute (&record_set, gchar_query, my_connection, my_command,tuples,0, TRUE);

    if (record_set!=NULL) {
	if (*tuples!=0) {
	    accounts = (Account **) g_malloc0(sizeof(Account*)*(*tuples+1));
		index = gda_recordset_move_first(record_set);
		while (!gda_recordset_eof(record_set)) {
		    if (index==GDA_RECORDSET_INVALID_POSITION) return NULL;
		    account = m_account_create (record_set);
		    accounts[index-1] = account;
		    index = gda_recordset_move_next(record_set);
		}
	    } else {
		accounts=NULL;
	    }
	    gda_recordset_close(record_set);
	} else {
	    accounts=NULL;
	}

	if (accounts!=NULL) {
	    accounts[*tuples]= NULL;
	}

	g_free(gchar_from);
	g_free(gchar_query);
	g_free(gchar_where);
	g_free(tuples);


    igalia_command_free(my_command);
    g_print(_("Freeing database completed\n"));

    return accounts;

}


