/* accounting_period_model.h - Modelo que representa un perodo de contabilidad.
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#ifndef _ACCOUNTING_PERIOD_MODEL_
#define _ACCOUNTING_PERIOD_MODEL_

#include <gda-client.h>
#include <gda-field.h>

#include "model.h"

#include "gda_igalia_util.h"
#include "accounting_period.h"
#include "gda_igalia_util.h"
#include "parameter_manager.h"
#include "shop_model.h"
#include "warehouse_model.h"
/*
 ConditionAccountingPeriod es una estructura utilizada para filtrar las bsquedas de temas relacionados
 con la contabilidad. Consta de tres campos:
 enterprise_code: Cdigo de la empresa que se desea buscar.
 initial_date: Fecha en la que empieza el perodo de contabilidad.
 fecha_hasta: Fecha en la que acaba el perodo de contabilidad.
 greater_than: Indica si los provedores o clientes que se buscarn sean de ms de 500000 pts.
  */
typedef struct conditionAccountingPeriodStruct {
    gulong enterprise_code;
    gchar * initial_date;
    gchar * ending_date;
    gboolean greater_than;
} ConditionAccountingPeriod;


AccountingPeriod * m_accounting_period_create (GdaRecordset *rs);

gulong m_accounting_period_insert (AccountingPeriod * accounting_period);
gboolean m_accounting_period_close (gulong accounting_code,gulong enterprise_code,GdaConnection * connection,GdaCommand * command);
gboolean m_accounting_period_exists (AccountingPeriod * accounting_period);
void m_accounting_period_clear_array (AccountingPeriod ** accounting_periods);
gboolean m_accounting_period_is_closed (AccountingPeriod * accounting_period);
AccountingPeriod * m_accounting_period_find_by_condition (ConditionAccountingPeriod * condition);
gboolean m_accounting_get_date_in_closed (gulong enterprise_code,time_t date);

/*FUNCIN QUE DEVOLVE A DA SEGUINTE  LTIMO DA DE CERRE DE CONTABILIDADE EN CASO DE QUE A DATA DE EMISION
 DA FACTURA SEXA ANTERIOR, NOUTRO CASO DEVOLVE A MESMA DATA QUE SE PASOU COMO PARAMETRO*/
/*NOTA: O MTODO RESERVE MEMORIA PARA O RESULTADO*/
time_t * m_accounting_get_day_after_to_last_closed (gulong enterprise_code,time_t * invoice_issue_date);

#endif

