
/* agent_model.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "agent_model.h"
#include "participant_model.h"

#include "util.h"

/**
 m_agent_create:
 * @rs: Record set que contiene los campos de la BD.
 *
 * Crea un objeto a partir de los valores de un record set de la base de datos.
 *
 * Returns:
 * Un objeto #Agent con los campos rellenados con los datos de la base de datos.
 */
Agent * m_agent_create (GdaRecordset *rs) {

    Agent * agent;

    agent = d_agent_create_empty();

    agent->participant = m_participant_create (rs,"agent");

    return agent;
}


/**
 * m_agent_find_by_name:
 * @name: Nombre o parte del nombre del agente que se pretende buscar.
 * 
 * Bsqueda de agentes del grupo con la condicin de nombre, es decir, que existan partes del
 * nombre y apellidos del agente que coincidan con name.
 *
 * Returns: Lista de agentes que coincidan con el nombre. %NULL en caso de no coincidencia.
 */
Agent ** m_agent_find_by_name (gchar *name){

  Agent ** list_of_agents = NULL; 
  GdaCommand         * my_command;
  GdaConnection      * my_connection = m_model_get_connection ();
  GdaRecordset       * record_set = NULL;
  Agent * agent;
  gchar * gchar_query = NULL;
  gulong * tuples;
  gint * column;
  gint index;
  gint length_of_list;
  
  my_command = gda_command_new();
  gda_command_set_connection(my_command,my_connection);
  
  gchar_query = g_strconcat("SELECT ",
			    " Participante.codigoparty as codigopartyparticipantagent, ",
			    " Participante.cif as cifparticipantagent, ",			    
			    " Participante.nick as nickparticipantagent, ",
			    " Participante.nombrecorto as shortnameparticipantagent ",
			    " FROM participante, cliente",
			    " WHERE participante.codigoparty=cliente.codigoparty",
			    " AND cliente.agente = true",
			    NULL);
  
  if (name != NULL){
    gchar * gchar_query_aux = NULL;

    gchar_query_aux = g_strconcat(gchar_query," AND participante.nick ILIKE '",name,"%'",NULL);
    g_free(gchar_query);
    gchar_query =  gchar_query_aux;
    gchar_query_aux = NULL;
  }

  
  tuples = (gulong *) g_malloc0(sizeof(gulong));

  m_model_command_execute (&record_set, gchar_query, my_connection, my_command,tuples,0, TRUE);
  g_print(gchar_query);
  g_free(gchar_query);
  
  length_of_list = *tuples;
  column = (gint *) g_malloc0(sizeof(gint));
  
  if (record_set!=NULL) {
    if (*tuples!=0) {
      list_of_agents = (Agent **) g_malloc0(sizeof(Agent*)*(*tuples+1));
      index = gda_recordset_move_first(record_set);
      while (!gda_recordset_eof(record_set)) {
	if (index==GDA_RECORDSET_INVALID_POSITION) return NULL;
	(*column)=0;
	agent = m_agent_create (record_set);
	list_of_agents[index-1] = agent;
	index = gda_recordset_move_next(record_set);
      }
      list_of_agents[length_of_list]=NULL;
      
    } else {
      list_of_agents=NULL;
    }
    gda_recordset_close(record_set);
  } else {
    list_of_agents=NULL;
  }
  
  igalia_command_free(my_command);
  gda_recordset_free(record_set);

  
  g_free(tuples);
  g_free(column);
  

  return list_of_agents;
}

/**
 * m_agent_clear_array:
 * @list_of_agents: El array de agentes a liberar.
 *
 * Libera la memoria ocupada por el array de agentes.
 * 
 */

void m_agent_clear_array (Agent **list_of_agents) {
  
  if (list_of_agents!=NULL) {
	gint index = 0;
	while (list_of_agents[index]!=NULL) {
	  d_agent_clear (list_of_agents[index]);
	  index++;
	}
    }

    g_free(list_of_agents);
    
    list_of_agents = NULL;
    
}
