/* agent.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */


/* Agente no tiene tabla ya que solo implica un rol de un participante*/

#include "agent.h"

#include "participant.h"
#include "company.h"

static gint agent_init(Agent *agent);



/**
 * d_agent_create_empty:
 *
 * Crea un objeto vaco, con memoria reservada e inicializado.
 *
 * Returns:
 * Un objeto #Agent vaco con los campos inicializados a un valor de inicializacin segn el tipo.
 */
Agent * d_agent_create_empty () {

    Agent * agent;

    agent = (Agent *) g_malloc0(sizeof(Agent));
    agent_init(agent);

    return agent;
}




/**
 d_agent_clear:
 * @agent: Agente que se desea liberar.
 *
 * Libera la memoria utilizada por el objeto.
 *
 * Returns:
 * 0 si se ha acabado la funcin correctamente.
 */
gint d_agent_clear (Agent *agent) {
    if (agent!=NULL) {
	d_participant_clear (d_agent_get_participant (agent));
	g_free(agent);
    }
    return 0;
}

/**
 * d_agent_set_participant:
 * @agent: Objeto en el que se establece el atributo.
 * @participant: Atributo a insertar.
 *
 * Establece el atributo @participant en el objeto. El atributo @participant
 * indica el objeto participante del que consta el agente.
 *
 */
void d_agent_set_participant (Agent * agent,Participant * participant) {
    agent->participant = participant;
}


/**
 * d_agent_get_participant:
 * @agent: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
Participant * d_agent_get_participant (Agent * agent){
    return agent->participant;
}


/**
 * d_agent_duplicate:
 * @agent: Objeto del que se desea obtener el duplicado.
 *
 * Duplica el objeto #Agent.
 *
 * Returns: Un objeto con todos los campos duplicados y con nueva memoria reservada.
 */
Agent *d_agent_duplicate (Agent *agent){

  Agent *result = d_agent_create_empty ();

  d_agent_set_participant (result,d_participant_duplicate (d_agent_get_participant (agent)));
  
  return result;

}

/* Private functions */

static gint agent_init(Agent *agent) {



    agent->participant = NULL;

    return 0;
}

