/* cash.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "cash.h"
#include "stdlib.h"

static gint cash_init(Cash *cash);



/**
 d_cash_create_empty:
 *
 * Crea un objeto vaco, con memoria reservada e inicializado.
 *
 * Returns:
 * Un objeto #Cash vaco con los campos inicializados a un valor de inicializacin segn el tipo.
 */
Cash *d_cash_create_empty () {
  Cash *cash = (Cash *) g_malloc (sizeof(Cash));

  cash_init(cash);
  
  return cash;
}




/**
 * d_cash_clear:
 * @cash: Objeto caja a liberar.
 *
 * Libera la memoria utilizada por el objeto.
 *
 * Returns:
 * 0 si se ha acabado la funcin correctamente.
 */
gint d_cash_clear_cash (Cash *cash) {

    if (cash!=NULL) {
        d_enterprise_clear (cash->enterprise);
        d_shop_clear (cash->shop);
	g_free(cash);
    }

  return 0;
}

/**
 * d_cash_set_cash_code:
 * @cash: Objeto en el que se establece el atributo.
 * @cash_code: Atributo a insertar.
 *
 * Establece el atributo @cash_code en el objeto. El atributo @cash_code
 * indica el cdigo interno que identifica la caja.
 *
 */
void d_cash_set_cash_code (Cash *cash, gulong cash_code){
    cash->cash_code = cash_code;
}
/**
 * d_cash_set_initial_balance:
 * @cash: Objeto en el que se establece el atributo.
 * @initial_balance: Atributo a insertar.
 *
 * Establece el atributo @initial_balance en el objeto. El atributo @initial_balance
 * indica el saldo inicial de la caja.
 *
 */
void d_cash_set_initial_balance (Cash *cash, gdouble initial_balance){
    cash->initial_balance = initial_balance;
}

/**
 * d_cash_set_currente_balance:
 * @cash: Objeto en el que se establece el atributo.
 * @current_balance: Atributo a insertar.
 *
 * Establece el atributo @current_balance en el objeto. El atributo @current_balance
 * indica el saldo actual de la caja.
 *
 */
void d_cash_set_current_balance (Cash *cash, gdouble current_balance){
    cash->current_balance = current_balance;
}

/**
 * d_cash_set_shop:
 * @cash: Objeto en el que se establece el atributo.
 * @shop: Atributo a insertar.
 *
 * Establece el atributo @shop en el objeto. El atributo @shop
 * indica la tienda que posee la caja.
 *
 */
void d_cash_set_shop (Cash *cash,Shop * shop){
    if (cash->shop!=NULL) {
        d_shop_clear (cash->shop);
    }
    cash->shop = shop;
}

/**
 * d_cash_set_enterprise:
 * @cash: Objeto en el que se establece el atributo.
 * @enterprise: Atributo a insertar.
 *
 * Establece el atributo @ en el objeto. El atributo @
 * indica la empresa a la que pertenece la tienda que tiene la caja.
 *
 */
void d_cash_set_enterprise (Cash *cash,Enterprise * enterprise){
    if (cash->enterprise!=NULL) {
        d_enterprise_clear (cash->enterprise);
    }
    cash->enterprise = enterprise;

}
/**
 * d_cash_get_cash_code:
 * @cash: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong  d_cash_get_cash_code (Cash *cash){
    return cash->cash_code;
}
/**
 * d_cash_get_shop:
 * @cash: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
Shop * d_cash_get_shop (Cash *cash){
    return cash->shop;
}

/**
 * d_cash_get_enterprise:
 * @cash: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
Enterprise * d_cash_get_enterprise (Cash *cash){
    return cash->enterprise;
}

/**
 * d_cash_get_initial_balance:
 * @cash: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gdouble d_cash_get_initial_balance (Cash *cash){
    return cash->initial_balance;
}

/**
 * d_cash_get_current_balance:
 * @cash: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gdouble d_cash_get_current_balance (Cash *cash){
    return cash->current_balance;
}


/* Private functions*/

static gint cash_init(Cash *cash) {

    cash->shop = NULL ;
    cash->enterprise = NULL ;
    cash->cash_code = ULONG_MAX;
    cash->initial_balance = -1;
    cash->current_balance = -1;
    return 0;
}
