/* charge_invoice.h
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */


#ifndef _CHARGE_INVOICE_
#define _CHARGE_INVOICE_

#include "config.h"

#ifndef _GLIB_LIBRARY_
#define _GLIB_LIBRARY_
#include <glib.h>
#endif


/**
 * D_CHARGE_INVOICE_PAID:
 *
 * Cdigo que indica uno de los estados de una factura de pago. Este estado indica que la factura est pagada.
 *
 */
#define D_CHARGE_INVOICE_PAID 1
/**
 * D_CHARGE_INVOICE_NOT_PAID:
 *
 * Cdigo que indica uno de los estados de una factura de pago. Este estado indica que la factura no est pagada.
 *
 */
#define D_CHARGE_INVOICE_NOT_PAID 0


/**
 * D_CHARGE_INVOICE_MESSAGE_NOT_PAID:
 *
 * Texto que se utiliza para mostrar que una factura no est pagada.
 *
 */
#define D_CHARGE_INVOICE_MESSAGE_NOT_PAID _("Not paid")
/**
 * D_CHARGE_INVOICE_MESSAGE_PAID:
 *
 * Texto que se utiliza para mostrar que una factura est pagada.
 *
 */
#define D_CHARGE_INVOICE_MESSAGE_PAID _("Paid")

/**
 * D_CHARGE_INVOICE_CODE:
 *
 * Cdigo que indica el tipo de factura que se est tratando, en este caso las facturas de pago.
 *
 */
#define D_CHARGE_INVOICE_CODE 4

/**
 * D_CHARGE_INVOICE_MESSAGE:
 *
 * Texto que se utiliza para mostrar que una factura es de pago.
 *
 */
#define D_CHARGE_INVOICE_MESSAGE _("Charge Invoice")

/**
 * D_CHARGE_INVOICE_NOTE_STATE:
 *
 * Cdigo que identifica en memoria el estado de las notas de pago.
 *
 */
#define D_CHARGE_INVOICE_NOTE_STATE -2

/**
 * D_CHARGE_INVOICE_NOTE_CODE:
 *
 * Texto que identifica el tipo de factura que se est tratando, en este caso una nota de pago.
 *
 */
#define D_CHARGE_INVOICE_NOTE_CODE 3
/**
 * D_CHARGE_INVOICE_NOTE_MESSAGE:
 *
 * Texto que se utiliza para mostrar que una factura es una nota de pago.
 *
 */
#define D_CHARGE_INVOICE_NOTE_MESSAGE _("Charge Invoice Note")


#include "time.h"
#include "charge_invoice_line.h"
#include "enterprise.h"
#include "shop.h"
#include "supplier.h"
#include "payment.h"

/* THE BOOLEAN IVA IS USED TO KNOW IF THAT OBJECT IS AN INVOICE OR A NOTE
 IF WE ARE TALKING ABOUT A NOTE chargeDeliveryPayments MUST BE NULL
 */
typedef struct chargeInvoiceStruct {
    gchar * invoice_number;
    gulong payment_note_number;
    time_t date;
    Enterprise * enterprise;
    Supplier * supplier;
    Shop * shop;
    gchar * section;
    gdouble total;
    Payment * payment;
    gint state;
    gboolean vat;
    ChargeInvoiceLine ** charge_invoice_lines;
    gulong code;
} ChargeInvoice;


ChargeInvoice * d_charge_invoice_create_empty (void);
void d_charge_invoice_set_invoice_number (ChargeInvoice * charge_invoice,gchar * invoice_number);
void d_charge_invoice_set_date (ChargeInvoice * charge_invoice,time_t date);
void d_charge_invoice_set_section (ChargeInvoice * charge_invoice,gchar * section);
void d_charge_invoice_set_shop (ChargeInvoice * charge_invoice,Shop * shop);
void d_charge_invoice_set_supplier (ChargeInvoice * charge_invoice,Supplier * supplier);
void d_charge_invoice_set_payment (ChargeInvoice * charge_invoice,Payment * payment);
void d_charge_invoice_set_enterprise (ChargeInvoice * charge_invoice,Enterprise * enterprise);
void d_charge_invoice_set_charge_invoice_lines (ChargeInvoice * charge_invoice,ChargeInvoiceLine ** cils);
void d_charge_invoice_set_state (ChargeInvoice * charge_invoice,gint state);
void d_charge_invoice_set_code (ChargeInvoice * charge_invoice,gulong code);
void d_charge_invoice_set_vat (ChargeInvoice * charge_invoice,gboolean vat);
void d_charge_invoice_set_total (ChargeInvoice * charge_invoice,gdouble total);
gchar * d_charge_invoice_get_invoice_number (ChargeInvoice * charge_invoice);
time_t d_charge_invoice_get_date (ChargeInvoice * charge_invoice);
gchar * d_charge_invoice_get_section (ChargeInvoice * charge_invoice);
Shop * d_charge_invoice_get_shop (ChargeInvoice * charge_invoice);
Supplier * d_charge_invoice_get_supplier (ChargeInvoice * charge_invoice);
Payment * d_charge_invoice_get_payment (ChargeInvoice * charge_invoice);
Enterprise * d_charge_invoice_get_enterprise (ChargeInvoice * charge_invoice);
ChargeInvoiceLine ** d_charge_invoice_get_charge_invoice_lines (ChargeInvoice * charge_invoice);
gdouble d_charge_invoice_get_total (ChargeInvoice * charge_invoice);
gint d_charge_invoice_get_state (ChargeInvoice * charge_invoice);
gulong d_charge_invoice_get_code (ChargeInvoice * charge_invoice);
gboolean d_charge_invoice_get_vat (ChargeInvoice * charge_invoice);

gint d_charge_invoice_clear (ChargeInvoice *charge_invoice);

gchar ***d_charge_d_invoice_array_to_content (ChargeInvoice **array, gulong *reccount);
gchar  **d_charge_invoice_get_content (ChargeInvoice *invoice,gdouble * saldoAct);
gchar  **d_charge_invoice_get_content_title (void);
gchar   *d_charge_invoice_get_listing_title (gchar * supplier_name);
gint    *d_charge_invoice_get_listing_columns_width (void);
gint    *d_charge_invoice_get_listing_columns_alignment (void);
gint     d_charge_invoice_get_content_size (void);


#endif

