
/* charge_invoice_line.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include "charge_invoice_line.h"
#include "model.h"

static gint charge_invoice_line_init(ChargeInvoiceLine *charge_invoice_line);


/**
 * d_charge_invoice_line_create_empty:
 *
 * Crea un objeto vaco, con memoria reservada e inicializado.
 *
 * Returns: Un objeto #ChargeInvoiceLine vaco con los campos inicializados a un valor de inicializacin segn el tipo.
 */
ChargeInvoiceLine  *d_charge_invoice_line_create_empty (){
    ChargeInvoiceLine *charge_invoice_line = (ChargeInvoiceLine *) g_malloc0(sizeof(ChargeInvoiceLine));

    charge_invoice_line_init(charge_invoice_line);

    return charge_invoice_line;
}


/**
 * d_charge_invoice_line_clear:
 * @charge_invoice_line: Objeto a liberar.
 *
 * Libera la memoria utilizada por el objeto.
 *
 * Returns:
 * 0 si se ha acabado la funcin correctamente.
 */
gint d_charge_invoice_line_clear (ChargeInvoiceLine *charge_invoice_line) {

    if (charge_invoice_line!=NULL) {
	if (charge_invoice_line->description!=NULL)
            g_free(charge_invoice_line->description);

	d_order_line_clear (charge_invoice_line->order_line);
	
	g_free(charge_invoice_line);
	charge_invoice_line=NULL;
    }

    return 0;
}

/**
 * d_charge_invoice_line_set_supplier_code:
 * @charge_invoice_line: Objeto en el que se establece el atributo.
 * @supplier_code: Atributo a insertar.
 *
 * Establece el atributo @supplier_code en el objeto. El atributo @supplier_code
 * indica el cdigo del provedor que emiti la factura.
 *
 */
void d_charge_invoice_line_set_supplier_code (ChargeInvoiceLine * charge_invoice_line, gulong supplier_code) {
    charge_invoice_line->supplier_code = supplier_code;
}

/**
 * d_charge_invoice_line_set_invoice_number:
 * @charge_invoice_line: Objeto en el que se establece el atributo.
 * @invoice_number: Atributo a insertar.
 *
 * Establece el atributo @invoice_number en el objeto. El atributo @invoice_number
 * indica el nmero de factura.
 *
 */
void d_charge_invoice_line_set_invoice_number (ChargeInvoiceLine * charge_invoice_line, gulong invoice_number) {
    charge_invoice_line->invoice_number = (invoice_number);
}


/**
 * d_charge_invoice_line_set_line_number:
 * @charge_invoice_line: Objeto en el que se establece el atributo.
 * @line_number: Atributo a insertar.
 *
 * Establece el atributo @line_number en el objeto. El atributo @line_number
 * indica el nmero de lnea dentro de la factura de cargo.
 *
 */
void d_charge_invoice_line_set_line_number (ChargeInvoiceLine * charge_invoice_line, gulong line_number) {
    charge_invoice_line->line_number = line_number;
}

/**
 * d_charge_invoice_line_set_description:
 * @charge_invoice_line: Objeto en el que se establece el atributo.
 * @description: Atributo a insertar.
 *
 * Establece el atributo @description en el objeto. El atributo @description
 * indica el concepto del la lnea de la factura.
 *
 */
void d_charge_invoice_line_set_description (ChargeInvoiceLine * charge_invoice_line, gchar * description) {
    charge_invoice_line->description = g_strdup(description);
}

/**
 * d_charge_invoice_line_set_vat_type:
 * @charge_invoice_line: Objeto en el que se establece el atributo.
 * @vat_type: Atributo a insertar.
 *
 * Establece el atributo @vat_type en el objeto. El atributo @vat_type
 * indica el tipo de iva que corresponde a la lnea de la factura.
 *
 */
void d_charge_invoice_line_set_vat_type (ChargeInvoiceLine * charge_invoice_line, gdouble vat_type) {
    charge_invoice_line-> vat_type= vat_type;
}

/**
 * d_charge_invoice_line_set_total:
 * @charge_invoice_line: Objeto en el que se establece el atributo.
 * @total: Atributo a insertar.
 *
 * Establece el atributo @total en el objeto. El atributo @total
 * indica el total de la lnea de la factura.
 *
 */
void d_charge_invoice_line_set_total (ChargeInvoiceLine * charge_invoice_line, gdouble total) {
    charge_invoice_line->total = total;
}

/**
 * d_charge_invoice_line_set_order_line:
 * @charge_invoice_line: Objeto en el que se establece el atributo.
 * @order_line: Atributo a insertar.
 *
 * Establece el atributo @order_line en el objeto. El atributo @order_line
 * indica la lnea de pedido a la que hace referencia la lnea de la factura.
 *
 */
void d_charge_invoice_line_set_order_line (ChargeInvoiceLine * charge_invoice_line, OrderLine *order_line) {
    charge_invoice_line->order_line = order_line;
}

/**
 * d_charge_invoice_line_get_supplier_code:
 * @charge_invoice_line: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_charge_invoice_line_get_supplier_code (ChargeInvoiceLine * charge_invoice_line) {
    return(charge_invoice_line->supplier_code);
}

/**
 * d_charge_invoice_line_get_invoice_number:
 * @charge_invoice_line: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_charge_invoice_line_get_invoice_number (ChargeInvoiceLine * charge_invoice_line) {
    return (charge_invoice_line->invoice_number);
}


/**
 * d_charge_invoice_line_get_line_number:
 * @charge_invoice_line: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_charge_invoice_line_get_line_number (ChargeInvoiceLine * charge_invoice_line) {
    return(charge_invoice_line->line_number);
}


/**
 * d_charge_invoice_line_get_description:
 * @charge_invoice_line: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar * d_charge_invoice_line_get_description (ChargeInvoiceLine * charge_invoice_line) {
    return g_strdup(charge_invoice_line->description);
}

/**
 * d_charge_invoice_line_get_vat_type:
 * @charge_invoice_line: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gdouble d_charge_invoice_line_get_vat_type (ChargeInvoiceLine * charge_invoice_line) {
    return charge_invoice_line->vat_type;
}

/**
 * d_charge_invoice_line_get_total:
 * @charge_invoice_line: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gdouble d_charge_invoice_line_get_total (ChargeInvoiceLine * charge_invoice_line) {
    return charge_invoice_line->total;
}

/**
 * d_charge_invoice_line_get_order_line:
 * @charge_invoice_line: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
OrderLine *d_charge_invoice_line_get_order_line (ChargeInvoiceLine * charge_invoice_line) {
    return charge_invoice_line->order_line;
}

/**
 * d_charge_invoice_line_duplicate:
 * @lines: Lista de lneas de factura de cargo que se quieren duplicar.
 *
 * Duplica el array con las lneas de factura.
 *
 * Returns: Un lista de objetos #ChargeInvoiceLine con todos los campos duplicados y con nueva memoria reservada.
 */
ChargeInvoiceLine **d_charge_invoice_line_duplicate (ChargeInvoiceLine **lines) {
    ChargeInvoiceLine **result = NULL;
    gint size = 0;
    gint index = 0;
    gchar *temp=NULL;

    while(lines[size]!=NULL){
	size++;
    }

    result = (ChargeInvoiceLine **)g_malloc0((size+1)*sizeof(ChargeInvoiceLine *));

    while(lines[index]!= NULL){
        OrderLine **par_aux = (OrderLine **)g_malloc(2*sizeof(OrderLine *));
	OrderLine **res_aux = NULL;

	result[index] = d_charge_invoice_line_create_empty ();

	par_aux[0] = d_charge_invoice_line_get_order_line (lines[index]);
	par_aux[1] = NULL;
	
	res_aux = d_order_line_duplicate (par_aux);

	d_charge_invoice_line_set_order_line (result[index], res_aux[0]);
	
	g_free(par_aux);
	g_free(res_aux);
	d_charge_invoice_line_set_supplier_code (result[index],
				    d_charge_invoice_line_get_supplier_code (lines[index]));


	d_charge_invoice_line_set_invoice_number (result[index],
	       			              d_charge_invoice_line_get_invoice_number (lines[index]));


	d_charge_invoice_line_set_line_number (result[index],
				   d_charge_invoice_line_get_line_number (lines[index]));

	d_charge_invoice_line_set_total (result[index],
				      d_charge_invoice_line_get_total (lines[index]));

        temp = d_charge_invoice_line_get_description (lines[index]);
	d_charge_invoice_line_set_description (result[index],
					   temp);
        g_free(temp);

	d_charge_invoice_line_set_vat_type (result[index],
				d_charge_invoice_line_get_vat_type (lines[index]));

	index++;
    }

    result[index] = NULL;
    return result;
}


gdouble d_charge_invoice_line_calculate_total_by_vat (ChargeInvoiceLine ** charge_invoice_lines) {
    GList * data=NULL;
    GList * list_of_vat=NULL;
    gdouble sub_total=0;
    if (charge_invoice_lines!=NULL) {
        gulong index=0;
	while (charge_invoice_lines[index]!=NULL) {
	    gchar * vat_type_char;
	    gchar * sum_vat_type_char;
	    gdouble total=0;
	    gdouble sub_total = 0;
            gdouble vat_tax = 0;
	    GroupedTipoIva * group ;
	    gdouble vat_temp= d_charge_invoice_line_get_vat_type (charge_invoice_lines[index]);

            ChargeInvoiceLine * charge_invoice_line = charge_invoice_lines[index];



	    sub_total = d_charge_invoice_line_get_total (charge_invoice_line);
	    total = sub_total * (1+((vat_temp*1.0)/100.0));

            vat_tax = sub_total * ((vat_temp*1.0)/100.0);

            vat_type_char = g_strdup_printf("%.2f",d_charge_invoice_line_get_vat_type (charge_invoice_line));
	    if (index==0) {
                group = (GroupedTipoIva *)g_malloc0(sizeof(GroupedTipoIva));
		group->vat_type=d_charge_invoice_line_get_vat_type (charge_invoice_line);
                group->subtotal_by_vat=sub_total;
		group->vat_share=vat_tax;
		data = g_list_append(data,group);
	    } else {
		GList * list = g_list_first(data);
		while (list!=NULL) {
                    group = (GroupedTipoIva *) (list -> data);
		    sum_vat_type_char = g_strdup_printf("%.2f",group->vat_type);
		    if (!strcmp(sum_vat_type_char,vat_type_char)) {
                        break;
		    }
                    list = g_list_next(list);
		}
		if (list!=NULL) {
		    group->subtotal_by_vat=group->subtotal_by_vat+sub_total;
		    group->vat_share=group->vat_share+vat_tax;
		} else {
		    group = (GroupedTipoIva *)g_malloc0(sizeof(GroupedTipoIva));
		    group->vat_type=d_charge_invoice_line_get_vat_type (charge_invoice_line);
		    group->subtotal_by_vat=group->subtotal_by_vat+sub_total;
		    group->vat_share=group->vat_share+vat_tax;
		    data = g_list_append(data,group);

		}

	    }

            index++;
	}
    }

    list_of_vat = g_list_first(data);
    while (list_of_vat!=NULL) {
	GroupedTipoIva * group = (GroupedTipoIva *) (list_of_vat -> data);
	g_print(_("\nVAT type: %.2f"),group->vat_type);
	g_print(_("\nTaxable income: %.2f"),group->subtotal_by_vat);
	g_print(_("\nTotal VAT: %.2f"),group->vat_share);
	list_of_vat = g_list_next(list_of_vat);

        sub_total = sub_total + group->subtotal_by_vat + group->vat_share;
    }

    //  return charge_invoice_lines_iva;
    return sub_total;
}



/* Private functions */

static gint charge_invoice_line_init(ChargeInvoiceLine *charge_invoice_line) {
    charge_invoice_line->supplier_code  = ULONG_MAX;
    charge_invoice_line->invoice_number = ULONG_MAX;
    charge_invoice_line->line_number   = ULONG_MAX;
    charge_invoice_line->description      = NULL;
    charge_invoice_line->total  = G_MINDOUBLE;
    charge_invoice_line->vat_type     = -1.0;
    charge_invoice_line->order_line = NULL;
    return 0;
}


