
/* consortium.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "consortium.h"
#include "participant.h"
static gint consortium_init(Consortium *consortium);

/**
 * d_consortium_create_empty:
 *
 * Crea un objeto vaco, con memoria reservada e inicializado.
 *
 * Returns:
 * Un objeto #Consortium vaco con los campos inicializados a un valor de inicializacin segn el tipo.
 */
Consortium * d_consortium_create_empty () {

    Consortium * consortium;

    consortium = (Consortium *) g_malloc0(sizeof(Consortium));
    consortium_init(consortium);

    return consortium;
}


/**
 * d_consortium_clear:
 * @consortium: Objeto a liberar.
 *
 * Libera la memoria utilizada por el objeto.
 *
 * Returns:
 * 0 si se ha acabado la funcin correctamente.
 */
gint d_consortium_clear (Consortium *consortium) {
    if (consortium!=NULL) {
	d_participant_clear (d_consortium_get_participant (consortium));
	g_free(consortium);
    }
    return 0;
}

/**
 * d_consortium_set_participant:
 * @consortium: Objeto en el que se establece el atributo.
 * @participant: Atributo a insertar.
 *
 * Establece el atributo @participant en el objeto. El atributo @participant
 * indica el participante asociado al consorcio.
 *
 */
void d_consortium_set_participant (Consortium * consortium,Participant * participant) {
    consortium->participant = participant;
}

/**
 * d_consortium_set_discount:
 * @consortium: Objeto en el que se establece el atributo.
 * @discount: Atributo a insertar.
 *
 * Establece el atributo @discount en el objeto. El atributo @discount
 * indica el descuento que se aplicara a una reparacin por el mero hecho de
 * pertenecer a un consorcio. Este descuento se aplica se aplica siempre en
 * detrimento del descuento de compaa, excepto en el caso de Winthertur.
 *
 */
void d_consortium_set_discount (Consortium * consortium, gdouble discount) {
    if (discount>100)
	consortium->discount=100;
    else if (discount<0)
	consortium->discount=0;
    else
	consortium->discount=discount;
}

/**
 * d_consortium_get_participant:
 * @consortium: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
Participant * d_consortium_get_participant (Consortium * consortium){
    return consortium->participant;
}

/**
 * d_consortium_get_discount:
 * @consortium: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gdouble d_consortium_get_discount (Consortium * consortium) {
    return consortium->discount;
}


/* Private functions */

static gint consortium_init(Consortium *consortium) {



    consortium->participant = NULL;
    consortium->discount = -1;
    
    return 0;
}

