/*  direct_debit.h
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#ifndef _DIRECT_DEBIT_
#define _DIRECT_DEBIT_

#include "config.h"
#include "office.h"

#ifndef _GLIB_LIBRARY_
#define _GLIB_LIBRARY_
#include <glib.h>
#endif

/**
 * D_DIRECT_DEBIT_GENERATED:
 *
 * Indica el cdigo del estado de una domiciliacin. En este caso indica que una domicialiacin est generada.
 *
 */
#define D_DIRECT_DEBIT_GENERATED 9
/**
 * D_DIRECT_DEBIT_PAID:
 *
 * 
 * Indica el cdigo del estado de una domiciliacin. En este caso indica que una domicialiacin est pagada.
 *
 */
#define D_DIRECT_DEBIT_PAID 10
/**
 * D_DIRECT_DEBIT_TOTAL_ZERO:
 *
 * Indica el cdigo de la causa por la que se produjo la devolucin. En este caso el importe de la domiciliacin era 0.
 *
 */
#define D_DIRECT_DEBIT_TOTAL_ZERO 0
/**
 * D_DIRECT_DEBIT_NOT_UP_TO_DATE:
 *
 * Indica el cdigo de la causa por la que se produjo la devolucin. En este caso error de falta de dinero en la cuenta.
 *
 */
#define D_DIRECT_DEBIT_NOT_UP_TO_DATE 1
/**
 * D_DIRECT_DEBIT_NO_DIRECT_DEBIT:
 *
 * Indica el cdigo de la causa por la que se produjo la devolucin. En este caso la cuenta no acepta domiciliaciones.
 *
 */
#define D_DIRECT_DEBIT_NO_DIRECT_DEBIT 2
/**
 * D_DIRECT_DEBIT_NO_OFFICE:
 *
 * Indica el cdigo de la causa por la que se produjo la devolucin. En este caso la oficina es incorrecta.
 *
 */
#define D_DIRECT_DEBIT_NO_OFFICE 3
/**
 * D_DIRECT_DEBIT_LEGAL_ID:
 *
 * Indica el cdigo de la causa por la que se produjo la devolucin. En este caso el nif es incorrecto.
 *
 */
#define D_DIRECT_DEBIT_LEGAL_ID 4
/**
 * D_DIRECT_DEBIT_ERROR_OR_CANCEL:
 *
 * Indica el cdigo de la causa por la que se produjo la devolucin. En este caso se produjo una baja de la cuenta o un error general.
 *
 */
#define D_DIRECT_DEBIT_ERROR_OR_CANCEL 5
/**
 * D_DIRECT_DEBIT_DISAGREEMENT:
 *
 * Indica el cdigo de la causa por la que se produjo la devolucin. En este caso indica rechazo del cliente.
 *
 */
#define D_DIRECT_DEBIT_DISAGREEMENT 6
/**
 * D_DIRECT_DEBIT_DUPLICATED:
 *
 * Indica el cdigo de la causa por la que se produjo la devolucin. En este caso indica duplicidad de la domiciliacin.
 *
 */
#define D_DIRECT_DEBIT_DUPLICATED 7
/**
 * D_DIRECT_DEBIT_NO_USED:
 *
 * Indica el cdigo de la causa por la que se produjo la devolucin. Este cdigo est sin utilizar.
 *
 */
#define D_DIRECT_DEBIT_NO_USED 8
/**
 * D_DIRECT_DEBIT_MESSAGE_GENERATED:
 *
 * Mensaje que se utiliza para mostrar por pantalla el estado en gestin de una domiciliacin.
 *
 */
#define D_DIRECT_DEBIT_MESSAGE_GENERATED _("Direct debit in management process")
/**
 * D_DIRECT_DEBIT_MESSAGE_PAID:
 *
 * Mensaje que se utiliza para mostrar por pantalla el estado pagada de una domiciliacin.
 *
 */
#define D_DIRECT_DEBIT_MESSAGE_PAID _("Direct debit paid") 
/**
 * D_DIRECT_DEBIT_MESSAGE_TOTAL_ZERO:
 *
 * Mensaje que se utiliza para mostrar por pantalla la causa de devolucin de una domiciliacin.
 *
 */
#define D_DIRECT_DEBIT_MESSAGE_TOTAL_ZERO _("Direct Debit ERROR: Ammount is Zero")

/**
 * D_DIRECT_DEBIT_MESSAGE_NOT_UP_TO_DATE:
 *
 * Mensaje que se utiliza para mostrar por pantalla la causa de devolucin de una domiciliacin.
 *
 */
#define D_DIRECT_DEBIT_MESSAGE_NOT_UP_TO_DATE _("Direct Debit ERROR: Not up to date")
/**
 * D_DIRECT_DEBIT_MESSAGE_NO_DIRECT_DEBIT:
 *
 * Mensaje que se utiliza para mostrar por pantalla la causa de devolucin de una domiciliacin.
 *
 */
#define D_DIRECT_DEBIT_MESSAGE_NO_DIRECT_DEBIT _("Direct Debit ERROR: No direct debit, or account canceled")
/**
 * D_DIRECT_DEBIT_MESSAGE_NO_OFFICE:
 *
 * Mensaje que se utiliza para mostrar por pantalla la causa de devolucin de una domiciliacin.
 *
 */
#define D_DIRECT_DEBIT_MESSAGE_NO_OFFICE _("Direct Debit ERROR: Account office not exists")
/**
 * D_DIRECT_DEBIT_MESSAGE_LEGAL_ID:
 *
 * Mensaje que se utiliza para mostrar por pantalla la causa de devolucin de una domiciliacin.
 *
 */
#define D_DIRECT_DEBIT_MESSAGE_LEGAL_ID _("Direct Debit ERROR: R.D. 338/90 Application about Legal ID.")
/**
 * D_DIRECT_DEBIT_MESSAGE_ERROR_OR_CANCEL:
 *
 * Mensaje que se utiliza para mostrar por pantalla la causa de devolucin de una domiciliacin.
 *
 */
#define D_DIRECT_DEBIT_MESSAGE_ERROR_OR_CANCEL _("Direct Debit ERROR: Client order: error or direct debit cancelation")
/**
 * D_DIRECT_DEBIT_MESSAGE_DISAGREEMENT:
 *
 * Mensaje que se utiliza para mostrar por pantalla la causa de devolucin de una domiciliacin.
 *
 */
#define D_DIRECT_DEBIT_MESSAGE_DISAGREEMENT _("Error en domiciliacion: Order del cliente: disconformidad con el importe")
/**
 * D_DIRECT_DEBIT_MESSAGE_DUPLICATED:
 *
 * Mensaje que se utiliza para mostrar por pantalla la causa de devolucin de una domiciliacin.
 *
 */
#define D_DIRECT_DEBIT_MESSAGE_DUPLICATED _("Direct Debit ERROR: Duplicated, unjustified or missing data")
/**
 * D_DIRECT_DEBIT_MESSAGE_NO_USED:
 *
 * Mensaje que se utiliza para mostrar por pantalla la causa de devolucin de una domiciliacin.
 *
 */
#define D_DIRECT_DEBIT_MESSAGE_NO_USED _("Direct Debit ERROR: Not used")


typedef struct _direct_debit {
    gulong direct_debit_code;
    gulong enterprise_code;
    gulong shop_code;
    gulong invoice_code;
    gint state;
    gboolean is_hire_purchase;
    gint current_instalment;
} DirectDebit;


DirectDebit *d_direct_debit_create_empty (void); 
gint d_direct_debit_clear (DirectDebit *direct_debit);

void d_direct_debit_set_code (DirectDebit *direct_debit, gulong direct_debit_code);
void d_direct_debit_set_enterprise_code (DirectDebit *direct_debit, gulong enterprise_code);
void d_direct_debit_set_shop_code (DirectDebit *direct_debit, gulong shop_code);
void d_direct_debit_set_invoice_code (DirectDebit *direct_debit, gulong invoice_code);
void d_direct_debit_set_state (DirectDebit *direct_debit, gint state);
void d_direct_debit_set_is_hire_purchase (DirectDebit *direct_debit, gboolean is_hire_purchase);
void d_direct_debit_set_current_instalment (DirectDebit *direct_debit, gint current_instalment);
gulong d_direct_debit_get_code (DirectDebit *direct_debit);
gulong d_direct_debit_get_enterprise_code (DirectDebit *direct_debit);
gulong d_direct_debit_get_shop_code (DirectDebit *direct_debit);
gulong d_direct_debit_get_invoice_code (DirectDebit *direct_debit);
gint d_direct_debit_get_state (DirectDebit *direct_debit);
gboolean d_direct_debit_get_is_hire_purchase (DirectDebit *direct_debit);
gint d_direct_debit_get_current_instalment (DirectDebit *direct_debit);

void d_direct_debit_free_glist (GList * list);
gchar * d_direct_debit_get_state_message (gint state);

#endif


