
/* enterprise.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "enterprise.h"



static gint enterprise_init(Enterprise *enterprise);

/**
 * d_enterprise_create_empty:
 *
 * Crea un objeto vaco, con memoria reservada e inicializado.
 *
 * Returns: Un objeto #Enterprise vaco con los campos inicializados a un valor de inicializacin segn el tipo.
 */
Enterprise * d_enterprise_create_empty () {
    Enterprise * enterprise = (Enterprise *) g_malloc (sizeof(Enterprise));

    enterprise_init(enterprise);

    return enterprise;
}

/**
 * d_enterprise_clear:
 * @enterprise: Objeto a liberar.
 *
 * Libera la memoria utilizada por el objeto.
 *
 * Returns:
 * 0 si se ha acabado la funcin correctamente.
 */
gint d_enterprise_clear (Enterprise *enterprise){
    if (enterprise!=NULL) {
	d_participant_clear (enterprise->participant);

	g_free(enterprise);
    }

    return 0;
}

/**
 * d_enterprise_set_participant:
 * @enterprise: Objeto en el que se establece el atributo.
 * @participant: Atributo a insertar.
 *
 * Establece el atributo @participant en el objeto. El atributo @participant
 * indica la referencia a los datos de la empresa como participante.
 *
 */
void d_enterprise_set_participant (Enterprise *enterprise, Participant * participant){
  enterprise->participant = participant;
}

/**
 * d_enterprise_set_belong_to_principal:
 * @enterprise: Objeto en el que se establece el atributo.
 * @belong_to_principal: Atributo a insertar.
 *
 * Establece el atributo @belong_to_principal en el objeto. El atributo @belong_to_principal
 * indica si una empresa pertenece al grupo del empresas o no.
 *
 */
void d_enterprise_set_belong_to_principal (Enterprise *enterprise, gboolean belong_to_principal){
  enterprise->belong_to_principal = belong_to_principal;
}

/**
 * d_enterprise_get_participant:
 * @enterprise: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
Participant *d_enterprise_get_participant (Enterprise *enterprise){
  return enterprise->participant;
}

/**
 * d_enterprise_get_belong_to_principal:
 * @enterprise: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gboolean d_enterprise_get_belong_to_principal (Enterprise *enterprise){
  return enterprise->belong_to_principal;
}

/**
 * d_enterprise_duplicate:
 * @enterprise: Objeto del que se desea obtener el duplicado.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: Un objeto con todos los campos duplicados y con nueva memoria reservada.
 */
Enterprise * d_enterprise_duplicate (Enterprise * enterprise) {
    Enterprise * enter = d_enterprise_create_empty ();
    if (enterprise !=NULL) {
	d_enterprise_set_participant (enter,d_participant_duplicate (d_enterprise_get_participant (enterprise)));
	d_enterprise_set_belong_to_principal (enter,d_enterprise_get_belong_to_principal (enterprise));
    }
    return enter;
}

/* Private functions*/

static gint enterprise_init(Enterprise *enterprise) 
{
  enterprise->participant = NULL;
  enterprise->belong_to_principal = FALSE;
   
  return 0;
}

