 /* issued_book.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include "issued_book.h"
#include "participant.h"
#include "util.h"


#define D_ISSUED_BOOK_CONTENT_SIZE 7

static gint issued_book_init(IssuedBook *issued_book);




/**
 * d_issued_book_create_empty:
 *
 * Crea un objeto vaco, con memoria reservada e inicializado.
 *
 * Returns:
 * Un objeto #IssuedBook vaco con los campos inicializados a un valor de inicializacin segn el tipo.
 */
IssuedBook * d_issued_book_create_empty () {

    IssuedBook * issued_book;

    issued_book = (IssuedBook *) g_malloc0(sizeof(IssuedBook));

    issued_book_init(issued_book);

    return issued_book;
}




/**
 * d_issued_book_set_invoice_number:
 * @issued_book: Objeto en el que se establece el atributo.
 * @invoice_number: Atributo a insertar.
 *
 * Establece el atributo @invoice_number en el objeto. El atributo @invoice_number
 * indica el nmero de la factura a la que se refiere el objeto perteneciente al libro
 * de facturas emitidas.
 *
 */
void d_issued_book_set_invoice_number (IssuedBook * issued_book, gchar * invoice_number) {
    if (issued_book->invoice_number!= NULL) {
        g_free(issued_book->invoice_number);
    }
    issued_book->invoice_number = g_strdup(invoice_number);
}

/**
 * d_issued_book_set_issue_date:
 * @issued_book: Objeto en el que se establece el atributo.
 * @invoice_issue_date: Atributo a insertar.
 *
 * Establece el atributo @invoice_issue_date en el objeto. El atributo @invoice_issue_date
 * indica la fecha de emisin de la factura.
 *
 */
void d_issued_book_set_issue_date (IssuedBook * issued_book, time_t invoice_issue_date) {
    issued_book->invoice_issue_date = invoice_issue_date;
}

/**
 * d_issued_book_set_trade_name:
 * @issued_book: Objeto en el que se establece el atributo.
 * @trade_name: Atributo a insertar.
 *
 * Establece el atributo @trade_name en el objeto. El atributo @trade_name
 * indica la razn social a quien se emiti la factura.
 *
 */
void d_issued_book_set_trade_name (IssuedBook * issued_book, gchar * trade_name) {
    if (issued_book->trade_name!= NULL) {
        g_free(issued_book->trade_name);
    }
    g_strdup(issued_book->trade_name= trade_name);
}

/**
 * d_issued_book_set_vat_type:
 * @issued_book: Objeto en el que se establece el atributo.
 * @vat_type: Atributo a insertar.
 *
 * Establece el atributo @vat_type en el objeto. El atributo @vat_type
 * indica el tipo de iva de las lneas que se agruparon con el mismo el mismo tipo iva.
 *
 */
void d_issued_book_set_vat_type (IssuedBook * issued_book, gdouble vat_type) {
    issued_book->vat_type = vat_type;
}
/**
 * d_issued_set_book_total:
 * @issued_book: Objeto en el que se establece el atributo.
 * @total: Atributo a insertar.
 *
 * Establece el atributo @total en el objeto. El atributo @total
 * indica el total de la agrupacin de las lneas por iva.
 *
 */
void d_issued_set_book_total (IssuedBook * issued_book, gdouble total) {
    issued_book->total = total;
}

/**
 * d_issued_book_set_group_code:
 * @issued_book: Objeto en el que se establece el atributo.
 * @group_code: Atributo a insertar.
 *
 * Establece el atributo @group_code en el objeto. El atributo @group_code
 * indica la letra que identifica las facturas de cada tienda.
 *
 */
void d_issued_book_set_group_code (IssuedBook * issued_book, gchar * group_code) {
    if (issued_book->group_code!=NULL)
	g_free(issued_book->group_code);
    issued_book->group_code = g_strdup(group_code);
}

/**
 * d_issued_book_set_agreed_module:
 * @issued_book: Objeto en el que se establece el atributo.
 * @agreed_module: Atributo a insertar.
 *
 * Establece el atributo @agreed_module en el objeto. El atributo @agreed_module
 * indica si el total de esta agrupacin proviene de un mdulo pactado con lo
 * cul contiene todas las lneas de una factura.
 *
 */
void d_issued_book_set_agreed_module (IssuedBook * issued_book, gboolean agreed_module) {
    issued_book->agreed_module=agreed_module;
}

/**
 * d_issued_book_set_total_summary:
 * @issued_book: Objeto en el que se establece el atributo.
 * @total_summary: Atributo a insertar.
 *
 * Establece el atributo @total_summary en el objeto. El atributo @total_summary
 * indica si este objeto se corresponde con el resumen de totales.
 *
 */
void d_issued_book_set_total_summary (IssuedBook * issued_book, gboolean total_summary) {
    issued_book->total_summary=total_summary;
}

/**
 * d_issued_book_set_vat_summary:
 * @issued_book: Objeto en el que se establece el atributo.
 * @vat_summary: Atributo a insertar.
 *
 * Establece el atributo @vat_summary en el objeto. El atributo @vat_summary
 * indica si el objeto se corresponde con la agrupacin de todas las facturas por tipo de
 * Iva.
 *
 */
void d_issued_book_set_vat_summary (IssuedBook * issued_book, gboolean vat_summary) {
    issued_book->vat_summary=vat_summary;

}

/**
 * d_issued_book_set_line_subtotal:
 * @issued_book: Objeto en el que se establece el atributo.
 * @line_subtotal: Atributo a insertar.
 *
 * Establece el atributo @line_subtotal en el objeto. El atributo @line_subtotal
 * indica la suma de todas las lneas de la misma factura con el mismo tipo de iva que
 * van agrupadas en este objeto.
 *
 */
void d_issued_book_set_line_subtotal (IssuedBook * issued_book, gdouble line_subtotal) {
    issued_book->line_subtotal=line_subtotal;
}

/**
 * d_issued_book_set_invoice_discount:
 * @issued_book: Objeto en el que se establece el atributo.
 * @tipo_iva: Atributo a insertar.
 *
 * Establece el atributo @invoice_discount en el objeto. El atributo @tipoIva
 * indica el descuento de la factura correspondiente.
 *
 */
void d_issued_book_set_invoice_discount(IssuedBook * issued_book, gdouble discount) {
    issued_book->invoice_discount = discount;
}

/**
 * d_issued_book_get_invoice_number:
 * @issued_book: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar * d_issued_book_get_invoice_number (IssuedBook * issued_book) {
    return g_strdup(issued_book->invoice_number);
}

/**
 * d_issued_book_get_issue_date:
 * @issued_book: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
time_t d_issued_book_get_issue_date (IssuedBook * issued_book) {
    return (issued_book->invoice_issue_date);

}

/**
 * d_issued_book_get_trade_name:
 * @issued_book: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar * d_issued_book_get_trade_name (IssuedBook * issued_book) {
    return g_strdup(issued_book->trade_name);

}

/**
 * d_issued_book_get_vat_type:
 * @issued_book: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gdouble d_issued_book_get_vat_type (IssuedBook * issued_book) {
    return (issued_book->vat_type);

}

/**
 * d_issued_get_book_total:
 * @issued_book: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gdouble d_issued_get_book_total (IssuedBook * issued_book) {
    return (issued_book->total);

}

/**
 * d_issued_book_get_line_subtotal:
 * @issued_book: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gdouble d_issued_book_get_line_subtotal (IssuedBook * issued_book) {
    return issued_book->line_subtotal;
}

/**
 * d_issued_book_get_agreed_module:
 * @issued_book: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gboolean d_issued_book_get_agreed_module (IssuedBook * issued_book) {
    return issued_book->agreed_module;
}

/**
 * d_issued_book_get_total_summary:
 * @issued_book: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gboolean d_issued_book_get_total_summary (IssuedBook * issued_book) {
    return issued_book->total_summary;
}

/**
 * d_issued_book_get_vat_summary:
 * @issued_book: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gboolean d_issued_book_get_vat_summary (IssuedBook * issued_book) {
    return issued_book->vat_summary;
}

/**
 * d_issued_book_get_group_code:
 * @issued_book: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar * d_issued_book_get_group_code (IssuedBook * issued_book) {
    return g_strdup(issued_book->group_code);
}

/**
 * d_issued_book_get_invoice_discount:
 * @issued_book: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gdouble d_issued_book_get_invoice_discount(IssuedBook * issued_book) {
    return (issued_book->invoice_discount);

}

/**
 * d_issued_book_clear:
 * @issued_book: Objeto a liberar.
 *
 * Libera la memoria utilizada por el objeto.
 *
 * Returns:
 * 0 si se ha acabado la funcin correctamente.
 *
 * Returns: El valor del atributo solicitado.
 */
gint d_issued_book_clear (IssuedBook * issued_book) {

    if (issued_book!=NULL) {

	if (issued_book->invoice_number!=NULL) {
	    g_free(issued_book->invoice_number);
	}
	if (issued_book->trade_name!=NULL) {
	    g_free(issued_book->trade_name);
	}
	if (issued_book->group_code!=NULL) {
	    g_free(issued_book->group_code);
	}
	g_free(issued_book);
	issued_book = NULL;
    }

    return 0;
}


/**
 * d_issued_book_array_to_content:
 * @array: Array de agrupaciones por lneas de facturas de abono que se transformarn en listables.
 * @reccount: Puntero a un %gulong que indicar la suma total de los objetos transformados.
 *
 * Funcin utilizada para transformar un array de objetos en un array de arrays de cadenas
 * de caracteres listables. Es utilizada para realizar los listados impresos en papel.
 *
 * Returns:
 * Un triple puntero a %gchar, es decir, un puntero a un array de arrays de cadenas de caracteres. Se
 * podra ver como un puntero a una tabla con filas y colummnas de cadenas de caracteres.
 */
gchar ***d_issued_book_array_to_content (IssuedBook **array, gulong *reccount) {
    gint     array_size = 0;
    gchar ***content = NULL;

    *reccount = 0;
    while(array[array_size++]!=NULL);

    content = (gchar ***) malloc(sizeof(gchar **)*array_size);

    while(array[*reccount] != NULL) {
	content[*reccount] = d_issued_book_get_content (array[*reccount]);
        (*reccount)++;
    }

    content[*reccount] = NULL;

    return content;

}

/**
 * d_issued_book_get_content_title:
 *
 * Devuelve las cabaceras de cada pgina de los listados para el objeto libro de facturas emitidas. 
 *
 * Returns:
 * Un array de cadena de caracteres con las cabeceras de pgina. EL array acaba en NULL.
 */
gchar **d_issued_book_get_content_title () {
    gchar **content = (gchar **) g_malloc0 (sizeof(gchar *)*d_issued_book_get_content_size ()+1);
    gint index = 0;

    content[index++] = g_strdup(_("INVOICE No."));
    content[index++] = g_strdup(_("DATE"));
    content[index++] = g_strdup(_("TRADE NAME"));
    content[index++] = g_strdup(_("T.I."));
    content[index++] = g_strdup(_("VAT TYPE"));
    content[index++] = g_strdup(_("VAT"));
    content[index++] = g_strdup(_("TOTAL"));
    content[index++] = NULL;

    return content;
}

/**
 * d_issued_book_get_listing_title:
 *
 * Obtiene el ttulo del listado.
 *
 * Returns:
 * El ttulo del listado.
 */
gchar *d_issued_book_get_listing_title () {
    gchar *content = g_strdup_printf(_("ISSUED INVOICES REGISTER BOOK"));
    return content;
}

/**
 * d_issued_book_get_listing_columns_width:
 *
 * Indica los anchos de columna de los listados del libro de facturas emitidas.
 *
 * Returns:
 * Un array de enteros que indican el ancho de cada columna en el listado. EL array acaba en -1.
 */
gint *d_issued_book_get_listing_columns_width () {
    gint *widths = (gint *) g_malloc0 (sizeof(gint)*d_issued_book_get_content_size ()+1);
    gint index = 0;

    widths[index++] = 10;
    widths[index++] = 10;
    widths[index++] = 46;
    widths[index++] = 10;
    widths[index++] = 6;
    widths[index++] = 8;
    widths[index++] = 10;
    widths[index++] = -1;

    return widths;
}

/**
 * d_issued_book_get_listing_columns_alignment:
 *
 * Obtiene el alineamiento de las columnas del listado. Los alineamientos pueden ser U_LEFT, RIGHT o
 * CENTER.
 *
 * Returns:
 * Un array con los alineamientos de cada columna. EL array acaba en -1.
 */
gint *d_issued_book_get_listing_columns_alignment () {
    gint *orientation = (gint *) g_malloc0 (sizeof(gint)*d_issued_book_get_content_size ()+1);
    gint index = 0;

    orientation[index++] = U_LEFT;
    orientation[index++] = U_LEFT;
    orientation[index++] = U_LEFT;
    orientation[index++] = U_RIGHT;
    orientation[index++] = U_RIGHT;
    orientation[index++] = U_RIGHT;
    orientation[index++] = U_RIGHT;
    orientation[index++] = -1;

    return orientation;
}

/**
 * d_issued_book_get_content_size:
 *
 * Obtiene el nmero de columnas que tendr el listado.
 *
 * Returns:
 * El nmero de columnas.
 */
gint d_issued_book_get_content_size () {
    return (D_ISSUED_BOOK_CONTENT_SIZE);
}


/**
 * d_issued_book_get_content:
 * @issued_book: Objeto #IssuedBook a partir de la que se obtendrn los valores para el listado.
 *
 * Transforma un objeto #IssuedBook  en un array de cadenas de caracteres para aadir al listado
 * del facturas emitidas.
 *
 * Returns:
 * Array con los valores a sacar por listado en modo de cadenas de caracteres.
 */
gchar **d_issued_book_get_content (IssuedBook *issued_book) {
    gchar **content = (gchar **)g_malloc0(sizeof(gchar*) *
				       d_issued_book_get_content_size ()+1);
    gint    index   = 0;
    time_t fecha = d_issued_book_get_issue_date (issued_book);
    gchar  *temp = u_get_parsed_date (&fecha);
    gdouble subtotal;
    gdouble vat_share;
    gdouble total;
    gdouble temp_vat= d_issued_book_get_vat_type (issued_book);
    gdouble invoice_discount = d_issued_book_get_invoice_discount (issued_book);    
    gchar * temp_number=NULL;
    gchar * group_code=NULL;


    if (d_issued_book_get_agreed_module (issued_book)!=FALSE) {
	/*Si es un mdulo pactado se utiliza el total de la factura para calcular la base imponible*/
	total=d_issued_get_book_total (issued_book);
	subtotal = u_round_to_decimal (total / (1+((temp_vat*1.0)/100.0)),2);
    } else {
	/*Si no es un mdulo pactado se utiliza el subtotal de la suma de las lineas
	 de la factura para calcular la base imponible y a partir de ellas se calcula el total*/
	subtotal = d_issued_book_get_line_subtotal (issued_book);
	subtotal = subtotal - (subtotal*invoice_discount/100.0);	
	total = u_round_to_decimal (subtotal * (1+((temp_vat*1.0)/100.0)),2);
    }
    /* Se calcula el total de iva segn la base imponible*/
    vat_share = u_round_to_decimal (((temp_vat*1.0)/100.0) * subtotal,2);

    u_delete_intros (temp);

    /*Segn el objeto contenga resumenes de totales o resumenes por iva sacar en el listado unos datos u
      otros*/
	if (d_issued_book_get_total_summary (issued_book)==FALSE) {
	    if (d_issued_book_get_vat_summary (issued_book)==FALSE) {
		temp_number =d_issued_book_get_invoice_number (issued_book);
		group_code = d_issued_book_get_group_code (issued_book);

// It's no needed it shoul be at database.
//		if (group_code == NULL)
//		    group_code = g_strdup("GE");
		content[index++] = g_strconcat(group_code," ",temp_number,NULL);
		content[index++] = temp;
		content[index++] = d_issued_book_get_trade_name (issued_book);
		content[index++] = g_strdup_printf("%.2f", subtotal);
		content[index++] = g_strdup_printf("%.2f", temp_vat);
		content[index++] = g_strdup_printf("%.2f", vat_share);
		content[index++] = g_strdup_printf("%.2f", total);
		content[index++] = NULL;
	    } else {
		content[index++] = g_strdup(_("VAT:"));
		content[index++] = g_strdup_printf("%.2f", d_issued_book_get_vat_type (issued_book));
		/*En total est la suma dos iva's*/
		/*En subtotal lneas est la suma de base imponibles*/
		content[index++] = g_strdup_printf(_("TOTAL VAT TAX REPERCUTED : %.2f"), d_issued_get_book_total (issued_book)-d_issued_book_get_line_subtotal (issued_book));
		content[index++] = g_strdup("");
		content[index++] = g_strdup(_("TOTAL SUM T.I.:"));
		content[index++] = g_strdup("");
		content[index++] = g_strdup_printf("%.2f", d_issued_book_get_line_subtotal (issued_book));
		content[index++] = NULL;
	    }
	} else {
	    content[index++] = g_strdup(_("TOTAL VAT TAX"));
	    content[index++] = g_strdup("");
	    /*En total est la suma dos iva's*/
	    /*En subtotal lneas est la suma de base imponibles*/
	    if (d_issued_book_get_vat_summary (issued_book)==FALSE)
		content[index++] = g_strdup_printf("%.2f", (d_issued_get_book_total (issued_book)));
	    else
		content[index++] = g_strdup_printf("%.2f", (d_issued_get_book_total (issued_book)-d_issued_book_get_line_subtotal (issued_book)));

	    content[index++] = g_strdup(_("TOTAL TAXABLE:"));
	    content[index++] = g_strdup("");
	    content[index++] = g_strdup_printf("%.2f", d_issued_book_get_line_subtotal (issued_book));
	    content[index++] = g_strdup("");
	    content[index++] = NULL;
	}



    g_free(temp_number);
    g_free(group_code);

    return content;
}

/**
 * d_issued_book_calculate_total_from_list:
 * @issued_books: Objeto #IssuedBook a partir del cual se realizarn los clculos.
 *
 * Calcula los totales a partir de un array de objetos #IssuedBook.
 *
 * Returns:
 * Array con un elemento que contendr los totales calculados.
 */
IssuedBook **  d_issued_book_calculate_total_from_list (IssuedBook ** issued_books) {
    IssuedBook * issued_book=NULL;
    IssuedBook ** result=(IssuedBook **) g_malloc0((sizeof(IssuedBook*)*2));
    gulong index=0;
    gdouble vat_total =0;
    gdouble subtotal_sum =0;
    gdouble total_sum = 0;

    if (issued_books!=NULL) {
	while (issued_books[index]!=NULL) {
	    gdouble total=0;
	    gdouble subtotal=0;
	    gdouble vat_share=0;
	    gdouble temp_vat= d_issued_book_get_vat_type (issued_books[index]);
	    gdouble invoice_discount = d_issued_book_get_invoice_discount (issued_books[index]);

//	    if (temp_vat<-1) {
//                temp_vat=16.00;
	    //	    }

	    if (d_issued_book_get_vat_summary (issued_books[index])==FALSE) {
		/*Si el objeto no es un resumen por Iva se calculan la base imponible y el total
		 dependiendo de si es un mdulo pactado (valor correcto en total) o no
		 (valor de base imponible en subtotal_lineas)*/

		if (d_issued_book_get_agreed_module (issued_books[index])!=FALSE) {
		    total=d_issued_get_book_total (issued_books[index]);
		    subtotal = u_round_to_decimal (total / (1+((temp_vat*1.0)/100.0)),2);
		} else {
		    subtotal = d_issued_book_get_line_subtotal (issued_books[index]);
		    subtotal = subtotal - (subtotal*invoice_discount/100.0);
		    total = u_round_to_decimal (subtotal * (1+((temp_vat*1.0)/100.0)),2);
		}

		/* As estaba antes:
		if (d_issued_book_get_agreed_module (issued_books[index])!=FALSE) {
		    total=d_issued_get_book_total (issued_books[index]);
		    subtotal = total / u_round_to_decimal ((1+((temp_vat*1.0)/100)),2);

		} else {
		    subtotal = d_issued_book_get_line_subtotal (issued_books[index]);
		    total = subtotal * u_round_to_decimal ((1+((temp_vat*1.0)/100)),2);
		    }
                    */
	    } else {
                /*Si es un resumen por Iva la base imponible est en subtotal*/
                subtotal = d_issued_book_get_line_subtotal (issued_books[index]);
                total = d_issued_get_book_total (issued_books[index]);
	    }

//	    vat_share =((temp_vat*1.0)/100) * subtotal;

            subtotal= u_round_to_decimal (subtotal,2);
            total = u_round_to_decimal (total,2);
            vat_share = total - subtotal;
	    vat_share = u_round_to_decimal (vat_share,2);
	    vat_total = vat_total + vat_share;
	    subtotal_sum = subtotal_sum + subtotal;
	    total_sum = total_sum + total;

/*	    g_print("\nCuota Iva: %.2f\n",vat_share);
	    g_print("\nBase imponible:  %.2f\n",subtotal);
	    g_print("\nTotal: %.2f\n",total);
	    g_print("\nSuma Ivas: %.2f\n",vat_total);
	    g_print("\nSuma Bases I:%.2f\n",subtotal_sum);
	    g_print("\nSuma totales: %.2f\n",total_sum);
*/            index++;
	}
    }
    issued_book = d_issued_book_create_empty ();
    g_print("\n%.2f",subtotal_sum);
    g_print("\n%.2f",vat_total);
    d_issued_book_set_total_summary (issued_book,TRUE);
    d_issued_book_set_group_code (issued_book," ");
    d_issued_book_set_invoice_number (issued_book," ");
    d_issued_book_set_trade_name (issued_book,_("TOTALS:"));
    d_issued_book_set_line_subtotal (issued_book,subtotal_sum);
    d_issued_set_book_total (issued_book,vat_total);
    result[0]=issued_book;
    return result;
    
}

/**
 * d_issued_book_concat:
 * @issued_books: Primer array a concatenar.
 * @issued_books2: Segundo array a concatenar.
 *
 * Concatena dos arrays de objetos gasto. #issued_book2 ir en segundo lugar en la concatenacin.
 *
 * Returns:
 * EL nuevo array que contiene los dos arrays pasados como parmetros.
 */
IssuedBook ** d_issued_book_concat (IssuedBook ** issued_books,IssuedBook ** issued_books2) {
    gulong size=0;
    gulong size2=0;
    gulong index=0;
    gulong index2=0;
    IssuedBook ** ib=NULL;
    if (issued_books!=NULL) {
	while (issued_books[size]!=NULL) {
	    size++;
	}


	if (issued_books2!=NULL) {
	    while (issued_books2[size2]!=NULL) {
		size2++;
	    }
	}
	ib = (IssuedBook **) g_malloc0(sizeof(IssuedBook*)*(size+size2+1));
	index = 0;
	while (index<(size)) {
	    ib[index]=issued_books[index];
	    issued_books[index]=NULL;
	    index++;
	    
	}

	index=size;
	while (index<(size+size2)) {
	    ib[index]=issued_books2[index2];
	    issued_books2[index2]=NULL;
	    index++;
	    index2++;
	}
        g_print("\n%lu\n",size+size2+1);

    }
    return ib;
}




/*Funcin que calcula o agrupamento por  IVA das facturas emitidas, inacabado, xa que funcionou
 a select coa base de datos que o calcula automticamente. A este algoritmo solo lle faltara
 crear os issued_books a mostrar.*/

/**
 * d_issued_book_group_by_vat:
 * @issued_books: Primer array a concatenar.
 *
 * INACABADA:
 * Funcin que calcula o agrupamento por  IVA das facturas emitidas, inacabado, xa que funcionou
 * a select coa base de datos que o calcula automticamente. A este algoritmo solo lle faltara
 * crear os issued_books a mostrar.
 *
 * Returns:
 * EL nuevo array que contiene los clculos realizados.
 */
IssuedBook **  d_issued_book_group_by_vat (IssuedBook ** issued_books) {
    GList * data=NULL;
    GList * vat_list=NULL;
    if (issued_books!=NULL) {
        gulong index=0;
	while (issued_books[index]!=NULL) {
	    gchar * vat_type_char;
	    gchar * tipoIvaAlmacenado_char;
	    gdouble total=0;
	    gdouble subtotal = 0;
            gdouble vat_share = 0;
	    GroupedTipoIva * group ;
	    gdouble temp_vat= d_issued_book_get_vat_type (issued_books[index]);

            IssuedBook * issued_book = issued_books[index];



	    if (d_issued_book_get_agreed_module (issued_book)!=FALSE) {
		total=d_issued_get_book_total (issued_book);
		subtotal = u_round_to_decimal (total / (1+((temp_vat*1.0)/100.0)),2);
	    } else {
		subtotal = d_issued_book_get_line_subtotal (issued_book);
		total = u_round_to_decimal (subtotal * (1+((temp_vat*1.0)/100.0)),2);
	    }

	    subtotal= u_round_to_decimal (subtotal,2);
            total = u_round_to_decimal (total,2);
            vat_share = total - subtotal;
	    vat_share = u_round_to_decimal (vat_share,2);

            vat_type_char = g_strdup_printf("%.2f",d_issued_book_get_vat_type (issued_book));
	    if (index==0) {
                group = (GroupedTipoIva *)g_malloc0(sizeof(GroupedTipoIva));
		group->vat_type=d_issued_book_get_vat_type (issued_book);
                group->subtotal_by_vat=subtotal;
		group->vat_share=vat_share;
		data = g_list_append(data,group);
	    } else {
		GList * list = g_list_first(data);
		while (list!=NULL) {
                    group = (GroupedTipoIva *) (list -> data);
		    tipoIvaAlmacenado_char = g_strdup_printf("%.2f",group->vat_type);
		    if (!strcmp(tipoIvaAlmacenado_char,vat_type_char)) {
                        break;
		    }
                    list = g_list_next(list);
		}
		if (list!=NULL) {
		    group->subtotal_by_vat=group->subtotal_by_vat+subtotal;
		    group->vat_share=group->vat_share+vat_share;
		} else {
		    group = (GroupedTipoIva *)g_malloc0(sizeof(GroupedTipoIva));
		    group->vat_type=d_issued_book_get_vat_type (issued_book);
		    group->subtotal_by_vat=group->subtotal_by_vat+subtotal;
		    group->vat_share=group->vat_share+vat_share;
		    data = g_list_append(data,group);

		}

	    }

            index++;
	}
    }

    vat_list = g_list_first(data);
    while (vat_list!=NULL) {
	GroupedTipoIva * group = (GroupedTipoIva *) (vat_list -> data);
	g_print(_("\nVAT Type: %.2f"),group->vat_type);
	g_print(_("\nTaxable Income: %.2f"),group->subtotal_by_vat);
	g_print(_("\nTotal VAT: %.2f"),group->vat_share);
	vat_list = g_list_next(vat_list);
    }

    //  return issued_books_iva;
    return NULL;
}

/* Private functions */

static gint issued_book_init(IssuedBook *issued_book) {
    issued_book->invoice_number=NULL;
    issued_book->invoice_issue_date = time(NULL);
    issued_book->trade_name=NULL;
    issued_book->group_code=NULL;
    issued_book->vat_type=16;
    issued_book->line_subtotal=G_MINDOUBLE;
    issued_book->total=G_MINDOUBLE;
    issued_book->agreed_module=FALSE;
    issued_book->total_summary=FALSE;
    issued_book->vat_summary=FALSE;

    return 0;
}

