 /* issued_book.h
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */



#ifndef _ISSUED_BOOK_
#define _ISSUED_BOOK_

#ifndef _GLIB_LIBRARY_
#define _GLIB_LIBRARY_
#include <glib.h>
#endif

#include <time.h>
#include "participant.h"

typedef struct issued_bookStruct {
    gulong number;
    gchar * invoice_number;
    gchar * group_code;
    time_t invoice_issue_date;
    gchar * trade_name;
    gboolean agreed_module;
    gboolean total_summary;
    gboolean vat_summary;
    gdouble line_subtotal;
    gdouble vat_type;
    gdouble total;
    gdouble invoice_discount;    
} IssuedBook;

typedef struct _vat_type {
    gdouble vat_type;
    gdouble subtotal_by_vat;
    gdouble vat_share;
} GroupedTipoIva ;


IssuedBook * d_issued_book_create_empty (void);

void d_issued_book_set_invoice_number (IssuedBook * issued_book, gchar * invoice_number);
void d_issued_book_set_group_code (IssuedBook * issued_book, gchar * group_code);
void d_issued_book_set_issue_date (IssuedBook * issued_book, time_t invoice_issue_date);
void d_issued_book_set_trade_name (IssuedBook * issued_book, gchar * trade_name);
void d_issued_book_set_vat_type (IssuedBook * issued_book, gdouble vat_type);
void d_issued_book_set_line_subtotal (IssuedBook * issued_book, gdouble line_subtotal);
void d_issued_set_book_total (IssuedBook * issued_book, gdouble total);
void d_issued_book_set_agreed_module (IssuedBook * issued_book, gboolean agreed_module);
void d_issued_book_set_total_summary (IssuedBook * issued_book, gboolean total_summary);
void d_issued_book_set_vat_summary (IssuedBook * issued_book, gboolean vat_summary);
void d_issued_book_set_invoice_discount (IssuedBook * issued_book, gdouble discount);

gchar * d_issued_book_get_invoice_number (IssuedBook * issued_book);
gchar * d_issued_book_get_group_code (IssuedBook * issued_book);
time_t d_issued_book_get_issue_date (IssuedBook * issued_book);
gchar * d_issued_book_get_trade_name (IssuedBook * issued_book);
gdouble d_issued_book_get_line_subtotal (IssuedBook * issued_book);
gdouble d_issued_book_get_vat_type (IssuedBook * issued_book);
gdouble d_issued_book_get_invoice_discount (IssuedBook * issued_book);
gdouble d_issued_get_book_total (IssuedBook * issued_book);
gboolean d_issued_book_get_agreed_module (IssuedBook * issued_book);
gboolean d_issued_book_get_total_summary (IssuedBook * issued_book);
gboolean d_issued_book_get_vat_summary (IssuedBook * issued_book);
IssuedBook **  d_issued_book_calculate_total_from_list (IssuedBook ** issued_books);
IssuedBook ** d_issued_book_concat (IssuedBook ** issued_books,IssuedBook ** issued_books2);
gint d_issued_book_clear (IssuedBook * issued_book);

gchar ***d_issued_book_array_to_content (IssuedBook **array, gulong *reccount);
gchar **d_issued_book_get_content_title (void);
gchar *d_issued_book_get_listing_title (void);
gint *d_issued_book_get_listing_columns_width (void);
gint *d_issued_book_get_listing_columns_alignment (void);
gint d_issued_book_get_content_size (void);
gchar **d_issued_book_get_content (IssuedBook *issued_book);

IssuedBook **  d_issued_book_group_by_vat (IssuedBook ** issued_books);
#endif


