/* measure.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "measure.h"

static gint measure_init(Measure *measure);

/**
 * d_measure_create_empty:
 *
 * Crea un objeto vaco, con memoria reservada e inicializado.
 *
 * Returns:
 * Un objeto #Measure vaco con los campos inicializados a un valor de inicializacin segn el tipo.
 */
Measure * d_measure_create_empty () {
    Measure * measure = (Measure *) g_malloc (sizeof(Measure));

    measure_init(measure);

    return measure;
}


/**
 * d_measure_clear:
 * @measure: Medida a liberar.
 *
 * Libera la memoria utilizada por el objeto.
 *
 * Returns:
 * 0 si se ha acabado la funcin correctamente.
 */
gint d_measure_clear (Measure *measure){
    if (measure!=NULL) {
	if (measure->measure_name!=NULL)
            g_free(measure->measure_name);
	if (measure->abbreviation!=NULL)
            g_free(measure->abbreviation);
	g_free(measure);
    }

    return 0;
}

/**
 * d_measure_set_measure_code:
 * @measure: Objeto en el que se establece el atributo.
 * @measure_code: Atributo a insertar.
 *
 * Establece el atributo @measure_code en el objeto. El atributo @measure_code
 * indica el cdigo de la medida.
 *
 */
void d_measure_set_measure_code (Measure *measure, gulong measure_code){
  measure->measure_code = measure_code;
}

/**
 * d_measure_set_name:
 * @measure: Objeto en el que se establece el atributo.
 * @measure_name: Atributo a insertar.
 *
 * Establece el atributo @measure_name en el objeto. El atributo @measure_name
 * indica el nombre de la medida.
 *
 */
void d_measure_set_name (Measure *measure, gchar * measure_name){
  measure->measure_name = g_strdup(measure_name);
}


/**
 * d_measure_set_abbreviation:
 * @measure: Objeto en el que se establece el atributo.
 * @abbreviation: Atributo a insertar.
 *
 * Establece el atributo @abbreviation en el objeto. El atributo @abbreviation
 * indica el abreviatura de la medida.
 *
 */
void d_measure_set_abbreviation (Measure *measure, gchar * abbreviation){
  measure->abbreviation = g_strdup(abbreviation);
}

/**
 * d_measure_get_measure_code:
 * @measure: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_measure_get_measure_code (Measure *measure){
  return measure->measure_code;
}

/**
 * d_measure_get_name:
 * @measure: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar * d_measure_get_name (Measure *measure){
  return g_strdup(measure->measure_name);
}

/**
 * d_measure_get_abbreviation:
 * @measure: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar * d_measure_get_abbreviation (Measure *measure){
  return g_strdup(measure->abbreviation);
}

/**
 * d_measure_duplicate:
 * @measure: Objeto del que se desea obtener el duplicado.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: Un objeto con todos los campos duplicados y con nueva memoria reservada.
 */
Measure * d_measure_duplicate (Measure * measure) {
    Measure * measuredup = d_measure_create_empty ();
    if (measure !=NULL) {
        gchar * temp = d_measure_get_name (measure);
	d_measure_set_name (measuredup,temp);
	g_free(temp);
        temp = d_measure_get_abbreviation (measure);
	d_measure_set_abbreviation (measuredup,temp);
	g_free(temp);
	d_measure_set_measure_code (measuredup,d_measure_get_measure_code (measure));
    }
    return measuredup;
}

/* Private functions*/

static gint measure_init(Measure *measure) 
{
    measure->measure_code = ULONG_MAX;
    measure->measure_name = NULL;
    measure->abbreviation = NULL;
   
  return 0;
}

