/* office.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "office.h"


static gint office_init(Office *office);



/**
 * d_office_create_empty:
 *
 * Crea un objeto vaco, con memoria reservada e inicializado.
 *
 * Returns:
 * Un objeto #Office vaco con los campos inicializados a un valor de inicializacin segn el tipo.
 */
Office *d_office_create_empty () {
  Office *office = (Office *) g_malloc (sizeof(Office));

  office_init(office);
  
  return office;
}

/**
 * d_office_clear:
 * @office: Objeto a liberar.
 *
 * Libera la memoria utilizada por el objeto.
 *
 * Returns:
 * 0 si se ha acabado la funcin correctamente.
 */
gint d_office_clear (Office *office) {

    if (office!=NULL) {
	if (office->bank_name!=NULL)
	    g_free(office->bank_name);
	if (office->name!=NULL)
	    g_free(office->name);
	if (office->address!=NULL)
	    g_free(office->address);
	g_free(office);
    }

  return 0;
}

/**
 * d_office_set_bank_name:
 * @office: Objeto en el que se establece el atributo.
 * @bank_name: Atributo a insertar.
 *
 * Establece el atributo @bank_name en el objeto. El atributo @bank_name
 * indica el nombre del banco al que pertenece la oficina.
 *
 */
void d_office_set_bank_name (Office *office, gchar *bank_name){
    if(office->bank_name!=NULL) {
	g_free(office->bank_name);
    }
    office->bank_name=bank_name;
}
/**
 * d_office_set_name:
 * @office: Objeto en el que se establece el atributo.
 * @name: Atributo a insertar.
 *
 * Establece el atributo @name en el objeto. El atributo @name
 * indica el nombre de la oficina bancaria.
 *
 */
void d_office_set_name (Office *office, gchar * name){
    if(office->name!=NULL) {
	g_free(office->name);
    }
    office->name=name;
}
/**
 * d_office_set_address:
 * @office: Objeto en el que se establece el atributo.
 * @address: Atributo a insertar.
 *
 * Establece el atributo @address en el objeto. El atributo @address
 * indica la direccin de la oficina.
 *
 */
void d_office_set_address (Office *office, gchar * address){
    if(office->address!=NULL) {
	g_free(office->address);
    }
    office->address=address;
}
/**
 * d_office_set_bank_code:
 * @office: Objeto en el que se establece el atributo.
 * @bank_code: Atributo a insertar.
 *
 * Establece el atributo @bank_code en el objeto. El atributo @bank_code
 * indica el cdigo del banco al que pertenece la oficina.
 *
 */
void d_office_set_bank_code (Office *office, gint bank_code){
    office->bank_code = bank_code;
}
/**
 * d_office_set_office_code:
 * @office: Objeto en el que se establece el atributo.
 * @office_code: Atributo a insertar.
 *
 * Establece el atributo @office_code en el objeto. El atributo @office_code
 * indica el cdigo de la oficina.
 *
 */
void d_office_set_office_code (Office *office, gint office_code){
    office->office_code = office_code;
}

/**
 * d_office_get_bank_name:
 * @office: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar *d_office_get_bank_name (Office *office){
  return g_strdup(office->bank_name);
}
/**
 * d_office_get_name:
 * @office: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar *d_office_get_name (Office *office){
  return g_strdup(office->name);
}
/**
 * d_office_get_address:
 * @office: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar *d_office_get_address (Office *office){
  return g_strdup(office->address);
}
/**
 * d_office_get_office_code:
 * @office: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gint d_office_get_office_code (Office *office){
  return office->office_code;
}
/**
 * d_office_get_bank_code:
 * @office: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gint d_office_get_bank_code (Office *office){
  return office->bank_code;
}




/* Private functions*/

static gint office_init(Office *office) {

    office->bank_code = -1;
    office->office_code = -1;
    office->bank_name=NULL;
    office->name=NULL;
    office->address=NULL;
    return 0;
}
