/*  operation.h
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#ifndef _OPERATION_
#define _OPERATION_

#ifndef _GLIB_LIBRARY_
#define _GLIB_LIBRARY_
#include <glib.h>
#endif

#include "account.h"
#include "payment.h"


typedef struct _operation {
    Account * account;
    Payment * payment;
    gulong operation_code;
    gchar * section;
    time_t date;
    time_t notification_date;
    gdouble amount;
    gchar * receipt_reference;
} Operation;


Operation *d_operation_create_empty (void); 
gint d_operation_clear (Operation *operation);
void d_operation_set_account (Operation *operation, Account * account);
void d_operation_set_payment (Operation *operation, Payment * payment);
void d_operation_set_receipt_reference (Operation *operation, gchar * receipt_reference);
void d_operation_set_amount (Operation *operation, gdouble amount);
void d_operation_set_section (Operation *operation, gchar * section);
void d_operation_set_date (Operation *operation, time_t date);
void d_operation_set_notification_date (Operation *operation, time_t notification_date);
void d_operation_set_code (Operation *operation, gulong operation_code);

Account * d_operation_get_account (Operation *operation);
Payment * d_operation_get_payment (Operation *operation);
gdouble d_operation_get_amount (Operation *operation);
gchar * d_operation_get_section (Operation *operation);
gulong d_operation_get_code (Operation *operation);
time_t d_operation_get_date (Operation *operation);
time_t d_operation_get_notification_date (Operation *operation);
gchar * d_operation_get_receipt_reference (Operation *operation);

gchar ***d_operation_array_to_content (Operation **array, gulong *reccount);
gchar  **d_operation_get_content (Operation *operation);
gchar  **d_operation_get_content_title (void);
gchar   *d_operation_get_listing_title (void);
gint    *d_operation_get_listing_columns_width (void);
gint    *d_operation_get_listing_columns_alignment (void);
gint     d_operation_get_content_size (void);

#endif


