
/* order_line.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "order_line.h"
#include "model.h"

static gint order_line_init(OrderLine *order_line);

/**
 * d_order_line_create_empty:
 *
 * Crea un objeto vaco, con memoria reservada e inicializado.
 *
 * Returns:
 * Un objeto #OrderLine vaco con los campos inicializados a un valor de inicializacin segn el tipo.
 */
OrderLine  *d_order_line_create_empty (){
    OrderLine *order_line = (OrderLine *) g_malloc(sizeof(OrderLine));

    order_line_init(order_line);

    return order_line;
}

/**
 * d_order_line_clear:
 * @order_line: Objeto a liberar.
 *
 * Libera la memoria utilizada por el objeto.
 *
 * Returns:
 * 0 si se ha acabado la funcin correctamente.
 */
gint d_order_line_clear (OrderLine *order_line) {

    if (order_line!=NULL) {
	if (d_order_line_get_piece (order_line)!=NULL) {
	    d_piece_clear (d_order_line_get_piece (order_line));
	}
	if (order_line->delivery_payment!=NULL)
	    g_free(order_line->delivery_payment);

	g_free(order_line);
	order_line=NULL;
    }

    return 0;
}

/**
 * d_order_line_set_code:
 * @order_line: Objeto en el que se establece el atributo.
 * @order_code: Atributo a insertar.
 *
 * Establece el atributo @order_code en el objeto. El atributo @order_code
 * indica el cdigo de pedido al que pertenece la lnea de pedido.
 *
 */
void d_order_line_set_code (OrderLine * order_line, gulong order_code) {
    order_line->order_code = order_code;
}

/**
 * d_order_line_set_enterprise_code:
 * @order_line: Objeto en el que se establece el atributo.
 * @enterprise_code: Atributo a insertar.
 *
 * Establece el atributo @enterprise_code en el objeto. El atributo @enterprise_code
 * indica el cdigo de la empresa a la que pertenece.
 *
 */
void d_order_line_set_enterprise_code (OrderLine * order_line, gulong enterprise_code) {
    order_line->enterprise_code = enterprise_code;
}

/**
 * d_order_line_set_warehouse_code:
 * @order_line: Objeto en el que se establece el atributo.
 * @warehouse_code: Atributo a insertar.
 *
 * Establece el atributo @warehouse_code en el objeto. El atributo @warehouse_code
 * indica el cdigo del almacn en el que se realiz el pedido.
 *
 */
void d_order_line_set_warehouse_code (OrderLine * order_line, gulong warehouse_code) {
    order_line->warehouse_code = warehouse_code;
}

/**
 * d_order_line_set_line_number:
 * @order_line: Objeto en el que se establece el atributo.
 * @line_number: Atributo a insertar.
 *
 * Establece el atributo @line_number en el objeto. El atributo @line_number
 * indica el nmero del lnea de la lnea de pedido.
 *
 */
void d_order_line_set_line_number (OrderLine * order_line, gulong line_number) {
    order_line->line_number = line_number;
}

/**
 * d_order_line_set_piece:
 * @order_line: Objeto en el que se establece el atributo.
 * @piece: Atributo a insertar.
 *
 * Establece el atributo @piece en el objeto. El atributo @piece
 * indica la pieza a la que referencia el pedido.
 *
 */
void d_order_line_set_piece (OrderLine * order_line, Piece *piece) {
    d_piece_clear (order_line->piece);
    order_line->piece = piece;
}

/**
 * d_order_line_set_quantity:
 * @order_line: Objeto en el que se establece el atributo.
 * @quantity: Atributo a insertar.
 *
 * Establece el atributo @quantity en el objeto. El atributo @quantity
 * indica las unidades que se piden de una pieza.
 *
 */
void d_order_line_set_quantity (OrderLine * order_line, gdouble quantity) {
    order_line->quantity = quantity;
}
/**
 * d_order_line_set_charge_invoice_supplier_code:
 * @order_line: Objeto en el que se establece el atributo.
 * @charge_invoice_supplier_code: Atributo a insertar.
 *
 * Establece el atributo @charge_invoice_supplier_code en el objeto. El atributo @charge_invoice_supplier_code
 * indica el cdigo del proveedor de la factura de cargo a la que referencia el pedido.
 *
 */
void d_order_line_set_charge_invoice_supplier_code (OrderLine * order_line, gulong charge_invoice_supplier_code) {
    order_line->charge_invoice_supplier_code = charge_invoice_supplier_code;
}
/**
 * d_order_line_set_charge_line_number:
 * @order_line: Objeto en el que se establece el atributo.
 * @charge_line_number: Atributo a insertar.
 *
 * Establece el atributo @charge_line_number en el objeto. El atributo @charge_line_number
 * indica el numero de lnea de la factura de cargo a la que referencia.
 *
 */
void d_order_line_set_charge_line_number (OrderLine * order_line, gulong charge_line_number) {
    order_line->charge_line_number = charge_line_number;
}

/**
 * d_order_line_set_delivery_payment:
 * @order_line: Objeto en el que se establece el atributo.
 * @delivery_payment: Atributo a insertar.
 *
 * Establece el atributo @delivery_payment en el objeto. El atributo @delivery_payment
 * indica el albarn en el que vino la lnea de pedido.
 *
 */
void d_order_line_set_delivery_payment (OrderLine * order_line, gchar * delivery_payment) {
    order_line->delivery_payment = g_strdup(delivery_payment);
}
/**
 * d_order_line_set_with_delivery_payment:
 * @order_line: Objeto en el que se establece el atributo.
 * @with_delivery_payment: Atributo a insertar.
 *
 * Establece el atributo @with_delivery_payment en el objeto. El atributo @with_delivery_payment
 * indica si el pedido incluye una referencia a una albarn.
 *
 */
void d_order_line_set_with_delivery_payment (OrderLine * order_line,gboolean with_delivery_payment) {
    order_line->with_delivery_payment = with_delivery_payment;
}

/**
 * d_order_line_set_charge_invoice_number:
 * @order_line: Objeto en el que se establece el atributo.
 * @charge_invoice_number: Atributo a insertar.
 *
 * Establece el atributo @charge_invoice_number en el objeto. El atributo @charge_invoice_number
 * indica el nmero de la factura de cargo a la que referencia el pedido.
 *
 */
void d_order_line_set_charge_invoice_number (OrderLine * order_line, gulong charge_invoice_number) {
    order_line->charge_invoice_number = (charge_invoice_number);
}

/* void     set_lineaFacturaCargo_order_line(OrderLine * order_line, ChargeInvoiceLine *lineaFacturaCargo) { */
/*     order_line->lineaFacturaCargo = lineaFacturaCargo; */
/* } */

/**
 * d_order_line_get_code:
 * @order_line: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_order_line_get_code (OrderLine * order_line) {
    return(order_line->order_code);
}

/**
 * d_order_line_get_enterprise_code:
 * @order_line: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_order_line_get_enterprise_code (OrderLine * order_line) {
    return(order_line->enterprise_code);
}

/**
 * d_order_line_get_warehouse_code:
 * @order_line: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_order_line_get_warehouse_code (OrderLine * order_line) {
    return(order_line->warehouse_code);
}

/**
 * d_order_line_get_line_number:
 * @order_line: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_order_line_get_line_number (OrderLine * order_line) {
    return(order_line->line_number);
}

/**
 * d_order_line_get_piece:
 * @order_line: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
Piece *d_order_line_get_piece (OrderLine * order_line) {
    return(order_line->piece);
}

/**
 * d_order_line_get_quantity:
 * @order_line: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gdouble d_order_line_get_quantity (OrderLine * order_line) {
    return(order_line->quantity);
}

/**
 * d_order_line_get_charge_line_number:
 * @order_line: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_order_line_get_charge_line_number (OrderLine * order_line) {
    return(order_line->charge_line_number);
}

/**
 * d_order_line_get_charge_invoice_supplier_code:
 * @order_line: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_order_line_get_charge_invoice_supplier_code (OrderLine * order_line) {
    return(order_line->charge_invoice_supplier_code);
}

/**
 * d_order_line_get_with_delivery_payment:
 * @order_line: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gboolean d_order_line_get_with_delivery_payment (OrderLine * order_line) {
    return(order_line->with_delivery_payment);
}

/* ChargeInvoiceLine   *get_lineaFacturaCargo_order_line(OrderLine * order_line) { */
/*     return order_line->lineaFacturaCargo; */
/* } */

/**
 * d_order_line_get_delivery_payment:
 * @order_line: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar * d_order_line_get_delivery_payment (OrderLine * order_line) {
    return g_strdup(order_line->delivery_payment);
}

/**
 * d_order_line_get_charge_invoice_number:
 * @order_line: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_order_line_get_charge_invoice_number (OrderLine * order_line) {
    return (order_line->charge_invoice_number);
}

/**
 * d_order_line_duplicate:
 * @lines: Array de lneas de pedido del que se desea obtener el duplicado.
 *
 * Duplica las lneas de pedido
 *
 * Returns: Un array con las lneas duplicadas y con nueva memoria reservada.
 */
OrderLine **d_order_line_duplicate (OrderLine **lines) {
    if (lines!=NULL) {
	OrderLine **result = NULL;
	gint size = 0;
	gint index = 0;
	gchar * temp=NULL;

	while(lines[size]!=NULL){
	    size++;
	}

	result = (OrderLine **)g_malloc((size+1)*sizeof(OrderLine *));

	while(lines[index]!=NULL) {
	    Piece *piece = d_order_line_get_piece (lines[index]);

	    result[index] = d_order_line_create_empty ();


	    d_order_line_set_code (result[index],
					d_order_line_get_code (lines[index]));
	    d_order_line_set_enterprise_code (result[index],
					 d_order_line_get_enterprise_code (lines[index]));
	    d_order_line_set_warehouse_code (result[index],
					 d_order_line_get_warehouse_code (lines[index]));
	    d_order_line_set_line_number (result[index],
				       d_order_line_get_line_number (lines[index]));
	    d_order_line_set_charge_line_number (result[index],
					    d_order_line_get_charge_line_number (lines[index]));
	    d_order_line_set_charge_invoice_supplier_code (result[index],
						       d_order_line_get_charge_invoice_supplier_code (lines[index]));

	    if (piece != NULL) {
		d_order_line_set_piece (result[index],
				     d_piece_duplicate (piece));
	    }

	    d_order_line_set_quantity (result[index],
				    d_order_line_get_quantity (lines[index]));
	    temp = d_order_line_get_delivery_payment (lines[index]);
	    d_order_line_set_delivery_payment (result[index],temp
					);
	    g_free(temp);
	    
	    d_order_line_set_charge_invoice_number (result[index],
					 d_order_line_get_charge_invoice_number (lines[index]));
	    

	    index++;
	}
    
	result[index] = NULL;
	return result;
    } else {
	return NULL;
    }
}


/* Private functions */

static gint order_line_init(OrderLine *order_line) {
    order_line->order_code  = ULONG_MAX;
    order_line->enterprise_code = ULONG_MAX;
    order_line->warehouse_code = ULONG_MAX;
    order_line->line_number   = ULONG_MAX;
    order_line->piece         = NULL;
    order_line->quantity      = G_MINDOUBLE;
    order_line->delivery_payment = NULL;
    order_line->charge_invoice_number = ULONG_MAX;
    order_line->charge_line_number = ULONG_MAX;
    order_line->charge_invoice_supplier_code = ULONG_MAX;
/*     order_line->lineaFacturaCargo = NULL; */

    return 0;
}


