/* particular_object.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "particular_object.h"



static gint particular_init(ParticularObject *particular);

/**
 * d_particular_create_empty:
 *
 * Crea un objeto vaco, con memoria reservada e inicializado.
 *
 * Returns:
 * Un objeto #Particular vaco con los campos inicializados a un valor de inicializacin segn el tipo.
 */
ParticularObject * d_particular_create_empty () {

    ParticularObject * particular;

    particular = (ParticularObject *) g_malloc0(sizeof(ParticularObject));
    particular_init(particular);

    return particular;
}



/**
 * d_particular_clear:
 * @particular: Objeto a liberar.
 *
 * Libera la memoria utilizada por el objeto.
 *
 * Returns:
 * 0 si se ha acabado la funcin correctamente.
 */
gint d_particular_clear (ParticularObject *particular) {
    if (particular!=NULL) {
	if (particular->first_surname!=NULL)
	    g_free(particular->first_surname);
	if (particular->second_surname!=NULL)
	    g_free(particular->second_surname);
	d_participant_clear (d_particular_get_participant (particular));
	g_free(particular);
    }
    return 0;
}

/**
 * d_particular_set_participant:
 * @particular: Objeto en el que se establece el atributo.
 * @participant: Atributo a insertar.
 *
 * Establece el atributo @participant en el objeto. El atributo @participant
 * indica los datos participante que posee un particular.
 *
 */
void d_particular_set_participant (ParticularObject * particular,Participant * participant) {
    particular->participant = participant;
}

/**
 * d_particular_set_first_surname:
 * @particular: Objeto en el que se establece el atributo.
 * @first_surname: Atributo a insertar.
 *
 * Establece el atributo @first_surname en el objeto. El atributo @first_surname
 * indica el primer apellido de una persona.
 *
 */
void d_particular_set_first_surname (ParticularObject * particular,gchar * first_surname){
    if(particular->first_surname!=NULL) {
	g_free(particular->first_surname);
    }
    particular->first_surname=g_strdup(first_surname);
}

/**
 * d_particular_set_second_surname:
 * @particular: Objeto en el que se establece el atributo.
 * @second_surname: Atributo a insertar.
 *
 * Establece el atributo @second_surname en el objeto. El atributo @second_surname
 * indica el segundo apellido de un particular.
 *
 */
void d_particular_set_second_surname (ParticularObject * particular,gchar * second_surname){
    if(particular->second_surname!=NULL) {
	g_free(particular->second_surname);
    }
    particular->second_surname=g_strdup(second_surname);
}


/**
 * d_particular_get_participant:
 * @particular: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
Participant * d_particular_get_participant (ParticularObject * particular){
    return particular->participant;
}

/**
 * d_particular_get_first_surname:
 * @particular: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar * d_particular_get_first_surname (ParticularObject * particular){
    return g_strdup(particular->first_surname);
}

/**
 * d_particular_get_second_surname:
 * @particular: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar * d_particular_get_second_surname (ParticularObject * particular){
    return g_strdup(particular->second_surname);
}

/* Private functions */

static gint particular_init(ParticularObject *particular) {

    particular->first_surname = NULL;
    particular->second_surname = NULL;
    particular->participant = NULL;

    return 0;
}

