/* policy.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "policy.h"



/**
 * d_policy_create_empty:
 *
 * Crea un objeto vaco, con memoria reservada e inicializado.
 *
 * Returns:
 * Un objeto #Policy vaco con los campos inicializados a un valor de inicializacin segn el tipo.
 */
Policy *d_policy_create_empty () {
  Policy *policy = (Policy *) g_malloc (sizeof(Policy));

  d_policy_init (policy);
  
  return policy;
}


/**
 * d_policy_clear:
 * @policy: Objeto a liberar.
 *
 * Libera la memoria utilizada por el objeto.
 *
 * Returns:
 * 0 si se ha acabado la funcin correctamente.
 */
gint d_policy_clear (Policy *policy) {

  if (policy!=NULL) {
    if (policy->policy_number!=NULL)
      g_free(policy->policy_number);
    
    if (policy->old_policy_number!=NULL)
      g_free(policy->old_policy_number);

    
    g_free(policy);
  }
  
  return 0;
}

/**
 * d_policy_set_company_code:
 * @policy: Objeto en el que se establece el atributo.
 * @company_code: Atributo a insertar.
 *
 * Establece el atributo @company_code en el objeto. El atributo @company_code
 * indica el cdigo de la compaa con la que se tiene contratada la pliza.
 *
 */
void d_policy_set_company_code (Policy *policy, gulong company_code) {
  policy->company_code = company_code;
}

/**
 * d_policy_set_policy_number:
 * @policy: Objeto en el que se establece el atributo.
 * @policy_number: Atributo a insertar.
 *
 * Establece el atributo @policy_number en el objeto. El atributo @policy_number
 * indica el nmero de la pliza de seguros contratada.
 *
 */
void d_policy_set_policy_number (Policy *policy, gchar *policy_number) {
  if (policy->policy_number != NULL) g_free(policy -> policy_number);
  policy->policy_number = g_strdup(policy_number);
}

/**
 * d_policy_set_holder_code:
 * @policy: Objeto en el que se establece el atributo.
 * @holder_code: Atributo a insertar.
 *
 * Establece el atributo @holder_code en el objeto. El atributo @holder_code
 * indica el cdigo del participante que figura como titular de la pliza de
 * seguro.
 *
 */
void d_policy_set_holder_code (Policy *policy, gulong holder_code) {
  policy->holder_code = holder_code;
}

/**
 * d_policy_set_old_company_code:
 * @policy: Objeto en el que se establece el atributo.
 * @company_code: Atributo a insertar.
 *
 * Establece el atributo @old_company_code en el objeto. El atributo @old_company_code
 * indica el cdigo de la compaa anterior a modificar la pliza.
 *
 */
void d_policy_set_old_company_code (Policy *policy, gulong company_code) {
  policy->old_company_code = company_code;
}

/**
 * d_policy_set_old_policy_number:
 * @policy: Objeto en el que se establece el atributo.
 * @policy_number: Atributo a insertar.
 *
 * Establece el atributo @old_policy_number en el objeto. El atributo @old_policy_number
 * indica el nmero de pliza antiguo anterior a modificar los datos de pliza.
 *
 */
void d_policy_set_old_policy_number (Policy *policy, gchar *policy_number) {
  if (policy->old_policy_number != NULL) g_free(policy -> old_policy_number);
  policy->old_policy_number = g_strdup(policy_number);
}

/**
 * d_policy_set_old_holder_code:
 * @policy: Objeto en el que se establece el atributo.
 * @holder_code: Atributo a insertar.
 *
 * Establece el atributo @old_holder_code en el objeto. El atributo @old_holder_code
 * indica el cdigo de titular antiguo anterior a modificar los datos de la pliza.
 *
 */
void d_policy_set_old_holder_code (Policy *policy, gulong holder_code) {
  policy->old_holder_code = holder_code;
}

/**
 * d_policy_get_company_code:
 * @policy: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_policy_get_company_code (Policy *policy) {
  return policy->company_code;
}

/**
 * d_policy_get_policy_number:
 * @policy: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar *d_policy_get_policy_number (Policy *policy) {
  return g_strdup(policy->policy_number);
}

/**
 * d_policy_get_holder_code:
 * @policy: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_policy_get_holder_code (Policy *policy) {
  return policy->holder_code;
}

/**
 * d_policy_get_old_company_code:
 * @policy: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_policy_get_old_company_code (Policy *policy) {
  return policy->old_company_code;
}

/**
 * d_policy_get_old_policy_number:
 * @policy: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar *d_policy_get_old_policy_number (Policy *policy) {
  return g_strdup(policy->old_policy_number);
}

/**
 * d_policy_get_old_holder_code:
 * @policy: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_policy_get_old_holder_code (Policy *policy) {
  return policy->old_holder_code;
}

/**
 * d_policy_init:
 * @policy: Pliza de seguros a la que se le inicializarn todos los atributos.
 *
 * Inicializa los atributos de una pliza a un valor predeterminado segn el tipo.
 *
 * Returns:
 * 0 si la funcin ha acabado correctamente.
 */
gint d_policy_init (Policy *policy) {

    policy->company_code = ULONG_MAX;
    policy->policy_number = NULL;
    policy->holder_code = ULONG_MAX;
    policy->old_company_code = ULONG_MAX;
    policy->old_policy_number = NULL;
    policy->old_holder_code = ULONG_MAX;


    return 0;
}

/**
 * d_policy_set_old_key:
 * @policy: Objeto que se desea modificar.
 *
 * Establece la pliza y recibo antiguos en el objeto cuando se intenta modificar el nmero de pliza y recibo.
 *
 */
void d_policy_set_old_key (Policy *policy) {

  gchar *numero_poliza = d_policy_get_policy_number (policy);
  gulong codigo_company = d_policy_get_company_code (policy);
  gulong codigo_titular = d_policy_get_holder_code (policy);

  d_policy_set_old_policy_number (policy, numero_poliza);
  d_policy_set_old_company_code (policy, codigo_company);
  d_policy_set_old_holder_code (policy, codigo_titular);

}

/**
 * d_policy_duplicate:
 * @policy: Objeto del que se desea obtener el duplicado.
 *
 * Duplica el objeto.
 *
 * Returns: Un objeto con todos los campos duplicados y con nueva memoria reservada.
 */
Policy *d_policy_duplicate (Policy *policy) {

  Policy *result = d_policy_create_empty ();
  gchar *numero_poliza = d_policy_get_policy_number (policy);
  gulong codigo_company = d_policy_get_company_code (policy);
  gulong codigo_titular = d_policy_get_holder_code (policy);
  gchar *numero_poliza_old = d_policy_get_old_policy_number (policy);
  gulong codigo_company_old = d_policy_get_old_company_code (policy);
  gulong codigo_titular_old = d_policy_get_old_holder_code (policy);

  d_policy_set_policy_number (result, numero_poliza);
  d_policy_set_company_code (result, codigo_company);
  d_policy_set_holder_code (result, codigo_titular);
  d_policy_set_old_policy_number (result, numero_poliza_old);
  d_policy_set_old_company_code (result, codigo_company_old);
  d_policy_set_old_holder_code (result, codigo_titular_old);

  g_free(numero_poliza);
  g_free(numero_poliza_old);

  return result;
  
}
