/* province.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "province.h"



static gint province_init(Province *province);

/**
 * d_province_create_empty:
 *
 * Crea un objeto vaco, con memoria reservada e inicializado.
 *
 * Returns:
 * Un objeto #Province vaco con los campos inicializados a un valor de inicializacin segn el tipo.
 */
Province * d_province_create_empty () {
    Province * province = (Province *) g_malloc (sizeof(Province));

    province_init(province);

    return province;
}


/**
 * d_province_clear:
 * @province: Provincia a liberar.
 *
 * Libera la memoria utilizada por el objeto.
 *
 * Returns:
 * 0 si se ha acabado la funcin correctamente.
 */
gint d_province_clear (Province *province){
    if (province!=NULL) {
	if (province->name!=NULL)
            g_free(province->name);
	g_free(province);
    }

    return 0;
}

/**
 * d_province_set_province_code:
 * @province: Objeto en el que se establece el atributo.
 * @province_code: Atributo a insertar.
 *
 * Establece el atributo @province_code en el objeto. El atributo @province_code
 * indica el cdigo de la provincia.
 *
 */
void d_province_set_province_code (Province *province, gulong province_code){
  province->province_code = province_code;
}

/**
 * d_province_set_name:
 * @province: Objeto en el que se establece el atributo.
 * @name: Atributo a insertar.
 *
 * Establece el atributo @name en el objeto. El atributo @name
 * indica el nombre de la provincia.
 *
 */
void d_province_set_name (Province *province, gchar * name){
  province->name = g_strdup(name);
}

/**
 * d_province_get_province_code:
 * @province: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_province_get_province_code (Province *province){
  return province->province_code;
}

/**
 * d_province_get_name:
 * @province: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar * d_province_get_name (Province *province){
  return g_strdup(province->name);
}

/**
 * d_province_duplicate:
 * @province: Objeto del que se desea obtener el duplicado.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: Un objeto con todos los campos duplicados y con nueva memoria reservada.
 */
Province * d_province_duplicate (Province * province) {
    Province * provincedup = d_province_create_empty ();
    if (province !=NULL) {
        gchar * temp = d_province_get_name (province);
	d_province_set_name (provincedup,temp);
        g_free(temp);
	d_province_set_province_code (provincedup,d_province_get_province_code (province));
    }
    return provincedup;
}

/* Private functions*/

static gint province_init(Province *province) 
{
    province->province_code = ULONG_MAX;
    province->name = NULL;
   
  return 0;
}

