
/* regularization.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "regularization.h"
#include "util.h"


static gint regularization_init(Regularization *regularization);

/**
 * d_regularization_create_empty:
 *
 * Crea un objeto vaco, con memoria reservada e inicializado.
 *
 * Returns:
 * Un objeto #Regularization vaco con los campos inicializados a un valor de inicializacin segn el tipo.
 */
Regularization * d_regularization_create_empty () {
    Regularization *regularization = (Regularization *) g_malloc(sizeof(Regularization));

    regularization_init(regularization);

    return regularization;                           
}


/**
 * d_regularization_clear:
 * @regularization: Objeto a liberar.
 *
 * Libera la memoria utilizada por el objeto.
 *
 * Returns:
 * 0 si se ha acabado la funcin correctamente.
 */
gint d_regularization_clear (Regularization *regularization) {
    if (regularization!=NULL) {
	g_free(regularization);
    }
    return 0;
}

/**
 * d_regularization_set_enterprise_code:
 * @regularization: Objeto en el que se establece el atributo.
 * @enterprise_code: Atributo a insertar.
 *
 * Establece el atributo @enterprise_code en el objeto. El atributo @enterprise_code
 * indica el cdigo de la empresa en la que se realiza la regularizacin del stock.
 *
 */
void d_regularization_set_enterprise_code (Regularization *regularization, gulong enterprise_code) {
    regularization->enterprise_code = enterprise_code;
}

/**
 * d_regularization_set_warehouse_code:
 * @regularization: Objeto en el que se establece el atributo.
 * @warehouse_code: Atributo a insertar.
 *
 * Establece el atributo @warehouse_code en el objeto. El atributo @warehouse_code
 * indica el cdigo del almacn en el que se realizar la regularizacin del stock.
 *
 */
void d_regularization_set_warehouse_code (Regularization *regularization, gulong warehouse_code) {
    regularization->warehouse_code = warehouse_code;
}


/**
 * d_regularization_set_product_code:
 * @regularization: Objeto en el que se establece el atributo.
 * @product_code: Atributo a insertar.
 *
 * Establece el atributo @product_code en el objeto. El atributo @product_code
 * indica el cdigo del producto para el que se realizar la regularizacin del stock.
 *
 */
void d_regularization_set_product_code (Regularization *regularization, gulong product_code) {
    regularization->product_code = product_code;
}

/**
 * d_regularization_set_quantity:
 * @regularization: Objeto en el que se establece el atributo.
 * @quantity: Atributo a insertar.
 *
 * Establece el atributo @quantity en el objeto. El atributo @quantity
 * indica la unidades que se regularizarn.
 *
 */
void d_regularization_set_quantity (Regularization *regularization, gdouble quantity) {
    regularization->quantity = quantity;
}

/**
 * d_regularization_get_enterprise_code:
 * @regularization: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_regularization_get_enterprise_code (Regularization *regularization) {
    return regularization->enterprise_code;
}

/**
 * d_regularization_get_warehouse_code:
 * @regularization: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_regularization_get_warehouse_code (Regularization *regularization) {
    return regularization->warehouse_code;
}

/**
 * d_regularization_get_product_code:
 * @regularization: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_regularization_get_product_code (Regularization *regularization) {
    return regularization->product_code;
}


/**
 * d_regularization_get_quantity:
 * @regularization: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gdouble d_regularization_get_quantity (Regularization *regularization) {
    return regularization->quantity;
}

/**
 * d_regularization_get_date:
 * @regularization: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
time_t d_regularization_get_date (Regularization *regularization) {
    return regularization->date;
}

/* Private functions*/

static gint regularization_init(Regularization *regularization) {
    regularization->enterprise_code = ULONG_MAX;
    regularization->warehouse_code = ULONG_MAX;
    regularization->movement_code = ULONG_MAX;
    regularization->product_code = ULONG_MAX;
    regularization->quantity = G_MINDOUBLE;
    regularization->date = time(NULL);

    return 0;
}
