/* repair_movement.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "repair_movement.h"
#include "util.h"

static gint repair_movement_init(RepairMovement *repair_movement);

/**
 * d_repair_movement_create_empty:
 *
 * Crea un objeto vaco, con memoria reservada e inicializado.
 *
 * Returns:
 * Un objeto #RepairMovement vaco con los campos inicializados a un valor de inicializacin segn el tipo.
 */
RepairMovement * d_repair_movement_create_empty () {
    RepairMovement *repair_movement = (RepairMovement *) g_malloc0(sizeof(RepairMovement));

    repair_movement_init(repair_movement);

    return repair_movement;                           
}

/**
 * d_repair_movement_clear:
 * @repair_movement: Objeto a liberar.
 *
 * Libera la memoria utilizada por el objeto.
 *
 * Returns:
 * 0 si se ha acabado la funcin correctamente.
 */
gint d_repair_movement_clear (RepairMovement *repair_movement) {
    if (repair_movement!=NULL) {
	if (repair_movement->license_number!=NULL) g_free(repair_movement->license_number);
	g_free(repair_movement);
    }
    return 0;
}

/**
 * d_repair_movement_set_enterprise_code:
 * @repair_movement: Objeto en el que se establece el atributo.
 * @enterprise_code: Atributo a insertar.
 *
 * Establece el atributo @enterprise_code en el objeto. El atributo @enterprise_code
 * indica el cdigo de la empresa en la que se realiza la regularizacin del stock.
 *
 */
void d_repair_movement_set_enterprise_code (RepairMovement *repair_movement, gulong enterprise_code) {
    repair_movement->enterprise_code = enterprise_code;
}

/**
 * d_repair_movement_set_warehouse_code:
 * @repair_movement: Objeto en el que se establece el atributo.
 * @warehouse_code: Atributo a insertar.
 *
 * Establece el atributo @warehouse_code en el objeto. El atributo @warehouse_code
 * indica el cdigo del almacn en el que se realizar la regularizacin del stock.
 *
 */
void d_repair_movement_set_warehouse_code (RepairMovement *repair_movement, gulong warehouse_code) {
    repair_movement->warehouse_code = warehouse_code;
}

/**
 * d_repair_movement_set_quantity:
 * @repair_movement: Objeto en el que se establece el atributo.
 * @quantity: Atributo a insertar.
 *
 * Establece el atributo @quantity en el objeto. El atributo @quantity
 * indica la unidades que se regularizarn.
 *
 */
void d_repair_movement_set_quantity (RepairMovement *repair_movement, gdouble quantity) {
    repair_movement->quantity = quantity;
}

/**
 * d_repair_movement_set_movement_code:
 * @repair_movement: Objeto en el que se establece el atributo.
 * @movement_code: Atributo a insertar.
 *
 * Establece el atributo @movement_code en el objeto. El atributo @movement_code
 * indica el cdigo del movimiento provocado por el albarn.
 *
 */
void d_repair_movement_set_movement_code (RepairMovement *repair_movement, gulong movement_code) {
    repair_movement->movement_code=movement_code;
}
/**
 * d_repair_movement_set_repair_code:
 * @repair_movement: Objeto en el que se establece el atributo.
 * @repair_code: Atributo a insertar.
 *
 * Establece el atributo @repair_code en el objeto. El atributo @repair_code
 * indica el cdigo de la reparacin que provoca el movimiento.
 *
 */
void d_repair_movement_set_repair_code (RepairMovement *repair_movement, gulong repair_code) {
    repair_movement->repair_code=repair_code;
}
/**
 * d_repair_movement_set_repair_enterprise_code:
 * @repair_movement: Objeto en el que se establece el atributo.
 * @repair_enterprise_code: Atributo a insertar.
 *
 * Establece el atributo @repair_enterprise_code en el objeto. El atributo @repair_enterprise_code
 * indica el cdigo de la empresa a la que pertenece la tienda en la que se realiz la reparacin.
 *
 */
void d_repair_movement_set_repair_enterprise_code (RepairMovement *repair_movement, gulong repair_enterprise_code) {
    repair_movement->repair_enterprise_code=repair_enterprise_code;
}
/**
 * d_repair_movement_set_line_number:
 * @repair_movement: Objeto en el que se establece el atributo.
 * @line_number: Atributo a insertar.
 *
 * Establece el atributo @line_number en el objeto. El atributo @line_number
 * indica el nmero de la lnea del pedido que ocupa la pieza que provoc el movimiento.
 *
 */
void d_repair_movement_set_line_number (RepairMovement *repair_movement, gulong line_number) {
    repair_movement->line_number=line_number;

}
/**
 * d_repair_movement_set_shop_code:
 * @repair_movement: Objeto en el que se establece el atributo.
 * @shop_code: Atributo a insertar.
 *
 * Establece el atributo @shop_code en el objeto. El atributo @shop_code
 * indica el cdigo de la tienda en la que se realiz la reparacin.
 *
 */
void d_repair_movement_set_shop_code (RepairMovement *repair_movement, gulong shop_code) {
    repair_movement->shop_code=shop_code;
}

/**
 * d_repair_movement_set_product_code:
 * @repair_movement: Objeto en el que se establece el atributo.
 * @product_code: Atributo a insertar.
 *
 * Establece el atributo @product_code en el objeto. El atributo @product_code
 * indica el cdigo de la tienda en la que se realiz la reparacin.
 *
 */
void d_repair_movement_set_product_code (RepairMovement *repair_movement, gulong product_code) {
    repair_movement->product_code=product_code;
}

/**
 * d_repair_movement_set_license_number:
 * @repair_movement: Objeto en el que se establece el atributo.
 * @license_number: Atributo a insertar.
 *
 * Establece el atributo @license_number en el objeto. El atributo @license_number
 * indica la matrcula del vehculo reparado.
 *
 */
void d_repair_movement_set_license_number (RepairMovement *repair_movement, gchar* license_number) {
    repair_movement->license_number=g_strdup(license_number);
}

/**
 * d_repair_movement_get_enterprise_code:
 * @repair_movement: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_repair_movement_get_enterprise_code (RepairMovement *repair_movement) {
    return repair_movement->enterprise_code;
}

/**
 * d_repair_movement_get_warehouse_code:
 * @repair_movement: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_repair_movement_get_warehouse_code (RepairMovement *repair_movement) {
    return repair_movement->warehouse_code;
}

/**
 * d_repair_movement_get_quantity:
 * @repair_movement: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gdouble d_repair_movement_get_quantity (RepairMovement *repair_movement) {
    return repair_movement->quantity;
}

/**
 * d_repair_movement_get_date:
 * @repair_movement: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
time_t d_repair_movement_get_date (RepairMovement *repair_movement) {
    return repair_movement->date;
}


/**
 * d_repair_movement_get_movement_code:
 * @repair_movement: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_repair_movement_get_movement_code (RepairMovement *repair_movement) {
    return(repair_movement->movement_code);
}

/**
 * d_repair_movement_get_repair_code:
 * @repair_movement: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_repair_movement_get_repair_code (RepairMovement *repair_movement) {
    return(repair_movement->repair_code);
}

/**
 * d_repair_movement_get_line_number:
 * @repair_movement: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_repair_movement_get_line_number (RepairMovement *repair_movement) {
    return repair_movement->line_number;
}

/**
 * d_repair_movement_get_repair_enterprise_code:
 * @repair_movement: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_repair_movement_get_repair_enterprise_code (RepairMovement *repair_movement) {
    return repair_movement->repair_enterprise_code;
}

/**
 * d_repair_movement_get_shop_code:
 * @repair_movement: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_repair_movement_get_shop_code (RepairMovement *repair_movement) {
    return repair_movement->shop_code;
}

/**
 * d_repair_movement_get_product_code:
 * @repair_movement: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_repair_movement_get_product_code (RepairMovement *repair_movement) {
    return repair_movement->product_code;
}

/**
 * d_repair_movement_get_license_number:
 * @repair_movement: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar * d_repair_movement_get_license_number (RepairMovement *repair_movement) {
    return g_strdup(repair_movement->license_number);
}


/* Private functions*/

static gint repair_movement_init(RepairMovement *repair_movement) {
    repair_movement->enterprise_code = ULONG_MAX;
    repair_movement->warehouse_code = ULONG_MAX;
    repair_movement->movement_code = ULONG_MAX;
    repair_movement->product_code = ULONG_MAX;
    repair_movement->license_number = NULL;
    repair_movement->repair_code = ULONG_MAX;
    repair_movement->repair_enterprise_code = ULONG_MAX;
    repair_movement->shop_code = ULONG_MAX;
    repair_movement->line_number = ULONG_MAX;
    repair_movement->quantity = G_MINDOUBLE;
    repair_movement->date = time(NULL);

    return 0;
}


