
/* repair_object.h
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#ifndef _REPAIR_OBJECT_
#define _REPAIR_OBJECT_

#ifndef _GLIB_LIBRARY_
#define _GLIB_LIBRARY_
#include <glib.h>
#endif

#include "company.h"
#include "enterprise.h"
#include "participant.h"
#include "accident.h"
#include "repair_line_object.h"
#include "garage.h"
#include "shop.h"
#include "vehicle.h"
#include "receipt.h"
#include "policy.h"
#include "client.h"
#include "agent.h"
#include "util.h"

#include "time.h"

typedef struct _repair {
  gulong repair_code;
  time_t in_date;
  time_t out_date;
  gchar *observations;
  gulong mobile_unit_distance;
  gchar *mobile_unit_place;
  gdouble total;
  Enterprise *enterprise;
  Shop *shop;   
  Garage *garage;
  Company *company;
  Vehicle *vehicle;
  Receipt *receipt;
  Policy *policy;
  Accident *accident;
  Client *client;
  Repair_line **repair_line;
  gboolean completed;
  Agent * agent;
} Repair;

Repair *d_repair_create_empty (void);
gint d_repair_clear (Repair * repair);
void d_repair_set_repair_code (Repair *repair,gulong repair_code);
void d_repair_set_in_date (Repair *repair,time_t in_date);
void d_repair_set_out_date (Repair *repair,time_t out_date);
void d_repair_set_observations (Repair *repair,gchar *observations);
void d_repair_set_mobile_unit_place (Repair *repair,gchar *mobile_unit_place);
void d_repair_set_mobile_unit_distance (Repair *repair,gulong mobile_unit_distance);
void d_repair_set_enterprise (Repair *repair,Enterprise *enterprise);
void d_repair_set_shop (Repair *repair,Shop *shop);
void d_repair_set_garage (Repair *repair,Garage *garage);
void d_repair_set_company (Repair *repair,Company *company);
void d_repair_set_receipt (Repair *repair,Receipt *receipt);
void d_repair_set_policy (Repair *repair,Policy *policy);
void d_repair_set_vehicle (Repair *repair,Vehicle *vehicle);
void d_repair_set_accident (Repair *repair,Accident *accident);
void d_repair_set_agent (Repair *repair,Agent *agent);
void d_repair_set_client (Repair *repair,Client *client);
void d_repair_set_repair_lines (Repair *repair,Repair_line **repair_line);
void d_repair_set_total (Repair *repair,gdouble total);
gulong d_repair_get_repair_code (Repair *repair);
time_t d_repair_get_in_date (Repair *repair);
time_t d_repair_get_out_date (Repair *repair);
gchar *d_repair_get_observations (Repair *repair);
gchar *d_repair_get_mobile_unit_place (Repair *repair);
gulong d_repair_get_mobile_unit_distance (Repair *repair);
Enterprise *d_repair_get_enterprise (Repair *repair);
Shop *d_repair_get_shop (Repair *repair);
Garage *d_repair_get_garage (Repair *repair);
Company *d_repair_get_company (Repair *repair);
Receipt *d_repair_get_receipt (Repair *repair);
Policy *d_repair_get_policy (Repair *repair);
Vehicle *d_repair_get_vehicle (Repair *repair);
Accident *d_repair_get_accident (Repair *repair);
Agent *d_repair_get_agent (Repair *repair);
Client *d_repair_get_client (Repair *repair);
Repair_line **d_repair_get_repair_lines (Repair *repair);
gdouble d_repair_get_total (Repair *repair);
gboolean d_repair_is_with_insurance (Repair * repair);
void d_repair_unlink (Repair *repair);

#endif

