/* vehicle.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "vehicle.h"


static gint vehicle_init(Vehicle *vehicle);


/**
 * d_vehicle_create_empty:
 *
 * Crea un objeto vaco, con memoria reservada e inicializado.
 *
 * Returns:
 * Un objeto #Vehicle vaco con los campos inicializados a un valor de inicializacin segn el tipo.
 */
Vehicle *d_vehicle_create_empty () {
    Vehicle * vehicle = (Vehicle *) g_malloc(sizeof(Vehicle));

    vehicle_init(vehicle);

    return vehicle;
}


/**
 * d_vehicle_clear:
 * @vehicle: Objeto a liberar.
 *
 * Libera la memoria utilizada por el objeto.
 *
 * Returns:
 * 0 si se ha acabado la funcin correctamente.
 */
gint d_vehicle_clear (Vehicle *vehicle) {

    if (vehicle!=NULL) {
	if (vehicle->license_number!=NULL)
	    g_free(vehicle->license_number);
	if (vehicle->chassis_number!=NULL)
	    g_free(vehicle->chassis_number);
	if (vehicle->model!=NULL)
	    g_free(vehicle->model);
	if (vehicle->make!=NULL)
	    g_free(vehicle->make);
	if (vehicle->colour!=NULL)
	    g_free(vehicle->colour);

	g_free(vehicle);
    }

    return 0;
}


/**
 * d_vehicle_set_code:
 * @vehicle: Objeto en el que se establece el atributo.
 * @vehicle_code: Atributo a insertar.
 *
 * Establece el atributo @vehicle_code en el objeto. El atributo @vehicle_code
 * indica cdigo interno que identifica un vehculo.
 *
 */
void d_vehicle_set_code (Vehicle * vehicle,gulong vehicle_code) {
    vehicle->vehicle_code = vehicle_code;
}

/**
 * d_vehicle_set_license_number:
 * @vehicle: Objeto en el que se establece el atributo.
 * @license_number: Atributo a insertar.
 *
 * Establece el atributo @license_number en el objeto. El atributo @license_number
 * indica la matrcula del vehculo.
 *
 */
void d_vehicle_set_license_number (Vehicle *vehicle, gchar *license_number){
    if (vehicle->license_number  != NULL) g_free(vehicle->license_number);
    vehicle->license_number = g_strdup(license_number);
}

/**
 * d_vehicle_set_chassis_number:
 * @vehicle: Objeto en el que se establece el atributo.
 * @chassis_number: Atributo a insertar.
 *
 * Establece el atributo @chassis_number en el objeto. El atributo @chassis_number
 * indica el nmero de bastidor del vehculo.
 *
 */
void d_vehicle_set_chassis_number (Vehicle *vehicle, gchar *chassis_number){
    if (vehicle->chassis_number  != NULL) g_free(vehicle->chassis_number);
    vehicle->chassis_number = g_strdup(chassis_number);
}

/**
 * d_vehicle_set_kilometres:
 * @vehicle: Objeto en el que se establece el atributo.
 * @kilometres: Atributo a insertar.
 *
 * Establece el atributo @kilometres en el objeto. El atributo @kilometres
 * indica los kilmetros actuales del vehculo.
 *
 */
void d_vehicle_set_kilometres (Vehicle *vehicle, gulong kilometres){
    vehicle->kilometres = kilometres;
}

/**
 * d_vehicle_set_model:
 * @vehicle: Objeto en el que se establece el atributo.
 * @model: Atributo a insertar.
 *
 * Establece el atributo @model en el objeto. El atributo @model
 * indica el modelo del vehculo.
 *
 */
void d_vehicle_set_model (Vehicle *vehicle, gchar *model){
    if (vehicle->model  != NULL) g_free(vehicle->model);
    vehicle->model = g_strdup (model);
}

/**
 * d_vehicle_set_make:
 * @vehicle: Objeto en el que se establece el atributo.
 * @make: Atributo a insertar.
 *
 * Establece el atributo @make en el objeto. El atributo @make
 * indica la marca del vehculo.
 *
 */
void d_vehicle_set_make (Vehicle *vehicle, gchar *make){
    if (vehicle->make  != NULL) g_free(vehicle->make);
    vehicle->make = g_strdup(make);
}

/**
 * d_vehicle_set_colour:
 * @vehicle: Objeto en el que se establece el atributo.
 * @colour: Atributo a insertar.
 *
 * Establece el atributo @colour en el objeto. El atributo @colour
 * indica el color del vehculo.
 *
 */
void d_vehicle_set_colour (Vehicle *vehicle, gchar *colour){
    if (vehicle->colour  != NULL) g_free(vehicle->colour);
    vehicle->colour = g_strdup(colour);
}

/**
 * d_vehicle_get_code:
 * @vehicle: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_vehicle_get_code (Vehicle * vehicle) {
    return vehicle->vehicle_code;
}

/**
 * d_vehicle_get_license_number:
 * @vehicle: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar *d_vehicle_get_license_number (Vehicle *vehicle){
    return g_strdup(vehicle->license_number);
}

/**
 * d_vehicle_get_chassis_number:
 * @vehicle: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar *d_vehicle_get_chassis_number (Vehicle *vehicle){
    return g_strdup(vehicle->chassis_number);
}

/**
 * d_vehicle_get_kilometres:
 * @vehicle: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_vehicle_get_kilometres (Vehicle *vehicle){
    return vehicle->kilometres;
}

/**
 * d_vehicle_get_model:
 * @vehicle: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar *d_vehicle_get_model (Vehicle *vehicle){
    return g_strdup(vehicle->model);
}

/**
 * d_vehicle_get_make:
 * @vehicle: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar *d_vehicle_get_make (Vehicle *vehicle){
    return g_strdup(vehicle->make);
}

/**
 * d_vehicle_get_:
 * @vehicle: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar *d_vehicle_get_colour (Vehicle *vehicle){
    return g_strdup(vehicle->colour);
}


/**
 * d_vehicle_duplicate:
 * @vehicle: Objeto del que se desea obtener el duplicado.
 *
 * Duplica el objeto.
 *
 * Returns: Un objeto con todos los campos duplicados y con nueva memoria reservada.
 */
Vehicle *d_vehicle_duplicate (Vehicle *vehicle) {

    Vehicle *result = d_vehicle_create_empty ();
    gulong vehicle_code = d_vehicle_get_code (vehicle);
    gchar *license_number = d_vehicle_get_license_number (vehicle);
    gchar *chassis_number = d_vehicle_get_chassis_number (vehicle);
    gchar *model = d_vehicle_get_model (vehicle);
    gchar *make = d_vehicle_get_make (vehicle);
    gchar *colour = d_vehicle_get_colour (vehicle);
    gulong kms = d_vehicle_get_kilometres (vehicle);

    d_vehicle_set_code (result,vehicle_code);
    d_vehicle_set_license_number (result, license_number);
    d_vehicle_set_chassis_number (result, chassis_number);
    d_vehicle_set_kilometres (result, kms);
    d_vehicle_set_model (result, model);
    d_vehicle_set_make (result, make);
    d_vehicle_set_colour (result, colour);

    g_free(license_number);
    g_free(chassis_number);
    g_free(make);
    g_free(model);
    g_free(colour);

    return result;
}


/* Private functions*/

static gint vehicle_init(Vehicle *vehicle)
{
  vehicle->vehicle_code = ULONG_MAX;
  vehicle->license_number = NULL;
  vehicle->chassis_number = NULL;
  vehicle->kilometres = ULONG_MAX;
  vehicle->model = NULL;
  vehicle->make = NULL;
  vehicle->colour = NULL;
  
    return 0;
}
