/* delivery_payment_movement_model.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */
#include "delivery_payment_movement_model.h"

#define TABLE      "MovimientoPorAlbaran"


/**
 * m_delivery_payment_movement_get_table:
 *
 * Devuelve un string con la tabla correspondiente al objeto de la base de datos. Si el
 * objeto se corresponde con varias tablas se devolveran los nombres separados por comas.
 *
 * Returns:
 * Cadena de caracteres con la tabla o lista de tablas a las que se refiere el objeto.
 */
gchar *m_delivery_payment_movement_get_table () {
    return TABLE;
}

/**
 * m_delivery_payment_movement_create:
 * @rs: Record set que contiene los campos de la BD.
 *
 * Crea un objeto a partir de los valores de un record set de la base de datos.
 *
 * Returns:
 * Un objeto #DeliveryPaymentMovement con los campos rellenados con los datos de la base de datos.
 */
DeliveryPaymentMovement  *m_delivery_payment_movement_create (GdaRecordset *rs){
    DeliveryPaymentMovement * result=d_delivery_payment_movement_create_empty ();
    gchar  *temp;
    GdaField * field = NULL;
    

    field = gda_recordset_field_name(rs,"codigoempresaMovimiento");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    result->enterprise_code = igalia_field_get_bigint_value (field);
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs,"codigoalmacenMovimiento");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    result->warehouse_code = igalia_field_get_bigint_value (field);
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs,"codigoMovimiento");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    result->movement_code = igalia_field_get_bigint_value (field);
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs,"unidadesMovimiento");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    result->quantity = u_round_to_decimal (igalia_field_get_double_value (field),3);
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs,"fechaMovimiento");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    time_t * my_time;
	    temp=gda_stringify_value(NULL, 0, field);
	    my_time = u_parse_date (temp);
	    result->date = (*my_time);
	    g_free(my_time);
	    my_time = NULL;
	    g_free(temp);
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs, "numeroAlbaranMovimiento");
    if (field!=NULL){
	if (!gda_field_is_null(field)) {
	    temp = gda_stringify_value(NULL, 0, field);
	    result->delivery_payment_number = g_strdup(temp);
	    g_free(temp);
	    result -> pending = FALSE;
	} else {
	    result -> pending = TRUE;
	}
	gda_field_free(field);
    } else {
        result -> pending = TRUE;
    }


    field = gda_recordset_field_name(rs,"codigoProveedorMovimiento");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    result-> supplier_code= igalia_field_get_bigint_value (field);
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs,"codigoPedidoMovimiento");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    result-> order_code=    igalia_field_get_bigint_value (field);
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs,"numeroLineaPedidoMovimiento");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    result-> order_line_number=     igalia_field_get_bigint_value (field);
	}
	gda_field_free(field);
    }

/*    field = gda_recordset_field_name(rs,"numeroAlbaranMovimiento");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    temp=gda_stringify_value(NULL, 0, field);
	    result->delivery_payment_number = g_strdup(temp);
	    g_free(temp);
	}
	gda_field_free(field);
    }
*/
    field = gda_recordset_field_name(rs,"codigoProductoMovimiento");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    result-> product_code=igalia_field_get_bigint_value (field)  ;
	}
	gda_field_free(field);
    }

    return result;

}


