/* listing_operation_model.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include "listing_operation_model.h"
#include "invoice.h"

/**
 * m_listing_operation_create:
 * @rs: Record set que contiene los campos de la BD.
 *
 * Crea un objeto a partir de los valores de un record set de la base de datos.
 *
 * Returns:
 * Un objeto #ListingOperation con los campos rellenados con los datos de la base de datos.
 */
ListingOperation *m_listing_operation_create (GdaRecordset *rs) {
    GdaField *field;
    gchar    *temp;
    ListingOperation *listing_operation = d_listing_operation_create_empty();
    field = gda_recordset_field_name(rs,"codigoMovListingOperation");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    listing_operation->operation_code = igalia_field_get_bigint_value (field);
    	}
        gda_field_free(field);
    }

    field = gda_recordset_field_name(rs,"codigoBancoListingOperation");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    listing_operation->bank_code = gda_field_get_smallint_value(field);
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs,"codigoOficinaListingOperation");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    listing_operation->office_code = gda_field_get_smallint_value(field);
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs,"companiaListingOperation");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    temp=gda_stringify_value(NULL, 0, field);
	    listing_operation->company_name = g_strdup(temp);
	    g_free(temp);
	}
	gda_field_free(field);
    }
    field = gda_recordset_field_name(rs,"codigoCompaniaListingOperation");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    listing_operation->company_code = igalia_field_get_bigint_value (field);
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs,"numeroCuentaListingOperation");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    temp=gda_stringify_value(NULL, 0, field);
	    listing_operation->account_number = g_strdup(temp);
	    g_free(temp);
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs,"numeroListingOperation");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    temp=gda_stringify_value(NULL, 0, field);
	    listing_operation->number = g_strdup(temp);
	    g_free(temp);
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs,"referenciaReciboListingOperation");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    temp=gda_stringify_value(NULL, 0, field);
	    listing_operation->receipt_reference = g_strdup(temp);
	    g_free(temp);
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs,"fechaListingOperation");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    gchar * temp;
	    time_t * my_time;
	    temp=gda_stringify_value(NULL, 0, field);
	    my_time = u_parse_date (temp);
	    listing_operation->date = (*my_time);
	    g_free(my_time);

	    my_time = NULL;
	    g_free(temp);
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs,"fechaNotListingOperation");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    gchar * temp;
	    time_t * my_time;
	    temp=gda_stringify_value(NULL, 0, field);
	    my_time = u_parse_date (temp);
	    listing_operation->notification_date = (*my_time);
	    g_free(my_time);

	    my_time = NULL;
	    g_free(temp);
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs,"cantidadListingOperation");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    temp=gda_stringify_value(NULL, 0, field);
	    listing_operation->amount = atof(temp);
	    g_free(temp);
	}
	gda_field_free(field);
    }

    return listing_operation;
}


/**
 * model_find_listing_operations_by_condition 
 * @condition: Filtro de bsqueda. Permite buscar por tipo de pago, cdigo de la tienda donde se realiz
 * el pago, cdigo de la compaa de seguros que realiz el pago, estado de la factura que contiene ese pago,
 * perodo de fechas y cuenta corriente.
 * 
 * Busca un listado de pagos por una serie de condiciones.
 *
 * Returns:
 * Lista de pagos que cumplan la condicin de bsqueda.
 */
ListingOperation ** m_listing_operation_find_by_condition (ConditionFindListingOperation * condition,gint type) {
    GdaCommand         *my_command;
    gchar              *gchar_query=NULL;
    gchar              *gchar_select_no_grouped=NULL;
    gchar              *gchar_select=NULL;
    gchar              *gchar_select_type=NULL;
    gchar              *gchar_select_type_no_grouped=NULL;
    gchar              *gchar_select_aux=NULL;
    gchar              *gchar_where=NULL;
    gchar              *gchar_where_grouped=NULL;
    gchar              *gchar_where_no_grouped=NULL;
    gchar              *gchar_where_aux=NULL;
    gchar              *gchar_from=NULL;
    gchar              *gchar_groupby=NULL;
    gchar              *gchar_groupby_type=NULL;
    gchar              *gchar_groupby_aux=NULL;
    GdaConnection      *my_connection = m_model_get_connection ();
    GdaRecordset       *record_set1 = NULL;
    gulong index_of_listing_operations=0;

    ListingOperation ** listing_operations=NULL;
    ListingOperation * listing_operation;
    gint index;
    gulong * tuples1;

    my_command = gda_command_new();
    tuples1 = (gulong *) g_malloc0(sizeof(gulong));
    gda_command_set_connection(my_command,my_connection);

    switch (type) {
    case M_LISTING_OPERATION_INCOMING:{
	gchar_select_type = " reparacion.codigoCompania as codigoCompaniaListingOperation, ";
	gchar_groupby_type = "reparacion.codigoCompania,";
	gchar_select_type_no_grouped = " tienda.codigoPertenenciaGrupo||' ' ||facturaabono.numeroFactura as numeroListingOperation,";
	gchar_from = g_strconcat(" from ",
				 " ((select ",
				 " codigoReparacion,numeroFactura from albaranabono where ",
				 " codigoReparacion is not null) union (select codigoReparacion,codigoFactura from facturaabono where codigoReparacion is not null)) as ",
				 " codigosReparacion,reparacion left outer join participante on ",
				 " participante.codigoParty=reparacion.codigoCompania,tienda,facturaabono,PAGO,movimiento ",
				 NULL);
	gchar_where=g_strconcat("  where ",
				" reparacion.codigoReparacion=codigosReparacion.codigoReparacion and ",
				" facturaabono.codigoFactura = codigosReparacion.numeroFactura AND ",
                                " tienda.codigoTienda = reparacion.codigoTienda and ",
				" PAGO.CODIGOPAGO=FACTURAABONO.CODIGOPAGO and pago.codigoTienda=facturaabono.codigoTienda ",
				" AND pago.codigoEMpresa=facturaabono.codigoEMpresa and movimiento.codigoPago=pago.codigoPago ",
				NULL);

	if (((condition->shop_code) != ULONG_MAX) && ((condition->shop_code) != 0)) {
	    gchar * temp = g_strdup_printf("%lu",condition->shop_code);
	    gchar_where_aux = g_strconcat(gchar_where, " and (facturaabono.codigoTienda=",
					  temp,")",NULL);
	    g_free(gchar_where);
	    g_free(temp);
	    gchar_where = gchar_where_aux;
	    gchar_where_aux=NULL;
	}
	if (((condition->company_code) != ULONG_MAX) && ((condition->company_code) != 0)) {
	    gchar * temp = g_strdup_printf("%lu",condition->company_code);
	    gchar_where_aux = g_strconcat(gchar_where, " and (reparacion.codigoCompania=",
				      temp,")",NULL);
	    g_free(gchar_where);
	    g_free(temp);
	    gchar_where = gchar_where_aux;
	    gchar_where_aux=NULL;
	} else if ((condition->company_code) == 0) {
	    gchar_where_aux = g_strconcat(gchar_where, " and (reparacion.codigoCompania is NULL)",
					  NULL);
	    g_free(gchar_where);
	    gchar_where = gchar_where_aux;
	    gchar_where_aux=NULL;
	}
	if ((condition->invoice_state) >= 0)  {
	    gchar * temp = g_strdup_printf("%d",condition->invoice_state);
	    gchar_where_aux = g_strconcat(gchar_where, " and (facturaabono.estado=",
					  temp,")",NULL);
	    g_free(gchar_where);
	    g_free(temp);
	    gchar_where = gchar_where_aux;
	    gchar_where_aux=NULL;
	}

	break;
    }
    case M_LISTING_OPERATION_EXPENSE:{
	gchar_select_type = " facturacargo.codigoProveedor as codigoCompaniaListingOperation, ";
	gchar_select_type_no_grouped = " facturacargo.numeroFactura as numeroListingOperation,";
	gchar_groupby_type = "facturacargo.codigoProveedor,";
	gchar_from = g_strconcat(" from ",
				 " facturacargo left outer join participante on ",
				 " participante.codigoParty=facturacargo.codigoProveedor,PAGO,movimiento ",
				 NULL);
	gchar_where=g_strconcat("  where ",
				" PAGO.CODIGOPAGO=FACTURACARGO.CODIGOPAGO and pago.codigoTienda=facturacargo.codigoTienda ",
				" AND pago.codigoEMpresa=facturacargo.codigoEMpresa and movimiento.codigoPago=pago.codigoPago ",
				NULL);
	if (((condition->shop_code) != ULONG_MAX) && ((condition->shop_code) != 0)) {
	    gchar * temp = g_strdup_printf("%lu",condition->shop_code);
	    gchar_where_aux = g_strconcat(gchar_where, " and (facturacargo.codigoTienda=",
					  temp,")",NULL);
	    g_free(gchar_where);
	    g_free(temp);
	    gchar_where = gchar_where_aux;
	    gchar_where_aux=NULL;
	}
	if (((condition->company_code) != ULONG_MAX) && ((condition->company_code) != 0)) {
	    gchar * temp = g_strdup_printf("%lu",condition->company_code);
	    gchar_where_aux = g_strconcat(gchar_where, " and (facturacargo.codigoProveedor=",
					  temp,")",NULL);
	    g_free(gchar_where);
	    g_free(temp);
	    gchar_where = gchar_where_aux;
	    gchar_where_aux=NULL;
	}
	if ((condition->invoice_state) >= 0)  {
	    gchar * temp = g_strdup_printf("%d",condition->invoice_state);
	    gchar_where_aux = g_strconcat(gchar_where, " and (facturacargo.estado=",
					  temp,")",NULL);
	    g_free(gchar_where);
	    g_free(temp);
	    gchar_where = gchar_where_aux;
	    gchar_where_aux=NULL;
	}

	break;
    }
    case M_LISTING_OPERATION_BOTH:{
        break;
    }

    }

    gchar_select = g_strconcat("select ",
			       "movimiento.codigomovimiento as codigoMovListingOperation, ",
			       "movimiento.codigobanco as codigoBancoListingOperation, ",
			       "movimiento.codigooficina as codigoOficinaListingOperation, "
			       "movimiento.numerocuenta as numerocuentaListingOperation, ",
			       " movimiento.referenciarecibo,to_char(sum(movimiento.cantidad),'99999999D99') as ",
			       "cantidadListingOperation, ",
                               gchar_select_type,
			       "NULL as numeroListingOperation, participante.nombreCorto as companiaListingOperation, ",
			       "movimiento.fecha as fechaListingOperation, ",
			       "movimiento.fechanotificacion as fechanotListingOperation "
			       ,NULL );

    gchar_select_no_grouped = g_strconcat("select ",
					  "movimiento.codigomovimiento as codigoMovListingOperation, ",
					  "movimiento.codigobanco as codigoBancoListingOperation, ",
					  "movimiento.codigooficina as codigoOficinaListingOperation, "
					  "movimiento.numerocuenta as numerocuentaListingOperation, ",
					  " movimiento.referenciarecibo,to_char(movimiento.cantidad,'99999999D99') as ",
					  "cantidadListingOperation, ",
                                          gchar_select_type,
                                          gchar_select_type_no_grouped,
					  " participante.nombreCorto as companiaListingOperation, ",
					  "movimiento.fecha as fechaListingOperation, ",
					  "movimiento.fechanotificacion as fechanotListingOperation "
					  ,NULL );




    gchar_where_grouped=g_strconcat("  AND  ",
				    " movimiento.referenciaRecibo is not NULL and movimiento.referenciaRecibo!='' ",NULL);
    gchar_where_no_grouped=g_strconcat("  AND  ",
				       " (movimiento.referenciaRecibo is NULL or movimiento.referenciaRecibo='') ",NULL);

    gchar_groupby=g_strconcat("  group by ",
			      " movimiento.codigomovimiento,movimiento.codigoBanco,movimiento.codigooficina, ",
			      " movimiento.numeroCuenta, ",
			      " movimiento.referenciarecibo,",gchar_groupby_type,"participante.nombreCorto, ",
			      " movimiento.fecha,movimiento.fechanotificacion ",NULL);

    if ((condition->initial_done_date)!= NULL)  {
	gchar * temp = ctime(condition->initial_done_date);
	gchar_where_aux = g_strconcat(gchar_where, " and (movimiento.fecha<='",
				      temp,"')",NULL);
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;
    }
    if ((condition->ending_done_date)!= NULL)  {
	gchar * temp = ctime(condition->ending_done_date);
	gchar_where_aux = g_strconcat(gchar_where, " and (movimiento.fecha>='",
				      temp,"')",NULL);
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;
    }
    if ((condition->amount)!= G_MINDOUBLE)  {
	gchar * temp = g_strdup_printf("%.2f",condition->amount);
	u_commas_to_points (temp);
	gchar_where_aux = g_strconcat(gchar_where, " and (movimiento.cantidad=CAST(",
				      temp," as DECIMAL(10,2)))",NULL);
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;
    }


    if (((condition->bank_code) != NULL) && (strlen(condition->bank_code) != 0)) {
	gchar_where_aux = g_strconcat(gchar_where, " and (movimiento.codigoBanco=",
				      condition->bank_code,")",NULL);
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;
    }
    if (((condition->office_code) != NULL) && (strlen(condition->office_code) != 0)) {
	gchar_where_aux = g_strconcat(gchar_where, " and (movimiento.codigoOficina=",
				      condition->office_code,")",NULL);
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;
    }
    if (((condition->account_number) != NULL) && (strlen(condition->account_number) != 0)) {
	gchar_where_aux = g_strconcat(gchar_where, " and (movimiento.numeroCuenta='",
				      condition->account_number,"')",NULL);
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;
    }

    if ((condition->type)!= ULONG_MAX)  {
	gchar * conditionPayment;
	if (condition->type==D_PAYMENT_TRANSFER_CODE) {
	    conditionPayment = g_strconcat(" and (pago.esTransferencia=true) ",NULL);
	    gchar_select_aux=g_strdup(" ");
	    gchar_groupby_aux=g_strdup(" union ");
	} else if (condition->type==D_PAYMENT_DIRECT_DEBIT_CODE) {
	    conditionPayment = g_strconcat(" and (pago.esDomiciliacion=true)",NULL);
	    gchar_select_aux=g_strdup(" ");
	    gchar_groupby_aux=g_strdup(" union ");
	} else if (condition->type==D_PAYMENT_CREDIT_CARD) {
	    conditionPayment = g_strconcat(" and (pago.esTarjeta=true)",NULL);
	    gchar_select_aux=g_strdup(" ,pago.numeroTarjeta as numeroListingOperation");
	    gchar_groupby_aux=g_strdup(" ,pago.numeroTarjeta union ");
	} else if (condition->type==D_PAYMENT_CHEQUE) {
	    conditionPayment = g_strconcat(" and (pago.esTalon=true)",NULL);
	    gchar_select_aux=g_strdup(" ,pago.numeroTalon as numeroListingOperation");
	    gchar_groupby_aux=g_strdup(" ,pago.numeroTalon union ");
	} else {
	    conditionPayment = g_strconcat(" ",NULL);
	    gchar_select_aux=g_strdup(" ");
	    gchar_groupby_aux=g_strdup(" union ");
	}
	gchar_where_aux = g_strconcat(gchar_where,conditionPayment,NULL);
	g_free(conditionPayment);
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;
    } else {
	gchar_select_aux=g_strdup(" ");
	gchar_groupby_aux=g_strdup(" union ");

    }


    gchar_query = g_strconcat(gchar_select,gchar_select_aux,gchar_from,gchar_where,gchar_where_grouped,gchar_groupby,gchar_groupby_aux,
			      gchar_select_no_grouped,gchar_select_aux,gchar_from,gchar_where,gchar_where_no_grouped,
			      NULL);


    g_print(_("\nQuery: %s\n"),gchar_query);

    m_model_command_execute (&record_set1, gchar_query, my_connection, my_command,tuples1,0, TRUE);



    if (record_set1!=NULL) {
	if (*tuples1!=0) {
	    listing_operations = (ListingOperation **) g_malloc0(sizeof(ListingOperation*)*(*tuples1+1));
	    index = gda_recordset_move_first(record_set1);
	    while (!gda_recordset_eof(record_set1)) {
		if (index==GDA_RECORDSET_INVALID_POSITION) return NULL;
		listing_operation = m_listing_operation_create (record_set1);
		listing_operations[index_of_listing_operations] = listing_operation;
		index = gda_recordset_move_next(record_set1);
		index_of_listing_operations ++;
	    }
	} else {
	    listing_operations=NULL;
	}
	gda_recordset_close(record_set1);
    } else {
	listing_operations=NULL;
    }

    if (listing_operations!=NULL) {
	listing_operations[index_of_listing_operations] = NULL;
    }

    g_free(gchar_where);
    g_free(gchar_select_aux);
    g_free(gchar_select);
    g_free(gchar_from);
    g_free(gchar_groupby);
    g_free(gchar_groupby_aux);
    g_free(gchar_query);
    g_free(tuples1);



    igalia_command_free(my_command);
    g_print(_("Freeing database completed\n"));

    return listing_operations;

}

/**
 * m_listing_operation_clear_array:
 * @listing_operations: Array de pagos
 * 
 * Libera la memoria utilizada por el array de pagos.
 *
 */
void m_listing_operation_clear_array (ListingOperation ** listing_operations) {
    if (listing_operations!=NULL) {
	gint index = 0;
	while (listing_operations[index]!=NULL) {
	    d_listing_operation_clear (listing_operations[index]);
	    index++;
	}
    }
    g_free(listing_operations);
    listing_operations=NULL;
}



