/* order_line_model.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include "gda_igalia_util.h"
#include "order_line_model.h"
#include "piece_model.h"
#include "product_model.h"
#include "find_pieces_model.h"
#include "find_stocked_pieces_model.h"
#include "util.h"
#include "stock.h"
#include "stock_model.h"
#include "parameter_manager.h"
#include "enterprise_model.h"
#include "shop_model.h"
#include "warehouse_model.h"

#define TABLE         "LineaPedido"
#define ATTRIBUTES    "codigoPedido as codigopedidoorderline, \
    codigoEmpresa as codigoempresaorderline, \
    codigoAlmacen as codigoalmacenorderline, \
    numeroLinea as numeroLineaorderline, \
    numeroAlbaran as numeroAlbaranorderline, \
    numeroFacturaCargo as numeroFacturaorderline, \
    numeroLineaCargo as numeroLineaCargoorderline, \
    codigoProveedorFacturaCargo as codigoProveedorFCorderline, \
    unidades as unidadesorderline"

static gint recordset_to_order_lines_array(GdaRecordset *rs, OrderLine **lines_array);





/**
 * m_order_line_get_table:
 *
 * Devuelve un string con la tabla correspondiente al objeto de la base de datos. Si el
 * objeto se corresponde con varias tablas se devolveran los nombres separados por comas.
 *
 * Returns:
 * Cadena de caracteres con la tabla o lista de tablas a las que se refiere el objeto.
 */
gchar *m_order_line_get_table () {
    return TABLE;
}

/**
 * m_order_line_get_attributes:
 *
 * Funcin que obtiene los atributos de un objeto, estos atributos son los que corresponden al
 * objeto en la base de datos.
 *
 * Returns:
 * Cadena de caracteres con los atributos a los que se refiere el objeto con un alias.
 */
gchar *m_order_line_get_attributes () {
    return ATTRIBUTES;
}


/**
 * m_order_line_create:
 * @rs: Record set que contiene los campos de la BD.
 *
 * Crea un objeto a partir de los valores de un record set de la base de datos.
 *
 * Returns:
 * Un objeto #OrderLine con los campos rellenados con los datos de la base de datos.
 */
OrderLine  *m_order_line_create (GdaRecordset *rs){
    GdaField    *field;
    OrderLine *order_line = d_order_line_create_empty();

    Piece      *piece = NULL;
    gchar * temp;
/*     ChargeInvoiceLine *cil = NULL; */


    if (m_model_record_set_contains_piece (rs)==TRUE) {
	piece = m_piece_create (rs);
    } else {
	piece = NULL;
    }

/*     if (m_charge_invoice_line_is_in_rs (rs)) */
/*       cil = m_charge_invoice_line_create (rs); */
    
    d_order_line_set_piece (order_line, piece);
/*     set_lineaFacturaCargo_order_line(order_line, cil); */

    field = gda_recordset_field_name(rs,"codigopedidoorderline");

    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    order_line->order_code = igalia_field_get_bigint_value (field);
	    gda_field_free(field);

	    field = gda_recordset_field_name(rs,"codigoempresaorderline");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    order_line->enterprise_code = igalia_field_get_bigint_value (field);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs,"codigoalmacenorderline");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    order_line->warehouse_code = igalia_field_get_bigint_value (field);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs,"numerolineaorderline");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    order_line->line_number = igalia_field_get_bigint_value (field);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs,"unidadesorderline");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    order_line->quantity = igalia_field_get_double_value (field);
		}
		gda_field_free(field);
	    }

	    order_line->with_delivery_payment=FALSE;

	    field = gda_recordset_field_name(rs,"numeroalbaranorderline");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    temp=gda_stringify_value(NULL, 0, field);
		    order_line->delivery_payment = g_strdup(temp);
		    g_free(temp);
                    order_line->with_delivery_payment=TRUE;
		}
		gda_field_free(field);
	    }
	    field = gda_recordset_field_name(rs,"numeroFacturaorderline");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    order_line->charge_invoice_number = igalia_field_get_bigint_value (field);
		}
		gda_field_free(field);
	    }
	    field = gda_recordset_field_name(rs,"codigoProveedorFCorderline");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    order_line->charge_invoice_supplier_code = igalia_field_get_bigint_value (field);
		}
		gda_field_free(field);
	    }
	    field = gda_recordset_field_name(rs,"numeroLineaCargoorderline");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    order_line->charge_line_number = igalia_field_get_bigint_value (field);
		}
		gda_field_free(field);
	    }

	} else {
	    gda_field_free(field);
	}
    }
    return order_line;
}



/**
 * m_order_line_insert:
 * @order_lines: Lneas de pedido a insertar.
 * @connection: Conexin a la base de datos.
 * @command: Comando en el que se ejecuta la sentencia sql.
 *
 * Inserta las lineas de un pedido en la base de datos dentro de una transaccin.
 *
 * Returns:
 * %TRUE si las lneas de pedido se insertaron correctamente, %FALSE en otro caso.
 */
gboolean m_order_line_insert (OrderLine **order_lines, GdaConnection *connection, GdaCommand *command) {
    if (order_lines!=NULL) {
	gchar         *gchar_query_order_line = NULL;
	gchar         *gchar_query_order_line_aux = NULL;
	GdaRecordset  *record_set = NULL;
	gchar         *temp = NULL;
	gulong        *tuples = (gulong *) g_malloc0(sizeof(gulong));
	gulong         code = ULONG_MAX;
	gint           index = 0;
	OrderLine    *order_line = NULL;
	gboolean       correct = FALSE;

	while(order_lines[index]!=NULL) {
	    order_line = order_lines[index];
	    gchar_query_order_line = "";
	    gchar_query_order_line = g_strconcat("INSERT INTO ",
						 m_order_line_get_table (),
						 " VALUES (", NULL);

	    code = d_order_line_get_code (order_line);
	    if ((code!=ULONG_MAX) &&(code!=0)){
		temp = g_strdup_printf("%lu", code);
		gchar_query_order_line_aux = g_strconcat(gchar_query_order_line,
							 " '",temp,"', '", NULL);
		g_free(temp);
	    } else {
		temp = m_model_currval_sequence ("seqPedido", connection);
		gchar_query_order_line_aux = g_strconcat(gchar_query_order_line,
							 temp, " , '", NULL);
		g_free(temp);
	    }
	    g_free(gchar_query_order_line);
	    gchar_query_order_line = gchar_query_order_line_aux;
	    gchar_query_order_line_aux = NULL;

	    temp = g_strdup_printf("%lu",d_order_line_get_enterprise_code (order_line));
	    gchar_query_order_line_aux = g_strconcat(gchar_query_order_line, temp,
						 "', '", NULL);
	    g_free(temp);
	    g_free(gchar_query_order_line);
	    gchar_query_order_line = gchar_query_order_line_aux;
	    gchar_query_order_line_aux = NULL;

	    temp = g_strdup_printf("%lu",d_order_line_get_warehouse_code (order_line));
	    gchar_query_order_line_aux = g_strconcat(gchar_query_order_line, temp,
						     "', '", NULL);
	    g_free(temp);
	    g_free(gchar_query_order_line);
	    gchar_query_order_line = gchar_query_order_line_aux;
	    gchar_query_order_line_aux = NULL;

	    temp = g_strdup_printf("%d",(index+1));
	    gchar_query_order_line_aux = g_strconcat(gchar_query_order_line, temp,
						     "', '", NULL);
	    g_free(temp);
	    g_free(gchar_query_order_line);
	    gchar_query_order_line = gchar_query_order_line_aux;
	    gchar_query_order_line_aux = NULL;

	    temp = g_strdup_printf("%lu",d_product_get_product_code ((Product *) d_order_line_get_piece (order_line)));
	    gchar_query_order_line_aux = g_strconcat(gchar_query_order_line, temp,
						     "', '", NULL);
	    g_free(temp);
	    g_free(gchar_query_order_line);
	    gchar_query_order_line = gchar_query_order_line_aux;
	    gchar_query_order_line_aux = NULL;

	    temp = g_strdup_printf("%.3f",d_order_line_get_quantity (order_line));
            u_commas_to_points (temp);
	    gchar_query_order_line_aux = g_strconcat(gchar_query_order_line, temp,
						     "', ", NULL);
	    g_free(temp);
	    g_free(gchar_query_order_line);
	    gchar_query_order_line = gchar_query_order_line_aux;
	    gchar_query_order_line_aux = NULL;

	    temp = d_order_line_get_delivery_payment (order_line);
	    if (temp !=NULL) {
		gchar * temp_char = g_strconcat("'",temp,"'",NULL);
		g_free(temp);
                temp = temp_char;
	    } else {
                temp = g_strdup("NULL");
	    }
	    gchar_query_order_line_aux = g_strconcat(gchar_query_order_line, temp,
						     ", ", NULL);
	    g_free(temp);
	    g_free(gchar_query_order_line);
	    gchar_query_order_line = gchar_query_order_line_aux;
	    gchar_query_order_line_aux = NULL;

            if (d_order_line_get_charge_line_number (order_line)!=ULONG_MAX)
		temp = g_strdup_printf("'%lu'",d_order_line_get_charge_line_number (order_line));
	    else
                temp = g_strdup("NULL");
	    gchar_query_order_line_aux = g_strconcat(gchar_query_order_line, temp,
						     ", ", NULL);
	    g_free(temp);
	    g_free(gchar_query_order_line);
	    gchar_query_order_line = gchar_query_order_line_aux;
	    gchar_query_order_line_aux = NULL;

            if (d_order_line_get_charge_invoice_number (order_line)!=ULONG_MAX)
		temp = g_strdup_printf("'%lu'",d_order_line_get_charge_invoice_number (order_line));
	    else
                temp = g_strdup("NULL");
	    gchar_query_order_line_aux = g_strconcat(gchar_query_order_line, temp,
						     ", ", NULL);
	    g_free(temp);
	    g_free(gchar_query_order_line);
	    gchar_query_order_line = gchar_query_order_line_aux;
	    gchar_query_order_line_aux = NULL;
            
            if (d_order_line_get_charge_invoice_number (order_line)!=ULONG_MAX)
		temp = g_strdup_printf("'%lu'",d_order_line_get_charge_invoice_supplier_code (order_line));
	    else
                temp = g_strdup("NULL");
	    gchar_query_order_line_aux = g_strconcat(gchar_query_order_line, temp,
						     ")", NULL);
	    g_free(temp);
	    g_free(gchar_query_order_line);
	    gchar_query_order_line = gchar_query_order_line_aux;
	    gchar_query_order_line_aux = NULL;

	    correct = m_model_command_execute (&record_set, gchar_query_order_line, connection, command,tuples,0, FALSE);

	    g_free(gchar_query_order_line);
	    index++;
	}

	gda_recordset_free(record_set);
	g_free(tuples);

	return correct;
    } else {
	return TRUE;
    }
}

/**
 * m_order_line_delete_in_transaction:
 * @order: Pedido del que se borrarn las lneas de la base de datos, dentro de una transaccin.
 * @connection: Conexin a la base de datos.
 * @command: Comando en el que se ejecuta la/s sentencia/s sql.
 *
 * Borra las lneas de un pedido de la base de datos.
 *
 * Returns:
 * %TRUE si se han borrado correctamente, %FALSE en otro caso.
 */
gboolean m_order_line_delete_in_transaction (Order *order, GdaConnection *connection, GdaCommand *command) {
    if (order!=NULL) {
	gchar         *gchar_query_order_line = NULL;
	gchar         *gchar_query_order_line_aux = NULL;
	GdaRecordset  *record_set = NULL;
	gchar         *temp = NULL;
	gulong        *tuples = (gulong *) g_malloc0(sizeof(gulong));
	gboolean       result;

	gchar_query_order_line = "DELETE FROM ";

	gchar_query_order_line_aux = g_strconcat(gchar_query_order_line,
						 m_order_line_get_table (),
						 " WHERE ", NULL);
	gchar_query_order_line = gchar_query_order_line_aux;
	gchar_query_order_line_aux = NULL;

	temp = g_strdup_printf("%lu",d_order_get_code (order));
	gchar_query_order_line_aux = g_strconcat(gchar_query_order_line,
						 "codigoPedido = ", temp,
						 " AND ", NULL);
	g_free(temp);
	g_free(gchar_query_order_line);
	gchar_query_order_line = gchar_query_order_line_aux;
	gchar_query_order_line_aux = NULL;

	temp = g_strdup_printf("%lu",d_order_get_enterprise_code (order));
	gchar_query_order_line_aux = g_strconcat(gchar_query_order_line,
						 "codigoEmpresa = ", temp,
						 " AND ", NULL);
	g_free(temp);
	g_free(gchar_query_order_line);
	gchar_query_order_line = gchar_query_order_line_aux;
	gchar_query_order_line_aux = NULL;

	temp = g_strdup_printf("%lu",d_order_get_warehouse_code (order));
	gchar_query_order_line_aux = g_strconcat(gchar_query_order_line,
						 "codigoAlmacen = ", temp,
						 " ", NULL);
	g_free(temp);
	g_free(gchar_query_order_line);
	gchar_query_order_line = gchar_query_order_line_aux;
	gchar_query_order_line_aux = NULL;

	result = m_model_command_execute (&record_set, gchar_query_order_line,
				       connection, command, tuples, 0, FALSE);

	gda_recordset_free(record_set);
	g_free(gchar_query_order_line);
	g_free(tuples);

	return result;
    } else {
	return TRUE;
    }
}

/**
 * delete_order_lines:
 * @order: Pedido del que se borrarn las lneas de la base de datos.
 *
 * Borra las lneas de un pedido de la base de datos.
 *
 * Returns:
 * %TRUE si se han borrado correctamente, %FALSE en otro caso.
 */
gboolean delete_order_lines(Order *order) {
    if (order!=NULL) {
	GdaConnection *connection = m_model_get_connection ();
	GdaCommand    *command = gda_command_new();
	gboolean       result = FALSE;
	gulong enterprise_code_order = ULONG_MAX;
	gulong warehouse_code_order = ULONG_MAX;

	if (order!=NULL) {
	    enterprise_code_order = d_order_get_enterprise_code (order);
	}
	if (order!=NULL) {
	    warehouse_code_order = d_order_get_warehouse_code (order);
	}
	gda_command_set_connection(command,connection);
	m_model_connection_begin_mutex_transaction_logged (connection,enterprise_code_order,ULONG_MAX,warehouse_code_order);
	result = m_order_line_delete_in_transaction (order, connection, command);
	result = m_model_execute_mutex_modification_logged (connection) && result;
	igalia_command_free(command);
	return result;
    } else {
	return TRUE;
    }
}

/**
 * m_order_line_get_lines_from_pieces:
 * @list: Lista enlazada que se rellenar a partir de los identificadores de piezas.
 * @pieces_ids: Lista de identificadores de piezas.
 * @number:  Puntero al nmero de identificadores de piezas de la lista.
 * @enterprise_code: Cdigo de empresa para la cual se crean las lneas de pedido.
 * @warehouse_code: Cdigo de almacn para la cual se crean las lneas de pedido.
 *
 * Construye lneas de pedido a partir de una lista de identificadores de piezas.
 *
 */
void m_order_line_get_lines_from_pieces (GList **list, glong *pieces_ids, gulong *number,
				  gulong enterprise_code, gulong warehouse_code) {
  gint index = 0;
  Piece ** pieces =
      m_piece_find_by_product_code (pieces_ids, number);
  
  if (pieces!=NULL) {
      GList *last = g_list_last(*list);
      gint last_pos = 0;

      if (last != NULL ) {
	  last_pos = (g_list_position(*list, last))+1;
      }

      while (pieces[index]!=NULL) {
	  OrderLine *order_line = d_order_line_create_empty ();
	  d_order_line_set_piece (order_line, pieces[index]);
          d_order_line_set_enterprise_code (order_line, enterprise_code);
          d_order_line_set_warehouse_code (order_line, warehouse_code);
	  *list = g_list_insert( *list, order_line, last_pos++);
	  index++;
      }
  }
}

/**
 * m_order_line_get_lines_from_order:
 * @list: Lista enlazada que se rellenar a partir de los identificadores de piezas.
 * @order: Pedido del que se obtendrn los datos de las llneas de pedido.
 *
 * Transforma el array de lneas de pedido de un pedido en un lista enlazada con
 * lneas de pedido.
 *
 */
void m_order_line_get_lines_from_order (GList **list, Order *order) {

    gint index = 0;

    if(order!=NULL) {
        OrderLine **lines = d_order_line_duplicate (d_order_get_order_lines (order));
	if (lines!=NULL) {
	    GList *last = g_list_last(*list);
	    gint last_pos = 0;

	    if (last != NULL ) {
		last_pos = (g_list_position(*list, last))+1;
	    }

	    while (lines[index]!=NULL) {
		*list = g_list_insert( *list, lines[index], last_pos++);
		index++;
	    }
	}
    }
}

/**
 * update_order_line: 
 * @order_line: Lnea de pedido que se desea modificar.
 * @supplier_code: Cdigo del proveedor del pedido al que pertenece la lnea de pedido.
 * @connection: Conexin a la base de datos.
 * @command: Comando en el que se ejecutan las sentencias sql.
 *
 * Modifica una lnea de pedido en la base de datos dentro de una transaccin.
 *
 * Returns:
 * %TRUE si se ha modificado correctamente, %FALSE en otro caso.
 */
gboolean m_order_line_update (OrderLine *order_line, gulong supplier_code,
			   GdaConnection *connection, GdaCommand *command) {
    if(order_line!=NULL) {
	gchar         *gchar_query_order_line = NULL;
	GdaRecordset  *record_set = NULL;
	GdaRecordset  *record_set2 = NULL;
	gulong        *tuples = (gulong *) g_malloc0(sizeof(gulong));
	gboolean       correct = TRUE;
	gchar * order_code_char = NULL;
	gchar * enterprise_code_char = NULL;
	gchar * warehouse_code_char = NULL;
	gchar * line_number_char = NULL;
	gchar * product_code_char = NULL;
	gchar * quantity = NULL;
	gchar * delivery_payment_number = NULL;
	gchar * invoice_number = NULL;
	gchar * delivery_payment_number_aux = NULL;
        gulong invoice_number_gulong =ULONG_MAX;
	gchar * invoice_number_aux = NULL;
	gchar * charge_invoice_line_number = NULL;
	gchar * charge_invoice_supplier_code = NULL;
	gchar * supplier_code_char=NULL;
	gulong enterprise_code_order = ULONG_MAX;
	gulong warehouse_code_order = ULONG_MAX;
 


//	gda_command_set_connection(command,connection);
	if (order_line!=NULL) {

	    enterprise_code_order=d_order_line_get_enterprise_code (order_line);
	    warehouse_code_order=d_order_line_get_warehouse_code (order_line);
	    order_code_char = g_strdup_printf("%lu", d_order_line_get_code (order_line));
	    enterprise_code_char = g_strdup_printf("%lu",enterprise_code_order);
	    warehouse_code_char = g_strdup_printf("%lu",warehouse_code_order);
	    line_number_char = g_strdup_printf("%lu", d_order_line_get_line_number (order_line));
	    product_code_char = g_strdup_printf("%lu",d_product_get_product_code ((Product *) d_order_line_get_piece (order_line)));

	    if (d_order_line_get_quantity (order_line)!=G_MINDOUBLE)
		quantity = g_strdup_printf("%.3f",d_order_line_get_quantity (order_line));
	    else
		quantity = g_strdup("NULL");
            u_commas_to_points (quantity);
	    delivery_payment_number_aux = d_order_line_get_delivery_payment (order_line);
	    invoice_number_gulong = d_order_line_get_charge_invoice_number (order_line);
            if (invoice_number_gulong!=ULONG_MAX)
		invoice_number_aux = g_strdup_printf("%lu",invoice_number_gulong);

	    supplier_code_char = g_strdup_printf("%lu",supplier_code);
	    if (delivery_payment_number_aux==NULL) {
		delivery_payment_number = g_strdup("NULL");
	    } else {
		delivery_payment_number = g_strconcat("'",delivery_payment_number_aux,"'",NULL);
		g_free(delivery_payment_number_aux);
	    }

	    if (invoice_number_aux==NULL)
		invoice_number = g_strdup("NULL");
	    else {
		invoice_number = g_strconcat(invoice_number_aux,NULL);
		g_free(invoice_number_aux);
	    }


	    if (d_order_line_get_charge_line_number (order_line)!=ULONG_MAX) {
		charge_invoice_line_number = g_strdup_printf("%lu", d_order_line_get_charge_line_number (order_line));
	    } else {
		charge_invoice_line_number = g_strdup("NULL");
	    }

	    if (d_order_line_get_charge_invoice_supplier_code (order_line)!=ULONG_MAX) {
		charge_invoice_supplier_code = g_strdup_printf("%lu", d_order_line_get_charge_invoice_supplier_code (order_line));
	    } else {
		charge_invoice_supplier_code = g_strdup("NULL");
	    }

	    gchar_query_order_line = g_strconcat("UPDATE LINEAPEDIDO SET ",
						 "unidades=",quantity,
						 ",numeroAlbaran=",delivery_payment_number,
						 ",numeroFacturaCargo=",invoice_number,
						 ",numeroLineaCargo=",charge_invoice_line_number,
						 ",codigoProveedorFacturaCargo=",charge_invoice_supplier_code,
						 " WHERE ",
						 "codigoPedido=",order_code_char,
						 " and codigoEmpresa=",enterprise_code_char,
						 " and codigoAlmacen=",warehouse_code_char,
						 " and numeroLinea=",line_number_char,
						 " and codigoProducto =",product_code_char,
						 NULL);

	    g_print(_("\nUpdate: %s\n"),gchar_query_order_line);
	    if (supplier_code!=ULONG_MAX) {
		if (delivery_payment_number==NULL) {
		    delivery_payment_number = g_strdup("NULL");
		} else {
		    gdouble new_quantity=0;
                    gdouble pack_quantity =1;
		    if (d_order_line_get_with_delivery_payment (order_line)==FALSE) {
			gchar * operation_code_char = m_model_nextval_sequence ("seqMovimientoPorAlbaran",connection);
			gulong * codes = (gulong *) g_malloc0(sizeof(gulong));
			gulong * number = (gulong *) g_malloc0(sizeof(gulong)*2);
                        gchar * product_code_char =NULL;
			time_t current_date = time(NULL);
			gchar * temp_query=NULL;
			Stock ** stocks;

			codes[0]=d_product_get_product_code ((Product *) d_order_line_get_piece (order_line));
			codes[1]=-1;
			*number = 1;
			stocks =m_stock_find_pieces_by_product_code (codes, d_order_line_get_enterprise_code (order_line),
									    d_order_line_get_warehouse_code (order_line),number);


			if (d_piece_get_units_per_packet (d_order_line_get_piece (order_line))!=G_MINDOUBLE) {
                            pack_quantity = d_piece_get_units_per_packet (d_order_line_get_piece (order_line));
			}
                        if (d_stock_get_quantity (stocks[0])!=G_MINDOUBLE)
			    new_quantity = d_stock_get_quantity (stocks[0])+pack_quantity*d_order_line_get_quantity (order_line);
			else
                            new_quantity = pack_quantity*d_order_line_get_quantity (order_line);
			d_stock_set_quantity (stocks[0],new_quantity);
			m_stock_update (stocks[0],connection,command);

                        m_stock_clear_array(stocks);
                        stocks=NULL;
			product_code_char = g_strdup_printf("%lu",codes[0]);

			temp_query = g_strconcat("INSERT INTO MOVIMIENTOPORALBARAN VALUES (",
						 enterprise_code_char,",",warehouse_code_char,",",operation_code_char,
						 ",'",ctime(&current_date),"',",quantity,",",supplier_code_char,",",delivery_payment_number,",",line_number_char,",",order_code_char
						 ,",",product_code_char,")",NULL);

                        g_free(product_code_char);
			correct = correct && m_model_command_execute (&record_set2, temp_query, connection, command,tuples,0, FALSE) && correct;

			g_free(temp_query);
			g_free(operation_code_char);
			if (record_set2!=NULL) gda_recordset_free(record_set2);


		    } else {
			gchar * temp_query=NULL;

			temp_query = g_strconcat("UPDATE MOVIMIENTOPORALBARAN SET numeroAlbaran=",delivery_payment_number,
						 " where codigoEmpresa=",enterprise_code_char,
						 " and codigoAlmacen=",warehouse_code_char,
						 " and codigoProveedor=",supplier_code_char,
						 " and codigoPedido=",order_code_char,
						 " and numeroLineaPedido=",line_number_char,NULL);


			correct = correct && m_model_command_execute (&record_set2, temp_query, connection, command,tuples,0, FALSE) && correct;

			g_free(temp_query);
			if (record_set2!=NULL) gda_recordset_free(record_set2);

		    }
		}
	    }
	    correct = correct && m_model_command_execute (&record_set, gchar_query_order_line, connection, command,tuples,0, FALSE);


	    g_free(gchar_query_order_line);
	    g_free(order_code_char);
	    g_free(enterprise_code_char);
	    g_free(warehouse_code_char);
	    g_free(line_number_char);
	    g_free(product_code_char);
	    g_free(quantity);
	    g_free(delivery_payment_number);
	    g_free(invoice_number);
	    g_free(charge_invoice_line_number);
	    g_free(charge_invoice_supplier_code);

	}
	if (correct!=FALSE) {
	    gchar * new_delivery_payment_number = d_order_line_get_delivery_payment(order_line);
            if (new_delivery_payment_number!=NULL)
		d_order_line_set_with_delivery_payment (order_line,TRUE);
	    g_free(new_delivery_payment_number);

	}

	if (record_set!=NULL) gda_recordset_free(record_set);
	g_free(tuples);

	return correct;
    } else {
	return TRUE;
    }
}

/**
 * m_order_line_update_lines: 
 * @order_lines: Lista enlazada con las lneas de pedido que se modificarn.
 * @supplier_code: Cdigo del proveedor del pedido al que pertenecen las lneas de pedido.
 *
 * Modifica una lista de lneas de pedido en la base de datos. Abre una transaccin y ejecuta todas
 * las modificaciones dentro de la misma.
 *
 * Returns:
 * %TRUE si se han modificado correctamente, %FALSE en otro caso.
 */
gboolean m_order_line_update_lines (GList * order_lines,gulong supplier_code) {
    GdaConnection *connection = m_model_get_connection ();
    GdaCommand    *command = gda_command_new();
    gboolean       result = FALSE;
    gulong index = 0;
    OrderLine * order_line=NULL;
    GList * list=NULL;
    gulong logged_shop_code;
    gulong logged_enterprise_code;
    gulong logged_warehouse_code;
    gchar * temp_name;

    temp_name = u_parameter_manager_get_parameter ("empresa");
    logged_enterprise_code = m_enterprise_get_code_from_name (temp_name);
    g_free(temp_name);
    temp_name = u_parameter_manager_get_parameter ("tienda");
    logged_shop_code = m_shop_get_code_from_name (temp_name);
    g_free(temp_name);
    temp_name = u_parameter_manager_get_parameter ("almacen");
    logged_warehouse_code = m_warehouse_get_code_from_name (temp_name);

    g_free(temp_name);

    gda_command_set_connection(command,connection);


    m_model_connection_begin_mutex_transaction_logged (connection,logged_enterprise_code,ULONG_MAX,logged_warehouse_code);

    if (order_lines!=NULL) {
	list = g_list_first(order_lines);
	while (list!=NULL) {
	    order_line = (OrderLine *) g_list_nth_data(order_lines,index);
	    m_order_line_update (order_line,supplier_code,connection,command);
	    list = g_list_next(list);
            index++;
	}

    }
    result = m_model_execute_mutex_modification_logged (connection) && result;
    igalia_command_free(command);
    return result;

}

/**
 * m_order_line_get_from_order:
 * @order: Pedido del que se desea obenter las lneas.
 * 
 * Busca las las lneas de un pedido.
 *
 * Returns:
 * Lista de lneas de pedido.
 */
OrderLine **m_order_line_get_from_order (Order *order) {
    GdaCommand    *command;
    GdaConnection *connection = m_model_get_connection ();
    GdaRecordset  *record_set = NULL;
    gchar         *temp;
    gchar         *query;
    gchar         *query_aux;
    gchar         *table_order_line;
    gchar         *table_product;
    gchar         *table_piece;
    gboolean       correct;
    gulong         code;
    gulong         tuples;
    gint           index = -1;
    OrderLine   **list;

    command = gda_command_new();
    gda_command_set_connection(command, connection);
    table_order_line =u_table_put_name (m_order_line_get_attributes (), m_order_line_get_table ());
    table_product = u_table_put_name (m_product_get_attributes (), m_product_get_table ());
    table_piece = u_table_put_name (m_piece_get_attributes (), m_piece_get_table ());

    query = g_strconcat("SELECT ", table_order_line,
			", ",table_product ,
                        ", ", table_piece,
			" FROM ",  m_order_line_get_table (),
			", ", m_product_get_table (),
                        ", ", m_piece_get_table (),
			" WHERE ", NULL);

    g_free(table_order_line);
    g_free(table_product);
    g_free(table_piece);
    // Tienen que estar los cuatro cdigos, pero lo comprobamos por si acaso.
    // La query fallar si no son correctos los cuatro cdigos

    code = d_order_get_code (order);
    if ((code!=0) && (code!=ULONG_MAX)) {
	temp = g_strdup_printf("%lu", code);
	query_aux = g_strconcat(query, " (codigoPedido = ", temp, ") AND ", NULL);
	g_free(temp);
	g_free(query);
	query = query_aux;
	query_aux = NULL;
    }

    code = d_order_get_enterprise_code (order);
    if ((code!=0) && (code!=ULONG_MAX)) {
	temp = g_strdup_printf("%lu", code);
	query_aux = g_strconcat(query, " (codigoEmpresa = ", temp, ") AND ", NULL);
	g_free(temp);
	g_free(query);
	query = query_aux;
	query_aux = NULL;
    }

    code = d_order_get_warehouse_code (order);
    if ((code!=0) && (code!=ULONG_MAX)) {
	temp = g_strdup_printf("%lu", code);
	query_aux = g_strconcat(query, " (codigoAlmacen = ", temp, ") AND ", NULL);
	g_free(temp);
	g_free(query);
	query = query_aux;
	query_aux = NULL;
    }

    query_aux = g_strconcat(query,
			    " (",m_product_get_table (), ".codigoProducto = ",
			    m_piece_get_table (), ".codigoProducto) AND ",
			    " (",m_piece_get_table (),".codigoProducto = ",
			    m_order_line_get_table (), ".codigoProducto) ",
			    NULL);
    g_free(query);
    query = query_aux;
    query_aux = NULL;

    correct = m_model_command_execute (&record_set, query, connection, command,
				    &tuples,0, TRUE);

    g_print(_("\nQuery: %s\n"),query);
    g_free(query);

    list = NULL;
    if (correct) {
	if (tuples!=0) {
	    list = (OrderLine **) g_malloc0(sizeof(OrderLine*)*(tuples+1));
	    index = recordset_to_order_lines_array(record_set, list);
	} 
    } 

    gda_recordset_free(record_set);
    igalia_command_free(command);
    return list;
}

/**
 * m_order_line_clear_array:
 * @lines: Array con las lneas a liberar.
 *
 * Libera la memoria utilizada por el array de lineas de pedido.
 *
 */
void m_order_line_clear_array (OrderLine ** lines) {
    if (lines!=NULL) {
	gint index = 0;
	while (lines[index]!=NULL) {
	    d_order_line_clear (lines[index]);
	    lines[index] = NULL;
	    index++;
	}
	g_free(lines);
	lines = NULL;
    }
}

/********************private******************/

/**
 * recordset_to_order_lines_array:
 * @rs: Record set con las tuplas de la BD.
 * @lines_array: Array que se desea rellenar. Es necesario reservar memoria previamente.
 *
 *  Transforma el recorset con las tuplas de lneas de pedido en
 * un array de lineas de pedido.
 *
 * Returns:
 * Tamao del array de lneas de pedido.
 */
static gint recordset_to_order_lines_array(GdaRecordset *rs, OrderLine **lines_array) {
    gint index;
    index = gda_recordset_move_first(rs);

    while (!gda_recordset_eof(rs)) {

	if (index==GDA_RECORDSET_INVALID_POSITION) return -1;

	lines_array[index-1] = m_order_line_create (rs);
	index = gda_recordset_move_next(rs);
    }
    lines_array[index] = NULL;

    return index;
}



