/* participant_model.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */


#define TABLE         "Participante"
#define IDENTIFICATOR "codigoParty"
#define ATTRIBUTES    "codigoParty as codigopartyparticipant, \
    nick as nickparticipant, \
    telefono as telefonoparticipant, \
    telefono2 as telefono2participant, \
    fax as faxparticipant, \
    email as emailparticipant, \
    calle as calleparticipant, \
    numero as numeroparticipant, \
    piso as pisoparticipant, \
    CP as cpparticipant, \
    municipio as municipioparticipant, \
    nombre as nombreparticipant, \
    cif as cifparticipant, \
    provincia as provinciaparticipant"

#include "participant_model.h"




/**
 * m_participant_get_table:
 *
 * Devuelve un string con la tabla correspondiente al objeto de la base de datos. Si el
 * objeto se corresponde con varias tablas se devolveran los nombres separados por comas.
 *
 * Returns:
 * Cadena de caracteres con la tabla o lista de tablas a las que se refiere el objeto.
 */
gchar *m_participant_get_table () {
    return TABLE;
}

/**
 * m_participant_get_attributes:
 *
 * Funcin que obtiene los atributos de un objeto, estos atributos son los que corresponden al
 * objeto en la base de datos.
 *
 * Returns:
 * Cadena de caracteres con los atributos a los que se refiere el objeto con un alias.
 */
gchar *m_participant_get_attributes () {
    return ATTRIBUTES;
}


/**
 * m_participant_create_partial:
 * @rs: Record set que contiene los campos de la BD.
 * @rol: Indica el rol que desempea en este caso el participante. Los roles pueden ser
 * enter (en caso de tratarse de una empresa),shop (en el caso de una
 * tienda de la empresa),agent (si acta como agente
 * de una reparacin),client (si acta como cliente de
 * una reparacin),company(si acta como aseguradora),consort(si acta como consorcio),
 * garage (si acta como taller) y supplier (en caso de proveedor).
 *
 * Crea un objeto a partir de los valores de un record set de la base de datos. Este mtodo
 * crea un participante con solo un cdigo.
 *
 * Returns:
 * Un objeto #Participant con los campos rellenados con los datos de la base de datos.
 */
Participant * m_participant_create_partial (GdaRecordset *rs, gchar *rol)
{
    GdaField * field;
    gchar *plusrol;
    Participant * participant = d_participant_create_empty();

    plusrol=g_strconcat("codigopartyparticipant",rol,NULL);
    field = gda_recordset_field_name(rs,plusrol);
    if ((field!=NULL) && (!gda_field_is_null(field))) {
	participant->code = igalia_field_get_bigint_value (field);
    }

    return participant;
}


/**
 * m_participant_create:
 * @rs: Record set que contiene los campos de la BD.
 * @rol: Indica el rol que desempea en este caso el participante. Los roles pueden ser
 * enter (en caso de tratarse de una empresa),shop (en el caso de una
 * tienda de la empresa),agent (si acta como agente
 * de una reparacin),client (si acta como cliente de
 * una reparacin),company(si acta como aseguradora),consort(si acta como consorcio),
 * garage (si acta como taller) y supplier (en caso de proveedor).
 *
 * Crea un objeto a partir de los valores de un record set de la base de datos. El objeto
 * participante es un objeto que representa los datos comunes a todas las entidades
 * fsicas o jurdicas que entran en juego en la aplicacin. Las objetos que tienen atributos
 * de participante son: empresas, compaas de seguros, clientes, tiendas, consorcios,
 * talleres y proveedores.
 *
 * Returns:
 * Un objeto #Participant con los campos rellenados con los datos de la base de datos.
 */
Participant * m_participant_create (GdaRecordset *rs, gchar *rol)
{
    gchar *temp;
    gchar *plusrol;
    GdaField * field;
    Participant * participant = d_participant_create_empty();


    plusrol=g_strconcat("codigopartyparticipant",rol,NULL);

    field = gda_recordset_field_name(rs,plusrol);

    g_free(plusrol);
    if (field!=NULL){
	if (!gda_field_is_null(field)) {
	    participant->code = igalia_field_get_bigint_value (field);
	    gda_field_free(field);

	    plusrol=g_strconcat("telefonoparticipant",rol,NULL);
	    field = gda_recordset_field_name(rs,plusrol);

	    g_free(plusrol);
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    temp = gda_stringify_value(NULL, 0, field);
		    participant->phone = g_strdup(temp);
		    g_free(temp);
		};
		gda_field_free(field);
	    }

	    plusrol=g_strconcat("telefono2participant",rol,NULL);
	    field = gda_recordset_field_name(rs,plusrol);
	    g_free(plusrol);
	    if (field!=NULL){
		if (!gda_field_is_null(field)) {
		    temp = gda_stringify_value(NULL, 0, field);
		    participant->phone2 = g_strdup(temp);
		    g_free(temp);
		};
		gda_field_free(field);
	    }

	    plusrol=g_strconcat("faxparticipant",rol,NULL);
	    field = gda_recordset_field_name(rs,plusrol);
	    g_free(plusrol);
	    if (field!=NULL){
		if (!gda_field_is_null(field)) {
		    temp = gda_stringify_value(NULL, 0, field);
		    participant->fax = g_strdup(temp);
		    g_free(temp);
		};
		gda_field_free(field);
	    }

	    plusrol=g_strconcat("nickparticipant",rol,NULL);
	    field = gda_recordset_field_name(rs,plusrol);
	    g_free(plusrol);
	    if (field!=NULL){
		if (!gda_field_is_null(field)) {
		    temp = gda_stringify_value(NULL, 0, field);
		    participant->nick = g_strdup(temp);
		    g_free(temp);
		};
		gda_field_free(field);
	    }

	    plusrol=g_strconcat("emailparticipant",rol,NULL);
	    field = gda_recordset_field_name(rs,plusrol);
	    g_free(plusrol);
	    if (field!=NULL){
		if (!gda_field_is_null(field)) {
		    temp = gda_stringify_value(NULL, 0, field);
		    participant->email = g_strdup(temp);
		    g_free(temp);
		};
		gda_field_free(field);
	    }

	    plusrol=g_strconcat("calleparticipant",rol,NULL);
	    field = gda_recordset_field_name(rs,plusrol);
	    g_free(plusrol);
	    if (field!=NULL){
		if (!gda_field_is_null(field)) {
		    temp = gda_stringify_value(NULL, 0, field);
		    participant->street = g_strdup(temp);
		    g_free(temp);
		};
		gda_field_free(field);
	    }

	    plusrol=g_strconcat("numeroparticipant",rol,NULL);
	    field = gda_recordset_field_name(rs,plusrol);
	    g_free(plusrol);
	    if (field!=NULL){
		if (!gda_field_is_null(field)) {
		    temp = gda_stringify_value(NULL, 0, field);
		    participant->number = g_strdup(temp);
		    g_free(temp);
		};
		gda_field_free(field);
	    }

	    plusrol=g_strconcat("pisoparticipant",rol,NULL);
	    field = gda_recordset_field_name(rs,plusrol);
	    g_free(plusrol);
	    if (field!=NULL){
		if (!gda_field_is_null(field)) {
		    temp = gda_stringify_value(NULL, 0, field);
		    participant->floor = g_strdup(temp);
		    g_free(temp);
		};
		gda_field_free(field);
	    }

	    plusrol=g_strconcat("cpparticipant",rol,NULL);
	    field = gda_recordset_field_name(rs,plusrol);
	    g_free(plusrol);
	    if (field!=NULL){
		if (!gda_field_is_null(field))
		    participant->postal_code = igalia_field_get_bigint_value (field);
		gda_field_free(field);
	    }

	    plusrol=g_strconcat("municipioparticipant",rol,NULL);
	    field = gda_recordset_field_name(rs,plusrol);
	    g_free(plusrol);
	    if (field!=NULL){
		if (!gda_field_is_null(field)) {
		    temp = gda_stringify_value(NULL, 0, field);
		    participant->town = g_strdup(temp);
		    g_free(temp);
		}
		gda_field_free(field);
	    }

	    plusrol=g_strconcat("provinciaparticipant",rol,NULL);
	    field = gda_recordset_field_name(rs,plusrol);
	    g_free(plusrol);
	    if (field!=NULL){
		if (!gda_field_is_null(field)) {
		    temp = gda_stringify_value(NULL, 0, field);
		    participant->province = g_strdup(temp);
		    g_free(temp);
		};
		gda_field_free(field);
	    }

	    plusrol=g_strconcat("nombreparticipant",rol,NULL);
	    field = gda_recordset_field_name(rs,plusrol);
	    g_free(plusrol);
	    if (field!=NULL){
		if (!gda_field_is_null(field)) {
		    temp = gda_stringify_value(NULL, 0, field);
		    participant->name = g_strdup(temp);
		    g_free(temp);
		};
		gda_field_free(field);
	    }

	    plusrol=g_strconcat("cifparticipant",rol,NULL);
	    field = gda_recordset_field_name(rs,plusrol);
	    g_free(plusrol);
	    if (field!=NULL){
		if (!gda_field_is_null(field)) {
		    temp = gda_stringify_value(NULL, 0, field);
		    participant->legal_id = g_strdup(temp);
		    g_free(temp);
		}
		gda_field_free(field);
	    }
	 

	    plusrol=g_strconcat("shortNameparticipant",rol,NULL);
	    field = gda_recordset_field_name(rs,plusrol);
	    g_free(plusrol);
	    if (field!=NULL){
		if (!gda_field_is_null(field)) {
		    temp = gda_stringify_value(NULL, 0, field);
		    participant->short_name = g_strdup(temp);
		    g_free(temp);
		}
		gda_field_free(field);
	    }
	    	    
	} else {
	    gda_field_free(field);
	}
    }
    
   // if (participant->short_name == NULL)
   // 	d_participant_set_short_name(participant, participant->nick);
	
    return participant;
}

