/* payment_model.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include "payment_model.h"
#include "enterprise_model.h"
#include "shop_model.h"
#include "util.h"


#define TABLE         "Pago"
#define ATTRIBUTES    "codigoEmpresa as codigoEmpresaPago,codigoTienda as codigoTiendaPago,\
    codigoPago as codigoPagoPago,\
    codigoBanco as codigoBancoPago,codigoOficina as codigoOficinaPago,numeroCuenta as numeroCuentaPago,\
    pagosRealizados as pagosRealizadosPago,fechaRealizado as fechaRealizadoPago,porcentajeEntrada as porcentajeEntradaPago,\
    numeroVencimientos as numeroVencimientosPago,fechaInicio as fechaInicioPago,\
    diasIntervalo as diasIntervaloPago,\
    esAPlazos as esAPlazosPago,esTransferencia as esTransferenciaPago,\
    esTalon as esTalonPago,numeroTalon as numeroTalonPago,\
    numeroTalon as numeroTalonPago,esTarjeta as esTarjetaPago,\
    numeroTarjeta as numeroTarjetaPago,validaHasta as validaHastaPago,\
    codigoBanco as codigoBancoPago,codigoOficina as codigoOficinaPago,\
    numeroCuenta as numeroCuentaPago,esPagoContado as esPagoContadoPago";




/**
 * m_payment_get_table:
 *
 * Devuelve un string con la tabla correspondiente al objeto de la base de datos. Si el
 * objeto se corresponde con varias tablas se devolveran los nombres separados por comas.
 *
 * Returns:
 * Cadena de caracteres con la tabla o lista de tablas a las que se refiere el objeto.
 */
gchar *m_payment_get_table () {
    return TABLE;
}

/**
 * m_payment_get_attributes:
 *
 * Funcin que obtiene los atributos de un objeto, estos atributos son los que corresponden al
 * objeto en la base de datos.
 *
 * Returns:
 * Cadena de caracteres con los atributos a los que se refiere el objeto con un alias.
 */
gchar *m_payment_get_attributes () {
    return ATTRIBUTES;
}


/**
 * m_payment_create:
 * @rs: Record set que contiene los campos de la BD.
 *
 * Crea un objeto a partir de los valores de un record set de la base de datos.
 *
 * Returns:
 * Un objeto #Payment con los campos rellenados con los datos de la base de datos.
 */
Payment * m_payment_create (GdaRecordset *rs) {

    GdaField * field;
    Payment * payment;

    payment = d_payment_create_empty();

    field = gda_recordset_field_name(rs,"codigopagopago");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {

	    gda_field_free(field);

	    payment->enterprise = m_enterprise_create (rs);

	    payment->shop = m_shop_create (rs);
	    field = gda_recordset_field_name(rs,"codigopagopago");
	    payment->payment_code = igalia_field_get_bigint_value (field);
	    gda_field_free(field);


	    field = gda_recordset_field_name(rs,"pagosRealizadospago");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    payment->achieved_payments = igalia_field_get_bigint_value (field);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs,"porcentajeentradapago");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    gchar * temp =  gda_stringify_value(NULL, 0, field);
		    payment->entry_percentage = atof(temp);
                    g_free(temp);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs,"numerovencimientospago");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    payment->number_expiries = gda_field_get_smallint_value(field);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs,"fechaRealizadopago");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    gchar * temp;
		    time_t * my_time;
		    temp=gda_stringify_value(NULL, 0, field);
		    my_time = u_parse_date (temp);
		    payment->achieved_date = (*my_time);
		    g_free(my_time);
		    my_time = NULL;
		    g_free(temp);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs,"fechainiciopago");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    gchar * temp;
		    time_t * my_time;
		    temp=gda_stringify_value(NULL, 0, field);
		    my_time = u_parse_date (temp);
		    payment->init_date = (*my_time);
		    g_free(my_time);
		    my_time = NULL;
		    g_free(temp);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs,"diasintervalopago");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    payment->interval_days = gda_field_get_smallint_value(field);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs,"esaplazospago");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    payment->is_hire_purchase = gda_field_get_boolean_value(field);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs,"esdomiciliacionpago");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    payment->is_direct_debit = gda_field_get_boolean_value(field);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs,"estransferenciapago");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    payment->is_transfer = gda_field_get_boolean_value(field);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs,"estalonpago");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    payment->is_cheque = gda_field_get_boolean_value(field);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs,"numerotalonpago");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    gchar * temp =  gda_stringify_value(NULL, 0, field);
		    payment->cheque_number = g_strdup(temp);
		    g_free(temp);
		}
		gda_field_free(field);
	    }


	    field = gda_recordset_field_name(rs,"estarjetapago");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    payment->is_credit_card = gda_field_get_boolean_value(field);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs,"numerotarjetapago");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    gchar * temp =  gda_stringify_value(NULL, 0, field);
		    payment->credit_card_number =g_strdup(temp);
		    g_free(temp);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs,"validahastapago");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    gchar * temp;
		    time_t * my_time;
		    temp=gda_stringify_value(NULL, 0, field);
		    my_time = u_parse_date (temp);
		    payment->expiry_date = (*my_time);
		    g_free(my_time);
		    my_time = NULL;
		    g_free(temp);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs,"espagocontado");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    payment->is_paid_cash = gda_field_get_boolean_value(field);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs,"codigoBancoPago");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    payment->bank_code = gda_field_get_smallint_value(field);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs,"codigoOficinaPago");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    payment->office_code = gda_field_get_smallint_value(field);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs,"numeroCuentapago");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    gchar * temp =  gda_stringify_value(NULL, 0, field);
		    payment->account_number =g_strdup(temp);
		    g_free(temp);
		}
		gda_field_free(field);
	    }
	} else {
	    gda_field_free(field);
	}
    }

    return payment;
}



/**
 * m_payment_find_by_id:
 * @condition: Identificador del pago.
 *
 * Busca un pago en la base de datos a partir del identificador del mismo.
 *
 * Returns:
 * El pago solicitado si existe y la operacin es correcta, %NULL en otro caso.
 */
Payment * m_payment_find_by_id (PaymentId * condition) {
    GdaCommand         *my_command;
    gchar              *gchar_query=NULL;
    gchar              *gchar_select=NULL;
    gchar              *gchar_where=NULL;
    gchar              *gchar_from=NULL;
    GdaConnection      *my_connection = m_model_get_connection ();
    GdaRecordset       *record_set = NULL;
    Payment * payment=NULL;
    gulong * tuples;
    gulong index;


    my_command = gda_command_new();
    gda_command_set_connection(my_command,my_connection);

    tuples = (gulong *) g_malloc0(sizeof(gulong));

    gchar_select = g_strconcat("select codigoEmpresa as codigoPartyParticipantEnter,",
			       "participanteEmpresa.nombre as nombreParticipantEnter,",
			       "participanteEmpresa.nick as nickParticipantEnter,",
			       "participanteEmpresa.cif as cifParticipantEnter,",
			       "participanteEmpresa.codigoParty as codigoPartyParticipantEnter,",
			       "participanteEmpresa.email as emailParticipantEnter,",
			       "participanteEmpresa.telefono as telefonoParticipantEnter,",
			       "participanteEmpresa.telefono2 as telefono2ParticipantEnter,",
			       "participanteEmpresa.fax as faxParticipantEnter,",
			       "participanteEmpresa.calle as calleParticipantEnter,",
			       "participanteEmpresa.numero as numeroParticipantEnter,",
			       "participanteEmpresa.piso as pisoParticipantEnter,",
			       "participanteEmpresa.cp as cpParticipantEnter,",
			       "participanteEmpresa.municipio as municipioParticipantEnter,",
			       "participanteEmpresa.provincia as provinciaParticipantEnter,",
			       "participanteTienda.codigoParty as codigoPartyParticipantShop,",
			       "participanteTienda.nombre as nombreParticipantShop,",
			       "participanteTienda.nick as nickParticipantShop,",
			       "participanteTienda.cif as cifParticipantShop,",
			       "codigoTienda as codigoPartyParticipantShop,",
                               "pago.codigoPago as codigoPagoPago,",
			       "pagosRealizados as pagosRealizadosPago,",
			       "fechaRealizado as fechaRealizadoPago,",
			       "esAPlazos as esAPlazosPago,",
			       "fechaInicio as fechaInicioPago,",
			       "porcentajeEntrada as porcentajeEntradaPago,",
			       "numeroVencimientos as numeroVencimientosPago,",
			       "diasIntervalo as diasIntervaloPago,",
			       "esDomiciliacion as esDomiciliacionPago,",
			       "pago.codigoBanco as codigoBancoPago,",
			       "pago.codigoOficina as codigoOficinaPago,",
			       "pago.numeroCuenta as numeroCuentaPago,",
			       "esTransferencia as esTransferenciaPago,",
			       "esTalon as esTalonPago,",
			       "numeroTalon as numeroTalonPago,",
			       "esTarjeta as esTarjetaPago,",
			       "numeroTarjeta as numeroTarjetaPago,",
			       "validaHasta as validaHastaPago,",
			       "esPagoContado as esPagoContadoPago",
			       NULL);
    gchar_from= g_strconcat(" from pago,participante participanteEmpresa,participante participanteTienda ",NULL);
    gchar_where = g_strconcat("where pago.codigoEmpresa = participanteEmpresa.codigoParty ",
                              " and pago.codigoTienda= participanteTienda.codigoParty",
                              " and pago.codigoPago=",condition->payment_code,
                              " and pago.codigoEmpresa=",condition->enterprise_code,
			      " and pago.codigoTienda=",condition->shop_code,NULL);

    gchar_query = g_strconcat(gchar_select,gchar_from,gchar_where, NULL);

    g_print(_("\nQuery: %s\n"),gchar_query);

    m_model_command_execute (&record_set, gchar_query, my_connection, my_command,tuples,0,TRUE);


    if (record_set!=NULL) {
	if (*tuples!=0) {
	    index = gda_recordset_move_first(record_set);
	    while (!gda_recordset_eof(record_set)) {
		if (index==GDA_RECORDSET_INVALID_POSITION) return NULL;
		payment = m_payment_create (record_set);
		index = gda_recordset_move_next(record_set);
	    }
	} else {
	    payment=NULL;
	}
	gda_recordset_close(record_set);
	gda_recordset_free(record_set);
    } else {
	payment=NULL;
    }

    g_free(gchar_where);
    g_free(gchar_select);
    g_free(gchar_from);
    g_free(gchar_query);
    g_free(tuples);



    igalia_command_free(my_command);
    g_print(_("Freeing database completed\n"));

    return payment;

}

/**
 * m_payment_update:
 * @payment: Pago a modificar en la base de datos.
 *
 * Modifica un pago en la base de datos, si no existiese inicialmente se inserta.
 *
 * Returns:
 * %ULONG_MAX si se ha producido un error, el cdigo del pago en caso de operacin correcta.
 */
gulong m_payment_update (Payment * payment) {
    gulong result=0;
    gchar * query1=NULL;
    gchar * query2=NULL;
    gchar * query_aux;
    gchar * query=NULL;
    gchar * values=NULL;
    gchar * values_aux;
    gchar * enterprise_char=NULL;
    gchar * payment_char=NULL;
    gchar * shop_char=NULL;
    gulong shop_code=ULONG_MAX;
    gulong enterprise_code=ULONG_MAX;
    gulong payment_code = ULONG_MAX;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    GdaRecordset *record_set1=NULL;
    GdaRecordset *record_set2=NULL;
    GdaConnection * connection = m_model_get_connection ();
    GdaCommand * command = gda_command_new();

    shop_code=d_participant_get_code (d_shop_get_participant (d_payment_get_shop (
									      payment
									     )));
    shop_char = g_strdup_printf("%lu",shop_code);
    enterprise_code=d_participant_get_code (d_enterprise_get_participant (d_payment_get_enterprise (
											   payment
											  )));
    enterprise_char = g_strdup_printf("%lu",enterprise_code);

    gda_command_set_connection(command,connection);
    m_model_connection_begin_mutex_transaction_logged (connection,enterprise_code,shop_code,ULONG_MAX);


    if ((enterprise_code!=ULONG_MAX) && (shop_code!=ULONG_MAX)) {
	payment_code = d_payment_get_payment_code (payment);
        payment_char = g_strdup_printf("%lu",payment_code);

	if (payment_code==ULONG_MAX) {
	    time_t time_aux;
	    gchar * temp_nextval=NULL;
	    query1 = g_strconcat(" INSERT INTO PAGO VALUES ",NULL);
	    values = g_strconcat(" (",NULL);
	    if (d_participant_get_code (d_enterprise_get_participant (d_payment_get_enterprise (payment)))!=ULONG_MAX) {
		gchar * temp = g_strdup_printf("%lu",d_participant_get_code (d_enterprise_get_participant (d_payment_get_enterprise (payment))));
		values_aux = g_strconcat(values,temp,",",NULL);
		g_free(values);
		g_free(temp);
		values = values_aux;
	    } else {
		values_aux = g_strconcat(values,"NULL,",NULL);
		g_free(values);
		values = values_aux;

	    }
	    if (d_participant_get_code (d_shop_get_participant (d_payment_get_shop (payment)))!=ULONG_MAX) {
		gchar * temp = g_strdup_printf("%lu",d_participant_get_code (d_shop_get_participant (d_payment_get_shop (payment))));
		values_aux = g_strconcat(values,temp,",",NULL);
		g_free(values);
		g_free(temp);
		values = values_aux;

	    } else {
		values_aux = g_strconcat(values,"NULL,",NULL);
		g_free(values);
		values = values_aux;

	    }
            temp_nextval=m_model_nextval_sequence ("seqPago",connection);
	    values_aux = g_strconcat(values,temp_nextval,",",NULL);
	    g_free(values);
	    values = values_aux;
            g_free(temp_nextval);

	    if (d_payment_get_achieved_payments (payment)!=ULONG_MAX) {
		gchar * temp = g_strdup_printf("%lu",d_payment_get_achieved_payments (payment));
		values_aux = g_strconcat(values,temp,",",NULL);
		g_free(temp);
		g_free(values);
		values = values_aux;
	    } else {
		values_aux = g_strconcat(values,"NULL,",NULL);
		g_free(values);
		values = values_aux;
	    }

	    if (d_payment_get_achieved_date (payment)!=time(NULL)) {
		time_t temp = d_payment_get_achieved_date (payment);
		values_aux = g_strconcat(values,"'",ctime(&temp),"',",NULL);
		g_free(values);
		values = values_aux;

	    } else {
		values_aux = g_strconcat(values,"NULL,",NULL);
		g_free(values);
		values = values_aux;

	    }


	    if (d_payment_get_is_hire_purchase (payment)==TRUE) {
		values_aux = g_strconcat(values,"TRUE,",NULL);
		g_free(values);
		values = values_aux;
	    } else {
		values_aux = g_strconcat(values,"FALSE,",NULL);
		g_free(values);
		values = values_aux;
	    }

	    time_aux = d_payment_get_init_date (payment);
	    values_aux = g_strconcat(values,"'",ctime(&time_aux),"',",NULL);
	    g_free(values);
	    values = values_aux;


	    if (d_payment_get_entry_percentage (payment)!=-1) {
		gchar * temp =g_strdup_printf("%.2f",d_payment_get_entry_percentage (payment));
		u_commas_to_points(temp);
		values_aux = g_strconcat(values,temp,",",NULL);
		g_free(temp);
		g_free(values);
		values = values_aux;
	    } else {
		values_aux = g_strconcat(values,"NULL,",NULL);
		g_free(values);
		values = values_aux;
	    }

	    if (d_payment_get_number_expiries (payment)!=-1) {
		gchar * temp = g_strdup_printf("%d",d_payment_get_number_expiries (payment));
		values_aux = g_strconcat(values,temp,",",NULL);
		g_free(temp);
		g_free(values);
		values = values_aux;
	    } else {
		values_aux = g_strconcat(values,"NULL,",NULL);
		g_free(values);
		values = values_aux;
	    }

	    if (d_payment_get_interval_days (payment)!=-1) {
		gchar * temp = g_strdup_printf("%d",d_payment_get_interval_days (payment));
		values_aux = g_strconcat(values,temp,",",NULL);
		g_free(temp);
		g_free(values);
		values = values_aux;

	    } else {
		values_aux = g_strconcat(values,"NULL,",NULL);
		g_free(values);
		values = values_aux;

	    }

	    if (d_payment_get_is_direct_debit (payment)==TRUE) {
		values_aux = g_strconcat(values,"TRUE,",NULL);
		g_free(values);
		values = values_aux;
	    } else {
		values_aux = g_strconcat(values,"FALSE,",NULL);
		g_free(values);
		values = values_aux;
	    }

	    if (d_payment_get_bank_code (payment)!=-1) {
		gchar * temp =g_strdup_printf("%d",d_payment_get_bank_code (payment));
		values_aux = g_strconcat(values,temp,",",NULL);
		g_free(temp);
		g_free(values);
		values = values_aux;

	    } else {
		values_aux = g_strconcat(values,"NULL,",NULL);
		g_free(values);
		values = values_aux;
	    }

	    if (d_payment_get_office_code (payment)!=-1) {
		gchar * temp =g_strdup_printf("%d",d_payment_get_office_code (payment));
		values_aux = g_strconcat(values,temp,",",NULL);
		g_free(temp);
		g_free(values);
		values = values_aux;

	    } else {
		values_aux = g_strconcat(values,"NULL,",NULL);
		g_free(values);
		values = values_aux;
	    }

	    if (d_payment_get_account_number (payment)!=NULL) {
		gchar * temp = d_payment_get_account_number (payment);
		values_aux = g_strconcat(values,temp,",",NULL);
		g_free(temp);
		g_free(values);
		values = values_aux;

	    } else {
		values_aux = g_strconcat(values,"NULL,",NULL);
		g_free(values);
		values = values_aux;
	    }


	    if (d_payment_get_is_transfer (payment)==TRUE) {
		values_aux = g_strconcat(values,"TRUE,",NULL);
		g_free(values);
		values = values_aux;
	    } else {
		values_aux = g_strconcat(values,"FALSE,",NULL);
		g_free(values);
		values = values_aux;
	    }

	    if (d_payment_get_is_cheque (payment)==TRUE) {
		values_aux = g_strconcat(values,"TRUE,",NULL);
		g_free(values);
		values = values_aux;
	    } else {
		values_aux = g_strconcat(values,"FALSE,",NULL);
		g_free(values);
		values = values_aux;
	    }


	    if (d_payment_c_util_get_cheque_number (payment)!=NULL) {
		gchar * temp = d_payment_c_util_get_cheque_number (payment);
		values_aux = g_strconcat(values,"'",temp,"',",NULL);
		g_free(temp);
		g_free(values);
		values = values_aux;

	    } else {
		values_aux = g_strconcat(values,"NULL,",NULL);
		g_free(values);
		values = values_aux;
	    }

	    if (d_payment_get_is_credit_card (payment)==TRUE) {
		values_aux = g_strconcat(values,"TRUE,",NULL);
		g_free(values);
		values = values_aux;
	    } else {
		values_aux = g_strconcat(values,"FALSE,",NULL);
		g_free(values);
		values = values_aux;
	    }

	    if (d_payment_get_credit_card_number (payment)!=NULL) {
		gchar * temp = d_payment_get_credit_card_number (payment);
		values_aux = g_strconcat(values,"'",temp,"',",NULL);
		g_free(temp);
		g_free(values);
		values = values_aux;

	    } else {
		values_aux = g_strconcat(values,"NULL,",NULL);;
		g_free(values);
		values = values_aux;

	    }

	    if (d_payment_get_is_credit_card (payment)==TRUE) {
		time_t temp = d_payment_get_expiry_date (payment);
		values_aux = g_strconcat(values,"'",ctime(&temp),"',",NULL);
		g_free(values);
		values = values_aux;

	    } else {
		values_aux = g_strconcat(values,"NULL,",NULL);;
		g_free(values);
		values = values_aux;

	    }

	    if (d_payment_get_is_paid_cash (payment)==TRUE) {
		values_aux = g_strconcat(values,"TRUE",NULL);
		g_free(values);
		values = values_aux;
	    } else {
		values_aux = g_strconcat(values,"FALSE",NULL);
		g_free(values);
		values = values_aux;
	    }

	    query_aux=g_strconcat(query1,values,")",NULL);
            g_free(values);
	    g_free(query1);
	    query1=query_aux;
	    query_aux=NULL;

	    if (m_model_command_execute (&record_set1, query1, connection, command,tuples,0, FALSE)==FALSE) {
                result = ULONG_MAX;
	    }

	    query2 = g_strdup("select currval('seqPago')");
	    if (m_model_command_execute (&record_set2, query2, connection, command,tuples,0, TRUE)==FALSE) {
                result = ULONG_MAX;
	    }

	    if ((record_set2!=NULL) && (result == 0)) {
		if (*tuples!=0) {
		    GdaField * field;
		    gda_recordset_move_first(record_set2);
		    field  = gda_recordset_field_idx(record_set2,0);
		    if (field!=NULL) {
			if (!gda_field_is_null(field)) {
			    gchar ** stop = (gchar **)g_malloc0(sizeof(gchar *));
                            g_free(payment_char);
			    payment_char = gda_stringify_value(NULL, 0, field);
			    result = strtoul(payment_char,stop,10);
			    g_free(stop);
			}
			gda_field_free(field);
		    }

		} else {
		    result = ULONG_MAX;
		}
	    } else {
		result = ULONG_MAX;
	    }


	} else {
	    time_t time_aux;
            gchar * gchar_where_aux;
            gchar * gchar_where =g_strdup(" where ");
	    query1 = g_strconcat(" UPDATE PAGO SET ",NULL);
	    values = g_strconcat(" ",NULL);

	    gchar_where_aux = g_strconcat(gchar_where," codigoPago=",payment_char," and codigoTienda= ",shop_char,
					  " and codigoEmpresa =",enterprise_char,NULL);
	    g_free(gchar_where);
	    gchar_where = gchar_where_aux;
            gchar_where_aux = NULL;


	    if (d_payment_get_achieved_payments (payment)!=ULONG_MAX) {
		gchar * temp = g_strdup_printf("%lu",d_payment_get_achieved_payments (payment));
		values_aux = g_strconcat(values,"pagosRealizados=",temp,",",NULL);
		g_free(temp);
		g_free(values);
		values = values_aux;
	    } 

	    if (d_payment_get_achieved_date (payment)!=time(NULL)) {
		time_t temp = d_payment_get_achieved_date (payment);
		values_aux = g_strconcat(values,"fechaRealizado=","'",ctime(&temp),"',",NULL);
		g_free(values);
		values = values_aux;
	    } 


	    if (d_payment_get_is_hire_purchase (payment)==TRUE) {
		values_aux = g_strconcat(values,"esAPlazos=TRUE,",NULL);
		g_free(values);
		values = values_aux;
	    } else {
		values_aux = g_strconcat(values,"esAPlazos=FALSE,",NULL);
		g_free(values);
		values = values_aux;
	    }

	    time_aux = d_payment_get_init_date (payment);
	    values_aux = g_strconcat(values,"fechaInicio='",ctime(&time_aux),"',",NULL);
	    g_free(values);
	    values = values_aux;


	    if (d_payment_get_entry_percentage (payment)!=-1) {
 		gchar * temp =g_strdup_printf("%.2f",d_payment_get_entry_percentage (payment));
		u_commas_to_points(temp);
		values_aux = g_strconcat(values,"porcentajeEntrada=",temp,",",NULL);
		g_free(temp);
		g_free(values);
		values = values_aux;
	    } 

	    if (d_payment_get_number_expiries (payment)!=-1) {
		gchar * temp = g_strdup_printf("%d",d_payment_get_number_expiries (payment));
		values_aux = g_strconcat(values,"numerovencimientos=",temp,",",NULL);
		g_free(temp);
		g_free(values);
		values = values_aux;
	    } 

	    if (d_payment_get_interval_days (payment)!=-1) {
		gchar * temp = g_strdup_printf("%d",d_payment_get_interval_days (payment));
		values_aux = g_strconcat(values,"diasIntervalo=",temp,",",NULL);
		g_free(temp);
		g_free(values);
		values = values_aux;
	    }

	    if (d_payment_get_is_direct_debit (payment)==TRUE) {
		values_aux = g_strconcat(values,"esDomiciliacion=TRUE,",NULL);
		g_free(values);
		values = values_aux;
	    } else {
		values_aux = g_strconcat(values,"esDomiciliacion=FALSE,",NULL);
		g_free(values);
		values = values_aux;
	    }

	    if (d_payment_get_bank_code (payment)!=-1) {
		gchar * temp =g_strdup_printf("%d",d_payment_get_bank_code (payment));
		values_aux = g_strconcat(values,"codigoBanco=",temp,",",NULL);
		g_free(temp);
		g_free(values);
		values = values_aux;

	    } 

	    if (d_payment_get_office_code (payment)!=-1) {
		gchar * temp =g_strdup_printf("%d",d_payment_get_office_code (payment));
		values_aux = g_strconcat(values,"codigoOficina=",temp,",",NULL);
		g_free(temp);
		g_free(values);
		values = values_aux;

	    } 

	    if (d_payment_get_account_number (payment)!=NULL) {
		gchar * temp = d_payment_get_account_number (payment);
		values_aux = g_strconcat(values,"numeroCuenta=",temp,",",NULL);
		g_free(temp);
		g_free(values);
		values = values_aux;

	    }

	    if (d_payment_get_is_transfer (payment)==TRUE) {
		values_aux = g_strconcat(values,"esTransferencia=TRUE,",NULL);
		g_free(values);
		values = values_aux;
	    } else {
		values_aux = g_strconcat(values,"esTransferencia=FALSE,",NULL);
		g_free(values);
		values = values_aux;
	    }

	    if (d_payment_get_is_cheque (payment)==TRUE) {
		values_aux = g_strconcat(values,"esTalon=TRUE,",NULL);
		g_free(values);
		values = values_aux;
	    } else {
		values_aux = g_strconcat(values,"esTalon=FALSE,",NULL);
		g_free(values);
		values = values_aux;
	    }


	    if (d_payment_c_util_get_cheque_number (payment)!=NULL) {
		gchar * temp = d_payment_c_util_get_cheque_number (payment);
		values_aux = g_strconcat(values,"numeroTalon='",temp,"',",NULL);
		g_free(temp);
		g_free(values);
		values = values_aux;

	    } 

	    if (d_payment_get_is_credit_card (payment)==TRUE) {
		values_aux = g_strconcat(values,"esTarjeta=TRUE,",NULL);
		g_free(values);
		values = values_aux;
	    } else {
		values_aux = g_strconcat(values,"esTarjeta=FALSE,",NULL);
		g_free(values);
		values = values_aux;
	    }

	    if (d_payment_get_credit_card_number (payment)!=NULL) {
		gchar * temp = d_payment_get_credit_card_number (payment);
		values_aux = g_strconcat(values,"numeroTarjeta='",temp,"',",NULL);
		g_free(temp);
		g_free(values);
		values = values_aux;

	    } 

	    if (d_payment_get_is_credit_card (payment)==TRUE) {
		time_t temp = d_payment_get_expiry_date (payment);
		values_aux = g_strconcat(values,"validaHasta='",ctime(&temp),"',",NULL);
		g_free(values);
		values = values_aux;

	    } else {
		values_aux = g_strconcat(values,"validaHasta=NULL,",NULL);
		g_free(values);
		values = values_aux;
	    }

	    if (d_payment_get_is_paid_cash (payment)==TRUE) {
		values_aux = g_strconcat(values,"esPagoContado=TRUE",NULL);
		g_free(values);
		values = values_aux;
	    } else {
		values_aux = g_strconcat(values,"esPagoContado=FALSE",NULL);
		g_free(values);
		values = values_aux;
	    }

	    query_aux=g_strconcat(query1,values,gchar_where,NULL);
	    g_free(query1);
	    query1=query_aux;
	    query_aux=NULL;

	    if (m_model_command_execute (&record_set1, query1, connection, command,tuples,0, FALSE)==FALSE) {
                result =ULONG_MAX;
	    }

	    if (result !=ULONG_MAX) {
		result = payment_code;
	    }
	}

    } 

    if (query2!=NULL)
	query = g_strconcat (query1,";\n",query2,";\n",NULL);
    else
	query = g_strconcat (query1,";\n",NULL);

    if (query2!=NULL) g_free(query2);
    g_free(query1);
    if (m_model_execute_mutex_modification_logged (connection)==FALSE) {
	result = ULONG_MAX;
    }

    g_free(tuples);
    g_free(query);
    g_free(payment_char);
    g_free(enterprise_char);
    g_free(shop_char);


    gda_recordset_free(record_set1);
    if (record_set2!=NULL) {
	gda_recordset_free(record_set2);
    }
    igalia_command_free(command);

    return result;

}

/**
 * m_payment_update_in_transaction:
 * @payment: Pago que se debe modificar en la base de datos.
 * @connection: Conexin a la base de datos.
 * @command: Comando de la base de datos en el que se ejecutar la sentencia sql.
 *
 * Modifica un pago dentro de una transaccin.
 *
 * Returns:
 * %ULONG_MAX si se ha producido un error, el cdigo del pago en caso de operacin correcta.
 */
gboolean m_payment_update_in_transaction (Payment * payment,GdaConnection * connection,GdaCommand * command) {
    gulong result=0;
    gchar * query1=NULL;
    gchar * query2=NULL;
    gchar * query_aux;
    gchar * query=NULL;
    gchar * values=NULL;
    gchar * values_aux;
    gchar * enterprise_char=NULL;
    gchar * payment_char=NULL;
    gchar * shop_char=NULL;
    gulong shop_code=ULONG_MAX;
    gulong enterprise_code=ULONG_MAX;
    gulong payment_code = ULONG_MAX;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    GdaRecordset *record_set1=NULL;
    GdaRecordset *record_set2=NULL;

    shop_code=d_participant_get_code (d_shop_get_participant (d_payment_get_shop (
									      payment
									     )));
    shop_char = g_strdup_printf("%lu",shop_code);
    enterprise_code=d_participant_get_code (d_enterprise_get_participant (d_payment_get_enterprise (
											   payment
											  )));
    enterprise_char = g_strdup_printf("%lu",enterprise_code);



    if ((enterprise_code!=ULONG_MAX) && (shop_code!=ULONG_MAX)) {
	payment_code = d_payment_get_payment_code (payment);
        payment_char = g_strdup_printf("%lu",payment_code);

	if (payment_code==ULONG_MAX) {
	    time_t time_aux;
	    gchar * temp_nextval=NULL;
	    query1 = g_strconcat(" INSERT INTO PAGO VALUES ",NULL);
	    values = g_strconcat(" (",NULL);
	    if (d_participant_get_code (d_enterprise_get_participant (d_payment_get_enterprise (payment)))!=ULONG_MAX) {
		gchar * temp = g_strdup_printf("%lu",d_participant_get_code (d_enterprise_get_participant (d_payment_get_enterprise (payment))));
		values_aux = g_strconcat(values,temp,",",NULL);
		g_free(values);
		g_free(temp);
		values = values_aux;
	    } else {
		values_aux = g_strconcat(values,"NULL,",NULL);
		g_free(values);
		values = values_aux;

	    }
	    if (d_participant_get_code (d_shop_get_participant (d_payment_get_shop (payment)))!=ULONG_MAX) {
		gchar * temp = g_strdup_printf("%lu",d_participant_get_code (d_shop_get_participant (d_payment_get_shop (payment))));
		values_aux = g_strconcat(values,temp,",",NULL);
		g_free(values);
		g_free(temp);
		values = values_aux;

	    } else {
		values_aux = g_strconcat(values,"NULL,",NULL);
		g_free(values);
		values = values_aux;

	    }
            temp_nextval=m_model_nextval_sequence ("seqPago",connection);
	    values_aux = g_strconcat(values,temp_nextval,",",NULL);
	    g_free(values);
	    values = values_aux;
            g_free(temp_nextval);

	    if (d_payment_get_achieved_payments (payment)!=ULONG_MAX) {
		gchar * temp = g_strdup_printf("%lu",d_payment_get_achieved_payments (payment));
		values_aux = g_strconcat(values,temp,",",NULL);
		g_free(temp);
		g_free(values);
		values = values_aux;
	    } else {
		values_aux = g_strconcat(values,"NULL,",NULL);
		g_free(values);
		values = values_aux;
	    }

	    if (d_payment_get_achieved_date (payment)!=time(NULL)) {
		time_t temp = d_payment_get_achieved_date (payment);
		values_aux = g_strconcat(values,"'",ctime(&temp),"',",NULL);
		g_free(values);
		values = values_aux;

	    } else {
		values_aux = g_strconcat(values,"NULL,",NULL);
		g_free(values);
		values = values_aux;

	    }


	    if (d_payment_get_is_hire_purchase (payment)==TRUE) {
		values_aux = g_strconcat(values,"TRUE,",NULL);
		g_free(values);
		values = values_aux;
	    } else {
		values_aux = g_strconcat(values,"FALSE,",NULL);
		g_free(values);
		values = values_aux;
	    }

	    time_aux = d_payment_get_init_date (payment);
	    values_aux = g_strconcat(values,"'",ctime(&time_aux),"',",NULL);
	    g_free(values);
	    values = values_aux;


	    if (d_payment_get_entry_percentage (payment)!=-1) {
		gchar * temp =g_strdup_printf("%.2f",d_payment_get_entry_percentage (payment));
		u_commas_to_points(temp);
		values_aux = g_strconcat(values,temp,",",NULL);
		g_free(temp);
		g_free(values);
		values = values_aux;
	    } else {
		values_aux = g_strconcat(values,"NULL,",NULL);
		g_free(values);
		values = values_aux;
	    }

	    if (d_payment_get_number_expiries (payment)!=-1) {
		gchar * temp = g_strdup_printf("%d",d_payment_get_number_expiries (payment));
		values_aux = g_strconcat(values,temp,",",NULL);
		g_free(temp);
		g_free(values);
		values = values_aux;
	    } else {
		values_aux = g_strconcat(values,"NULL,",NULL);
		g_free(values);
		values = values_aux;
	    }

	    if (d_payment_get_interval_days (payment)!=-1) {
		gchar * temp = g_strdup_printf("%d",d_payment_get_interval_days (payment));
		values_aux = g_strconcat(values,temp,",",NULL);
		g_free(temp);
		g_free(values);
		values = values_aux;

	    } else {
		values_aux = g_strconcat(values,"NULL,",NULL);
		g_free(values);
		values = values_aux;

	    }

	    if (d_payment_get_is_direct_debit (payment)==TRUE) {
		values_aux = g_strconcat(values,"TRUE,",NULL);
		g_free(values);
		values = values_aux;
	    } else {
		values_aux = g_strconcat(values,"FALSE,",NULL);
		g_free(values);
		values = values_aux;
	    }

	    if (d_payment_get_bank_code (payment)!=-1) {
		gchar * temp =g_strdup_printf("%d",d_payment_get_bank_code (payment));
		values_aux = g_strconcat(values,temp,",",NULL);
		g_free(temp);
		g_free(values);
		values = values_aux;

	    } else {
		values_aux = g_strconcat(values,"NULL,",NULL);
		g_free(values);
		values = values_aux;
	    }

	    if (d_payment_get_office_code (payment)!=-1) {
		gchar * temp =g_strdup_printf("%d",d_payment_get_office_code (payment));
		values_aux = g_strconcat(values,temp,",",NULL);
		g_free(temp);
		g_free(values);
		values = values_aux;

	    } else {
		values_aux = g_strconcat(values,"NULL,",NULL);
		g_free(values);
		values = values_aux;
	    }

	    if (d_payment_get_account_number (payment)!=NULL) {
		gchar * temp = d_payment_get_account_number (payment);
		values_aux = g_strconcat(values,temp,",",NULL);
		g_free(temp);
		g_free(values);
		values = values_aux;

	    } else {
		values_aux = g_strconcat(values,"NULL,",NULL);
		g_free(values);
		values = values_aux;
	    }


	    if (d_payment_get_is_transfer (payment)==TRUE) {
		values_aux = g_strconcat(values,"TRUE,",NULL);
		g_free(values);
		values = values_aux;
	    } else {
		values_aux = g_strconcat(values,"FALSE,",NULL);
		g_free(values);
		values = values_aux;
	    }

	    if (d_payment_get_is_cheque (payment)==TRUE) {
		values_aux = g_strconcat(values,"TRUE,",NULL);
		g_free(values);
		values = values_aux;
	    } else {
		values_aux = g_strconcat(values,"FALSE,",NULL);
		g_free(values);
		values = values_aux;
	    }


	    if (d_payment_c_util_get_cheque_number (payment)!=NULL) {
		gchar * temp = d_payment_c_util_get_cheque_number (payment);
		values_aux = g_strconcat(values,"'",temp,"',",NULL);
		g_free(temp);
		g_free(values);
		values = values_aux;

	    } else {
		values_aux = g_strconcat(values,"NULL,",NULL);
		g_free(values);
		values = values_aux;
	    }

	    if (d_payment_get_is_credit_card (payment)==TRUE) {
		values_aux = g_strconcat(values,"TRUE,",NULL);
		g_free(values);
		values = values_aux;
	    } else {
		values_aux = g_strconcat(values,"FALSE,",NULL);
		g_free(values);
		values = values_aux;
	    }

	    if (d_payment_get_credit_card_number (payment)!=NULL) {
		gchar * temp = d_payment_get_credit_card_number (payment);
		values_aux = g_strconcat(values,"'",temp,"',",NULL);
		g_free(temp);
		g_free(values);
		values = values_aux;

	    } else {
		values_aux = g_strconcat(values,"NULL,",NULL);;
		g_free(values);
		values = values_aux;

	    }

	    if (d_payment_get_is_credit_card (payment)==TRUE) {
		time_t temp = d_payment_get_expiry_date (payment);
		values_aux = g_strconcat(values,"'",ctime(&temp),"',",NULL);
		g_free(values);
		values = values_aux;

	    } else {
		values_aux = g_strconcat(values,"NULL,",NULL);;
		g_free(values);
		values = values_aux;

	    }

	    if (d_payment_get_is_paid_cash (payment)==TRUE) {
		values_aux = g_strconcat(values,"TRUE",NULL);
		g_free(values);
		values = values_aux;
	    } else {
		values_aux = g_strconcat(values,"FALSE",NULL);
		g_free(values);
		values = values_aux;
	    }

	    query_aux=g_strconcat(query1,values,")",NULL);
	    g_free(query1);
	    query1=query_aux;
	    query_aux=NULL;

	    if (m_model_command_execute (&record_set1, query1, connection, command,tuples,0, FALSE)==FALSE) {
                result = ULONG_MAX;
	    }

	    query2 = g_strdup("select currval('seqPago')");
	    if (m_model_command_execute (&record_set2, query2, connection, command,tuples,0, TRUE)==FALSE) {
                result = ULONG_MAX;
	    }

	    if ((record_set2!=NULL) && (result == 0)) {
		if (*tuples!=0) {
		    GdaField * field;
		    gda_recordset_move_first(record_set2);
		    field  = gda_recordset_field_idx(record_set2,0);
		    if (field!=NULL) {
			if (!gda_field_is_null(field)) {
			    gchar ** stop = (gchar **)g_malloc0(sizeof(gchar *));
                            g_free(payment_char);
			    payment_char = gda_stringify_value(NULL, 0, field);
			    result = strtoul(payment_char,stop,10);
			    g_free(stop);
			}
			gda_field_free(field);
		    }

		} else {
		    result = ULONG_MAX;
		}
	    } else {
		result = ULONG_MAX;
	    }


	} else {
	    time_t time_aux;
            gchar * gchar_where_aux;
            gchar * gchar_where =g_strdup(" where ");
	    query1 = g_strconcat(" UPDATE PAGO SET ",NULL);
	    values = g_strconcat(" ",NULL);

	    gchar_where_aux = g_strconcat(gchar_where," codigoPago=",payment_char," and codigoTienda= ",shop_char,
					  " and codigoEmpresa =",enterprise_char,NULL);
	    g_free(gchar_where);
	    gchar_where = gchar_where_aux;
            gchar_where_aux = NULL;


	    if (d_payment_get_achieved_payments (payment)!=ULONG_MAX) {
		gchar * temp = g_strdup_printf("%lu",d_payment_get_achieved_payments (payment));
		values_aux = g_strconcat(values,"pagosRealizados=",temp,",",NULL);
		g_free(temp);
		g_free(values);
		values = values_aux;
	    } 

	    if (d_payment_get_achieved_date (payment)!=time(NULL)) {
		time_t temp = d_payment_get_achieved_date (payment);
		values_aux = g_strconcat(values,"fechaRealizado=","'",ctime(&temp),"',",NULL);
		g_free(values);
		values = values_aux;
	    } 


	    if (d_payment_get_is_hire_purchase (payment)==TRUE) {
		values_aux = g_strconcat(values,"esAPlazos=TRUE,",NULL);
		g_free(values);
		values = values_aux;
	    } else {
		values_aux = g_strconcat(values,"esAPlazos=FALSE,",NULL);
		g_free(values);
		values = values_aux;
	    }

	    time_aux = d_payment_get_init_date (payment);
	    values_aux = g_strconcat(values,"fechaInicio='",ctime(&time_aux),"',",NULL);
	    g_free(values);
	    values = values_aux;


	    if (d_payment_get_entry_percentage (payment)!=-1) {
		gchar * temp =g_strdup_printf("%.2f",d_payment_get_entry_percentage (payment));
		u_commas_to_points(temp);
		values_aux = g_strconcat(values,"porcentajeEntrada=",temp,",",NULL);
		g_free(temp);
		g_free(values);
		values = values_aux;
	    } 

	    if (d_payment_get_number_expiries (payment)!=-1) {
		gchar * temp = g_strdup_printf("%d",d_payment_get_number_expiries (payment));
		values_aux = g_strconcat(values,"numerovencimientos=",temp,",",NULL);
		g_free(temp);
		g_free(values);
		values = values_aux;
	    } 

	    if (d_payment_get_interval_days (payment)!=-1) {
		gchar * temp = g_strdup_printf("%d",d_payment_get_interval_days (payment));
		values_aux = g_strconcat(values,"diasIntervalo=",temp,",",NULL);
		g_free(temp);
		g_free(values);
		values = values_aux;
	    }

	    if (d_payment_get_is_direct_debit (payment)==TRUE) {
		values_aux = g_strconcat(values,"esDomiciliacion=TRUE,",NULL);
		g_free(values);
		values = values_aux;
	    } else {
		values_aux = g_strconcat(values,"esDomiciliacion=FALSE,",NULL);
		g_free(values);
		values = values_aux;
	    }

	    if (d_payment_get_bank_code (payment)!=-1) {
		gchar * temp =g_strdup_printf("%d",d_payment_get_bank_code (payment));
		values_aux = g_strconcat(values,"codigoBanco=",temp,",",NULL);
		g_free(temp);
		g_free(values);
		values = values_aux;

	    } 

	    if (d_payment_get_office_code (payment)!=-1) {
		gchar * temp =g_strdup_printf("%d",d_payment_get_office_code (payment));
		values_aux = g_strconcat(values,"codigoOficina=",temp,",",NULL);
		g_free(temp);
		g_free(values);
		values = values_aux;

	    } 

	    if (d_payment_get_account_number (payment)!=NULL) {
		gchar * temp = d_payment_get_account_number (payment);
		values_aux = g_strconcat(values,"numeroCuenta=",temp,",",NULL);
		g_free(temp);
		g_free(values);
		values = values_aux;

	    }

	    if (d_payment_get_is_transfer (payment)==TRUE) {
		values_aux = g_strconcat(values,"esTransferencia=TRUE,",NULL);
		g_free(values);
		values = values_aux;
	    } else {
		values_aux = g_strconcat(values,"esTransferencia=FALSE,",NULL);
		g_free(values);
		values = values_aux;
	    }

	    if (d_payment_get_is_cheque (payment)==TRUE) {
		values_aux = g_strconcat(values,"esTalon=TRUE,",NULL);
		g_free(values);
		values = values_aux;
	    } else {
		values_aux = g_strconcat(values,"esTalon=FALSE,",NULL);
		g_free(values);
		values = values_aux;
	    }


	    if (d_payment_c_util_get_cheque_number (payment)!=NULL) {
		gchar * temp = d_payment_c_util_get_cheque_number (payment);
		values_aux = g_strconcat(values,"numeroTalon='",temp,"',",NULL);
		g_free(temp);
		g_free(values);
		values = values_aux;

	    } 

	    if (d_payment_get_is_credit_card (payment)==TRUE) {
		values_aux = g_strconcat(values,"esTarjeta=TRUE,",NULL);
		g_free(values);
		values = values_aux;
	    } else {
		values_aux = g_strconcat(values,"esTarjeta=FALSE,",NULL);
		g_free(values);
		values = values_aux;
	    }

	    if (d_payment_get_credit_card_number (payment)!=NULL) {
		gchar * temp = d_payment_get_credit_card_number (payment);
		values_aux = g_strconcat(values,"numeroTarjeta='",temp,"',",NULL);
		g_free(temp);
		g_free(values);
		values = values_aux;

	    }

	    if (d_payment_get_is_credit_card (payment)==TRUE) {
		time_t temp = d_payment_get_expiry_date (payment);
		values_aux = g_strconcat(values,"validaHasta='",ctime(&temp),"',",NULL);
		g_free(values);
		values = values_aux;

	    } else {
		values_aux = g_strconcat(values,"validaHasta=NULL,",NULL);
		g_free(values);
		values = values_aux;
	    }

	    if (d_payment_get_is_paid_cash (payment)==TRUE) {
		values_aux = g_strconcat(values,"esPagoContado=TRUE",NULL);
		g_free(values);
		values = values_aux;
	    } else {
		values_aux = g_strconcat(values,"esPagoContado=FALSE",NULL);
		g_free(values);
		values = values_aux;
	    }

	    query_aux=g_strconcat(query1,values,gchar_where,NULL);
            g_free(gchar_where);
            g_free(values);
	    g_free(query1);
	    query1=query_aux;
	    query_aux=NULL;

	    if (m_model_command_execute (&record_set1, query1, connection, command,tuples,0, FALSE)==FALSE) {
                result =ULONG_MAX;
	    }

	    if (result !=ULONG_MAX) {
		result = payment_code;
	    }
	}

    } 

    if (query2!=NULL)
	query = g_strconcat (query1,";\n",query2,";\n",NULL);
    else
	query = g_strconcat (query1,";\n",NULL);

    if (query2!=NULL) g_free(query2);
    g_free(query1);

    g_free(tuples);
    g_free(query);
    g_free(payment_char);
    g_free(enterprise_char);
    g_free(shop_char);


    gda_recordset_free(record_set1);
    if (record_set2!=NULL) {
	gda_recordset_free(record_set2);
    }

    return result;

}



/**
 * m_payment_calculate_next_total:
 * @payment_id: Pago para el que se calcular el total del prximo movimiento.
 * @is_direct_debit: Indica si el pago se calcula para una domiciliacin (post-generacin del cuaderno). Para
 * el clculo del movimiento.
 *
 * Funcin utilizada en los pagos a plazos, para calcular cual es el total del siguiente plazo que
 * debe ser pagado.
 *
 * Returns:
 * La cantidad que se debe pagar en el siguiente plazo.
 */
gdouble m_payment_calculate_next_total (PaymentId * payment_id,gboolean is_direct_debit) {
    GdaCommand         *my_command;
    gchar              *gchar_query=NULL;
    gchar              *gchar_select=NULL;
    gchar              *gchar_where=NULL;
    gchar              *gchar_from=NULL;
    gchar              *gchar_query_invoice=NULL;
    gchar              *gchar_select_invoice=NULL;
    gchar              *gchar_where_invoice=NULL;
    gchar              *gchar_from_invoice=NULL;
    GdaConnection      *my_connection = m_model_get_connection ();
    GdaRecordset       *record_set = NULL;
    gdouble total=G_MINDOUBLE;
    gdouble deposit_percentage=-1;
    gulong * tuples;
    gboolean is_hire_purchase=FALSE;
    time_t initial_date=time(NULL);
    gulong instalment_paid=ULONG_MAX;
    gshort instalment_number=-1;
    gulong index;
    gboolean invoice=FALSE;


    my_command = gda_command_new();
    gda_command_set_connection(my_command,my_connection);

    tuples = (gulong *) g_malloc0(sizeof(gulong));

    gchar_select = g_strconcat("select to_char(total,'99999999D99') as total,numeroVencimientos,pagosRealizados,fechaInicio,esaplazos,porcentajeEntrada ",
			       NULL);
    gchar_from= g_strconcat(" from facturacargo,pago ",NULL);

    gchar_where = g_strconcat("where facturacargo.codigoPago = pago.codigoPago ",
                              " and facturacargo.codigoPago=",payment_id->payment_code,
                              " and facturacargo.codigoEmpresa=",payment_id->enterprise_code,
			      " and facturacargo.codigoTienda=",payment_id->shop_code,NULL);

    gchar_select_invoice = g_strconcat("select to_char(baseImponibleNueva,'99999999D99') as baseImponibleNueva,numeroVencimientos,pagosRealizados,fechaInicio,esaplazos,porcentajeEntrada ",
			       NULL);
    gchar_from_invoice= g_strconcat(" from facturaabono,pago ",NULL);

    gchar_where_invoice = g_strconcat("where facturaabono.codigoPago = pago.codigoPago ",
                              " and facturaabono.codigoPago=",payment_id->payment_code,
                              " and facturaabono.codigoEmpresa=",payment_id->enterprise_code,
			      " and facturaabono.codigoTienda=",payment_id->shop_code,NULL);

    gchar_query = g_strconcat(gchar_select,gchar_from,gchar_where, NULL);

    gchar_query_invoice = g_strconcat(gchar_select_invoice,gchar_from_invoice,gchar_where_invoice, NULL);

    g_print(_("\nQuery: %s\n"),gchar_query);

    g_print(_("\nQuery: %s\n"),gchar_query_invoice);

    if (m_model_command_execute (&record_set, gchar_query, my_connection, my_command,tuples,0, TRUE)==FALSE) {
        total = (G_MINDOUBLE+1);
    } 
    if ((record_set==NULL) || (*tuples==0)) {
	if (record_set!=NULL)
	    gda_recordset_free(record_set);
        record_set=NULL;
	if (m_model_command_execute (&record_set, gchar_query_invoice, my_connection, my_command,tuples,0, TRUE)==FALSE) {
	    total = (G_MINDOUBLE+1);
	}
        invoice = TRUE;
    } else {
        invoice = FALSE;
    }

    if ((record_set!=NULL) && (total==G_MINDOUBLE)) {
	if (*tuples==1) {
            GdaField * field;
            gchar * total_gchar;
	    index = gda_recordset_move_first(record_set);
            if (invoice == FALSE)
		field = gda_recordset_field_name(record_set,"total");
            else
		field = gda_recordset_field_name(record_set,"baseImponibleNueva");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    total_gchar = gda_stringify_value(NULL,0,field);
                    g_strstrip(total_gchar);
		    total = atof(total_gchar);
		    g_free(total_gchar);
		}
                gda_field_free(field);
	    }
	    field = gda_recordset_field_name(record_set,"numerovencimientos");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    instalment_number = gda_field_get_smallint_value(field);
		}
                gda_field_free(field);
	    }
	    field = gda_recordset_field_name(record_set,"pagosrealizados");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    instalment_paid = igalia_field_get_bigint_value (field);
		}
		gda_field_free(field);
	    }
	    field = gda_recordset_field_name(record_set,"fechainicio");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    gchar * temp;
		    time_t * my_time;
		    temp=gda_stringify_value(NULL, 0, field);
		    my_time = u_parse_date (temp);
		    initial_date = (*my_time);
		    my_time = NULL;
		    g_free(temp);
		}
	    }
	    field = gda_recordset_field_name(record_set,"porcentajeentrada");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    total_gchar = gda_stringify_value(NULL,0,field);
		    deposit_percentage = atof(total_gchar);
		    g_free(total_gchar);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(record_set,"esAPlazos");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    is_hire_purchase = gda_field_get_boolean_value(field);
		}
                gda_field_free(field);
	    }
	    if (is_hire_purchase!=FALSE) {
		if (is_direct_debit == FALSE) {
		    if (instalment_paid>0) {
			total = ((total - (deposit_percentage/100.0*total))/(instalment_number-1));
		    } else {
			if (deposit_percentage == 0) {
			    total = (total/instalment_number);
			} else {
			    total = total * (deposit_percentage/100.0);
			}
		    }
		} else {
		    if (instalment_paid>1) {
			total = ((total - (deposit_percentage/100.0*total))/(instalment_number-1));
		    } else {
			if (deposit_percentage == 0) {
			    total = (total/instalment_number);
			} else {
			    total = total * (deposit_percentage/100.0);
			}
		    }
		}
	    }
	} else {
	    total = G_MINDOUBLE;
	}
	gda_recordset_close(record_set);
	gda_recordset_free(record_set);
    } else {
	total = G_MINDOUBLE;
    }

    g_free(gchar_where);
    g_free(gchar_select);
    g_free(gchar_from);
    g_free(gchar_query);
    g_free(tuples);



    igalia_command_free(my_command);
    g_print(_("Freeing database completed\n"));

    return total;


}

/**
 * m_payment_insert_in_transaction:
 * @payment: Pago a insertar en la base de datos.
 * @command: Comando en el que se ejecutar la sentencia sql.
 * @connection: Conexin a la base de datos.
 *
 * Inserta un pago en la base de datos, dentro de una transaccin.
 *
 * Returns:
 * %ULONG_MAX en caso de que no haya sido insertado en pago, el cdigo del pago en otro caso.
 */
gulong m_payment_insert_in_transaction (Payment * payment,GdaCommand * command,GdaConnection * connection) {
    gulong result=0;
    gchar * query1=NULL;
    gchar * query2=NULL;
    gchar * query_aux;
    gchar * values=NULL;
    gchar * values_aux;
    gchar * enterprise_char=NULL;
    gchar * payment_char=NULL;
    gchar * shop_char=NULL;
    gulong shop_code=ULONG_MAX;
    gulong enterprise_code=ULONG_MAX;
    gulong payment_code = ULONG_MAX;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    GdaRecordset *record_set1=NULL;
    GdaRecordset *record_set2=NULL;

    
    shop_code=d_participant_get_code (d_shop_get_participant (d_payment_get_shop (
									      payment
									     )));
    shop_char = g_strdup_printf("%lu",shop_code);
    enterprise_code=d_participant_get_code (d_enterprise_get_participant (d_payment_get_enterprise (
											   payment
											  )));
    enterprise_char = g_strdup_printf("%lu",enterprise_code);


    if ((enterprise_code!=ULONG_MAX) && (shop_code!=ULONG_MAX)) {
	payment_code = d_payment_get_payment_code (payment);
	payment_char = g_strdup_printf("%lu",payment_code);

	if (payment_code==ULONG_MAX) {
	    time_t time_aux;
            gchar * temp_nextval=NULL;
	    query1 = g_strconcat(" INSERT INTO PAGO VALUES ",NULL);
	    values = g_strconcat(" (",NULL);
	    if (d_participant_get_code (d_enterprise_get_participant (d_payment_get_enterprise (payment)))!=ULONG_MAX) {
		gchar * temp = g_strdup_printf("%lu",d_participant_get_code (d_enterprise_get_participant (d_payment_get_enterprise (payment))));
		values_aux = g_strconcat(values,temp,",",NULL);
		g_free(values);
		g_free(temp);
		values = values_aux;
	    } else {
		values_aux = g_strconcat(values,"NULL,",NULL);
		g_free(values);
		values = values_aux;

	    }
	    if (d_participant_get_code (d_shop_get_participant (d_payment_get_shop (payment)))!=ULONG_MAX) {
		gchar * temp = g_strdup_printf("%lu",d_participant_get_code (d_shop_get_participant (d_payment_get_shop (payment))));
		values_aux = g_strconcat(values,temp,",",NULL);
		g_free(values);
		g_free(temp);
		values = values_aux;

	    } else {
		values_aux = g_strconcat(values,"NULL,",NULL);
		g_free(values);
		values = values_aux;

	    }
            temp_nextval=m_model_nextval_sequence ("seqPago",connection);
	    values_aux = g_strconcat(values,temp_nextval,",",NULL);
	    g_free(values);
	    values = values_aux;
            g_free(temp_nextval);

	    if (d_payment_get_achieved_payments (payment)!=ULONG_MAX) {
		gchar * temp = g_strdup_printf("%lu",d_payment_get_achieved_payments (payment));
		values_aux = g_strconcat(values,temp,",",NULL);
		g_free(temp);
		g_free(values);
		values = values_aux;
	    } else {
		values_aux = g_strconcat(values,"NULL,",NULL);
		g_free(values);
		values = values_aux;
	    }

	    if (d_payment_get_achieved_date (payment)!=time(NULL)) {
		time_t temp = d_payment_get_achieved_date (payment);
		values_aux = g_strconcat(values,"'",ctime(&temp),"',",NULL);
		g_free(values);
		values = values_aux;

	    } else {
		values_aux = g_strconcat(values,"NULL,",NULL);
		g_free(values);
		values = values_aux;

	    }


	    if (d_payment_get_is_hire_purchase (payment)==TRUE) {
		values_aux = g_strconcat(values,"TRUE,",NULL);
		g_free(values);
		values = values_aux;
	    } else {
		values_aux = g_strconcat(values,"FALSE,",NULL);
		g_free(values);
		values = values_aux;
	    }

	    time_aux = d_payment_get_init_date (payment);
	    values_aux = g_strconcat(values,"'",ctime(&time_aux),"',",NULL);
	    g_free(values);
	    values = values_aux;


	    if (d_payment_get_entry_percentage (payment)!=-1) {
		gchar * temp =g_strdup_printf("%.2f",d_payment_get_entry_percentage (payment));
		u_commas_to_points(temp);
		values_aux = g_strconcat(values,temp,",",NULL);
		g_free(temp);
		g_free(values);
		values = values_aux;
	    } else {
		values_aux = g_strconcat(values,"NULL,",NULL);
		g_free(values);
		values = values_aux;
	    }

	    if (d_payment_get_number_expiries (payment)!=-1) {
		gchar * temp = g_strdup_printf("%d",d_payment_get_number_expiries (payment));
		values_aux = g_strconcat(values,temp,",",NULL);
		g_free(temp);
		g_free(values);
		values = values_aux;
	    } else {
		values_aux = g_strconcat(values,"NULL,",NULL);
		g_free(values);
		values = values_aux;
	    }

	    if (d_payment_get_interval_days (payment)!=-1) {
		gchar * temp = g_strdup_printf("%d",d_payment_get_interval_days (payment));
		values_aux = g_strconcat(values,temp,",",NULL);
		g_free(temp);
		g_free(values);
		values = values_aux;

	    } else {
		values_aux = g_strconcat(values,"NULL,",NULL);
		g_free(values);
		values = values_aux;

	    }

	    if (d_payment_get_is_direct_debit (payment)==TRUE) {
		values_aux = g_strconcat(values,"TRUE,",NULL);
		g_free(values);
		values = values_aux;
	    } else {
		values_aux = g_strconcat(values,"FALSE,",NULL);
		g_free(values);
		values = values_aux;
	    }

	    if (d_payment_get_bank_code (payment)!=-1) {
		gchar * temp =g_strdup_printf("%d",d_payment_get_bank_code (payment));
		values_aux = g_strconcat(values,temp,",",NULL);
		g_free(temp);
		g_free(values);
		values = values_aux;

	    } else {
		values_aux = g_strconcat(values,"NULL,",NULL);
		g_free(values);
		values = values_aux;
	    }

	    if (d_payment_get_office_code (payment)!=-1) {
		gchar * temp =g_strdup_printf("%d",d_payment_get_office_code (payment));
		values_aux = g_strconcat(values,temp,",",NULL);
		g_free(temp);
		g_free(values);
		values = values_aux;

	    } else {
		values_aux = g_strconcat(values,"NULL,",NULL);
		g_free(values);
		values = values_aux;
	    }

	    if (d_payment_get_account_number (payment)!=NULL) {
		gchar * temp = d_payment_get_account_number (payment);
		values_aux = g_strconcat(values,temp,",",NULL);
		g_free(temp);
		g_free(values);
		values = values_aux;

	    } else {
		values_aux = g_strconcat(values,"NULL,",NULL);
		g_free(values);
		values = values_aux;
	    }


	    if (d_payment_get_is_transfer (payment)==TRUE) {
		values_aux = g_strconcat(values,"TRUE,",NULL);
		g_free(values);
		values = values_aux;
	    } else {
		values_aux = g_strconcat(values,"FALSE,",NULL);
		g_free(values);
		values = values_aux;
	    }

	    if (d_payment_get_is_cheque (payment)==TRUE) {
		values_aux = g_strconcat(values,"TRUE,",NULL);
		g_free(values);
		values = values_aux;
	    } else {
		values_aux = g_strconcat(values,"FALSE,",NULL);
		g_free(values);
		values = values_aux;
	    }


	    if (d_payment_c_util_get_cheque_number (payment)!=NULL) {
		gchar * temp = d_payment_c_util_get_cheque_number (payment);
		values_aux = g_strconcat(values,"'",temp,"',",NULL);
		g_free(temp);
		g_free(values);
		values = values_aux;

	    } else {
		values_aux = g_strconcat(values,"NULL,",NULL);
		g_free(values);
		values = values_aux;
	    }

	    if (d_payment_get_is_credit_card (payment)==TRUE) {
		values_aux = g_strconcat(values,"TRUE,",NULL);
		g_free(values);
		values = values_aux;
	    } else {
		values_aux = g_strconcat(values,"FALSE,",NULL);
		g_free(values);
		values = values_aux;
	    }

	    if (d_payment_get_credit_card_number (payment)!=NULL) {
		gchar * temp = d_payment_get_credit_card_number (payment);
		values_aux = g_strconcat(values,"'",temp,"',",NULL);
		g_free(temp);
		g_free(values);
		values = values_aux;

	    } else {
		values_aux = g_strconcat(values,"NULL,",NULL);;
		g_free(values);
		values = values_aux;

	    }

	    if (d_payment_get_is_credit_card (payment)==TRUE) {
		time_t temp = d_payment_get_expiry_date (payment);
		values_aux = g_strconcat(values,"'",ctime(&temp),"',",NULL);
		g_free(values);
		values = values_aux;

	    } else {
		values_aux = g_strconcat(values,"NULL,",NULL);;
		g_free(values);
		values = values_aux;

	    }

	    if (d_payment_get_is_paid_cash (payment)==TRUE) {
		values_aux = g_strconcat(values,"TRUE",NULL);
		g_free(values);
		values = values_aux;
	    } else {
		values_aux = g_strconcat(values,"FALSE",NULL);
		g_free(values);
		values = values_aux;
	    }

	    query_aux=g_strconcat(query1,values,")",NULL);
	    g_free(query1);
	    query1=query_aux;
	    query_aux=NULL;

            
	    if (m_model_command_execute (&record_set1, query1, connection, command,tuples,0, FALSE)==FALSE) {
                result = ULONG_MAX;
	    }

	    query2 = g_strdup("select currval('seqPago')");
	    if (m_model_command_execute (&record_set2, query2, connection, command,tuples,0, TRUE)==FALSE) {
                result = ULONG_MAX;
	    }

	    if ((record_set2!=NULL) && (result == 0)) {
		if (*tuples!=0) {
		    GdaField * field;
		    gda_recordset_move_first(record_set2);
		    field  = gda_recordset_field_idx(record_set2,0);
		    if (field!=NULL) {
			if (!gda_field_is_null(field)) {
			    gchar ** stop = (gchar **)g_malloc0(sizeof(gchar *));
                            g_free(payment_char);
			    payment_char = gda_stringify_value(NULL, 0, field);
			    result = strtoul(payment_char,stop,10);
			    g_free(stop);
			}
			gda_field_free(field);
		    }

		} else {
		    result = ULONG_MAX;
		}
	    } else {
		result = ULONG_MAX;
	    }
	} else {
	    result = 0;
	}
    } else {
	result = 0;
    }

    return result;
}

/**
 * m_payment_delete:
 * @shop_code: Cdigo de la tienda en la que se realiz el pago.
 * @enterprise_code: Cdigo de la empresa a la que pertenece la tienda en la que se realiz el pago.
 * @payment_code: Cdigo del pago realizado.
 * @operation_type: Tipo de movimiento: de caja o bancario.
 *
 * Borra un pago de la base de datos
 *
 * Returns:
 * %FALSE en caso de que no haya sido borrado pago.
 */
gboolean m_payment_delete (gulong shop_code,gulong enterprise_code,gulong payment_code,gint operation_type) {
    GdaConnection * connection = m_model_get_connection ();
    GdaCommand * command = gda_command_new();
    gboolean       result=FALSE;

    gda_command_set_connection(command,connection);

    m_model_connection_begin_mutex_transaction_logged (connection,
						    enterprise_code,
						    shop_code,
						    ULONG_MAX
						   );

    if (operation_type==0) {
	gchar         *gchar_query= NULL;
	GdaRecordset  *record_set = NULL;
	gchar         *temp = NULL;
	gulong        *tuples = (gulong *) g_malloc0(sizeof(gulong));

        temp = g_strdup_printf("%lu",payment_code);
	gchar_query = g_strconcat("DELETE FROM MOVIMIENTO WHERE CODIGOPAGO=",temp,NULL);

	g_free(temp);

	result = m_model_command_execute (&record_set, gchar_query,
				       connection, command, tuples, 0, FALSE);

	gda_recordset_free(record_set);
	g_free(gchar_query);
	g_free(tuples);

    } else {
	gchar         *gchar_query= NULL;
	GdaRecordset  *record_set = NULL;
	gchar         *shop_temp = NULL;
	gchar         *enter_temp = NULL;
	gchar         *temp = NULL;
	gulong        *tuples = (gulong *) g_malloc0(sizeof(gulong));

        temp = g_strdup_printf("%lu",payment_code);
        shop_temp = g_strdup_printf("%lu",shop_code);
        enter_temp = g_strdup_printf("%lu",enterprise_code);
	gchar_query = g_strconcat("DELETE FROM MOVIMIENTOCAJA WHERE CODIGOPAGO=",temp
				  ," AND CODIGOTIENDA=",shop_temp," AND CODIGOEMPRESA=",enter_temp,NULL);

	g_free(temp);
	g_free(shop_temp);
	g_free(enter_temp);

	result = m_model_command_execute (&record_set, gchar_query,
				       connection, command, tuples, 0, FALSE);

	gda_recordset_free(record_set);
	g_free(gchar_query);
	g_free(tuples);


    }

    result = m_model_execute_mutex_modification_logged (connection) && result;

    return result;
}

/**
 * m_payment_is_pay_instalment:
 * @payment_to_pay: Doble puntero al pago a modificar.
 *
 * Modifica los datos de un pago a plazos para indicar que est en proceso de pago.
 *
 * Returns:
 * %FALSE en caso de que no se haya pagado correctamente el plazo.
 */
gboolean m_payment_is_pay_instalment (Payment ** payment_to_pay) {
    PaymentId * payment_id = (PaymentId *) g_malloc0(sizeof(PaymentId));
    Payment * payment;
    gboolean error=FALSE;
    gint paid_instalment;
    gint instalment_number;

    payment_id -> payment_code = g_strdup_printf("%lu",d_payment_get_payment_code (payment_to_pay[0]));
    payment_id -> shop_code = g_strdup_printf("%lu",d_participant_get_code (d_shop_get_participant (d_payment_get_shop (payment_to_pay[0]))));
    payment_id -> enterprise_code = g_strdup_printf("%lu",d_participant_get_code (d_enterprise_get_participant (d_payment_get_enterprise (payment_to_pay[0]))));

    payment = m_payment_find_by_id (payment_id);

    paid_instalment = d_payment_get_achieved_payments (payment);
    instalment_number = d_payment_get_number_expiries (payment);
    if (paid_instalment==(instalment_number-1)) {
	d_payment_set_achieved_payments (payment,instalment_number);
	d_payment_set_achieved_payments (payment_to_pay[0],instalment_number);
	d_payment_set_achieved_date (payment,time(NULL));
	d_payment_set_achieved_date (payment_to_pay[0],time(NULL));
	d_payment_set_init_date (payment,time(NULL));
	d_payment_set_init_date (payment_to_pay[0],time(NULL));
	
    } else {
	time_t * time_aux = (time_t *) g_malloc0(sizeof(time_t));
	struct tm * time_tm;// = (struct tm *) g_malloc0(sizeof(struct tm));
	GDate * date;
	gint days;

        paid_instalment=paid_instalment+1;
	d_payment_set_achieved_payments (payment,paid_instalment);
	d_payment_set_achieved_payments (payment_to_pay[0],paid_instalment);
	*time_aux = d_payment_get_init_date (payment);
	time_tm = localtime(time_aux);
	date = g_date_new_dmy(time_tm->tm_mday,time_tm->tm_mon+1,time_tm->tm_year+1900);
	days = d_payment_get_interval_days (payment);
	g_date_add_days(date,days);
	time_tm ->tm_mday = g_date_day(date);
	time_tm ->tm_mon= g_date_month(date)-1;
	time_tm ->tm_year = g_date_year(date)-1900;
	*time_aux = mktime(time_tm);
	d_payment_set_init_date (payment,*time_aux);
	d_payment_set_init_date (payment_to_pay[0],*time_aux);
	g_free(time_aux);
    }
    error = m_payment_update (payment);

    g_free(payment_id->payment_code);
    g_free(payment_id->shop_code);
    g_free(payment_id->enterprise_code);
    d_payment_clear (payment);
    g_free(payment_id);
    return error;
}

/**
 * m_payment_get_instalment:
 * @payment_to_pay: Doble puntero al pago a modificar.
 * @connection: Conexin a la BD.
 *
 * Modifica los datos de un pago a plazos para retroceder el pago del plazo.
 *
 * Returns:
 * %FALSE en caso de que no se haya retrocedido correctamente el plazo.
 */
gboolean m_payment_get_instalment (Payment ** payment_to_pay,GdaConnection * connection) {
    Payment * payment;
    PaymentId * payment_id = (PaymentId *) g_malloc0(sizeof(PaymentId));
    gboolean error=FALSE;
    gint paid_instalment;
    gint instalment_number;
    GdaCommand * command = gda_command_new();

    payment_id -> payment_code = g_strdup_printf("%lu",d_payment_get_payment_code (payment_to_pay[0]));
    payment_id -> shop_code = g_strdup_printf("%lu",d_participant_get_code (d_shop_get_participant (d_payment_get_shop (payment_to_pay[0]))));
    payment_id -> enterprise_code = g_strdup_printf("%lu",d_participant_get_code (d_enterprise_get_participant (d_payment_get_enterprise (payment_to_pay[0]))));

    gda_command_set_connection(command,connection);

    payment = m_payment_find_by_id (payment_id);

    paid_instalment = d_payment_get_achieved_payments (payment);
    instalment_number = d_payment_get_number_expiries (payment);

    if (paid_instalment==0) {
	d_payment_set_achieved_date (payment,time(NULL));
	d_payment_set_init_date (payment,time(NULL));
	d_payment_set_achieved_date (payment_to_pay[0],time(NULL));
	d_payment_set_init_date (payment_to_pay[0],time(NULL));
    } else {
	time_t * time_aux = (time_t *) g_malloc0(sizeof(time_t));
	struct tm * time_tm;// = (struct tm *) g_malloc0(sizeof(struct tm));
	GDate * date;
	gint days;

        paid_instalment = paid_instalment-1;
	d_payment_set_achieved_payments (payment,paid_instalment);
	d_payment_set_achieved_payments (payment_to_pay[0],paid_instalment);
	*time_aux = d_payment_get_init_date (payment);
	time_tm = localtime(time_aux);
	date = g_date_new_dmy(time_tm->tm_mday,time_tm->tm_mon+1,time_tm->tm_year+1900);
	days = d_payment_get_interval_days (payment);
	g_date_subtract_days(date,days);
	time_tm ->tm_mday = g_date_day(date);
	time_tm ->tm_mon= g_date_month(date)-1;
	time_tm ->tm_year = g_date_year(date)-1900;
	*time_aux = mktime(time_tm);
	d_payment_set_init_date (payment,*time_aux);
	d_payment_set_init_date (payment_to_pay[0],*time_aux);
	g_free(time_aux);
    }
    error = m_payment_update_in_transaction (payment,connection,command);
    g_free(payment_id->payment_code);
    g_free(payment_id->shop_code);
    g_free(payment_id->enterprise_code);
    d_payment_clear (payment);
    g_free(payment_id);
    igalia_command_free(command);
    return error;
}

/**
 * m_payment_find_number_of_operation:
 * @payment_code: Cdigo del pago.
 * @shop_code: Cdigo de la tienda.
 * @enterprise_code: Cdigo de la empresa.
 *
 * Busca el nmero de pagos realizados para los pagos a plazos.
 *
 * Returns:
 * Nmero de movimientos realizados para el pago.
 */
gint m_payment_find_number_of_operation (gulong payment_code,gulong shop_code,gulong enterprise_code) {
    GdaCommand         *my_command;
    gchar              *gchar_query=NULL;
    gchar              *gchar_select=NULL;
    gchar              *gchar_where=NULL;
    gchar              *gchar_from=NULL;
    GdaConnection      *my_connection = m_model_get_connection ();
    GdaRecordset       *record_set = NULL;
    gchar * payment_code_char = g_strdup_printf("%lu",payment_code);
    gchar * enterprise_code_char = g_strdup_printf("%lu",enterprise_code);
    gchar * shop_code_char = g_strdup_printf("%lu",shop_code);
    gint result=-1;
    gulong * tuples;
    gulong index;


    my_command = gda_command_new();
    gda_command_set_connection(my_command,my_connection);

    tuples = (gulong *) g_malloc0(sizeof(gulong));

    gchar_select = g_strconcat("select COUNT(*)",NULL);
    gchar_from= g_strconcat(" from pago,movimiento ",NULL);
    gchar_where = g_strconcat("where pago.codigoPago = movimiento.codigoPago ",
                              " and pago.codigoPago=",payment_code_char,
                              " and pago.codigoEmpresa=",enterprise_code_char,
			      " and pago.codigoTienda=",shop_code_char,NULL);

    gchar_query = g_strconcat(gchar_select,gchar_from,gchar_where, NULL);

    g_print(_("\nQuery: %s\n"),gchar_query);

    m_model_command_execute (&record_set, gchar_query, my_connection, my_command,tuples,0,TRUE);


    if (record_set!=NULL) {
	if (*tuples==1) {
	    GdaField * field;

	    index = gda_recordset_move_first(record_set);
	    if (index==GDA_RECORDSET_INVALID_POSITION) return -1;
	    field = gda_recordset_field_idx(record_set,0);
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    result = igalia_field_get_bigint_value (field);
		}
	    }
	}
	gda_recordset_close(record_set);
	gda_recordset_free(record_set);
    }

    g_free(payment_code_char);
    g_free(enterprise_code_char);
    g_free(shop_code_char);
    g_free(gchar_where);
    g_free(gchar_select);
    g_free(gchar_from);
    g_free(gchar_query);
    g_free(tuples);



    igalia_command_free(my_command);
    g_print(_("Freeing database completed\n"));

    return result;

}

/**
 * m_payment_find_by_invoice_id:
 * @invoice_code: Cdigo de factura.
 * @shop_code: Cdigo de la tienda.
 * @enterprise_code: Cdigo de la empresa.
 *
 * Busca un pago en la base de datos a partir del identificador de la factura emitida que lo referencia.
 *
 * Returns:
 * El pago solicitado si existe y la operacin es correcta, %NULL en otro caso.
 */
Payment * m_payment_find_by_invoice_id (gulong invoice_code,gulong shop_code,gulong enterprise_code){
    GdaCommand         *my_command;
    gchar              *gchar_select_invoice=NULL;
    GdaConnection      *my_connection = m_model_get_connection ();
    GdaRecordset       *record_set_invoice = NULL;
    gchar * enterprise_code_char=g_strdup_printf("%lu",enterprise_code);
    gchar * payment_code_char=NULL;
    gchar * shop_code_char=g_strdup_printf("%lu",shop_code);
    gchar * invoice_code_char = g_strdup_printf("%lu",invoice_code);
    Payment * payment=NULL;
    PaymentId * payment_id = (PaymentId *) g_malloc0(sizeof(PaymentId));
    gulong * tuples;
    gulong index;


    my_command = gda_command_new();
    gda_command_set_connection(my_command,my_connection);

    gchar_select_invoice = g_strconcat("select pago.codigoPago from",
			       " pago,facturaabono where pago.codigoPago=facturaabono.codigoPago",
			       " and pago.codigoTienda=facturaabono.codigoTienda and pago.codigoEmpresa=facturaabono.codigoEmpresa",
			       " and facturaabono.codigoFactura=",invoice_code_char,
			       " and facturaabono.codigoEmpresa=",enterprise_code_char,
			       " and facturaabono.codigoTienda=",shop_code_char,
			       NULL);

    tuples = (gulong *) g_malloc0(sizeof(gulong));

    m_model_command_execute (&record_set_invoice, gchar_select_invoice, my_connection, my_command,tuples,0,TRUE);


    if (record_set_invoice!=NULL) {
	if (*tuples==1) {
            GdaField * field;
	    index = gda_recordset_move_first(record_set_invoice);
	    if (index==GDA_RECORDSET_INVALID_POSITION) return NULL;
            field = gda_recordset_field_idx(record_set_invoice,0);
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
                    payment_code_char =gda_stringify_value(NULL, 0, field);
		} else {
		    payment_code_char=g_strdup("0");
		}
	    } else {
		payment_code_char=g_strdup("0");
	    }


	} else {
	    payment_code_char=g_strdup("0");
	}
	gda_recordset_close(record_set_invoice);
	gda_recordset_free(record_set_invoice);
    } else {
	payment_code_char=g_strdup("0");
    }

    payment_id -> shop_code = g_strdup(shop_code_char);
    payment_id -> enterprise_code =g_strdup(enterprise_code_char);
    payment_id -> payment_code =   g_strdup(payment_code_char);

    payment = m_payment_find_by_id (payment_id);


    g_free(gchar_select_invoice);
    g_free(invoice_code_char);
    g_free(payment_code_char);
    g_free(enterprise_code_char);
    g_free(shop_code_char);
    g_free(tuples);



    igalia_command_free(my_command);
    g_print(_("Freeing database completed\n"));

    return payment;


}




