/* receipt_model.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#define M_FIRST_ERROR_MESSAGE "Error en los datos del RECIBO"
#define TABLE         "Recibo"
#define ATTRIBUTES    "numeroRecibo as numeroreciboreceipt , \
                       numeroPoliza as numeropolizareceipt , \
                       codigoCompania as codigocompaniareceipt, \
                       codigoTitular as codigotitularreceipt , \
                       sucursalMediador as sucursalmediadorreceipt , \
                       inicioValidez as iniciovalidezreceipt, \
                       finValidez as finvalidezreceipt"

#include "receipt_model.h"

#include "util.h"



/**
 * m_receipt_get_table:
 *
 * Devuelve un string con la tabla correspondiente al objeto de la base de datos. Si el
 * objeto se corresponde con varias tablas se devolveran los nombres separados por comas.
 *
 * Returns:
 * Cadena de caracteres con la tabla o lista de tablas a las que se refiere el objeto.
 */
gchar *m_receipt_get_table () {
    return TABLE;
}

/**
 * m_receipt_get_attributes:
 *
 * Funcin que obtiene los atributos de un objeto, estos atributos son los que corresponden al
 * objeto en la base de datos.
 *
 * Returns:
 * Cadena de caracteres con los atributos a los que se refiere el objeto con un alias.
 */
gchar *m_receipt_get_attributes () {
    return ATTRIBUTES;
}


/**
 * m_receipt_create:
 * @rs: Record set que contiene los campos de la BD.
 *
 * Crea un objeto a partir de los valores de un record set de la base de datos.
 *
 * Returns:
 * Un objeto #Receipt con los campos rellenados con los datos de la base de datos.
 */
Receipt *m_receipt_create (GdaRecordset *rs) {
    GdaField * field;
    gchar *temp;
    Receipt *receipt = (Receipt *) g_malloc (sizeof(Receipt));

    d_receipt_init (receipt);

    field = gda_recordset_field_name(rs,"numeropolizareceipt");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    temp=gda_stringify_value(NULL, 0, field);
	    receipt->policy_number = g_strdup(temp);
            g_free(temp);
	    gda_field_free(field);

	    field = gda_recordset_field_name(rs,"numeroreciboreceipt");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    temp=gda_stringify_value(NULL, 0, field);
		    receipt->receipt_number = g_strdup(temp);
                    g_free(temp);
		}
		gda_field_free(field);
	    }
      
	    field = gda_recordset_field_name(rs,"numerocompaniareceipt");
	    if (field!=NULL){
		if(!gda_field_is_null(field)) {
		    receipt->company_code = igalia_field_get_bigint_value (field);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs,"codigotitularreceipt");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    receipt->holder_code = igalia_field_get_bigint_value (field);
		}
		gda_field_free(field);
	    }
      
	    field = gda_recordset_field_name(rs,"sucursalmediadorreceipt");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    temp=gda_stringify_value(NULL, 0, field);
		    receipt->subsidiary_mediator = g_strdup(temp);
                    g_free(temp);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs,"iniciovalidezreceipt");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    time_t * my_time;
		    temp=gda_stringify_value(NULL, 0, field);
		    my_time = u_parse_date (temp);
		    receipt->validity_start = (*my_time);
		    g_free(my_time);
		    my_time = NULL;
                    g_free(temp);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs,"finvalidezreceipt");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    time_t * my_time;
		    temp=gda_stringify_value(NULL, 0, field);
		    my_time = u_parse_date (temp);
		    receipt->validity_end = (*my_time);
		    g_free(my_time);
		    my_time = NULL;
		    g_free(temp);
		}
		gda_field_free(field);
	    }
	} else {
	    gda_field_free(field);
	}
    }

    return receipt;
}

/**
 * m_receipts_find_by_name:
 * @receipt_code: Cdigo del recibo que se desea buscar, si es %NULL se buscan todos los recibos.
 * @policy_number: Nmero de pliza del recibo.
 * @client: Cdigo del titular de la pliza.
 * @company: Cdigo de la compaa de seguros con la que est contratada la compaa.
 *
 * Busca el recibo con un nmero de recibo, nmero de pliza y de un cliente para una aseguradora. Si el
 * cdigo de cliente o el cdigo de la compaa de seguros es %NULL, no se utiliza como filtro de bsqueda.
 *
 * Returns:
 * Lista de recibos que cumplan la condicin de bsqueda.
 */
Receipt ** m_receipts_find_by_name (gchar *receipt_code, gchar *policy_number, gchar * client, gchar * company) {

  Receipt ** receipts_list = NULL; 
  GdaCommand         * my_command;
  GdaConnection      * my_connection = m_model_get_connection ();
  GdaRecordset       * record_set = NULL;
  Receipt * receipt;
  gchar * gchar_query = NULL;
  gulong * tuples;
  gint * column;
  gint index;
  gint length_of_list;
  
  my_command = gda_command_new();
  gda_command_set_connection(my_command,my_connection);
    
  if (policy_number != NULL){
    gchar_query = g_strconcat("SELECT ",
			      "recibo.codigocompania as codigocompanyreceipt,",
			      "recibo.numeropoliza as numeropolizareceipt,",
			      "recibo.codigotitular as codigotitularreceipt,",
			      "recibo.numerorecibo as numeroreciboreceipt,",
			      "recibo.sucursalmediador as sucursalmediadorreceipt,",
			      "recibo.iniciovalidez as iniciovalidezreceipt,",
			      "recibo.finvalidez as finvalidezreceipt",
			      " FROM recibo",
			      ", poliza WHERE poliza.numeropoliza=recibo.numeropoliza AND poliza.codigocompania = recibo.codigocompania AND poliza.codigotitular = recibo.codigotitular AND poliza.numeropoliza='",policy_number,"'",NULL);

  }else{
    gchar_query = g_strconcat("SELECT ",
			      "recibo.codigocompania as codigocompanyreceipt,",
			      "recibo.numeropoliza as numeropolizareceipt,",
			      "recibo.codigotitular as codigotitularreceipt,",
			      "recibo.numerorecibo as numeroreciboreceipt,",
			      "recibo.sucursalmediador as sucursalmediadorreceipt,",
			      "recibo.iniciovalidez as iniciovalidezreceipt,",
			      "recibo.finvalidez as finvalidezreceipt",
			      " FROM recibo",
			      NULL);

    if (receipt_code != NULL){
      gchar *gchar_query_aux = NULL;
      
      gchar_query_aux = g_strconcat(gchar_query," WHERE numerorecibo='",receipt_code,"'",NULL);
      g_free(gchar_query);
      gchar_query = gchar_query_aux;
      gchar_query_aux = NULL;
    }    
  }

  if (client != NULL) {
    gchar *gchar_query_aux = NULL;
      
    gchar_query_aux = g_strconcat(gchar_query," AND recibo.codigotitular='",client,"'",NULL);
    g_free(gchar_query);
    gchar_query = gchar_query_aux;
    gchar_query_aux = NULL;
  }

  if (company != NULL) {
    gchar *gchar_query_aux = NULL;
    
    gchar_query_aux = g_strconcat(gchar_query," AND recibo.codigocompania='",company,"'",NULL);
    g_free(gchar_query);
    gchar_query = gchar_query_aux;
    gchar_query_aux = NULL;
  }
  
  
  tuples = (gulong *) g_malloc0(sizeof(gulong));
  m_model_command_execute (&record_set, gchar_query, my_connection, my_command,tuples,0,TRUE);

  g_print(gchar_query);
  g_free(gchar_query);
  
  length_of_list = *tuples;
  column = (gint *) g_malloc0(sizeof(gint));
  
  if (record_set!=NULL) {
    if (*tuples!=0) {
      receipts_list = (Receipt **) g_malloc0(sizeof(Receipt*)*(*tuples+1));
      index = gda_recordset_move_first(record_set);
      while (!gda_recordset_eof(record_set)) {
	if (index==GDA_RECORDSET_INVALID_POSITION) return NULL;
	(*column)=0;
	receipt = m_receipt_create (record_set);
	receipts_list[index-1] = receipt;
	index = gda_recordset_move_next(record_set);
      }
      receipts_list[length_of_list]=NULL;
      
    } else {
      receipts_list=NULL;
    }
    gda_recordset_close(record_set);
    gda_recordset_free(record_set);
  } else {
    receipts_list=NULL;
  }
  
  igalia_command_free(my_command);
  
  g_free(tuples);
  g_free(column);
  
  return receipts_list;
}

/**
 * m_receipts_clear_array:
 * @receipts_list:  Lista de recibos a liberar.
 *
 * Libera la memoria utulizada por un array de recibos.
 *
 */
void m_receipts_clear_array (Receipt **receipts_list) {
  
    if (receipts_list!=NULL) {
	gint index = 0;
	while (receipts_list[index]!=NULL) {
	  d_receipt_clear (receipts_list[index]);
	  index++;
	}
    }

    g_free(receipts_list);
    
    receipts_list = NULL;
    
}

/**
 * m_receipts_insert:
 * @receipt: Recibo a insertar.
 * @participant_code: Cdigo del participante para el cual se insertar el recibo
 * @update: Si es %TRUE se modifica el recibo actual, en otro caso se inserta.
 * @command: Comando en el que ejecutar la sentencia sql.
 * @connection: Conexin a la base de datos.
 * @exception: Objeto que indica si se produce un error en la insercin.
 *
 * Inserta un recibo en la base de datos, dentro de una transaccin.
 *
 * Returns:
 * %TRUE si se ha insertado correctamente, %FALSE en otro caso.
 */
gboolean m_receipts_insert (Receipt *receipt, gchar *participant_code, gboolean update, GdaCommand * command, GdaConnection *connection, Exception *exception) {

  gboolean result = FALSE;

  if(receipt != NULL) {
    gchar * query = NULL;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    GdaRecordset *record_set = NULL;
    gulong company_code_gulong = d_receipt_get_company_code (receipt);
    gchar *company_code = NULL;   
    gchar *policy_number = d_receipt_get_policy_number (receipt);
    gchar *receipt_number = d_receipt_get_receipt_number (receipt);
    gchar *mediator_branch = d_receipt_get_subsidiary_mediator (receipt);
    time_t validity_date_time = d_receipt_get_validity_end (receipt);
    gchar *validity_date = NULL;
    gchar *validity_date_aux = NULL;
    time_t initial_validity_date_time = d_receipt_get_validity_start (receipt);
    gchar *initial_validity_date = NULL;
    gchar *local_participant_code = g_strdup(participant_code);
       
    if (company_code_gulong!=ULONG_MAX)
      company_code = g_strdup_printf("%lu",company_code_gulong);
    else company_code = g_strdup("NULL");
    
    if (policy_number == NULL) policy_number = g_strdup("NULL");
    else{
      gchar *policy_number_aux = NULL;
    
      policy_number_aux = g_strconcat("'",policy_number,"'",NULL);
      g_free(policy_number);
      policy_number = policy_number_aux;
      policy_number_aux = NULL;
    }

    if (receipt_number == NULL) receipt_number = g_strdup("NULL");
    else{
      gchar *receipt_number_aux = NULL;
    
      receipt_number_aux = g_strconcat("'",receipt_number,"'",NULL);
      g_free(receipt_number);
      receipt_number = receipt_number_aux;
      receipt_number_aux = NULL;
    }

    if (mediator_branch == NULL) mediator_branch = g_strdup("NULL");
    else{
      gchar *mediator_branch_aux = NULL;
    
      mediator_branch_aux = g_strconcat("'",mediator_branch,"'",NULL);
      g_free(mediator_branch);
      mediator_branch = mediator_branch_aux;
      mediator_branch_aux = NULL;
    }

    if (local_participant_code == NULL) local_participant_code = g_strdup("NULL");
    else{
      gchar *local_participant_code_aux = NULL;

      local_participant_code_aux = g_strconcat("'",local_participant_code,"'",NULL);
      g_free(local_participant_code);
      local_participant_code = local_participant_code_aux;
      local_participant_code_aux = NULL;
    }

    validity_date = ctime(&validity_date_time);
    validity_date_aux = g_strconcat("'",validity_date,"'",NULL);
    validity_date = validity_date_aux;
    validity_date_aux = NULL;

    initial_validity_date = ctime(&initial_validity_date_time);
    validity_date_aux = g_strconcat("'",initial_validity_date,"'",NULL);
    initial_validity_date = validity_date_aux;
    validity_date_aux = NULL;

    if (update) {
      query = g_strconcat("UPDATE ",m_receipt_get_table ()," SET ",
			  "sucursalmediador = ",mediator_branch,",",
			  "iniciovalidez = ",initial_validity_date,",",
			  "finvalidez = ",validity_date,
			  " WHERE codigocompania = ",company_code,
			  " AND numeropoliza = ",policy_number,
			  " AND codigotitular = ",local_participant_code,
			  " AND numerorecibo = ",receipt_number,
			  NULL);
    }
    else {
      query = g_strconcat("INSERT INTO ",m_receipt_get_table ()," VALUES (",
			  company_code,",",
			  policy_number,",",
			  local_participant_code,",",
			  receipt_number,",",
			  mediator_branch,",",
			  initial_validity_date,",",
			  validity_date,
			  ")",
			  NULL); 
    }
    
    result = m_model_command_execute (&record_set, query, connection,command,tuples,0, FALSE);

    if (!result) d_exception_set_message (exception, M_FIRST_ERROR_MESSAGE);

    g_print(query);

    gda_recordset_free(record_set);

    g_free(local_participant_code);
    g_free(company_code);
    g_free(policy_number);
    g_free(receipt_number);
    g_free(mediator_branch);
    g_free(validity_date);
    g_free(query);
    g_free(tuples);
  }

  return result;
}
