/* warehouse_model.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */
#define TABLE         "Almacen"
#define ATTRIBUTES    "codigoEmpresa as codigoempresawarehouse, \
    codigoAlmacen as codigoalmacenwarehouse, \
    nick      as nickwarehouse, \
    telefono  as telefonowarehouse, \
    fax       as faxwarehouse, \
    email     as emailwarehouse, \
    calle     as callewarehouse, \
    numero    as numerowarehouse, \
    piso      as pisowarehouse, \
    CP        as cpwarehouse, \
    municipio as municipiowarehouse, \
    provincia as provinciawarehouse "

#include "warehouse_model.h"

#include "util.h"

static Warehouse **warehouses_list=NULL;
static gulong      length_of_list;




/**
 * m_warehouse_get_table:
 *
 * Devuelve un string con la tabla correspondiente al objeto de la base de datos. Si el
 * objeto se corresponde con varias tablas se devolveran los nombres separados por comas.
 *
 * Returns:
 * Cadena de caracteres con la tabla o lista de tablas a las que se refiere el objeto.
 */
gchar *m_warehouse_get_table () {
    return TABLE;
}

/**
 * m_warehouse_get_attributes:
 *
 * Funcin que obtiene los atributos de un objeto, estos atributos son los que corresponden al
 * objeto en la base de datos.
 *
 * Returns:
 * Cadena de caracteres con los atributos a los que se refiere el objeto con un alias.
 */
gchar *m_warehouse_get_attributes () {
    return ATTRIBUTES;
}


/**
 * m_warehouse_create:
 * @rs: Record set que contiene los campos de la BD.
 *
 * Crea un objeto a partir de los valores de un record set de la base de datos.
 *
 * Returns:
 * Un objeto #Wwarehouse con los campos rellenados con los datos de la base de datos.
 */
Warehouse *m_warehouse_create (GdaRecordset *rs) {
    gchar     *temp;
    GdaField  *field;
    Warehouse *warehouse = d_warehouse_create_empty();

    field = gda_recordset_field_name(rs, "codigoalmacenwarehouse");

    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    warehouse->code = igalia_field_get_bigint_value (field);
	    gda_field_free(field);

	    field = gda_recordset_field_name(rs, "codigoempresawarehouse");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    warehouse->enterprise_code = igalia_field_get_bigint_value (field);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs, "nickwarehouse");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    temp = gda_stringify_value(NULL, 0, field);
		    warehouse->nick = g_strdup(temp);
		    g_free(temp);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs,"telefonowarehouse");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    temp = gda_stringify_value(NULL, 0, field);
		    warehouse->phone = g_strdup(temp);
		    g_free(temp);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs,"faxwarehouse");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    temp = gda_stringify_value(NULL, 0, field);
		    warehouse->fax = g_strdup(temp);
		    g_free(temp);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs, "emailwarehouse");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    temp = gda_stringify_value(NULL, 0, field);
		    warehouse->email = g_strdup(temp);
		    g_free(temp);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs, "callewarehouse");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    temp = gda_stringify_value(NULL, 0, field);
		    warehouse->street = g_strdup(temp);
		    g_free(temp);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs, "numerowarehouse");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    temp = gda_stringify_value(NULL, 0, field);
		    warehouse->number = g_strdup(temp);
		    g_free(temp);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs, "pisowarehouse");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    temp = gda_stringify_value(NULL, 0, field);
		    warehouse->floor = g_strdup(temp);
		    g_free(temp);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs, "cpwarehouse");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    warehouse->postal_code = igalia_field_get_bigint_value (field);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs, "municipiowarehouse");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    temp = gda_stringify_value(NULL, 0, field);
		    warehouse->town = g_strdup(temp);
		    g_free(temp);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs, "provinciawarehouse");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    temp = gda_stringify_value(NULL, 0, field);
		    warehouse->province = g_strdup(temp);
		    g_free(temp);
		}
		gda_field_free(field);
	    }
	} else gda_field_free(field);
    }
    return warehouse;
}


/**
 * m_warehouse_find_by_codes:
 * @warehouse_code: Cdigo del almacn.
 * @enterprise_code: Cdigo de la empresa a la que pertenenece el almacn.
 * 
 * Busca un almacn a partir del cdigo de empresa y del cdigo de almacn.
 *
 * Returns:
 * El almacn que cumple la condicin de bsqueda, %NULL en caso de datos incorrectos o error.
 */
Warehouse * m_warehouse_find_by_codes (gint warehouse_code, gint enterprise_code) {
    GdaCommand    *my_command;
    GdaConnection *my_connection = m_model_get_connection ();
    GdaRecordset  *record_set = NULL;
    Warehouse     *warehouse;
    gchar         *gchar_query;
    gchar         *aux1, *aux2;
    gulong        *tuples;
    gboolean       result = FALSE;

    my_command = gda_command_new();
    gda_command_set_connection(my_command,my_connection);
    aux1 = g_strdup_printf("%d",warehouse_code);
    aux2 = g_strdup_printf("%d",enterprise_code);
    gchar_query = g_strconcat("SELECT ", m_warehouse_get_attributes (),
			      " FROM ",  m_warehouse_get_table (),
			      " WHERE (codigo=",aux1,
			      ") AND (codigoEmpresa=",aux2, ") ", NULL);

    g_free(aux1);
    g_free(aux2);

    tuples = (gulong *) g_malloc0(sizeof(gulong));
    result = m_model_command_execute (&record_set, gchar_query, my_connection,my_command,tuples,0,TRUE);
    g_free(gchar_query);

    warehouse=NULL;
    if (result) {
	if (*tuples!=0) {
	    gda_recordset_move_first(record_set);
	    warehouse = m_warehouse_create (record_set);
	}
    }

    gda_recordset_free(record_set);
    igalia_command_free(my_command);
    g_free(tuples);
    return warehouse;
}

/**
 * m_warehouse_find_all:
 *
 * Busca todos los almacenes del grupo. Si es la primera vez que se ejecuta la bsqueda de almacenes
 * es necesario buscar en la base de datos, en otro caso se devuelve la lista de almacenes en memoria.
 *
 * Returns:
 * Devuelve una lista de almacenes con los almacenes.
 */
Warehouse ** m_warehouse_find_all () {
    GdaCommand    *my_command;
    GdaConnection *my_connection = m_model_get_connection ();
    GdaRecordset  *record_set = NULL;
    Warehouse     *warehouse;
    gchar         *gchar_query;
    gulong        *tuples;
    gint           index;
    gboolean       result = FALSE;


    if (warehouses_list==NULL) {
	my_command = gda_command_new();
	gda_command_set_connection(my_command,my_connection);

	gchar_query = g_strconcat("SELECT ", m_warehouse_get_attributes (),
				  " FROM ",  m_warehouse_get_table (), NULL);

	tuples = (gulong *) g_malloc0(sizeof(gulong));
	result = m_model_command_execute (&record_set, gchar_query, my_connection, my_command,tuples,0, TRUE);
	g_free(gchar_query);

	length_of_list = *tuples;

	warehouses_list=NULL;
	if (result) {
	    if (*tuples!=0) {
		warehouses_list = (Warehouse **) g_malloc0(sizeof(Warehouse*)*(*tuples+1));
		index = gda_recordset_move_first(record_set);
		while (!gda_recordset_eof(record_set)) {
		    if (index==GDA_RECORDSET_INVALID_POSITION) return NULL;
		    warehouse = m_warehouse_create (record_set);
		    warehouses_list[index-1] = warehouse;
		    index = gda_recordset_move_next(record_set);
		}
		warehouses_list[length_of_list]=NULL;

	    }
	}

	gda_recordset_free(record_set);
	igalia_command_free(my_command);
	g_free(tuples);
    }
    return warehouses_list;
}

/**
 * m_warehouse_find_by_enterprise_code:
 * @enterprise_code: Cdigo de la empresa de la que se quiere buscar los almacenes.
 *
 * Busca los almacenes de una empresa del grupo.
 *
 * Returns:
 * Lista con los almacenes que cumplen la condicin, %NULL en caso de error o cdigo de empresa incorrecto.
 */
Warehouse ** m_warehouse_find_by_enterprise_code (gint enterprise_code) {


    GdaCommand    *my_command;
    GdaConnection *my_connection = m_model_get_connection ();
    GdaRecordset  *record_set = NULL;
    Warehouse     *warehouse = NULL;
    Warehouse    **warehouses = NULL;
    gchar         *gchar_query;
    gchar         *aux;
    gulong        *tuples;
    gint           index;
    gboolean       result = FALSE;

    if (enterprise_code!=ULONG_MAX) {
	my_command = gda_command_new();
	gda_command_set_connection(my_command,my_connection);

	aux = g_strdup_printf("%d",enterprise_code);
	gchar_query = g_strconcat("SELECT ", m_warehouse_get_attributes (),
				  " FROM ",  m_warehouse_get_table (),
				  " WHERE (codigoEmpresa=",aux, ") ", NULL);
	tuples = (gulong *) g_malloc0(sizeof(gulong));
	result = m_model_command_execute (&record_set, gchar_query, my_connection, my_command,tuples,0, TRUE);
        g_free(aux);
	g_free(gchar_query);

	warehouses=NULL;
	if (result) {
	    if (*tuples!=0) {
		warehouses = (Warehouse **) g_malloc0(sizeof(Warehouse*)*(*tuples+1));
		index = gda_recordset_move_first(record_set);
		while (!gda_recordset_eof(record_set)) {
		    if (index==GDA_RECORDSET_INVALID_POSITION) return NULL;

		    warehouse = m_warehouse_create (record_set);
		    warehouses[index-1] = warehouse;
		    index = gda_recordset_move_next(record_set);
		}
		warehouses[*tuples]=NULL;

	    }
       }

	gda_recordset_close(record_set);
	gda_recordset_free(record_set);
	igalia_command_free(my_command);
 
	g_free(tuples);

    }

    return warehouses;

}


/**
 * m_warehouse_clear_array:
 * @warehouses: Array de almacenes a liberar.
 *
 * Libera la memoria utilizada por el array de almacenes.
 *
 */
void m_warehouse_clear_array (Warehouse ** warehouses) {
    if (warehouses!=NULL) {
	gint index = 0;
	while (warehouses[index]!=NULL) {
	    d_warehouse_clear (warehouses[index]);
            warehouses[index] = NULL;
	    index++;
	}
	g_free(warehouses);
        warehouses = NULL;
    }
}

/**
 * m_warehouse_clear_cached_array:
 *
 * Libera la memoria utilizada por la variable global que contiene el array de todos los almacenes.
 *
 * Returns:
 * void
 */
void m_warehouse_clear_cached_array () {
    m_warehouse_clear_array (warehouses_list);
}

/**
 * m_warehouse_get_name_from_code:
 * @code: Cdigo del almacn.
 *
 * Devuelve el nombre del almacn que es identificado por @code.
 *
 * Returns:
 * El nombre del almacn que cumple la condicin, %NULL en otro caso.
 */
gchar *m_warehouse_get_name_from_code (gulong code) {
    gchar *name = NULL;
    gint   index=0;

    if (warehouses_list==NULL) {
	m_warehouse_find_all ();
    }

    while(warehouses_list[index]!=NULL) {
	if (code == d_warehouse_get_code (warehouses_list[index])) {
	    name = d_warehouse_get_nick (warehouses_list[index]);
	}
        index++;
    }

    return name;
}

/**
 * m_warehouse_get_code_from_name:
 * @name: Nombre del almacn.
 *
 * Devuelve el cdigo del almacn que tienen como nombre @name.
 *
 * Returns:
 * Cdigo del almacn que cumple la condicin, %ULONG_MAX en otro caso.
 */
gulong m_warehouse_get_code_from_name (gchar *name) {
    gulong code  = ULONG_MAX;
    gint   index = 0;
    gchar  *temp = NULL;

    if (warehouses_list==NULL) {
	m_warehouse_find_all ();
    }

    while(warehouses_list[index]!=NULL) {
        temp = d_warehouse_get_nick (warehouses_list[index]);
	if (!strcmp(name,temp)) {
	    code = d_warehouse_get_code (warehouses_list[index]);
	    g_free(temp);
            break;
	}
        g_free(temp);
        index++;
    }
    return code;
}

/**
 * m_warehouse_get_enterprise_code_from_code:
 * @code: Cdigo del almacn.
 *
 * Busca el cdigo de la empresa a la que pertenece el almacn identificado por @code.
 *
 * Returns:
 * El cdigo de la empresa, %ULONG_MAX en caso de error o cdigo de almacn incorrecto.
 */
gulong m_warehouse_get_enterprise_code_from_code (gulong code) {
    gulong  w_code = ULONG_MAX;
    gint   index=0;

    if (warehouses_list==NULL) {
	m_warehouse_find_all ();
    }

    while(warehouses_list[index]!=NULL) {
	if (code == d_warehouse_get_code (warehouses_list[index])) {
            w_code = d_warehouse_get_enterprise_code (warehouses_list[index]);
	}
        index++;
    }
    return w_code;

}
