/* execute_file.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include "model.h"
#include "log2todo.h"

static gint execute_file(gchar *query);
static gchar *help(gchar *executable);

// Execute a query file in a only one transaction
static gint execute_file(gchar *file_name) {
    GdaCommand    *command = gda_command_new();
    GdaConnection *connection = m_model_get_connection ();
    GdaRecordset  *record_set = NULL;
    gboolean       correct_execute = TRUE;
    gulong         tuples;
    FILE          *in_file  = NULL;
    gchar         *buffer = NULL;
    gint           old_position = 0;
    gint           new_position = 0;
    gint           result = 0;
    gint           line   =0;

    gda_command_set_connection(command, connection);

    in_file  = fopen(file_name, "r");

    if(in_file==NULL) {
	perror("replication/execute_file.c/execute_file:fopen");
	printf(_("\tTryint open: %s\n"), file_name);
        m_model_destroy_connection ();
	exit(-1);
    }

    gda_connection_begin_transaction(connection);

    buffer = (gchar *) g_malloc0(sizeof(gchar *)*R_BLOCK_SIZE);
    new_position = r_read_line (&buffer, old_position, in_file);

    if (!((new_position!=old_position) && (buffer!=NULL) && correct_execute)) {
        //add 1 to lines counter if never goes into while
	line++;
    }

    while ((new_position!=old_position) && (buffer!=NULL) && correct_execute) {
	line++;
	if (r_is_query (buffer)) {
	    u_delete_intros (buffer);

	    gda_command_set_text(command, buffer);

	    record_set = gda_command_execute(command, &tuples, 0);

	    correct_execute = correct_execute && (record_set!=NULL)
		&& (gda_connection_get_errors(connection)==NULL);

	}

	if(record_set!=NULL) {
	    gda_recordset_free(record_set);
	    record_set = NULL;
	}
	new_position = r_read_line (&buffer, old_position, in_file);
    }

    correct_execute = (gda_connection_commit_transaction(connection)==0) &&
	correct_execute;
    gda_command_free(command);

    if(correct_execute) {
        result = 0;
    } else {
        result = line;
    }
    g_print(_("\nProcessed: %d\n"),line);
    g_print(_("Returns 0 if all OK or wrong linenumber otherwise: %d\n"),result);

    return result;
}

static gchar *help(gchar *executable) {
    gchar *help_text = NULL;

    help_text = g_strconcat(_("Use: "), executable, _(" (sql_file)\nReturns 0 if all OK or wrong linenumber otherwise\n"), NULL);
    return help_text;
}

int main(int argc, char *argv[]) {

    gint result;
    if (argc!=2) {
	g_print("\n%s\n", help(argv[0]));
        m_model_destroy_connection ();
	exit(-1);
    }
    m_model_init (argc, argv);

    result = execute_file(argv[1]);
    m_model_destroy_connection ();
    return result;
}
