/* util.h
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#ifndef _UTIL_
#define _UTIL_

#include "config.h"
#include <gnome.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <errno.h>
#include <time.h>
#include "math.h"


/**
 * U_LEFT:
 *
 * Indica el cdigo para la alineacin a la izquierda.
 *
 */
#define U_LEFT 0
/**
 * U_RIGHT:
 *
 * Indica el cdigo para la alineacin a la derecha.
 *
 */
#define U_RIGHT 2
/**
 * U_CENTER:
 *
 * Indica el cdigo para la alineacin centrada.
 *
 */
#define U_CENTER 1


void u_commas_to_points (char *text);
void u_points_to_commas (char *text);
void u_blanks_to_percentage (char *text);
void u_delete_intros (char *text);

//Blocking threads
void u_mutex_init (void);
void u_mutex_on (void);
void u_mutex_off (void);

//Blocking process
void u_set_lock (const gchar *path);
void u_remove_lock (const gchar *path);

void u_kill_gda (void);

gint u_get_index (GArray * rows,gint val);
gint u_enterprise_legal_id_is_correct (gulong legal_id);
gint u_person_legal_id_is_correct (gchar * legal_id);
gint u_get_hour (time_t time);
gint u_get_minute (time_t time);
gchar  *u_reverse (gchar *string);
gchar  *u_weight_legal_id (gchar *string);
gchar  *u_legal_id_calculate_letter (gulong legal_id);
gchar  *u_table_put_name (gchar *query, gchar *name);
gchar  *u_remove_aliases (gchar *query, const gchar *delim1, const gchar *delim2);
gchar  *u_fill_with_zeros_until (gchar * number_to_fill,gint number_total_digits);
gchar  *u_fill_n_spaces (gchar * char_to_fill,gint n);
gchar  *u_get_name_of_month (gint number);
gchar  *u_calculate_control_digit (gchar *entity, gchar *office, gchar *account);
gchar  *u_extract_element (gchar * number,gint index,const gchar * delimit,gint number_of_tokens);
gchar  *u_get_substring (gchar * original,gint pos_ini,gint pos_fin);
gchar  *u_get_substring_complete_words (gchar * original,gint pos_ini,gint * pos_fin);
gchar  *u_get_parsed_date (time_t *date);
time_t *u_parse_date (gchar * date);
time_t *u_get_current_date (void);
gint    u_exists (const gchar *path);
gchar *u_replace (gchar *original, gchar *delimiter, gchar *new_delimiter);
gint   u_send_mail (gchar *email, gchar *file_path, gchar *subject);
gdouble u_round_to_decimal (gdouble value,gint number_of_decimals);
gchar * u_string_fill_left (gchar character, gint total_size, gchar * string);
gchar * u_string_fill_right (gchar character, gint total_size, gchar * string);
gint u_cif_strcmp (gchar * legal_id1,gchar * legal_id2);
gchar ** u_cif_get_tokens (gchar * legal_id);
#endif



